//=============================================================================
// KNH_Message.js
//=============================================================================

/*:
 * @plugindesc 蓄積型メッセージ表示プラグイン
 * @author こんにちは
 *
 * @help 戦闘行動などを蓄積で表示させる
 */

(function() {
    'use strict';
    var pluginName = 'KNH_Message';
    
    var getNumber = function(text, max, min) {
        text = applyCharacters(text);
        if (isNaN(text)) return NaN;
        return Math.max(Math.min(Number(text), max || Infinity), min || -Infinity);
    };
    
    var applyCharacters = function(text) {
        if (text === null) text = '';
        text = '' + text;
        text = text.replace(/\\/g, '\x1b');
        text = text.replace(/\x1b\x1b/g, '\\');
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bN\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameActors.actor(parseInt(arguments[1], 10)) : null;
            return actor ? actor.name() : '';
        }.bind(this));
        text = text.replace(/\x1bP\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameParty.members()[parseInt(arguments[1], 10) - 1] : null;
            return actor ? actor.name() : '';
        }.bind(this));
        if ($dataSystem) text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
        return text;
    };
    
    var _Game_System_initialize = Game_System.prototype.initialize;
    Game_System.prototype.initialize = function() {
        _Game_System_initialize.apply(this, arguments);
        this._popMessage = [];
    };
    
    var maxMessage = function() {return [2, 3, 4][$gameVariables.value(33)]};
    var startY = 6.5 * 48 + 4; //83

    //-----------------------------------------------------------------------------
    // Scene_Map
    //-----------------------------------------------------------------------------
    
    var _Scene_Map_start = Scene_Map.prototype.start;
    Scene_Map.prototype.start = function() {
        _Scene_Map_start.apply(this, arguments);
        this._popMessageWindows = [];
        for (var i = 0; i < maxMessage(); i++) {
            var window = new Window_popMessage();
            this._popMessageWindows.push(window);
            this.addChild(window);
        }
        this._messageCount = 0;
    };
    
    var _Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function() {
        _Scene_Map_update.apply(this, arguments);
        this.updatePopMessages();
    };
    
    Scene_Map.prototype.updatePopMessages = function() {
        for (;$gameSystem._popMessage.length;) {
            if ($gameSystem._popMessage[0] !== '') {
                $gameSystem.pushDisplayLog(' ' + $gameSystem._popMessage[0]);
                var window = this._popMessageWindows[maxMessage() - 1];
                this._popMessageWindows.splice(maxMessage() - 1, 1);
                this._popMessageWindows.unshift(window);
                window.x = 0, window.y = startY + 8;
                window.setMessage($gameSystem._popMessage[0].toString());
                this._messageCount = 6 * 60;
            }
            $gameSystem._popMessage.shift();
        }
        this._popMessageWindows.forEach(function(w, i) {
            w._targetY = startY - i * w.height;
            if ($gameSystem.isBusy() || $gameSystem.inMenu() || $gameSwitches.value(3) ||
                this._messageCount <= 0) {
                w.visible = false;
            } else {
                w.visible = true;
            }
        }, this)
//        this._messageCount--;
    };
    
    //-----------------------------------------------------------------------------
    // Window_popMessage
    //-----------------------------------------------------------------------------
    
    function Window_popMessage() {
        this.initialize.apply(this, arguments);
    }
    
    Window_popMessage.prototype = Object.create(Window_Base.prototype);
    Window_popMessage.prototype.constructor = Window_popMessage;

    Window_popMessage.prototype.initialize = function() {
        var w = Graphics.boxWidth, h = 24;
        Window_Base.prototype.initialize.call(this, 0, 0, w, h);
        this.setBackgroundType(1);
        this._dimmerSprite.bitmap.clear();
    };
    
    Window_popMessage.prototype.standardFontSize = function() {
        return 16;
    };
    
    Window_popMessage.prototype.standardPadding = function() {
        return 0;
    };
    
    Window_popMessage.prototype.clearMessage = function() {
        this._dimmerSprite.bitmap.clear();
        this.contents.clear();
    };
    
    Window_popMessage.prototype.setMessage = function(text) {
        this.clearMessage();
        var w = this.drawTextEx(text, 0, 999);
        var x = ((this.contents.width - 12) - w) / 2;
        this.drawTextEx(text, x, 0);
        var bitmap = this._dimmerSprite.bitmap;
//        w += 50;
        var color1 = 'rgba(0, 0, 0, 0.5)', color2 = 'rgba(0, 0, 0, 0)'
//        bitmap.fillRect(0, 0, w / 2, bitmap.height, color1);
//        bitmap.gradientFillRect(w / 2, 0, w / 2, bitmap.height, color1, color2);
        bitmap.gradientFillRect(x - 48, 0, 72, bitmap.height, color2, color1);
        bitmap.fillRect(x + 24, 0, w - 48, bitmap.height, color1);
        bitmap.gradientFillRect(x + w - 24, 0, 72, bitmap.height, color1, color2);
        this.contentsOpacity = 0;
        this._count = 9;
    };
    
    Window_popMessage.prototype.update = function() {
        Window_Base.prototype.update.apply(this, arguments);
        if (!this.visible) return;
        this.y += ((this._targetY || this.y) - this.y) / 6;
        if (this._count > 0) this.contentsOpacity += 255 / 9;
        else this.contentsOpacity = this._dimmerSprite.opacity = 
            (1 - (startY - this.y - 6 * maxMessage()) / (18 * maxMessage())) * 255;
        if (this.visible) this._count--;
    };
    
})();
