//=============================================================================
// KNH_MiniMap.js
//=============================================================================

/*:
 * @plugindesc ミニマッププラグイン
 * @author こんにちは
 *
 * @help ミニマップを表示　参照はその時点での通行判定
 */

(function() {
    'use strict';
    var pluginName = 'KNH_MiniMap';
    
    var getNumber = function(text, max, min) {
        text = applyCharacters(text);
        if (isNaN(text)) return NaN;
        return Math.max(Math.min(Number(text), max || Infinity), min || -Infinity);
    };
    
    var applyCharacters = function(text) {
        if (text === null) text = '';
        text = '' + text;
        text = text.replace(/\\/g, '\x1b');
        text = text.replace(/\x1b\x1b/g, '\\');
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bN\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameActors.actor(parseInt(arguments[1], 10)) : null;
            return actor ? actor.name() : '';
        }.bind(this));
        text = text.replace(/\x1bP\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameParty.members()[parseInt(arguments[1], 10) - 1] : null;
            return actor ? actor.name() : '';
        }.bind(this));
        if ($dataSystem) text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
        return text;
    };
    
    //-----------------------------------------------------------------------------
    
    var HK = function() {
        return $gameSystem.HK();
    };
    
    Game_System.prototype.allMapOpen = function() {
        for (var x = 0; x < $dataMap.width; x++)
            for (var y = 0; y < $dataMap.height; y++)
                HK()._groundList[[x, y]] = true;
        HK()._allMapOpened = true;
        $gameSystem._needRefreshMiniMapGround = true;
    };

    //-----------------------------------------------------------------------------
    // Scene_Map
    //-----------------------------------------------------------------------------
    
    var _Scene_Map_start = Scene_Map.prototype.start;
    Scene_Map.prototype.start = function() {
        _Scene_Map_start.apply(this, arguments);
        if ($gameMap.isDMap()) {
            this._miniMapWindow = new Window_MiniMap();
            this.addWindow(this._miniMapWindow);
            if (HK()._allMapOpened) $gameSystem.allMapOpen();
        }
    };
    
    var _Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function() {
        _Scene_Map_update.apply(this, arguments);
        if ($gameMap.isDMap()) {
            if (!this._miniMapWindow) return;
            //デバッグ
            //if (!this._debugAllMapOpened) $gameSystem.allMapOpen(), this._debugAllMapOpened = 1;
            
            //消したり表示したり
            var special = [1, 12, 29].contains($gameVariables.value(9));
            (($gameSystem.isBusy() && !special) ||
             ($gameSystem.inMenu() && SceneManager._scene._menuWindow._menuMode !== 'map')) ?
                this._miniMapWindow.hide() : this._miniMapWindow.show();
            
            //通った所を表示
            var xyList = [[0, 0], [0, 1], [-1, 0], [0, -1], [1, 0], [-1, 1], [-1, -1], [1, -1], [1, 1]];
            if (xyList.some(function(xy) {
                    return !HK()._groundList[[$gamePlayer._x + xy[0], $gamePlayer._y + xy[1]]];
                }, this))
            {
                xyList.forEach(function(xy) {
                    HK()._groundList[[$gamePlayer.x + xy[0], $gamePlayer.y + xy[1]]] = 1;
                }, this)
                if ($gamePlayer.currentRoom()) {
                    var room = $gamePlayer.currentRoom();
                    if (isFinite(room)) {
                        for (var x = 0; x < $dataMap.width; x++)
                            for (var y = 0; y < $dataMap.height; y++) 
                                if ($gameMap.roomByXY(x, y) === room) HK()._groundList[[x, y]] = 1;
                    } else {
                        for (var x = room.x; x < room.x + room.w; x++) {
                            for (var y = room.y; y < room.y + room.h; y++) {
                                HK()._groundList[[x, y]] = 1;
                            }
                        }
                        for (var i = 0; i < 4; i++) {
                            var xy = room.passXy[['t', 'b', 'l', 'r'][i]];
                            if (xy) HK()._groundList[[xy[0], xy[1]]] = 1;
                        }
                    }
                }
                if (!$gameSystem._needRefreshMiniMapGround) $gameSystem._needRefreshMiniMapGround = 2;
            }
        }
    };
    
    Game_Map.prototype.isDMap = function() {
        return ($dataMapInfos[this.mapId()] && $dataMapInfos[this.mapId()].name.indexOf('<d>') === 0);
    };
    
    //-----------------------------------------------------------------------------
    // Window_MiniMap
    //-----------------------------------------------------------------------------
    
    function Window_MiniMap() {
        this.initialize.apply(this, arguments);
    }
    
    Window_MiniMap.prototype = Object.create(Window_Base.prototype);
    Window_MiniMap.prototype.constructor = Window_MiniMap;

    Window_MiniMap.prototype.initialize = function() {
        Window_Base.prototype.initialize.call(this, 0, 0, Graphics.boxWidth, Graphics.boxHeight);
        this.opacity = 0;
        this._nonMask = true;
        this._count = 0;
        this._eventsLastXy = [];
        this._characterSprites = [];
        this._groundDrawed = {};
        this._tileSize = 4;
        this._sizeType = 0;
        this.setPosition();
        this.createContents();
        $gameSystem._needRefreshMiniMapGround = true;
        this.refresh();
    };
    
    Window_MiniMap.prototype.standardPadding = function() {
        return 0;
    };
    
    Window_MiniMap.prototype.setPosition = function() {
//        var s = Math.min(Math.min(90 / $gameMap.width(), 42 / $gameMap.height()), 1);
//        this.x = Math.round(360 - this.width * s / 2);
//        this.y = Math.round(168 - this.height * s / 2);
        var s = 1;
        this.width  = $gameMap.width()  * this._tileSize;
        this.height = $gameMap.height() * this._tileSize;
        this.x = -20, this.y = 28;
        this.createContents();
        this.scale.x = this.scale.y = s;
    };
    
    Window_MiniMap.prototype.refresh = function() {
        var events = [].concat($gameMap._events);
        events[0] = $gamePlayer;
        this._needDeleteCharacters = [];
        this._characterSprites.forEach(function(sprite, id) {
            if (id !== 0 && sprite && !$gameMap._events[id]) this._needDeleteCharacters.push(id);
        }, this)
        this._needRefreshCharacters = [];
        events.forEach(function(event, id) {
            if (!event) return;
            var marker = id === 0 ? 0 : Number(event.event().meta.marker) || -1;
            var dark = HK()._dark && !$gamePlayer._D.metaState('light')
            var info = [
                event.x, event.y, event._direction, marker, this._sizeType, dark
            ].toString();
            if (!this._eventsLastXy[id] || this._eventsLastXy[id] !== info) {
                this._eventsLastXy[id] = info;
                event._marker = marker;
                this._needRefreshCharacters.push(event);
            }
            if (this._characterSprites[id] && id)
                this._characterSprites[id].visible = 
                    !(event._eventId && event._pageIndex < 0) &&
                    ((event._marker === 3 && this._enemyDisplay) ||
                     (!(dark && $gamePlayer.distanceFrom(event) > 3.5) &&
                      (!!HK()._groundList[[event._x, event._y]] || $gamePlayer._D.metaState('seeMarker'))));
        }, this);
        if (this._needDeleteCharacters.length > 0) this.deleteCharacters();
        if (this._needRefreshCharacters.length > 0) this.refreshCharacters();
        
        if ($gameSystem._needRefreshMiniMapGround) {
            this.refreshGround($gameSystem._needRefreshMiniMapGround === 2);
            $gameSystem._needRefreshMiniMapGround = false;
        }
    };
    
    Window_MiniMap.prototype.refreshGround = function(add) {
        if (!add) this.contents.clear(), this._groundDrawed = {};
        for (var x = 0; x < $gameMap.width(); x++) {
            for (var y = 0; y < $gameMap.height(); y++) {
                if (!HK()._groundList[[x, y]]) continue;
                if (add && this._groundDrawed[[x, y]]) continue;
                if ($gameSystem.isGround(x, y)) { //通行可能
                    var isPass = $gameMap.isPass(x, y);
                    var size = this._tileSize;
                    var dx = x * size;
                    var dy = y * size;
                    var cId = Number($dataMap.meta.miniMapColor) || 1;
                    var color = [
                        ['rgba(103, 212, 255, 0.6)' , 'rgba( 32, 128, 204, 0.6)'],
                        ['rgba(180, 108,  19, 0.6)' , 'rgba(150, 160,  82, 0.6)'],
                        ['rgba(  0, 150, 90, 0.6)' , 'rgba(  0, 80,  16, 0.6)'],
                    ][cId - 1][isPass ? 0 : 1];
                    this.contents.fillRect(dx, dy, size, size, color);
                    var axyList = [[0, 1], [-1, 1], [-1, 0], [-1, -1], [0, -1], [1, -1], [1, 0], [1, 1]];
                    axyList.forEach(function(axy, i) {
                        if (!$gameSystem.isGround(x + axy[0], y + axy[1])) {
                            var ldx = [0, 4].contains(i) ? dx : [1, 2, 3].contains(i) ? dx - 1 : dx + size;
                            var ldy = [2, 6].contains(i) ? dy : [3, 4, 5].contains(i) ? dy - 1 : dy + size;
                            var ldw = [0, 4].contains(i) ? size : 1;
                            var ldh = [2, 6].contains(i) ? size : 1;
                            this.contents.fillRect(ldx, ldy, ldw, ldh, 'white');
                        }
                    }, this)
                }
                this._groundDrawed[[x, y]] = true;
            }
        }
    };
    
    Window_MiniMap.prototype.refreshCharacters = function() {
        this._needRefreshCharacters.forEach(function(event) {
            var id = event._eventId || 0;
            if (!this._characterSprites[id]) {
                var sprite = new Sprite();
                var imgName = ['MinimapMarker', 'MinimapMarker2'][this._sizeType];
                sprite.bitmap = ImageManager.loadSystem(imgName);
                sprite.anchor.x = sprite.anchor.y = 0.5;
//                sprite.scale.x = sprite.scale.y = this._tileSize / 4;
                this._characterSprites[id] = sprite;
                this._characterSprites[id]._marker = event._marker;
                for (var index = 0; index < this.children.length; index++) {
                    var child = this.children[index];
                    var arr = [7, 6, 5, 4, 2, 1, 3, 0];
                    if (child && arr.indexOf(child._marker) > arr.indexOf(event._marker)) break;
                }
                this.addChildAt(sprite, index);
            }
            this._characterSprites[id].bitmap.addLoadListener(function(ls, lm){
                    var size = ls.bitmap.height;
                    ls.setFrame(size * lm, 0, size, size);
                }.bind(this, this._characterSprites[id], event._marker));
            this._characterSprites[id].x = (event._x + 0.5) * this._tileSize;
            this._characterSprites[id].y = (event._y + 0.5) * this._tileSize;
            if (id === 0) this._characterSprites[id].rotation = 
                [0, Math.PI / 2, -Math.PI / 2, Math.PI][event._direction / 2 - 1];
        }, this);
    };
    
    Window_MiniMap.prototype.deleteCharacters = function() {
        this._needDeleteCharacters.forEach(function(id) {
            this.removeChild(this._characterSprites[id]);
            delete this._characterSprites[id];
        }, this);
        
    };
    
    Window_MiniMap.prototype.changeSize = function(sizeType) {
        if (sizeType !== this._sizeType) {
            this._sizeType = sizeType;
            this._tileSize = [4, 6][sizeType];
            this.setPosition();
            $gameSystem._needRefreshMiniMapGround = true;
            this._characterSprites.forEach(function(sprite) {
                this.removeChild(sprite);
            }, this)
            this._characterSprites = [];
            this.refresh();
        }
    };
    
    Window_MiniMap.prototype.update = function() {
        Window_Base.prototype.update.apply(this, arguments);
        if (this._count % 6 === 0) this.refresh();
        this._characterSprites[0].visible = this._count % 30 < 15;
        this._count++;
    };
    
})();
