//=============================================================================
// KNH_MySetting.js
//=============================================================================

/*:
 * @plugindesc こんにちはセッティング
 * @author こんにちは
 *
 * @help こんにちは開発用セッティング
 *
 */

var TN_gameMap = []; 
(function() {
    'use strict';
    var pluginName = 'KNH_MySetting';
    
    var getBoolean = function(text) {
        return (applyCharacters(text) || '').toUpperCase() === 'ON';
    };
    
    var getNumber = function(text, max, min) {
        text = applyCharacters(text);
        if (isNaN(text)) return NaN;
        return Math.max(Math.min(Number(text), max || Infinity), min || -Infinity);
    };
    
    var getArrayString = function(text) {
        var values = applyCharacters(text).split(',');
        for (var i = 0; i < values.length; i++) {
            values[i] = values[i].trim();
            if (!isNaN(values[i])) values[i] = Number(values[i]);
        }
        return values;
    };
    
    var applyCharacters = function(text) {
        if (text === null) text = '';
        text = '' + text;
        text = text.replace(/\\/g, '\x1b');
        text = text.replace(/\x1b\x1b/g, '\\');
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bN\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameActors.actor(parseInt(arguments[1], 10)) : null;
            return actor ? actor.name() : '';
        }.bind(this));
        text = text.replace(/\x1bP\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameParty.members()[parseInt(arguments[1], 10) - 1] : null;
            return actor ? actor.name() : '';
        }.bind(this));
        if ($dataSystem) text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
        return text;
    };
    
    var getMeta = function(text, str) {
        var re = /<([^<>:]+)(:?)([^>]*)>/g;
        var meta = {};
        for (;;) {
            var match = re.exec(text);
            if (match) {
                if (match[2] === ':') {
                    meta[match[1]] = match[3];
                } else {
                    meta[match[1]] = true;
                }
            } else {
                break;
            }
        }
        return meta['' + str];
    };
    
    var rand = function(min,max) {
        return Math.floor(Math.random() * (max + 1 - min)) + min;
    };
    
    var shuffle = function(array) {
        var n = array.length, t, i;
        while (n) {
            i = Math.floor(Math.random() * n--);
            t = array[n];
            array[n] = array[i];
            array[i] = t;
        }
        return array;
    };
    //-----------------------------------------------------------------------------
    
    Input.keyMapper[27] = 'cancel';
    Input.keyMapper[45] = 'cancel';
    Input.keyMapper[88] = 'cancel';
    Input.keyMapper[96] = 'cancel';
    Input.keyMapper[65] = 'A';
    Input.keyMapper[83] = 'S';
    Input.keyMapper[68] = 'D';
    Input.keyMapper[70] = 'F';
    Input.keyMapper[71] = 'G';
    
    Input.gamepadMapper = {
        0: 'ok',        // A
        1: 'cancel',    // B
        2: 'F',         // X
        3: 'D',         // Y
        4: 'A',    // L1
        5: 'S',  // R1
        6: 'shift', //L2
        7: 'shift', //R2
        8: 'G', //select
        9: 'cancel', //start

        12: 'up',       // D-pad up
        13: 'down',     // D-pad down
        14: 'left',     // D-pad left
        15: 'right',    // D-pad right
    };
    
    //-----------------------------------------------------------------------------
    // ウィンドウ
    //-----------------------------------------------------------------------------
    
    Window_Base.prototype.standardFontSize = function() {
        return _lg() === 1 || _lg() === 2 ? 18 : 20;
    };
    Window_Base.prototype.standardPadding = function() {
        return 12;
    };
    Window_Base.prototype.textPadding = function() {
        return 4;
    };
    Window_Base.prototype.lineHeight = function() {
        return 28;
    };
    Window_Base.prototype.standardBackOpacity = function() {
        return 255;
    };
    
    Window_Base.prototype.makeFontBigger = function() {
        this.contents.fontSize = Math.min(this.contents.fontSize + 4, 96);
    };
    Window_Base.prototype.makeFontSmaller = function() {
        this.contents.fontSize = Math.max(this.contents.fontSize - 4, 0);
    };
    
    Window_Base.prototype.updateOpen = function() {
        if (this._opening) {
            this.openness += 255 / 6;
            if (this.isOpen()) this._opening = false;
        }
    };
    Window_Base.prototype.updateClose = function() {
        if (this._closing) {
            this.openness -= 255 / 6;
            if (this.isClosed()) this._closing = false;
        }
    };
    
    var _Window_updateCursor = Window.prototype._updateCursor;
    Window.prototype._updateCursor = function() {
        _Window_updateCursor.apply(this, arguments);
        var blinkCount = this._animationCount % 80;
        var cursorOpacity = this.contentsOpacity;
        if (this.active) {
            if (blinkCount < 40) {
                cursorOpacity -= blinkCount * 3;
            } else {
                cursorOpacity -= (80 - blinkCount) * 3;
            }
        }
        this._windowCursorSprite.alpha = cursorOpacity / 255;
    };
    
    var _Window_Base_textWidth = Window_Base.prototype.textWidth;
    Window_Base.prototype.textWidth = function(text) {
        return Math.ceil(_Window_Base_textWidth.apply(this, arguments));
    };
    
    var _Window_Base_update = Window_Base.prototype.update;
    Window_Base.prototype.update = function() {
        //軽量化
        if (!this.visible || (this.openness <= 0 && !this._opening)) {
            if (this._titleWindow) this._titleWindow.openness = 0;
            return false;
        }
        _Window_Base_update.apply(this, arguments);
        //タイトルウィンドウ
        this._lastTitleText = this._lastTitleText || '';
        if (this._titleText) {
            if (!this._titleWindow) {
                this._titleWindow = new Window_Base(0, 0, 0, 0);
                this._titleWindow.standardFontSize = function() {return 20};
                this._titleWindow.standardPadding = function() {return 8};
                this._titleWindow._padding = 8
                this._titleWindow.height = this._titleWindow.fittingHeight(1);
                this._titleWindow.contents.fontBold = true;
                this.addChild(this._titleWindow);
            }
            this._titlePos = this._titlePos || [0, 0];
            this._titleWindow.x = 0 + this._titlePos[0];
            this._titleWindow.y = 12 - this._titleWindow.height + this._titlePos[1];
            if (this._lastTitleText !== this._titleText) {
                this._titleWindow._windowskinName = 
                    (this._windowskinName.indexOf('_p') === -1) ? this._windowskinName + '_p' : 'Window_null';
                this._titleWindow.windowskin = ImageManager.loadSystem(this._titleWindow._windowskinName);
                var textWidth = this._titleWindow.drawTextEx(this._titleText, 0, 480);
                this._titleWindow.width = textWidth + this._titleWindow.padding * 2 + 1 + 8;
                this._titleWindow.createContents();
                this._titleWindow.contents.fontBold = true;
                var ow = this._titleWindow.contents.outlineWidth;
                this._titleWindow._noResetFontSettings = false;
                this._titleWindow.drawTextEx(this._titleText, 4, 0, textWidth);
                this._titleWindow._noResetFontSettings = true;
                this._titleWindow.contents.outlineWidth = 1;
                this._titleWindow.drawTextEx(this._titleText, 4, 0, textWidth);
                this._titleWindow._noResetFontSettings = false;
                this._titleWindow.contents.outlineWidth = ow;
                this._lastTitleText = this._titleText;
            }
            this._titleWindow.openness = this.openness >= 255 ? 255 : 0;
        } else if (this._titleWindow) {
            this._lastTitleText = '';
            if (!this._titleWindow.isClosed) this._titleWindow.close();
            else this.removeChild(this._titleWindow), this._titleWindow = null;
        }
        //キャンセルボタン
        if (this._cancelAction) {
            if (!this._cancelButton) {
                this._cancelButton = new Window_Base(this.width - 8, -17, 48, 48);
                this._cancelButton.opacity = 0;
                var bitmap = ImageManager.loadSystem('ButtonSet');
                var button = new Sprite_Button();
                var x = 48 * 6;
                button.bitmap = bitmap;
                button.bitmap.addLoadListener(function(){
                    button.setFrame(x, 0, 48, 48);
                }.bind(this));
                button.setColdFrame(x, 0, 48, 48);
                button.setHotFrame(x, 48, 48, 48);
                button.x -= 24, button.y -= 22;
                button.visible = true;
                this._cancelButton._button = button;
                this._cancelButton.addChild(button);
                this._cancelButton._button.setClickHandler((function() {
                    SoundManager.playCancel();
                    this._cancelAction();
                }).bind(this));
                this.addChild(this._cancelButton);
            }
            if (!this.isOpen()) this._cancelButton.hide();
            else this._cancelButton.show(), this._cancelButton._button.active = true;
//            _p(this._cancelButton._button.isActive(),this._cancelButton.active)
        } else if (this._cancelButton) {
            this.removeChild(this._cancelButton), this._cancelButton = null;
        }
    };
    
    //謎仕様無効化 & スワイプで選択肢移動しない
    Window_Selectable.prototype.onTouch = function(triggered) {
        var lastIndex = this.index();
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        var hitIndex = this.hitTest(x, y);
        if (hitIndex >= 0) {
            if (hitIndex === this.index()) {
                if (triggered && this.isTouchOkEnabled()) this.processOk();
            } else if (triggered && this.isCursorMovable()) {
                this.select(hitIndex);
            }
        }
        if (this.index() !== lastIndex) SoundManager.playCursor();
    };
    
    //文字潰れ防止
    var _Window_Base_drawText = Window_Base.prototype.drawText;
    Window_Base.prototype.drawText = function(text, x, y, maxWidth, align) {
        if (this.contents.fontSize >= 21) return _Window_Base_drawText.apply(this, arguments);
        var w = maxWidth || 144, h = this.lineHeight();
        var bitmap = this.newDrawText1(text, x, y, w, h, align);
        this.newDrawText2(bitmap, this.contents);
    };
    var _Window_Base_drawTextEx = Window_Base.prototype.drawTextEx;
    Window_Base.prototype.drawTextEx = function(text, x, y) {
        var wid = _Window_Base_drawTextEx.apply(this, arguments);
        if (this.contents.fontSize >= 21) return wid;
        var bitmap = this._newDrawBitmap;
        if (bitmap) this.newDrawText2(bitmap, this.contents);
        return wid;
    };
    var _Window_Base_processNormalCharacter = Window_Base.prototype.processNormalCharacter;
    Window_Base.prototype.processNormalCharacter = function(textState) {
        if (this.contents.fontSize >= 21) return _Window_Base_processNormalCharacter.apply(this, arguments);
        var c = textState.text[textState.index++];
        var w = this.textWidth(c);
        this.newDrawText1(c, textState.x, textState.y, w * 2, textState.height, 'left');
        textState.x += w;
    };
    Window_Base.prototype.newDrawText1 = function(text, x, y, w, h, align) {
        var cw = this.contents.width * 2, ch = this.contents.height * 2;
        if (!this._newDrawBitmap || this._newDrawBitmap.width < cw || this._newDrawBitmap.height < ch) {
            this._newDrawBitmap = new Bitmap(cw, ch);
            this._newDrawBitmap.smooth = true;
        }
        var bitmap = this._newDrawBitmap;
        bitmap.fontSize = this.contents.fontSize * 2;
        bitmap.textColor = this.contents.textColor;
        bitmap.outlineWidth = this.contents.outlineWidth * 2;
        bitmap.outlineColor = this.contents.outlineColor;
        bitmap.fontBold = this.contents.fontBold;
        bitmap.drawText(text, x * 2, y * 2, w * 2, h * 2, align);
        return bitmap;
    };
    Window_Base.prototype.newDrawText2 = function(textBitmap, drawBitmap) {
        if (!textBitmap) return;
        drawBitmap.blt(textBitmap, 0, 0, textBitmap.width, textBitmap.height,
                       0, 0, textBitmap.width / 2, textBitmap.height / 2);
        textBitmap.clear();
    };
    
    //アイコンの大きさをフォントサイズに合わせる
    Window_Base.prototype.drawIcon = function(iconIndex, x, y) {
        var bitmap = ImageManager.loadSystem('IconSet');
        var pw = Window_Base._iconWidth, ph = Window_Base._iconHeight;
        var sx = iconIndex % 16 * pw,    sy = Math.floor(iconIndex / 16) * ph;
        var ss = pw * this.contents.fontSize / 24;
        this.contents.blt(bitmap, sx, sy, pw, ph, x, y, ss, ss);
    };
    Window_Base.prototype.processDrawIcon = function(iconIndex, textState) {
        this.drawIcon(iconIndex, textState.x + 2 * this.contents.fontSize / 24, textState.y);
        textState.x += Window_Base._iconWidth * this.contents.fontSize / 24;
    };
    
    //マスクしない
    var _WindowLayer_maskWindow = WindowLayer.prototype._maskWindow;
    WindowLayer.prototype._maskWindow = function(window, shift) {
        _WindowLayer_maskWindow.apply(this, arguments);
        this._windowRect.x++, this._windowRect.y++, this._windowRect.width--, this._windowRect.height--;
//        if (window._nonMask) this._windowRect.width = 0;
        this._windowRect.width = 0;
    };
    
    //リフレッシュOpen
    Window_Base.prototype.refreshOpen = function() {
        if (this.refresh) this.refresh();
        this.openness = 0, this.open();
    };
    
    //右へのはみ出しを許容
    Window.prototype._updateContents = function() {
        var w = this._width - this._padding * 2;
        var h = this._height - this._padding * 2;
        if (w > 0 && h > 0) {
            this._windowContentsSprite.setFrame(this.origin.x, this.origin.y, w + 12, h);
            this._windowContentsSprite.visible = this.isOpen();
        } else {
            this._windowContentsSprite.visible = false;
        }
    };
    Window_Base.prototype.createContents = function() {
        this.contents = new Bitmap(this.contentsWidth() + 12, this.contentsHeight());
        this.resetFontSettings();
    };
    
    //-----------------------------------------------------------------------------
    // 文章ウィンドウデザイン変更
    //-----------------------------------------------------------------------------
    
    var _Window_Message_initialize = Window_Message.prototype.initialize;
    Window_Message.prototype.initialize = function() {
        _Window_Message_initialize.apply(this, arguments);
        this.loadWindowskin(4);
    };
    
    var _Window_Message_updatePlacement = Window_Message.prototype.updatePlacement;
    Window_Message.prototype.updatePlacement = function() {
        _Window_Message_updatePlacement.apply(this, arguments);
        this.loadWindowskin(4);
        this.y += this._positionType === 0 ? 32 : this._positionType === 2 ? -32 : 0;
        this.x = (Graphics.boxWidth - this.width) / 2;
    };
    
    Window_Message.prototype.windowWidth = function() {
        return Graphics.boxWidth - 144;
    };
    
    var _Window_ChoiceList_initialize = Window_ChoiceList.prototype.initialize;
    Window_ChoiceList.prototype.initialize = function() {
        _Window_ChoiceList_initialize.apply(this, arguments);
        this.loadWindowskin('1_p2');
    };
    
    var _Window_ChoiceList_updatePlacement = Window_ChoiceList.prototype.updatePlacement;
    Window_ChoiceList.prototype.updatePlacement = function() {
        _Window_ChoiceList_updatePlacement.apply(this, arguments);
        var positionType = $gameMessage.choicePositionType();
        this._pos = this._pos || [];
        if (this._pos[0]) 
            this.x = this._pos[0] ? this._pos[0] - this.width / 2 : this.x;
        else
            this.x += positionType === 0 ? 24 : positionType === 2 ? -64 : -24;
        if (this._pos[1]) 
            this.y = this._pos[1] ? this._pos[1] - this.height / 2 : this.y;
        else
            this.y += (this._messageWindow.y >= Graphics.boxHeight / 2) ? 20 : -20;
    };
    Window_ChoiceList.prototype.textWidthEx = function(text) {
        return Math.ceil(this.drawTextEx(text, 0, 9999) + 8);
    };
    Window_ChoiceList.prototype.isCurrentItemEnabled = function() {
        return this.index() !== $gameVariables.value(7) - 1 &&
            !NWS()._cannnotChoiceIndex.contains(this.index());
    };
    Window_ChoiceList.prototype.drawItem = function(index) {
        var rect = this.itemRectForText(index);
        this.changePaintOpacity(
            index !== $gameVariables.value(7) - 1 &&
            !NWS()._cannnotChoiceIndex.contains(index)
        );
        this.drawTextEx(this.commandName(index), rect.x + 4, rect.y);
        this.changePaintOpacity(true);
    };
    Game_Message.prototype.choices = function() {
        this._choices.length = NWS()._choiceText.length || this._choices.length;
        var list = [];
        for (var i = 0; i < this._choices.length; i++) {
            list.push(NWS()._choiceText[i] || this._choices[i]);
        }
        return list;              
    };
    var _Game_Message_onChoice = Game_Message.prototype.onChoice;
    Game_Message.prototype.onChoice = function(n) {
        _Game_Message_onChoice.apply(this, arguments);
        $gameVariables.setValue(13, n);
    };
    
    Game_Message.prototype.choiceDefaultType = function() {
        return NWS()._choiceDefault >= 0 ? NWS()._choiceDefault : this._choiceDefaultType;
    };
    
    //-----------------------------------------------------------------------------
    // アイテム画面デザイン・処理変更
    //-----------------------------------------------------------------------------
    
    var _Scene_Item_start = Scene_Item.prototype.start;
    Scene_Item.prototype.start = function() {
        _Scene_Item_start.apply(this, arguments);
        this.createItemNameWindow();
        this._helpWindow.x = 48, this._helpWindow.y = 66, this._helpWindow.width -= 96;
        this._itemWindow.x = 24, this._itemWindow.y = 192, this._itemWindow.width -= 48, this._itemWindow.height = 216;
        this._itemWindow.refresh();
        this._categoryWindow.hide();
        this._categoryWindow.select(0);
        this._categoryWindow.deactivate();
        this.onCategoryOk();
    };
    
    Scene_Item.prototype.createItemNameWindow = function() {
        this._itemNameWindow = new Window_Base(24, 24, 0, 56);
        this._itemNameWindow.padding = 15;
        this._itemNameWindow.refresh = function(name) {
            if (this._name !== undefined) this.openness = 0;
            this._name = name;
            if (!name || name === '') return;
            this.open();
            var textWidth = this.textWidth(name);
            this.width = textWidth + this.padding * 2;
            this.contents = new Bitmap(textWidth, 36);
            this.drawText(name, 0, -4, textWidth);
        }
        this.addWindow(this._itemNameWindow);
    };
    
    Scene_Item.prototype.onItemCancel = function() {
        this.popScene();
    };
    
    var _Scene_Item_update = Scene_Item.prototype.update;
    Scene_Item.prototype.update = function() {
        _Scene_Item_update.apply(this, arguments);
        var name = this.item() ? this.item().name : '';
        if (name !== this._itemNameWindow._name) this._itemNameWindow.refresh(name);
    };
    
    Window_ItemList.prototype.spacing = function() {
        return 24;
    };
    
    //-----------------------------------------------------------------------------
    // オプション画面デザイン変更
    //-----------------------------------------------------------------------------
    
    var _Scene_Options_start = Scene_Options.prototype.start;
    Scene_Options.prototype.start = function() {
        _Scene_Options_start.apply(this, arguments);
        this._optionsWindow._titleText = TextManager.options;
        this._optionsWindow._cancelAction = this.popScene.bind(this);
    };
    
    //-----------------------------------------------------------------------------
    // セーブ画面デザイン変更
    //-----------------------------------------------------------------------------
    
    var _Scene_File_start = Scene_File.prototype.start;
    Scene_File.prototype.start = function() {
        _Scene_File_start.apply(this, arguments);
        this._helpWindow.hide();
        this._listWindow._titleText = this.helpWindowText();
        this._listWindow._cancelAction = function() {
            $gameTemp._saveCanceled = true;
            this.popScene();
        }.bind(this);
        this._listWindow.setHandler('cancel', this._listWindow._cancelAction);
        this._listWindow.x = 48, this._listWindow.y = 64, this._listWindow.width -= 96, this._listWindow.height = 332;
        this._listWindow.refresh();
        this._listWindow.select(this._listWindow.index());
    };
    
    DataManager.maxSavefiles = function() {
        return 5;
    };
    
    Window_SavefileList.prototype.maxVisibleItems = function() {
        return 5;
    };
    
    //-----------------------------------------------------------------------------
    // 名前入力画面デザイン変更
    //-----------------------------------------------------------------------------
    
    Scene_Name.prototype.start = function() {
        Scene_MenuBase.prototype.start.call(this);
        this._editWindow.x = 39, this._editWindow.y = 64, this._editWindow.width = 240, this._editWindow.height = 192;
        this._inputWindow.x = 264, this._inputWindow.y = 36;
        this._editWindow.refresh();
        this._windowLayer.removeChild(this._editWindow);
        this._windowLayer.addChild(this._editWindow);
        this._titleText = ['名前を入力', 'Please enter a name', '이름을 입력하십시오', '請輸入該名稱', '请输入该名称'][_lg()];
    };
    
    Window_NameEdit.prototype.itemRect = function(index) {
        return {
            x: this.left() + (index + 0.5) * this.charWidth(),
            y: 0,
            width: this.charWidth(),
            height: this.lineHeight()
        };
    };
    
    Window_NameEdit.prototype.faceWidth = function() {
        return 0;
    };
    
    var nj1 = Window_NameInput.JAPAN1, nj2 = Window_NameInput.JAPAN2, nj3 = Window_NameInput.JAPAN3;
    nj1[75] = 'ゔ', nj1[76] = 'を', nj1[77] = 'ー', nj1[78] = '～', nj1[79] = '＝', nj1[85] = '・', nj1[86] = '　', nj1[87] = '←',
    nj2[75] = 'ヴ', nj2[76] = 'ヲ', nj2[77] = 'ー', nj2[78] = '～', nj2[79] = '＝', nj2[85] = '・', nj2[86] = '　', nj2[87] = '←',
    nj3[86] = '　', nj3[87] = '←',  nj3[88] = '漢字1';
    Window_NameInput.JAPAN4 =
        [ '剣','魔','法','者','神',  '上','下','左','右','前',
          '聖','男','女','死','王',  '東','西','南','北','後',
          '天','竜','術','悪','命',  '国','塔','破','殴','裂',
          '炎','氷','雷','光','闇',  '陸','海','突','攻','撃',
          '地','水','火','風','無',  '夢','歌','長','短','色',
          '幻','戦','木','石','心',  '数','飛','行','刀','爆',
          '薬','泉','野','原','家',  '波','砲','蹴','斬','壊',
          '愛','勇','勝','負','金',  '子','気','必','殺','旅',
          '大','中','小','世','界',  '々','　','←','かな','決定' ];
    
    Window_NameInput.prototype.table = function() {
        if ($gameSystem.isJapanese()) {
            return [Window_NameInput.JAPAN1, Window_NameInput.JAPAN2,
                    Window_NameInput.JAPAN3, Window_NameInput.JAPAN4];
        } else if ($gameSystem.isRussian()) {
            return [Window_NameInput.RUSSIA];
        } else {
            return [Window_NameInput.LATIN1, Window_NameInput.LATIN2];
        }
    };
    
    var _Window_NameInput_processOk = Window_NameInput.prototype.processOk;
    Window_NameInput.prototype.processOk = function() {
        if (this._index === 87) this.processBack();
        else _Window_NameInput_processOk.apply(this, arguments);
    };
    
    //-----------------------------------------------------------------------------
    // ポップアップ位置を微乱数化
    //-----------------------------------------------------------------------------
    
    Sprite_Character.prototype.damageOffsetX = function() {
        return rand(-2, 2);
    };

    Sprite_Character.prototype.damageOffsetY = function() {
        return -24 + rand(-2, 2);
    };
    
    //-----------------------------------------------------------------------------
    // アニメ名に<rand>で角度・拡大率などランダム化
    //-----------------------------------------------------------------------------
    
    var _Sprite_Animation_updateCellSprite = Sprite_Animation.prototype.updateCellSprite;
    Sprite_Animation.prototype.updateCellSprite = function(sprite, cell) {
        _Sprite_Animation_updateCellSprite.apply(this, arguments);
        if (!this._setted && this._animation.name.indexOf('<rand>') >= 0) {
            this._rand = [rand(0, 360), rand(-3, 3), rand(0, 1), rand(0, 100)];
        }
        var ind = this._animation.name.indexOf('<scale:');
        if (!this._setted && ind >= 0) {
            for (var i = 4; !this._settingScale && i > 0; i--)
                this._settingScale = Number(this._animation.name.slice(ind + 7, ind + 7 + i));
        }
        this._setted = true;
        if (cell[3] >= 1000) sprite.scale.x = sprite.scale.y = 2000;
        if (this._rand) {
            sprite.rotation += this._rand[0] * Math.PI / 180;
            sprite.x += this._rand[1] * (this._rand[2] == 0 ? 1 : -1);
            sprite.scale.x *= -0.90 + this._rand[3] * 0.002;
            sprite.scale.y *= -0.90 + this._rand[3] * 0.002;
        }
        if (this._settingScale) {
            var scale = this._settingScale / 100;
            sprite.x *= scale, sprite.y *= scale;
            sprite.scale.x *= scale, sprite.scale.y *= scale;
        }
    };
    
    //-----------------------------------------------------------------------------
    // アニメ名に空白のフラッシュSEを入れたらそこでウェイト終了
    //-----------------------------------------------------------------------------

    var _Sprite_Animation_processTimingData = Sprite_Animation.prototype.processTimingData;
    Sprite_Animation.prototype.processTimingData = function(timing) {
        _Sprite_Animation_processTimingData.apply(this, arguments);
        if (timing.flashScope === 0 && !timing.se) {
            this._finish = true;
            if (this._target._character) this._target._character._animationNum--;
        }
    };
    
    Sprite_Base.prototype.isFinishAnimation = function() {
        return this._animationSprites.every(function(sprite) {
            sprite._finish
        });
    };
    
    Game_CharacterBase.prototype.isAnimationPlaying = function() {
        return (this._animationId > 0 || this._animationPlaying) && this._animationNum;
    };
    
    var _Game_CharacterBase_requestAnimation = Game_CharacterBase.prototype.requestAnimation;
    Game_CharacterBase.prototype.requestAnimation = function() {
        _Game_CharacterBase_requestAnimation.apply(this, arguments);
        this._animationNum = (this._animationNum || 0) + 1;
    };
    
    //-----------------------------------------------------------------------------
    // 開発用機能
    //-----------------------------------------------------------------------------
    
    Game_Interpreter.prototype.command355 = function() {
        var script = this.currentCommand().parameters[0] + '\n';
        var sc = script.slice(0, 30);
        while (this.nextEventCode() === 655) {
            this._index++;
            script += this.currentCommand().parameters[0] + '\n';
        }
        try {
            eval(script);
        } catch (e) {
            console.log('ID:%1, "%2"'.format(this._eventId, sc), e);
            eval(script);
        }
        return true;
    };
    
    Scene_Base.prototype.fadeOutInTest = function() {
        AudioManager.fadeOutBgm(1);
        AudioManager.fadeOutBgs(1);
        AudioManager.fadeOutMe(1);
        this.startFadeOut(1);
    };
    
//    Scene_Title.prototype.commandNewGame = function() {
//        DataManager.setupNewGame();
//        this._commandWindow.close();
//        !$gameTemp.isPlaytest() ? this.fadeOutAll() : this.fadeOutInTest();
//        SceneManager.goto(Scene_Map);
//    };
    
    //-----------------------------------------------------------------------------
    // タイトル画面
    //-----------------------------------------------------------------------------
    
    var _Window_TitleCommand_initialize = Window_TitleCommand.prototype.initialize;
    Window_TitleCommand.prototype.initialize = function() {
        _Window_TitleCommand_initialize.apply(this, arguments);
        this.loadWindowskin('1_p2');
        this.y += 30;
    };
    Window_TitleCommand.prototype.itemTextAlign = function() {
        return 'center';
    };
    Window_TitleCommand.prototype.windowWidth = function() {
        return 200;
    };
    
    var _Scene_Title_create = Scene_Title.prototype.create;
    Scene_Title.prototype.create = function() {
        _Scene_Title_create.apply(this, arguments);
        this._versionText = new Window_Base(Graphics.boxWidth - 104, 0, 104, 96);
        this._versionText.setBackgroundType(2);
        this._versionText.drawTextEx('ver\\b' + version, 4, 0);
        this.addWindow(this._versionText);
    };
    //-----------------------------------------------------------------------------
    // バグ対策
    //-----------------------------------------------------------------------------
    
    AudioManager.replayBgm = function(bgm) {
        this.playBgm(bgm);
    };

    AudioManager.replayBgs = function(bgs) {
        this.playBgs(bgs);
    };
    
    //-----------------------------------------------------------------------------
    // オプションヘルプ
    //-----------------------------------------------------------------------------
    
    var optionHelpTexts = function(index) {
        return _lgCheck([
            'メッセージのテキストのアニメーションの ON/OFF を設定します。',
            'オートセーブの ON/OFF を設定します。\nONにすると、現在のセーブデータへ定期的にオートセーブを行います。\nセーブ時は少し動作が重くなる可能性があります。',
            'メッセージログの画面最大表示行数を設定します。',
            '戦闘のテンポの速さを設定します。\nある程度ゲームに慣れてきてから高速にすることをお薦めします。',
            $dataMap ? '言語はタイトル画面でのみ変更できます。\nThe language can only be changed on the title screen.' : '表示する言語を変更します。',
            '操作ボタンの表示タイプを設定します。\nゲームパッドでのプレイ時は「ゲームパッド」に設定してください。',
            'ゲームで使用するキーの割り当てを自由に編集することができます。',
            'ゲームパッドボタンの割り当てを自由に編集することができます。',
            'ゲーム全体の音量を変更します。',
            'BGMの音量を変更します。',
            'BGSの音量を変更します。',
            'MEの音量を変更します。',
            'SEの音量を変更します。'
        ][index] || '');
    };
    var _Scene_Options_create = Scene_Options.prototype.create;
    Scene_Options.prototype.create = function() {
        _Scene_Options_create.call(this);
        var w = Graphics.boxWidth - 96, h = 36 + 24 * 3;
        var x = (Graphics.boxWidth - w) / 2, y = Graphics.boxHeight - h - 24;
        this._optionHelpWindow = new Window_Base(x, y, w, h);
        this._optionHelpWindow.setBackgroundType(1);
        this.addWindow(this._optionHelpWindow);
        this._helpText = this._lastHelpText = '';
    };
    var _Scene_Options_update = Scene_Options.prototype.update;
    Scene_Options.prototype.update = function() {
        _Scene_Options_update.call(this);
        this._helpText = optionHelpTexts(this._optionsWindow.index());
        if (this._helpText !== this._lastHelpText) {
            this._lastHelpText = this._helpText;
            this._optionHelpWindow.contents.clear();
            this._optionHelpWindow.drawTextEx(this._helpText, 0, 0);
        }
    };
    
    Window_Options.prototype.windowHeight = function() {
        return this.fittingHeight(Math.min(this.numVisibleRows(), 7));
    };
    Window_Options.prototype.updatePlacement = function() {
        this.x = (Graphics.boxWidth - this.width) / 2;
        this.y = (Graphics.boxHeight - 88 - this.height) / 2;
    };
    Window_Options.prototype.drawItem = function(index) {
        var rect = this.itemRectForText(index);
        var statusWidth = this.statusWidth();
        var titleWidth = rect.width - statusWidth;
        this.resetTextColor();
        if ($dataMap && this._list[index].name === '言語') this._list[index].enabled = false;
        this.changePaintOpacity(this.isCommandEnabled(index));
        this.drawText(this.commandName(index), rect.x + 4, rect.y, titleWidth, 'left');
        this.drawText(this.statusText(index), titleWidth - 4, rect.y, statusWidth, 'right');
    };
    var _Window_Options_commandName = Window_Options.prototype.commandName;
    Window_Options.prototype.commandName = function(index) {
        var name = _Window_Options_commandName.apply(this, arguments);
        if (_lg() <= 0) return name;
        var options = ['テキストのアニメ表示', 'オートセーブ', 'ログ最大表示行数',
                       '戦闘テンポ', 'ボタン表示タイプ', '言語', '全体音量'];
        if (!options.contains(name)) return _lgCheck(name);
        var lgOptions = [
            ['Text Animation Display','텍스트 애니메이션 표시', '文字動畫顯示', '文字动画显示'],
            ['Auto Save', '자동 저장', '自動保存', '自动保存'],
            ['Log Display Max Lines', '로그 최대 행 표시', '日誌顯示最大行數', '日志显示最大行数'],
            ['Battle Tempo', '전투 템포', '戰鬥節奏', '战斗节奏'],
            ['Button Display Type', '버튼 표시 유형', '按鈕顯示類型', '按钮显示类型'],
            ['Language', '언어', '語言', '语言'],
            ['Master Volume', '마스터 볼륨', '主音量', '主音量']
        ]
        return lgOptions[options.indexOf(name)][_lg() - 1];
    };
    var _Window_Options_statusText = Window_Options.prototype.statusText;
    Window_Options.prototype.statusText = function(index) {
        var name = _Window_Options_statusText.apply(this, arguments);
        if (_lg() <= 0) return name;
        var status = ['普通', '高速', 'キーボード', 'ゲームパッド'];
        if (!status.contains(name)) return name;
        var lgStatus = [
            ['Normal','일반', '正常', '正常'],
            ['High Speed', '고속', '高速', '高速'],
            ['Keyboard', '키보드', '鍵盤', '键盘'],
            ['Gamepad', '게임 패드', '控制器', '控制器']
        ]
        return lgStatus[status.indexOf(name)][_lg() - 1];
    };
    
    var _Window_Options_processOk = Window_Options.prototype.processOk;
    Window_Options.prototype.processOk = function() {
        if ($dataMap && this._list[this.index()].name === '言語') return SoundManager.playBuzzer();
        _Window_Options_processOk.apply(this, arguments);
    };
    
    //-----------------------------------------------------------------------------
    // 専用
    //-----------------------------------------------------------------------------

    //影は消します
    var _Tilemap_drawShadow = Tilemap.prototype._drawShadow;
    Tilemap.prototype._drawShadow = function(bitmap, shadowBits, dx, dy) {
    };
    var _ShaderTilemap_drawShadow = ShaderTilemap.prototype._drawShadow;
    ShaderTilemap.prototype._drawShadow = function(layer, shadowBits, dx, dy) {
    };
    
    //ダッシュ機能を消します
    Game_Player.prototype.updateDashing = function() {
        if (this.isMoving()) return;
        this._dashing = false;
    };
    
    var _Game_Player_initialize = Game_Player.prototype.initialize;
    Game_Player.prototype.initialize = function() {
        _Game_Player_initialize.call(this);
        this._moveSpeed = 4;
    };
    
    //移動ルート設定の前に this._forceCharacter に入れると そのキャラの移動ルート設定になる
    Game_Interpreter.prototype.command205 = function() {
        $gameMap.refreshIfNeeded();
        this._character = this._forceCharacter || this.character(this._params[0]);
        this._forceCharacter = null;
        if (this._character) {
            this._character.forceMoveRoute(this._params[1]);
            if (this._params[1].wait) this.setWaitMode('route');
        }
        return true;
    };
    
    //マップにアニメーションを書き込む用
    var _Spriteset_Map_createCharacters = Spriteset_Map.prototype.createCharacters;
    Spriteset_Map.prototype.createCharacters = function() {
        _Spriteset_Map_createCharacters.apply(this, arguments);
        this.createPlayAnimationSprite();
    };
    Spriteset_Map.prototype.createPlayAnimationSprite = function() {
        this._playAnimationSprite = new Sprite_Base();
        this._playAnimationSprite.move(0, 0, Graphics.boxWidth, Graphics.boxHeight);
        this._baseSprite.addChild(this._playAnimationSprite);
    };
    var _Sprite_Base_startAnimation = Sprite_Base.prototype.startAnimation;
    Sprite_Base.prototype.startAnimation = function(animation, mirror, delay) {
        _Sprite_Base_startAnimation.apply(this, arguments);
        if ($gameSystem._animationMapXy) {
            var sprite = this._animationSprites[this._animationSprites.length - 1];
            sprite._mapXy = [].concat($gameSystem._animationMapXy);
            sprite.z += sprite._mapXy[1] / 100;
            $gameSystem._animationMapXy = null;
        }
    };
    var _Sprite_Animation_updatePosition = Sprite_Animation.prototype.updatePosition;
    Sprite_Animation.prototype.updatePosition = function() {
        _Sprite_Animation_updatePosition.apply(this, arguments);
        if (this._mapXy !== undefined) {
            this.x = 24 + $gameMap.adjustX(this._mapXy[0]) * 48;
            this.y = 24 + $gameMap.adjustY(this._mapXy[1]) * 48;
            if (this._animation.position === 0) this.y -= 24;
            if (this._animation.position === 2) this.y += 24;
        }
    };
    
    //画面外のキャラにアニメは表示しない
    var __Game_CharacterBase_requestAnimation = Game_CharacterBase.prototype.requestAnimation;
    Game_CharacterBase.prototype.requestAnimation = function(animationId) {
        var x = this.screenX(), y = this.screenY();
        if (!this.isOutOfScreen())
            __Game_CharacterBase_requestAnimation.apply(this, arguments);
    };
    
    //イベントの消去を完全消去にする
    Game_Event.prototype.erase = function() {
        this._erased = true;
        this.refresh();
        delete this._D;
        delete $gameMap._events[this._eventId];
        SceneManager._scene._spriteset._characterSprites.forEach(function(sprite, index) {
            if (sprite._character instanceof Game_Event && sprite._character._eventId === this._eventId) {
                SceneManager._scene._spriteset._characterSprites.splice(index, 1);
                sprite._animationSprites.forEach(function(animation) {
                    animation.remove();
                });
                if (sprite._hpGaugeSprite) {
                    SceneManager._scene._spriteset._baseSprite.removeChild(sprite._hpGaugeSprite);
                    delete sprite._hpGaugeSprite;
                }
                if (sprite._weaponSprite) {
                    sprite.removeChild(sprite._weaponSprite);
                    delete sprite._weaponSprite;
                }
                if (sprite._stateSprite) {
                    sprite.removeChild(sprite._stateSprite);
                    delete sprite._stateSprite;
                }
                sprite.visible = false;
                sprite = null;
            }
        }, this);
        SceneManager._scene._spriteset._tilemap.children.forEach(function(sprite, index) {
            if (sprite._character instanceof Game_Event && sprite._character._eventId === this._eventId) {
                SceneManager._scene._spriteset._tilemap.children.splice(index, 1);
                sprite.visible = false;
                sprite = null;
            }
        }, this);
    };
    Game_Map.prototype.eraseEvent = function(eventId) {
        if (this._events[eventId]) this._events[eventId].erase();
    };
    
    //キャラの表示位置
    Game_CharacterBase.prototype.shiftY = function() {
        return this.isObjectCharacter() ? 0 : 16;
    };
    
    //ダメージ最大値設定
    var _Game_Action_makeDamageValue = Game_Action.prototype.makeDamageValue;
    Game_Action.prototype.makeDamageValue = function(target, critical) {
        var value = _Game_Action_makeDamageValue.apply(this, arguments);
        return value.clamp(-999999, 999999);
    };
    
    //マウス位置の記録を常に行う
    TouchInput._onMouseMove = function(event) {
        var x = Graphics.pageToCanvasX(event.pageX);
        var y = Graphics.pageToCanvasY(event.pageY);
        this._onMove(x, y);
    };

    //スイッチ2のON OFF では更新しない　軽量化のため
    Game_Switches.prototype.setValue = function(switchId, value) {
        if (switchId > 0 && switchId < $dataSystem.switches.length) {
            this._data[switchId] = value;
            if (switchId !== 2) this.onChange();
            else $gameMap._commonEvents.forEach(function(event) {
                if (event._commonEventId === 31) event.refresh();
            })
        }
    };
    //だがスイッチ2が起動条件のイベントは常に起動開始判定を行う
    var _Game_CommonEvent_update = Game_CommonEvent.prototype.update;
    Game_CommonEvent.prototype.update = function() {
        _Game_CommonEvent_update.apply(this, arguments);
        if (this._commonEventId === 31 && !this._interpreter) {
            this.refresh();
        }
    };
    //便利変数の操作では更新しない　軽量化のため
    Game_Variables.prototype.setValue = function(variableId, value) {
        if (variableId > 0 && variableId < $dataSystem.variables.length) {
            if (typeof value === 'number') value = Math.floor(value);
            this._data[variableId] = value;
            if (![3, 4, 5, 6, 7].contains(variableId)) this.onChange();
        }
    };
    
    //ボタン仕様変更　ボタンスプライトをキー降下中の際にもhotFrameに設定　キーによる起動
//    Sprite_Button.prototype.updateFrame = function() {
//        var frame;
//        if (this._touching) frame = this._hotFrame;
//        else frame = this._coldFrame;
//        if (frame) this.setFrame(frame.x, frame.y, frame.width, frame.height);
//    };
    Sprite_Button.prototype.processTouch = function() {
        if (this.isActive() && this.active) {
            if ((TouchInput.isTriggered() && this.isButtonTouched()) ||
                (this._input && Input.isTriggered(this._input))) 
                if (!SceneManager._scene._buttons ||
                    !SceneManager._scene._buttons.contains(this) ||
                    !SceneManager._scene.isAnyButtonTouching()) this._touching = true;
            if (this._touching) {
                if (TouchInput.isPressed() && !this.isButtonTouched())
                    this._touching = false;
                else if (!TouchInput.isPressed() && !(this._input && Input.isPressed(this._input))) {
                    this._touching = false;
                    this.callClickHandler();
                }
            }
        } else {
            this._touching = false;
        }
    };
    
    //ボタン　透明部分の判定をなくす
    var _Sprite_Button_isButtonTouched = Sprite_Button.prototype.isButtonTouched;
    Sprite_Button.prototype.isButtonTouched = function() {
        if (!_Sprite_Button_isButtonTouched.apply(this, arguments)) return false;
        var px = this.canvasToLocalX(TouchInput.x) + this._frame.x;
        var py = this.canvasToLocalY(TouchInput.y) + this._frame.y;
        return this.bitmap.getPixel(px, py) !== '#000000';
    };
    //ボタン　テキストを表示　非アクティブで半透明
    var _Sprite_Button_update = Sprite_Button.prototype.update;
    Sprite_Button.prototype.update = function() {
        _Sprite_Button_update.apply(this, arguments);
        if (this._text) {
            if (!this._textSprite) {
                this._textSprite = new Sprite(new Bitmap(this.width + 8, this.height));
                this._textSprite.bitmap.fontSize = this._textFontSize || 12;
                this._textSprite.bitmap.outlineColor = 'rgba(73,41,10,1)';
                this.addChild(this._textSprite);
            }
            this._lastInfo = this._lastInfo || '';
            var info = [this._text, this._textColor].toString();
            if (this._lastInfo !== info) {
                this._lastInfo = info;
                this._textSprite.bitmap.clear();
                this._textSprite.bitmap.textColor = this._textColor || 'white';
                var a = 2;
                this._textSprite.bitmap.drawText(this._text, a, 0, this._textSprite.width - a * 2, 24, 'center');
            }
            this._textSprite.visible = this.visible;
            this._textSprite.move(this._textX || -4, this._textY || 0)
            this.opacity = this.isActive() && this.active ? 255 : 192;
        }
    };
    
    //スクロールバー判定からソートボタン判定を除外
    var _Window_Selectable_isTouchedInsideScrollBox = Window_Selectable.prototype.isTouchedInsideScrollBox;
    Window_Selectable.prototype.isTouchedInsideScrollBox = function() {
        if (SceneManager._scene._sortButton && SceneManager._scene._sortButton.isButtonTouched()) {
            return false;
        }
        return _Window_Selectable_isTouchedInsideScrollBox.apply(this, arguments);
    };
    
    //移動サーチを軽量化
    Game_Character.prototype.searchLimit = function() {
        return 3; //12;
    };

    //タッチ移動仕様変更
    Scene_Map.prototype.processMapTouch = function() {
        if (TouchInput.isTriggered() || this._touchCount > 0) {
            if (TouchInput.isPressed()) {
                if (this._touchCount === 0 || this._touchCount >= 2) {
                    var x = $gameMap.canvasToMapX(TouchInput.x);
                    var y = $gameMap.canvasToMapY(TouchInput.y);
                    $gameTemp.setDestination(x, y);
                }
                this._touchCount++;
            } else {
                this._touchCount = 0;
            }
        }
    };
    
    //bgm自動再生無効スイッチ 10
    var _Game_Map_autoplay = Game_Map.prototype.autoplay;
    Game_Map.prototype.autoplay = function() {
        if ($gameSwitches.value(10)) return;
        _Game_Map_autoplay.apply(this, arguments);
    };
    
    //自動でゲームオーバーにしない
    Scene_Base.prototype.checkGameover = function() {
    };
    
    Game_BattlerBase.prototype.hpRate = function() {
        return Math.ceil(this.hp) / this.mhp;
    };
    Game_BattlerBase.prototype.mpRate = function() {
        return this.mmp > 0 ? Math.ceil(this.mp) / this.mmp : 0;
    };
    
    //イベント中かどうかの判定を変更　何らかのウェイトありのときのみイベント中
    var _Game_Interpreter_setWaitMode = Game_Interpreter.prototype.setWaitMode;
    Game_Interpreter.prototype.setWaitMode = function(waitMode) {
        _Game_Interpreter_setWaitMode.apply(this, arguments);
        if (!this._noHideDisplays) $gameSwitches.setValue(3, true);
    };
    var _Game_Interpreter_wait = Game_Interpreter.prototype.wait;
    Game_Interpreter.prototype.wait = function(duration) {
        _Game_Interpreter_wait.apply(this, arguments);
        if (!this._noHideDisplays) $gameSwitches.setValue(3, true);
    };
    var _Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function() {
        _Scene_Map_update.apply(this, arguments);
        if (!$gameMap.isEventRunning() && $gameSwitches.value(3)) $gameSwitches.setValue(3, false);
    };
    
    //呼び出し先コモンイベントから呼び出し元を読み込める
    var _Game_Interpreter_setupChild = Game_Interpreter.prototype.setupChild;
    Game_Interpreter.prototype.setupChild = function(list, eventId) {
        _Game_Interpreter_setupChild.apply(this, arguments);
        this._childInterpreter._parent = this;
    };
    
    //マップの中心を常に半マスずらす
    Game_Player.prototype.centerY = function() {
        return (Graphics.height / $gameMap.tileHeight() - 1) / 2.0 - 0.5;
    };
    
//    //イベントをセーブから除外
//    var TN_Game_System_onBeforeSave      = Game_System.prototype.onBeforeSave;
//    Game_System.prototype.onBeforeSave = function() {
//        TN_Game_System_onBeforeSave.apply(this, arguments);
//        TN_gameMap = $gameMap._events;
//        $gameMap._events = [];
//    };
//    //除外したイベントデータを復帰
//    var TN_Scene_Save_onSavefileOk = Scene_Save.prototype.onSavefileOk;
//    Scene_Save.prototype.onSavefileOk = function() {
//        TN_Scene_Save_onSavefileOk.call(this);
//        $gameMap._events = TN_gameMap;
//    };
    
    //セーブするやつを整理
    DataManager.makeSaveContents = function() {
        // A save data does not contain $gameTemp, $gameMessage, and $gameTroop.
        var contents = {};
        contents.system       = $gameSystem;
        contents.screen       = $gameScreen;
//        contents.timer        = $gameTimer;
        contents.switches     = $gameSwitches;
        contents.variables    = $gameVariables;
        contents.selfSwitches = $gameSelfSwitches;
        contents.actors       = $gameActors;
        contents.party        = $gameParty;
//        contents.map          = $gameMap;
        contents.player       = $gamePlayer;
        
        contents.mapInfo      = [$gameMap.mapId(), $gamePlayer.x, $gamePlayer.y];
        return contents;
    };

    DataManager.extractSaveContents = function(contents) {
        $gameSystem        = contents.system;
        $gameScreen        = contents.screen;
        $gameTimer         = new Game_Timer();
        $gameSwitches      = contents.switches;
        $gameVariables     = contents.variables;
        $gameSelfSwitches  = contents.selfSwitches;
        $gameActors        = contents.actors;
        $gameParty         = contents.party;
        $gameMap           = new Game_Map();
        $gamePlayer        = contents.player;
        $gamePlayer._loadMapInfo = contents.mapInfo;
    };
    
    //常にリロード時読み込み直す
    Scene_Load.prototype.reloadMapIfUpdated = function() {
        $gamePlayer.reserveTransfer($gameMap.mapId(), $gamePlayer.x, $gamePlayer.y);
        $gamePlayer.requestMapReload();
    };
    Scene_Boot.prototype.reloadMapIfUpdated = Scene_Load.prototype.reloadMapIfUpdated;
    
    //時間を取得
    Game_System.prototype.getNowTime = function() {
        return (new Date()).getTime();
    };
    
    //popupを軽量化
    var _Sprite_Damage_update = Sprite_Damage.prototype.update;
    Sprite_Damage.prototype.update = function() {
        if (this._duration % 2 === 1) {
            this._duration--;
            return;
        }
        _Sprite_Damage_update.call(this);
    };
    Sprite_Damage.prototype.setupCriticalEffect = function() {
        this._updateScale = true;
        for (var i = 0; i < this.children.length; i++) {
            var sprite = this.children[i];
            sprite.scale.x = sprite.scale.y = 2.5;
        }
    };
    Sprite_Damage.prototype.updateFlash = function() {
        if (!this._updateScale) return;
        for (var i = 0; i < this.children.length; i++) {
            var sprite = this.children[i];
            if (sprite.scale.x > 1 && !sprite._ups) {
                sprite.scale.x -= 0.07;
                sprite.scale.y -= 0.07;
            } else {
                sprite.scale.x = sprite.scale.y = 1;
                sprite._ups = true;
            }
        }
    };
    
    Game_Screen.prototype.updateZoom = function() {
        var t = this._zoomScaleTarget;
        if (Math.abs(this._zoomScale - t) >= 0.01)
            this._zoomScale += (t - this._zoomScale) / 12;
        else this._zoomScale = t;
        if (this._zoomDuration > 0) {
            this._zoomDuration--;
        }
    };
    
    var _Window_Message_pdate = Window_Message.prototype.update;
    Window_Message.prototype.update = function() {
        _Window_Message_pdate.apply(this, arguments);
        if (NWS()._choiceOpen) {
            NWS()._choiceOpen = false;
            this.startMessage();
            this._showFast = true;
            this.updateMessage();
            this.startInput();
            this.updateMessage();
        }
    };
    
    //太字
    var _Bitmap__makeFontNameText      = Bitmap.prototype._makeFontNameText;
    Bitmap.prototype._makeFontNameText = function() {
        return (this.fontBold ? 'bold ' : '') + _Bitmap__makeFontNameText.apply(this, arguments);
    };
    var _Window_Base_processEscapeCharacter = Window_Base.prototype.processEscapeCharacter;
    Window_Base.prototype.processEscapeCharacter = function(code, textState) {
        switch (code) {
        case 'B':
            this.contents.fontBold = !this.contents.fontBold;
            break;
        case 'W':
            var value = !!Number(this.obtainEscapeParam(textState));
            if (![0, 3, 4].contains(_lg())) this._wrap = value;
            break;
        case 'S':
            this.contents.fontSize = this.obtainEscapeParam(textState);
            break;
        case 'X':
            textState.x += this.obtainEscapeParam(textState);
            break;
        case 'MX':
            textState.x -= this.obtainEscapeParam(textState);
            break;
        case 'Y':
            textState.y += this.obtainEscapeParam(textState);
            break;
        case 'MY':
            textState.y -= this.obtainEscapeParam(textState);
            break;
        default:
            _Window_Base_processEscapeCharacter.call(this, code, textState);
            break;
        }
    };
    var _Window_Base_processCharacter = Window_Base.prototype.processCharacter;
    Window_Base.prototype.processCharacter = function(textState) {
        switch (textState.text[textState.index]) {
        case '\n':
            if (!this._wrap) this.processNewLine(textState);
            else textState.x += 4, textState.index++;
            break;
        default:
            _Window_Base_processCharacter.apply(this, arguments);
            break;
        }
    };
    
    //メッセージを中央に表示したい
    var _Window_Message_newLineX = Window_Message.prototype.newLineX;
    Window_Message.prototype.newLineX = function() {
        var x = _Window_Message_newLineX.apply(this, arguments);
        if (!$gameSwitches.value(49)) return x;
        var ww = this.contents.width - 12 - x;
        var tw = this.drawTextEx(this._textState.text, 0, 999);
        return x + (ww - tw) / 2 - 30;
    };
    
    //最初からフェードアウト状態のマップ
    var _Game_Map_setup = Game_Map.prototype.setup;
    Game_Map.prototype.setup = function(mapId) {
        this._numRegionEnemies = [];
        _Game_Map_setup.apply(this, arguments);
        if ($dataMap.meta && $dataMap.meta.fadeOutStart) $gameScreen.startFadeOut(1);
    };
    
    //\i[]とかで数字以外も読み取れる
    Window_Base.prototype.obtainEscapeParam = function(textState) {
        var arr = /^\[([^\]]+)\]/.exec(textState.text.slice(textState.index));
        if (arr) {
            textState.index += arr[0].length;
            var a = arr[0].slice(1);
            a = a.substr(0, a.length - 1)
            return Number(a) || a;
        } else {
            return '';
        }
    };
    
    var _Scene_Map_fadeInForTransfer = Scene_Map.prototype.fadeInForTransfer;
    Scene_Map.prototype.fadeInForTransfer = function() {
        _Scene_Map_fadeInForTransfer.apply(this, arguments);
        $gameMap.rejudgeEvents();
    };
    
    //シーン移行時にスプライトを保持
    
    var _SceneManager_goto = SceneManager.goto;
    SceneManager.goto = function(sceneClass) {
        _SceneManager_goto.apply(this, arguments);
        var current = this._scene ? this._scene.constructor : null, next = sceneClass;
        if (current === Scene_Map && [Scene_Options, Scene_Save, Scene_Load].contains(next)) {
            this._saveSprites = this._scene._spriteset;
        }
    };
    
    Scene_Map.prototype.createSpriteset = function() {
        if (SceneManager._saveSprites) {
            this._spriteset = SceneManager._saveSprites;
            delete SceneManager._saveSprites;
        } else {
            this._spriteset = new Spriteset_Map();
        }
        this.addChild(this._spriteset);
    };
    
    Scene_Map.prototype.create = function() {
        Scene_Base.prototype.create.call(this);
        this._transfer = $gamePlayer.isTransferring();
        var mapId = this._transfer ? $gamePlayer.newMapId() : $gameMap.mapId();
        if (!SceneManager._saveSprites || !DataManager.isMapLoaded()) DataManager.loadMapData(mapId);
    };
    
    //ピクチャ1,2を最前面に
    Spriteset_Base.prototype.createPictures = function() {
    var width = Graphics.boxWidth;
        var height = Graphics.boxHeight;
        var x = (Graphics.width - width) / 2;
        var y = (Graphics.height - height) / 2;
        this._pictureContainer = new Sprite();
        this._pictureContainer.setFrame(x, y, width, height);
        for (var i = 3; i <= $gameScreen.maxPictures(); i++) {
            this._pictureContainer.addChild(new Sprite_Picture(i));
        }
        this._pictureContainer.addChild(new Sprite_Picture(1));
        this._pictureContainer.addChild(new Sprite_Picture(2));
        this.addChild(this._pictureContainer);
    };
    Game_Screen.prototype.maxPictures = function() {
        return 10;
    };
    
    //SE再生　キャラから再生
    Game_CharacterBase.prototype.playSe = function(seName, volume, pitch, noChangePan) {
        var pan = noChangePan ? 0 : this.sePan();
        var cx = Graphics.boxWidth / 2, cy = Graphics.boxHeight / 2;
        var d = Math.sqrt(Math.pow(this.screenX() - cx, 2) + Math.pow(this.screenY() - cy, 2)) / 48;
        AudioManager.playSe({
            name : seName,
            volume : (this.isOutOfScreen() ? 0 : Math.round((volume || 80) * (1 - d / 8 * 0.9))),
            pitch : Math.round(pitch || 100),
            pan : Math.round(pan)
        });
        this._playingSe = AudioManager._seBuffers[AudioManager._seBuffers.length - 1];
        this._playingSeInfo = [volume, noChangePan];
    };
    Game_CharacterBase.prototype.stopSe = function() {
        if (this._playingSe) {
            this._playingSe.stop();
            this._playingSe = null;
            this._playingSeInfo = [];
        }
    };
    
    Game_CharacterBase.prototype.sePan = function() {
        var x = $gameMap.adjustX(this.x);
        var pan = ((x - 7.5) / 15 * 100).clamp(-100, 100);
        return pan;
    };
    
    //拡張文字描写
    Window_Base.prototype.drawTextKNH = function(text, x, y, w, align, align2, fontSize, color) {
        var lastNoResetFontSettings = this._noResetFontSettings;
        this._noResetFontSettings = true;
        var lastFontSize = this.contents.fontSize;
        var lastColor = this.contents.textColor;
        if (fontSize) this.contents.fontSize = fontSize;
        if (color) this.changeTextColor(this.textColor(color));
        
        var width, height;
        
        if (align2 && align2 !== 'down') {
            this.drawTextEx(text, -999, -999);
            if (align2 === 'center') y -= this._lastTextExHeight / 2;
            if (align2 === 'up')     y -= this._lastTextExHeight;
        }
        
        if (align && align !== 'left') {
            var texts = text.split('\n');
            var dx = x, dy = y, dw = 0;
            for (var i = 0; i < texts.length; i++) {
                dw = this.drawTextEx(texts[i], -999, -999);
                if (align === 'center') dx = x + (w - dw) / 2;
                if (align === 'right')  dx = x + w - dw;
                width = Math.max(this.drawTextEx(texts[i], dx, dy), width || 0);
                height += this._lastTextExHeight;
                dy += this._lastTextExHeight;
            }
        } else {
            width = this.drawTextEx(text, x, y);
            height = this._lastTextExHeight;
        }
        
        this._noResetFontSettings = lastNoResetFontSettings;
        this.contents.fontSize = lastFontSize;
        this.contents.textColor = lastColor;
        
        return {x : x, y : y, w : Math.ceil(width), h : Math.ceil(height)};
    };
    
    
})();
