//=============================================================================
// KNH_HKCommand.js
//=============================================================================

/*:
 * @plugindesc ローグライクシステム用コマンドプラグイン
 * @author こんにちは
 *
 * @help 長いコマンドとかをここに書いた
 */

(function() {
    'use strict';
    var pluginName = 'KNH_HKCommand';
    
    var getNumber = function(text, max, min) {
        text = applyCharacters(text);
        if (isNaN(text)) return NaN;
        return Math.max(Math.min(Number(text), max || Infinity), min || -Infinity);
    };
    
    var applyCharacters = function(text) {
        if (text === null) text = '';
        text = '' + text;
        text = text.replace(/\\/g, '\x1b');
        text = text.replace(/\x1b\x1b/g, '\\');
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bN\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameActors.actor(parseInt(arguments[1], 10)) : null;
            return actor ? actor.name() : '';
        }.bind(this));
        text = text.replace(/\x1bP\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameParty.members()[parseInt(arguments[1], 10) - 1] : null;
            return actor ? actor.name() : '';
        }.bind(this));
        if ($dataSystem) text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
        return text;
    };
    
    var getMeta = function(text, str) {
        var re = /<([^<>:]+)(:?)([^>]*)>/g;
        var meta = {};
        for (;;) {
            var match = re.exec(text);
            if (match) {
                if (match[2] === ':') {
                    meta[match[1]] = match[3];
                } else {
                    meta[match[1]] = true;
                }
            } else {
                break;
            }
        }
        return meta['' + str];
    };
    
    var rand = function(min,max) {
        return Math.floor(Math.random() * (max + 1 - min)) + min;
    };
    
    //-----------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------
    
    var goldIconIndex = 313;
    var careAnimationId = 4;
    var buffAnimationId = 5;
    var debuffAnimationId = 6;
    
    //-----------------------------------------------------------------------------
    // 魔王城コマンド
    //-----------------------------------------------------------------------------
    
    Game_Interpreter.prototype.setWeaponsClone = function(type) {
        NWS()._weaponsClone = [].concat(NWS()._weapons);
        NWS()._weaponsClone2 = [];
        NWS()._weaponsClone.forEach(function(w) {
            if (!w) return;
            var item = $gameSystem.newItem(w._itemId);
            NWS()._weaponsClone2.push(Object.assign(item, w));
        })
        NWS()._weaponsClone2.sort(function(a, b) {
            return a.power() * a.object().repeats - b.power() * b.object().repeats;
        })
        
        if (type === 1) { //中間地点
            NWS()._weapons = [];
            var len = NWS()._weaponsClone2.length;
            for (var floor = 1; floor <= 9; floor++) {
                var _f = floor + (floor >= 11 ? -1 : 0);
                for (var i = 1; i <= len; i++) {
                    if (Math.floor((18 / len) * i) - Math.floor((18 / len) / 2) === _f &&
                        !NWS()._weapons.contains(NWS()._weaponsClone2[i - 1])) {
                        NWS()._weapons.push(NWS()._weaponsClone2[i - 1]);
                    }
                }
            }
            if (!NWS()._weapons.contains(NWS()._equip)) $gamePlayer.removeWeapon(true);
        } else if (type === 2) { //最上階
            NWS()._weapons = [];
            NWS()._weaponsClone2.forEach(function(w) {if (w) NWS()._weapons.push(w)});
        } else { //1階から
            $gamePlayer.removeWeapon(true);
            NWS()._weapons = [];
        }
    };
    
    Game_Interpreter.prototype.getMaojoWeapons = function() {
        NWS()._weapons.forEach(function(w) {
            if (!w) return;
            if (NWS()._weaponsClone2.contains(w)) return;
            if (!$gameSystem.stocksMax()) NWS()._stocks.push(w);
        })
    };
    
    Game_Interpreter.prototype.maojoPutEnemies = function() {
        this.__list = [
            [16, 6, 41],
            [22, 23, 24],
            [10, 13, 47, 44],
            [27, 28, 31, 38],
            [7, 16, 19, 36],
            [53, 33, 50, 53], 
            [14, 50, 50, 50, 14],
            [31, 16, 17, 16, 31],
            [27, 51, 28, 51, 48], 
            [22, 25, 23, 25, 24]
        ];
        this.__list2 = [
            [14, 17, 25, 39, 44, 48, 51],
            [11, 20, 29, 34, 42]
        ];
        
        this.__xy = [];
        for (var x = 0; x < $dataMap.width; x++) for (var y = 0; y < $dataMap.height; y++)
            if ($gameMap.regionId(x, y) === 23) this.__xy.push([x, y]);
        var n = $gameVariables.value(15);
        if (n <= 10) {
            this.__enemies = this.__list[n - 1];
        } else {
            this.__enemies = [];
            var l1 = this.__list2[1], l2 = l1.concat(this.__list2[0]);
            var max = Math.min(5 + Math.floor((n - 10) / 4) + _rand(-1, 1), 8);
            for (var i = 0; i < 2; i++) this.__enemies.push(l1[Math.randomInt(l1.length)]);
            while (this.__enemies.length < max) this.__enemies.push(l2[Math.randomInt(l2.length)]);
        }
        var num = this.__enemies.length;
        for (var i = 0; i < num; i++) {
            var ind = Math.randomInt(this.__xy.length)
            var xy = this.__xy[ind];
            $gameSystem.bornEnemy(this.__enemies.shift(), xy[0], xy[1], 0);
            this.__xy.splice(ind, 1);
        }
        $gameVariables.setValue(17, num);
    };
    Game_Interpreter.prototype.maojoPutWeapon = function() {
        var xyList = [];
        for (var x = 0; x < $dataMap.width; x++) for (var y = 0; y < $dataMap.height; y++)
            if ($gameMap.regionId(x, y) === 25) xyList.push([x, y]);
        var list = [
            ['草Ａ', '草Ｂ','草Ｃ', 'きれいな花', 'でかい岩', '丸テーブル', 'ツボ', 'クリスタル', 'しかばね', '動物のしかばね'],
            ['枯れ木', '倒れた木', 'くさい花', 'ススキ', 'ヤバそうなキノコ', '柱', 'ゴミ箱', 'フタつきのツボ', '中ぐらいテーブル', '花瓶テーブル', '魚の丸焼き', '高級ワイン'],
            ['大きな土の山', 'でかいクリスタル', '薪', '街灯', '水桶', '石の墓', '大きめのテーブル', '鏡テーブル', '水晶テーブル', 'クローゼット', '料理のようなもの', 'ベッド'],
            ['落とし穴', '道案内看板', '井戸', 'かかし', '冷蔵庫', 'コンビニ商品棚', '冷凍食品コーナー'],
            ['木箱タワー', 'ダブルベッド', 'タンス', 'たくさんのリンゴ', '戦士の像']
        ];
        var floor = $gameVariables.value(15);
        var array = [[0, 0, 0, 1, 1, 2], [0, 1, 1, 2, 2, 3], [0, 1, 2, 3, 3, 4], [1, 2, 3, 3, 4, 4]][Math.floor(floor / 5)];
        _shuffle(array), array.length = _rand(3, 5);
        array.forEach(function(a) {
            var name = list[a][Math.randomInt(list[a].length)];
            var event = DataManager.searchDataItem($dataTemplateEvents, 'name', name);
            if (event) {
                var xyInd = Math.randomInt(xyList.length);
                var event = $gameMap.spawnEvent(event.id, xyList[xyInd][0], xyList[xyInd][1], true);
                event._noRevivalWeapon = true;
                xyList.splice(xyInd, 1);
            }
        })
        $gameMap.rejudgeEvents();
    };
    Game_Interpreter.prototype.maojoSetWeapon = function() {
        var ch = this.character(2);
        $gameSelfSwitches.setValue([$gameMap.mapId(), ch._eventId, 'B'], false);
        var len = NWS()._weaponsClone2.length;
        var floor = $gameVariables.value(15);
        if (floor >= 11) floor--;
        for (var i = 1; i <= len; i++) {
            if (Math.floor((18 / len) * i) - Math.floor((18 / len) / 2) === floor &&
                !NWS()._weapons.contains(NWS()._weaponsClone2[i - 1])) {
                ch.__w = NWS()._weaponsClone2[i - 1];
                return;
            }
        }
        $gameMap.eraseEvent(ch._eventId);
    };
    Game_Interpreter.prototype.maojoSetEnemies = function() {
        var enemies = $gameSystem.getAllDEvents(1);
        enemies.forEach(function(en) {
            en._D.removeState(9); en.setDirection(2);
            if ($gameVariables.value(19) >= 11) {
                if ($gameVariables.value(15) <= 10) en._powerUp = 2 + 0.8 * $gameVariables.value(15) / 10;
                else en._powerUp = 1.5 + 1 * ($gameVariables.value(15) - 10) / 10;
            } else {
                if ($gameVariables.value(15) <= 10) en._powerUp = 1 + 1.2 * $gameVariables.value(15) / 10;
                else en._powerUp = 1.5 + 1 * ($gameVariables.value(15) - 10) / 10;
            }
            en._D.gainHp(9999);
        });
    };
    
    //必殺奥義の選択肢作成
    Game_Interpreter.prototype.setSkillChoiceList = function() {
        var isReady = $gamePlayer._sp >= 100, throwing = $gameSwitches.value(51), flag = $gameVariables.value(12);
        var list = this.__list = [];
        
        NWS()._cannnotChoiceIndex = [];
        if (!isReady) {
            var a = throwing ? 1 : 0
            var max = Math.min($gameVariables.value(12), 4);
            for (var i = a; i < a + max; i++)
            NWS()._cannnotChoiceIndex.push(i);
        }
        if (throwing && NWS()._equip._lock) NWS()._cannnotChoiceIndex.push(0);
        if (isReady || throwing) $gamePlayer.playSe('Fire1');
        
        var sl = this.__skillList = ['武器を投げる', 'ザ・ミラクル斬り', 'ぐるぐる回転斬り', 'ロック・オン', '斬り捨てゴメン']
        if (throwing) list.push(sl[0]);
        if (flag >= 1) list.push(sl[1]);
        if (flag >= 2) list.push(sl[2]);
        if (flag >= 3) list.push(sl[3]);
        if (flag >= 4) list.push(sl[4]);
        list.push('やめる');
        
        this.setupChoices([list, list.length - 1, 0, 2, 0]);
        this.setWaitMode('message');
    };
    Game_Interpreter.prototype.skillChoiceResult = function() {
        var list = this.__list, sl = this.__skillList, ind = this._branch[this._indent];
        if (sl.contains(list[ind])) {
            var sId = 60 + sl.indexOf(list[ind]);
            $gamePlayer.useSkill(sId);
            if (sId >= 61) $gamePlayer._sp = 0;
        }
    };
    
    //-----------------------------------------------------------------------------
    // コモン「アクション本体」から実行
    //-----------------------------------------------------------------------------
    
    Game_Interpreter.prototype.useItem = function() {
        var item, clear;
        if (this._useItem) {
            item = this._useItem;
            item.use(this._subject); //使用可能回数の消費
            if (item._requestErase) {
                if (this._subject._footId) $gameMap.eraseEvent(this._subject._footId);
                else this._subject.eraseItem(item);
            }
            this._subject._useItem = this._subject._needUseFoot = null;
        }
    };
    
    Game_System.prototype.setSister = function() {
        var mlvs = [6, 6, 6, 6, 3, 4, 100, 100, 100];
        var golds = [2000, 5000, 3500, 4000, 5000, 6000, 0, 0, 0];
        var ups = [2.5, 2, 2.5, 2.5, 2, 5, 1, 1, 1]
        var list = NWS()._sisterList;
        for (var i = 0 ; i < 9; i++) {
            list[i] = {name : '', lv : 1, mlv : mlvs[i], gold : golds[i], up : ups[i],
                       value : [], value2 : [], text : '', text2 : ''};
        }
        this.setSisterName();
        this.setSisterValue();
    };
    
    Game_System.prototype.setSisterName = function() {
        var list = NWS()._sisterList;
        var names = ['応援', '武器庫拡張', '呼び込み', '商売術の勉強', '商品棚拡張', '妹のプレゼント',
                     '最大ＨＰアップ', '攻撃力アップ', '運アップ'];
        for (var i = 0 ; i < 9; i++) if (list[i]) list[i].name = _lgCheck(names[i]);
    };
    
    Game_System.prototype.setSisterValue = function() {
        var list = NWS()._sisterList;
        if (!list || !list[0]) return;
        list[0].value = [1, 1.1, 1.3, 1.5, 1.7, 2];     list[0].text = _lgCheck('✕%1');
        list[1].value = [48, 64, 80, 96, 112, 128];     list[1].text = _lgCheck('%1個');
        list[2].value = [20, 15, 12, 9, 7, 5];          list[2].text = _lgCheck('最大 %1分');
        list[3].value = [1, 1.2, 1.5, 1.8, 2.2, 2.5];   list[3].text = _lgCheck('✕%1');
        list[4].value = [1, 2, 3];                      list[4].text = _lgCheck('%1個');
        
        var array;
        //HP
        array = list[6].value = [];
        for (var i = 0; i < 99; i++) array.push(
            i === 0 ? 120 : Math.min(Math.ceil((array[array.length - 1]) + 10 * Math.pow(0.8, (i - 1) / 18)), 737)
        );
        list[6].text = '%1';
        //ATK
        array = list[7].value = [];
        for (var i = 0; i < 99; i++) array.push(
            i === 0 ? 0 : Math.min((array[array.length - 1]) + 1 * Math.pow(0.8, (i - 1) / 13.6), 50)
        );
        array.forEach(function(a, i){array[i] = Math.ceil(a)});
        list[7].text = '+%1';
        //運
            //耐久値減少確率
            array = list[8].value = [];
            for (var i = 0; i < 99; i++) array.push(
                i === 0 ? 100 : Math.max(Math.floor((array[array.length - 1]) -  5 * Math.pow(0.8, i - 1)), 0)
            );
            //回避率
            array = list[8].value2 = [];
            for (var i = 0; i < 99; i++) array.push(
                i === 0 ? 5 : Math.min((array[array.length - 1]) + 1 * Math.pow(0.8, (i - 1) / 11.5), 50)
            );
            array.forEach(function(a, i){array[i] = Math.ceil(a)});
            list[8].text = _lgCheck('%1％'), list[8].text2 = _lgCheck('%1％');
    };
    
    Game_System.prototype.sisterParam = function(index, type) {
        var list = NWS()._sisterList[index];
        if (!list) return 0;
        return list['value' + (type === 2 ? 2 : '')][list.lv - 1];
    };
    
    Game_Interpreter.prototype.sister = function(n) {
        
        if (n === 1) {
            
            NWS()._choiceText = [];
            NWS()._cannnotChoiceIndex = [];
            var list = NWS()._sisterList;
            for (var i = 0 ; i < 6; i++) {
                var text = [(list[i].name + '　　　　　').slice(0, 7),
                            (list[i].name + '               ').slice(0, 21),
                            (list[i].name + '　　　　　　　').slice(0, 8),
                            (list[i].name + '　　　　').slice(0, 6),
                            (list[i].name + '　　　　').slice(0, 6)
                           ][_lg()];
                text += ' %1  %2';
                var lv = list[i].lv >= list[i].mlv ? '\\c[14]Lv.MAX\\c[0]' : 'Lv.' + list[i].lv;
                var gold = list[i].lv < list[i].mlv ? (('     ' + list[i].gold).slice(-6) + '\\g') : '';
                NWS()._choiceText.push(text.format(lv, gold));
                if (list[i].lv >= list[i].mlv || $gameParty.gold() < list[i].gold) NWS()._cannnotChoiceIndex.push(i);
            }
            NWS()._choiceText.push('やめる');
            
        } else if (n === 2) {
            if (WindowManager.windows['sister']) WindowManager.hide('sister');
        }
    };
    
    var sisterTexts = [
//        '闘志溢れる儀式を行う。\n\\n[1]の攻撃力が上昇する。\n攻撃力 %1',
//        '守りを強化する儀式を行う。\n\\n[1]の防御力が上昇する。\n防御力 %1',
//        '運気を高める儀式を行う。\n\\n[1]の会心率と回避率が上昇する。\n会心率 %1\n回避率 %2',
//        'おまじないをしてもらう。\n武器の耐久値が減少する確率が下がる。\n耐久値減少確率 %1',
        'ぽん美に応援してもらう。\nもらえるすべての経験値が増加する。\n経験値 %1 ',
        '武器庫を拡張する。\n収納しておける武器の数が増える。\n武器庫の容量 %1 ',
        'お客さんの呼び込みをしてもらう。\n武器の売れる速度が上がる。\n売れる時間 %1 ',
        'ぽん美に商売術を学んでもらう。\n武器の売れる値段が増加する。\n売れる価格 %1 ',
        '商品棚をひとつ増やす。\n一度に売れる武器の数が増える。\n商品棚の数 %1 ',
        '妹から嬉しいプレゼントを受け取る。\nどんなプレゼントかはお楽しみ。'
    ]
    
    Game_Interpreter.prototype.updateSister = function() {
        var mw = SceneManager._scene._messageWindow;
        var cw = mw._choiceWindow;
        if (!cw.active) return;
        if (!WindowManager.windows['sister']) {
            var my = SceneManager._scene._messageWindow.y;
            WindowManager.show('sister', 0, 0, 0, 0);
            var wd = WindowManager.windows['sister'];
            wd.openness = 0;
            wd.standardFontSize = function() {return 16};
            this.character(0)._lastCwInfo = null;
        }
        var wd = WindowManager.windows['sister'];
        var ind = cw.index();
        var info = [ind].toString();
        if (!this.character(0)._lastCwInfo || this.character(0)._lastCwInfo !== info) {
            this.character(0)._lastCwInfo = info;
            wd.openness = 0;
            if (ind >= 6) return;
            var list = NWS()._sisterList[ind];
            var text = _lgCheck(sisterTexts[ind]).format(
                (list.lv >= list.mlv ? '\\c[14]%1\\c[0]' : '%1 → \\c[14]%2\\c[0]').format(
                    list.text.format(list.value[list.lv - 1]), list.text.format(list.value[list.lv])
                ),
                (list.lv >= list.mlv ? '\\c[14]%1\\c[0]' : '%1 → \\c[14]%2\\c[0]').format(
                    list.text2.format(list.value2[list.lv - 1]), list.text2.format(list.value2[list.lv])
                )
            );
            var texts = text.split('\n');
            wd.width = 0;
            for (var i = 0; i < texts.length; i++)
                wd.width = Math.max(wd.drawTextEx(texts[i], 0, -999), wd.width);
            wd.width += 20;
            wd.height = 24 + 24 * texts.length;
            wd.x = cw.x - wd.width;
            wd.y = (cw.y + 12 + ind * 28).clamp(cw.y, cw.y + 6 * 28 - wd.height);
            wd.createContents();
            wd.drawTextEx(text, 1, 0);
            wd.open();
        }
    };
    
    
    var skillHelpTexts = {};
    skillHelpTexts['武器を投げる'] = 
        '装備中の武器を直線上に投げつける。\n武器を失うが、必ず会心攻撃となり\n武器の追加効果が必ず発生する。'
    skillHelpTexts['ザ・ミラクル斬り'] = 
        '目の前の敵に1.8倍の威力で攻撃し、\n与えたダメージのぶんだけ回復する。'
    skillHelpTexts['ぐるぐる回転斬り'] = 
        '回転しながら周囲の敵に\n2.2倍の威力で攻撃し、\n遠くにぶっ飛ばす。'
    skillHelpTexts['ロック・オン'] = 
        '5ターンの間武器が必ず命中し、\n武器の効果が100％発生するようになる。'
    skillHelpTexts['斬り捨てゴメン'] = 
        '直線の敵に2.0倍の威力で攻撃し、\n一番遠い敵と場所を入れ替わる。\n喰らった敵は2ターン石化する。'
    
    skillHelpTexts['Weapon Throwing'] = 
        'Throw a weapon equipped on a\nstraight line.You lose a weapon,\nbut it will be always a critical hit\nand an additional effect of\nthe weapon will always occur.'
    skillHelpTexts['Miracle Slash'] = 
        'Attack with enemy in front\nof you with 1.8 times the power\nand recovers the damage value\ngiven.'
    skillHelpTexts['Spin Attack'] = 
        'While spinning, attack\nenemies of the surroundings\nwith 2.2 times the power and\nblow them away in the distance.'
    skillHelpTexts['Lock On'] = 
        'For 5 turns the weapon always\nhits and the effects of weapons\nwill appear to be 100%.'
    skillHelpTexts['Sorry Not Sorry'] = 
        'Attack with enemies on the\nline with 2.0 times the power,\nand switche place with the\nfarthest enemy.\nThe Attacked enemies are\npetrified 2 turns.'
	
    skillHelpTexts['무기 투척'] = 
        '장비중인 무기를 직선 상에 던진다.\n무기를 잃어 버리지 만, 반드시 크리티컬 히트가 발생하고\n무기의 추가 효과가 반드시 발생한다.'
    skillHelpTexts['THE 미라클 베기'] = 
        '눈앞의 적에게 1.8 배의\n위력으로 공격하고 준\n데미지의만큼 회복한다.'
    skillHelpTexts['빙글빙글 회전베기'] = 
        '회전하면서 주변의 적에게\n2.2 배의 위력으로 공격하고\n멀리 채질하다.'
    skillHelpTexts['록 온'] = 
        '5 턴간 무기가 반드시 명중\n무기의 효과가 100 % 발생하게된다.'
    skillHelpTexts['정당방위 베기'] = 
        '선형 적에게 2.0 배의 위력으로\n공격하고 가장 먼 적을과 위치를\n바뀐다. 공격을받은 적은\n2 차례 석화한다.'
	
    skillHelpTexts['武器投擲'] = 
        '把武器裝在一條直線上。\n你失去了一把武器，但它將永遠是一個重擊，\n並且武器的額外效果將永遠發生。'
    skillHelpTexts['THE‧奇蹟斬'] = 
        '用1.8倍的力量攻擊你面前的\n敵人並恢復給定的傷害值。'
    skillHelpTexts['轉轉迴旋斬'] = 
        '在繞過周圍的敵人的同時\n以2.2倍的力量攻擊，並在\n遠處將其吹走。'
    skillHelpTexts['LOCK‧ON'] = 
        '對於5轉，武器總是命中，\n武器的效果將100％生成。'
    skillHelpTexts['斬捨御免'] = 
        '它以2.0倍的力量攻擊線的\n敵人，並與最遠的敵人交換\n位置。被攻擊的敵人被石化了\n2轉。'
	
    skillHelpTexts['武器投掷'] = 
        '把武器装在一条直线上。\n你失去了一把武器，但它将永远是一个重击，\n并且武器的额外效果将永远发生。'
	skillHelpTexts['THE‧奇迹斩'] =
        '用1.8倍的力量攻击你面前的\n敌人并恢复给定的伤害值。 '
    skillHelpTexts['转转回旋斩'] =
        '在绕过周围的敌人的同时\n以2.2倍的力量攻击，并在\n远处将其吹走。 '
    skillHelpTexts['LOCK‧ON'] =
        '对于5转，武器总是命中，\n武器的效果将100％生成。 '
    skillHelpTexts['斩舍御免'] =
        '它以2.0倍的力量攻击线的\n敌人，并与最远的敌人交换\n位置。被攻击的敌人被石化了\n2转。 '
    
    Game_Interpreter.prototype.showSkillHelp = function() {
        WindowManager.show('skillHelp', 0,0,0,0);
        WindowManager.windows['skillHelp'].standardFontSize = function() {return 16};
        var cw = SceneManager._scene._messageWindow._choiceWindow;
        var _update = WindowManager.windows['skillHelp'].update;
        WindowManager.windows['skillHelp'].update = function() {
            _update.apply(this, arguments);
            if (cw.isClosed()) return this.close();
            var info = [cw.index()].toString();
            if (!this._lastCwInfo || this._lastCwInfo !== info) {
                this._lastCwInfo = info;
                if (!cw._list[cw.index()] || !skillHelpTexts[cw.commandName(cw.index())]) {
                    this.close();
                } else {
                    this.openness = 0;
                    var text = skillHelpTexts[cw.commandName(cw.index())];
                    var texts = text.split('\n');
                    this.width = 0;
                    for (var i = 0; i < texts.length; i++)
                        this.width = Math.max(this.drawTextEx(texts[i], 0, -999), this.width);
                    this.width += 26;
                    this.height = 24 + 24 * texts.length;
                    this.x = 24;
                    this.y = 24;
                    this.createContents();
                    this.drawTextEx(text, 1, 0);
                    this.open();
                }
                var skillId = cw.commandName(cw.index()) === 'やめる' ? 0 : 60 + ($gameSwitches.value(51) ? 0 : 1) + cw.index();
                $gameVariables.setValue(14, skillId);
            }
        }
    };
    
    Game_CharacterBase.prototype.setWepAnime = function(xyAdd, countAdd, countSet, wait) {
        var d = this._direction;
        if (d === 2 || d === 8) this._additionalY += d === 2 ? xyAdd : -xyAdd;
                           else this._additionalX += d === 4 ? -xyAdd : xyAdd;
        if (countAdd) this._weaponAnime += countAdd;
        if (countSet) this._weaponAnime = countSet;
        this._waitCount = (wait || 1) * (1 + (this._wepAnimeSpeed || 0)) - 1;
    };
    
    var levelUpTexts = [
        '\\n[1]の最大ＨＰがアップ！\n最大ＨＰ %1 ',
        '\\n[1]の攻撃力がアップ！\nすべての武器の\\i[469]威力が上昇する。\n\\i[469]上昇値 %1 ',
        '\\n[1]の運がアップ！\n武器の\\i[470]耐久値減少確率が下がる。\nまた、\\n[1]の回避率が上昇する。\n\\i[470]減少確率 %1 \n回避率 %2 '
    ]
    var levelDownTexts = [
        '\\n[1]の最大ＨＰがダウン！\n最大ＨＰ %1 ',
        '\\n[1]の攻撃力がダウン！\nすべての武器の\\i[469]威力が低下する。\n\\i[469]上昇値 %1 ',
        '\\n[1]の運がダウン！\n武器の\\i[470]耐久値減少確率が上がる。\nまた、\\n[1]の回避率が低下する。\n\\i[470]減少確率 %1 \n回避率 %2 '
    ]
    
    Game_Interpreter.prototype.updateLevelUp = function(down) {
        var mw = SceneManager._scene._messageWindow;
        var cw = mw._choiceWindow;
        if (!cw.active) return;
        if (!WindowManager.windows['levelUp']) {
            var my = SceneManager._scene._messageWindow.y;
            WindowManager.show('levelUp', 0, 0, 0, 0);
            var wd = WindowManager.windows['levelUp'];
            wd.openness = 0;
            wd.standardFontSize = function() {return 16};
            this._lastCwInfo = null;
        }
        var wd = WindowManager.windows['levelUp'];
        var ind = cw.index();
        var info = [ind].toString();
        if (!this._lastCwInfo || this._lastCwInfo !== info) {
            this._lastCwInfo = info;
            wd.openness = 0;
            var list = NWS()._sisterList[ind + 6];
            var text = _lgCheck((down ? levelDownTexts : levelUpTexts)[ind]);
            if (!text) return wd.close();
            var info = down ? [4, list.lv - 1, list.lv - 2] : [14, list.lv - 1, list.lv];
            var type = (!down && list.lv >= list.mlv) || (down && list.lv <= 1);
            text = text.format(
                (type ? '\\c[0]%1\\c[0]' : '%1 → \\c[%3]%2\\c[0]').format(
                    list.text.format(list.value[info[1]]), list.text.format(list.value[info[2]]), info[0]
                ),
                (type ? '\\c[0]%1\\c[0]' : '%1 → \\c[%3]%2\\c[0]').format(
                    list.text2.format(list.value2[info[1]]), list.text2.format(list.value2[info[2]]), info[0]
                )
            );
            var texts = text.split('\n');
            wd.width = 0;
            for (var i = 0; i < texts.length; i++)
                wd.width = Math.max(wd.drawTextEx(texts[i], 0, -999), wd.width);
            wd.width += 20;
            wd.height = 24 + 24 * texts.length;
            wd.x = cw.x - wd.width - 34;
            wd.y = cw.y + cw.height / 2 - wd.height / 2;
            wd.createContents();
            wd.drawTextEx(text, 1, 0);
            wd.open();
        }
        
    };
    
    //-----------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------
    
    Game_Event.prototype.updateShopItems = function(index) {
        this._updateShopItem = index;
        
    };
    Sprite_Character.prototype.updateShopItems = function() {
        var items = NWS()._shopItems[this._character._updateShopItem];
        if (!items) return;
        if (!this._shopItemSprite) {
            this._shopItemSprite = new Sprite(new Bitmap(5 * 48 + 24, 72));
            this._shopItemSprite.anchor.x = 0.5;
            this._shopItemSprite.anchor.y = 1;
            this._shopItemSprite.y = -12;
            this.addChild(this._shopItemSprite);
            this._drawWeaponWindow = new Window_Base(0, 0, 96, 96);
            this._drawWeaponWindow.update = function() {};
            this._drawWeaponWindow._noWeaponOutline = true;
//            this.addChild(this._drawWeaponWindow);
        }
        
        var sprite = this._shopItemSprite;
        var window = this._drawWeaponWindow;
        var infos = [];
        var time = $gameSystem.getNowTime();
        for (var i = 0; i < 5; i++) {
            if (!items[i]) infos.push('null');
            else if (items[i]._time <= time) infos.push('sold');
            else infos.push([items[i]._itemId, items[i].imageName()].join(','));
        }
        if (!this._lastSIInfo || this._lastSIInfo !== infos.toString()) {
            this._lastSIInfo = infos.toString();
            sprite.bitmap.clear();
            infos.forEach(function(info, i) {
                if (info === 'sold') {
                    var bmp = ImageManager.loadBitmap('img/weapons/', '_soldOut', 0, 0);
                    bmp.addLoadListener(function(){
                        sprite.bitmap.blt(bmp, 0, 0, 48, 48, i * 48 + 10, 12);
                    }.bind(this));
                } else if (info !== 'null') {
                    var _info = info.split(',');
                    var bmp = ImageManager.loadBitmap('img/weapons/', _info[1], 0, 0);
                    bmp.smooth = false;
                    bmp.addLoadListener(function(){
                        window.createContents();
                        window.contents.smooth = false;
                        window.drawWeapon(_info[0], _info[1], 0, 0, 68);
                        sprite.bitmap.blt(window.contents, 0, 0, 68, 68, i * 48, 0);
                    }.bind(this));
                }
            }, this)
        }
    };
    
    Sprite_Character.prototype.updatePoint = function(type) {
        var name = '_pointSprite' + type;
        if (!this[name]) {
            var image = ['eventPoint', 'endPoint'][type];
            this[name] = new Sprite(ImageManager.loadSystem(image));
            this[name].anchor.x = 0.5;
            this[name].anchor.y = 1;
            this[name]._baseY = -48 + (type === 1 ? 20 : 0);
            this[name]._count = 0;
            this[name].opacity = 0;
            this.addChild(this[name]);
        }
        var d = 80;
        var a = this[name]._count++ % d;
        var b = a < d / 4 ? -1 : a >= d / 2 && a < d * 3 / 4 ? 1 : 0;
        this[name].y = this[name]._baseY + b;
        this[name].opacity += 255 / 6 *
            (!$gameSystem.isBusy() && !$gameSystem.inMenu() && !$gameMap.isEventRunning() ? 1 : -1);
    };
    
    var _Sprite_Character_update = Sprite_Character.prototype.update;
    Sprite_Character.prototype.update = function() {
        _Sprite_Character_update.apply(this, arguments);
        if (!this._character.event || !this._character.event()) return;
        if (this._character._updateShopItem !== undefined) this.updateShopItems();
        if (this._character.event().meta.point && this._character._characterName) this.updatePoint(0);
        if (this._character._endPoint) this.updatePoint(1);
    };
    
    
})();
