//=============================================================================
// KNH_NWS_Effects.js
//=============================================================================

/*:
 * @plugindesc エフェクト集
 * @author こんにちは
 */

(function() {
    'use strict';
    var pluginName = 'KNH_NWS_Effects';
    
    var getNumber = function(text, max, min) {
        text = applyCharacters(text);
        if (isNaN(text)) return NaN;
        return Math.max(Math.min(Number(text), max || Infinity), min || -Infinity);
    };
    
    var applyCharacters = function(text) {
        if (text === null) text = '';
        text = '' + text;
        text = text.replace(/\\/g, '\x1b');
        text = text.replace(/\x1b\x1b/g, '\\');
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bN\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameActors.actor(parseInt(arguments[1], 10)) : null;
            return actor ? actor.name() : '';
        }.bind(this));
        text = text.replace(/\x1bP\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameParty.members()[parseInt(arguments[1], 10) - 1] : null;
            return actor ? actor.name() : '';
        }.bind(this));
        if ($dataSystem) text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
        return text;
    };
    
    var getMeta = function(text, str) {
        var re = /<([^<>:]+)(:?)([^>]*)>/g;
        var meta = {};
        for (;;) {
            var match = re.exec(text);
            if (match) {
                if (match[2] === ':') {
                    meta[match[1]] = match[3];
                } else {
                    meta[match[1]] = true;
                }
            } else {
                break;
            }
        }
        return meta['' + str];
    };
    
    var rand = function(min,max) {
        return Math.floor(Math.random() * (max + 1 - min)) + min;
    };
    
    //-----------------------------------------------------------------------------
    // 敵消滅エフェクト
    //-----------------------------------------------------------------------------
    
    Game_System.prototype.addEnemyDeadEffect = function(character, direction, power) {
        var scene = SceneManager._scene;
        var sprite = new Sprite_EnemyDead(character, direction, power);
        scene.addChild(sprite);
    };
    Game_System.prototype.eraseEnemyDeadEffect = function(sprite) {
        var scene = SceneManager._scene;
        scene.removeChild(sprite);
        sprite = null;
    };

    function Sprite_EnemyDead() {
        this.initialize.apply(this, arguments);
    }

    Sprite_EnemyDead.prototype = Object.create(Sprite_Base.prototype);
    Sprite_EnemyDead.prototype.constructor = Sprite_EnemyDead;

    Sprite_EnemyDead.prototype.initialize = function(character, direction, power) {
        Sprite_Base.prototype.initialize.call(this);
        this._characterSprite = 
            SceneManager._scene._spriteset._characterSprites.filter(function(sprite, index) {
                return sprite._character instanceof Game_Event && sprite._character === character;
            }, this)[0];
        if (!this._characterSprite || !this._characterSprite.bitmap) this.erase();
        
        this.anchor.x = 0.5;
        this.anchor.y = 0.5;
        this._direction = Math.PI / 2 - Math.atan2(direction[3] - direction[1], direction[2] - direction[0]);
        this._direction += rand(-Math.PI / 6 * 100, Math.PI / 6 * 100) / 100;
        this._direction = (this._direction + 2 * Math.PI) % (2 * Math.PI);
        this._rotationSpeed = Math.PI / 5 * [1, -1][rand(0, 1)] * rand(80, 120) / 100;
        this._count = 0;
        this._boundCount = 0;
        this._noUpdateAnimetionPosition = true;
        this._power = Math.ceil(power / 10).clamp(1, 6);
        
        var chSprite = this._characterSprite;
        var pw = chSprite.patternWidth();
        var ph = chSprite.patternHeight();
        var sx = (chSprite.characterBlockX() + chSprite.characterPatternX()) * pw;
        var sy = (chSprite.characterBlockY() + chSprite.characterPatternY()) * ph;
        this.bitmap = new Bitmap(pw, ph);
        this.bitmap.blt(chSprite.bitmap, sx, sy, pw, ph, 0, 0);
        
        this.x = character.screenX(), this.y = character.screenY() - ph / 2;
        this.parent = SceneManager._scene;
        this.startAnimation($dataAnimations[2], 0, 0);
        this._waitAnimation = true;
    };
    
    Sprite_EnemyDead.prototype.update = function() {
        Sprite_Base.prototype.update.call(this);
//        _p(this.x, this.y, this._direction, this.rotation, this.bitmap)
        if (this._waitAnimation && this.isAnimationPlaying()) return;
        else this._waitAnimation = false;
        if (this._noMoveCount-- > 0 && this._boundCount < 3) {
            this.x += (this._noMoveCount % 2) ? 2 : -2;
            return;
        }
        var speed = 7 + this._power * 2;
        this.x += speed * Math.sin(this._direction) + rand(-2, 2);
        this.y += speed * Math.cos(this._direction) + rand(-2, 2);
        this.rotation += this._rotationSpeed;
        
        var a = Math.max(this.bitmap.width, this.bitmap.height) / 2 - 12;
        var noMove = 4;
        if (this.x < a) {
            this.x = 2 * a - this.x;
            this._direction = -this._direction;
            this._boundCount++;
            this._noMoveCount = noMove;
            this.startAnimation($dataAnimations[71], 0, 0);
        }
        if (this.x >= Graphics.boxWidth - a) {
            this.x = 2 * (Graphics.boxWidth - a) - this.x;
            this._direction = -this._direction;
            this._boundCount++;
            this._noMoveCount = noMove;
            this.startAnimation($dataAnimations[71], 0, 0);
        }
        if (this.y < a) {
            this.y = 2 * a - this.y;
            this._direction = Math.PI - this._direction;
            this._boundCount++;
            this._noMoveCount = noMove;
            this.startAnimation($dataAnimations[71], 0, 0);
        }
        if (this.y >= Graphics.boxHeight - a) {
            this.y = 2 * (Graphics.boxHeight - a) - this.y;
            this._direction = Math.PI - this._direction;
            this._boundCount++;
            this._noMoveCount = noMove;
            this.startAnimation($dataAnimations[71], 0, 0);
        }
        this._direction = (this._direction + 2 * Math.PI) % (2 * Math.PI);
        
        if (this._boundCount >= this._power || this._count >= 15 + this._power * 15) {
            if (!this._erase) {
                this._erase = true;
                this.startAnimation($dataAnimations[3], 0, 0);
                this._waitAnimation = true;
                this.bitmap.clear();
            } else if (!this.isAnimationPlaying()) {
                this.erase();
            }
        } else {
            if (!(this._count % 6)) this.startAnimation($dataAnimations[72], 0, 0);
        }
        this._count++
    };
    
    Sprite_EnemyDead.prototype.erase = function() {
        $gameSystem.eraseEnemyDeadEffect(this);
    };
    
    var _Sprite_Animation_updatePosition = Sprite_Animation.prototype.updatePosition;
    Sprite_Animation.prototype.updatePosition = function() {
        if (this._target._noUpdateAnimetionPosition) {
            if (!this.x) this.x = this._target.x, this.y = this._target.y;
            return;
        }
        _Sprite_Animation_updatePosition.apply(this, arguments);
    };
    //-----------------------------------------------------------------------------
    // 破壊エフェクト
    //-----------------------------------------------------------------------------

    Sprite_Base.prototype.startBreakEffect = function(num) {
        var num = Math.min(this.bitmap.width * this.bitmap.height / (48 * 48) * num, num * 3);
        num = num.clamp(3, 90);
        if (!this._breakPartsSprites) this._breakPartsSprites = [];
        for (var i = 0; i < num; i++) {
            for (var j = 0; j < 100; j++) {
                var x = rand(0, this.bitmap.width);
                var y = rand(0, this.bitmap.height);
                var color = this.bitmap.getPixel(x, y);
                if (color !== '#000000') {
                    var parent = SceneManager._scene._spriteset._tilemap;
                    var px = this.parent.x + this.x + x - this.bitmap.width * this.anchor.x;
                    var py = this.parent.y + this.y + y - this.bitmap.height * this.anchor.y;
                    var sprite = new Sprite_BreakParts(this, px, py, color);
                    parent.addChild(sprite);
                    j = 100;
                }
            }
        }
        
    };
    Sprite_Base.prototype.eraseBreakEffect = function(sprite) {
        var parent = SceneManager._scene._spriteset._tilemap;
        parent.removeChild(sprite);
        this._breakPartsSprites.splice(this._breakPartsSprites.indexOf(sprite), 1);
        sprite = null;
    };
    
    function Sprite_BreakParts() {
        this.initialize.apply(this, arguments);
    }

    Sprite_BreakParts.prototype = Object.create(Sprite.prototype);
    Sprite_BreakParts.prototype.constructor = Sprite_BreakParts;

    Sprite_BreakParts.prototype.initialize = function(target, x, y, color) {
        Sprite.prototype.initialize.call(this);
        this._target = target;
        this.x = x, this.y = y;
        this.anchor.x = this.anchor.y = 0.5;
        this.z = 11;
        this.rotation = rand(0, 360) / 180 * Math.PI;
        this._dx = rand(-15, 15) / 10, this._dy = rand(-30, -10) / 10;
        this._dr = rand(-9, 9) / 180 * Math.PI;
        var size = rand(3, 7);
        this.bitmap = new Bitmap(size, size);
        this.bitmap.fillAll(color);
        var lColor = 'rgba(0, 0, 0, 0.5)';
        this.bitmap.fillRect(0, 0, size, 1, lColor);
        this.bitmap.fillRect(0, size - 1, size, 1, lColor);
        this.bitmap.fillRect(0, 1, 1, size - 2, lColor);
        this.bitmap.fillRect(size - 1, 1, 1, size - 2, lColor);
        if (!Math.randomInt(3)) this.bitmap.fillRect(0, 0, size, size / 2, lColor);
        this._count = 0;
        this._erase = false;
    }
    Sprite_BreakParts.prototype.update = function() {
        Sprite.prototype.update.call(this);
        this.x += this._dx;
        this.y += this._dy, this._dy += 0.1;
        this.rotation = (this.rotation + this._dr + Math.PI * 2) % (Math.PI * 2);
        this._count++;
        if (this._erase || this._count >= rand(30, 60)) {
            this._erase = true;
            this.opacity -= 255 / 15;
            if (this.opacity <= 0) this._target.eraseBreakEffect(this);
        }
    }
    
})();
