//=============================================================================
// KNH_NWSMenu.js
//=============================================================================

/*:
 * @plugindesc ローグライク風メニュープラグイン
 * @author こんにちは
 *
 * @help メニュー関連はここにまとめる
 */

(function() {
    'use strict';
    var pluginName = 'KNH_NWSMenu';
    
    var getNumber = function(text, max, min) {
        text = applyCharacters(text);
        if (isNaN(text)) return NaN;
        return Math.max(Math.min(Number(text), max || Infinity), min || -Infinity);
    };
    
    var applyCharacters = function(text) {
        if (text === null) text = '';
        text = '' + text;
        text = text.replace(/\\/g, '\x1b');
        text = text.replace(/\x1b\x1b/g, '\\');
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bN\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameActors.actor(parseInt(arguments[1], 10)) : null;
            return actor ? actor.name() : '';
        }.bind(this));
        text = text.replace(/\x1bP\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameParty.members()[parseInt(arguments[1], 10) - 1] : null;
            return actor ? actor.name() : '';
        }.bind(this));
        if ($dataSystem) text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
        return text;
    };
    
    var getMeta = function(text, str) {
        var re = /<([^<>:]+)(:?)([^>]*)>/g;
        var meta = {};
        for (;;) {
            var match = re.exec(text);
            if (match) {
                if (match[2] === ':') {
                    meta[match[1]] = match[3];
                } else {
                    meta[match[1]] = true;
                }
            } else {
                break;
            }
        }
        return meta['' + str];
    };
    
    var rand = function(min,max) {
        return Math.floor(Math.random() * (max + 1 - min)) + min;
    };
    
    var nFormat = function(num, char, cnt){
        var arr = [];
        for (var i = 0; i < cnt; i++) arr.push(char);
        return (arr.join('') + num).substr(-1*cnt); 
    };
    
    //-----------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------
    
    var goldIconIndex = 313;
    
    Game_System.prototype.inMenu = function() {
        var scene = SceneManager._scene;
        if (!scene._menuWindow) return false;
        return scene._menuWindow.active || (scene._menuWindow._menuMode && scene._menuWindow._menuMode !== 'exp');
    };
    Game_System.prototype.isExpPlaying = function() {
        var scene = SceneManager._scene;
        if (!scene._menuWindow) return false;
        return scene._menuWindow._menuMode === 'exp';
    };
    
    Game_System.prototype.canMenu = function() {
        var scene = SceneManager._scene;
        return !this.inMenu() && !this.isBusy() && !$gameMap.isEventRunning() &&
            $gameSystem.canAction() && $gamePlayer._DCount > 0;
    };
    
    Game_System.prototype.noMenu = function() {
        return $gameSwitches.value(8) || ($dataMap.meta && !!$dataMap.meta.noMenu);
    };
    
    //メニュー開いた時Destinationを無効化
    var _Game_Temp_setDestination = Game_Temp.prototype.setDestination;
    Game_Temp.prototype.setDestination = function(x, y) {
        if ($gameSystem.inMenu()) return;
        _Game_Temp_setDestination.apply(this, arguments);
    };
    
    //メニュー時決定ボタンでイベント起動を無効化
    var _Game_Player_canStartLocalEvents = Game_Player.prototype.canStartLocalEvents;
    Game_Player.prototype.canStartLocalEvents = function() {
        if ($gameSystem.inMenu()) return false;
        return _Game_Player_canStartLocalEvents.apply(this, arguments);
    };
    
    //-----------------------------------------------------------------------------
    // Scene_Map
    //-----------------------------------------------------------------------------
    
    var _Scene_Map_start = Scene_Map.prototype.start;
    Scene_Map.prototype.start = function() {
        _Scene_Map_start.apply(this, arguments);
        this._statusWindow = new Window_NWSStatus();
        this.addWindow(this._statusWindow);
        this._menuWindow = new Window_NWSMenu();
        this.addWindow(this._menuWindow);
        this._statesInfoWindow = new Window_NWSStatesInfo();
        this.addWindow(this._statesInfoWindow);
        
        this._weaponHelpWindow = new Window_NWSWeaponHelp();
        this._weaponHelpWindow.openness = 0;
        this.addWindow(this._weaponHelpWindow);
        
        this._weaponListWindow = new Window_NWSWeaponList();
        this._weaponListWindow._cancelAction = (function() {
            var wd = this._weaponListWindow;
            if (wd._commandWindow.active) {
                wd._commandWindow.close();
                wd._commandWindow.deactivate();
                wd.activate();
            } else if (wd._moving >= 0) {
                var movingIndex = wd._moving;
                wd._moving = -1;
                wd.select(movingIndex);
                wd.redrawItem(movingIndex);
                wd.activate();
            } else {
                wd.close();
            }
        }).bind(this);
        this._weaponListWindow.setHandler('ok', this.onWeaponOk.bind(this));
        this._weaponListWindow.setHandler('cancel', this._weaponListWindow._cancelAction.bind(this));
        this.addWindow(this._weaponListWindow);
        this._weaponListWindow.setHelpWindow(this._weaponHelpWindow);
        
        this._sortButton = new Sprite_Button();
        this._sortButton.bitmap = ImageManager.loadSystem('ButtonSet');
        this._sortButton.bitmap.addLoadListener(function(){
            var x = $gameVariables.value(34) ? 384 : 336;
            this._sortButton.setFrame(x, 0, 48, 48);
            this._sortButton.setColdFrame(x, 0, 48, 48);
            this._sortButton.setHotFrame(x, 48, 48, 48);
        }.bind(this));
        this._sortButton.x = 650 + 38, this._sortButton.y = 374 - 4;
        this._sortButton.visible = false;
        this._sortButton.setClickHandler(this.onSort.bind(this));
        this._sortButton._input = 'A';
        this._sortButton.active = true;
        this.addWindow(this._sortButton);
        
        this._editWindow = new Window_NameEdit($gamePlayer._D, 8);
        this._editWindow.move(120, 96 - 8, 480, 48), this._editWindow.openness = 0;
        this._editWindow._titleText = _lg() ? '' : '名前を入力';
        this._editWindow._cancelAction = this.onInputCancel.bind(this);
        this._inputWindow = new Window_NameInput(this._editWindow);
        this._inputWindow.setHandler('ok', this.onInputOk.bind(this));
        this._inputWindow.move(120, 150 - 8, 480, this._inputWindow.fittingHeight(9));
        this._inputWindow.openness = 0;
        this._inputWindow.deactivate();
        this.addWindow(this._inputWindow);
    };
    //アイテムOK
    Scene_Map.prototype.onWeaponOk = function() {
        
    };
    
    var _Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function() {
        _Scene_Map_update.apply(this, arguments);
        if (this._menuWindow.openness <= 0 && !$gameSwitches.value(3)) {
            $gameScreen.erasePicture(1), $gameScreen.erasePicture(2);
        }
        if ($gameSwitches.value(3)) {
            if (!$gameScreen.picture(4) || $gameScreen.picture(4)._scaleY === 113) {
                $gameScreen.showPicture(4, 'eventRunning', 1, Graphics.boxWidth / 2, Graphics.boxHeight / 2, 
                                        (Graphics.boxWidth + 96) * 100, 113, 255, 0);
                $gameScreen.picture(4)._targetScaleY = 100, $gameScreen.picture(4)._duration = 15;
            }
        } else {
            if ($gameScreen.picture(4) && $gameScreen.picture(4)._scaleY === 100)
                $gameScreen.picture(4)._targetScaleY = 113, $gameScreen.picture(4)._duration = 15;
        }
        
        if (this._menuWindow._menuMode === 'weaponList') {
            if (this._weaponListWindow.isClosing()) { //武器リスト画面を終了
                this._weaponHelpWindow.close();
                this._sortButton.visible = false;
                this._weaponListWindow._type = 0;
                this._menuWindow._menuMode = '';
                return;
            }
            //ソートボタン
            this._sortButton.visible = 
                this._weaponListWindow.active && ![3, 4].contains(this._weaponListWindow._type);
            this._sortButton._text = [
                ['種類順', '名前順', '使用順', '入手順', '威力順'],
                ['By number', 'By name', 'By last used', 'By date', 'By ATK'],
                ['반호순', '', '사용순', '최신순', '위력순'],
                ['按編號順序', '', '按用過的順序', '從新到舊', '威力的順序'],
                ['按编号顺序', '', '按用过的顺序', '从新到旧', '威力的顺序']
            ][_lg()][this._weaponListWindow._sortType];
            this._sortButton._textX = 24, this._sortButton._textY = 27;
            
            return;
        }
        
        if (this._menuWindow._menuMode === 'description') { //アイテム説明画面の更新
            if (this._menuWindow._bCount++ > 0 &&
                (Input.isTriggered('cancel') || TouchInput.isCancelled())) {
                SoundManager.playCancel();
                this._descriptionWindow._cancelAction();
            }
            return;
        }
        
        if (this._menuWindow._menuMode === 'displayLog') { //メッセージ履歴画面の更新
            if (this._menuWindow._bCount++ > 0 &&
                (Input.isTriggered('cancel') || TouchInput.isCancelled())) {
                SoundManager.playCancel();
                this._displayLogWindow._cancelAction();
            }
            return;
        }
        
        if (this._menuWindow._menuMode === 'exp') { //経験値画面の更新
            if (!this._expWindow.visible) {
                this._menuWindow._menuMode = '';
            }
            return;
        }
        
        if ($gameSystem.noMenu()) return;
        
        if ($gameSystem.canMenu() &&
            (Input.isTriggered('cancel') || TouchInput.isCancelled())) { //メニュー呼び出し
            this._menuWindow.startMenu();
            return;
        } else if (this._menuWindow.active && !$gameSystem.isBusy() &&
                   (Input.isTriggered('cancel') || TouchInput.isCancelled() ||
                    (TouchInput.isTriggered() &&
                     !Window_Selectable.prototype.isTouchedInsideFrame.call(this._menuWindow))))
        { //メニュー終わり
            this._menuWindow.endMenu();
            return;
        }
    };
    
    //ボタンなど
    var _Scene_Map_createDisplayObjects = Scene_Map.prototype.createDisplayObjects;
    Scene_Map.prototype.createDisplayObjects = function() {
        _Scene_Map_createDisplayObjects.apply(this, arguments);
        
        this._NWSLineSprite = new Sprite();
        this._NWSLineSprite.move(0, 0, 768, 432);
        this._NWSLineSprite.bitmap = new Bitmap(768, 432);
        this._NWSLineSprite.z = 2.7;
        this._NWSLineSprite.opacity = 0;
        this._NWSLineSprite._childSprite = new Sprite();
        this._NWSLineSprite._childSprite.bitmap = ImageManager.loadSystem('DLine');
        this._NWSLineSprite.addChild(this._NWSLineSprite._childSprite);
        this._spriteset._tilemap.addChild(this._NWSLineSprite);
        
        this._NWSInfoSprite = new Sprite();
        this._NWSInfoSprite.move(0, 0, 768, 432);
        this._NWSInfoSprite.bitmap = new Bitmap(768, 432);
        this._NWSInfoSprite.z = 6.5;
        this._NWSInfoSprite.opacity = 0;
        this._spriteset._tilemap.addChild(this._NWSInfoSprite);
        
        this._buttons = [];
        var name = $gameVariables.value(34) ? 'ButtonBig_gp' : 'ButtonBig';
        var buttonBitmap = ImageManager.loadSystem(name);
        for (var i = 0; i < 7; i++) {
            this._buttons[i] = new Sprite_Button();
            this._buttons[i].bitmap = buttonBitmap;
            this._buttons[i].x = 338 + i * 78;
            this._buttons[i].y = i >= 5 ? (14 + 90 * (i - 5)) : 330;
            if (i >= 4) this._buttons[i].x = [662, 674, 674][i - 4];
            this._buttons[i].visible = false;
            this._buttons[i]._text = [
                ['', '待機', '武器を拾う', '武器チェンジ', '', 'メニュー', ''],
                ['', 'Wait', 'Pick Up', 'Change Weapon', '', 'Menu', ''],
                ['', '대기', '무기 줍기', '무기 변경하기', '', '메뉴', ''],
                ['', '待機', '撿起武器', '更換武器', '', '選單', ''],
                ['', '待机', '捡起武器', '更换武器', '', '选单', ''],
            ][_lg()][i];
            this._buttons[i]._textY = i < 4 ? 12 : -12
            if (i >= 4) this._buttons[i]._textFontSize = 16;
            this.addWindow(this._buttons[i]);
        }
        buttonBitmap.addLoadListener(function(){
            for (var i = 0; i < 7; i++) {
                var x = ([5, 6].contains(i) ? i + 1 : i) * 96;
                this._buttons[i].setFrame(x, 0, 96, 96);
                this._buttons[i].setColdFrame(x, 0, 96, 96);
                this._buttons[i].setHotFrame(x, 96, 96, 96);
            }
        }.bind(this));
        
        this._healButton = this._buttons[0];
        this._skipButton = this._buttons[1];
        this._getWeaponButton = this._buttons[2];
        this._changeWeaponButton = this._buttons[3];
        this._attackButton = this._buttons[4];
        this._menuButton = this._buttons[5];
        this._flyButton = this._buttons[6];
        
        this._healButton.setClickHandler(this.onHeal.bind(this));
        this._healButton._input = 'A';
        this._skipButton.setClickHandler(this.onSkip.bind(this));
        this._skipButton._input = 'S';
        this._getWeaponButton.setClickHandler(this.onGetWeapon.bind(this));
        this._getWeaponButton._input = 'D';
        this._changeWeaponButton.setClickHandler(this.onChangeWeapon.bind(this));
        this._changeWeaponButton._input = 'F';
        this._attackButton.setClickHandler(this.onAttack.bind(this));
        this._attackButton._input = 'ok';
        this._menuButton.setClickHandler(this.startMenu.bind(this));
        this._menuButton._input = 'cancel';
        this._flyButton.setClickHandler(this.onFly.bind(this));
        this._flyButton._input = 'G';
        
        this._wpInfoWindow = new Window_NWSFukidashi();
        this.addWindow(this._wpInfoWindow);
    };
    Scene_Map.prototype.updateSprites = function() {
        var canAction = NWS()._canAction && $gamePlayer._DCount > 0;
        var skipping = $gamePlayer.isFootStepping();
        this._buttons.forEach(function(button, i) {
            button.visible = !$gameSystem.isBusy() && !$gameSystem.inMenu() && !$gameSwitches.value(3) && !$gameSystem._inCutIn;
            button.active = canAction && !$gameSwitches.value(9) &&
                !$gameSystem.isExpPlaying();
            if (i >= 0 && i <= 4 && $gameSystem.isFlying()) button.active = false;
            if (button === this._skipButton && skipping) button.active = true;
        }, this)
        
        //武器情報ふきだし
        var visible = this._buttons[4].visible && !$gameSystem._inCutIn;
        var info = [$gamePlayer.x, $gamePlayer.y, $gamePlayer._direction, NWS()._turn, $gamePlayer._sp,
                    $gamePlayer.attackItem()._itemId, visible].toString();
        var changed;
        if (!NWS()._noChangeAtkButton &&
            (!this._lastInfo || this._lastInfo !== info)) {
            this._lastInfo = info;
            this._areTriggerEventsInFront = $gamePlayer.areTriggerEventsInFront();
            changed = true;
        }
        if (!visible) {
            if (this._statusWindow.isOpen()) {
                this.wpInfoRefresh();
                this._wpInfoWindow.openness = 255;
            } else {
                this._wpInfoWindow.openness = 0;
            }
        } else {
            this.wpInfoRefresh();
            this._wpInfoWindow.openness = 255;
        }
        this._wpInfoWindow.x = this._statusWindow.x + 48;
        this._wpInfoWindow.y = this._statusWindow.y - this._wpInfoWindow.height - 6;
        this._wpInfoWindow._partsVisible = true;
        this._wpInfoWindow._partsPos = 1;
        
        //攻撃ボタンを自動で話す/調べるボタンに変更
        if (changed) {
            this._buttons[4]._frameType = this._areTriggerEventsInFront ? 1 : 0;
            var x = (this._areTriggerEventsInFront ? 5 : 4) * 96;
            this._buttons[4].setColdFrame(x, 0, 96, 96);
            this._buttons[4].setHotFrame(x, 96, 96, 96);
            var text = this._areTriggerEventsInFront ?
                ['話す/調べる', 'Talk/Search', '말하기/조사', '談話/調查', '谈话/調查'][_lg()] :
                ['攻撃', 'Attack', '공격', '攻擊', '攻擊'][_lg()];
            this._buttons[4]._text = text;
            this._buttons[4]._effect = this._areTriggerEventsInFront ? 0 : $gamePlayer._sp;
        }
        
        //薬草ボタン 個数更新
        if (this._buttons[0].visible) {
            this._buttons[0]._text = ['薬草', 'Use Herb', '약초', '藥草', '藥草'][_lg()] + ' × ' +NWS()._heal;
            this._buttons[0]._textColor = NWS()._heal >= NWS()._healsMax ? 'rgb(255, 204, 32)' : null;
        }
        
        //空を飛ぶボタン
        this._buttons[6].visible = this._buttons[6].visible && $gameSystem.canFly();
        if (this._buttons[6].visible) {
            this._buttons[6]._text = $gameSystem.isFlying() ?
                ['おりる', 'To land', '착지', '降落', '降落'][_lg()] :
                ['空を飛ぶ', 'Fly', '하늘 날기', '空中飛翔', '空中飞翔'][_lg()];
            this._buttons[6].active = this._buttons[6].active && 
                (!$gameSystem.isFlying() || $gameMap.isXyFree($gamePlayer.x, $gamePlayer.y, false, true));
        }
        
        if (!this._NWSLineSprite) return;
        if ($gameSystem.isBusy() || $gameSystem.inMenu()) { //全部非表示
            this._NWSLineSprite.opacity = 0;
            this._NWSInfoSprite.opacity = 0;
            return;
        }
        if (NWS()._isShift || $gameVariables.value(14)) { //shift
            var xyList = $gameSystem.getSkillRange(
                ($gameVariables.value(14) ? $dataSkills[$gameVariables.value(14)] : $gamePlayer.attackItem().object()),
                $gamePlayer.x, $gamePlayer.y, $gamePlayer._direction
            );
            var info = xyList.toString();
            if (this._NWSLineSprite._lastRange !== info) {
                this._NWSLineSprite._lastRange = info;
                this._NWSLineSprite.bitmap.clear();
                this._NWSInfoSprite.bitmap.clear();
                for (var x = $gamePlayer.x - 8; x <= $gamePlayer.x + 8; x++)
                    for (var y = $gamePlayer.y - 5; y <= $gamePlayer.y + 5; y++) {
                        var dx = $gameMap.adjustX(x) * 48, dy = $gameMap.adjustY(y) * 48;
                        var info = xyList.filter(function(xy) {return xy[0] === x && xy[1] === y})[0];
                        if (info) {
                            this._NWSLineSprite.bitmap.fillRect(dx, dy, 48, 48, 'rgba(255, 120, 76, 0.5)');
                            //情報描写
//                            var drawIcon = (function(iconIndex, px, py, ps) {
//                                var bitmap = ImageManager.loadSystem('IconSet');
//                                var sx = iconIndex % 16 * 32;
//                                var sy = Math.floor(iconIndex / 16) * 32;
//                                this._NWSInfoSprite.bitmap.blt(bitmap, sx, sy, 32, 32, px, py, ps, ps);
//                            }).bind(this);
//                            if (info[3] >= 2) {
//                                drawIcon(475 + info[3] - 2, dx + 24, dy + 24, 24); //属性
//                                var enemy = $gameMap.eventsXy(x, y).filter(function(ev) {return ev && ev.isCharacter()})[0];
//                                if (enemy) { //弱点と耐性
//                                    if (enemy._D.weakElements().contains(info[3])) drawIcon(473, dx + 32, dy + 1, 32);
//                                    else if (enemy._D.resistElements().contains(info[3])) drawIcon(474, dx + 32, dy + 1, 32);
//                                }
//                            }
//                            drawIcon(464 + info[2] - 1, dx + 2, dy + 34, 32); //威力
                        } else {
                            this._NWSLineSprite.bitmap.fillRect(dx, dy, 48, 48, 'rgba(0, 0, 0, 0.3)');
                        }
                    }
            }
            this._NWSLineSprite.opacity += 255 / 3;
            this._NWSInfoSprite.opacity += 255 / 3;
        } else {
            this._NWSLineSprite._lastRange = null;
            this._NWSLineSprite.opacity -= 255 / 3;
            this._NWSInfoSprite.opacity -= 255 / 3;
        }
        
//        if ($gamePlayer._customPriority) $gamePlayer._customPriority = 0;
    };
    
    Scene_Map.prototype.wpInfoRefresh = function() {
        if (!NWS()._equip) this._wpInfoWindow._text = '\\b\\w[0]%1\\b'.format(
            ['装備なし', 'No Equipment', '장비없이', '沒有設備', '没有设备'][_lg()]
        );
        else this._wpInfoWindow._text = NWS()._equip.detailText(0);
        this._wpInfoWindow.refresh();
    };
    
    //何らかのボタンが押されている
    Scene_Map.prototype.isAnyButtonTouching = function() {
        return this._buttons.some(function(button) {
            return button._touching;
        }, this)
    };
    
    //通常メニューは禁止
    Scene_Map.prototype.isMenuCalled = function() {};
    
    //メニューを開く
    Scene_Map.prototype.startMenu = function() {
        this._menuWindow.startMenu();
    };
    
    //武器リストを開く
    Scene_Map.prototype.startWeaponList = function(type) {
        this._weaponListWindow.setupByType(type);
        this._menuWindow._menuMode = 'weaponList';
    };
    
    //武器リストを選択モードで開く
    Scene_Map.prototype.startWeaponChoice = function(type, weaponCondition, max) {
        NWS()._choiceCondition = [weaponCondition, max || 1];
        NWS()._choiceWeapons = [];
        this.startWeaponList(type);
    };
    
    //アイテムの詳細を表示
    Scene_Map.prototype.displayItemDescription = function(item) {
        if (!item) return;
        var lastMenuMode = this._menuWindow._menuMode;
        this._menuWindow._menuMode = 'description';
        if (!this._descriptionWindow) {
            this._descriptionWindow = new Window_NWSMessageScroll(48, 96 - 8, Graphics.boxWidth - 96, 317);
            this._descriptionWindow._cancelAction = (function() {
//                this._descriptionWindow.deactivate();
//                this._descriptionWindow.hide();
//                this._descriptionWindow._titleWindow.hide();
//                var w1 = this._itemWindow, w2 = this._itemWindow2;
//                w1.show();
//                if (w2) w2.show();
//                this._itemHelpWindow.show();
//                if (w2.isOpen()) w2.activate();
//                else w1.activate();
//                this._itemHelpWindow._titleWindow.show();
//                this._menuWindow._menuMode = lastMenuMode;
            }).bind(this);
            this.addChild(this._descriptionWindow);
        }
        this._descriptionWindow._titleText = (item.itemName(true, true));
        this._itemWindow.deactivate(), this._itemWindow.hide();
        this._itemWindow2.deactivate(), this._itemWindow2.hide();
        this._itemHelpWindow.hide(), this._itemHelpWindow._titleWindow.hide();
        var text = '';
        if (item.isArmor()) text += item.itemDescription();
        else text += item.itemDescription().split('\n')[0] + this.getItemDetail(item);
        if (!item.object().meta.noDetail)
            for (var i = 0; i < item._children.length; i++) text += this.getItemDetail(item._children[i]);
        this._descriptionWindow.setText(text);
        this._descriptionWindow.activate();
        this._menuWindow._bCount = 0;
        this._descriptionWindow.show();
        if (this._descriptionWindow._titleWindow) this._descriptionWindow._titleWindow.show();
    };
    
    //アイテムの詳細文章を返す
    Scene_Map.prototype.getItemDetail = function(item) { 
        if (!item) return '';
        if (!item.isArmor()) {
            var detail = (item.object().meta.detail || '');
            detail = detail.replace('[damage]', item.object().damage.formula);
            var eff = item.object().effects[0];
            detail = detail.replace('[effect1]', eff ? eff.value1 : '').replace('[effect2]', eff ? eff.value2 : '');
        }
        return detail;
    };
    
    //アイテムソート
    Scene_Map.prototype.onSort = function() {
        SoundManager.playOk();
        var items = this._weaponListWindow._type === 2 ? NWS()._stocks : NWS()._weapons;
        $gameSystem.sortItems(items, this._weaponListWindow._sortType);
        this._weaponListWindow._sortType = (this._weaponListWindow._sortType + 1) % 5;
        if (_lg() >= 2 && this._weaponListWindow._sortType === 1) this._weaponListWindow._sortType = 2;
        this._weaponListWindow.refresh();
        this._weaponListWindow.updateHelp();
    };
    
    //名前入力決定
    Scene_Map.prototype.startNameInput = function(max) {
        this._lastMenuMode = this._menuWindow._menuMode;
        this._menuWindow._menuMode = 'nameInput'
        if ($gameVariables.value(11) === 0) { //名前自由入力
            var endAction = function() {
                NWS()._inputName = $gameVariables.value(3) || '';
                this._menuWindow._menuMode = this._lastMenuMode;
                if (this._restartAction) this._restartAction();
            }
            max = max || 8;
            $gameSystem.startNameInput(null, ['x=180;y=180;v=3;max=%1;btn_x=24;btn_y=50;'.format(max)], 
                                       NWS()._inputName || '', endAction.bind(this));
            NWS()._inputName = '';
        } else { //文字選択入力
            this._editWindow._name = NWS()._inputName || '', NWS()._inputName = '';
            this._editWindow._index = this._editWindow._name.length;
            this._editWindow.refresh(), this._editWindow.open();
            this._inputWindow.createContents(), this._inputWindow.refresh(), this._inputWindow.open();
            this._inputWindow.select(0), this._inputWindow.activate();
        }
    };
    Scene_Map.prototype.onInputOk = function() {
        NWS()._inputName = this._editWindow.name();
        this._inputWindow.deactivate();
        this._editWindow.close();
        this._inputWindow.close();
        this._menuWindow._menuMode = this._lastMenuMode;
        if (this._restartAction) this._restartAction();
    };
    Scene_Map.prototype.onInputCancel = function() {
        this._inputWindow.deactivate();
        this._editWindow.close();
        this._inputWindow.close();
        this._menuWindow._menuMode = this._lastMenuMode;
        if (this._restartAction) this._restartAction();
    };
    //名前入力画面　最大までもどるとキャンセル扱いに
    Window_NameInput.prototype.processBack = function() {
        SoundManager.playCancel();
        if (!this._editWindow.back()) this.deactivate(), SceneManager._scene.onInputCancel();
    };
    
    //メッセージ履歴を表示
    Scene_Map.prototype.displayDisplayLog = function() {
        var lastMenuMode = this._menuWindow._menuMode;
        this._menuWindow._menuMode = 'displayLog';
        if (!this._displayLogWindow) {
            this._displayLogWindow = new Window_NWSMessageScroll(48, 64, Graphics.boxWidth - 96, 0);
            this._displayLogWindow.height = this._displayLogWindow.fittingHeight(11);
            this._displayLogWindow._titleText = 
                ['メッセージログ', 'Message Log', '메시지 로그', '訊息履歷', '消息历史'][_lg()];
            this._displayLogWindow._cancelAction = (function() {
                this._displayLogWindow.deactivate();
                this._displayLogWindow.close();
                this._menuWindow._menuMode = lastMenuMode;
            }).bind(this);
            this.addChild(this._displayLogWindow);
        }
        var text = '';
        NWS()._displayLog.forEach(function(log) {text += (text ? '\n' : '') + '\\w[1]' + log + '\\c[0]\\w[0]'});
        var info = this._displayLogWindow.drawTextKNH(text, 0, 9999, this._displayLogWindow.contents.width - 12);
        this._displayLogWindow._contentsHeight = info.h + 28;
        this._displayLogWindow.createContents();
        this._displayLogWindow.setText(text);
        this._displayLogWindow.activate();
        this._displayLogWindow.setTopRow(999);
        this._menuWindow._bCount = 0;
        this._displayLogWindow.openness = 0, this._displayLogWindow.open();
    };
    
    //経験値を表示
    Scene_Map.prototype.displayExp = function(plus, type) {
        if (!type) this._menuWindow._menuMode = 'exp';
        if (!this._expWindow) {
            this._expWindow = new Window_NWSExp();
            this.addChild(this._expWindow);
        }
        this._expWindow.setup(type || 0, plus);
    };
    Game_System.prototype.displayExp = function(plus) {
        SceneManager._scene.displayExp(plus);
    };
    
    // Show Picture
    var _Game_Interpreter_command231 = Game_Interpreter.prototype.command231;
    Game_Interpreter.prototype.command231 = function() {
        var id = this._params[0];
        if (id >= 3) return _Game_Interpreter_command231.apply(this, arguments);
        var x = id === 1 ? 216 : (Graphics.boxWidth - 216), y = 264 + $gameVariables.value(16);
        var scale = 62;
        var nrev = id === 1 ? 1 : -1;
        if (!$gameScreen.picture(id)) {
            $gameScreen.showPicture(id, this._params[1], 1, x + (id === 1 ? -48 : 48), y, scale * nrev, scale, 0, 0);
            var pic = $gameScreen.picture(id);
            pic._targetX = x, pic._targetOpacity = 255, pic._duration = 15;
        } else {
            $gameScreen.showPicture(id, this._params[1], 1, x, y, scale * nrev, scale, 255, 0);
        }
        return true;
    };
    
    // Erase Picture
    Game_Interpreter.prototype.command235 = function() {
        $gameScreen.erasePicture(this._params[0]);
        if ([1].contains(this._params[0])) for (var i = 1; i <= 2; i++) $gameScreen.erasePicture(i);
        return true;
    };
    
    //-----------------------------------------------------------------------------
    // Window_NWSStatus
    //-----------------------------------------------------------------------------
    
    function Window_NWSStatus() {
        this.initialize.apply(this, arguments);
    }
    
    Window_NWSStatus.prototype = Object.create(Window_Base.prototype);
    Window_NWSStatus.prototype.constructor = Window_NWSStatus;

    Window_NWSStatus.prototype.initialize = function() {
        var w = 320, h = 24 * 2 + 24;
        Window_Base.prototype.initialize.call(this, 10 - 48, Graphics.boxHeight - 10 - h, w + 48, h);
        this._count = 0;
        this._lastInfo = '';
        this._menuMode = '';
        this.createContents();
    };
    
    Window_NWSStatus.prototype.standardFontSize = function() {
        return 16;
    };
    
    Window_NWSStatus.prototype.update = function() {
        Window_Base.prototype.update.apply(this, arguments);
        if ($gameSystem.noMenu()) {
            this.openness = 0;
            return;
        }
        var mode = SceneManager._scene._menuWindow._menuMode;
        if ($gameSystem.isBusy() || $gameSwitches.value(3) ||
            ['displayLog'].contains(mode)) {
            this.openness = 0;
        } else {
            this.openness = 255;
//            var targetY = Graphics.boxHeight - this.height - ($gameSystem.inMenu() ? 38 : 10);
//            this.y += Math.round((targetY - this.y) / 2);
        }
        if ($gameMap.isEventRunning() && ['weaponList'].contains(mode)) {
            this.openness = 255;
            this.y = Graphics.boxHeight - 10 - this.height - 24;
        } else {
            this.y = Graphics.boxHeight - 10 - this.height;
        }
        if (this.isOpen()) {
            var d = $gamePlayer._D;
            var info = [d.name(), d._level, $gameParty.gold(), $gameSystem.arma(),
                        Math.ceil(d.hp), Math.ceil(d.mhp)].toString();
            if (this._lastInfo !== info) {
                this._lastInfo = info;
                this.refresh();
            }
        }
    };
    
    Window_NWSStatus.prototype.refresh = function() {
        var x = 48, y = -1, w = this.contents.width - 48 - 12;
        var d = $gamePlayer._D;
        this.contents.clear();
        
        var nx = this.drawTextEx('%1　Lv.%2　'.format(d.name(), d._level), x, y, w);
        
        var wh = w - nx - 56;
        x = w - wh;
        this.drawTextEx([1, 2].contains(_lg()) ? ' HP' : 'ＨＰ', x, y, w);
        this.drawGauge(x + 34, y - 8, wh.clamp(96, 384 - 8), d.hpRate(), this.hpGaugeColor1(), this.hpGaugeColor2());
        x = w - 78;
        this.drawTextEx('%1\\}\\} \\{\\{/\\}\\} \\{\\{%2'.format(
            nFormat(Math.ceil(d.hp), ' ', 3), nFormat(Math.ceil(d.mhp), ' ', 3)
        ), x + 40, y);
        
        x = 48, y = 26;
        this.drawTextEx('%3：%1\\}\\} \\{\\{\\g　　\\i[141]：%2'.format(
            $gameParty.gold(), $gameSystem.arma(), ['所持金', 'Money', '소지금', '持有金錢', '持有金钱'][_lg()]
        ), x, y);
    };
    
    Window_NWSStatus.prototype.drawGauge = function(x, y, width, rate, color1, color2) {
        this.contents.fillRect(x - 1, y + this.lineHeight() - 8 - 1, width + 2, 6 + 2, this.textColor(0));
        var fillW = Math.ceil(width * rate);
        var gaugeY = y + this.lineHeight() - 8;
        this.contents.fillRect(x, gaugeY, width, 6, this.gaugeBackColor());
        this.contents.gradientFillRect(x, gaugeY, fillW, 6, color1, color2);
    };
    
    //-----------------------------------------------------------------------------
    // Window_NWSMenu
    //-----------------------------------------------------------------------------
    
    function Window_NWSMenu() {
        this.initialize.apply(this, arguments);
    }
    
    Window_NWSMenu.prototype = Object.create(Window_Selectable.prototype);
    Window_NWSMenu.prototype.constructor = Window_NWSMenu;

    Window_NWSMenu.prototype.initialize = function() {
        Window_Selectable.prototype.initialize.call(this, 0, 0, 0, 0);
        this._iconBitmap = ImageManager.loadSystem('KNH_MenuIcon');
        this.setBackgroundType(2);
//        this._fontNormal = true;
//        this.loadWindowskin(3);
//        this._titleText = '\\i[465]\\i[466]\\i[467]\\i[468]';
        NWS()._menuItems = 
            ['weaponList', 'stocks', 'book', 'map', 'log', 'backHome', 'help', 'option', 'save'] ;
        if (true || $gameTemp.isPlaytest()) NWS()._menuItems.push('debug');
        var w, h; w = h = 3 * this.itemHeight() + 24;
        var x = Graphics.boxWidth - 72 - w, y = (Graphics.boxHeight - h) / 2;
        this.move(x, y, w, h);
        this._scrollY = 0;
        this.createContents();
        this.createContentsBack();
        this.refresh();
        this.select(0);
        this.openness = 0;
    };
    Window_NWSMenu.prototype.spacing = function() {
        return 0;
    };
    Window_NWSMenu.prototype.itemWidth = function() {
        return 112;
    };
    Window_NWSMenu.prototype.itemHeight = function() {
        return 112;
    };
    Window_NWSMenu.prototype.maxCols = function() {
        return 3;
    };
    Window_NWSMenu.prototype.maxItems = function() {
        return NWS()._menuItems.length;
    };
//    Window_NWSMenu.prototype.itemRect = function(index) {
//        var rect = Window_Selectable.prototype.itemRect.apply(this, arguments);
//        rect.y += Math.floor(index / 3) * 12;
//        return rect;
//    };
    
    Window_NWSMenu.prototype.drawItem = function(index) {
        var rect = this.itemRect(index);
        var cList = ['weaponList', 'stocks', 'book', 'map', 'log', 'backHome', 'help', 'option', 'save', 'debug'];
        var i = cList.indexOf(NWS()._menuItems[index]); //画像の順番
        this.changePaintOpacity(this.isEnabled(index));
        this.drawMenuIcon(0, rect.x, rect.y);
        this.drawMenuIcon(i + 1, rect.x, rect.y);
        var text = [
            ['武器一覧', '武器庫', '図鑑', '地図', 'ログ', '家に帰る', 'ヘルプ', '設定', 'セーブ', 'デバッグ'],
            ['Inventory', 'Armory', 'Gallery', 'Map', 'Log', 'Return Home', 'Help', 'Settings', 'Save', 'デバッグ'],
            ['무기 목록', '무기고', '도감', '지도', '이력', '집에 가기', '도움말', '설정', '저장', 'デバッグ'],
            ['武器一覽', '武器庫', '圖鑑', '地圖', '履歷', '回家', '幫助', '設定', '保存', 'デバッグ'],
            ['武器一览', '武器库', '图鉴', '地图', '历史', '回家', '帮助', '设置', '保存', 'デバッグ']
        ][_lg()][i];
        var x = rect.x, y = rect.y + 2, w = rect.width;
        this.changeTextColor('white');
        this.contents.fontBold = true;
        this.contents.fontSize = 18;
        this.drawText(text, x, y, w, 'center');
        this.contents.fontBold = false;
        if (NWS()._menuItems[index] === 'weaponList') {
            var wNum = NWS()._weapons.filter(function(w) {return !!w}).length;
            this.drawText('%1/%2'.format(wNum, NWS()._weaponsMax), x, y + 78, w - 8, 'right');
        }
        if (NWS()._menuItems[index] === 'stocks') {
            this.drawText('%1/%2'.format(NWS()._stocks.length, $gameSystem.sisterParam(1)), x, y + 78, w - 8, 'right');
        }
        this.changePaintOpacity(true);
    };
    Window_NWSMenu.prototype.drawMenuIcon = function(index, x, y) {
        var bitmap = this._iconBitmap;
        var pw = 96, ph = 96;
        var sx = index * pw;
        x += (this.itemWidth() - pw) / 2, y += (this.itemHeight() - ph) / 2;
        this.contents.blt(bitmap, sx, 0, pw, ph, x, y);
    };
    Window_NWSMenu.prototype.drawContentsBack = function(index) {
        this._drawDependScrollY = true;
        var rect = this.itemRect(index);
        this._contentsBack.bitmap.clearRect(index);
        if (index >= this.maxItems()) return;
        var w = rect.width - 24, h = rect.height - 24;
        var x = rect.x + (rect.width - w) / 2, y = rect.y + (rect.height - h) / 2;
        this._contentsBack.bitmap.fillRect(x, y, w, h, 'rgba(80,54,14,0.6)');
        var a = 12;
        this._contentsBack.bitmap.clearRect(x + a, y + a, w - a * 2, h - a * 2);
        this._contentsBack.bitmap.fillRect(x + a, y + a, w - a * 2, h - a * 2, 'rgba(80,54,14,0.9)');
        this._drawDependScrollY = false;
    };
    Window_NWSMenu.prototype.refresh = function() {
        this.contents.clear();
        this.createContentsBack();
        this.drawAllContentsBack();
        this.drawAllItems();
    };
    
    Window_NWSMenu.prototype.update = function() {
        Window_Selectable.prototype.update.apply(this, arguments);
        if (this._iconBitmap.isReady() && !this._refreshed) this.refresh(), this._refreshed = true;
        if (this.active && this.openness < 255) this.openness += 255 / 3;
        else if (!this.active && this.openness > 0) this.openness -= 255 / 3;
    };
    
    //メニュー開く
    Window_NWSMenu.prototype.startMenu = function() {
        $gameTemp.clearDestination();
        this.refresh();
        AudioManager.playSe({name:'Book1', volume:90, pitch:110, pan:0});
        this.openness = 255;
        var value = $gamePlayer._D.hpRate();
        $gameScreen.showPicture(1, 'menuParts', 0, 0, 0, 100, 100, 255, 0);
        $gameScreen.showPicture(2, 't_Weko_0%1'.format(
            value >= 1 ? 4 : value >= 0.5 ? 1 : value >= 0.1 ? 2 : 3
        ), 1, 192 + 24, 264, 67, 67, 255, 0);
        this.activate();
        SceneManager._scene.displayExp(0, 1);
    };
    //メニュー閉じる
    Window_NWSMenu.prototype.endMenu = function() {
        SoundManager.playCancel();
        this.deactivate();
        this.openness = 255;
        this.erasePicture();
    };
    Window_NWSMenu.prototype.translucentOpacity = function() {
        return 128;
    };

    
    //OK
    Window_NWSMenu.prototype.isOkEnabled = function() {
        return true;
    };
    Window_NWSMenu.prototype.isEnabled = function(index) {
        if (!$dataMap.meta) return true;
        var item = NWS()._menuItems[index];
        if (item === 'stocks')
            return $gameSystem.isStocksOk();
        if (item === 'save')
            return $gameSystem.isSaveOk();
        if (item === 'backHome')
            return $gameVariables.value(9) <= 0;
        return true;
    };
    Window_NWSMenu.prototype.processOk = function() {
        if (!this.isEnabled(this.index())) return this.playBuzzerSound();
        this.playOkSound();
        this.erasePicture();
        this.deactivate();
        var item = NWS()._menuItems[this.index()];
        var decided = false;
        if (item === 'weaponList') {
            decided = true;
            NWS()._choiceCondition = [];
            SceneManager._scene.startWeaponList(0);
        } else if (item === 'stocks') {
            decided = true;
            NWS()._choiceCondition = [];
            SceneManager._scene.startWeaponList(2);
        } else if (item === 'book') {
            decided = true;
            NWS()._choiceCondition = [];
            SceneManager._scene.startWeaponList(3);
        } else if (item === 'map') {
            decided = true;
            $gameSwitches.setValue(22, true);
        } else if (item === 'log') {
            decided = true;
            SceneManager._scene.displayDisplayLog();
        } else if (item === 'backHome') {
            decided = true;
            $gameSwitches.setValue(21, true);
        } else if (item === 'help') {
            decided = true;
            $gameSwitches.setValue(23, true);
        } else if (item === 'option') {
            decided = true;
            SceneManager.push(Scene_Options);
        } else if (item === 'save') {
            decided = true;
            SceneManager.push(Scene_Save);
        } else if (item === 'debug') {
            decided = true;
            $gameSwitches.setValue(6, true);
        }
        if (decided) {
            this.openness = 0;
        }
    };
    Window_NWSMenu.prototype.erasePicture = function() {
        $gameScreen.erasePicture(1), $gameScreen.erasePicture(2);
    };
    
    //-----------------------------------------------------------------------------
    // Window_NWSWeaponList
    //-----------------------------------------------------------------------------
    
    function Window_NWSWeaponList() {
        this.initialize.apply(this, arguments);
    }
    Window_NWSWeaponList.prototype = Object.create(Window_ItemList.prototype);
    Window_NWSWeaponList.prototype.constructor = Window_NWSWeaponList;
    
    Window_NWSWeaponList.prototype.initialize = function() {
        Window_ItemList.prototype.initialize.call(this, 0, 0, 0, 0);
        this.openness = 0;
        this._commandWindow = new Window_NWSCommand();
        this._commandWindow.openness = 0;
        this._commandWindow.setHandler('ok', this.commandOk.bind(this));
        this._commandWindow.setHandler('cancel', this.commandCancel.bind(this));
        this.addChild(this._commandWindow);
        this._type = 0;
        this._moving = -1;
    };
    
    Window_NWSWeaponList.prototype.maxCols = function() {
        return [3].contains(this._type) ? 6 : 4;
    };
    Window_NWSWeaponList.prototype.spacing = function() {
        return 0;
    };
    Window_NWSWeaponList.prototype.itemHeight = function() {
        return this.itemWidth();
    };
    Window_NWSWeaponList.prototype.maxItems = function() {
        return [2, 3, 4].contains(this._type) ? this._data.length : NWS()._weaponsMax;
    };

    Window_NWSWeaponList.prototype.makeItemList = function() {
        if (this._type === 2) {
            this._data = NWS()._stocks;
        } else if (this._type === 3) {
            this._data = $gameSystem.booksData();
        } else if (this._type === 4) {
            this._data = $gameSystem.booksData().filter(function(data) {
                return !!data;
            })
        } else {
            this._data = NWS()._weapons;
        }
    };
    
    Window_NWSWeaponList.prototype.setupByType = function(type) {
        this._type = type;
        var x = 48, y = 64, w = 332, h = 332;
        x = Graphics.boxWidth - x - w;
        var windowskin = 1;
        if (type === 1) { //武器チェンジ
            this._titleText = ['武器チェンジ', 'Change Weapon', '무기 변경하기', '更換武器', '更换武器'][_lg()];
//            x = (Graphics.boxWidth - w) / 2;
            this._scrollY = 0;
        } else if (type === 2) { //武器庫
            this._titleText = ['武器庫', 'Armory', '무기고', '武器庫', '武器库'][_lg()];
            windowskin = 2;
        } else if (type === 3) { //武器図鑑
            this._titleText = '%3\\}  %1％（%2\\}\\}\\{\\{%4）\\{'.format(
                $gameSystem.booksRate(), NWS()._books.filter(function(item) {return !!item && item._detail}).length,
                ['武器図鑑', 'Weapons Gallery', '도감', '圖鑑', '图鉴'][_lg()],
                ['個', '', '개', '', ''][_lg()],
            );
            windowskin = 3;
        } else if (type === 4) { //鍛冶屋
            this._titleText = ['鍛冶屋', 'blacksmith', '대장장이', '鐵匠', '铁匠'][_lg()];
            windowskin = 3;
        } else { //武器一覧
            this._type = type = 0;
            this._titleText = ['武器一覧', 'Inventory', '무기 목록', '武器一覽', '武器一览'][_lg()];
            this._scrollY = 0
        }
        this._sortType = 0;
        this.changeWindowskin(windowskin);
        this._helpWindow.refreshOpen();
        this.move(x, y, w, h);
        this.refreshOpen();
        var index = [3, 4].contains(this._type) ? 0 : Math.max(this._data.indexOf(NWS()._equip), 0);
        if (!this.maxItems()) index = -1;
        this.select(index);
        this.activate();
    };
    
     Window_NWSWeaponList.prototype.changeWindowskin = function(windowskin) {
         this.loadWindowskin(windowskin);
         this.resetFontSettings();
         this._helpWindow.loadWindowskin(windowskin);
         this._helpWindow.resetFontSettings();
         this._commandWindow.loadWindowskin(windowskin + '_p2');
         this._commandWindow.resetFontSettings();
         if (this._titleWindow) {
             this._titleWindow.windowskin = ImageManager.loadSystem(this._windowskinName + '_p');
             this._titleWindow.resetFontSettings();
         }
     };
    
    Window_NWSWeaponList.prototype.drawItem = function(index) {
        var weapon = this._data[index];
        var rect = this.itemRect(index);
        if (!weapon || !$gameSystem.hasDetail(weapon._itemId)) {
            if ([3].contains(this._type))
                this.drawIcon(464, rect.x + (rect.width - 32) / 2 + 2, rect.y + (rect.height - 32) / 2 + 2);
            if (!weapon) return;
        }
        var isEquip = NWS()._equip === weapon;
        var isChoiced = NWS()._choiceCondition[0] && NWS()._choiceWeapons.contains(weapon);
        this.contents.fontSize = 16;
        if (this._moving === index) { //移動中
            this.contents.fillRect(rect.x, rect.y, rect.width, rect.height, this.pendingColor());
        } else if (isChoiced) { //選択中
            var a = 2;
            this.contents.fillRect(rect.x + a, rect.y + a, rect.width - a*2, rect.height - a*2, 'rgba(255,120,76,1)');
            a += 4;
            this.contents.clearRect(rect.x + a, rect.y + a, rect.width - a*2, rect.height - a*2);
        }
        var text1, text2, text3, text4;
        if (NWS()._displayWeaponInfo === 'price') {
            text3 = weapon.price() + TextManager.currencyUnit;
        } else if (['needsArma'].contains(NWS()._displayWeaponInfo)) {
            text3 = '\\i[141]✕' + weapon.needArma();
            if (weapon._level >= 99) text3 = '\\c[14]MAX';
        } else if (this._type === 4) {
            text3 = weapon.makePrice() + TextManager.currencyUnit;
            text4 = '✕' + (NWS()._makeItems[weapon._itemId] || 0);
        } else {
            text1 = ![3, 4].contains(this._type) ? (
                $gameSystem.hasDetail(weapon._itemId) ? weapon.power() : '？'
            ) : '';
//            text2 = ![3].contains(this._type) ? '(%1)'.format(weapon._useable) : '';
        }
        if ([0, 1, 2].contains(this._type)) {
            text4 = '%1%2'.format(
                weapon._togi && weapon._mekki ? '\\i[471]\\i[472]' : weapon._togi ? '\\i[471]\\i[0]' :
                weapon._mekki ? '\\i[472]\\i[0]' : '\\i[0]\\i[0]',
                weapon._level > 1 || NWS()._displayWeaponInfo === 'needsArma' ?
                ((weapon._level < 10 ? ' Lv.' : 'Lv.') + weapon._level) : ''
            );
            if (weapon._lock) text2 = '\\i[92]';
        }
        this.changePaintOpacity(this.isEnabled(weapon, index));
        this.drawWeapon(weapon._itemId, weapon.imageName(), rect.x, rect.y, rect.width, text1, text2, text3, text4);
        if (![3, 4].contains(this._type)) {
            if (isEquip) { //装備中
                this.changeTextColor(this.textColor(24));
                var text = ['装備中', 'Equipped', '장비 중', '在設備', '在设备'][_lg()];
                this.drawText(text, rect.x, rect.y + 24, rect.width, 'center');
            }
        }
        if (isChoiced) { //選択中
            this.changeTextColor(this.textColor(2));
            var text = ['選択中', 'Selected', '선택 중', '選', '选'][_lg()];
            this.drawText(text, rect.x, rect.y + 24, rect.width, 'center');
        }
        this.changePaintOpacity(1), this.resetFontSettings();
    };
    
    Window_NWSWeaponList.prototype.isEnabled = function(weapon, index) {
        if (NWS()._choiceCondition[0] && !NWS()._choiceCondition[0](weapon)) return false;
//        if (this._type === 2 && $gameSystem.weaponsMax()) return false;
        if (this._type === 3) return $gameSystem.hasDetail(weapon._itemId);
        if (this._type === 4) 
            return NWS()._makeItems[weapon._itemId] > 0 && $gameParty.gold() >= weapon.makePrice();
        return true;
    };
    
    Window_NWSWeaponList.prototype.isOkEnabled = function() {
        return true;
    };
    
    Window_NWSWeaponList.prototype.processOk = function() {
        if (((this._type !== 1 && this._moving < 0) && !this.item()) || this._type === 3) return;
        if (this._moving >= 0) { //移動
            this.playOkSound();
            var movingIndex = this._moving, move = this._data[movingIndex];
            this._data[movingIndex] = this.item();
            this._data[this.index()] = move;
            this._moving = -1;
            this.redrawItem(movingIndex);
            this.redrawCurrentItem();
            this.activate();
            return;
        }
        if (this.isCurrentItemEnabled()) {
            this.playOkSound();
            if (this._type === 1) { //武器チェンジ処理
                if (this.item() && this.item() !== NWS()._equip) $gamePlayer.equipWeapon(this.item());
                else $gamePlayer.removeWeapon();
                this.close();
            } else { //コマンド表示
                this._commandWindow._list = this.makeCommandList();
                var rect = this.itemRect(this.index()), padd = this.padding - 5;
                var x = padd + rect.x, y = padd + rect.y;
                var cw = 192, ch = this._commandWindow.fittingHeight(this._commandWindow._list.length);
                var cx = this.index() % 4 < 2 ? x + rect.width : x - cw + 10;
                var cy = y.clamp(padd, this.height - padd - ch);
                this._commandWindow.move(cx, cy, cw, ch);
                this._commandWindow.createContents();
                this._commandWindow.select(0);
                this._commandWindow.refreshOpen();
                this.deactivate(), this._commandWindow.activate();
            }
        } else {
            this.playBuzzerSound();
        }
    };
    
    Window_NWSWeaponList.prototype.makeCommandList = function() {
        var list = [];
        var normal = ![1, 2, 3, 4].contains(this._type);
        var cannotChange = $gamePlayer._D.metaState('cannotChangeWeapon');
        var flying = $gameSystem.isFlying();
        if (NWS()._choiceCondition[0]) {
            if (!NWS()._choiceWeapons.contains(this.item())) list.push('選ぶ');
            else list.push('選ばない');
        } else {
            if (normal && NWS()._equip !== this.item()) {
                if (!cannotChange && !flying) list.push('装備する');
                if (!$gameSystem.stocksMax() && !$dataMap.meta.maojo && !$dataMap.meta.SBI)
                    list.push('武器庫にしまう');
            }
            if (normal && NWS()._equip === this.item()) {
                if (!cannotChange && !flying) list.push('装備をはずす');
            }
            if (this._type !== 4) {
                if (this._type === 2 && !$gameSystem.weaponsMax()) list.push('取り出す');
                if ([0, 2].contains(this._type)) list.push(this.item()._lock ? 'ロック解除' : 'ロックする');
                list.push('移動する');
                if (NWS()._equip !== this.item() &&
                    !$dataMap.meta.SBI && !this.item()._lock) list.push('破棄する');
            } else {
                list.push('作成する');
            }
        }
        list.push('やめる');
        return list;
    };
    
    Window_NWSWeaponList.prototype.commandOk = function() {
        this._commandWindow.close();
        var action = this._commandWindow._list[this._commandWindow.index()];
        if (['装備する'].indexOf(action) >= 0) {
            $gameSystem.playDisplaySe('equip');
            var redrawIndex = this._data.indexOf(NWS()._equip);
            $gamePlayer.equipWeapon(this.item(), true);
            this.redrawItem(redrawIndex);
            this.redrawCurrentItem();
            this.activate();
        } else if (['装備をはずす'].indexOf(action) >= 0) {
            $gameSystem.playDisplaySe('equip');
            $gamePlayer.removeWeapon(true);
            this.redrawCurrentItem();
            this.activate();
        } else if (['武器庫にしまう'].indexOf(action) >= 0) {
            $gameSystem.playDisplaySe('bow');
            NWS()._stocks.push(this.item());
            NWS()._weapons[NWS()._weapons.indexOf(this.item())] = null;
            this.redrawCurrentItem();
            this.activate();
        } else if (['破棄する'].indexOf(action) >= 0) {
            $gameSystem.playDisplaySe('fall');
//            $gameSystem.revivalGotEvent(this.item().object().id);
            $gameSystem.revivalGotEvent(this.item());
            if (this._type === 2) {
                NWS()._stocks.splice(NWS()._stocks.indexOf(this.item()), 1);
                this.refresh();
                this.decreaseIndex();
            } else {
                NWS()._weapons[NWS()._weapons.indexOf(this.item())] = null;
                this.redrawCurrentItem();
            } 
            this.activate();
        } else if (['取り出す'].indexOf(action) >= 0) {
            $gameSystem.playDisplaySe('bow');
            $gamePlayer.getItem(this.item(), false, true);
            NWS()._stocks.splice(NWS()._stocks.indexOf(this.item()), 1);
            this.refresh();
            this.decreaseIndex();
            this.activate();
        } else if (['移動する'].indexOf(action) >= 0) {
            this._moving = this.index();
            this.redrawCurrentItem();
            this.activate();
        } else if (['選ぶ'].indexOf(action) >= 0) {
            NWS()._choiceWeapons.push(this.item());
            if (NWS()._choiceWeapons.length >= NWS()._choiceCondition[1]) {
                this.close();
            } else {
                this.redrawCurrentItem();
                this.activate();
            }
        } else if (action === '選ばない') {
            var index = NWS()._choiceWeapons.indexOf(this.item());
            if (index >= 0) NWS()._choiceWeapons.splice(index, 1);
            this.redrawCurrentItem();
            this.activate();
        } else if (action === 'ロックする') {
            $gameSystem.playDisplaySe('equip');
            this.item()._lock = true;
            this.redrawCurrentItem();
            this.activate();
        } else if (action === 'ロック解除') {
            $gameSystem.playDisplaySe('equip');
            this.item()._lock = false;
            this.redrawCurrentItem();
            this.activate();
        } else if (['作成する'].indexOf(action) >= 0) {
            $gameSystem.playDisplaySe('shop');
            $gameParty.loseGold(this.item().makePrice());
            NWS()._madeItems.push($gameSystem.newItem(this.item()._itemId));
            NWS()._makeItems[this.item()._itemId]--;
            if (NWS()._madeItems.length >= 5) {
                this.close();
            } else {
                this.redrawCurrentItem();
                this.activate();
            }
        } else if (['詳細'].indexOf(action) >= 0) {
            this.activate();
        } else {
            this.activate();
        }
    };
    Window_NWSWeaponList.prototype.commandCancel = function() {
        this._commandWindow.close();
        this.activate();
    };
    Window_NWSWeaponList.prototype.decreaseIndex = function() {
        this.select(Math.min(this.index(), this._data.length - 1));
    };
    
    var _Window_NWSWeaponList_update = Window_NWSWeaponList.prototype.update;
    Window_NWSWeaponList.prototype.update = function() {
        if (!this.visible || (this.openness <= 0 && !this._opening)) {
            if (this._titleWindow) this._titleWindow.openness = 0;
            return false;
        }
        if ($gamePlayer.isUsefulCalling()) {
            if (!$gameSystem.isStocksOk()) { //武器庫を使用できない場合、ブザーをならす
                if (Input.isTriggered('ok') || Input.isTriggered('cancel')) SoundManager.playBuzzer();
                return;
            }
            if (Input.isTriggered('ok')) {
                $gameSystem.playDisplaySe('bow');
                for (var i = 0; i < NWS()._weaponsMax && NWS()._stocks.length; i++) {
                    if (!NWS()._weapons[i]) {
                        var ind = Math.randomInt(NWS()._stocks.length);
                        $gamePlayer.getItem(NWS()._stocks[ind], false, true);
                        NWS()._stocks.splice(ind, 1);
                        this.redrawItem(i);
                    }
                }
                this._helpWindow.setItem(this._data[this.index()]);
                return;
            } else if (Input.isTriggered('cancel')) {
                $gameSystem.playDisplaySe('bow');
                for (var i = 0; i < NWS()._weaponsMax; i++) {
                    if (!!NWS()._weapons[i] && NWS()._equip !== NWS()._weapons[i] && !$gameSystem.stocksMax()) {
                        NWS()._stocks.push(NWS()._weapons[i]);
                        NWS()._weapons[i] = null;
                        this.redrawItem(i);
                    }
                }
                this._helpWindow.setItem(this._data[this.index()]);
                return;
            }
        }
        _Window_NWSWeaponList_update.apply(this, arguments);
    };

    Window_NWSWeaponList.prototype.refresh = function() {
        this.makeItemList();
        this.createContents();
        this.createContentsBack();
//        if ([3, 4].contains(this._type)) this._contentsBack.bitmap.clear();
//        else this.drawAllContentsBack();
        this.drawAllContentsBack();
        this.drawAllItems();
    };
    var _Window_NWSWeaponList_refreshScrollBar = Window_NWSWeaponList.prototype._refreshScrollBar;
    Window_NWSWeaponList.prototype._refreshScrollBar = function () {
        _Window_NWSWeaponList_refreshScrollBar.apply(this, arguments);
        if (this._scrollBarSprite && this._commandWindow &&
            this.children.indexOf(this._scrollBarSprite) > this.children.indexOf(this._commandWindow)) {
            this.addChild(this._commandWindow);
        }
    };
    
    //-----------------------------------------------------------------------------
    // Window_NWSWeaponHelp
    //-----------------------------------------------------------------------------
    
    function Window_NWSWeaponHelp() {
        this.initialize.apply(this, arguments);
    }
    
    Window_NWSWeaponHelp.prototype = Object.create(Window_Help.prototype);
    Window_NWSWeaponHelp.prototype.constructor = Window_NWSWeaponHelp;
    
    Window_NWSWeaponHelp.prototype.initialize = function() {
        Window_Help.prototype.initialize.apply(this, arguments);
        var x = 408, y = 64, w = 312;
        x = Graphics.boxWidth - x - w;
        this.move(x, y, w, 0);
        this.update();
        this.openness = 0;
    };
    
    Game_Item.prototype.detailText = function(type, noIcon) {
        var detail = $gameSystem.hasDetail(this._itemId);
        var unreleasedText = '\\c[6]%1\\c[0]'.format(
            ['見たことのない武器だ。\n使って効果を確かめてみよう。',
             'You haven\'t used this weapon yet.\nUse it against an enemy to check its information.',
             '사용한 적이 없는 무기다.\n사용해서 효과를 확인해 보자.',
             '不曾用過的武器。\n使用看看有什麼效果吧。',
             '不曾用过的武器。\n使用看看有什么效果吧。'][_lg()]
        );
        var etexts = (type === 2 ?
                      ['%2  \\b%1\\b'.format(this.object().meta.rangeText,
                        ['攻撃範囲', 'Range ', '공격범위', '攻擊範圍', '攻击范围'][_lg()]),
                       '%2  \\b%1\\g\\b'.format(this.object().price,
                        ['売値', 'Price', '매매', '賣價', '卖价'][_lg()])
                      ] : []).concat(this.object().meta.effectTexts);
        return '%1\\{%8\\}\\}\n\\i[469]\\} \\{%3 \\i[470]\\} \\{%4\\{%5%6\\c[0]%2%7'.format(
            '\\b\\w[0]\\mx[2]\\y[2]%1\\mx[2]\\b'.format(this.itemName(!noIcon, 0, 1)) +
                (this._level > 1 ? ' \\}\\b\\y[2]\\mx[2]Lv\\mx[2].\\mx[2]%1\\{\\b\\my[4] '.format(this._level) : '\\my[2]'),
            type ? ('\n\\}\\w[1]' + ((detail ? this.itemDescription() : unreleasedText)) + '\\w[0] \\{') : '',
            '\\b%1\\b'.format(detail ? this.power() : '\\c[0]？\\c[0]'),
            '\\b%1\\b'.format(detail ? (this._useable <= 1 ? '\\c[2]' : '') + this._useable : '\\c[0]？\\c[0]'),
            this._togi ? '\\} \\i[471]\\{' : '',
            this._mekki ? '\\}%1\\i[472]\\{'.format(this._togi ? '' : ' ') : '',
            !type ? '\\}' : !this.effectText(etexts) || !detail ? '' : '\\}\\}\\}\\}\\}\n\\{\\{\\{\\{\n' + this.effectText(etexts),
            !type ? '' : '\\{\\}'
        );
    }
    
    Window_NWSWeaponHelp.prototype.setItem = function(item) {
        if (!item) {
            this.height = this.fittingHeight(3);
            this._weapon = null;
            this.setText('');
            return;
        }
        this._weapon = item;
        var type = SceneManager._scene._weaponListWindow._type;
        var useable = type !== 3;
        var text = item.detailText(type === 3 ? 2 : 1);
//        var height = this.fittingHeight(text.split('\n').length) + 4;
        this.resetFontSettings();
        this.drawTextEx(text, 0, -999);
        var height = this._lastTextExHeight + 50;
        if (height > this.height) {
            this.height = height;
            this.createContents();
        } else {
            this.height = height;
        }
        this.resetFontSettings();
        this.setText(text);
    };
    
    Window_NWSWeaponHelp.prototype.refresh = function() {
        this.contents.clear();
        if (this._weapon && $gameSystem.hasDetail(this._weapon._itemId) &&
            SceneManager._scene._weaponListWindow._type === 3) {
            var size = 144, x = this.contents.width - 12 - size, y = -4;
            this.contents.paintOpacity = 180;
            this.drawWeapon(this._weapon._itemId, this._weapon.imageName(), x, y, size);
            this.contents.paintOpacity = 255;
        }
        this.drawTextEx(this._text, this.textPadding(), 0);
    };
    
    //-----------------------------------------------------------------------------
    // Sprite_Button
    //-----------------------------------------------------------------------------
    
    var _Sprite_Button_update = Sprite_Button.prototype.update;
    Sprite_Button.prototype.update = function() {
        _Sprite_Button_update.apply(this, arguments);
        if (this._effect) {
            if (!this._effectSprite) {
                this._effectSprite = new Sprite(ImageManager.loadSystem('ButtonEffect'));
                this._effectSprite.setFrame(0, 0, 0, 0);
                this._effectSprite._animeCount = 0;
                this.addChild(this._effectSprite);
                this.addChild(this._textSprite);
            }
            var size = 96;
            var c = this._effectSprite._animeCount++;
            var x = this._effect >= 100 ? (1 + Math.floor(c / 25) % 3) * size : 0;
            var y = this._effect >= 100 ? 0 : Math.ceil((1 - this._effect / 100) * size);
            this._effectSprite.setFrame(x, y, size, size - y);
            this._effectSprite.y = y;
            this._effectSprite.opacity = this._touching ? 180 : 255;
            var cw = SceneManager._scene._messageWindow._choiceWindow;
            if (this._effect >= 100 && !cw.isOpen()) {
                if (!this._skillOkSprite) {
                    var name = 'skillOkWd' + (_lg() ? '_' + _lg() : '');
                    this._skillOkSprite = new Sprite(ImageManager.loadSystem(name));
                    this._skillOkSprite.anchor.x = 0.5;
                    this._skillOkSprite.x = this.width / 2, this._skillOkSprite.y = -85;
                    this.addChild(this._skillOkSprite);
                }
            } else if (this._skillOkSprite) {
                this.removeChild(this._skillOkSprite);
                this._skillOkSprite = null;
            }
        } else if (this._effectSprite) {
            this.removeChild(this._effectSprite);
            this._effectSprite = null;
            if (this._skillOkSprite) {
                this.removeChild(this._skillOkSprite);
                this._skillOkSprite = null;
            }
        }
    };
    
    //-----------------------------------------------------------------------------
    // Window_MapName　デザイン＆仕様変更
    //-----------------------------------------------------------------------------
    
    var _Window_MapName_initialize = Window_MapName.prototype.initialize;
    Window_MapName.prototype.initialize = function() {
        _Window_MapName_initialize.apply(this, arguments);
        this.setBackgroundType(1);
        this._dimmerSprite.bitmap = ImageManager.loadPicture('mapNameBack');
        this.contentsOpacity = 255;
        this.padding = 0;
        this.move(0, 0, 336, 72);
        this.createContents();
        this.contents.fontBold = true;
        this.refresh();
        this.openness = 0;
        this._showCount = 0;
    };
    
    Window_MapName.prototype.update = function() {
        Window_Base.prototype.update.call(this);
        if ($gameMap.displayName() && $gameMap.isNameDisplayEnabled() && !$gameSwitches.value(3) &&
            !SceneManager._scene.isBusy() && !$gameSystem.inMenu()) {
            this.openness = 255;
        } else {
            this.openness = 0;
        }
        this._dimmerSprite.opacity = this.openness;
    };
    
    Window_MapName.prototype.refresh = function() {
        this.contents.clear();
        if ($gameMap.displayName()) {
            this.drawText($gameMap.displayName(), 48, 22, 240, 'center');
        }
    };
    
    //-----------------------------------------------------------------------------
    // Window_NWSMessageScroll
    //-----------------------------------------------------------------------------
    
    function Window_NWSMessageScroll() {
        this.initialize.apply(this, arguments);
    }
    
    Window_NWSMessageScroll.prototype = Object.create(Window_Selectable.prototype);
    Window_NWSMessageScroll.prototype.constructor = Window_NWSMessageScroll;
    
    Window_NWSMessageScroll.prototype.setText = function(text) {
        this._text = text,  this._scrollY = 0, this._lastScrollY = 0;
        this.createContents();
        this.refresh();
    };
    
    Window_NWSMessageScroll.prototype.refresh = function() {
        this.contents.clear();
        this.drawTextEx(this._text || '', 0, 0);
        this._refreshScrollBar();
    };
    
    Window_NWSMessageScroll.prototype.maxItems = function() {
        return this._contentsHeight / this.itemHeight();
    };
    Window_NWSMessageScroll.prototype.itemWidth = function() {
        return 0;
    };
    Window_NWSMessageScroll.prototype.itemHeight = function() {
        return 28;
    };
    Window_NWSMessageScroll.prototype.cursorDown = function() {
        this.scrollDown();
    };
    Window_NWSMessageScroll.prototype.cursorUp = function() {
        this.scrollUp();
    };
    
    
    //-----------------------------------------------------------------------------
    // Window_NWSStatesInfo
    //-----------------------------------------------------------------------------
    
    function Window_NWSStatesInfo() {
        this.initialize.apply(this, arguments);
    }
    
    Window_NWSStatesInfo.prototype = Object.create(Window_NWSMessageScroll.prototype);
    Window_NWSStatesInfo.prototype.constructor = Window_NWSStatesInfo;

    Window_NWSStatesInfo.prototype.initialize = function() {
        Window_NWSMessageScroll.prototype.initialize.call(this, 10, 0, 192, 0);
        this.loadWindowskin(5);
    };
    Window_NWSStatesInfo.prototype.standardFontSize = function() {
        return 14;
    };
    Window_NWSStatesInfo.prototype.calcTextHeight = function() {
        return Window_Base.prototype.calcTextHeight.apply(this, arguments) - 4;
    };
    Window_NWSStatesInfo.prototype.makeFontBigger = function() {
        this.contents.fontSize = Math.min(this.contents.fontSize + 2, 96);
    };
    Window_NWSStatesInfo.prototype.makeFontSmaller = function() {
        this.contents.fontSize = Math.max(this.contents.fontSize - 2, 8);
    };
    
    Window_NWSStatesInfo.prototype.update = function() {
        Window_NWSMessageScroll.prototype.update.apply(this, arguments);
        if (!SceneManager._scene._buttons[0]) return;
        var vis = SceneManager._scene._buttons[0].visible;
        if (this.isClosed() && this._text && vis) this.openness = 255;
        if (this.isOpen() && (!this._text || !vis)) this.openness = 0;
        
        var lastInfo = [
            $gamePlayer._D._states.toString(),
            $gamePlayer._D._buffs.toString(),
            NWS()._turn
        ].toString();
        if (!this._lastInfo || this._lastInfo !== lastInfo) {
            this._lastInfo = lastInfo;
            var text = this.getText();
            if (text === this._text) return;
            var texts = text.split('\n');
            this.drawTextEx(text, 0, -999);
            this._barMaxHeight = ((this._lastTextExHeight || 0) + 14 + 4);
            var h = 152 + 48;
            this.height = (this._barMaxHeight + (text ? 24 : 0)).clamp(0, h);
            this.y = (64 + h - this.height).clamp(64, 64 + h);
            this.createContents();
            this.setText(text);
            this._text ? this.activate() : this.deactivate();
        }
    };
    Window_NWSStatesInfo.prototype.maxRows = function() {
        return this._barMaxHeight;
    };
    Window_NWSStatesInfo.prototype.maxPageRows = function() {
        return this.height - 24;
    };
    Window_NWSStatesInfo.prototype.itemHeight = function() {
        return 1;
    };
    Window_NWSStatesInfo.prototype.scrollDown = function() {
        for (var i = 0; i < 28; i++) Window_Selectable.prototype.scrollDown.apply(this, arguments);
    };
    Window_NWSStatesInfo.prototype.scrollUp = function() {
        for (var i = 0; i < 28; i++) Window_Selectable.prototype.scrollUp.apply(this, arguments);
    };
    
    Window_NWSStatesInfo.prototype.getText = function() {
        var states = [];
        var battler = $gamePlayer._D;
        battler._states.forEach(function(stateId) {
            if (!stateId || !$dataStates[stateId].meta.text) return;
            states.push(['state', $dataStates[stateId], battler._stateTurns[stateId]]);
        }, this);
        battler._buffs.forEach(function(value, paramId) {
            if (!value) return;
            states.push(['buff', [paramId, value], battler._buffTurns[paramId]]);
        }, this);
        states.sort(function(a, b) {
            return a[2] - b[2];
        });
        var text = '';
        states.forEach(function(state) {
            var isBuff = state[0] === 'buff';
            var color = isBuff ? (state[1][1] > 0 ? 17 : 24) : (state[1].meta.textColor || 24);
            var name = isBuff ? (TextManager.param(state[1][0]) + _lgCheck(state[1][1] > 0 ? 'アップ' : 'ダウン')) :
                       _lgCheck(state[1].name);
            var turn = (state[2] + (isBuff ? 1 : 0)).clamp(1, isBuff ? 5 : 999);
            var mt = isBuff ? '' : state[1].meta['text' + (!_lg() ? '' : _lg())];
            var detail = isBuff ? '' : (mt && mt !== true ? '\n' + mt : '')
            var t = '\\c[%1]● %2\n【\\}%3\\{】\\}%4\\{'.format(color, name, _lgCheck('のこり\\{%1\\}ターン').format(turn), detail);
            text += (text ? '\n' : '') + t;
        })
        return text;
    };
    
    Window_NWSStatesInfo.prototype.cursorDown = function() {};
    Window_NWSStatesInfo.prototype.cursorUp = function() {};
    
    //-----------------------------------------------------------------------------
    // Window_NWSCommand
    //-----------------------------------------------------------------------------
    
    function Window_NWSCommand() {
        this.initialize.apply(this, arguments);
    }
    
    Window_NWSCommand.prototype = Object.create(Window_Selectable.prototype);
    Window_NWSCommand.prototype.constructor = Window_NWSCommand;
    
    Window_NWSCommand.prototype.maxItems = function() {
        return this._list ? this._list.length : 0;
    };
    Window_NWSCommand.prototype.drawItem = function(index) {
        var text = this._list[index];
        if (_lg()) {
            var list = [
                ['装備する', '装備をはずす', 'ロックする', 'ロック解除', '移動する', '破棄する', '武器庫にしまう', '取り出す', '作成する', '選ぶ', '選ばない', 'やめる'],
                ['Equip', 'Remove', 'Lock', 'Unlock', 'Move', 'Discard', 'Stash', 'Retrieve', 'Create', 'Select', 'Deselect', 'Cancel'],
                ['장비하기', '장비 해제하기', '잠그기', '잠금', '해제하기', '이동하기', '버리기', '무기고에 넣기', '꺼내기', '주문하기', '선택을 취소하다', '그만둔다'],
                ['裝備', '卸下裝備', '鎖定', '解除鎖定', '移動', '捨棄', '收進武器庫', '取出', '製作', '選擇', '取消', '算了'],
                ['装备', '卸下装备', '锁定', '解除锁定', '移动', '舍弃', '收进武器库', '取出', '制作', '选择', '取消', '算了']
            ];
            text = list[_lg()][list[0].indexOf(text)] || text;
        }
        var rect = this.itemRect(index);
        this.drawText(text, rect.x, rect.y, rect.width, 'center');
    };
    
    //-----------------------------------------------------------------------------
    // Window_NWSFukidashi
    //-----------------------------------------------------------------------------
    
    function Window_NWSFukidashi() {
        this.initialize.apply(this, arguments);
    }
    
    Window_NWSFukidashi.prototype = Object.create(Window_Base.prototype);
    Window_NWSFukidashi.prototype.constructor = Window_NWSFukidashi;
    
    Window_NWSFukidashi.prototype.initialize = function() {
        Window_Base.prototype.initialize.apply(this, arguments);
        this._text = this._lastText = '';
        this._partsVisible = true;
        this._partsPos = 0;
        this.loadWindowskin(4);
        this._partsSprite = new Sprite(ImageManager.loadSystem('wParts'));
        this._partsSprite.anchor.x = 0.5;
        this.addChild(this._partsSprite);
    };
    
    Window_NWSFukidashi.prototype.lineHeight = function() {
        return 28;
    };
    Window_NWSFukidashi.prototype.standardFontSize = function() {
        return 19;
    };
    
    Window_NWSFukidashi.prototype.refresh = function() {
        if (this._lastText === this._text) return;
        this._lastText = this._text;
        var ts = this._text.split('\n'), _this = this;
        this.width = Math.ceil(ts.reduce(function(a, b) {
            return Math.max(_this.drawTextEx(b, 0, 999), a);
        }, 0)) + 28;
        this.drawTextEx(this._text + '\n ', 0, 999);
        this.height = this._lastTextExHeight + 24;
        this.createContents();
        this.drawTextEx(this._text, 0, 0);
    };
    
    Window_NWSFukidashi.prototype.update = function() {
        Window_Base.prototype.update.apply(this, arguments);
        this._partsSprite.visible = this._partsVisible;
        if (!this._partsSprite.visible) return;
        var pos = this._partsPos || 0;
        this._partsSprite.x = [0, 1, 4, 5].contains(pos) ? this.width  / 2 : [2, 3].contains(pos) ? 7 : this.width - 7;
        this._partsSprite.y = [2, 3, 6, 7].contains(pos) ? this.height / 2 : [4, 5].contains(pos) ? 7 : this.height - 7;
        this._partsSprite.rotation = Math.floor(pos / 2) * Math.PI / 2;
        this._partsSprite.scale.x = pos % 2 ? -1 : 1;
        this._partsSprite.opacity = this.openness >= 255 ? 255 : 0;
    };

    var _Window_Base_resetFontSettings = Window_Base.prototype.resetFontSettings;
    Window_Base.prototype.resetFontSettings = function() {
        if (this._noResetFontSettings) return;
        _Window_Base_resetFontSettings.apply(this, arguments);
    };
    
    //-----------------------------------------------------------------------------
    // 経験値表示
    //-----------------------------------------------------------------------------
    
    function Window_NWSExp() {
        this.initialize.apply(this, arguments);
    }
    
    Window_NWSExp.prototype = Object.create(Window_NWSFukidashi.prototype);
    Window_NWSExp.prototype.constructor = Window_NWSExp;
    
    Window_NWSExp.prototype.initialize = function() {
        Window_NWSFukidashi.prototype.initialize.apply(this, arguments);
        this._partsVisible = false;
        this.width = 3.5 * 48 + 10, this.height = 48 + 10;
        this.opacity = 0;
        this.visible = false;
        
        var a = 18;
        this._gaugeSprite = new Sprite(new Bitmap(144, 18));
        this._gaugeSprite.x = 17, this._gaugeSprite.y = 20;
        this.addChild(this._gaugeSprite);
        this._textWindow = new Window_Base(-a, -a - 24, this.width + a * 2, this.height + a * 2 + 24);
        this._textWindow.opacity = 0;
        this.addChild(this._textWindow);
        
        this._count = 999;
        this._plus = 0;
        this._plusSpeed = 0;
        this._exp = 0;
        this._requiredExp = 0;
        this._level = 0;
        this._isMax = false;
        this._fast = false;
        this._type = 0;
    };
    
    Window_NWSExp.prototype.setup = function(type, plus) {
        var actor = $gamePlayer._D;
        this._exp = actor.currentExp();
        this._requiredExp = actor.nextRequiredExp();
        this._level = actor.level;
        this._isMax = actor.isMaxLevel();
        this._textWindow.contents.clear();
        this._fast = false;
        if (type === 0) {
            if (this._isMax) return this.visible = false;
            this._type = 0;
            this.x = (Graphics.boxWidth - this.width) / 2;
            this.y = 112 - this.height / 2;
            this._baseX = this.x;
            this._baseY = this.y;
            this.opacity = 0;
            this.plus = plus;
            this._plus = plus;
            this._plusSpeed = Math.ceil(Math.min(plus, this._requiredExp) / 15);
        } else if (type === 1) {
            this._type = 1;
            this.x = 10, this.y = 266 + (!NWS()._equip ? 28 : 0) - this.height;
            this._baseX = this.x;
            this._baseY = this.y;
            this.refreshText(0);
            this.refreshText(1);
            this.opacity = 255;
            this._gaugeSprite.opacity = 255;
        }
        this._count = 0;
        this.refreshGauge();
        this.visible = true;
    };
    
    Window_NWSExp.prototype.update = function() {
        Window_NWSFukidashi.prototype.update.apply(this, arguments);
        if (this._type === 1) {
            if (!SceneManager._scene._menuWindow.active) this.visible = false;
            return;
        }
        var tw = this._textWindow;
        if (this._count >= 16 && (Input.isPressed('ok') || TouchInput.isPressed())) this._fast = true;
        if (this._count < 6) {
            this.opacity += 255 / 6;
        } else if (this._count === 6) {
            this.refreshText(0);
            this.refreshText(1);
        } else if (this._count < 32) {
            if (this._fast) this._count = 32;
        } else if (this._count < 101) {
            if ((this._count - 32) % 5 === 0) AudioManager.playSe({name:'01_Exp', volume:90, pitch:100, pan:0});
            if (!(this._count % 2)) {
                var value = this._plusSpeed.clamp(0, Math.min(this._requiredExp, this._plus));
                this._plus -= value;
                this._requiredExp = (this._requiredExp - value).clamp(0, Infinity);
                this.refreshText(1);
                this.refreshGauge();
            }
            if (this._requiredExp === 0) {
                NWS()._levelUpParam = this._plus;
                $gamePlayer._D.gainExp(this.plus - this._plus);
                this._level++;
                this.refreshText(1);
                this.y = this._baseY;
                $gameScreen.startFlash([255, 255, 255, 180], 15);
                AudioManager.playSe({name:'Saint9', volume:100, pitch:100, pan:0});
                this._count = 201;
            } else if (this._plus === 0) {
                NWS()._levelUpParam = 0;
                $gamePlayer._D.gainExp(this.plus);
                this.y = this._baseY;
                this._count = 101;
            } else {
                this.y = this._baseY + [-2, -3, -3, -2, -1, 0][(this._count - 32) % 6];
            }
        } else if (this._count < 115) {
        } else if (this._count < 130) {
            if (this._fast) this._count = 130;
        } else if (this._count < 201) {
            this.opacity -= 255 / 6;
            if (this.opacity <= 0) this.visible = false, this._count = 999;
        } else if (this._count < 231) {
            var c = this._count - 201;
            var a = (30 - c) / 30;
            var b = Math.cos((Math.PI * 2) / 30 * c);
            this._windowSpriteContainer.scale.x = 1 + 0.5 * a * b;
            this._windowSpriteContainer.scale.y = 1 + 0.5 * a * -b;
            this.x = this._baseX + this.width / 2  - this._windowSpriteContainer.scale.x * this.width / 2;
            this.y = this._baseY + this.height / 2 - this._windowSpriteContainer.scale.y * this.height / 2;
        } else if (this._count < 251) {
            this._windowSpriteContainer.scale.x = this._windowSpriteContainer.scale.y = 1;
        } else if (this._count < 301) {
            this.opacity -= 255 / 6;
            if (this.opacity <= 0) this.visible = false, this._count = 999;
        } else {
        }
        var tw = this._textWindow, gw = this._gaugeSprite;
        tw.x = this._baseX - this.x - 18, tw.y = this._baseY - this.y - 42;
        gw.scale.x = this._windowSpriteContainer.scale.x;
        gw.scale.y = this._windowSpriteContainer.scale.y;
        gw.opacity = this.opacity;
        this._count++;
    };
    
    Window_NWSExp.prototype.refreshText = function(type) {
        //type 0(最初) 1(数値表示)
        var tw = this._textWindow;
        var x, y, w, h, text;
        var sm = this._type === 1;
        tw.contents.fontSize = sm ? 16 : 19;
        if (type === 0) {
            x = 12, y = !sm ? 18 : 24;
            tw.contents.fontBold = true;
            tw.drawText(['経験値', 'EXP', '경험치', '經驗值', '经验值'][_lg()], x, y, tw.width);
            y += !sm ? 55 : 43;
            tw.contents.fontBold = false;
            if (this._type === 0) {
                tw.contents.fontSize -= 8;
                tw.drawText('Lv.', x, y, tw.width);
                tw.contents.fontSize += 8;
            }
        } else if (type === 1) {
            x = 80, y = !sm ? 18 : 24, w = tw.contents.width - x - 24, h = 24;
            tw.contents.clearRect(x, y, w + 2, h);
            tw.contents.fontBold = true;
            text = this._type === 0 ? '+' + this._plus : this._exp;
            tw.drawText(text, x, y, w, 'right');
            x = 32, y += !sm ? 52 : 40, w = 32;
            if (this._type === 0) {
                tw.contents.clearRect(x, y, w, h);
                tw.drawText('' + this._level, x, y, w);
            }
            x += 32, w = tw.contents.width - x - 24;
            tw.contents.clearRect(x, y, w + 8, h);
            text = this._isMax ? 'MAX!!' : ['あと', 'Next', '앞으로', '還差', '还差'][_lg()] + ' ' + this._requiredExp;
            tw.drawText(text, x, y, w, 'right');
        }
    };
        
    Window_NWSExp.prototype.refreshGauge = function() {
        var gw = this._gaugeSprite, w = gw.width, h = gw.height;
        gw.bitmap.clear();
        gw.bitmap.fillRect(0, 0, w, h, 'white');
        var color1 = this.textColor(29), color2 = this.textColor(24);
        gw.bitmap.gradientFillRect(1, 1, w - 2, h - 2, color1, color2);
        var actor = $gamePlayer._D;
        var rate = this._isMax ? 0 : (this._requiredExp / (actor.nextLevelExp() - actor.currentLevelExp()));
        var w2 = Math.ceil((w - 2) * rate);
        gw.bitmap.fillRect(1 + w - 2 - w2, 1, w2, h - 2, this.textColor(19));
    };
    
    //-----------------------------------------------------------------------------
    // 武器アイコン描写機能追加
    //-----------------------------------------------------------------------------
    
    var _Window_Base_drawIcon = Window_Base.prototype.drawIcon;
    Window_Base.prototype.drawIcon = function(iconIndex, x, y, size) {
        var alpha = '';
        if (typeof iconIndex === 'string') {
            for (var i = 0; i < iconIndex.length; i++) {
                alpha = iconIndex.substr(-1) + alpha;
                iconIndex = iconIndex.substr(0, iconIndex.length - 1);
                if (Number(iconIndex)) break;
            }
            iconIndex = Number(iconIndex) || 0;
        }
        if (iconIndex >= 1000) {
            iconIndex -= 1000;
            if (!size) {
                size = Window_Base._iconHeight * this.contents.fontSize / 20;
                var baseSize = Window_Base._iconHeight * this.contents.fontSize / 24;
                x += -(size - baseSize) / 2;
                y += -(size - baseSize) / 2;
            }
            this.drawWeapon(iconIndex, 'Weapon_' + iconIndex + alpha, x, y, size);
        }
        _Window_Base_drawIcon.apply(this, arguments);
    };
    var _Window_Base_processDrawIcon = Window_Base.prototype.processDrawIcon;
    Window_Base.prototype.processDrawIcon = function(iconIndex, textState) {
        _Window_Base_processDrawIcon.apply(this, arguments);
        if (iconIndex >= 1000) textState.x += 4;
    };
    
    Window_Base.prototype.drawWeapon = function(weaponId, imageName, x, y, size, text1, text2, text3, text4) {
        if (!$dataItems[weaponId]) return;
        imageName = imageName || ('Weapon_' + weaponId);
        var bitmap = ImageManager.loadBitmap('img/weapons/', imageName, 0, 0);
        var opa = this.contents.paintOpacity;
        bitmap.addLoadListener(function(){
            var s = (bitmap.width + bitmap.height) / Math.sqrt(2);
            var bitmap2 = new Bitmap(s, s);
            bitmap2._context.translate(0, bitmap.width / Math.sqrt(2));
            bitmap2._context.rotate(-Math.PI / 4);
            bitmap2.blt(bitmap, 0, 0, bitmap.width, bitmap.height, 0, 0);
            
            var opa2 = this.contents.paintOpacity;
            this.contents.paintOpacity = opa;
            this.contents._context.shadowColor = 'rgba(73,41,10,1)';
            if (!this._noWeaponOutline && this.contents.paintOpacity === 255 &&
                ($dataItems[weaponId].meta.outline || $dataItems[weaponId].meta.outlineHalf)) {
                for (var i = 0; i < 4; i++) {
                    var a = $dataItems[weaponId].meta.outline ? 2 : 1;
                    this.contents._context.shadowOffsetX = [0, a, -a, 0][i];
                    this.contents._context.shadowOffsetY = [a, 0, 0, -a][i];
                    this.contents.blt(bitmap2, 0, 0, bitmap2.width, bitmap2.height, x + 2, y + 2, size - 4, size - 4);
                }
            } else {
//                if (!this._noWeaponOutline) {
//                    this.contents._context.shadowOffsetX = [0, 1, -1, 0][i];
//                    this.contents._context.shadowOffsetY = [1, 0, 0, -1][i];
//                }
                this.contents.blt(bitmap2, 0, 0, bitmap2.width, bitmap2.height, x + 2, y + 2, size - 4, size - 4);
            }
            this.contents._context.shadowOffsetX = this.contents._context.shadowOffsetY = 0;
            this.contents._context.shadowColor = 'rgba(0,0,0,0)';
            this.contents.paintOpacity = opa;
            
            if (text1) {
                this.contents.fontSize = 14;
                var tw = 20, th = 16, tx = x + 2, ty = y + size - th - 2;
                this.contents.fillRect(tx, ty, tw, th, 'rgba(61, 41, 9, 1)');
                this.drawText(text1, tx, ty - 6, tw, 'center');
            }
            if (text2) {
                this.contents.fontSize = 14;
                this.drawTextEx(text2, x + 49, y + size - 29);
            }
            if (text3) {
                this.contents.fontSize = 14;
                this._noResetFontSettings = true;
                var tw = this.drawTextEx(text3, 0, -999) + 6, th = 16, tx = x + 2, ty = y + size - th - 2;
                this.contents.fillRect(tx, ty, tw, th, 'rgba(61, 41, 9, 1)');
                this.drawTextEx(text3, tx + 3, ty - 6 + 3);
                this._noResetFontSettings = false;
            }
            if (text4) {
                this.contents.fontSize = 13;
                this._noResetFontSettings = true;
                this.drawTextEx(text4, x + 2, y + 2);
                this._noResetFontSettings = false;
            }
            
            this.contents.paintOpacity = opa2;
        }.bind(this));
    };
    
    //-----------------------------------------------------------------------------
    // 言語選択機能
    //-----------------------------------------------------------------------------
    
    var _Scene_Title_start = Scene_Title.prototype.start;
    Scene_Title.prototype.start = function() {
        if (_lg() < 0) {
            this._blackSprite = new Sprite(new Bitmap(48, 48));
            this._blackSprite.bitmap.fillAll('black');
            this._blackSprite.scale.x = Graphics.boxWidth / 48, this._blackSprite.scale.y = Graphics.boxHeight / 48;
            this.addChildAt(this._blackSprite, this.children.indexOf(this._gameTitleSprite));
            
            var wd = new Window_Command(0, 0);
            wd.addCommand('日本語', 'jpn');
            wd.addCommand('English', 'eng',);
            wd.addCommand('한국어', 'kor');
            wd.addCommand('简体中文', 'kan');
            wd.addCommand('繁體中文', 'han');
            wd.setHandler('jpn', this.onLanguage.bind(this, true));
            wd.setHandler('eng', this.onLanguage.bind(this, true));
            wd.setHandler('kor', this.onLanguage.bind(this, true));
            wd.setHandler('kan', this.onLanguage.bind(this, true));
            wd.setHandler('han', this.onLanguage.bind(this, true));
            wd.itemTextAlign = function() {return 'center'};
            wd.windowWidth = function() {return 144};
            wd.height = wd.windowHeight();
            wd.x = (Graphics.boxWidth - wd.width) / 2, wd.y = (Graphics.boxHeight - wd.height) / 2;
            wd.createContents(), wd.drawAllItems();
            wd._titleText = '言語選択 / Language';
            wd.openness = 0, wd.open(), wd.activate();
            this._languageWindow = wd;
            this.addWindow(wd);
            this._waitLanguage = true;
        } else {
            _Scene_Title_start.call(this);
        }
    };
    Scene_Title.prototype.onLanguage = function(refresh) {
        $gameVariables.setValue(35, this._languageWindow.index());
        ConfigManager.importCustomParams();
        this._languageWindow.close();
        this._waitLanguage = false;
        this.centerSprite(this._backSprite1);
        this.centerSprite(this._backSprite2);
        this._backSprite1.bitmap = ImageManager.loadTitle1($dataSystem.title1Name + (_lg() || ''));
        this._windowLayer.removeChild(this._commandWindow), this._commandWindow = null;
        this.createCommandWindow();
        this.playTitleMusic();
        this.startFadeIn(this.fadeSpeed(), false);
        this.removeChild(this._blackSprite);
        ConfigManager.save();
    };
    var _Scene_Title_update = Scene_Title.prototype.update;
    Scene_Title.prototype.update = function() {
        if (this._waitLanguage) Scene_Base.prototype.update.call(this);
        else _Scene_Title_update.apply(this, arguments);
    };
    
    //-----------------------------------------------------------------------------
    // セーブしました表示機能
    //-----------------------------------------------------------------------------
    
    var _SceneManager_update = SceneManager.update;
    SceneManager.update = function() {
        _SceneManager_update.call(this);
        if ($gameSystem && !this._savedWindow) {
            var sw = this._savedWindow = new Window_Base(384, 12, 288, 60);
            sw.setBackgroundType(2);
            var text = '\\i[182] ' + _lgCheck('セーブしました。');
            sw.contents.fontBold = true, sw.contents.outlineWidth = 6;
            sw.drawTextKNH(text, 4, 0, sw.contents.width - 32, 'right');
            sw.contents.outlineWidth = 0;
            sw.drawTextKNH(text, 4, 0, sw.contents.width - 32, 'right');
            sw.contentsOpacity = 0;
            this._scene.addChild(this._savedWindow);
        }
        if (this._savedWindow) {
            var sw = this._savedWindow;
            sw._showCount = sw._showCount || 0;
            if (sw._showCount <= 0) sw.contentsOpacity -= 255 / 60;
            else sw._showCount--, sw.contentsOpacity = 255;
            sw.update();
        }
    };
    
    Game_System.prototype.displaySaved = function() {
        var sw = SceneManager._savedWindow;
        sw._showCount = 120;
    };

    var _SceneManager_onSceneStart = SceneManager.onSceneStart;
    SceneManager.onSceneStart = function() {
        _SceneManager_onSceneStart.call(this);
        if (this._savedWindow) this._scene.addChild(this._savedWindow);
    };
    
    var _SceneManager_snapForBackground = SceneManager.snapForBackground;
    SceneManager.snapForBackground = function() {
        this._savedWindow.visible = false;
        _SceneManager_snapForBackground.call(this);
        this._savedWindow.visible = true;
    };

    var _Scene_Base_createFadeSprite = Scene_Base.prototype.createFadeSprite;
    Scene_Base.prototype.createFadeSprite = function(white) {
        _Scene_Base_createFadeSprite.call(this, white);
        if (this._savedWindow) this.addChild(this._savedWindow);
    };
    
    var _DataManager_saveGame = DataManager.saveGame;
    DataManager.saveGame = function(savefileId) {
        $gameSystem.displaySaved();
        return _DataManager_saveGame.apply(this, arguments);
    }
    
})();
