//=============================================================================
// AltWindowFramejs
//=============================================================================

/*:
 * @plugindesc Alternative Windowflame.
 * @author KADOKAWA
 *
 * @help This plugin does not provide plugin commands.
 * @requiredAssets img/system/Window_Talk
 * @requiredAssets img/system/Window_Battle
 * @requiredAssets img/system/Window_Status
 * @requiredAssets img/system/Window_Other
 */

/*:ja
 * @plugindesc ウィンドウのフレームを変更します。
 * @author KADOKAWA
 *
 * @help このプラグインには、プラグインコマンドはありません。
 * @requiredAssets img/system/Window_null
 * @requiredAssets img/system/Window1
 * @requiredAssets img/system/Window1_p
 * @requiredAssets img/system/Window1_p2
 * @requiredAssets img/system/Window2
 * @requiredAssets img/system/Window2_p
 * @requiredAssets img/system/Window2_p2
 * @requiredAssets img/system/Window3
 * @requiredAssets img/system/Window3_p
 * @requiredAssets img/system/Window3_p2
 * @requiredAssets img/system/Window4
 * @requiredAssets img/system/Window4_p
 * @requiredAssets img/system/Window5
 * @requiredAssets img/system/wParts
 */

 (function() {
    
    Window_Base.prototype.standardBackOpacity = function() {
        return 255;
    };

    Window_Base.prototype.loadWindowskin = function(type) {
        var name = 'Window' + (type || 1);
        this._windowskinName = name;
        this.windowskin = ImageManager.loadSystem(name);
    };
     
     var _Window_Base_resetFontSettings = Window_Base.prototype.resetFontSettings;
     Window_Base.prototype.resetFontSettings = function() {
        _Window_Base_resetFontSettings.apply(this, arguments);
        if (this._windowskinName && !this._fontNormal) {
            if (['Window3', 'Window3_p'].contains(this._windowskinName))
                this.contents.outlineColor = 'rgba(238, 233, 225, 1)';
            else 
                this.contents.outlineColor = 'rgba(73,41,10,1)';
            
            if (['Window4_p'].contains(this._windowskinName))
                this.contents.outlineWidth = 7;
        }     
    };
     
    Scene_Boot.prototype.loadSystemWindowImage = function() {
        ImageManager.reserveSystem('Window1');
        ImageManager.reserveSystem('Window1_p');
        ImageManager.reserveSystem('Window1_p2');
        ImageManager.reserveSystem('Window2');
        ImageManager.reserveSystem('Window2_p');
        ImageManager.reserveSystem('Window2_p2');
        ImageManager.reserveSystem('Window3');
        ImageManager.reserveSystem('Window3_p');
        ImageManager.reserveSystem('Window3_p2');
        ImageManager.reserveSystem('Window4');
        ImageManager.reserveSystem('Window4_p');
        ImageManager.reserveSystem('Window5');
        ImageManager.reserveSystem('Window_null');
    };
    
    var _Window_Base_initialize = Window_Base.prototype.initialize;
    Window_Base.prototype.initialize = function() {
        _Window_Base_initialize.apply(this, arguments);
        this.loadWindowskin();
        this._refreshAllParts();
    };

    Window.prototype._refreshFrame = function() {
        var w = this._width;
        var h = this._height;
        var m = 24;
        var bitmap = new Bitmap(w, h);

        this._windowFrameSprite.bitmap = bitmap;
        this._windowFrameSprite.setFrame(0, 0, w, h);

        if (w > 0 && h > 0) {
            var skin = this.windowskin;

            // frame
            var p = 96;
            var q = 96;
            bitmap.blt(skin, p+0,   0+0,   m,     m,     0,   0,   m,     m);       // left top corner
            bitmap.blt(skin, p+0,   0+q-m, m,     m,     0,   h-m, m,     m);       // left bottom corner
            bitmap.blt(skin, p+q-m, 0+0,   m,     m,     w-m, 0,   m,     m);       // right top corner
            bitmap.blt(skin, p+q-m, 0+q-m, m,     m,     w-m, h-m, m,     m);       // right bottom corner

            var frameHeight = 48;
            var heightCount = Math.floor((h-frameHeight) / frameHeight);
            var remainder = (h-frameHeight) % frameHeight;            
            for(var i = 0; i < heightCount; i++) {
                bitmap.blt(skin, p+0,   0+m, m, p-m*2, 0,   m+frameHeight*i, m, frameHeight);   // left frame
                bitmap.blt(skin, p+q-m, 0+m, m, p-m*2, w-m, m+frameHeight*i, m, frameHeight);   // right frame
            }
            if(remainder != 0) {
                bitmap.blt(skin, p+0,   0+m, m, remainder, 0,   m+frameHeight*heightCount, m, remainder);   // left frame
                bitmap.blt(skin, p+q-m, 0+m, m, remainder, w-m, m+frameHeight*heightCount, m, remainder);   // right frame
            }

            var frameWidth = 48;
            var widthCount = Math.floor((w-frameWidth) / frameWidth);
            remainder = (w-frameWidth) % frameWidth;
            for(var j = 0; j < widthCount; j++) {
                bitmap.blt(skin, p+m,   0+0,   p-m*2, m, m+frameWidth*j, 0,   frameWidth, m); // top frame
                bitmap.blt(skin, p+m,   0+q-m, p-m*2, m, m+frameWidth*j, h-m, frameWidth, m); // bottom frame
            }
            if(remainder != 0) {
                bitmap.blt(skin, p+m,   0+0,   remainder, m, m+frameWidth*widthCount, 0,   remainder, m); // top frame                
                bitmap.blt(skin, p+m,   0+q-m, remainder, m, m+frameWidth*widthCount, h-m, remainder, m); // bottom frame
            }


            // corner
            var r = 48;
            var s = 48;
            bitmap.blt(skin, 0,   0+q*4, r, s, 0,   0,   r, s); // left top
            bitmap.blt(skin, r,   0+q*4, r, s, w-r, 0,   r, s); // right top
            bitmap.blt(skin, r*2, 0+q*4, r, s, 0,   h-r, r, s); // left bottom
            bitmap.blt(skin, r*3, 0+q*4, r, s, w-r, h-r, r, s); // right bottom

        }
    };

    Window.prototype._refreshBack = function() {
        var m = this._margin + 6;
        var w = this._width - m * 2;
        var h = this._height - m * 2;
        var bitmap = new Bitmap(w, h);

        this._windowBackSprite.bitmap = bitmap;
        this._windowBackSprite.setFrame(0, 0, w, h);
        this._windowBackSprite.move(m, m);

        if (w > 0 && h > 0 && this._windowskin) {            
            var p = 192;
            for (var y = 0; y < h; y += p) {
                for (var x = 0; x < w; x += p) {
                    bitmap.blt(this._windowskin, 0, p, p, p, x, y, p, p);
                }
            }
        }
    };
     
    Window.prototype._refreshCursor = function() {
        var pad = this._padding;
        var x = this._cursorRect.x + pad - this.origin.x;
        var y = this._cursorRect.y + pad - this.origin.y;
        var w = this._cursorRect.width;
        var h = this._cursorRect.height;
        var m = 8;
        var x2 = Math.max(x, pad);
        var y2 = Math.max(y, pad);
        var ox = x - x2;
        var oy = y - y2;
        var w2 = Math.min(w, this._width - pad - x2);
        var h2 = Math.min(h, this._height - pad - y2);
        var bitmap = new Bitmap(w2, h2);

        this._windowCursorSprite.bitmap = bitmap;
        this._windowCursorSprite.setFrame(0, 0, w2, h2);
        this._windowCursorSprite.move(x2, y2);

        if (w > 0 && h > 0 && this._windowskin) {
            var skin = this._windowskin;
            var p = 96;
            var q = 48;
            bitmap.blt(skin, p+m, p+m, q-m*2, q-m*2, ox+m, oy+m, w-m*2, h-m*2);
            bitmap.blt(skin, p+m, p+0, q-m*2, m, ox+m, oy+0, w-m*2, m);
            bitmap.blt(skin, p+m, p+q-m, q-m*2, m, ox+m, oy+h-m, w-m*2, m);
            bitmap.blt(skin, p+0, p+m, m, q-m*2, ox+0, oy+m, m, h-m*2);
            bitmap.blt(skin, p+q-m, p+m, m, q-m*2, ox+w-m, oy+m, m, h-m*2);
            bitmap.blt(skin, p+0, p+0, m, m, ox+0, oy+0, m, m);
            bitmap.blt(skin, p+q-m, p+0, m, m, ox+w-m, oy+0, m, m);
            bitmap.blt(skin, p+0, p+q-m, m, m, ox+0, oy+h-m, m, m);
            bitmap.blt(skin, p+q-m, p+q-m, m, m, ox+w-m, oy+h-m, m, m);
        }
    };

})();
