//=============================================================================
// KNH_WindowScrollRefresh.js
//=============================================================================

/*:
 * @plugindesc スクロールのリフレッシュを軽量化したい
 * @author こんにちは
 */

(function() {
    'use strict';
    var pluginName = 'KNH_WindowScrollRefresh';
    
    var getNumber = function(text, max, min) {
        text = applyCharacters(text);
        if (isNaN(text)) return NaN;
        return Math.max(Math.min(Number(text), max || Infinity), min || -Infinity);
    };
    
    var getArrayString = function(text) {
        var values = applyCharacters(text).split(',');
        for (var i = 0; i < values.length; i++) {
            values[i] = values[i].trim();
            if (!isNaN(values[i])) values[i] = Number(values[i]);
        }
        return values;
    };
    
    var applyCharacters = function(text) {
        if (text === null) text = '';
        text = '' + text;
        text = text.replace(/\\/g, '\x1b');
        text = text.replace(/\x1b\x1b/g, '\\');
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bN\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameActors.actor(parseInt(arguments[1], 10)) : null;
            return actor ? actor.name() : '';
        }.bind(this));
        text = text.replace(/\x1bP\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameParty.members()[parseInt(arguments[1], 10) - 1] : null;
            return actor ? actor.name() : '';
        }.bind(this));
        if ($dataSystem) text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
        return text;
    };
    
    var _Window_Selectable_initialize = Window_Selectable.prototype.initialize;
    Window_Selectable.prototype.initialize = function() {
        _Window_Selectable_initialize.apply(this, arguments);
        this.createContentsBack();
        this._reserveDrawItems = [];
    }

    Window_Selectable.prototype.createContents = function() {
        var w = this.contentsWidth();
        var h = this._contentsHeight || (this.itemHeight() * this.maxRows());
        this.contents = new Bitmap(w, h);
        this.resetFontSettings();
        this._refreshScrollBar();
    };
    Window_Selectable.prototype.createContentsBack = function() {
        var w = this.contentsWidth();
        var h = this._contentsHeight || (this.itemHeight() * this.maxRows());
        if (this._contentsBack) this.removeChild(this._contentsBack);
        this._contentsBack = new Sprite(new Bitmap(w, h));
        this.addChildAt(this._contentsBack, this.getChildIndex(this._windowCursorSprite));
    };
    
    var _Window_Selectable_updateContents = Window_Selectable.prototype._updateContents;
    Window_Selectable.prototype._updateContents = function() {
        _Window_Selectable_updateContents.apply(this, arguments);
        var w = this._width - this._padding * 2;
        var h = this._height - this._padding * 2;
        if (w > 0 && h > 0) {
            var y = this._scrollY + this.origin.y;
            this._windowContentsSprite.setFrame(this.origin.x, y, w, h);
            if (this._contentsBack) {
                this._contentsBack.x = this._contentsBack.y = this.padding;
                this._contentsBack.setFrame(this.origin.x, y, w, h);
                this._contentsBack.opacity = this.isOpen() ? 255 : 0;
            }
        }
    };

    Window_Selectable.prototype.setTopRow = function(row) {
        var scrollY = row.clamp(0, this.maxTopRow()) * this.itemHeight();
        if (this._scrollY !== scrollY) {
            this._scrollY = scrollY;
//            this.refresh();
            this.updateCursor();
        }
    };
    
    Window_Selectable.prototype.drawAllItems = function() {
        this._drawDependScrollY = true;
        this._reserveDrawItems = [];
        for (var i = 0; i < this.maxItems(); i++) {
            if (this.shouldDrawItem(i)) this.drawItem(i);
            else this._reserveDrawItems.push(i);
        }
        this._drawDependScrollY = false;
        var centerIndex = this.topIndex();
        this._reserveDrawItems.sort(function(a, b) {
           return (Math.abs(centerIndex - a) - Math.abs(centerIndex - b)) || -1;
        })
    };
    Window_Selectable.prototype.shouldDrawItem = function(index) {
        return this.topIndex() <= index &&
            index < this.topIndex() + this.maxPageItems();
    };
    
    var _Window_Selectable_redrawItem = Window_Selectable.prototype.redrawItem;
    Window_Selectable.prototype.redrawItem = function(index) {
        this._drawDependScrollY = true;
        _Window_Selectable_redrawItem.apply(this, arguments);
        this._drawDependScrollY = false;
    };
    
    var _Window_Selectable_update = Window_Selectable.prototype.update;
    Window_Selectable.prototype.update = function() {
        _Window_Selectable_update.apply(this, arguments);
        this._rdCount = (this._rdCount || 0) + 1;
        if (this._rdCount % 2 !== 1) return;
        var centerIndex = this.topIndex();
        this._reserveDrawItems.sort(function(a, b) {
           return (Math.abs(centerIndex - a) - Math.abs(centerIndex - b)) || -1
        })
        if (this._reserveDrawItems.some(function(index, i) {
            if (this.shouldDrawItem(index)) {
                this.redrawItem(index);
                this._reserveDrawItems.splice(i, 1);
                return true;
            }
        }, this)) return;
        if (this._rdCount % 12 !== 1) return;
        if (this._reserveDrawItems.length) this.redrawItem(this._reserveDrawItems.shift());
    };
    
    //背景
    Window_Selectable.prototype.drawContentsBack = function(index) {
        this._drawDependScrollY = true;
        var rect = this.itemRect(index);
        this._contentsBack.bitmap.clearRect(index);
        if (index >= this.maxItems()) return;
        var w = rect.width - 12, h = rect.height - 12;
        var x = rect.x + (rect.width - w) / 2, y = rect.y + (rect.height - h) / 2;
        this._contentsBack.bitmap.fillRect(x, y, w, h, 'rgba(0,0,0,0.15)');
        var a = 6;
        this._contentsBack.bitmap.clearRect(x + a, y + a, w - a * 2, h - a * 2);
        this._contentsBack.bitmap.fillRect(x + a, y + a, w - a * 2, h - a * 2, 'rgba(0,0,0,0.25)');
        this._drawDependScrollY = false;
    };
    Window_Selectable.prototype.drawAllContentsBack = function() {
        for (var i = 0; i < this.maxItems(); i++) this.drawContentsBack(i);
    };
    
    var _Window_Selectable_itemRect = Window_Selectable.prototype.itemRect;
    Window_Selectable.prototype.itemRect = function(index) {
        var rect = _Window_Selectable_itemRect.apply(this, arguments);
        if (this._drawDependScrollY) rect.y += this._scrollY;
        return rect;
    };
    
    var _Window_Selectable_scrollDown = Window_Selectable.prototype.scrollDown;
    Window_Selectable.prototype.scrollDown = function() {
        _Window_Selectable_scrollDown.apply(this, arguments);
        if (this.bottomRow() + 1 >= this.maxRows() && this.origin.y > 0) this.origin.y = 0;
    };

    var _Window_Selectable_scrollUp = Window_Selectable.prototype.scrollUp;
    Window_Selectable.prototype.scrollUp = function() {
        if (this.topRow() <= 0) this.origin.y = 0;
        _Window_Selectable_scrollUp.apply(this, arguments);
    };
    
})();