//=============================================================================
// KNH_NWS_Data.js
//=============================================================================

/*:
 * @plugindesc 武器商人　データ
 * @author こんにちは
 *
 * @help 武器データ
 */

var $dataTr_m = {};
var $dataTr_w = {};

var _lgCheck = function(text) {
    return $gameSystem ? $gameSystem.lgCheck(text) : text;
};

(function() {
    'use strict';
    var pluginName = 'KNH_NWS_Data';
    
    var getNumber = function(text, max, min) {
        text = applyCharacters(text);
        if (isNaN(text)) return NaN;
        return Math.max(Math.min(Number(text), max || Infinity), min || -Infinity);
    };
    
    var applyCharacters = function(text) {
        if (text === null) text = '';
        text = '' + text;
        text = text.replace(/\\/g, '\x1b');
        text = text.replace(/\x1b\x1b/g, '\\');
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bV\[(\d+)\]/gi, function() {
            return $gameVariables.value(parseInt(arguments[1], 10));
        }.bind(this));
        text = text.replace(/\x1bN\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameActors.actor(parseInt(arguments[1], 10)) : null;
            return actor ? actor.name() : '';
        }.bind(this));
        text = text.replace(/\x1bP\[(\d+)\]/gi, function() {
            var actor = parseInt(arguments[1], 10) >= 1 ? $gameParty.members()[parseInt(arguments[1], 10) - 1] : null;
            return actor ? actor.name() : '';
        }.bind(this));
        if ($dataSystem) text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
        return text;
    };
    
    var HK = function() {
        return $gameSystem.HK();
    };
    
    var w = {};
    var _tr_w = {eng : [], kor : [], han : [], kan : []};
    var _tr_m = [];
    var _tr_s = {};
    var _tr_h = [];
    var _tr_e = [];
    
    //-----------------------------------------------------------------------------
    // DataManager
    //-----------------------------------------------------------------------------
    
    //Excelデータ読み取り＆$dataItemsに反映
    var _Scene_Boot_start = Scene_Boot.prototype.start;
    Scene_Boot.prototype.start = function() {
        this.setWeaponsData();
        this.setElse();
        _Scene_Boot_start.apply(this, arguments);
    };
    Scene_Boot.prototype.setWeaponsData = function() {
        if (!$dataItems) return;
        var _get = function(infoIndex, info, item) {
            if (_lg() <= 0) return info[infoIndex];
            return _tr_w[['', 'eng', 'kor', 'han', 'kan'][_lg()]][item.id][infoIndex];
        }
        $dataItems.forEach(function(item) {
            if (!item || !item.name || !w[item.name]) return;
            var info = w[item.name];
            item.description = _get(6, info, item) || '-';
            item.price = Number(info[5]) || 100;
            item.repeats = Number(info[3]) || 1;
            var elementId = 
                info[1].indexOf('打撃') >= 0 ? 1 : info[1].indexOf('斬撃') >= 0 ? 2 :
                info[1].indexOf('魔法') >= 0 ? 3 : item.damage.elementId;
            item.damage.elementId = elementId;
            item.hitType = 
                [1, 2].contains(elementId) ? 1 : [3].contains(elementId) ? 2 : 0;
            if (!item.meta.heal) item.damage.formula =
                ([1, 2].contains(elementId) ? 'damW(%1)' :
                 [3].contains(elementId) ? 'damM(%1)' : 'damN(%1)'
                ).format('_item.power()');
            var power;
            if (info[0] === 'en') {
                var enemy = $dataEnemies.filter(function(en) {
                    return en && en.name === item.name;
                })[0];
                power = enemy ? enemy.params[2] : 0;
            } else {
                power = Number(info[0] || 1);
            }
            item.meta.power = power > 0 ? power : item.meta.power;
            item.meta.useable = Number(info[4]) || 3;
            item.meta.message1 = _get(7, info, item);
            item.meta.message2 = _get(8, info, item);
            item.meta.message3 = _get(9, info, item);
            item.meta.message4 = _get(10, info, item);
            item.trueName = _get(11, info, item);
            item.subName = _get(12, info, item);
            var ranges = [];
            ranges[2] = '直線２';
            ranges[3] = '直線３';
            ranges[4] = '直線４';
            ranges[6] = '前３';
            ranges[7] = '前９';
            ranges[22] = '前６';
            item.meta.rangeType = Math.max(ranges.indexOf(info[2]), 1);
            item.meta.reading = info[13];
            item.meta.effectTexts = _get(14, info, item).split(',');
            item.meta.hintText = _get(15, info, item);
            item.meta.rangeText = _get(16, info, item) || 
                ['前１マス', '1 squares in front', '1 매스 앞으로', '1個方格正面', '1个方格正面'][_lg()];
            item.meta.alive = !!info[17];
        })
    };
    Scene_Boot.prototype.setElse = function() {
    };
    
    var _Scene_Map_onMapLoaded = Scene_Map.prototype.onMapLoaded;
    Scene_Map.prototype.onMapLoaded = function() {
        var width = $dataMap.width;
        var height = $dataMap.height;
        var mapId = this._transfer ? $gamePlayer.newMapId() : $gameMap.mapId();
        
        $gameSystem._respawnReserve = [];
        for (var x = 0; x < width; x++) for (var y = 0; y < height; y++) {
            var infos = [[0, 17, 80], [0, 1587, 83], [0, 1591, 84], [0, 1585, 85], [0, 1589, 86],
                         [0, 1578, 87], [0, 1586, 88], [0, 1576, 89],
                         [1, 31, 81], [1, 39, 82], [1, 47, 98],
                         [3, 88, 2], [3, 89, 3], [3, 91, 4], [3, 96, 5], [3, 97, 6]];
            infos.sort(function(a, b) {
                return b[0] - a[0];
            })
            infos.forEach(function(info) {
                var dataId = (info[0] * height + y) * width + x;
                var tileId = $dataMap.data[dataId];
                if (tileId >= 2048) tileId = Math.floor((tileId - 2048) / 48);
                if (tileId === info[1]) {
                    var event = JSON.parse(JSON.stringify($dataTemplateEvents[info[2]]));
                    event.x = x, event.y = y;
                    $dataMap.events.push(event);
                    event.id = $dataMap.events.indexOf(event);
                    if (info[0] === 0 || info[0] === 1) {
                        var code = [mapId, event.x, event.y, event.meta.weapon];
                        if ($gameSelfSwitches.value(code)) {
                            $gameSystem.setAutoTileId(x, y, info[0]);
                        }
                    } else if (info[0] === 3) {
                        $dataMap.data[dataId] = 0;
                    }
                }
            }, this)
            
            var dataId = (0 * height + y) * width + x;
            var tile = $dataMap.data[dataId];
            var kind = Tilemap.isAutotile(tile) ? Math.floor((tile - 2048) / 48) : tile;
            //海
            if ($gameSelfSwitches.value([6, 18, 15, "海"])) {
                if (kind === 4) $dataMap.data[dataId] = tile + 96;
            }
            //魔王城内装
            if (mapId === 12 && $gameVariables.value(15) > 5) {
                var type = Math.floor(($gameVariables.value(15) - 5) / 5);
                if (kind === 119 || kind === 127) $dataMap.data[dataId] = tile + [-1, -17, -2][type] * 48;
            }
        }
        _Scene_Map_onMapLoaded.apply(this, arguments);
    };
    
    //-----------------------------------------------------------------------------
    // 翻訳用
    //-----------------------------------------------------------------------------
    
    //タイトル
    Game_System.prototype.title = function() {
        return ['ぶきあつめ ～なんでも武器になるRPG～', 'The World is Your Weapon', '무기 모으기 ~뭐든지 무기가 되는 RPG~',
                '蒐集武器～萬物皆武器的RPG～', '搜集武器～万物皆武器的RPG～'][_lg()];
    };
    
    //「うぇ子」の翻訳
    Game_Actor.prototype.name = function() {
        return ['うぇ子', 'Weaco', '웨코', '武子', '武子'][_lg()];
        return this._name;
    };
    
    //言語変更
    Game_System.prototype.setLanguage = function() {
        if (_lg() < 0) return;
        var lg = ['jpn', 'eng', 'kor', 'han', 'kan'][_lg()];
        $dataTr_m = {}, $dataTr_w = {};
        if (lg !== 'jpn') {
            //シナリオ文章
            _tr_m.forEach(function(_m) {
                if (_m && _m[0]) $dataTr_m[_m[0].replace(/\\/g, 'K=N=H===1')] = '' + _m[_lg()];
            });
            //武器文章
            _tr_w[lg].forEach(function(_w, id) {
                if (_w && _w[0]) $dataTr_w[id] = [].concat(_w);
            });
        }
        NWS()._displayLog = [];
        Scene_Boot.prototype.setWeaponsData.call(this);
        if (NWS()._sisterList) $gameSystem.setSisterName(), $gameSystem.setSisterValue();
        
        var scene = SceneManager._scene;
        scene.removeChild(SceneManager._savedWindow), SceneManager._savedWindow = null;
    };
    var _Game_Variables_setValue = Game_Variables.prototype.setValue;
    Game_Variables.prototype.setValue = function(variableId, value) {
        _Game_Variables_setValue.apply(this, arguments);
        if (variableId === 35) $gameSystem.setLanguage();
    };
    
    //elseに含まれる配列で翻訳チェック
    Game_System.prototype.lgCheck = function(text) {
        if (!_lg()) return text;
        var data = _tr_e.filter(function(el) {
            return el && el[0] === text;
        })[0];
        return data ? data[_lg()] : text;
    };
    
    //文章を自動で翻訳文章に変換
    var _Game_Interpreter_command101 = Game_Interpreter.prototype.command101;
    Game_Interpreter.prototype.command101 = function() {
        if (!$gameMessage.isBusy()) {
            if (_lg() > 0) {
                $gameMessage.setFaceImage(this._params[0], this._params[1]);
                $gameMessage.setBackground(this._params[2]);
                $gameMessage.setPositionType(this._params[3]);
                for (var text = '', i = this._index + 1; this._list[i] && this._list[i].code === 401; i++) {
                    text += (text ? '__n' : '') + this._list[i].parameters[0];
                }
                var text2 = text.replace(/__n/g, ' ').replace(/\\/g, 'K=N=H===1');
                var message = $dataTr_m[text2] ||
                    $dataTr_m[text] || text2;
                if (message) {
                    if ([1, 2].contains(_lg())) message = '<wrap>' + message;
                    else message = '<wrap><breakword>' + message;
                    message = message.replace(/K=N=H===1/g, '\\');
                    $gameMessage.add(message);
                    this._index = i - 1;
                }
                switch (this.nextEventCode()) {
                case 102:  // Show Choices
                    this._index++;
                    this.setupChoices(this.currentCommand().parameters);
                    break;
                case 103:  // Input Number
                    this._index++;
                    this.setupNumInput(this.currentCommand().parameters);
                    break;
                case 104:  // Select Item
                    this._index++;
                    this.setupItemChoice(this.currentCommand().parameters);
                    break;
                }
                this._index++;
                this.setWaitMode('message');
                return false;
            }
        }
        return _Game_Interpreter_command101.apply(this, arguments);
    };
    
    //選択肢も自動で翻訳
    var _Game_Message_choices = Game_Message.prototype.choices;
    Game_Message.prototype.choices = function() {
        var list = _Game_Message_choices.apply(this, arguments);
        list = list.map(function(text) {
            if (!_lg()) return text;
            text = text.replace(/\\/g, 'K=N=H===1');
            var text2 = $dataTr_m['>' + text], text3 = $dataTr_m[text]
            return (text2 ? text2.slice(1) : text3 ? text3 : text).replace(/K=N=H===1/g, '\\');
        });
        return list;
    };
    
    //データベース
    var _TextManager_basic = TextManager.basic;
    TextManager.basic = function(basicId) {
        if (_lg() > 0) return _tr_s['' + basicId][_lg()] || '';
        return _TextManager_basic.apply(this, arguments);
    };
    var _TextManager_param  = TextManager.param ;
    TextManager.param = function(paramId) {
        if (_lg() > 0) return _tr_s['' + (paramId + 10)][_lg()] || '';
        return _TextManager_param.apply(this, arguments);
    };
    var _TextManager_command = TextManager.command;
    TextManager.command = function(commandId) {
        if (_lg() > 0) return _tr_s['' + (commandId + 20)][_lg()] || '';
        return _TextManager_command.apply(this, arguments);
    };
    var _TextManager_message = TextManager.message;
    TextManager.message = function(messageId) {
        if (_lg() > 0) return _tr_s['' + messageId][_lg()] || '';
        return _TextManager_message.apply(this, arguments);
    };
    Object.defineProperty(TextManager, 'currencyUnit', {
        get: function() {
            if (!_lg()) return $dataSystem.currencyUnit;
            return 'Ｇ';
        },
        configurable: true
    });
    
    //ヘルプ
    Game_Interpreter.prototype.getHelpText = function(id) {
        return _tr_h[id][_lg()] || '';
    };
    
    //マップ名
    Game_Map.prototype.displayName = function(name) {
        var value = name || ($dataMap ? $dataMap.displayName : '');
        if (!_lg() || _lg() < 0) return value;
        var list = [
            ['ヘイワ村', 'フィールド', '鋼鉄の洞窟', '弱者の洞窟', '幻術の洞窟', '霊魂の洞窟', '伝説の武器が眠る洞窟', '決戦の地', '魔王城', '魔王城　中間地点', '魔王城　最上階', '生き延びろ！サバイバル島'],
            ['Peaceville', 'Field', 'Cave of Iron', 'Cave of the Weak', 'Cave of Illusion', 'Cave of Lost Souls', 'Legendary Weapon Cave', 'Decisive battle with God', 'Demonic Castle', 'Demonic Castle Waypoint', 'Demonic Castle Top floor', 'Try to Survive! Survival Island'],
            ['평화 마을', '팔드', '강철의 동굴', '약자의 동굴', '환술의 동굴', '영혼의 동굴', '전설의 무기가 잠든 동굴', '하나님과의 결전', '마왕성', '마왕성 중간 지점', '마왕성 최상층', '살아남아라! 서바이벌 아일랜드'],
            ['和平村', '地圖', '鋼鐵洞窟', '弱者洞窟', '幻術洞窟', '靈魂洞窟', '傳說武器沉眠的洞窟', '與上帝決戰', '魔王城', '魔王城 中間點', '魔王城 最上層', '活下去！生存島'],
            ['和平村', '地图', '钢铁洞窟', '弱者洞窟', '幻术洞窟', '灵魂洞窟', '传说武器沉眠的洞窟', '与上帝决战', '魔王城', '魔王城 中间点', '魔王城 最上層', '活下去！生存岛']
        ]
        return list[_lg()][list[0].indexOf(value)] || '';
    };
    
    //-----------------------------------------------------------------------------
    // ↓ここに貼り付ける
    //-----------------------------------------------------------------------------

    //武器データ
    (function(){

w['薬草' + ('' ? ' ' + '' : '')]=['-1','なし','','','1','80','たくさん持っておきたい冒険の必需品。\n体力も状態異常も回復しちゃう。','%1は %2 を手に持ってふってみた！','%2が%3の口の中に吸い込まれていく！','','','薬草','','やくそう','相手のＨＰを回復','たまに魔物が落とします。',''];
w['アルマの石' + ('' ? ' ' + '' : '')]=['5','打撃','','','1','1000','武器の秘める力を引き出す不思議な石。\n手に入りにくく貴重な素材だ。','%1は %2 を手に持ってふってみた！','%2の角が%3に刺さった！','','','アルマの石','','あるまのいし','','たまに魔物が落とします。',''];
w['デバッグのツボ' + ('' ? ' ' + '' : '')]=['10','斬撃','前３','3','5','200','武器の神・ブッキーが創った\nデバッグ用の唯一無二の武器。','%1は%2を振り回した！','%2は%3の頭に直撃した！','%2の発するオーラが%3を刺激した！','%2の中から炎が飛び出した！','デバッグのツボ','','でばっぐのつぼ','３回攻撃','存在しません。','前方３マス'];
w['旅人の剣' + ('' ? ' ' + '' : '')]=['10','斬撃','','','20','1000','大量生産が可能な銅製の剣。\n誰でも扱いやすい反面威力は心許ない。','%1は%2を振った！','','','','旅人の剣','','たびびとのけん','','あなたの家の宝箱の中に眠っています。',''];
w['業火の剣' + ('' ? ' ' + '' : '')]=['24','斬撃 / 炎','','','20','2100','炎の魔力を封じた魔法の剣。\nひとたび振るえば辺りは炎に包まれる。','%1は%2を振った！','','','なんと剣先から炎が燃え盛った！','業火の剣','','ごうかのけん','炎が発生','南の地の宝箱の中に眠っています。',''];
w['エクスカリバー' + ('' ? ' ' + '' : '')]=['28','斬撃 / 雷','','','25','7500','世界に名の知れ渡る名剣。\n雷鎚を呼びすべてを斬り裂く。','%1は%2を振った！','','','なんと剣先から雷が走った！','エクスカリバー','','えくすかりばー','雷が発生','宝箱の中に眠っています。','',''];
w['ツヨスギテ＝クサハエル' + ('' ? ' ' + '' : '')]=['56','斬撃','','','999','19999','武器の神が創った黄金の剣。\n万物を斬り裂く最強の武器である。','%1は%2を振るった！','','','','ツヨスギテ＝クサハエル','','つよすぎてくさはえる','','すみませんがこの私すらわかりません・・・。','',''];
w['ビッグブレード' + ('' ? ' ' + '' : '')]=['18','斬撃','直線２','','20','1700','巨大で鋭い刃を持つ鉄製の大剣。\n意外と軽いためブンブン振り回せる。','%1は%2を振り回した！','','','','ビッグブレード','','びっぐぶれーど','','北の地の宝箱の中に眠っています。','直線２マス',''];
w['巨人の大剣' + ('' ? ' ' + '' : '')]=['29','斬撃','直線２','','25','3600','古代の巨人族が愛用していた大剣。\nひとたび振るえば大陸をも斬り裂く。','%1は%2を振り回した！','','','','巨人の大剣','','きょじんのたいけん','','どこかのコンビニで売っています。','直線２マス',''];
w['アサシンダガー' + ('' ? ' ' + '' : '')]=['14','斬撃','','','20','1200','暗殺用に作られた短剣。\n急所をつくには高度な技術が必要。','%1は%2を突き出した！','','','','アサシンダガー','','あさしんだがー','即死（10％）','村のどこかの家の宝箱の中に眠っています。','',''];
w['悪魔のナイフ' + ('' ? ' ' + '' : '')]=['19','斬撃 / 闇','','2','25','4666','悪魔の呪いを受けた短剣。\n敏速な動きで瞬時に敵を斬り刻む。','%1は%2を突き出した！','','','','悪魔のナイフ','','あくまのないふ','２回攻撃,毒（30％）','北東の海沿いの宝箱の中に眠っています。','',''];
w['クラッシュハンマー' + ('' ? ' ' + '' : '')]=['21','打撃','','','20','1500','物を壊すことだけを目的に作られた槌。\nその破壊力は侮れない。','%1は%2を振り回した！','','','','クラッシュハンマー','','くらっしゅはんまー','混乱（20％）','村の北東の宝箱の中に眠っています。','',''];
w['モーニングスター' + ('' ? ' ' + '' : '')]=['28','打撃','','','25','8000','星のような柄頭を持つメイス。\nマッサージに使うのはやめておこう。','%1は%2を振り回した！','','','','モーニングスター','','もーにんぐすたー','混乱（20％）','東の地の物売りが売っています。','',''];
w['トライデント' + ('' ? ' ' + '' : '')]=['16','斬撃 / 氷','直線３','','20','1800','海神の武器として広く知られる槍。\n三叉の矛があらゆる物を貫く。','%1は%2で貫いた！','','','','トライデント','','とらいでんと','','毒沼で覆われた洞窟の宝箱の中に眠っています。','直線３マス',''];
w['地獄の魔槍' + ('' ? ' ' + '' : '')]=['26','斬撃 / 炎','直線３','','25','7600','地獄からやってきた槍。\n獄炎を放ちすべてを焼き貫く。','%1は%2で貫いた！','','','','地獄の魔槍','','じごくのまそう','','東の地の物売りが売っています。','直線３マス',''];
w['鋼鉄の斧' + ('' ? ' ' + '' : '')]=['19','斬撃','','','20','2000','重く鋭い刃を持つ鋼鉄製の斧。\n鎧の上からでも強力な傷を負わせる。','%1は%2を振り回した！','','','','鋼鉄の斧','','こうてつのおの','防御力を下げる（30％）','北東の地の宝箱の中に眠っています。','',''];
w['ギガントアックス' + ('' ? ' ' + '' : '')]=['32','斬撃','','','25','9500','斬れない物はない天下無双の斧。\nかつて鬼神の王が扱ったと言われる。','%1は%2を振り回した！','','','','ギガントアックス','','ぎがんとあっくす','防御力を下げる（30％）','どこかの小屋に住む男が持っています。','',''];
w['魔道士の杖' + ('' ? ' ' + '' : '')]=['8','打撃 / 炎','','','20','2100','古来から魔道士が愛用する魔法の杖。\n炎の玉を飛ばし焼き尽くす。','%1は%2を振った！','','','杖の先から炎が飛び出した！','魔道士の杖','','まどうしのつえ','【火炎の魔法】が発生','鉄でできた洞窟の宝箱の中に眠っています。','',''];
w['雷鎚の杖' + ('' ? ' ' + '' : '')]=['8','打撃 / 雷','','','20','2500','雷神の力を宿した魔法の杖。\n振るえば天から雷鎚が舞い降りる。','%1は%2を振った！','','','杖の先から雷が放たれた！','雷鎚の杖','','いかづちのつえ','【雷の魔法】が発生','村の住人の願いを叶えるともらえます。','',''];
w['氷魔女の杖' + ('' ? ' ' + '' : '')]=['8','打撃 / 氷','','','20','2600','伝説の氷魔女が持つと言われる杖。\n吹雪を呼び世界を凍らせる。','%1は%2を振った！','','','杖の先から氷が出現した！','氷魔女の杖','','こおりまじょのつえ','【氷の魔法】が発生','魔力に満ちた洞窟の宝箱の中に眠っています。','',''];
w['大悪魔の杖' + ('' ? ' ' + '' : '')]=['18','打撃 / 闇','','','25','6666','地獄に住まう大悪魔の扱う杖。\n闇の力を呼び起こす。','%1は%2を振った！','','','杖の先から闇の力が放たれた！','大悪魔の杖','','だいあくまのつえ','【闇の魔法・極】が発生','魔族の王の住む城の宝箱の中に眠っています。','',''];
w['大賢者の杖' + ('' ? ' ' + '' : '')]=['18','打撃 / 光','','','25','9999','かつて大賢者が闇を封じるために\n用いた杖。聖なる力が封じられている。','%1は%2を振った！','','','杖の先から聖なる力が放たれた！','大賢者の杖','','だいけんじゃのつえ','ランダムで【極】攻撃魔法が発生','どこかに住む魔女が持っています。','',''];
w['ピッケル' + ('' ? ' ' + '' : '')]=['20','斬撃','','','15','1500','壁を掘るための道具。\n十分武器として扱える。','%1は%2を振った！','','','','ピッケル','','ぴっける','防御力を下げる（50％）','南西の地に落ちてます。','',''];
w['スコップ' + ('' ? ' ' + '' : '')]=['17','斬撃','','','15','1300','穴を掘るための道具。\n武器として扱うことも可能。','%1は%2を振った！','%2は%3に直撃した！','ついでに%3の足元に穴を掘った！','','スコップ','','すこっぷ','１ターン行動不能（30％）','南の地に落ちてます。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['木' + ('' ? ' ' + '' : '')]=['7','打撃','前３','','3','300','そこら辺に生息していた木。\n無闇な伐採による森林破壊が問題だ。','%1は%2を振り回した！','%2は%3の頭に直撃した！','','','木','','き','','そこらへんにたくさん生えてます。','前方３マス',''];
w['木×４' + ('' ? ' ' + '' : '')]=['6','打撃','前６','4','3','1000','木を４本まとめて拾った物。\n通常より４倍のお得感がある。','%1は木を４本同時に振り回した！','木は%3の頭に直撃した！','','','木×４','','きかけるよん','４回攻撃','そこらへんにたくさん生えてます。','前方６マス',''];
w['小さい枯れ木' + ('' ? ' ' + '' : '')]=['3','打撃','','','3','50','そこら辺に生えていた小さめの枯れ木。\n命ある物はいつか枯れる運命だ。','%1は%2を振ってみた！','%2はガサッと音を立て%3に当たった！','','','小さい枯れ木','','ちいさいかれき','ＨＰを必ず１残す','南東の地にたくさん生えてます。','',''];
w['枯れ木' + ('' ? ' ' + '' : '')]=['5','打撃','直線２','','3','200','儚くも枯れてしまった木。\nどこか物悲しさを感じる。','%1は%2を振ってみた！','%2はガサッと音を立て%3に当たった！','','','枯れ木','','かれき','ＨＰを必ず１残す','南東の地にたくさん生えてます。','直線２マス',''];
w['切り株' + ('' ? ' ' + '' : '')]=['7','打撃','','','5','200','イスとして丁度よい切り株。\n見ているだけで座りたくなってくる。','%1は%2を手に持って振ってみた！','%2は%3の頭に直撃した！','%3は%2に座って一休みした！','','切り株','','きりかぶ','１ターン行動不能（75％）','村のどこかに生えてます。','',''];
w['倒れた木' + ('' ? ' ' + '' : '')]=['4','打撃','','','2','100','倒れ朽ちてしまった木。\n今や森のオブジェとして佇んでいる。','%1は%2を振り回した！','%2は%3の頭に直撃した！','%3はバランスを崩し倒れてしまった！','','倒れた木','','たおれたき','１ターン行動不能（90％）','村のどこかに落ちてます。','',''];
w['ヤシの木' + ('' ? ' ' + '' : '')]=['10','打撃','直線２','','4','700','南国に生息する木。\n見ていると南国気分になってくる。','%1は%2を陽気に振り回した！','%2は%3の頭に直撃した！','','','ヤシの木','','やしのき','','南の海沿いに生えてます。','直線２マス',''];
w['草Ａ' + ('' ? ' ' + '' : '')]=['4','斬撃','','','3','100','色も形もオーソドックスな草。\n他にもＢとＣが存在する。','%1は%2を手に持って振ってみた！','%3は%2で左手の人差し指を切ってしまった！','','','草Ａ','','くさA','','そこらへんにたくさん生えてます。','',''];
w['草Ｂ' + ('' ? ' ' + '' : '')]=['5','斬撃','','','3','100','草Ａと比較すると一回り大きい草。\n大きいのでややお得感がある。','%1は%2を手に持って振ってみた！','%3は%2で左手の薬指を切ってしまった！','','','草Ｂ','','くさB','','そこらへんにたくさん生えてます。','',''];
w['草Ｃ' + ('' ? ' ' + '' : '')]=['4','斬撃','','3','3','150','草Ａ・Ｂより更に大きい草。\n草の中では長男的存在である。','%1は%2を手に持って振ってみた！','%3は%2で右手の親指、人差し指、薬指を切ってしまった！','','','草Ｃ','','くさC','３回攻撃','そこらへんにたくさん生えてます。','',''];
w['枯れ草' + ('' ? ' ' + '' : '')]=['2','斬撃','','','2','50','枯れて生涯を終えてしまった草。\n武器として使うには心許ない。','%1は%2を手に持って振ってみた！','%2はカサッと音を立て%3に当たった！','','','枯れ草','','かれくさ','ＨＰを必ず１残す','南東の地にたくさん生えてます。','',''];
w['白い花' + ('' ? ' ' + '' : '')]=['8','斬撃','','','3','200','白く可憐に咲いた花。\n見ていると心が洗われるようだ。','%1は%2を手に持って振ってみた！','%3は花びらで左手の小指を切ってしまった！','%3は%2に癒やされ戦意を失った！','','白い花','','しろいはな','攻撃力を下げる（50％）','そこらへんにたくさん生えてます。','',''];
w['赤い花' + ('' ? ' ' + '' : '')]=['9','斬撃','','','3','250','赤く情熱的な花。\nその可愛らしさに戦意も喪失する。','%1は%2を手に持って振ってみた！','%3は花びらで右手の中指を切ってしまった！','%3は%2に癒やされ戦意を失った！','','赤い花','','あかいはな','攻撃力を下げる（40％）','そこらへんにたくさん生えてます。','',''];
w['きれいな花' + ('' ? ' ' + '' : '')]=['6','斬撃','','','3','400','種類はわからないがきれいな花。\n眺めているだけで血圧が下がるぞ。','%1は%2を手に持って振ってみた！','%3は花びらで左手の中指を切ってしまった！','%3は%2に癒やされ戦意を失った！','','きれいな花','','きれいなはな','攻撃力を下げる（75％）','村や森に生えてます。','',''];
w['くさい花' + ('' ? ' ' + '' : '')]=['23','打撃','','','5','900','おぞましい異臭を放つ花。\nできれば拾いたくなかった。','%1は鼻をつまみながら%2を振った！','%2は%3の鼻の穴付近に直撃した！','%3はあまりの臭さに気分が悪そうだ！','','くさい花','','くさいはな','毒（100％）','西の森に生えてます。','',''];
w['草むら' + ('' ? ' ' + '' : '')]=['7','打撃','前６','3','4','900','いかにも飛び出してきそうな草むら。\nこれ以上の言及は避けておこう。','%1は%2を振り回した！','%2から無数の野生のポ◯◯ンが飛び出し%3を襲う！','','','草むら','','くさむら','３回攻撃','そこらへんにたくさん生えてます。','前方６マス',''];
w['強そうな草むら' + ('' ? ' ' + '' : '')]=['10','斬撃','前９','3','5','1800','よく見ると剣みたいな形の草むら。\n斬れ味も悪くなさそうだ。','%1は%2を振り回した！','剣みたいな形の草むらで%3を斬り刻んだ！','','','強そうな草むら','','つよそうなくさむら','３回攻撃','村から西に出ると生えてます。','前方９マス',''];
w['黄金の草むら' + ('' ? ' ' + '' : '')]=['8','打撃','前６','3','5','1500','レア物が飛び出してきそうな草むら。\n例えば色違い、おっと誰か来たようだ。','%1は%2を振り回した！','%2から無数の色違いポ◯◯ンが飛び出し%3を襲う！','','','黄金の草むら','','おうごんのくさむら','３回攻撃','村から東に出ると生えてます。','前方６マス',''];
w['しげみ' + ('' ? ' ' + '' : '')]=['11','打撃','','','4','300','きれいに切り揃えられたしげみ。\nぶつけてもあまり痛くない。','%1は%2を手に持って振ってみた！','%2はファサと音を立て%3に当たった！','','','しげみ','','しげみ','','民家の近くに飾ってあります。','',''];
w['花のしげみ' + ('' ? ' ' + '' : '')]=['12','打撃','','','4','600','花が美しく咲いたオシャレなしげみ。\n置くだけで街が美しくなる。','%1は%2を手に持って振ってみた！','%2はファサと音を立て%3に当たった！','','','花のしげみ','','はなのしげみ','','あなたの家の近くに飾ってあります。','',''];
w['落ち葉' + ('' ? ' ' + '' : '')]=['4','斬撃','','3','3','200','秋を演出する落ち葉の絨毯。\nヒラヒラ舞う様はどこかロマンチック。','%1は%2を手に持って振ってみた！','たくさんの%2が%3を切り刻んだ！','','','落ち葉','','おちば','３回攻撃','村から東に出るとたくさん落ちてます。','',''];
w['ススキ' + ('' ? ' ' + '' : '')]=['7','打撃','','','4','500','黄金に輝くススキ。\n風に揺られる様は美しい。','%1は%2をワサワサと揺らした！','%3の脇腹を%2が撫でる！','%3は笑いすぎて意識が飛びそうだ！','','ススキ','','すすき','混乱（100％）','村から東に出ると生えてます。','',''];
w['キノコ' + ('' ? ' ' + '' : '')]=['6','打撃','','','3','400','こう見えて毒キノコである。\nキノコは見た目で判断してはならない。','%1は%2を手に持って振ってみた！','%2は%3の顔面にペチッと当たった！','%2の欠片が%3の口の中に入った！','','キノコ','','きのこ','毒（75％）','西の森に生えてます。','',''];
w['ヤバそうなキノコ' + ('' ? ' ' + '' : '')]=['8','打撃','','','4','300','見るからにアレなキノコ。\n猛毒が詰まっていておいしい。','%1は%2を手に持って振ってみた！','%2は%3の顔面にペチッと当たった！','%2の欠片が%3の口の中に入った！','','ヤバそうなキノコ','','やばそうなきのこ','猛毒（50％）','西の森に生えてます。','',''];
w['ツタ' + ('' ? ' ' + '' : '')]=['14','打撃','直線３','','4','700','壁を伝って生息する植物。\nムチのように使用できそうだ。','%1は%2を振り回した！','%2はムチのようにしなり%3に直撃した！','','','ツタ','','つた','','南の地のガケに生えてます。','直線３マス',''];
w['ツル' + ('' ? ' ' + '' : '')]=['12','斬撃','直線３','','4','800','何かに巻き付いて成長する植物。\n長いため遠くにも攻撃できる。','%1は%2を振り回した！','%2は鋭く飛んでいき%3に刺さった！','','','ツル','','つる','','南の地のガケに生えてます。','直線３マス',''];
w['サボテン' + ('' ? ' ' + '' : '')]=['16','斬撃','','','5','1200','砂漠地帯に生息するトゲのある植物。\nトゲは刺さると地味に痛い。','%1は痛みに耐えながら%2を振った！','%2のトゲが%3の全身に食い込んだ！','','','サボテン','','さぼてん','','南西の地に生えてます。','',''];
w['へんな草' + ('' ? ' ' + '' : '')]=['15','打撃','','','3','600','種類のわからないへんな草。\nその成分も謎に包まれている。','%1は%2を手に持って振ってみた！','%3は%2で左手の小指を切ってしまった！','%2に含まれる謎の成分が%3に効いた！','','へんな草','','へんなくさ','防御力を下げる（100％）','西の森に生えてます。','',''];
w['壁に生えた草' + ('' ? ' ' + '' : '')]=['13','斬撃','','','3','300','よく壁から生えている草。\n実は催眠成分を含んでいる。','%1は%2を手に持って振ってみた！','%3は%2で右手の甲を切ってしまった！','なんと%2には催眠成分が含まれていた！','','壁に生えた草','','かべにはえたくさ','睡眠（25％）','鉄でできた洞窟にあります。','',''];
w['怪しい芽' + ('' ? ' ' + '' : '')]=['10','打撃','','','3','200','いかにも怪しい植物の芽。\n根には不思議な力があるらしい。','%1は%2を手に持って振ってみた！','%2が%3にブスブス突き刺さった！','なんと%2は%3の能力を吸い取った！','','怪しい芽','','あやしいめ','能力変化をコピーする','毒沼で覆われた洞窟に生えてます。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['土' + ('' ? ' ' + '' : '')]=['3','打撃','','','3','50','地面に落ちている土や石ころ。\n目に入ると痛い。','%1は%2を握りしめ振ってみた！','%3は頭から%2をかぶった！','%3の目に土が入った！','','土','','つち','命中率を下げる（100％）','そこらへんに落ちてます。','',''];
w['土の山' + ('' ? ' ' + '' : '')]=['6','打撃','','','4','200','土が堆積し小さい山になったもの。\n頭からかぶると精神的に辛い。','%1は%2を振り回した！','%3は頭から%2をかぶった！','%3の目に土が入った！','','土の山','','つちのやま','命中率を下げる（50％）','南西の地にあります。','',''];
w['大きな土の山' + ('' ? ' ' + '' : '')]=['14','打撃','前６','','5','400','大量の土が堆積し山になったもの。\n塵も積もれば山となる。','%1は%2を振り回した！','%3は頭から%2をかぶった！','%3の目に土が入った！','','大きな土の山','','おおきなつちのやま','命中率を下げる（50％）','南西の地にあります。','前方６マス',''];
w['岩Ａ' + ('' ? ' ' + '' : '')]=['11','打撃','','','4','300','茶色いボディが特徴の岩Ａ。\n他にも岩Ｂと岩Ｃがある。','%1は%2を振り回した！','%2は%3に直撃した！','','','岩Ａ','','いわA','','南西の地にたくさん落ちてます。','',''];
w['岩Ｂ' + ('' ? ' ' + '' : '')]=['12','打撃','','','4','400','黒く整った形が特徴の岩Ｂ。\n尖った部分を持って振り回しやすい。','%1は%2を振り回した！','%2は%3に直撃した！','','','岩Ｂ','','いわB','','南西の地にたくさん落ちてます。','',''];
w['岩Ｃ' + ('' ? ' ' + '' : '')]=['15','打撃','','','4','500','ゴツゴツした白いボディが特徴の岩Ｃ。\nＡ～Ｃではいちばんの強度を誇る。','%1は%2を振り回した！','%2は%3に直撃した！','','','岩Ｃ','','いわC','','南西の地にたくさん落ちてます。','',''];
w['でかい岩' + ('' ? ' ' + '' : '')]=['12','打撃','直線２','','5','700','でかく堂々とそびえ立つ岩。\n見た目よりは意外と柔らかい。','%1は%2を振り回した！','%2は%3に直撃した！','','','でかい岩','','でかいいわ','','南西の地にたくさん落ちてます。','直線２マス',''];
w['緑の岩' + ('' ? ' ' + '' : '')]=['14','打撃','','','4','200','くっきりと緑色な岩。\n色以外は普通の岩と変わらない。','%1は%2を振り回した！','%2は%3に直撃した！','','','緑の岩','','みどりのいわ','','毒沼で覆われた洞窟にたくさん落ちてます。','',''];
w['でかい緑の岩' + ('' ? ' ' + '' : '')]=['14','打撃','直線２','','4','600','緑の岩のでかいバージョン。\n振り回して使う。','%1は%2を振り回した！','%2は%3に直撃した！','','','でかい緑の岩','','でかいみどりのいわ','','毒沼で覆われた洞窟にたくさん落ちてます。','直線２マス',''];
w['朽ちた岩' + ('' ? ' ' + '' : '')]=['11','打撃','','','2','100','朽ちて苔に覆われた岩。\n触れるとこちらも朽ちてきそうだ。','%1は%2を振り回した！','%2は%3に直撃した！','%3の防御も朽ちてしまった！','','朽ちた岩','','くちたいわ','防御力を下げる（50％）','北東の地にたくさん落ちてます。','',''];
w['岩のかけら' + ('' ? ' ' + '' : '')]=['7','打撃','','','2','50','岩が削れて地面に落ちたもの。\n脆く武器としては頼りない。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','','','岩のかけら','','いわのかけら','','南西の地にたくさん落ちてます。','',''];
w['クリスタル' + ('' ? ' ' + '' : '')]=['12','斬撃','','','3','700','美しい輝きを放つ神秘的なクリスタル。\n尖った部分を突き刺すと痛い。','%1は%2を振り回した！','%2の尖った部分が%3に刺さった！','','なんと%2から魔法が飛び出した！','クリスタル','','くりすたる','ランダムで攻撃魔法が発生','魔力に満ちた洞窟にたくさん落ちてます。','',''];
w['でかいクリスタル' + ('' ? ' ' + '' : '')]=['14','斬撃','直線２','','4','1200','強い輝きを放つ巨大なクリスタル。\n触れると魔力を感じる。','%1は%2を振り回した！','%2の尖った部分が%3に刺さった！','','なんと%2から魔法が飛び出した！','でかいクリスタル','','でかいくりすたる','ランダムで攻撃魔法が発生','魔力に満ちた洞窟にたくさん落ちてます。','直線２マス',''];
w['クリスタルのかけら' + ('' ? ' ' + '' : '')]=['6','斬撃','','','2','300','クリスタルが削れたもの。\n魔力は残っていないようだ。','%1は%2を手に持って振ってみた！','%2の尖った部分が%3に刺さった！','','','クリスタルのかけら','','くりすたるのかけら','','魔力に満ちた洞窟にたくさん落ちてます。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['ガケ' + ('' ? ' ' + '' : '')]=['13','打撃','前９','','3','800','天高くそびえ立つガケ。\n振り回してぶつけるとけっこう痛い。','%1は%2を振り回した！','%2は勢いよく%3に直撃した！','','','ガケ','','がけ','','村の外にたくさん落ちてます。','前方９マス',''];
w['蜘蛛の巣' + ('' ? ' ' + '' : '')]=['9','なし','','','5','200','部屋の隅によく作られる蜘蛛の巣。\n粘着性が強く、まとわりつくと厄介。','%1は嫌々%2を手に持って振ってみた。','%3の顔に%2が張り付いた！','%2は%3の足元にも張り付いた！','','蜘蛛の巣','','くものす','行動回数を下げる（60％）','魂のさまよう洞窟にあります。','',''];
w['ヒビ' + ('' ? ' ' + '' : '')]=['5','なし','','','3','400','地面に入っていたヒビ。\nぶつけた相手にもヒビが入りそうだ。','%1は%2を手に持って振ってみた！','%3に%2が当たった！','%3にヒビが入った！','','ヒビ','','ひび','防御力を下げる（100％）','南西の地に生えてます。','',''];
w['落とし穴' + ('' ? ' ' + '' : '')]=['23','打撃','','','4','600','誰かが掘った深い落とし穴。\n敵を足止めするのに便利だ。','%1は%2を手に持って振ってみた！','%3に%2が当たった！','%3は%2にまんまと落ちた！','','落とし穴','','おとしあな','２ターン行動不能（25％）','村のどこかにあります。','',''];
w['巨大な穴' + ('' ? ' ' + '' : '')]=['28','打撃','前９','','5','9500','底が見えないほど深く巨大な穴。\n落ちたら最後帰って来れない。','%1は%2を手に持って振ってみた！','%3に%2が当たった！','%3は%2に吸い込まれて行った！','','巨大な穴','','きょだいなあな','３ターン行動不能（20％）','鉄でできた洞窟にあります。','前方９マス',''];
w['小さな池' + ('' ? ' ' + '' : '')]=['14','なし','前６','','5','2000','メダカやカエルの住む小さな溜池。\n振ると氾濫が起きる。','%1は%2を手に持って振ってみた！','','','池の水が氾濫を起こした！','小さな池','','ちいさないけ','水が発生,雷耐性を下げる（100％）','そこらへんにあります。','前方６マス',''];
w['蓮の池' + ('' ? ' ' + '' : '')]=['20','なし','前６','','5','2500','蓮の葉が浮かぶ風情のある池。\n蓮の分、普通の池より威力が高い。','%1は%2を手に持って振ってみた！','','','池の水が氾濫を起こした！','蓮の池','','はすのいけ','水が発生,雷耐性を下げる（100％）','村から南に出るとあります。','前方６マス',''];
w['毒沼' + ('' ? ' ' + '' : '')]=['16','なし','前６','','4','800','猛毒の液体が溜まった危険な毒沼。\n振ると周囲は大惨事である。','%1は%2を手に持って振ってみた！','','','毒の水が氾濫を起こした！','毒沼','','どくぬま','毒の水が発生,毒（80％）','南東の地にたくさんあります。','前方６マス',''];
w['海' + ('' ? ' ' + '' : '')]=['45','なし','前９','','5','19999','地球の表面積の70%を占める、命の源。\n手に持って振ると津波が起きるぞ。','%1は%2を手に持って振ってみた！','','','なんと津波が起きた！','海','','うみ','津波が発生,雷耐性を下げる（100％）','北西の岬からよく見渡せます。','前方９マス',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['看板' + ('' ? ' ' + '' : '')]=['16','打撃','','','5','800','見た者に情報を伝える看板。\n角をぶつけるとそこそこ痛い。','%1は%2を振り回した！','%2の角の部分が%3に当たった！','','','看板','','かんばん','','村のどこかにあります。','',''];
w['道案内看板' + ('' ? ' ' + '' : '')]=['24','斬撃','','','5','1000','道を示すのに特化した看板。\nかなりわかりにくいと不評だ。','%1は%2を振り回した！','%2の尖った部分で%3を斬り裂いた！','%3はわかりにくい道案内に戸惑っている！','','道案内看板','','みちあんないかんばん','混乱（50％）','村のどこかにあります。','',''];
w['薪' + ('' ? ' ' + '' : '')]=['18','打撃','','','4','500','火を燃やすのに特化した木の棒。\n太い物で叩けばなかなか痛い。','%1は%2を振り回した！','太めの%2が%3に当たった！','%3は薪のように燃えやすい体質になった！','','薪','','まき','炎耐性を下げる（100％）','村のどこかに落ちてます。','',''];
w['井戸' + ('' ? ' ' + '' : '')]=['28','打撃','','','5','1000','おばちゃんの溜まり場になりがち。\n底に落ちると出るのは難しい。','%1は%2を振り回した！','%2の壁面が%3に当たった！','%3は井戸の底に落ちてしまった！','','井戸','','いど','１ターン行動不能（25％）','村のどこかにあります。','',''];
w['街灯' + ('' ? ' ' + '' : '')]=['17','打撃 / 炎','直線３','','4','2000','街を優しく照らし続ける街灯。\nリーチが長く武器としても優秀。','%1は%2を振り回した！','%2はピカピカ光りながら%3に直撃した！','','','街灯','','がいとう','','南の地にあります。','直線３マス',''];
w['かかし' + ('' ? ' ' + '' : '')]=['16','打撃','','','4','1200','畑からカラスを守る畑の英雄。\n常に無表情で人間が見ても怖い。','%1は%2を振り回した！','%2は無表情のまま%3に直撃した！','%3は%2に怯えている！','','かかし','','かかし','１ターン行動不能（40％）','東の地にいます。','',''];
w['アプローチ' + ('' ? ' ' + '' : '')]=['15','打撃','','','4','400','家の扉の前に置くオシャレなやつ。\n石製なので殴ると普通に痛いぞ。','%1は%2を手に持って振ってみた！','%2の比較的尖った部分が%3に当たった！','','','アプローチ','','あぷろーち','','村のどこかに落ちてます。','',''];
w['空の桶' + ('' ? ' ' + '' : '')]=['14','打撃','','','3','200','何も入っていない木製の桶。\n叩くといい音が鳴る。','%1は%2を振り回した！','%2はパコンと音を立てて%3にぶつかった！','','','空の桶','','からのおけ','','村のどこかに落ちてます。','',''];
w['水桶' + ('' ? ' ' + '' : '')]=['14','打撃','','','4','500','水が入った桶。\n水をかけるのに使えるぞ。','%1は%2を振り回した！','%2はパコンと音を立てて%3にぶつかった！','','%2から水が溢れた！','水桶','','みずおけ','水が発生,雷耐性を下げる（100％）','村のどこかに落ちてます。','',''];
w['洗濯セット' + ('' ? ' ' + '' : '')]=['12','打撃','','','4','800','衣服を洗濯途中のようだ。\n振り回せば洗剤を飛ばして攻撃できる。','%1は%2を振り回した！','%2は%3に直撃した！','','%2から洗剤が飛び散った！','洗濯セット','','せんたくせっと','洗剤が発生,氷耐性を下げる（100％）','村から南に出ると落ちてます。','',''];
w['木の板' + ('' ? ' ' + '' : '')]=['7','打撃','','','2','100','ただの木の板。使用方法は\nぶつけて使うぐらいしか思いつかない。','%1は%2を振り回した！','%2は%3に直撃した！','','','木の板','','きのいた','','南西の地に落ちてます。','',''];
w['柱' + ('' ? ' ' + '' : '')]=['13','打撃','直線２','','3','700','建築物を支え続ける縁の下の力持ち。\n振り回すのに丁度よい形状をしている。','%1は%2を振り回した！','%2は%3に直撃した！','','','柱','','はしら','','北東の地に生えてます。','直線２マス',''];
w['折れた柱' + ('' ? ' ' + '' : '')]=['9','斬撃','','','3','600','折れてしまった柱。\n折れた面はけっこうギザギザしている。','%1は%2を振り回した！','%2の折れてギザギザした面が%3に刺さった！','','','折れた柱','','おれたはしら','','北東の地に生えてます。','',''];
w['朽ちた柱' + ('' ? ' ' + '' : '')]=['13','打撃','直線２','','3','400','朽ちて苔の生えた柱。作られてから\n長い年月が経ったことがわかる。','%1は%2を振り回した！','%2は%3に直撃した！','%3の防御も朽ちてしまった！','','朽ちた柱','','くちたはしら','防御力を下げる（50％）','北東の地に生えてます。','直線２マス',''];
w['朽ちて折れた柱' + ('' ? ' ' + '' : '')]=['9','斬撃','','','2','200','朽ちた上に折れてしまった柱。\n柱としての役目を果たし尽くした。','%1は%2を振り回した！','%2の折れてギザギザした面が%3に刺さった！','%3の防御も朽ちてしまった！','','朽ちて折れた柱','','くちておれたはしら','防御力を下げる（50％）','北東の地に生えてます。','',''];
w['石碑' + ('' ? ' ' + '' : '')]=['12','打撃','','','4','2000','重要なことが書いてありそうな石碑。\nしかし文字の解読は非常に困難。','%1は%2を振り回した！','%2は%3に直撃した！','%3は石碑の文字の解読に必死だ！','','石碑','','せきひ','混乱（70％）','洞窟のあるところにあります。','',''];
w['木の墓' + ('' ? ' ' + '' : '')]=['13','斬撃','','','3','900','死者を祀るための簡易な墓。\n乱暴に扱うとバチが当たるぞ。','%1は%2を振り回した！','%2の先端が%3に突き刺さった！','%3に墓を乱暴にしたバチが当たった・・・。','','木の墓','','きのはか','即死（5％）','村のどこかに生えてます。','',''];
w['石の墓' + ('' ? ' ' + '' : '')]=['17','打撃','','','4','2400','死者を祀るための石製の墓。\n乱暴に扱うとバチが当たるぞ。','%1は%2を振り回した！','%2は激しく%3に直撃した！','%3に墓を乱暴にしたバチが当たった・・・。','','石の墓','','いしのはか','即死（10％）','魂のさまよう洞窟にあります。','',''];
w['自販機' + ('' ? ' ' + '' : '')]=['12','打撃','直線２','3','4','3200','色々な飲み物が買える便利な機械。\nなぜか「あったか～い」しかない。','%1は%2を振り回した！','あったか～い飲み物が%3の頭上に降り注いだ！','','','自販機','','じはんき','３回攻撃','ー','直線２マス',''];
w['ゴミ箱' + ('' ? ' ' + '' : '')]=['13','打撃','','','4','800','ゴミを分別するための箱。振り回して\nゴミをまき散らしたりしてはならない。','%1は%2を振り回した！','%2はゴミをまき散らしながら%3に直撃した！','%3はゴミの放つ異臭をもろに吸い込んだ！','','ゴミ箱','','ごみばこ','毒（100％）','どこかのコンビニの近くにあります。','',''];
w['石かまど' + ('' ? ' ' + '' : '')]=['14','打撃','','','5','1100','丈夫に組まれた手作り石かまど。\n着火剤を使用し燃えやすくしている。','%1は%2を振り回した！','%2は%3に直撃した！','%3に着火剤が付着した！','','石かまど','','いしかまど','炎耐性を下げる（100％）','南の地に落ちてます。','',''];
w['炎' + ('' ? ' ' + '' : '')]=['25','炎','','','4','2200','メラメラと燃える炎。\n拾う際にけっこう火傷した。','%1は火傷に耐えながら%2を振った！','%3はキャンプファイヤーのようにメラメラと燃えた！','','','炎','','ほのお','','南の地に落ちてます。','',''];
w['不思議な鉄板' + ('' ? ' ' + '' : '')]=['30','打撃','直線４','','5','8500','宙に浮かび時折回転する不思議な鉄板。\nこの回転を利用して武器にできそうだ。','%1は%2を振り回した！','%2は回転しながら%3に直撃した！','','','不思議な鉄板','','ふしぎなてっぱん','','鉄でできた洞窟にあります。','直線４マス',''];
w['不思議なスイッチ' + ('' ? ' ' + '' : '')]=['25','打撃','','','5','4000','操作するたび何かが回転した気がする\n不思議なスイッチ。意外と硬い。','%1は%2を振り回した！','%2は切り替わりながら%3に直撃した！','','','不思議なスイッチ','','ふしぎなすいっち','','鉄でできた洞窟にあります。','',''];
w['不思議な橋' + ('' ? ' ' + '' : '')]=['30','打撃','直線２','','5','3800','通過するたびに向きが変わる一方通行の\n不思議な橋。武器としても使える。','%1は%2を振り回した！','%2は向きを変えながら%3に直撃した！','','','不思議な橋','','ふしぎなはし','','毒沼で覆われた洞窟にあります。','直線２マス',''];
w['不思議なパネル' + ('' ? ' ' + '' : '')]=['28','打撃','前９','','5','6000','刺激を加えると光る不思議なパネル。\nこれを用いたパズルはけっこう難解。','%1は%2を振り回した！','%2は%3に直撃した！','%3は正解のルートがわからない！','','不思議なパネル','','ふしぎなぱねる','混乱（25％）','魔力に満ちた洞窟にあります。','前方９マス',''];
w['不思議な灯籠' + ('' ? ' ' + '' : '')]=['32','打撃 / 炎','','','5','5500','どこか神秘的な雰囲気の灯籠。\n一度ついた火は永久に灯し続ける。','%1は%2を振り回した！','%2は%3に直撃した！','','なぜか%2は自己発火した！','不思議な灯籠','','ふしぎなとうろう','炎が発生','魂のさまよう洞窟にあります。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['ツボ' + ('' ? ' ' + '' : '')]=['12','打撃','','','1','600','どこのご家庭にもあるお馴染みのツボ。\nぶつけると会心の一撃を出して壊れる。','%1は%2を振り回した！','%2は%3にぶち当たった！','','','ツボ','','つぼ','必ず会心','そこらへんの家の中にあります。','',''];
w['フタつきのツボ' + ('' ? ' ' + '' : '')]=['12','打撃','','','1','1200','フタで中身を隠された怪しいツボ。\n中には一体何が・・・？','%1は%2を振り回した！','%2は%3にぶち当たった！','ツボから飛び出した「うまのふん」が%3の口の中に入った！','','フタつきのツボ','','ふたつきのつぼ','必ず会心,毒（100％）','村のどこかの家の中にあります。','',''];
w['タル' + ('' ? ' ' + '' : '')]=['15','打撃','','','3','700','至る所に置いてあるお馴染みのタル。\n武器としては敵にぶち当てて使う。','%1は%2を振り回した！','%2は%3にぶち当たった！','','','タル','','たる','','村のどこかに落ちてます。','',''];
w['ミニテーブル' + ('' ? ' ' + '' : '')]=['11','斬撃','','','3','600','四角い小さなテーブル。\nちょっとした何かを置くのに便利。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','','','ミニテーブル','','みにてーぶる','','どこかの小屋の中にあります。','',''];
w['丸テーブル' + ('' ? ' ' + '' : '')]=['8','打撃','','','3','600','丸くてかわいい小さなテーブル。\n角がないため威力は頼りない。','%1は%2を振り回した！','%2は%3に直撃した！','','','丸テーブル','','まるてーぶる','','どこかの小屋の中にあります。','',''];
w['中ぐらいテーブル' + ('' ? ' ' + '' : '')]=['13','斬撃','直線２','','4','900','丁度よいサイズの四角いテーブル。\n角を思い切り突き刺して使う。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','','','中ぐらいテーブル','','ちゅうぐらいてーぶる','','村のどこかの家の中にあります。','直線２マス',''];
w['大きめのテーブル' + ('' ? ' ' + '' : '')]=['14','斬撃','前６','','5','2000','何人かで囲むのに便利なテーブル。\nこれを武器として扱えたら一人前。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','','','大きめのテーブル','','おおきめのてーぶる','','そこらへんの家の中にあります。','前方６マス',''];
w['やや長テーブル' + ('' ? ' ' + '' : '')]=['13','斬撃','直線３','','4','1400','微妙に長めの微妙なテーブル。\n角が刺さると微妙に痛い。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','','','やや長テーブル','','ややながてーぶる','','村のどこかの家の中にあります。','直線３マス',''];
w['長テーブル' + ('' ? ' ' + '' : '')]=['15','斬撃','直線４','','5','2500','かなりの長さを誇るテーブル。\nそのリーチは武器としても優秀。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','','','長テーブル','','ながてーぶる','','村のどこかの家の中にあります。','直線４マス',''];
w['花瓶テーブル' + ('' ? ' ' + '' : '')]=['12','斬撃','','','3','1100','花瓶とセットになったテーブル。\n見ていると心が安らぐ。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','%3は花瓶に癒やされ戦意を失った！','','花瓶テーブル','','かびんてーぶる','攻撃力を下げる（60％）','村のどこかの家の中にあります。','',''];
w['鏡テーブル' + ('' ? ' ' + '' : '')]=['15','斬撃','','','3','1200','鏡とセットになったテーブル。\nその鏡は真実のみを映す。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','鏡の反射した光が%3に当たった！','','鏡テーブル','','かがみてーぶる','能力強化をリセットする','村のどこかの家の中にあります。','',''];
w['教会テーブル' + ('' ? ' ' + '' : '')]=['17','斬撃','直線２','','4','3600','いかにも教会にありそうなテーブル。\n聖書には不思議な力があるらしい。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','%3に何らかの聖なる力が働いた！','','教会テーブル','','きょうかいてーぶる','炎耐性を下げる（50%）,氷耐性を下げる（50%）,雷耐性を下げる（50%）','どこかの教会の中にあります。','直線２マス',''];
w['水晶テーブル' + ('' ? ' ' + '' : '')]=['17','斬撃','','','3','4000','怪しい水晶とセットになったテーブル。\n水晶は怪しいが、魔力は感じる。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','','なんと水晶から魔法が飛び出した！','水晶テーブル','','すいしょうてーぶる','ランダムで攻撃魔法が発生','どこかの奇抜な家の中にあります。','',''];
w['丸イス' + ('' ? ' ' + '' : '')]=['8','打撃','','','3','300','安っぽいが座りやすい丸イス。\n武器としては期待できそうにない。','%1は%2を振り回した！','%2は%3に直撃した！','%3は%2に座って一休みした！','','丸イス','','まるいす','１ターン行動不能（50%）','村のどこかの家の中にたくさんあります。','',''];
w['四角イス' + ('' ? ' ' + '' : '')]=['10','斬撃','','','3','300','四角く小さなイス。\n角があるため丸イスより強い。','%1は%2を振り回した！','%2の角が%3に突き刺さった！','%3は%2に座って一休みした！','','四角イス','','しかくいす','１ターン行動不能（50%）','村のどこかの家の中にたくさんあります。','',''];
w['いいかんじのイス' + ('' ? ' ' + '' : '')]=['12','斬撃','直線２','','4','1500','高級そうでいいかんじのイス。\n座り心地も抜群だ。','%1は%2を振り回した！','%2の背もたれの角が%3に突き刺さった！','%3は%2に座ってゆったり一休みした！','','いいかんじのイス','','いいかんじのいす','１ターン行動不能（80%）','どこかの小屋の中にあります。','直線２マス',''];
w['本棚' + ('' ? ' ' + '' : '')]=['8','打撃','直線２','3','4','1000','色んな本が整頓された本棚。\n本を落下させ敵にぶつけて使う。','%1は%2を振り回した！','無数の本が%3の頭上に降り注いだ！','%3は色んな本を読み賢くなった！','','本棚','','ほんだな','３回攻撃,貧弱（30%）','そこらへんの家の中にあります。','直線２マス',''];
w['タンス' + ('' ? ' ' + '' : '')]=['30','斬撃','直線２','','4','1500','衣服がしまってある棚。\n左足の小指をぶつけると死ぬほど痛い。','%1は%2を振り回した！','%3の左足の小指に%2の角がぶち当たった！','%3は左足の小指を抑えて苦しそうにしている！','','タンス','','たんす','行動回数を下げる（30%）','村のどこかの家の中にあります。','直線２マス',''];
w['クローゼット' + ('' ? ' ' + '' : '')]=['9','打撃','直線２','3','4','1200','衣服をかけておく棚。\n実は中はすごいことになっている。','%1は%2を振り回した！','無数の服が%3の頭上に降り注いだ！','%3はたくさんの服で身動きがとりにくい！','','クローゼット','','くろーぜっと','３回攻撃,回避を下げる（100%）','村のどこかの家の中にあります。','直線２マス',''];
w['パンとか入った棚' + ('' ? ' ' + '' : '')]=['7','打撃','直線２','3','4','1400','なぜかパンが入っている棚。\nフランスパンは硬いので武器になる。','%1は%2を振り回した！','無数のフランスパンが%3の頭上に降り注いだ！','%3は夢中でパンを食べ始めた！','','パンとか入った棚','','ぱんとかはいったたな','３回攻撃,１ターン行動不能（50%）','どこかの小屋の中にあります。','直線２マス',''];
w['よくわからない棚' + ('' ? ' ' + '' : '')]=['8','打撃','直線２','3','4','900','よくわからない薬とかが入ってる棚。\nとりあえず振り回して使うのが吉。','%1は%2を振り回した！','無数のよくわからないやつが%3の頭上に降り注いだ！','%3はよくわからないやつに怯えている！','','よくわからない棚','','よくわからないたな','３回攻撃,混乱（50%）','どこかの小屋の中にあります。','直線２マス',''];
w['本を置いてる台' + ('' ? ' ' + '' : '')]=['12','打撃','','','3','600','壁に取り付けて本を置いておく台。\n正直この位置では誰もとれない。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','%3は本を読み賢くなった！','','本を置いてる台','','ほんをおいてるだい','貧弱（30%）','村のどこかの家の中にあります。','',''];
w['調味料台' + ('' ? ' ' + '' : '')]=['13','打撃','','','3','800','色々な調味料を置いておける便利な台。\n賞味期限切れには注意。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','なんと調味料の賞味期限は２年前だった！','','調味料台','','ちょうみりょうだい','毒（30%）','村のどこかの家の中にあります。','',''];
w['花瓶' + ('' ? ' ' + '' : '')]=['12','打撃','','','3','700','かわいい花と花瓶のセット。\n美しい花には美しい花瓶が重要だ。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','%3は花に癒やされ戦意を失った！','','花瓶','','かびん','攻撃力を下げる（50％）','どこかの教会の中にあります。','',''];
w['木箱' + ('' ? ' ' + '' : '')]=['14','打撃','','','3','400','何か適当なものを入れておける木箱。\n普通にぶつけて使える。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','','','木箱','','きばこ','','村のどこかの家の中にあります。','',''];
w['木箱タワー' + ('' ? ' ' + '' : '')]=['6','打撃','前６','5','4','1800','木箱がたくさん積み重なったタワー。\n上手く敵の頭上に雪崩れさせて使う。','%1は%2を振り回した！','無数の木箱が%3の頭上に降り注いだ！','','','木箱タワー','','きばこたわー','５回攻撃','どこかの小屋の中にあります。','前方６マス',''];
w['ボロい木箱' + ('' ? ' ' + '' : '')]=['8','打撃','','','2','300','ボロボロになってしまった木箱。\n強度も脆くあまり使えない。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','','','ボロい木箱','','ぼろいきばこ','','どこかの小屋の中にあります。','',''];
w['大きい木箱' + ('' ? ' ' + '' : '')]=['18','打撃','直線２','','3','700','身の丈ほど大きな木箱。\n振り回せばなかなか威力が出る。','%1は%2を振り回した！','%2は%3に直撃した！','','','大きい木箱','','おおきいきばこ','','村のどこかにあります。','直線２マス',''];
w['米袋' + ('' ? ' ' + '' : '')]=['12','打撃','','','3','500','パンパンに詰まった米袋。\n農家への感謝を忘れずに振り回そう。','%1は%2を手に持って振ってみた！','パンパンに詰まった%2が%3に直撃した！','','','米袋','','こめぶくろ','','村のどこかの家の中にあります。','',''];
w['米袋タワー' + ('' ? ' ' + '' : '')]=['9','打撃','直線２','3','4','1400','パンパンの米袋が積まれたタワー。\n敵の頭上で雪崩を起こして使う。','%1は%2を振り回した！','無数の米袋が%3の頭上に降り注いだ！','','','米袋タワー','','こめぶくろたわー','３回攻撃','村のどこかの家の中にあります。','直線２マス',''];
w['たくさんのリンゴ' + ('' ? ' ' + '' : '')]=['5','打撃','','5','3','1100','たくさんの美味しそうなリンゴだが、\n果たして・・・？','%1は%2を手に持って振ってみた！','%2が%3の頭上に降り注いだ！','なんとすべて毒リンゴだった！','','たくさんのリンゴ','','たくさんのりんご','５回攻撃,猛毒（60％）','どこかの奇抜な家の中にあります。','',''];
w['キッチン' + ('' ? ' ' + '' : '')]=['16','打撃','直線２','','4','1400','きれいに掃除された広めのキッチン。\n振り回せばなかなかのリーチだ。','%1は%2を振り回した！','%2は%3に直撃した！','','','キッチン','','きっちん','','村のどこかの家の中にあります。','直線２マス',''];
w['ガスコンロ台' + ('' ? ' ' + '' : '')]=['16','打撃','','','4','900','油汚れもないきれいなガスコンロ台。\n炎が出るので武器として普通に強い。','%1は%2を振り回した！','%2は%3に直撃した！','','強火でアメ色になるまで加熱しましょう！','ガスコンロ台','','がすこんろだい','炎が発生','村のどこかの家の中にあります。','',''];
w['換気扇' + ('' ? ' ' + '' : '')]=['18','斬撃','','','3','900','少し汚れた換気扇。\n羽根の回転を武器に応用できそうだ。','%1は%2を手に持って振ってみた！','%3は%2の回転に巻き込まれた！','','','換気扇','','かんきせん','','村のどこかの家の中にあります。','',''];
w['まな板セット' + ('' ? ' ' + '' : '')]=['23','斬撃','','','5','800','何かを切りかけのまな板と包丁。\n包丁はなかなかの切れ味だ。','%1は%2を振り回した！','%3をまな板に乗せみじん切りにした！','','','まな板セット','','まないたせっと','斬撃耐性を下げる（100%）','村のどこかの家の中にあります。','',''];
w['調理器具セット' + ('' ? ' ' + '' : '')]=['11','打撃','','3','5','700','フライパンやおたまなど様々ある。\nもちろん武器として使える。','%1は%2を振り回した！','%3を色んな調理器具で殴った！','','','調理器具セット','','ちょうりきぐせっと','３回攻撃','村のどこかの家の中にあります。','',''];
w['冷蔵庫' + ('' ? ' ' + '' : '')]=['20','打撃 / 氷','直線２','','4','1900','余り物でいっぱいの冷蔵庫。\n冷気で敵を凍らせる戦術が可能だ。','%1は%2を振り回した！','%2は%3に直撃した！','','扉が開いて冷気が漏れた！','冷蔵庫','','れいぞうこ','冷気が発生','村のどこかの家の中にあります。','直線２マス',''];
w['魚の丸焼き' + ('' ? ' ' + '' : '')]=['12','打撃','','','3','700','魚を頭からしっぽまで焼いた一品。\n骨がたくさん入っている。','%1は%2を振ってみた！ ※食べ物は大切に！','%2は%3に激しくぶつかった！','%3は魚の骨を喉にひっかけてしまった！','','魚の丸焼き','','さかなのまるやき','魔法を封じる（50%）','村のどこかの家の中にあります。','',''];
w['野菜炒めセット' + ('' ? ' ' + '' : '')]=['14','打撃','','','3','600','シンプルで美味しい炒めもの。\n舌が火傷しそうなぐらいアツアツだ。','%1は%2を振ってみた！ ※食べ物は大切に！','熱々の野菜が%3を襲う！','','','野菜炒めセット','','やさいいためせっと','','村のどこかの家の中にあります。','',''];
w['豚の生姜焼き定食' + ('' ? ' ' + '' : '')]=['12','打撃','','','3','750','ご飯はおかわり自由なお得な定食。\nしかし豚肉は若干赤い気がする。','%1は%2を振ってみた！ ※食べ物は大切に！','%2は%3に激しくぶつかった！','なんと%2は生焼けだった。%3は腹を下した！','','豚の生姜焼き定食','','ぶたのしょうがやきていしょく','毒（30%）','村のどこかの家の中にあります。','',''];
w['食後のコーヒー' + ('' ? ' ' + '' : '')]=['7','打撃','','','3','320','飲めば午後からも頑張れる一杯。\nカフェインで眠気もぶっ飛ぶ。','%1は%2で乾杯した！','%3「乾杯～！！」','%3の目が冴え渡った！','','食後のコーヒー','','しょくごのこーひー','睡眠耐性を上げる（100%）','村のどこかの家の中にあります。','',''];
w['高級ワイン' + ('' ? ' ' + '' : '')]=['12','打撃','','','3','3000','濃厚な味わいが特徴のフルボディ。\n香りだけで良い気分になる。','%1は%2で乾杯した！','%3「乾杯～！！」','%3はワインの香りに酔ってしまった！','','高級ワイン','','こうきゅうわいん','通常攻撃以外を封じる（50%）','村のどこかの家の中にあります。','',''];
w['料理のようなもの' + ('' ? ' ' + '' : '')]=['21','打撃','','','3','50','料理のような、そうでないような何か。\nなぜこれを客に出したのか・・・。','%1は%2を手に持って振ってみた！','%2は%3の口の中に吸い込まれた！','%3の様子が明らかにおかしい！','','料理のようなもの','','りょうりのようなもの','毒（75%）,魔法封じ（50%）','村のどこかの家の中にあります。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['ベッド' + ('' ? ' ' + '' : '')]=['12','打撃','直線２','','4','2400','寝心地抜群のふかふかベッド。\n吸い込むように敵の眠りを誘う。','%1は%2を振り回した！','%2はボフッと音を立て%3に当たった！','%3は%2に吸い込まれた！','','ベッド','','べっど','睡眠（55%）','村のどこかの家の中にあります。','直線２マス',''];
w['ダブルベッド' + ('' ? ' ' + '' : '')]=['14','打撃','前６','','5','7000','二人同時に寝られるダブルベッド。\n寝心地は最高である。','%1は%2を振り回した！','%2はボフッと音を立て%3に当たった！','%3は%2に吸い込まれた！','','ダブルベッド','','だぶるべっど','睡眠（80%）','村のどこかの家の中にあります。','前方６マス',''];
w['ボロい布団' + ('' ? ' ' + '' : '')]=['7','打撃','直線２','','3','300','質素で寝心地の悪そうな布団。\n翌朝関節が痛む。','%1は%2を振り回した！','%2はパサッと音を立て%3に当たった！','%3は今夜だけ%2で我慢することにした！','','ボロい布団','','ぼろいふとん','睡眠（25%）','どこかの小屋の中にあります。','直線２マス',''];
w['暖炉' + ('' ? ' ' + '' : '')]=['18','打撃','直線２','','5','1400','豪華な雰囲気の暖炉。\n上の細い部分を持って振り回す。','%1は%2を振り回した！','%2は%3に直撃した！','','なぜか%2は自己発火した！','暖炉','','だんろ','炎が発生','村のどこかの家の中にあります。','直線２マス',''];
w['日記' + ('' ? ' ' + '' : '')]=['11','斬撃','','','3','400','誰かが書いた日記。\n読むと自分の黒歴史が蘇ってくる。','%1は%2を手に持って振ってみた！','%3は%2で左手の親指を切ってしまった！','%3は黒歴史を思い出してしまった・・・。','','日記','','にっき','混乱（50%）','村のどこかの家の中にあります。','',''];
w['何かの紙' + ('' ? ' ' + '' : '')]=['5','斬撃','','','3','200','何かが書かれた紙。\n端で手を切ると少し痛い。','%1は%2を手に持って振ってみた！','%3は%2の端で左手の甲を切ってしまった！','','','何かの紙','','なにかのかみ','','どこかの小屋の中にあります。','',''];
w['何かの袋' + ('' ? ' ' + '' : '')]=['13','打撃','','','3','500','何かが入った袋。中身は不明。\nぶつける以外に使い道はない。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！中身が何なのかは不明だ。','','','何かの袋','','なにかのふくろ','','村のどこかの家の中にあります。','',''];
w['ピアノ' + ('' ? ' ' + '' : '')]=['27','打撃','前６','','5','3700','きれいに磨かれた年代物のピアノ。\nメロディを奏でながら敵にぶつけよう。','%1は%2を振り回した！','%2の鍵盤部分が%3にぶつかり不協和音を奏でた！','%3は不協和音で気分が悪くなった！','','ピアノ','','ぴあの','能力強化をリセットする','村のどこかの家の中にあります。','前方６マス',''];
w['プレゼント箱' + ('' ? ' ' + '' : '')]=['15','打撃','','','4','1000','かわいいラッピングが施された箱。\n中から魔力のようなものを感じる。','%1は%2を振り回した！','%2は%3に直撃した！','','%2の中から魔法が飛び出した！','プレゼント箱','','ぷれぜんとばこ','ランダムで攻撃魔法が発生','どこかの奇抜な家の中にあります。','',''];
w['魔女の服' + ('' ? ' ' + '' : '')]=['12','打撃','','','5','1500','魔女の家にあった服。\n拾う際にいささか罪悪感を感じた。','%1は%2を振ってみた！いい匂いがする！','%2は%3の顔に当たった！','','','魔女の服','','まじょのふく','','どこかの奇抜な家の中にあります。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['コンビニ商品棚' + ('' ? ' ' + '' : '')]=['11','打撃','前６','3','4','1400','コンビニの日用品などが並んだ商品棚。\n武器として拾ったので万引きではない。','%1は%2を振り回した！','色々な商品が%3の頭上に降り注いだ！','なんと商品はすべて不良品だった！','','コンビニ商品棚','','こんびにしょうひんだな','３回攻撃,混乱（50%）','どこかのコンビニの中にあります。','前方６マス',''];
w['冷凍食品コーナー' + ('' ? ' ' + '' : '')]=['10','打撃 / 氷','前６','3','4','2200','色々な冷凍食品を集めたコーナー。\nこの冷気は武器として使えそうだ。','%1は%2を振り回した！','冷凍食品が%3の頭上に降り注いだ！','','冷気が漏れ出した！','冷凍食品コーナー','','れいとうしょくひんこーなー','３回攻撃,冷気が発生','どこかのコンビニの中にあります。','前方６マス',''];
w['弁当売場' + ('' ? ' ' + '' : '')]=['12','打撃','前６','3','4','1800','色とりどりの弁当が並んだコーナー。\nすべて「半額」シールが貼ってある。','%1は%2を振り回した！','色々な弁当が%3の頭上に降り注いだ！','なんと弁当はすべて賞味期限切れだった！','','弁当売場','','べんとううりば','３回攻撃,毒（50%）','どこかのコンビニの中にあります。','前方６マス',''];
w['電話' + ('' ? ' ' + '' : '')]=['13','打撃','','','4','3000','我が家の大事な電話機。\nオレオレ詐欺には気をつけよう。','%1は%2を振り回した！','%2は%3に直撃した！','%3はオレオレ詐欺にひっかかってしまった！','','電話','','でんわ','混乱（100％）','あなたの家の中にあります。','',''];
w['ボロ絨毯' + ('' ? ' ' + '' : '')]=['9','打撃','前９','','3','400','かなりボロい絨毯。\n若干変な匂いもする。','%1は%2を振り回した！','%3は%2で足をとられ 床に顔を強打した！','%3は足をとられ動きが鈍った！','','ボロ絨毯','','ぼろじゅうたん','回避率を下げる（75％）','どこかの小屋の中にあります。','前方９マス',''];
w['青い絨毯' + ('' ? ' ' + '' : '')]=['14','打撃','前９','','4','800','青くてかわいい絨毯。\n手触りはなめらかだ。','%1は%2を振り回した！','%3は%2で足をとられ 床に顔を強打した！','%3は足をとられ動きが鈍った！','','青い絨毯','','あおいじゅうたん','回避率を下げる（50％）','村のどこかの家の中にあります。','前方９マス',''];
w['豪華な絨毯' + ('' ? ' ' + '' : '')]=['18','打撃','前９','','4','2500','真っ赤で豪華な絨毯。\n最高級の生地で作られている。','%1は%2を振り回した！','%3は%2で足をとられ 床に顔を強打した！','%3は足をとられ動きが鈍った！','','豪華な絨毯','','ごうかなじゅうたん','回避率を下げる（25％）','どこかの教会の中にあります。','前方９マス',''];
w['怪しい絨毯' + ('' ? ' ' + '' : '')]=['13','打撃','前９','','4','1100','不思議な雰囲気の絨毯。\nしかし生地は意外と安っぽい。','%1は%2を振り回した！','%3は%2で足をとられ 床に顔を強打した！','%3は足をとられ動きが鈍った！','','怪しい絨毯','','あやしいじゅうたん','回避率を下げる（100％）','どこかの奇抜な家の中にあります。','前方９マス',''];
w['時計' + ('' ? ' ' + '' : '')]=['16','打撃','','','4','800','正確に時を刻み続ける時計。\nこれで殴ってみるのも面白そうだ。','%1は%2を手に持って振ってみた！','%2は時を刻みながら%3に直撃した！','なんと%3周辺の時空が変化した！','','時計','','とけい','行動回数を下げる（50％）','村のどこかの家の中にあります。','',''];
w['壁掛けの剣' + ('' ? ' ' + '' : '')]=['21','斬撃','','','5','1200','壁に掛けられる剣のレプリカ。\nしかし普通に斬れ味は良さそう。','%1は%2を振った！','%2は普通に%3を斬り裂いた！','','','壁掛けの剣','','かべかけのけん','','村のどこかの家の中にあります。','',''];
w['壁掛けの盾' + ('' ? ' ' + '' : '')]=['17','打撃','','','5','900','壁に掛けられる盾のレプリカ。\nレプリカとはいえなかなかの硬度だ。','%1は%2を振り回した！','%2は勢いよく%3に直撃した！','','','壁掛けの盾','','かべかけのたて','防御力を下げる（50％）','村のどこかの家の中にあります。','',''];
w['壁掛けの斧' + ('' ? ' ' + '' : '')]=['24','斬撃','','','5','1500','壁に掛けられる斧のレプリカ。\n本物のようにずっしりと重い。','%1は%2を振り回した！','%2は普通に%3を斬り裂いた！','','','壁掛けの斧','','かべかけのおの','','どこかの小屋の中にあります。','',''];
w['地図' + ('' ? ' ' + '' : '')]=['16','斬撃','前３','','3','600','手書きの地図。\n冒険の必需品だ。','%1は%2を手に持って振ってみた！','%2は%3に当たった！','%3は地図を読むのが下手だった！','','地図','','ちず','混乱（75％）','どこかの小屋の中にあります。','前方３マス',''];
w['花の絵' + ('' ? ' ' + '' : '')]=['14','斬撃','','','3','700','美しい花の絵。\n額縁は硬いので武器になる。','%1は%2を手に持って振ってみた！','額縁が%3に直撃した！','%3は美しい花に癒やされ戦意を失った！','','花の絵','','はなのえ','攻撃力を下げる（60％）','どこかの奇抜な家の中にあります。','',''];
w['人の絵' + ('' ? ' ' + '' : '')]=['15','斬撃','','','3','800','知らない人の顔だ。誰だお前？\n芸術とは難しい。','%1は%2を手に持って振ってみた！','額縁が%3に直撃した！','%3は芸術とは何かを考え始めてしまった！','','人の絵','','ひとのえ','混乱（40％）','どこかの奇抜な家の中にあります。','',''];
w['赤い旗' + ('' ? ' ' + '' : '')]=['13','斬撃','直線２','','4','1000','国旗のような真っ赤な旗。\n威力は期待できないがかっこいい。','%1は%2を振った！','%2ははためきながら%3に当たった！','','','赤い旗','','あかいはた','','魂のさまよう洞窟にあります。','直線２マス',''];
w['ボロい旗' + ('' ? ' ' + '' : '')]=['7','斬撃','直線２','','3','300','ボロボロになってしまった旗。\nもはやただのゴミである。','%1は%2を振った！','%2ははためきながら%3に当たった！','','','ボロい旗','','ぼろいはた','','魂のさまよう洞窟にあります。','直線２マス',''];
w['真っ赤のカーテン' + ('' ? ' ' + '' : '')]=['18','斬撃','前６','','4','900','趣味を疑うほどに真っ赤なカーテン。\n遮光性は高そうだ。','%1は%2を手に持って振ってみた！','%2はヒラヒラと%3に当たった！','%3の視界が%2で遮られた！','','真っ赤のカーテン','','まっかのかーてん','命中率を下げる（90％）','どこかの奇抜な家の中にあります。','前方６マス',''];
w['窓' + ('' ? ' ' + '' : '')]=['12','打撃','','','3','700','きれいに拭かれた窓。\n二重構造なので意外と頑丈だ。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','','','窓','','まど','','村のどこかの家の中にあります。','',''];
w['緑カーテン窓' + ('' ? ' ' + '' : '')]=['12','打撃','','','4','900','緑色のカーテンつきの窓。カーテンで\n敵の視界を遮ることが可能。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','%3の視界がカーテンで遮られた！','','緑カーテン窓','','みどりかーてんまど','命中率を下げる（100％）','村のどこかの家の中にあります。','',''];
w['赤カーテン窓' + ('' ? ' ' + '' : '')]=['13','打撃','','','4','1000','赤いカーテンつきの少し大きめな窓。\n叩きつけても割れないぐらい丈夫。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','%3の視界がカーテンで遮られた！','','赤カーテン窓','','あかかーてんまど','命中率を下げる（100％）','村のどこかの家の中にあります。','',''];
w['ステンドグラス' + ('' ? ' ' + '' : '')]=['15','打撃','','','4','1600','きれいな模様が描かれた窓。\n教会といったらこれである。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','','','ステンドグラス','','すてんどぐらす','','どこかの教会の中にあります。','',''];
w['扉' + ('' ? ' ' + '' : '')]=['20','打撃','','','4','800','セキリュティに少々不安がある扉。\n武器としては頼もしい。','%1は%2をバタバタ振り回した！','%2は%3に直撃した！','','','扉','','とびら','','そこらへんの家の入り口にあります。','',''];
w['牢屋' + ('' ? ' ' + '' : '')]=['16','打撃','前６','','5','1200','頑丈な鉄製の牢獄。\n振り回せば十分強力な武器となる。','%1は%2を振り回した！','%2は%3に直撃した！','%3は牢屋に閉じ込められた気分になった！','','牢屋','','ろうや','１ターン行動不能（30％）','鉄でできた洞窟にあります。','前方６マス',''];
w['牢屋の扉' + ('' ? ' ' + '' : '')]=['17','打撃','直線２','','5','900','牢屋の鍵によって開く頑丈な扉。\n強度は保証されている。','%1は%2を振り回した！','%2は%3に直撃した！','%3は牢屋に閉じ込められた気分になった！','','牢屋の扉','','ろうやのとびら','１ターン行動不能（20％）','鉄でできた洞窟にあります。','直線２マス',''];
w['戦士の像' + ('' ? ' ' + '' : '')]=['28','打撃','直線２','','5','6000','勇ましい顔つきの戦士の像。\n持っている剣は武器に使えそうだ。','%1は%2を振り回した！','%2の剣の部分が%3を斬り裂いた！','','','戦士の像','','せんしのぞう','','東の地に落ちてます。','直線２マス',''];
w['右向き女神像' + ('' ? ' ' + '' : '')]=['24','打撃','直線２','','5','4500','常に右を向いている不思議な女神像。\n聖なる力でかなり頑丈にできている。','%1は%2を振り回した！','%2は%3にヘッドバッドを決めた！','%3に何らかの聖なる力が働いた！','','右向き女神像','','みぎむきめがみぞう','貧弱（15%）','どこかの教会の中にあります。','直線２マス',''];
w['左向き女神像' + ('' ? ' ' + '' : '')]=['24','打撃','直線２','','5','4500','常に左を向いている不思議な女神像。\n頭突きさせたら強そうだ。','%1は%2を振り回した！','%2は%3にヘッドバッドを決めた！','%3に何らかの聖なる力が働いた！','','左向き女神像','','ひだりむきめがみぞう','魔法を封じる（50%）','どこかの教会の中にあります。','直線２マス',''];
w['ドラゴン像' + ('' ? ' ' + '' : '')]=['40','斬撃 / 炎','前３','','5','18000','凶暴なドラゴンを象った迫力のある像。\nドラゴンの魂を封じ込めている。','%1は%2を振り回した！','%2の牙の部分が%3に突き刺さった！','','なんと%2は炎を吐いた！','ドラゴン像','','どらごんぞう','炎が発生','魂のさまよう洞窟にあります。','前方３マス',''];
w['宝箱' + ('' ? ' ' + '' : '')]=['18','打撃','','','3','3000','レアアイテムが入っていそうな宝箱。\n武器としてはぶつける他ないようだ。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','','','宝箱','','たからばこ','','そこらへんに落ちてます。','',''];
w['黄金の柱' + ('' ? ' ' + '' : '')]=['19','打撃','直線２','','4','5600','まばゆいほどに輝く黄金でできた柱。\n直視すると目に悪そうだ。','%1は%2を振り回した！','%2は輝きながら%3に直撃した！','%3は黄金の輝きを直視してしまった！','','黄金の柱','','おうごんのはしら','命中率を下げる（50％）','黄金に輝く洞窟の中にあります。','直線２マス',''];
w['黄金の折れた柱' + ('' ? ' ' + '' : '')]=['15','斬撃','','','4','4800','折れてもなお輝きを放つ黄金の柱。\n金とはいえ強度も申し分ない。','%1は%2を振り回した！','%2の折れてギザギザした面が%3に刺さった！','%3は黄金の輝きを直視してしまった！','','黄金の折れた柱','','おうごんのおれたはしら','命中率を下げる（30％）','黄金に輝く洞窟の中にあります。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['赤いクマ' + ('キャサリン' ? ' ' + 'キャサリン' : '')]=['20','打撃','','','4','500','赤くてラブリーなクマのヌイグルミ。\nヌイグルミ・・・だよな？','%1は%2を振り回した！','%2は%3にヘッドバッドを決めた！','','','赤いクマ','キャサリン','あかいくま','','村のどこかの家の中に座っています。','',''];
w['青いクマ' + ('ジェームズ' ? ' ' + 'ジェームズ' : '')]=['20','打撃','','','4','500','青くてクールなクマのヌイグルミ。\nヌイグルミ・・・だよな？','%1は%2を振り回した！','%2は%3にヘッドバッドを決めた！','','','青いクマ','ジェームズ','あおいくま','','南の岬に座っています。','',''];
w['黒いクマ' + ('トム' ? ' ' + 'トム' : '')]=['20','打撃','','','4','500','黒くてブラックなクマのヌイグルミ。\nヌイグルミ・・・だよな？','%1は%2を振り回した！','%2は%3にヘッドバッドを決めた！','','','黒いクマ','トム','くろいくま','','東の地に座っています。','',''];
w['茶色クマ' + ('ブラウン' ? ' ' + 'ブラウン' : '')]=['20','打撃','','','4','500','茶色くてキュートなクマのヌイグルミ。\nヌイグルミ・・・だよな？','%1は%2を振り回した！','%2は%3にヘッドバッドを決めた！','','','茶色クマ','ブラウン','ちゃいろいくま','','北西の地に座っています。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['自宅右' + ('' ? ' ' + '' : '')]=['30','打撃','前９','','5','2500','長年暮らしてきた我が家の右半分。\nブンブン振り回せば武器として使える。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','自宅右','','じたくみぎ','','あなたの家の右半分を形成しています。','前方９マス',''];
w['自宅左' + ('' ? ' ' + '' : '')]=['30','打撃','前９','','5','3200','我が家の左半分、武器屋スペース。\nブンブン振り回せば武器として使える。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','自宅左','','じたくひだり','','あなたの家の左半分を形成しています。','前方９マス',''];
w['民家' + ('' ? ' ' + '' : '')]=['29','打撃','前９','','5','2200','赤の他人の住んでいる家。\n武器にして怒られないか不安だ。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','民家','','みんか','','村に何個か落ちてます。','前方９マス',''];
w['木こりの家' + ('' ? ' ' + '' : '')]=['27','打撃','前９','','5','1900','木こりが住んでいるボロい家。\nボロくて汚いが武器にはなる。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','木こりの家','','きこりのいえ','','西の地に落ちてます。','前方９マス',''];
w['魔女の家' + ('' ? ' ' + '' : '')]=['25','打撃','前９','','5','2800','魔女の住んでいそうな怪しい家。\n屋根の色合いが派手だ。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','魔女の家','','まじょのいえ','','西の森の中に隠れています。','前方９マス',''];
w['教会' + ('' ? ' ' + '' : '')]=['32','打撃','前９','','5','3300','海沿いに建つ神聖な教会。\n神に感謝しつつ振り回そう。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','%3に何らかの聖なる力が働いた！','','教会','','きょうかい','能力強化をリセットする','北東の海沿いに建っています。','前方９マス',''];
w['コンビニ' + ('' ? ' ' + '' : '')]=['31','打撃','前９','','5','3000','なんでも買える便利なコンビニ。\n便利なので武器にもできるぞ。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','コンビニ','','こんびに','','北の地に建っています。','前方９マス',''];
w['テント' + ('' ? ' ' + '' : '')]=['25','打撃','前９','','5','1500','誰かが設置した簡易式のテント。\n持ち運びもできて便利な武器だ。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','テント','','てんと','','東の地に建っています。','前方９マス',''];
w['鋼鉄の洞窟' + ('' ? ' ' + '' : '')]=['33','打撃','前９','','5','4500','硬い敵ばかり住む洞窟。\n洞窟自体も硬いので武器として強い。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','鋼鉄の洞窟','','こうてつのどうくつ','','南西の地に佇んでいます。','前方９マス',''];
w['弱者の洞窟' + ('' ? ' ' + '' : '')]=['24','打撃','前９','','5','4400','弱いがずる賢い敵が住む洞窟。\n毒々しい色が特徴的だ。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','弱者の洞窟','','じゃくしゃのどうくつ','','南東の地に佇んでいます。','前方９マス',''];
w['幻術の洞窟' + ('' ? ' ' + '' : '')]=['28','打撃','前９','','5','4800','魔法を好む敵が住み着く洞窟。\n洞窟自体にも魔力が蓄えられている。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','幻術の洞窟','','げんじゅつのどうくつ','ランダムで【極】攻撃魔法が発生','西の森のなかに佇んでいます。','前方９マス',''];
w['霊魂の洞窟' + ('' ? ' ' + '' : '')]=['26','打撃','前９','','5','5200','廃城に霊が住みつき、洞窟となった。\n恐ろしい呪いの力を感じる。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','霊魂の洞窟','','れいこんのどうくつ','即死（5％）','北東の地に佇んでいます。','前方９マス',''];
w['伝説の洞窟' + ('' ? ' ' + '' : '')]=['35','打撃','前９','','5','7500','伝説の武器が眠る黄金の洞窟。\n実は洞窟本体も普通に強い武器となる。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','伝説の洞窟','','でんせつのどうくつ','','村の北から気配を感じます。','前方９マス',''];
w['魔王城' + ('' ? ' ' + '' : '')]=['38','打撃','前９','','5','8700','魔王が住んでいる邪悪な城。\n武器として使えば魔王並みに強力。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','魔王城','','まおうじょう','','最北の地にそびえ立っています。','前方９マス',''];
w['ヘイワ村' + ('' ? ' ' + '' : '')]=['42','打撃','前９','','5','9800','いつも平和なヘイワ村。\n生まれたときから住む大好きな村だ。','%1は%2をブンブン振り回した！','%2はものすごい勢いで%3にぶち当たった！','','','ヘイワ村','','へいわむら','','あなたの住む村です。','前方９マス',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['大きめのヒビ' + ('' ? ' ' + '' : '')]=['8','打撃','直線２','','4','600','普通のヒビより大きめのヒビ。\n大きいぶん威力も高め。','%1は%2を手に持って振ってみた！','%3に%2が当たった！','%3にヒビが入った！','','大きめのヒビ','','おおきめのひび','防御力を下げる（100％）','鉄でできた洞窟にあります。','直線２マス',''];
w['壁のヒビ穴' + ('' ? ' ' + '' : '')]=['6','打撃','','','3','300','壁の下部に空いた穴。\n老朽化によるものだろうか。','%1は%2を手に持って振ってみた！','%3に%2が当たった！','%3にヒビが入った！','','壁のヒビ穴','','かべのひびあな','防御力を下げる（70％）','鉄でできた洞窟にあります。','',''];
w['むき出した壁' + ('' ? ' ' + '' : '')]=['4','打撃','','','3','500','外層が崩れ内層がむき出した壁。\n脆いため武器としては微妙。','%1は%2を手に持って振ってみた！','%3に%2が当たった！','','','むき出した壁','','むきだしたかべ','','魂のさまよう洞窟にあります。','',''];
w['黒ずみ' + ('' ? ' ' + '' : '')]=['2','打撃','','','2','100','地面の黒ずみの一部。\n汚そうなのであまり触りたくない。','%1は%2を手に持って振ってみた！','%3は少し黒ずんだ！','黒ずみの一部が%3の目に入った！','','黒ずみ','','くろずみ','命中率を下げる（100％）','最北の地に落ちてます。','',''];
w['緑ばんだ地面' + ('' ? ' ' + '' : '')]=['2','打撃','','','2','100','緑ばんでヌチョヌチョしてる地面。\nじめじめした場所に多い。','%1は%2を手に持って振ってみた！','%3は少し緑ばんだ！','緑のヌチョヌチョが%3の足にまとわりついた！','','緑ばんだ地面','','みどりばんだじめん','回避率を下げる（100％）','鉄でできた洞窟にあります。','',''];
w['道' + ('' ? ' ' + '' : '')]=['2','打撃','','','2','200','そこら辺にある道。\nこれがあると歩きやすい。','%1は%2を手に持って振ってみた！','%3に%2が当たった！','','','道','','みち','','そこらへんにたくさん落ちてます。','',''];
w['白いタイル' + ('' ? ' ' + '' : '')]=['4','斬撃','','','3','400','石製の白いタイル。\n４タイルで１セットだ。','%1は%2を手に持って振ってみた！','%2の角が%3を切り裂いた！','','','白いタイル','','しろいたいる','','どこかのコンビニの中にあります。','',''];
w['白いヒビ割れタイル' + ('' ? ' ' + '' : '')]=['3','斬撃','','','2','200','経年劣化によりヒビ割れてしまった\nボロボロの白タイル。','%1は%2を手に持って振ってみた！','%2の角が%3を切り裂いた！','','','白いヒビ割れタイル','','しろいひびわれたいる','','魂のさまよう洞窟にあります。','',''];
w['青いタイル' + ('' ? ' ' + '' : '')]=['5','斬撃','','','3','500','石製の青いタイル。\n白いタイルより若干硬い。','%1は%2を手に持って振ってみた！','%2の角が%3を切り裂いた！','','','青いタイル','','あおいたいる','','北東の地にあります。','',''];
w['青いヒビ割れタイル' + ('' ? ' ' + '' : '')]=['3','斬撃','','','2','250','古くなりヒビ割れた青タイル。\n見るからにボロボロだ。','%1は%2を手に持って振ってみた！','%2の角が%3を切り裂いた！','','','青いヒビ割れタイル','','あおいひびわれたいる','','北東の地にあります。','',''];
w['フローリング床' + ('' ? ' ' + '' : '')]=['5','斬撃','','','3','400','きれいなフローリングの床。\nきちんと掃除されているようだ。','%1は%2を手に持って振ってみた！','%2の角が%3を切り裂いた！','','','フローリング床','','ふろーりんぐゆか','','そこらへんの家の中にあります。','',''];
w['チェックタイル' + ('' ? ' ' + '' : '')]=['8','斬撃','','','3','500','チェックで派手なタイル。\n若干趣味が・・・なんでもない。','%1は%2を手に持って振ってみた！','%2の角が%3を切り裂いた！','','','チェックタイル','','ちぇっくたいる','','どこかの奇抜な家の中にあります。','',''];
w['教会の床' + ('' ? ' ' + '' : '')]=['6','斬撃','','','3','300','教会の木製の床。\nナチュラルな雰囲気だ。','%1は%2を手に持って振ってみた！','%2の角が%3を切り裂いた！','','','教会の床','','きょうかいのゆか','','どこかの教会の中にあります。','',''];
w['金床' + ('' ? ' ' + '' : '')]=['12','打撃','','','5','1500','武器の手入れができる金床。\n大きな切り株の上に乗っている。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','','','金床','','かなどこ','','そこらへんに置いてあります。','',''];
w['毒沼(小)' + ('' ? ' ' + '' : '')]=['7','なし','','','2','500','弱者の洞窟内の巨大な毒沼の一部を\n小さく切り取ったもの。','%1は%2を手に持って振ってみた！','','','','毒沼(小)','','どくぬましょう','毒（100％）','毒沼で覆われた洞窟にあります。','',''];
w['魔界の灯籠' + ('' ? ' ' + '' : '')]=['23','打撃','','','4','2000','魔界で作られた邪悪な灯籠。\n紫の炎がかなり魔界っぽい。','%1は%2を振り回した！','%2は%3に直撃した！','','%2から闇の炎が溢れ出した！','魔界の灯籠','','まかいのとうろう','炎が発生','魔族の王の住む城の中にあります。','',''];
w['クマのつるぎ' + ('' ? ' ' + '' : '')]=['12','斬撃','直線２','4','25','15000','クマたちがくれた不思議なつるぎ。\n握るとクマたちとの思い出が蘇る。','%1は%2を振り回した！','','','','クマのつるぎ','','くまのつるぎ','４回攻撃','４匹のクマから授かることができます。','直線２マス',''];
w['空飛ぶほうき' + ('' ? ' ' + '' : '')]=['11','打撃','直線２','','15','9000','乗って空を飛べるものすごいほうき。\n魔女が乗ってそうなイメージだ。','%1は%2を振り回した！','%2は%3に直撃した！','','','空飛ぶほうき','','そらとぶほうき','空を飛べる','あなたの妹が隠し持っています。','直線２マス',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['しかばね' + ('' ? ' ' + '' : '')]=['13','打撃','','','2','499','誰かのしかばねのようだ。\n話しかけても返事がない。','%1は%2を振り回した！','%2は%3に直撃した！','%3は呪われてしまった！','','しかばね','','しかばね','即死（5％）','村から南に出ると落ちてます。','',''];
w['動物のしかばね' + ('' ? ' ' + '' : '')]=['11','打撃','','','2','399','動物のしかばねのようだ。\nお手をしても返事がない。','%1は%2を振り回した！','%2は%3に直撃した！','%3は呪われてしまった！','','動物のしかばね','','どうぶつのしかばね','即死（5％）','南西の地に落ちてます。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['スライム' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','500','外を彷徨く謎のヌメヌメ。\n人を見かけると理由もなく襲う。','ゆけっ！%2！','%2はぷるんっと%3に体当たりした！','','','スライム','','すらいむ','','村の近くによく出現します。','',''];
w['レッドスライム' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','700','赤く変色したスライムの亜種。\n凶暴性がより増している。','ゆけっ！%2！','%2はぷるんっと%3に体当たりした！','','','レッドスライム','','れっどすらいむ','','北の地に出現します。','',''];
w['ゴールドスライム' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','2000','金色に輝くスライムの希少種。\n売ったら高そうだ。','ゆけっ！%2！','%2はぷるんっと%3に体当たりした！','','','ゴールドスライム','','ごーるどすらいむ','','黄金に輝く洞窟の中に出現します。','',''];
w['火の馬' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','800','常に体に引火した状態の馬。\n常に若干熱いと感じているらしい。','ゆけっ！%2！','%2は%3を蹴り飛ばした！','','','火の馬','','ひのうま','炎をふき出す','村を西に出ると出現します。','',''];
w['獄炎馬' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','1600','禍々しい色の炎を纏う馬。\n地獄からやってきたとか。','ゆけっ！%2！','%2は%3を蹴り飛ばした！','','','獄炎馬','','ごくえんば','炎をふき出す','魔族の王の住む城の中に出現します。','',''];
w['ゴブリン' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','600','いたずら好きなモンスター。\nなにかと物を投げるのが好き。','ゆけっ！%2！','%2は%3に体当たりした！','','','ゴブリン','','ごぶりん','武器を投げる','村を南に出ると出現します。','',''];
w['ミノタウロス' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','900','凶暴な牛の化物。懐にしまった\n無数の武器を投げつけてくる。','ゆけっ！%2！','%2は%3に体当たりした！','','','ミノタウロス','','みのたうろす','武器を投げる','北の地に出現します。','',''];
w['岩男' + ('' ? ' ' + '' : '')]=['en','打撃','','','5','500','内蔵まで岩石でできた大男。\n刃物で斬ってもビクともしない。','ゆけっ！%2！','%2は%3に岩の腕を叩きつけた！','','','岩男','','いわおとこ','','鉄でできた洞窟に出現します。','',''];
w['岩オヤジ' + ('' ? ' ' + '' : '')]=['en','打撃','','','5','700','すべての細胞が岩石でできた大男。\nその腕は破壊力抜群。','ゆけっ！%2！','%2は%3に岩の腕を叩きつけた！','','','岩オヤジ','','いわおやじ','','黄金に輝く洞窟の中に出現します。','',''];
w['鉄鋼虫' + ('' ? ' ' + '' : '')]=['en','打撃','','','9','600','ものすごい硬さを誇る害虫。\nその硬さは岩をも砕くほど。','ゆけっ！%2！','%2は岩砕きを放った！','','','鉄鋼虫','','てっこうちゅう','防御力を下げる（50％）','鉄でできた洞窟に出現します。','',''];
w['ダイヤモンド虫' + ('' ? ' ' + '' : '')]=['en','打撃','','','9','3000','世界一硬い生き物。\n物理攻撃などまったく効かない。','ゆけっ！%2！','%2は鎧砕きを放った！','','','ダイヤモンド虫','','だいやもんどむし','防御力を下げる（50％）','魔族の王の住む城の中に出現します。','',''];
w['炎の精' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','1000','炎から生まれた炎の精霊。\n近づくとたちまち燃やされてしまう。','ゆけっ！%2！','%2は%3に体当たりした！','','','炎の精','','ほのおのせい','【火炎の魔法】を唱える','魔力に満ちた洞窟に出現します。','',''];
w['氷の精' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','1000','氷から生まれた氷の精霊。\n辺りはいつも氷点下だ。','ゆけっ！%2！','%2は%3に体当たりした！','','','氷の精','','こおりのせい','【氷の魔法】を唱える','魔力に満ちた洞窟に出現します。','',''];
w['雷の精' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','1000','雷から生まれた雷の精霊。\n怒ると空から雷を降らす。','ゆけっ！%2！','%2は%3に体当たりした！','','','雷の精','','かみなりのせい','【雷の魔法】を唱える','魔力に満ちた洞窟に出現します。','',''];
w['精霊王' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','1800','すべての精霊を束ねる精霊の王。\nその姿は美しく神秘的だ。','ゆけっ！%2！','%2は%3に体当たりした！','','','精霊王','','せいれいおう','ランダムで【極】攻撃魔法を唱える','黄金に輝く洞窟の中に出現します。','',''];
w['レッドゲイズ' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','800','無数の目を持つ不気味なモンスター。\n攻撃力を下げてくる嫌な奴。','ゆけっ！%2！','%2は%3に体当たりした！','','','レッドゲイズ','','れっどげいず','【攻撃弱体の魔法】を唱える','南の地に出現します。','',''];
w['ブルーゲイズ' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','700','真っ青な体が不気味なモンスター。\n無数の目で睨まれると防御が下がる。','ゆけっ！%2！','%2は%3に体当たりした！','','','ブルーゲイズ','','ぶるーげいず','【防御弱体の魔法】を唱える','南の地に出現します。','',''];
w['ブラッドゲイズ' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','1500','返り血を浴びすぎて変色したゲイズ。\nこいつの弱体魔法には注意。','ゆけっ！%2！','%2は%3に体当たりした！','','','ブラッドゲイズ','','ぶらっどげいず','【全弱体の魔法】を唱える','魔族の王の住む城の中に出現します。','',''];
w['マジックラゲ' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','900','不思議なベールで包まれたクラゲ。\n魔法が一切効かないようだ。','ゆけっ！%2！','%2はシビれる腕で%3を攻撃した！','','','マジックラゲ','','まじっくらげ','感電（25％）','魔力に満ちた洞窟に出現します。','',''];
w['死の使い' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','1100','人々を死へと誘う恐ろしいモンスター。\n死神が送り出したと言われる。','ゆけっ！%2！','%2は笑いながら%3を攻撃した！','','','死の使い','','しのつかい','【闇の魔法】を唱える','魂のさまよう洞窟に出現します。','',''];
w['死神' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','1700','死を司る邪悪なモンスター。\n見つけた者を地獄へ引きずり降ろす。','ゆけっ！%2！','%2は笑いながら%3を攻撃した！','','','死神','','しにがみ','【闇の魔法・極】を唱える','魔族の王の住む城の中に出現します。','',''];
w['スケルトン' + ('' ? ' ' + '' : '')]=['en','打撃','','','5','699','しかばねに魂が宿り動き出した。\n笑いながら骨で殴ってくる。','ゆけっ！%2！','%2は笑いながら%3を攻撃した！','','','スケルトン','','すけるとん','','魂のさまよう洞窟に出現します。','',''];
w['ばけキノコ' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','700','ほうしを撒き散らしながら襲ってくる\nとても危険なキノコ。','ゆけっ！%2！','%2は%3に体当りした！','','','ばけキノコ','','ばけきのこ','ほうしをまき散らす','西の森に出現します。','',''];
w['鬼キノコ' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','1000','見るからに毒々しいボディを持つ。\n間違えても食べないように。','ゆけっ！%2！','%2は%3に体当りした！','','','鬼キノコ','','おにきのこ','ほうしをまき散らす','北の地に出現します。','',''];
w['うじゃ虫' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','500','常にうじゃうじゃしている虫。\n気持ち悪い上に非常に厄介。','ゆけっ！%2！','%2はうじゃうじゃした！','','','うじゃ虫','','うじゃむし','','毒沼で覆われた洞窟に出現します。','',''];
w['デシベル鳥' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','800','想像を絶する音量で鳴く鳥。\n朝鳴くと起きない人はいない。','ゆけっ！%2！','%2は150デシベルの鳴き声をあげた！','','','デシベル鳥','','でしべるちょう','通常攻撃以外を封じる（50%）','村から西に出ると出現します。','',''];
w['泣き虫デビル' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','1200','弱くて泣き虫だがずる賢い。\n魔法で自分を強化してくる厄介な敵。','ゆけっ！%2！','%2は泣きながら%3をひっかいた！','','','泣き虫デビル','','なきむしでびる','ランダムで強化魔法を唱える','毒沼で覆われた洞窟に出現します。','',''];
w['勇敢デビル' + ('' ? ' ' + '' : '')]=['en','打撃','','','4','900','心も体もたくましいデビル。\nただしずる賢さも衰えていない。','ゆけっ！%2！','%2は勇敢に%3をひっかいた！','','','勇敢デビル','','ゆうかんでびる','ランダムで強化魔法を唱える','黄金に輝く洞窟の中に出現します。','',''];
w['こうもり' + ('' ? ' ' + '' : '')]=['en','打撃','','2','3','600','パタパタと飛んでは噛みついてくる。\n動きが素早いので要注意。','ゆけっ！%2！','%2は%3に素早く二度噛み付いた！','','','こうもり','','こうもり','２回攻撃','村の近くに出現します。','',''];
w['闇こうもり' + ('' ? ' ' + '' : '')]=['en','打撃','','2','3','900','闇に紛れて襲ってくる厄介な奴。\n噛みつかれるとわりと痛い。','ゆけっ！%2！','%2は%3に素早く二度噛み付いた！','','','闇こうもり','','やみこうもり','２回攻撃','魂のさまよう洞窟に出現します。','',''];
w['ばくだん岩' + ('' ? ' ' + '' : '')]=['30','なし','','','1','1100','とにもかくにも危険な岩。\n瀕死になると自爆してくるぞ。','ゆけっ！%2！','%2は大爆発した！','','','ばくだん岩','','ばくだんいわ','爆発する','鉄でできた洞窟に出現します。','',''];
w['うじゃうじゃ虫' + ('' ? ' ' + '' : '')]=['en','打撃','','','3','2200','うじゃ虫の王。武器を風化させる\nうじゃうじゃ玉を飛ばして攻撃する。','ゆけっ！%2！','%2はうじゃうじゃした！','','','うじゃうじゃ虫','','うじゃむし','','魔族の王の住む城の中に出現します。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['鎧竜エリムサルエ' + ('' ? ' ' + '' : '')]=['18','斬撃','','3','9','8500','強固な鎧に全身を覆われた巨竜。\nその爪は如何なる物も斬り裂く。','ゆけっ！%2！','%2は鋼鉄の爪で何度も切り裂いた！','','','鎧竜エリムサルエ','','よろいりゅうえりむさるえ','３回攻撃','鉄でできた洞窟の最奥に出現します。','',''];
w['怪鳥ヴィジャヤ' + ('' ? ' ' + '' : '')]=['29','打撃','直線３','','9','7700','弱者の洞窟に現れた不気味な怪鳥。\n戦いでは自己を強化する戦法をとる。','ゆけっ！%2！','%2は毒液を吐き出した！','','','怪鳥ヴィジャヤ','','かいちょうゔぃじゃや','毒（50％）,【全強化の魔法】を唱える','毒沼で覆われた洞窟の最奥に出現します。','直線３マス',''];
w['魔塊テュルソス' + ('' ? ' ' + '' : '')]=['31','打撃','前６','','9','9900','意思を持った謎の塊。触手を操って\n戦うが、本体は特に何もしない。','ゆけっ！%2！','%2は%3にのしかかった！','','','魔塊テュルソス','','まかいてゅるそす','','魔力に満ちた洞窟の最奥に出現します。','前方６マス',''];
w['魔触手' + ('' ? ' ' + '' : '')]=['en','打撃','直線３','','4','2400','不気味な色をした触手。\n巨大な魔力を蓄えているようだ。','ゆけっ！%2！','%2はムチのようにしなり%3に直撃した！','','','魔触手','','ましょくしゅ','ランダムで攻撃魔法を唱える','魔力に満ちた洞窟の最奥に出現します。','直線３マス',''];
w['霊騎ダインスレイフ' + ('' ? ' ' + '' : '')]=['35','斬撃','','','9','8800','霊魂の洞窟に現れた騎士の霊。\n強い憎しみや怨念を感じる。','ゆけっ！%2！','%2は%3を串刺しにした！','','','霊騎ダインスレイフ','','れいきだいんすれいふ','【闇の魔法・極】を唱える','魂のさまよう洞窟の最奥に出現します。','',''];
w['武器の神' + ('アメノオハバリ' ? ' ' + 'アメノオハバリ' : '')]=['49','打撃','直線２','','9','19999','武器を司る、武器の神。\nすべてを超越した存在。','ゆけっ！%2！','%2は「大神斬」を放った！','%3は神の威厳に怯えている・・・。','','武器の神','アメノオハバリ','ぶきのかみあめのおはばり','１ターン行動不能（30％）','黄金に輝く洞窟の最奥に出現します。','直線２マス',''];
w['魔王ドルハーバ' + ('' ? ' ' + '' : '')]=['30','打撃','','2','9','19999','世界を滅ぼさんとする魔王。\n実は世界平和を願っている。','ゆけっ！%2！','%2は「魔界拳骨」を放った！','','','魔王ドルハーバ','','まおうどるはーば','２回攻撃,混乱（20％）','魔王城の最上階に出現します。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['犬' + ('' ? ' ' + '' : '')]=['14','斬撃','','','4','800','そこら辺にいた野良犬。\n人懐っこくてかわいい。','ゆけっ！%2！','%2は%3に噛み付いた！','%2はワンワンと吠えて%3をビビらせた！','','犬','','いぬ','１ターン行動不能（20％）','村のどこかを散歩しています。','','1'];
w['猫' + ('' ? ' ' + '' : '')]=['13','打撃','','','4','800','そこら辺にいた野良猫。\nンゴロンゴロと喉を鳴らしている。','ゆけっ！%2！','%2は%3に鋭い猫パンチを放った！','%2は喉を鳴らしながら%2に甘え始めた！','','猫','','ねこ','攻撃力を下げる（50％）','村のどこかを散歩しています。','','1'];
w['馬' + ('' ? ' ' + '' : '')]=['16','打撃','','','4','1500','誰かが飼っている馬。\n毛並みがきれいで美しい。','ゆけっ！%2！','%2は%3を蹴り飛ばした！','','','馬','','うま','吹っ飛ばす','北西の地をうろついてます。','','1'];
w['羊' + ('' ? ' ' + '' : '')]=['19','打撃','','','4','1600','ヒントを教えてくれるすごい羊。\nフワフワの毛が可愛らしい。','ゆけっ！%2！','%2は%3に体当りした！','%2のモフモフな毛が%3に纏わりついた！','','羊','','ひつじ','行動回数を下げる（30％）','私です。','','1'];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['村の女性' + ('ピランカ' ? ' ' + 'ピランカ' : '')]=['18','打撃','','','3','720','ヘイワ村に住む一人暮らしの女性。\n美人だし性格もいい。','ゆけっ！%2！','%2は優しく頭突きを放った！','','','村の女性','ピランカ','むらのじょせい','','村のどこかの家の中に住んでいます。','','1'];
w['一家の父親' + ('テルショ' ? ' ' + 'テルショ' : '')]=['8','打撃','','','3','1050','村に住む仲良し一家の大黒柱。\nいつも外で子供と遊んでいる。','ゆけっ！%2！','%2は一家の大黒柱パンチを放った！','','','一家の父親','テルショ','いっかのちちおや','','村のどこかで子供を見守っています。','','1'];
w['一家の母' + ('ホクトーマ' ? ' ' + 'ホクトーマ' : '')]=['26','打撃','','','5','960','ヘイワ村に住む家族の母親。\n普段は優しいが怒ると怖いらしい。','ゆけっ！%2！','%2はゲンコツを放った！','','%2は炎を吐き出した！','一家の母','ホクトーマ','いっかのはは','炎をふき出す','村のどこかの家の中に住んでいます。','','1'];
w['元気な兄弟' + ('クボタ' ? ' ' + 'クボタ' : '')]=['18','打撃','','','3','210','村で走り回る元気な兄弟の兄。\n常に「わーい」と言っている。','ゆけっ！%2！','%2は「わーい！」と言いながらパンチを放った！','','','元気な兄弟','クボタ','げんきなきょうだい','','村のどこかで元気に遊んでいます。','','1'];
w['元気な兄弟' + ('ナギナ' ? ' ' + 'ナギナ' : '')]=['6','打撃','','','3','180','兄と一緒に遊んでいるかわいい弟。\n将来の夢はYout◯uberだそう。','ゆけっ！%2！','%2は必殺ウルトラライダーキックを放った！','','','元気な兄弟','ナギナ','げんきなきょうだい','','村のどこかで元気に遊んでいます。','','1'];
w['食堂の料理長' + ('フランベル' ? ' ' + 'フランベル' : '')]=['14','打撃','','','4','1530','ヘイワ食堂を切り盛りする料理長。\n戦いでは足技が得意。','ゆけっ！%2！','%2は強烈な蹴りをお見舞いした！','','','食堂の料理長','フランベル','しょくどうのりょうりちょう','','村のどこかの家の中で料理長をしています。','','1'];
w['食堂の常連' + ('トンファ' ? ' ' + 'トンファ' : '')]=['11','打撃','','','3','840','ヘイワ食堂に毎日通う常連さん。\n最近血糖値がヤバい。','ゆけっ！%2！','%2は高血糖パンチを繰り出した！','%3も高血糖になった！','','食堂の常連','トンファ','しょくどうのじょうれん','行動回数を下げる（20％）','村のどこかの店で食事をしています。','','1'];
w['グルメ通' + ('カツバルガ' ? ' ' + 'カツバルガ' : '')]=['13','打撃','','','5','1500','世界を渡り歩く、自称グルメ通。\nヘイワ食堂を絶賛している。','ゆけっ！%2！','%2は出っ張った腹で%3を押しつぶした！','','','グルメ通','カツバルガ','ぐるめつう','','村のどこかの店で食事をしています。','','1'];
w['墓参りの老人' + ('ブレイグ' ? ' ' + 'ブレイグ' : '')]=['4','打撃','','','3','2520','村に住む優しい目をした老人。\n毎日妻の墓の前で手を合わせている。','ゆけっ！%2！','%2は弱々しく体当りした！','なんと%2の死んだ妻が%3を呪った！','','墓参りの老人','ブレイグ','はかまいりのろうじん','即死（5％）','村のどこかで墓参りをしています。','','1'];
w['井戸端のおばさん' + ('フキヤ' ? ' ' + 'フキヤ' : '')]=['25','打撃','','','5','1740','よく井戸端会議をしているおばさん。\n一度話し始めると２時間は止まらない。','ゆけっ！%2！','%2はヒステリック・ビンタを放った！','%3はヒステリックなおばさんを見てショックを受けた！','','井戸端のおばさん','フキヤ','いどばたのおばさん','混乱（20％）','村のどこかで井戸端会議をしています。','','1'];
w['井戸端のおばさん' + ('イルミサ' ? ' ' + 'イルミサ' : '')]=['15','打撃','','2','5','1680','噂話が大好きなおばさん。\nいつも井戸端で話に夢中だ。','ゆけっ！%2！','%2は往復ビンタを放った！','','','井戸端のおばさん','イルミサ','いどばたのおばさん','２回攻撃','村のどこかで井戸端会議をしています。','','1'];
w['散歩中の老人' + ('ロツド' ? ' ' + 'ロツド' : '')]=['7','打撃','','','3','2160','村でよく散歩している老人。\n聞いてもないことを語ってくる。','ゆけっ！%2！','%2は杖で%3を殴った！','','','散歩中の老人','ロツド','さんぽちゅうのろうじん','','村のどこかを散歩しています。','','1'];
w['散歩中のおじさん' + ('ランヤパ' ? ' ' + 'ランヤパ' : '')]=['13','打撃','','','5','1050','村を散歩していたおじさん。\n遠い故郷から出稼ぎに来たらしい。','ゆけっ！%2！','%2は散歩で鍛えた足で%3を蹴り飛ばした！','','','散歩中のおじさん','ランヤパ','さんぽちゅうのおじさん','吹っ飛ばす','村のどこかを散歩しています。','','1'];
w['' + ('' ? ' ' + '' : '')]=['','','','','','0','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','0','','','','','','','','','','','',''];
w['冒険者' + ('キカンジ' ? ' ' + 'キカンジ' : '')]=['5','打撃','','','3','630','伝説の武器を求める冒険者。\n三度の飯よりも母ちゃんが好き。','ゆけっ！%2！','%2は母ちゃんに教わったパンチを繰り出した！','','','冒険者','キカンジ','ぼうけんしゃ','','南の地を歩いています。','','1'];
w['風呂じじい' + ('テツケン' ? ' ' + 'テツケン' : '')]=['2','打撃','','','9','1470','毎日池で水を浴びているじじい。\nはやめに通報されてほしい。','ゆけっ！%2！','%2は笑いながら%3に体当たりした！','%3は変態を見てショックを受けた！','','風呂じじい','テツケン','ふろじじい','混乱（40％）','村を南に出ると立っています。','','1'];
w['キャンパー' + ('バスタード' ? ' ' + 'バスタード' : '')]=['18','打撃','','','4','810','海辺でキャンプをしていた青年。\nどちらかというとインドア派。','ゆけっ！%2！','%2はキャンプ道具を振り回して攻撃した！','','','キャンパー','バスタード','きゃんぱー','','南の地でキャンプしています。','','1'];
w['キャンパー' + ('イクパ' ? ' ' + 'イクパ' : '')]=['15','打撃','','','3','540','ソロキャンプが好きな女性。\n祖父の影響でキャンプを始めた。','ゆけっ！%2！','%2はキャンプ道具を振り回して攻撃した！','','','キャンパー','イクパ','きゃんぱー','','南の地でキャンプしています。','','1'];
w['キャンパー' + ('チャラク' ? ' ' + 'チャラク' : '')]=['17','打撃','','','4','960','一人ではしゃいでいるキャンパー。\n炎を見てテンションが上がっている。','ゆけっ！%2！','%2はキャンプ道具を振り回して攻撃した！','','%2はキャンプファイヤーを始めた！','キャンパー','チャラク','きゃんぱー','炎が発生','南の地でキャンプしています。','','1'];
w['旅の女性' + ('カトラス' ? ' ' + 'カトラス' : '')]=['14','斬撃','','','3','750','自分を探して旅する女性。\n洞窟の危険性を教えてくれた。','ゆけっ！%2！','%2は短剣で%3を斬り裂いた！','','','旅の女性','カトラス','たびのじょせい','','南の地を旅しています。','','1'];
w['冒険者' + ('フルイレ' ? ' ' + 'フルイレ' : '')]=['22','斬撃','','','4','600','毒沼の危険性を教えてくれた冒険者。\n薬草もくれてとても親切な人だ。','ゆけっ！%2！','%2は愛用の剣で%3を斬り裂いた！','','','冒険者','フルイレ','ぼうけんしゃ','','南東の地に立っています。','','1'];
w['きこりの男性' + ('ウォンマー' ? ' ' + 'ウォンマー' : '')]=['28','斬撃','','','4','1950','古びた小屋にひとりで暮らすきこり。\n斧の扱いがわりと上手い。','ゆけっ！%2！','%2は木を伐るように斧を振るった！','','','きこりの男性','ウォンマー','きこりのだんせい','防御力を下げる（30％）','どこかの小屋に住んでいます。','','1'];
w['旅の老人' + ('ブロウド' ? ' ' + 'ブロウド' : '')]=['5','打撃','','','3','2310','のんびりと旅をする老人。\n人生とは何かを探し続けている。','ゆけっ！%2！','%2は弱々しく頭突きを放った！','','','旅の老人','ブロウド','たびのろうじん','','村を西に出たところを旅しています。','','1'];
w['冒険者' + ('エストー' ? ' ' + 'エストー' : '')]=['11','斬撃','','','3','990','強さを求めて旅する冒険者の男。\n熟練者ぶってるがそこまで強くない。','ゆけっ！%2！','%2は剣で%3を攻撃した！','','','冒険者','エストー','ぼうけんしゃ','','村を西に出たところを歩いています。','','1'];
w['魔女' + ('クレモア' ? ' ' + 'クレモア' : '')]=['9','打撃','','','3','5400','森に住む小さな魔女。\nこう見えて魔法の腕はピカイチ。','ゆけっ！%2！','%2は杖を振って攻撃した！','','%2の杖の先から魔法が飛び出した！','魔女','クレモア','まじょ','ランダムで【極】攻撃魔法を唱える','どこかの奇抜な家に住んでいます。','','1'];
w['バザーに向かう女性' + ('タカナ' ? ' ' + 'タカナ' : '')]=['26','打撃','','','5','1260','日々バザーに向かって歩く女性。\nバザーにたどり着く日を夢見ている。','ゆけっ！%2！','%2は歩いて鍛えた足で%3を蹴り飛ばした！','','','バザーに向かう女性','タカナ','ばざーにむかうじょせい','吹っ飛ばす','村を東に出ると歩いています。','','1'];
w['物売りの女性' + ('ヤワラ' ? ' ' + 'ヤワラ' : '')]=['24','打撃','','','5','630','バザーで店を構えていた女性。\nかなり勉強熱心だ。','ゆけっ！%2！','%2はモーニングスターを振り回した！','','','物売りの女性','ヤワラ','ものうりのじょせい','混乱（20％）','東の地で物売りをしています。','','1'];
w['物売りの男性' + ('レイピア' ? ' ' + 'レイピア' : '')]=['21','斬撃','','2','5','1020','バザーで野菜を売っていた男性。\n実はかなりのおばあちゃん子。','ゆけっ！%2！','%2は素早い剣さばきで%3を斬り刻んだ！','','','物売りの男性','レイピア','ものうりのだんせい','２回攻撃','東の地で物売りをしています。','','1'];
w['物売りの男性' + ('プッサ' ? ' ' + 'プッサ' : '')]=['9','打撃','','','4','1650','見るからにイカサマしてそうな物売り。\nこういう人は関わらないほうがよい。','ゆけっ！%2！','%2は出っ張った腹で%3を押しつぶした！','','','物売りの男性','プッサ','ものうりのだんせい','','東の地で物売りをしています。','','1'];
w['バザーの客' + ('シュリンケ' ? ' ' + 'シュリンケ' : '')]=['19','打撃','','','4','720','バザーを楽しんでいた赤髪の男。\n所持金は 28 \\g。','ゆけっ！%2！','%2は右ストレートパンチを放った！','','','バザーの客','シュリンケ','ばざーのきゃく','','東の地のバザーで物色しています。','','1'];
w['バザーの客' + ('スクラマ' ? ' ' + 'スクラマ' : '')]=['8','斬撃','','','4','660','バザーに来ていた女性。\n半額セールが始まるのを待っている。','ゆけっ！%2！','%2はバザーで買った剣で%3を攻撃した！','','','バザーの客','スクラマ','ばざーのきゃく','','東の地のバザーで物色しています。','','1'];
w['バザーの客' + ('ブンディ' ? ' ' + 'ブンディ' : '')]=['15','打撃','','','4','1350','暇なのでバザーに来ていた男性。\n怪しいツボを買ったようだ。','ゆけっ！%2！','%2はバザーで買ったツボを%2にぶつけた！','','','バザーの客','ブンディ','ばざーのきゃく','必ず会心','東の地のバザーで物色しています。','','1'];
w['バザーの客' + ('ムルギザ' ? ' ' + 'ムルギザ' : '')]=['23','打撃','','2','3','1440','金持ちなバザーの客。\n札束で仰ぎながら歩いている。','ゆけっ！%2！','%2は札束を取り出しビンタした！','%3は大金を見てテンションが上がってしまった！','','バザーの客','ムルギザ','ばざーのきゃく','２回攻撃,相手の行動回数を上げる','東の地のバザーで物色しています。','','1'];
w['教会の牧師' + ('ファルシオ' ? ' ' + 'ファルシオ' : '')]=['17','打撃','','','5','1230','海沿いの教会の牧師。\nブンナゲ教の教徒でもある。','ゆけっ！%2！','%2はアーメンパンチを放った！','%3に何らかの聖なる力が働いた！','','教会の牧師','ファルシオ','きょうかいのぼくし','能力強化をリセットする','どこかの教会の中に立っています。','','1'];
w['教会のシスター' + ('ランシア' ? ' ' + 'ランシア' : '')]=['22','打撃','','','5','1140','優しそうなシスター。\n好きな麺類は、きしめん。','ゆけっ！%2！','%2は勢いよく体当たりを放った！','','','教会のシスター','ランシア','きょうかいのしすたー','','どこかの教会の中に立っています。','','1'];
w['お祈りする老婆' + ('ベルサ' ? ' ' + 'ベルサ' : '')]=['19','打撃','','','3','2460','教会で祈りを捧げていた老婆。\nブンナゲ教の熱烈な信者である。','ゆけっ！%2！','%2はお祈りしながらタックルした！','','','お祈りする老婆','ベルサ','おいのりするろうば','','どこかの教会の中でお祈りをしています。','','1'];
w['お祈りする青年' + ('イラルフ' ? ' ' + 'イラルフ' : '')]=['13','打撃','','','4','450','教会でお祈りしていた青年。\nとても真面目な性格。','ゆけっ！%2！','%2はお祈りしながら頭突きを放った！','','','お祈りする青年','イラルフ','おいのりするせいねん','','どこかの教会の中でお祈りをしています。','','1'];
w['冒険者' + ('クリク' ? ' ' + 'クリク' : '')]=['14','打撃','','','4','750','メガネの似合う緑髪の冒険者。\n廃墟巡りが趣味。','ゆけっ！%2！','%2は杖を振って攻撃した！','%2の杖の先から魔法が飛び出した！','','冒険者','クリク','ぼうけんしゃ','【氷の魔法】が発生','北東の地に立っています。','','1'];
w['釣り人' + ('フィシグロ' ? ' ' + 'フィシグロ' : '')]=['10','打撃','','','4','2040','釣りと海が好きな老人。\nしかし釣りの腕前はあまりない。','ゆけっ！%2！','%2は釣り竿を振り回して攻撃した！','なんと釣り竿の先には腐ったミミズがついていた！','','釣り人','フィシグロ','つりびと','毒（60％）','北西の岬で釣りをしています。','','1'];
w['冒険者' + ('ダーガ' ? ' ' + 'ダーガ' : '')]=['12','斬撃','','3','5','510','気の強そうな冒険者の女性。\n仕事を探しているらしい。','ゆけっ！%2！','%2は短剣で%3を素早く斬り裂いた！','','','冒険者','ダーガ','ぼうけんしゃ','３回攻撃','北の地を歩いています。','','1'];
w['コンビニ店員' + ('ミャオタ' ? ' ' + 'ミャオタ' : '')]=['27','打撃','','','4','660','新人のコンビニ店員。\nマニュアルどおりにしか話せない。','ゆけっ！%2！','%2はバーコードを読み取るやつで%3を殴った！','','','コンビニ店員','ミャオタ','こんびにてんいん','','どこかのコンビニの中に立っています。','','1'];
w['コンビニの客' + ('アルサト' ? ' ' + 'アルサト' : '')]=['15','斬撃','','','3','900','コンビニで買い物をしていた男性。\nどのジュースにするか悩んでいた。','ゆけっ！%2！','%2はコンビニのカードで%3を斬り裂いた！','','','コンビニの客','アルサト','こんびにのきゃく','','どこかのコンビニの中に立っています。','','1'];
w['ヤンキー' + ('レザビリー' ? ' ' + 'レザビリー' : '')]=['33','打撃','','','5','990','コンビニ前に屯する厳ついヤンキー。\n怒らせると怖そうだ。','ゆけっ！%2！','%2は%3をボコボコにした！','%3は%2にビビってしまった！','','ヤンキー','レザビリー','やんきー','１ターン行動不能（20％）','どこかのコンビニの前で屯しています。','','1'];
w['旅のおじさん' + ('グラディス' ? ' ' + 'グラディス' : '')]=['29','斬撃','','','5','2160','片目の傷が特徴的な熟練の旅人。\nいかにも強そうな見た目だ。','ゆけっ！%2！','%2は大剣を豪快に振り回した！','','','旅のおじさん','グラディス','たびのおじさん','','北の地を旅しています。','','1'];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['ぽん美' + ('' ? ' ' + '' : '')]=['1','打撃','','','5','180','\\n[1]の妹。小さいがしっかり者。\n家事も店番もひとりでこなしている。','ゆけっ！%2！','%2はがんばってパンチしてみた！','%2は%3の財布からお金を抜き取った！','','ぽん美','','ぽんみ','お金が手に入る','あなたの妹です。','','1'];
w['勇者シュヴァルディ' + ('' ? ' ' + '' : '')]=['42','斬撃','','','5','1140','やる気の無さそうな勇者。\n楽して魔王を倒したいらしい。','ゆけっ！%2！','%2は面倒臭そうに剣を振った！','%3は%2にやる気を吸い取られた！','','勇者シュヴァルディ','','ゆうしゃしゅゔぁるでぃ','攻撃力を下げる（40％）','あなたの店が発展すると訪れるでしょう。','','1'];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['魔王の剣' + ('' ? ' ' + '' : '')]=['26','斬撃','直線２','','25','18564','魔王がくれた闇の大剣。\n一撃、一撃が恐ろしく重い。','%1は%2を振り回した！','','','','魔王の剣','','まおうのけん','必ず会心','魔族の王が持っているでしょう。','直線２マス',''];
w['トロフィー入れ' + ('' ? ' ' + '' : '')]=['17','打撃','','','4','1000','獲得したトロフィーをしまっとく箱。\n手に持つとずっしりと重い。','%1は%2を手に持って振ってみた！','%2は%3に直撃した！','','','トロフィー入れ','','とろふぃーいれ','','あなたの家の中にあります。','',''];
w['ムシとりアミ' + ('' ? ' ' + '' : '')]=['0','なし','','','20','14500','色んなムシを捕まえられそうなアミ。\n吸引力はブラックホール並み。','%1は%2を振った！','','','','ムシとりアミ','','むしとりあみ','目の前の物を武器として入手','あなたの妹が隠し持っています。','',''];
w['うんち' + ('' ? ' ' + '' : '')]=['25','打撃','','','5','2','見事な一本巻きグソ。\nしめりけや色も完璧。','%1は%2を手に持って振ってみた！','%2は%3の顔面に直撃した！','%2はとっても臭い！','','うんち','','うんち','攻撃力を下げる（20％）,防御力を下げる（20％）,行動回数を下げる（20％）','北東の地にホカホカと落ちています。','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','物理封じ','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','魔法封じ','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','攻撃低下/上昇','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','防御低下/上昇','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','命中低下/上昇','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','回避低下/上昇','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','強化打ち消し','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','属性弱体','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','毒','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','混乱','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','睡眠','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','たまに即死','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','会心率高い','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
w['' + ('' ? ' ' + '' : '')]=['','','','','','','','','','','','','','','','','',''];
})();
    //翻訳データ
    //・セリフ
    (function(){
_tr_m.push(["テキスト","英語","韓国語","繁体文字","簡体文字"]);
_tr_m.push(["【お願い】こまめにセーブ または 設定→オートセーブを「ON」にしてプレイしてね！","[Tip] Save regularly or go to Settings and turn Auto-Save to ON!","【주의】틈틈이 저장하세요! 또는 설정→자동 저장을 「ON」으로 설정하고 플레이하세요!","【注意】請您經常儲存進度，或是在設定中將自動存檔功能調成「ON」再玩喔！","【注意】请您经常储存进度，或是在设置中将自动存档功能调成「ON」再玩喔！"]);
_tr_m.push(["ゲームパッドでのプレイですか？\\!  ","Are you playing with a gamepad?\\! ","게임 패드로 플레이하시겠어요?\\! ","請問是否要使用控制器操作？\\! ","请问是否要使用控制器操作？\\! "]);
_tr_m.push([">はい",">Yes",">예","＞是","＞是"]);
_tr_m.push(["ゲームパッドでのプレイの場合、設定→「ボタン表示タイプ」から ゲームパッドボタンを表示してプレイすることができます。","When using a gamepad, go to Settings > Button Layout to show gamepad buttons on the game screen.","게임 패드로 플레이할 경우, 설정→「버튼 표시 타입」에서 게임 패드 버튼을 표시하여 플레이할 수 있습니다.","使用控制器操作時，可在「設置」→「按鍵顯示類型」進行調整，如此在遊玩時即可顯示控制器按鈕。","使用控制器操作时，可在「设置」→「按键显示类型」进行调整，如此在游玩时即可显示控制器按钮。"]);
_tr_m.push(["","You can also check gamepad controls from the Help menu.","게임 패드의 조작 방법은 메뉴→「도움말」에서 확인할 수 있습니다.","此外，可從「選單」→「幫助」中確認控制器的操作方法。","此外，可从「菜单」→「帮助」中确认控制器的操作方法。"]);
_tr_m.push([">いいえ",">No",">아니요","＞否","＞否"]);
_tr_m.push([">わかりません",">I don't know",">모르겠어요","＞不清楚","＞不清楚"]);
_tr_m.push(["とある平和な世界の平和な村に住む 見習い武器商人の \\n[1]。","\\n[1] is a young weapons merchant living in a tranquil village set amidst a peaceful world.","어느 평화로운 세계의 평화로운 마을에 사는 견습 무기 상인, \\n[1].","在某個和平的世界中，有位名叫 \\n[1]的見習武器商人住在一個和平的村子裡。","在某个和平的世界中，有位名叫\\n[1]的见习武器商人住在一个和平的村子里。"]);
_tr_m.push(["一人前の武器商人を目指す\\n[1]は 今日も強い武器を探して歩いていた。","Following her dreams to become a renowned merchant, \\n[1] continues her search for powerful weapons.","번듯한 무기 상인이 되기 위해 \\n[1]는 오늘도 강력한 무기를 찾아다닌다.","\\n[1]為了成為能獨當一面的武器商人，今天也在各地尋找強大的武器，四處奔走。","\\n[1]为了成为能独当一面的武器商人，每天都在各地寻找强大的武器，四处奔走。"]);
_tr_m.push(["そのとき！ なんと、スライムの群れに囲まれてしまった！","Stinkin' stiletto! She is suddenly surrounded by slimes!","그때! 슬라임 무리에게 둘러싸이고 말았다!","此時！她竟然被一群史萊姆包圍了！","此时！她竟然被一群史莱姆包围了！"]);
_tr_m.push(["戦うにも、\\n[1]は武器を持っていない。 絶体絶命のピンチ！","\\n[1] finds herself in trouble because she has no weapon to fight with!","싸우고 싶어도 \\n[1]에게는 무기가 없다. 절체절명의 위기!","就算想要戰鬥，\\n[1]身上也沒有武器。這是攸關性命的危機！","就算想要战斗，\\n[1]身上也没有武器。这是攸关性命的危机！"]);
_tr_m.push(["しかし \\n[1]には一人前の武器商人になる夢がある。 こんな所で負けていられない！","But \\n[1] refuses to give up on her dream! She still has so much left to do!","하지만 \\n[1]에게는 번듯한 무기 상인이 되겠다는 꿈이 있다. 이런 곳에서 질 수는 없다!","然而， \\n[1]的夢想是成為獨當一面的武器商人。還不能在這裡輸掉！","然而，\\n[1]的梦想是成为独当一面的武器商人。不能在这里输掉！"]);
_tr_m.push(["開始早々ゲームオーバーを避けるため、 \\n[1]のすべてをかけた戦いがはじまる―――！","To avoid a game over from the get-go, she'll give her all in this battle!","시작하자마자 게임 오버가 되는 사태를 피하기 위해 \\n[1]의 모든 것을 건 싸움이 시작된다!","為了避免才開始就GAMEOVER， \\n[1]賭上一切的戰鬥即將開始──！","为了避免才开始就GAMEOVER，\\n[1]赌上一切的战斗即将开始──！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["キシャー！ \\nw[スライムの群れ]","Rraaah! \\nw[A group of slimes]","크아앙! \\nw[슬라임들의 무리]","嘰唰─！ \\nw[一群史萊姆]","叽唰─！\\nw[一群史莱姆]"]);
_tr_m.push(["\\n[1]は考えた。「素手で戦っても勝ち目はない。 なにか武器になるものはないだろうか」と。","\\n[1] thought, \"I can't win bare-handed. There must be something here I can use as a weapon.\"","\\n[1]는 생각했다. 「맨손으로 싸우면 승산이 없어. 뭔가 무기로 쓸 만한 게 없을까?」","\\n[1]心想：「空手戰鬥沒有勝算，有什麼東西可以拿來當武器嗎？」。","\\n[1]心想：「空手战斗没有胜算，有什么东西可以拿来当武器吗？」。"]);
_tr_m.push(["そのとき、\\n[1]は 死んだお父さんの最期の言葉を思い出した・・・！","It was then that she remembered her father's last words!","그때 \\n[1]는 돌아가신 아버지의 마지막 말을 떠올렸다・・・!","這時，\\n[1]想起了死去的父親留下的遺言……！","这时，\\n[1]想起了死去的父亲留下的遗言……！"]);
_tr_m.push(["\\n[1]、ぽん美。 いいか、よく聞け・・・。 \\nw[死んだお父さん]","\\n[1], Ponmi... I don't have much longer so listen carefully. \\nw[Dead father]","\\n[1], 폰미. 이 아비의 말을 잘 듣거라・・・ \\nw[죽은 아버지]","\\n[1]、器美。妳們仔細聽好了…… \\nw[死去的父親]","\\n[1]，器美。你们仔细听好了……\\nw[死去的父亲]"]);
_tr_m.push(["\\c[14]どんな物でも武器にして戦う・・・それが真の武器商人だ！！ \\nw[死んだお父さん]","\\c[14]When you fight, arm yourself with anything and everything available to you. That's what it means to be a true weapons merchant! \\nw[Dead father]","\\c[14]무엇이든 무기로 삼아 싸워라・・・ 그래야 진정한 무기 상인이다!! \\nw[죽은 아버지]","\\c[14]無論什麼東西都能拿來當成武器戰鬥……這才是真正的武器商人！！ \\nw[死去的父親]","\\c[14]无论什么东西都能拿来当成武器战斗……这才是真正的武器商人！！\\nw[死去的父亲]"]);
_tr_m.push(["ガクッ・・・。 \\nw[死んだお父さん]","Unngh... \\nw[Dead father]","꼴까닥・・・ \\nw[죽은 아버지]","（倒下）…… \\nw[死去的父親]","（倒下）……\\nw[死去的父亲]"]);
_tr_m.push(["\\n[1]は思った。 「お父さん、ありがとう・・・。」と。","\\n[1] whispered, \"Thanks, Dad...\"","\\n[1]는 생각했다. 「고마워요, 아빠・・・」","\\n[1]心想：「爸爸，謝謝你……」。","\\n[1]心想：「爸爸，谢谢你……」。"]);
_tr_m.push(["武器がないなら・・・ 武器となるものを探せばいい！！","If you don't have a weapon, find yourself one!","무기가 없으면・・・ 무기가 될 물건을 찾으면 되지!!","既然沒有武器……只要找出能當成武器的東西就行了！！","既然没有武器……只要找出能当成武器的东西就行了！！"]);
_tr_m.push(["さあ！武器になりそうな物を拾って あのスライムの群れをやっつけよう！","Okay! Look for something you can use to take care of the slimes!","자! 무기로 쓸 만한 물건을 주워서 저 슬라임 무리를 해치우자!","去吧！撿起感覺能當成武器的東西，打倒那群史萊姆！","去吧！捡起感觉能当成武器的东西，打倒那群史莱姆！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["スライムの群れを倒した！\\|\\|\\|\\|\\|\\|\\|\\|\\^","You beat those slime balls!\\|\\|\\|\\|\\|\\|\\|\\|\\^","슬라임 무리를 쓰러뜨렸다!\\|\\|\\|\\|\\|\\|\\|\\|\\^","打倒整群史萊姆了！\\|\\|\\|\\|\\|\\|\\|\\|\\^","打倒所有史莱姆了！\\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["こうして \\n[1]は様々な武器を使って 見事スライムの群れを撃退したのであった。","Thus, \\n[1] successfully defeated the slimes using various weapons.","이렇게 \\n[1]는 다양한 무기를 사용하여 멋지게 슬라임 무리를 무찔렀다.","就這樣，\\n[1]使用各式各樣的武器，順利打倒了整群史萊姆。","就这样，\\n[1]使用各式各样的武器，顺利打倒了所有史莱姆。"]);
_tr_m.push(["「この調子で、一人前の武器商人になるぞ！」 \\n[1]は心の中でそう思った。","She thought to herself, \"This is how I become a famous weapons merchant!\"","\\n[1]는 마음속으로 생각했다. 「이 기세를 몰아서 번듯한 무기 상인이 되어야지!」","「我要靠這方式，成為能獨當一面的武器商人！」，\\n[1]在心中默默地想著。","「我要靠这种方式，成为能独当一面的武器商人！」\\n[1]在心中默默地想着。"]);
_tr_m.push(["\\n[1]の「ぶきあつめ」の旅が 今、幕を開けたのである―――！","And thus begins \\n[1]'s foray into the wonderful world of weapons!","\\n[1]의 「무기 모으기」 여행이 막을 올렸다!","\\n[1]的「收集武器」之旅，現在正式揭幕────！","\\n[1]的「收集武器」之旅，现在正式开始────！"]);
_tr_m.push(["戦いを終えて疲れた\\n[1]は いったん自宅に帰ることにした。","Tired from the fight, \\n[1] headed home.","배틀을 마치고 지친 \\n[1]는 일단 집에 돌아가기로 했다.","\\n[1]在戰鬥結束後感到十分疲累，決定先回家一趟。","\\n[1]在战斗结束后感到十分疲累，决定先回家一趟。"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["あっ！お姉ちゃん！ お帰りなさーい。 \\nw[ぽん美]","Oh! Hey, Sis! Where'd ya go? \\nw[Ponmi]","앗! 언니! 어서 와~ \\nw[폰미]","啊！姐姐！妳回來啦。 \\nw[器美]","啊！姐姐！你回来啦。\\nw[器美]"]);
_tr_m.push(["・・・へえー！そんなことがあったんだ。 大変だったね。 \\nw[ぽん美]","...Wow! What a story... Musta been crazy! \\nw[Ponmi]","・・・아아~ 그런 일이 있었구나. 고생했어. \\nw[폰미]","……哦！原來發生了那種事啊。真是辛苦妳了。 \\nw[器美]","……哦！原来发生了那种事啊。真是辛苦你了。\\nw[器美]"]);
_tr_m.push(["だけど、無事でよかったよ。 お姉ちゃん、強いんだね！ \\nw[ぽん美]","I'm just glad you're all right. And apparently really strong too! \\nw[Ponmi]","그래도 무사해서 다행이야. 언니는 강하구나! \\nw[폰미]","不過，妳平安無事就太好了。姐姐好強喔！！ \\nw[器美]","不过，你平安无事就太好了。姐姐好强喔！！\\nw[器美]"]);
_tr_m.push(["そうそう！ \\nw[ぽん美]","That reminds me! \\nw[Ponmi]","아 참! \\nw[폰미]","對了對了！ \\nw[器美]","对了对了！\\nw[器美]"]);
_tr_m.push(["今週の週間ブキ春にこんな記事があったんだ。 読んでみてよ。 \\nw[ぽん美]","This article was in this week's Armsweek. Give it a read! \\nw[Ponmi]","이번 주 무기 타임즈에 이런 기사가 났어. 읽어 봐. \\nw[폰미]","這禮拜的「武器爽報」裡有這種報導耶，妳看一下。 \\nw[器美]","这礼拜的「武器周报」里有这种报道哦，你快看。\\nw[器美]"]);
_tr_m.push(["『発見！！伝説の武器！？』 \\nw[記事]","\"Breaking News! Legendary Weapon Discovered!\" \\nw[article]","『발견!! 전설의 무기!?』 \\nw[기사]","「發現！！傳說的武器！？」 \\nw[記事]","「传说中的武器已被发现！！？」\\nw[记事]"]);
_tr_m.push(["この世界にはいくつものロマンが眠っているが、 とてつもなくロマンに満ち溢れた情報が我々のもとに舞い込んだ。 \\nw[記事]","There are several wonders of the world, but we've gotten word about something that simply screams adventure. \\nw[article]","이 세계 곳곳에 잠들어 있는 로망, 그중에서도 어마어마한 로망으로 가득 찬 정보를 본지가 입수했다.\\nw[기사]","在這個世界上沉眠著許多的浪漫情懷，其中有個充滿浪漫的情報來到了我們的身邊。 \\nw[記事]","在这个世界上沉眠着许多的浪漫情怀，其中有个极具浪漫气息的消息来到了我们的身边。\\nw[记事]"]);
_tr_m.push(["なんと、この世界のどこかに、太古の昔に封印された 「伝説の武器」が眠っているという情報だ。 \\nw[記事]","An ancient, legendary weapon is hidden somewhere in the world. \\nw[article]","이 세계 어딘가에, 먼 옛날에 봉인된 「전설의 무기」가 잠들어 있다는 정보다. \\nw[기사]","這驚人的情報是，在這個世界的某處，沉眠著曾在太古往昔遭到封印的「傳說武器」。 \\nw[記事]","这惊人的消息便是，在这个世界的某处，沉眠着曾在太古往昔遭到封印的「传说武器」。\\nw[记事]"]);
_tr_m.push(["その武器の名は・・・ 「\\c[14]ツヨスギテ＝クサハエル・ソード\\c[0]」！ \\nw[記事]","It's known to us only as the Legend Of Legends Sword, a.k.a. the \\c[14]LOL Sword\\c[0]! \\nw[article]","그 무기의 이름은・・・ 「\\c[14]너무강해서 현웃터지는 소드\\c[0]」! \\nw[기사]","這把武器的名字是……「\\c[14]暴強‧草叢生劍\\c[0]」！ \\nw[記事]","这把武器的名字是……「\\c[14]无敌‧草丛生剑\\c[0]」！\\nw[记事]"]);
_tr_m.push(["この噂を聞きつけた冒険者たちが夢中で探し回っているが、 未だに手がかりすら得られていないらしい。 \\nw[記事]","After catching wind of its existence, adventurers far and wide have searched everywhere to no avail. \\nw[article]","이 소문을 접한 모험가들이 혈안이 되어 찾고 있으나 아직 단서조차 얻지 못했다고 한다. \\nw[기사]","聽到這個消息的眾多冒險者們，全都拚了命在尋找，然而他們目前甚至連線索都還沒找到。 \\nw[記事]","听到这个消息的众多冒险者们，全都拚了命在寻找，然而他们目前甚至连线索都还没找到。\\nw[记事]"]);
_tr_m.push(["あなたも伝説の武器を求めて ロマン溢れる大冒険に旅立ってみてはいかが？ \\nw[記事]","Will you set off on your own adventure to search for this legendary weapon? \\nw[article]","당신도 전설의 무기를 찾아서 로망 가득한 대모험을 떠나 보면 어떨까? \\nw[기사]","你要不要也去追尋傳說的武器，展開一場充滿浪漫情懷的大冒險呢？ \\nw[記事]","你要不要也去追寻传说的武器，展开一场充满浪漫情怀的大冒险呢？\\nw[记事]"]);
_tr_m.push(["どう！？なんか面白そうでしょ！！ \\nw[ぽん美]","Whaddya think? Sounds like a sack full of machetes if you ask me! \\nw[Ponmi]","어때!? 재밌을 것 같지!? \\nw[폰미]","怎麼樣？感覺很有趣吧！！ \\nw[器美]","怎么样？很有趣吧！！\\nw[器美]"]);
_tr_m.push(["伝説の武器かー。 お姉ちゃん、とってきてよ。 \\nw[ぽん美]","I can't be bothered, but you should totally go find this legendary weapon! \\nw[Ponmi]","전설의 무기래~ 언니가 가져오면 되겠네. \\nw[폰미]","傳說中的武器啊……姐姐，去把它找回來吧。 \\nw[器美]","传说中的武器啊……姐姐，去把它找回来吧。\\nw[器美]"]);
_tr_m.push(["超高く売れるかもよ。 \\nw[ぽん美]","Just imagine the price it could fetch! \\nw[Ponmi]","아주 비싸게 팔 수 있을지도 몰라. \\nw[폰미]","或許能賣一大筆錢呢。 \\nw[器美]","或许能卖一大笔钱呢。\\nw[器美]"]);
_tr_m.push(["うぇ子は驚きのあまり声が出ませんでした。","Weaco had no words.","웨코는 너무 놀라 목소리가 나오지 않았습니다.","武子因為太過驚訝，連聲音都發不出來。","武子因为太过惊讶，一句话都说不出来。"]);
_tr_m.push(["「ツヨスギテ＝クサハエル・ソード・・・。 　未だ誰も見たことのない・・・伝説の武器！！」","(The LOL Sword... A legendary weapon that no one has ever seen...)","「너무강해서 현웃터지는 소드・・・ 아무도 본 적이 없는・・・ 전설의 무기!!」","「暴強‧草叢生劍……！目前沒有任何人見過……傳說中的武器！！」","「无敌‧草丛生剑……！目前还没有任何人见过……传说中的武器！！」"]);
_tr_m.push(["「そんなすごい物が、この世界に眠っている！ 　なんてロマンがあってワクワクする話なんだ！！」","(To imagine that something like that exists in this world! How could I not fantasize about it!)","「그렇게 엄청난 물건이 이 세계에 잠들어 있다니! 얼마나 로망이 넘치고 두근거리는 얘기람!!」","「這麼厲害的東西，正沉眠於這個世界！這是多麼浪漫，多麼令人興奮的一件事！！」","「这么厉害的东西，正沉眠于这个世界！这是多么浪漫，多么令人兴奋的一件事！！」"]);
_tr_m.push(["と、\\n[1]は思いました。","...\\n[1] thought in a trance.","\\n[1]는 생각했습니다.","這時，\\n[1]心中做出了某種決定。","这时，\\n[1]心中做出了某个决定。"]);
_tr_m.push(["ん？どうしたの、お姉ちゃん？ \\nw[ぽん美]","You okay, Sis? \\nw[Ponmi]","응? 왜 그래, 언니? \\nw[폰미]","嗯？怎麼了嗎，姐姐？ \\nw[器美]","嗯？怎么了，姐姐？\\nw[器美]"]);
_tr_m.push(["えっ！？本当に伝説の武器を探しに行くの！？ 冗談で言ったのに！ \\nw[ぽん美]","Wha--?! I was just kidding! You're seriously going to look for it? \\nw[Ponmi]","뭐!? 정말로 전설의 무기를 찾으러 가겠다고!? 농담으로 한 얘기인데! \\nw[폰미]","咦！？妳真的要去找傳說中的武器嗎！？我只是開玩笑的耶！ \\nw[器美]","咦！？你真的要去找传说中的武器吗！？我只是开玩笑的呀！\\nw[器美]"]);
_tr_m.push(["・・・でも、その顔、本気なんだね。 \\nw[ぽん美]","Actually, you don't need to answer that. It's written all over your face. \\nw[Ponmi]","・・・그런데 표정을 보니 진심이구나. \\nw[폰미]","……不過，看妳的表情，妳是認真的吧。 \\nw[器美]","……不过，看你的表情，你是认真的吧。\\nw[器美]"]);
_tr_m.push(["お姉ちゃんは止めても聞くような人じゃないのは わかってるよ。 \\nw[ぽん美]","And I know better than to try and stop you when you've already made up your mind. \\nw[Ponmi]","언니는 내가 말려도 듣지 않을 사람이라는 거 알아. \\nw[폰미]","我知道，就算我阻止妳，妳也不會聽吧。 \\nw[器美]","我知道，就算我阻止你，你也不会听。\\nw[器美]"]);
_tr_m.push(["行ってきなよ！お姉ちゃん！ \\nw[ぽん美]","If you're gonna go, I'm all for it! \\nw[Ponmi]","다녀와! 언니! \\nw[폰미]","路上小心啊！姐姐！ \\nw[器美]","路上小心啊！姐姐！\\nw[器美]"]);
_tr_m.push(["お店のことは任せておいて。 店番ならやっといてあげる！ \\nw[ぽん美]","Leave the shop to me. I can handle things here! \\nw[Ponmi]","가게는 나한테 맡겨. 내가 지키고 있을게! \\nw[폰미]","店裡的事情就交給我。我會代替妳看店的！ \\nw[器美]","店里的事情就交给我。我会替你看店的！\\nw[器美]"]);
_tr_m.push(["その代わり・・・。 \\nw[ぽん美]","In return... \\nw[Ponmi]","그 대신・・・ \\nw[폰미]","相對的…… \\nw[器美]","相对的……\\nw[器美]"]);
_tr_m.push(["・・・その代わり、絶対に見つけてきてよね！ ツヨスギテ＝クサハエル・ソード！ \\nw[ぽん美]","You'd better find it! The LOL Sword! \\nw[Ponmi]","・・・그 대신 반드시 찾아와! 너무강해서 현웃터지는 소드! \\nw[폰미]","……相對的，妳一定要找到「暴強‧草叢生劍」喔！ \\nw[器美]","……相对的，你一定要找到「无敌‧草丛生剑」喔！\\nw[器美]"]);
_tr_m.push(["死んじゃった、お父さんのためにもね！ \\nw[ぽん美]","Do it for dad, too! \\nw[Ponmi]","돌아가신 아빠를 위해서도! \\nw[폰미]","這也是為了告慰爸爸的在天之靈！ \\nw[器美]","这也是为了告慰爸爸的在天之灵！\\nw[器美]"]);
_tr_m.push(["自宅では色々なことができます。 とりあえず「\\i[91]」を調べてみよう！","There are various thing to do at home. First, check the \\i[91] icons!","집에서는 다양한 일을 할 수 있습니다. 우선 「\\i[91]」를 조사해 보세요!","在自己家裡能做很多事。總之，先去調查「\\i[91]」吧！","在自己家里能做很多事。总之，先去调查「\\i[91]」吧！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["村から旅立とうとしたその時・・・。","As she left the village...","마을을 떠나려던 그때・・・","正當她準備要離開村子踏上旅程時……","正当她准备要离开村子踏上旅程时……"]);
_tr_m.push(["\\n[1]は、ふと父のことを思い出した。","\\n[1] remembered something about her father.","\\n[1]는 문득 아버지의 기억을 떠올렸다.","\\n[1]突然想起了父親的事。","\\n[1]突然想起了父亲的事。"]);
_tr_m.push(["―――それは、\\n[1]がまだ幼い頃の話。","It was a memory from her childhood.","\\n[1]가 아직 어렸을 때의 일이었다.","───那是\\n[1]年紀還很小時的事。","───那是在\\n[1]还很小的时候。"]);
_tr_m.push(["\\n[1]とぽん美の父、ウェポチチ。 彼は一流の武器商人であった。","\\n[1] and Ponmi's father, Dirk, was an expert weapons merchant.","\\n[1]와 폰미의 아버지 웨포파파. 그는 일류 무기 상인이었다.","\\n[1]與器美的父親，武父。他曾是一流的武器商人。","\\n[1]与器美的父亲，武父。他曾是一流的武器商人。"]);
_tr_m.push(["ウェポチチは商売の上手さもさることながら、 武器の腕も一流であった。","He was a cutlass above the rest, so to speak, and also proficient with any weapon.","웨포파파는 장사에 능했을 뿐더러 무기를 다루는 실력도 일류였다.","武父不僅擅長經商，使用武器的能力也是一流的高手。","武父不仅擅长经商，使用武器的能力也是一流的。"]);
_tr_m.push(["「俺はこの世のすべての武器を手に入れる！」 それがウェポチチの口癖だった。","He'd often exclaim, \"I'm going to collect all the weapons in the world!\"","웨포파파는 입버릇처럼 말했다. 「난 이 세계의 모든 무기를 가지겠어!」","「我要拿到這世界上所有的武器！」這是武父常掛在嘴邊的一句話。","「我要拿到这世界上所有的武器！」这是武父常挂在嘴边的一句话。"]);
_tr_m.push(["\\n[1]は父の姿を真似て、 そこらへんの草を武器に見立てて遊んだりしていた。","\\n[1] would frequently play and mimic her father with a blade of grass in hand.","\\n[1]는 아버지를 흉내 내어 주변의 풀을 무기로 삼아 놀고는 했다.","\\n[1]也會模仿父親的樣子，把附近的草當成武器來玩。","\\n[1]也会模仿父亲的样子，把附近的草当成武器来玩。"]);
_tr_m.push(["ウェポチチは、\\n[1]の憧れの父だった・・・。","She really looked up to him...","웨포파파는 \\n[1]에게 자랑스러운 아버지였다・・・","武父是\\n[1]很崇拜的父親……","\\n[1]很崇拜父亲武父……"]);
_tr_m.push(["しかしある日、ウェポチチは急に姿を消した。","But one day, Dirk was gone.","그러던 어느 날, 웨포파파는 갑자기 자취를 감췄다.","然而就在某一天，武父突然消失不見。","然而就在某一天，武父突然消失不见了。"]);
_tr_m.push(["そして、数日後・・・。","A few days later...","그리고 며칠 후・・・","數日後……","数日后……"]);
_tr_m.push(["ボロボロの姿で、彼は帰ってきたのであった。","He came home, bruised and battered.","웨포파파는 만신창이가 된 모습으로 돌아왔다.","武父渾身是傷地回來了。","武父浑身是伤地回来了。"]);
_tr_m.push(["急いで手当をしたが、全身に受けた傷は深く、 既に手遅れであった・・・。","His wounds were tended to, but it was too little too late.","급히 치료했지만 온몸에 깊은 부상을 입어 더는 손을 쓸 수가 없었다・・・","她連忙去幫武父包紮，但武父全身所受的傷十分嚴重，已經太遲了……","她连忙去帮武父包扎，但武父全身伤得很重，已经太迟了……"]);
_tr_m.push(["\\n[1]とぽん美が見守る中、 ウェポチチは言った。","As \\n[1] and Ponmi stood by his side, Dirk said to them...","\\n[1]와 폰미가 지켜보는 가운데 웨포파파가 말했다.","在\\n[1]與器美的看照中，武父說道：","武父看着\\n[1]与器美，说道："]);
_tr_m.push(["\\n[1]、ぽん美。 いいか、よく聞け・・・。 \\nw[父]","\\n[1], Ponmi... I don't have much longer so listen carefully. \\nw[father]","\\n[1], 폰미. 이 아비의 말을 잘 듣거라・・・ \\nw[아버지]","\\n[1]、器美。妳們仔細聽好了…… \\nw[父]","\\n[1]，器美。你们仔细听好了……\\nw[父]"]);
_tr_m.push(["どんな物でも武器にして戦う・・・それが真の武器商人だ！！ \\nw[父]","When you fight, arm yourself with anything and everything available to you. That's what it means to be a true weapons merchant! \\nw[father]","무엇이든 무기로 삼아 싸워라・・・ 그래야 진정한 무기 상인이다!! \\nw[아버지]","無論什麼東西都能拿來當成武器戰鬥……這才是真正的武器商人！！ \\nw[父]","无论什么东西都能拿来当成武器战斗……这才是真正的武器商人！！\\nw[父]"]);
_tr_m.push(["ガクッ・・・。 \\nw[父]","Unngh... \\nw[father]","꼴까닥・・・ \\nw[아버지]","（倒下）…… \\nw[父]","（倒下）……\\nw[父]"]);
_tr_m.push(["\\n[1]は思った。","\\n[1] thought to herself...","\\n[1]는 생각했다.","\\n[1]心想：","\\n[1]心想："]);
_tr_m.push(["名言っぽいけどまったく意味わかんねえ、と・・・。","That sounds epic, but I have no idea what it means.","명언스러운데 무슨 소린지 모르겠어・・・","雖然這句話聽起來像名言，但卻完全搞不懂是什麼意思……","虽然这句话听起来像一句名言，但却完全搞不懂是什么意思……"]);
_tr_m.push(["そしてウェポチチは息を引き取った。","And with that, Dirk took his last breath.","그리고 웨포파파는 숨을 거두었다.","接著，武父就這樣嚥下了最後一口氣。","接着，武父就这样咽下了最后一口气。"]);
_tr_m.push(["・・・\\n[1]はその後、 自然と武器商人を目指していた。","Afterwards, \\n[1] naturally followed in her father's footsteps.","・・・\\n[1]는 그 후로 자연스럽게 무기 상인을 꿈꾸게 되었다.","……在那之後，\\n[1]自然而然地也想要成為一名武器商人。","……在那之后，\\n[1]自然而然地也想要成为一名武器商人。"]);
_tr_m.push(["心の中のどこかで、 父の背中を追いたかったのかもしれない・・・。","Somewhere deep down, she probably wanted to be just like him.","마음속 어딘가에서 아버지의 뒤를 잇고 싶었는지도 모른다・・・","在她心中的某處，或許還想繼續追尋父親的背影吧……","在她心中的某处，或许还想继续追寻父亲的背影吧……"]);
_tr_m.push(["――\\n[1]は空を見上げ、 父の武器を振る姿を思い出していた。","\\n[1] looked to the sky and reminisced about how her father used to swing his sword.","\\n[1]는 하늘을 바라보며 아버지가 무기를 휘두르던 모습을 떠올렸다.","──\\n[1]抬頭望向天空，回想起父親揮動武器的樣子。","──\\n[1]抬头望向天空，回想起父亲挥动武器的样子。"]);
_tr_m.push(["「・・・さあ！お父さんみたいな武器商人になるために・・・ 　絶対に、ツヨスギテ＝クサハエル・ソードを手に入れるぞ！」","(I'm going to find this LOL Sword and become a weapons merchant as great as he was!)","「・・・자! 아빠 같은 무기 상인이 되기 위해・・・ 너무강해서 현웃터지는 소드를 반드시 찾고 말겠어!」","「……好！為了成為像爸爸那樣的武器商人……我一定要拿到暴強‧草叢生劍！」","「……好！为了成为像爸爸那样的武器商人……我一定要拿到无敌‧草丛生剑！」"]);
_tr_m.push(["\\n[1]は改めてそう思ったのであった！","\\n[1] was reminded of her life-long dream.","\\n[1]는 다시 한 번 다짐했다!","\\n[1]再次下定了決心！","\\n[1]再次下定了决心！"]);
_tr_m.push(["世界を周って伝説の武器を手に入れよう！","Explore the world and acquire that legendary weapon!","세계를 돌아다니며 전설의 무기를 손에 넣자!","踏遍世界，拿到傳說的武器吧！","踏遍世界，拿到传说中的武器吧！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["伝説の武器を探し歩く\\n[1]は、 怪しい洞窟にたどり着いた・・・。","As \\n[1] explores the land searching for the legendary weapon, she stumbles across a mysterious cave...","전설의 무기를 찾아다니던 \\n[1]는 수상한 동굴에 도착했다・・・","四處尋找傳說武器的\\n[1]，來到了一個奇怪的洞窟……","四处寻找传说武器的\\n[1]，来到了一个奇怪的洞窟……"]);
_tr_m.push(["伝説の武器を探し歩く\\n[1]は、 二つ目の洞窟にたどり着いた・・・。","As \\n[1] explores the land searching for the legendary weapon, she stumbles across a second cave...","전설의 무기를 찾아다니던 \\n[1]는 두 번째 동굴에 도착했다・・・","四處尋找傳說武器的\\n[1]，來到了第二個奇怪的洞窟……","四处寻找传说武器的\\n[1]，来到了第二个奇怪的洞窟……"]);
_tr_m.push(["伝説の武器を探し歩く\\n[1]は、 三つ目の洞窟にたどり着いた・・・。","As \\n[1] explores the land searching for the legendary weapon, she stumbles across a third cave...","전설의 무기를 찾아다니던 \\n[1]는 세 번째 동굴에 도착했다・・・","四處尋找傳說武器的\\n[1]，來到了第三個奇怪的洞窟……","四处寻找传说武器的\\n[1]，来到了第三个奇怪的洞窟……"]);
_tr_m.push(["伝説の武器を探し歩く\\n[1]は、 四つ目の洞窟にたどり着いた・・・。","As \\n[1] explores the land searching for the legendary weapon, she stumbles across a fourth cave...","전설의 무기를 찾아다니던 \\n[1]는 네 번째 동굴에 도착했다・・・","四處尋找傳說武器的\\n[1]，來到了第四個奇怪的洞窟……","四处寻找传说武器的\\n[1]，来到了第四个奇怪的洞窟……"]);
_tr_m.push(["・・・。 最奥から何かの気配を感じる。","...She senses something from deep within the cave.","・・・가장 깊숙한 곳에서 어떤 기척이 느껴진다.","……她感受到從最深處傳來某種氣息。","……她感受到了从最深处传来的某种气息。"]);
_tr_m.push(["\\n[1]は勇気を出して 足を踏み入れていくのであった―――。","\\n[1] musters up the courage to take a step forward.","\\n[1]는 용기를 내어 발걸음을 떼었다.","\\n[1]鼓起勇氣，踏入了洞窟──","\\n[1]鼓起勇气，踏入了洞窟──"]);
_tr_m.push(["ここが・・・最後の洞窟なのだろうか？","Is this the final cave...?","여기가・・・ 마지막 동굴일까?","這裡就是……最後的洞窟嗎？","这里就是……最后的洞窟吗？"]);
_tr_m.push(["でも、もしかしたら伝説の武器に関する 手がかりがあるかもしれない。","There might be a clue here regarding the legendary weapon.","하지만 어쩌면 전설의 무기에 관한 단서가 있을지도 모른다.","不過，或許這裡會有關於傳說武器的線索。","不过，或许这里会有关于传说武器的线索。"]);
_tr_m.push(["そう言い聞かせて、\\n[1]は 恐る恐る足を踏み入れていくのであった―――。","Although afraid, that thought alone kept her moving forward.","그렇게 마음을 다잡으며 \\n[1]는 머뭇머뭇 발걸음을 떼었다.","\\n[1]如此對自己說道，戰戰兢兢地踏入洞窟──","\\n[1]如此对自己说道，战战兢兢地踏入了洞窟──"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["そのとき！","Suddenly!","그때였다!","就在此時！","就在此时！"]);
_tr_m.push(["\\n[1]の頭の中に、何かが聞こえてきた！","\\n[1] heard a voice from within her head!","\\n[1]의 머릿속에서 누군가의 목소리가 들렸다!","\\n[1]的腦海裡傳來了一些聲音！","\\n[1]的脑海里传来了一些声音！"]);
_tr_m.push(["・・・。 \\nw[？？？？？]","... \\nw[?????]","・・・. \\nw[?????]","…… \\nw[？？？？？]","…… \\nw[？？？？？]"]);
_tr_m.push(["・・・強き者よ。よくぞ倒した。 \\nw[？？？？？]","You've done well, unwavering one. \\nw[?????]","・・・강한 자여. 잘 쓰러뜨렸도다. \\nw[?????]","……強者啊。妳能打倒如此強敵，真是不簡單。 \\nw[？？？？？]","……强者啊。你能打倒如此强敌，真是不简单。 \\nw[？？？？？]"]);
_tr_m.push(["そなたに力を与えよう・・・。 \\nw[？？？？？]","I shall grant you power. \\nw[?????]","그대에게 힘을 주리라・・・ \\nw[?????]","我就賜給妳力量吧…… \\nw[？？？？？]","就让我赐给你力量吧…… \\nw[？？？？？]"]);
_tr_m.push(["\\n[1]は\\c[14]必殺奥義『ザ・ミラクル斬り』\\c[0]を覚えた！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1] learned the \\c[14]special move [Miracle Slash]\\c[0]!","\\n[1]는 \\c[14]필살오의 『THE 미라클 베기』\\c[0]를 익혔다! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]學會了\\c[14]必殺奧義「THE‧奇蹟斬」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]学会了\\c[14]必杀奥义「THE‧奇迹斩」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["強き者よ・・・。 \\nw[？？？？？]","Unwavering one... \\nw[?????]","강한 자여・・・ \\nw[?????]","強者啊…… \\nw[？？？？？]","强者啊…… \\nw[？？？？？]"]);
_tr_m.push(["この世界に待ち構える\\c[14]のこり３つの試練\\c[0]・・・ そのすべてを制してみせよ。 \\nw[？？？？？]","\\c[14]Three more challenges\\c[0] await you in the world... Do your best to prevail over them all. \\nw[?????]","이 세계에서 그대를 기다리는 \\c[14]나머지 3개의 시련\\c[0]・・・ 그 전부를 제패하라. \\nw[?????]","這個世界\\c[14]還剩下３個試煉\\c[0]在等著妳……妳就去稱霸所有的試煉吧。 \\nw[？？？？？]","这个世界\\c[14]还剩下3个试炼\\c[0]在等着你……快去通过所有的试炼吧。 \\nw[？？？？？]"]);
_tr_m.push(["さすれば・・・ \\c[14]伝説の武器への道\\c[0]が開かれん・・・。 \\nw[？？？？？]","Once you do... The \\c[14]path to the legendary weapon\\c[0] shall be revealed to you... \\nw[?????]","그리하면・・・ \\c[14]전설의 무기로 향하는 길\\c[0]이 열리리라・・・ \\nw[?????]","如此一來…… \\c[14]通往傳說武器的道路\\c[0]就會在妳眼前展開…… \\nw[？？？？？]","如此一来…… \\c[14]通往传说武器的道路\\c[0]就会在你眼前展开…… \\nw[？？？？？]"]);
_tr_m.push(["\\n[1]は思った。","\\n[1] thought to herself...","\\n[1]는 생각했다.","\\n[1]心想：","\\n[1]心想："]);
_tr_m.push(["「なんだったんだ今の声は。 　もしかしたら新手の詐欺かもしれない・・・」と。","(What was that voice just now? It could be some new type of scam.)","「방금 그 목소리는 뭐였지? 어쩌면 신종 사기일지도 몰라・・・」","「剛剛那個聲音是怎樣？該不會是新的詐騙手法吧……」。","「刚刚那个声音是怎么回事？该不会是新的诈骗手法吧……」。"]);
_tr_m.push(["・・・しかし。","However...","・・・그러나","……然而。","……然而。"]);
_tr_m.push(["\\c[14]「のこり３つの試練を制すれば、 　伝説の武器への道が開かれる」\\c[0]。","\\c[14]\"Prevail over three more challenges and the path to the legendary weapon shall be revealed...\"\\c[0]","\\c[14]「나머지 3개의 시련을 제패하면 전설의 무기로 향하는 길이 열린다」\\c[0]","\\c[14]「只要稱霸剩下的３個試煉，通往傳說武器的道路就會在我眼前展開」\\c[0]。","\\c[14]「只要通过剩下的3个试炼，通往传说武器的道路就会在你眼前展开」\\c[0]。"]);
_tr_m.push(["その声ははっきりとそう言っていた。","That much was clear.","그 목소리는 분명히 그렇게 말했다.","那個聲音說得十分明確。","那个声音说得很清楚。"]);
_tr_m.push(["まるで、伝説の武器の在り処を知っているかのように・・・。","It's as if she had a map to the legendary weapon itself.","마치 전설의 무기가 어디 있는지 알고 있다는 듯이・・・","彷彿像是知道傳說武器在哪裡一樣……","彷佛像是知道传说武器在哪里一样……"]);
_tr_m.push(["\\n[1]は思った。","\\n[1] thought to herself...","\\n[1]는 생각했다.","\\n[1]心想：","\\n[1]心想："]);
_tr_m.push(["「他に伝説の武器の手がかりはないし・・・。 　どうせなら信じてやってみよう！」","(There are no other clues to the legendary weapon... I might as well take what I've got!)","「전설의 무기에 대한 다른 단서도 없고・・・ 이렇게 된 이상, 그냥 믿고 가 보자!」","「反正也沒有其他關於傳說武器的線索……那就相信它一次吧！」","「反正也没有其他关于传说武器的线索……那就相信它一次吧！」"]);
_tr_m.push(["「もし詐欺だとしても・・・慌てずに 　国民生活センターまたは警察へ相談しよう！」と。","(Even if it is a scam, all I'd have to do is... contact the authorities like the Consumer Protection Agency or the police.)","「만약 정말 사기라고 해도・・・ 당황하지 말고 보이스피싱 지킴이나 경찰에 신고하자!」","「就算是詐騙……也不用慌張，只要打去反詐騙專線，或是找警察商量就好了！」。","「就算是诈骗……也不用慌张，大不了打去反诈骗专线，或是找警察商量就好了！」。"]);
_tr_m.push(["こうして、\\n[1]の新手の詐欺（？）との戦いが 幕を開けたのであった・・・！","And thus begins \\n[1]'s fight against a new...scam? What adventures await our heroine?!","이리하여 신종 사기(?)에 맞서는 \\n[1]의 싸움이 막을 올리게 되었다・・・!","就這樣，\\n[1]與新人詐騙集團（？）之間的戰鬥正式揭幕了……！","就这样，\\n[1]与新人诈骗集团（？）之间的战斗正式揭幕了……！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["・・・強き者よ。 また来ると思っていたぞ・・・。 \\nw[？？？？？]","...Unwavering one. I knew you would return... \\nw[?????]","・・・강한 자여. 다시 올 줄 알고 있었다・・・ \\nw[?????]","……強者啊。我就知道妳還會再來…… \\nw[？？？？？]","……强者啊。我就知道你还会再来…… \\nw[？？？？？]"]);
_tr_m.push(["褒美だ。二つ目の力を与えよう。 \\nw[？？？？？]","Allow me to congratulate you. Once again, I shall grant you power. \\nw[?????]","상으로 두 번째 힘을 주마. \\nw[?????]","這是獎賞。我就賜給妳第二個力量吧。 \\nw[？？？？？]","这是奖赏。我就赐给你第二个力量吧。 \\nw[？？？？？]"]);
_tr_m.push(["\\n[1]の体の中に 何かがみなぎってきた・・・！","From within \\n[1]... a feeling of power surged forth!","\\n[1]의 몸속에서 무언가가 솟구쳤다・・・!","\\n[1]的體內湧出了某種神秘力量……！","\\n[1]的体内涌出了某种神秘力量……！"]);
_tr_m.push(["\\n[1]は\\c[14]必殺奥義『ぐるぐる回転斬り』\\c[0]を覚えた！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1] learned the \\c[14]special move [Spin Attack]\\c[0]! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]는 \\c[14]필살오의 『빙글빙글 회전베기』\\c[0]를 익혔다! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]學會了\\c[14]必殺奧義「轉轉迴旋斬」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]学会了\\c[14]必杀奥义「转转回旋斩」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["さらに・・・！","That is not all!","그리고・・・!","此外……！","此外……！"]);
_tr_m.push(["必殺奥義ゲージがたまる速度が上昇した！","Your SP Gauge now fills up faster!","필살오의 게이지가 쌓이는 속도가 빨라졌다!","必殺奧義量表累積的速度上升了！","必杀奥义量表累积的速度加快了！"]);
_tr_m.push(["残る試練はあと２つ。 健闘を祈ろう・・・。 \\nw[？？？？？]","There are two challenges left. I wish you luck with your task. \\nw[?????]","남은 시련은 앞으로 2개. 건투를 비마・・・ \\nw[?????]","還剩下２個試煉。祝妳武運昌隆…… \\nw[？？？？？]","还剩下2个试炼。祝你武运昌隆…… \\nw[？？？？？]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["強き者よ。 三つ目の試練も制してしまうとはな・・・。 \\nw[？？？？？]","Unwavering one... You have prevailed over the third challenge. \\nw[?????]","강한 자여. 세 번째 시련마저 제패할 줄이야・・・ \\nw[?????]","強者啊。想不到妳竟然通過了第三個試煉…… \\nw[？？？？？]","强者啊。想不到你竟然通过了第三个试炼…… \\nw[？？？？？]"]);
_tr_m.push(["さすがは、かつて・・・。 \\nw[？？？？？]","There has... \\nw[?????]","역시 과거에・・・ \\nw[?????]","真不愧是從前的…… \\nw[？？？？？]","真不愧是从前的…… \\nw[？？？？？]"]);
_tr_m.push(["・・・いや。今はいいだろう。 \\nw[？？？？？]","No, this is not the time. \\nw[?????]","・・・아니, 그 얘기는 나중에 하도록 하지. \\nw[?????]","……不，現在先別提這個了。 \\nw[？？？？？]","……不，现在先别提这个了。 \\nw[？？？？？]"]);
_tr_m.push(["褒美をやろう。新たなる力だ。 そなたの武器の力をさらに引き出すだろう・・・。 \\nw[？？？？？]","Allow me to reward you with a new power. It shall unlock your weapon's potential to its fullest. \\nw[?????]","상을 주마. 새로운 힘이다. 그대가 가진 무기의 힘을 더욱 끌어내 줄 것이다・・・ \\nw[?????]","我給妳獎賞吧。這是新的力量。這應該能讓妳引出更多武器的力量…… \\nw[？？？？？]","我给你奖赏吧。这是新的力量。这应该能让你引出更多武器的力量…… \\nw[？？？？？]"]);
_tr_m.push(["\\n[1]は\\c[14]必殺奥義『ロック・オン』\\c[0]を覚えた！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1] learned the \\c[14]special move [Lock On]\\c[0]! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]는 \\c[14]필살오의 『록 온』\\c[0]을 익혔다! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]學會了\\c[14]必殺奧義「LOCK‧ON」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]学会了\\c[14]必杀奥义「LOCK‧ON」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["次が最後の試練だ。 健闘を祈ろう・・・。 \\nw[？？？？？]","The next challenge is also the last. May you harden yourself and endure. \\nw[?????]","다음이 마지막 시련이다. 건투를 비마・・・ \\nw[?????]","接下來就是最後的試煉了。祝妳武運昌隆…… \\nw[？？？？？]","接下来就是最后的试炼了。祝你武运昌隆…… \\nw[？？？？？]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["・・・強き者よ。 最後の試練を制したようだな。 \\nw[？？？？？]","...Unwavering one. You have prevailed over the last challenge. \\nw[?????]","・・・강한 자여. 네 번째 시련을 제패하였구나. \\nw[?????]","……強者啊。看來妳稱霸了最後的試煉呢。 \\nw[？？？？？]","……强者啊。看来你通过了最后的试炼呢。 \\nw[？？？？？]"]);
_tr_m.push(["まずは褒美を与えよう・・・。 \\nw[？？？？？]","First, allow me to give you your reward. \\nw[?????]","우선 상을 주마・・・ \\nw[?????]","我先給妳獎賞吧…… \\nw[？？？？？]","我先给你奖赏吧…… \\nw[？？？？？]"]);
_tr_m.push(["\\n[1]は\\c[14]必殺奥義『斬り捨てゴメン』\\c[0]を覚えた！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1] learned the \\c[14]special move [Sorry Not Sorry]\\c[0]! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]는 \\c[14]필살오의 『정당방위 베기』\\c[0]를 익혔다! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]學會了\\c[14]必殺奧義「斬捨御免」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]学会了\\c[14]必杀奥义「斩舍御免」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["そして、そなたは今 我の与えしすべての試練を制した。 \\nw[？？？？？]","You have done what I have asked of you. You have faced and prevailed over every one of my challenges. \\nw[?????]","이것으로 그대는 내가 내린 모든 시련을 제패했다. \\nw[?????]","然後，妳現在……已經稱霸了我給妳的所有試煉。 \\nw[？？？？？]","然后，你现在……已经通过了我给你的所有试炼。 \\nw[？？？？？]"]);
_tr_m.push(["そなたに、\\c[14]最終試練\\c[0]を与えよう。 \\nw[？？？？？]","There is \\c[14]one final challenge\\c[0] you must face. \\nw[?????]","그대에게 \\c[14]마지막 시련\\c[0]을 내리겠노라. \\nw[?????]","接下來，我將會給妳一個\\c[14]最終試煉\\c[0]。 \\nw[？？？？？]","接下来，我将会给你一个\\c[14]最终试炼\\c[0]。 \\nw[？？？？？]"]);
_tr_m.push(["\\c[14]ヘイワ村の北の洞窟\\c[0]・・・ そこでそなたを待つ。 \\nw[？？？？？]","Come to the \\c[14]cave north of Peaceville\\c[0]... I shall await you there. \\nw[?????]","\\c[14]평화 마을 북쪽의 동굴\\c[0]・・・ 그곳에서 그대를 기다리겠다. \\nw[?????]","\\c[14]和平村北方的洞窟\\c[0]……我在那裡等妳。 \\nw[？？？？？]","\\c[14]和平村北方的洞窟\\c[0]……我在那里等你。 \\nw[？？？？？]"]);
_tr_m.push(["最終試練に打ち勝ち・・・伝説の武器、 \\c[14]「ツヨスギテ＝クサハエル・ソード」\\c[0]を手に入れてみせよ！ \\nw[？？？？？]","Overcome the final challenge and the legendary weapon known as... the \\c[14]LOL Sword\\c[0] shall be yours! \\nw[?????]","마지막 시련을 이겨 내어・・・ 전설의 무기, \\c[14]「너무강해서 현웃터지는 소드」\\c[0]를 가져가 보아라! \\nw[?????]","妳就想辦法通過最終試煉……獲得傳說武器\\c[14]「暴強‧草叢生劍」\\c[0]吧！ \\nw[？？？？？]","你就想办法通过最终试炼……获得传说武器\\c[14]「无敌‧草丛生剑」\\c[0]吧！ \\nw[？？？？？]"]);
_tr_m.push(["・・・。","...","・・・.","……","……"]);
_tr_m.push(["最後の戦いが始まろうとしている・・・。","The final battle is about the begin...","마지막 시련이 시작되려고 한다・・・","最後的戰鬥即將開始……","最后的战斗即将开始……"]);
_tr_m.push(["\\n[1]は思った。","\\n[1] thought to herself...","\\n[1]는 생각했다.","\\n[1]心想：","\\n[1]心想："]);
_tr_m.push(["「最終試練とやらをクリアして、 　絶対に伝説の武器を手に入れよう！」と・・・！","(I'm going to complete this final challenge... and I WILL get that legendary weapon!)","「마지막 시련이란 걸 클리어해서 반드시 전설의 무기를 손에 넣겠어!」","「我一定要通過那個什麼最終試煉，拿到傳說武器！」……","「我一定要通过那个什么最终试炼，拿到传说武器！」……"]);
_tr_m.push(["ヘイワ村の北の洞窟へ行き、 最終試練に臨もう！","Go to the cave north of Peaceville...","평화 마을 북쪽의 동굴로 가서","前往和平村北方的洞窟，","前往和平村北方的洞窟，"]);
_tr_m.push(["ここが・・・最後の試練の場所・・・。","This should be where the final challenge is...","여기가・・・ 마지막 시련의 장소・・・","這裡就是……最終試煉的地點……","这里就是……最终试炼的地点……"]);
_tr_m.push(["ここが最後の試練の場所・・・ 「伝説の武器が眠る洞窟」だ。 \\nw[？？？？？]","This is the Legendary Weapon Cave, the location of your final challenge. \\nw[?????]","여기가 마지막 시련의 장소・・・ 「전설의 무기가 잠든 동굴」이다. \\nw[?????]","這裡就是最終試煉的地點……「傳說武器沉眠的洞窟」。 \\nw[？？？？？]","这里就是最终试炼的地点……「传说武器沉眠的洞窟」。\\nw[？？？？？]"]);
_tr_m.push(["そなたが拾い集めてきたすべての力を使い、 突き進め！強き者よ・・・！ \\nw[？？？？？]","Unwavering one... Use everything you have picked up on your journey and press forward! \\nw[?????]","그대가 주워서 모아 온 모든 힘을 써서 돌파하라! 강한 자여・・・! \\nw[?????]","強者啊……！使用妳收集來的所有力量向前挺進吧！ \\nw[？？？？？]","强者啊……！使用你积累的所有力量向前挺进吧！\\nw[？？？？？]"]);
_tr_m.push(["最奥でそなたを待つ。 \\nw[？？？？？]","I shall be waiting in the inner sanctuary. \\nw[?????]","가장 깊숙한 곳에서 그대를 기다리겠다. \\nw[?????]","我會在洞窟最深處等妳。\\nw[？？？？？]","我会在洞窟最深处等你。\\nw[？？？？？]"]);
_tr_m.push(["なんと宝箱の中には \\i[261]薬草が 5個 入っていた！","There are 5 \\i[261]herbs in the chest!","보물상자 안에는 무려 \\i[261]약초 5개가 들어 있었다!","居然，寶箱裡有5個\\i[261]藥草！","宝箱里居然有5个\\i[261]药草！"]);
_tr_m.push(["\\i[261]薬草を 5個 手に入れた！","You got 5 \\i[261]herbs!","\\i[261]약초 5개를 획득했다!","獲得5個\\i[261]藥草！","获得5个\\i[261]药草！"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","宝箱里似乎空无一物。"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["その中に眠るのが、 伝説の武器、ツヨスギテ＝クサハエル・ソード・・・。 \\nw[？？？？？]","The legendary weapon, the LOL Sword, is hidden in there. \\nw[?????]","그 안에 잠들어 있는 것이 바로 전설의 무기, 너무강해서 현웃터지는 소드・・・ \\nw[?????]","沉眠於其中的是傳說中的武器「暴強‧草叢生劍」…… \\nw[？？？？？]","沉眠于其中的是传说中的武器「无敌‧草丛生剑」……\\nw[？？？？？]"]);
_tr_m.push(["真にそなたがその剣を握る資格があるかどうか。 \\nw[？？？？？]","Are you truly worthy of wielding its greatness? \\nw[?????]","진정 그대에게 그 검을 쥘 자격이 있는지 \\nw[?????]","妳是否真的有資格拿起這把劍呢？ \\nw[？？？？？]","你是否真的有资格拿起这把剑呢？\\nw[？？？？？]"]);
_tr_m.push(["それを試すのが、この最後の試練である。 \\nw[？？？？？]","This final challenge exists to test just that. \\nw[?????]","이 마지막 시련을 통해 검증하게 되리라. \\nw[?????]","測試妳是否有此資格，就是最後的試煉。 \\nw[？？？？？]","测试你是否有此资格，就是最后的试炼。\\nw[？？？？？]"]);
_tr_m.push(["数年前、この試練に挑んだ男がいた。 \\nw[？？？？？]","There was one man who took this challenge many years ago. \\nw[?????]","몇 년 전, 이 시련에 도전한 사내가 있었지. \\nw[?????]","數年前，有個男人曾經挑戰過這個試煉。 \\nw[？？？？？]","数年前，有个男人曾经挑战过这个试炼。\\nw[？？？？？]"]);
_tr_m.push(["しかしその男は試練に敗れ、命を落とした。 \\nw[？？？？？]","However, he failed and died as a result. \\nw[?????]","하지만 그자는 시련을 뛰어넘지 못하고 목숨을 잃었다. \\nw[?????]","然而，那個男人敗給了試煉，並因此喪命。 \\nw[？？？？？]","然而，那个男人败给了试炼，并因此丧命。\\nw[？？？？？]"]);
_tr_m.push(["その男に足りなかった力。 その男がこの試練の中で気づいた力・・・！ \\nw[？？？？？]","That man lacked the strength. But he gained something during the challenge! \\nw[?????]","그 사내에게 부족했던 힘. 그 사내가 이 시련을 통해 깨달은 힘・・・! \\nw[?????]","那個男人所欠缺的力量……那個男人在這場試煉中察覺到的力量……！ \\nw[？？？？？]","那个男人所欠缺的力量……那个男人在这场试炼中察觉到的力量……！\\nw[？？？？？]"]);
_tr_m.push(["その力を受け継ぐ者よ！ \\nw[？？？？？]","And that something has been passed on to you! \\nw[?????]","그 힘을 계승할 자여! \\nw[?????]","繼承了那股力量的人啊！ \\nw[？？？？？]","继承了那股力量的人啊！\\nw[？？？？？]"]);
_tr_m.push(["この試練、乗り越えてみせよ！！ \\nw[？？？？？]","Come forth and overcome this trial! \\nw[?????]","이 시련을 극복해 보거라!! \\nw[?????]","努力通過這個試煉吧！！ \\nw[？？？？？]","努力通过这个试炼吧！！\\nw[？？？？？]"]);
_tr_m.push(["最後の試練を始める！！！ \\nw[？？？？？]","The final challenge is upon you! \\nw[?????]","마지막 시련을 시작한다!!! \\nw[?????]","最後的試煉要開始了！！！ \\nw[？？？？？]","最后的试炼要开始了！！！\\nw[？？？？？]"]);
_tr_m.push(["\\{\\{武器の神アメノオハバリ\\|\\|\\|\\^","\\{\\{God of Weaponry Ameno Ohbari\\|\\|\\|\\^","\\{\\{무기의 신 아메노오하바리\\|\\|\\|\\^","\\{\\{武器之神 天之尾羽張\\|\\|\\|\\^","\\{\\{武器之神 天之尾羽张\\|\\|\\|\\^"]);
_tr_m.push(["我の名は武器の神アメノオハバリ！！ \\nw[武器の神アメノオハバリ]","I am Ameno Ohbari! The god of weaponry! \\nw[God of Weaponry Ameno Ohbari]","나의 이름은 무기의 신, 아메노오하바리!! \\nw[무기의 신 아메노오하바리]","我的名字是武器之神「天之尾羽張」！！ \\nw[武器之神 天之尾羽張]","我的名字是武器之神「天之尾羽张」！！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["神を倒してみせよ！\\n[1]！！！ \\nw[武器の神アメノオハバリ]","Now, let us see if you can defeat a god, \\n[1]! \\nw[God of Weaponry Ameno Ohbari]","신을 쓰러뜨려 보아라! \\n[1]!!! \\nw[무기의 신 아메노오하바리]","妳就試著來打倒神吧！\\n[1]！！！ \\nw[武器之神 天之尾羽張]","你就试着来打倒神吧！\\n[1]！！！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","宝箱里似乎空无一物。"]);
_tr_m.push(["\\{武器の神アメノオハバリ　最終形態\\|\\|\\|\\^","\\{God of Weaponry Ameno Ohbari Final form\\|\\|\\|\\^","\\{무기의 신 아메노오하바리　최종 형태\\|\\|\\|\\^","\\{武器之神 天之尾羽張　最終表格\\|\\|\\|\\^","\\{武器之神 天之尾羽张　最终表格\\|\\|\\|\\^"]);
_tr_m.push(["・・・。 \\nw[武器の神アメノオハバリ]","... \\nw[God of Weaponry Ameno Ohbari]","・・・. \\nw[무기의 신 아메노오하바리]","…… \\nw[武器之神 天之尾羽張]","……\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["見事だ、\\n[1]・・・。 \\nw[武器の神アメノオハバリ]","Excellent work, \\n[1]... \\nw[God of Weaponry Ameno Ohbari]","훌륭하구나, \\n[1]・・・ \\nw[무기의 신 아메노오하바리]","了不起，\\n[1]…… \\nw[武器之神 天之尾羽張]","了不起，\\n[1]……\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["――数年前、そなたがまだ幼い頃。 この洞窟を訪れた男がいた。 \\nw[武器の神アメノオハバリ]","Years ago, when you were but a child, a man showed up to this cave. \\nw[God of Weaponry Ameno Ohbari]","몇 년 전, 그대가 아직 어렸을 적에 이 동굴을 찾아온 사내가 있었지. \\nw[무기의 신 아메노오하바리]","──數年前，在妳年紀還很小時，有個男人曾來過這個洞窟。 \\nw[武器之神 天之尾羽張]","──数年前，在你年纪还很小时，有个男人曾来过这个洞窟。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["その男の名は\\c[14]ウェポチチ\\c[0]。一流の武器商人だった。 \\nw[武器の神アメノオハバリ]","His name was \\c[14]Dirk\\c[0] and he was a first-rate weapons merchant. \\nw[God of Weaponry Ameno Ohbari]","그 사내의 이름은 \\c[14]웨포파파\\c[0]. 일류 무기 상인이었다. \\nw[무기의 신 아메노오하바리]","他名叫\\c[14]武父\\c[0]，是一流的武器商人。 \\nw[武器之神 天之尾羽張]","他名叫\\c[14]武父\\c[0]，是一流的武器商人。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["くそっ、強い・・・！！ \\nw[ウェポチチ]","How can you be this strong! \\nw[Dirk]","젠장, 강하군・・・!! \\nw[웨포파파]","可惡，好強啊……！！ \\nw[武器父]","可恶，好强啊……！！\\nw[武器父]"]);
_tr_m.push(["やべえ、来なきゃよかったかも・・・。 \\nw[ウェポチチ]","I should never have come here... \\nw[Dirk]","큰일이다, 오지 말았어야 했는데・・・ \\nw[웨포파파]","糟糕，早知道不該來的…… \\nw[武器父]","糟糕，早知道不该来的……\\nw[武器父]"]);
_tr_m.push(["そうだ。 \\n[1]とぽん美が待っている・・・。 \\nw[ウェポチチ]","That reminds me. \\n[1] and Ponmi are waiting for me at home... \\nw[Dirk]","그래. \\n[1]와 폰미가 나를 기다리고 있어・・・ \\nw[웨포파파]","對了， \\n[1]和器美還在等我…… \\nw[武器父]","对了，\\n[1]和器美还在等我……\\nw[武器父]"]);
_tr_m.push(["ここは生きて帰ることを優先せねば・・・！ \\nw[ウェポチチ]","I've got to survive this ordeal and get home to them...! \\nw[Dirk]","여기서 살아서 나가는 게 가장 중요해・・・! \\nw[웨포파파]","我必須想辦法活著回去……！ \\nw[武器父]","我必须想办法活着回去……！\\nw[武器父]"]);
_tr_m.push(["よし！逃げ・・・ \\nw[ウェポチチ]","Okay! Time to scram! \\nw[Dirk]","좋았어! 도망치・・・ \\nw[웨포파파]","好！逃跑吧…… \\nw[武器父]","好！逃跑吧……\\nw[武器父]"]);
_tr_m.push(["ぬわーーっっ！！ \\nw[ウェポチチ]","Gyaaah! \\nw[Dirk]","으아아아아악!! \\nw[웨포파파]","嗚哇──！！ \\nw[武器父]","呜哇──！！\\nw[武器父]"]);
_tr_m.push(["・・・。 \\nw[ウェポチチ]","... \\nw[Dirk]","・・・. \\nw[웨포파파]","…… \\nw[武器父]","……\\nw[武器父]"]);
_tr_m.push(["ううっ・・・。 \\nw[ウェポチチ]","Ungh... \\nw[Dirk]","으윽・・・ \\nw[웨포파파]","嗚嗚…… \\nw[武器父]","呜呜……\\nw[武器父]"]);
_tr_m.push(["俺は・・・。 \\nw[ウェポチチ]","I... \\nw[Dirk]","나는・・・ \\nw[웨포파파]","我…… \\nw[武器父]","我……\\nw[武器父]"]);
_tr_m.push(["『たたかう』『にげる』の二択のうち、 『にげる』を選択してしまった。 \\nw[ウェポチチ]","Fight or flight...those were my two choices. And I chose to flee. \\nw[Dirk]","『싸운다』와 『도망친다』. 2개의 선택지 중 『도망친다』를 선택했다. \\nw[웨포파파]","在「戰鬥」與「逃跑」這兩個選項間，選擇了「逃跑」。 \\nw[武器父]","在「战斗」与「逃跑」这两个选项间，选择了「逃跑」。\\nw[武器父]"]);
_tr_m.push(["\\n[1]とぽん美のことがふと頭をよぎり・・・ 家に帰らねば。その気持ちに支配されてしまった。 \\nw[ウェポチチ]","All I could think of was \\n[1] and Ponmi. Getting home was the only thing that mattered. \\nw[Dirk]","순간적으로 \\n[1]와 폰미가 머릿속에 떠올라・・・ 집에 돌아가야 한다는 생각에 지배당하고 말았다. \\nw[웨포파파]","我的腦海裡突然浮現出\\n[1]與器美的臉……必須要趕快回家。這種心情控制了我。 \\nw[武器父]","我的脑海里突然浮现出\\n[1]与器美的脸……必须要赶快回家。这种心情控制了我。\\nw[武器父]"]);
_tr_m.push(["あそこで、『二人に勝った姿を見せてやりたい！』とでも 思っていれば・・・。 \\nw[ウェポチチ]","I had wanted to show them a man that reigned victorious... \\nw[Dirk]","그때 『두 사람에게 내가 이긴 모습을 보여 줘야지!』라고 생각했더라면・・・ \\nw[웨포파파]","如果我當時心中想的是「要讓她們兩個見識一下我打贏的樣子！」…… \\nw[武器父]","如果我当时心中想的是「要让她们两个见识一下我打赢的样子！」……\\nw[武器父]"]);
_tr_m.push(["\\n[1]とぽん美の存在を、 力に・・・武器に変えていれば！！ \\nw[ウェポチチ]","I should have let \\n[1] and Ponmi empower me...to become my weapons! \\nw[Dirk]","\\n[1]와 폰미의 존재를, 힘으로・・・ 무기로 삼았더라면!! \\nw[웨포파파]","如果我能把\\n[1]與器美的存在轉化成力量……轉化成武器的話！！ \\nw[武器父]","如果我能把\\n[1]与器美的存在转化成力量……转化成武器的话！！\\nw[武器父]"]);
_tr_m.push(["結末は変わっていたかもしれない！！ \\nw[ウェポチチ]","Things might have turned out differently! \\nw[Dirk]","결말이 바뀌었을지도 모르는데!! \\nw[웨포파파]","或許結局就會不一樣了！！ \\nw[武器父]","或许结局就会不一样了！！\\nw[武器父]"]);
_tr_m.push(["俺は『心』を武器にできなかった。武器商人失格だ。 \\nw[ウェポチチ]","I failed to arm myself with the power of love. And I call myself a weapons merchant... \\nw[Dirk]","나는 『마음』을 무기로 삼지 못했다. 무기 상인으로서 실격이야. \\nw[웨포파파]","我沒能把「心」轉化成武器。我不配當武器商人。 \\nw[武器父]","我没能把「心」转化成武器。我不配当武器商人。\\nw[武器父]"]);
_tr_m.push(["だから、あの二人に伝えたい・・・。 \\nw[ウェポチチ]","The least I can do is tell them... \\nw[Dirk]","그러니 두 사람에게 전해야 해・・・ \\nw[웨포파파]","因此，我想告訴她們兩個…… \\nw[武器父]","因此，我想告诉她们两个……\\nw[武器父]"]);
_tr_m.push(["どんな物でも、武器に変えること！！ \\nw[ウェポチチ]","...that anything and everything can become a weapon! \\nw[Dirk]","무엇이든 무기로 삼아라!! \\nw[웨포파파]","無論什麼東西都能拿來當成武器！！ \\nw[武器父]","无论什么东西都能拿来当成武器！！\\nw[武器父]"]);
_tr_m.push(["背負うものすべて武器に変え、戦うこと！！ \\nw[ウェポチチ]","To arm themselves with everything they've got! \\nw[Dirk]","등에 짊어진 모든 것을 무기로 삼아 싸워라!! \\nw[웨포파파]","將肩負的一切事物都變成武器，用來戰鬥！！ \\nw[武器父]","将肩负的一切事物都变成武器，用来战斗！！\\nw[武器父]"]);
_tr_m.push(["それが、俺のなれなかった・・・ 真の武器商人の姿であるということを。 \\nw[ウェポチチ]","That is what a true weapons merchant should aim to be. One I could not become! \\nw[Dirk]","그것이 내가 될 수 없었던・・・ 진정한 무기 상인의 모습이라는 사실을. \\nw[웨포파파]","這就是我沒能達成……真正的武器商人該有的樣子。 \\nw[武器父]","这就是我没能实现……真正的武器商人该有的样子。\\nw[武器父]"]);
_tr_m.push(["うっ、意識が・・・。 \\nw[ウェポチチ]","Ungh... It's getting hazy... \\nw[Dirk]","윽, 의식이・・・ \\nw[웨포파파]","唔，我的意識…… \\nw[武器父]","唔，意识……\\nw[武器父]"]);
_tr_m.push(["ん？あれは・・・。 \\nw[村人]","Hm? Isn't that... \\nw[villager]","응? 저건・・・ \\nw[마을]","嗯？那是…… \\nw[村民]","嗯？那是……\\nw[村民]"]);
_tr_m.push(["大変だ！！ウェポチチさんが！ \\nw[村人]","Somebody get help! Dirk is hurt! \\nw[villager]","큰일이야! 웨포파파 씨가! \\nw[마을]","不好了！！武父他……！ \\nw[村民]","不好了！！武父他……！\\nw[村民]"]);
_tr_m.push(["いったいどうしたんだ！ \\nw[村人]","What the heck happened! \\nw[villager]","대체 어떻게 된 거야! \\nw[마을]","發生什麼事了！ \\nw[村民]","发生什么事了！\\nw[村民]"]);
_tr_m.push(["急いで村へ運べ！！ \\nw[村人]","We've got to carry him back to the village! \\nw[villager]","어서 마을로 옮겨!! \\nw[마을]","快點把他送回村子！！ \\nw[村民]","快点把他送回村子！！\\nw[村民]"]);
_tr_m.push(["はあ、はあ・・・。 \\nw[ウェポチチ]","Ack... Ack... \\nw[Dirk]","허억, 허억・・・ \\nw[웨포파파]","呼、呼…… \\nw[武器父]","呼，呼……\\nw[武器父]"]);
_tr_m.push(["\\n[1]、ぽん美・・・そこにいるのか。 \\nw[ウェポチチ]","\\n[1], Ponmi...are you there? \\nw[Dirk]","\\n[1], 폰미・・・ 거기에 있니? \\nw[웨포파파]","\\n[1]、器美……妳們在那裡嗎？ \\nw[武器父]","\\n[1]，器美……你们在那里吗？\\nw[武器父]"]);
_tr_m.push(["伝えなければ・・・最後の力を振り絞って！！ \\nw[ウェポチチ]","I've got to muster up the strength to tell them! \\nw[Dirk]","전해야 한다・・・ 마지막 힘을 쥐어짜서라도!! \\nw[웨포파파]","我必須把這件事傳達給她們……我要擠出最後的力氣！！ \\nw[武器父]","我必须把这件事传达给她们……我要挤出最后的力气！！\\nw[武器父]"]);
_tr_m.push(["\\n[1]、ぽん美。 いいか、よく聞け・・・。 \\nw[ウェポチチ]","\\n[1], Ponmi... I don't have much longer so listen carefully. \\nw[Dirk]","\\n[1], 폰미. 이 아비의 말을 잘 듣거라・・・ \\nw[웨포파파]","\\n[1]、器美。妳們仔細聽好了…… \\nw[武器父]","\\n[1]，器美。你们仔细听好了……\\nw[武器父]"]);
_tr_m.push(["どんな物でも武器にして戦う・・・それが真の武器商人だ！！ \\nw[ウェポチチ]","When you fight, arm yourself with anything and everything available to you. That's what it means to be a true weapons merchant! \\nw[Dirk]","무엇이든 무기로 삼아 싸워라・・・ 그래야 진정한 무기 상인이다!! \\nw[웨포파파]","無論什麼東西都能拿來當成武器戰鬥……這才是真正的武器商人！！ \\nw[武器父]","无论什么东西都能拿来当成武器战斗……这才是真正的武器商人！！\\nw[武器父]"]);
_tr_m.push(["ガクッ・・・。 \\nw[ウェポチチ]","Unngh... \\nw[Dirk]","꼴까닥・・・ \\nw[웨포파파]","（倒下）…… \\nw[武器父]","（倒下）……\\nw[武器父]"]);
_tr_m.push(["・・・。 \\nw[ウェポチチ]","... \\nw[Dirk]","・・・. \\nw[웨포파파]","…… \\nw[武器父]","……\\nw[武器父]"]);
_tr_m.push(["俺の言いたいことはちゃんと伝わっただろうか・・・？ \\nw[ウェポチチ]","Did they understand the meaning of those words? \\nw[Dirk]","내가 하려던 말이 제대로 전해졌을까・・・? \\nw[웨포파파]","我想說的話，有順利傳達給她們了嗎……？ \\nw[武器父]","我想说的话，顺利地传达给她们了吗……？\\nw[武器父]"]);
_tr_m.push(["もっと言うべきことがあったんじゃないか？ \\nw[ウェポチチ]","There's so much more I wanted to teach them. \\nw[Dirk]","해야 할 말이 더 있지 않았을까? \\nw[웨포파파]","是不是還有其他應該要說的話？ \\nw[武器父]","是不是还有其他应该要说的话？\\nw[武器父]"]);
_tr_m.push(["料理のしかたとか、洗濯とか。 金のやりくりのしかたとか・・・。 \\nw[ウェポチチ]","How to cook, how to clean, how to conduct business... \\nw[Dirk]","요리하는 방법이라든지, 세탁이라든지. 돈을 변통하는 방법이라든지・・・ \\nw[웨포파파]","比如說煮飯的方法、洗衣服的方法。以及該如何善用金錢…… \\nw[武器父]","比如说煮饭的方法，洗衣服的方法。以及该如何善用金钱……\\nw[武器父]"]);
_tr_m.push(["でも、ぽん美はしっかりしてるからな。 きっと大丈夫だ。 \\nw[ウェポチチ]","Ponmi's as sharp as a tack. I'm sure things'll be fine. \\nw[Dirk]","그래도 폰미는 야무진 아이니까 분명 괜찮겠지. \\nw[웨포파파]","不過，器美她很精明能幹。一定沒問題的。 \\nw[武器父]","不过，器美她很精明能干。一定没问题的。\\nw[武器父]"]);
_tr_m.push(["それに、\\n[1]は強い。 二人が力を合わせれば、乗り越えられないことなどない。 \\nw[ウェポチチ]","Besides, \\n[1] is strong. With the two of them together, there isn't anything they can't overcome. \\nw[Dirk]","그리고 \\n[1]는 강하다. 둘이 힘을 합치면 어떤 일이든지 극복할 수 있을 거야. \\nw[웨포파파]","而且，\\n[1]也很強。只要兩人同心協力，一定能度過任何難關。 \\nw[武器父]","而且，\\n[1]也很强。只要两人同心协力，一定能渡过任何难关。\\nw[武器父]"]);
_tr_m.push(["・・・。 \\nw[ウェポチチ]","... \\nw[Dirk]","・・・. \\nw[웨포파파]","…… \\nw[武器父]","……\\nw[武器父]"]);
_tr_m.push(["\\n[1]。ぽん美。 \\nw[ウェポチチ]","\\n[1], Ponmi... \\nw[Dirk]","\\n[1], 폰미. \\nw[웨포파파]","\\n[1]。器美。 \\nw[武器父]","\\n[1]。器美。\\nw[武器父]"]);
_tr_m.push(["元気でな。 \\nw[ウェポチチ]","Take care of yourselves and live a happy life. \\nw[Dirk]","잘 있거라. \\nw[웨포파파]","妳們要多保重啊。 \\nw[武器父]","你们要多保重啊。\\nw[武器父]"]);
_tr_m.push(["そなたはウェポチチの娘であろう？ \\nw[武器の神アメノオハバリ]","You are Dirk's daughter, are you not? \\nw[God of Weaponry Ameno Ohbari]","그대는 웨포파파의 딸이지? \\nw[무기의 신 아메노오하바리]","妳就是武父的女兒吧？ \\nw[武器之神 天之尾羽張]","你就是武父的女儿吧？\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["なに、神はすべてを知っておる。 \\nw[武器の神アメノオハバリ]","Surprised I know that? Well, I am an omniscient god after all. \\nw[God of Weaponry Ameno Ohbari]","놀라지 마라. 신은 모든 것을 알고 있으니. \\nw[무기의 신 아메노오하바리]","沒什麼。神知道一切啊。 \\nw[武器之神 天之尾羽張]","没什么。神知道一切啊。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["そなたは父の意思を継ぎ、そして父を超えたのだ。 \\nw[武器の神アメノオハバリ]","You have taken up your father's mantle and surpassed him. \\nw[God of Weaponry Ameno Ohbari]","그대는 아버지의 뜻을 이었고, 아버지를 뛰어넘었다. \\nw[무기의 신 아메노오하바리]","妳繼承了父親的遺志，並超越了妳的父親。 \\nw[武器之神 天之尾羽張]","你继承了父亲的遗志，并超越了你的父亲。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["さあ。 \\nw[武器の神アメノオハバリ]","Hark! \\nw[God of Weaponry Ameno Ohbari]","자. \\nw[무기의 신 아메노오하바리]","來吧。 \\nw[武器之神 天之尾羽張]","来吧。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["そなたは今、伝説の武器 ツヨスギテ＝クサハエル・ソードを手にする資格を得た。 \\nw[武器の神アメノオハバリ]","You have earned the right to the legendary weapon, the LOL Sword. \\nw[God of Weaponry Ameno Ohbari]","그대는 지금 전설의 무기, 너무강해서 현웃터지는 소드를 쥘 자격을 얻었다. \\nw[무기의 신 아메노오하바리]","妳現在有資格能獲得傳說武器「暴強‧草叢生劍」了。 \\nw[武器之神 天之尾羽張]","你现在有资格能获得传说武器「无敌‧草丛生剑」了。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["武器を極めし者よ。その宝箱を空けるがよい。 \\nw[武器の神アメノオハバリ]","Oh Master of Weapons, open the chest you see there before you. \\nw[God of Weaponry Ameno Ohbari]","무기의 극한을 깨우친 자여. 그 보물상자를 열어도 좋다. \\nw[무기의 신 아메노오하바리]","窮究武器之人啊，打開那個寶箱吧。 \\nw[武器之神 天之尾羽張]","穷究武器之人啊，打开那个宝箱吧。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["それは我が創った、最強の剣。 \\nw[武器の神アメノオハバリ]","You will find the most powerful blade I've ever forged. \\nw[God of Weaponry Ameno Ohbari]","그것은 내가 만든 최강의 검. \\nw[무기의 신 아메노오하바리]","那就是我所創造的最強之劍。 \\nw[武器之神 天之尾羽張]","那就是我所创造的最强之剑。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["その武器をどうするかはそなたの自由だ。好きにするがよい。 \\nw[武器の神アメノオハバリ]","Do with it as you please, for it now belongs to you. \\nw[God of Weaponry Ameno Ohbari]","그 무기를 어찌할지는 그대에게 달렸다. 마음대로 하거라. \\nw[무기의 신 아메노오하바리]","要怎麼處理那把武器是妳的自由。就隨妳的意思去做吧。 \\nw[武器之神 天之尾羽張]","要怎么处理那把武器是你的自由。就随你的意思去做吧。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["さらばだ、武器を極めし者よ・・・。 \\nw[武器の神アメノオハバリ]","Farewell, Master of Weapons... \\nw[God of Weaponry Ameno Ohbari]","잘 가거라, 무기의 극한을 깨우친 자여・・・ \\nw[무기의 신 아메노오하바리]","再見了，窮究武器之人…… \\nw[武器之神 天之尾羽張]","再见了，穷究武器之人……\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["かくして、神との戦いを終え ついに伝説の武器を手に入れた\\n[1]。","Thus, \\n[1] fought a divine being and obtained the legendary weapon.","이리하여 신과 싸움을 마치고 드디어 전설의 무기를 손에 넣은 \\n[1].","就這樣，與神之間的戰鬥結束，\\n[1]終於拿到了傳說武器。","就这样，与神之间的战斗结束，\\n[1]终于拿到了传说武器。"]);
_tr_m.push(["「とりあえず、疲れたので家に帰ろう。」 \\n[1]はそう思った。","\\n[1] thought, \"I'm exhausted. I just want to go home.\"","「피곤하니까 일단 집에 가자.」 \\n[1]는 그렇게 생각했다.","\\n[1]心想：「總之我累了，先回家吧。」","\\n[1]心想：「总之，我累了，先回家吧。」"]);
_tr_m.push(["ヘイワ村のぽん美に話しかけよう！","Talk to Ponmi in Peaceville!","평화 마을의 폰미에게 말을 걸자!","跟和平村的器美說話吧！","跟和平村的器美说话吧！"]);
_tr_m.push(["伝説の武器が眠る洞窟内の武器を すべて拾えるようになった！","All weapons in the Legendary Weapon Cave can now be picked up!","전설의 무기가 잠든 동굴 안의 무기를 전부 주울 수 있게 되었다!","傳說武器沉眠的洞窟內所有的武器都可以拾取了！","传说武器沉眠的洞窟内所有的武器都可以拾取了！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["そなたは今、伝説の武器 ツヨスギテ＝クサハエル・ソードを手にする資格を得た。 \\nw[武器の神アメノオハバリ]","You have earned the right to the legendary weapon, the LOL Sword.\\nw[God of Weaponry Ameno Ohbari]","그대는 지금 전설의 무기, 너무강해서 현웃터지는 소드를 쥘 자격을 얻었다. \\nw[무기의 신 아메노오하바리]","妳現在有資格能獲得傳說武器「暴強‧草叢生劍」了。 \\nw[武器之神 天之尾羽張]","你现在有资格能获得传说武器「暴强‧草丛生剑」了。 \\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["武器を極めし者よ。その宝箱を空けるがよい。 \\nw[武器の神アメノオハバリ]","Oh Master of Weapons, open the chest you see there.\\nw[God of Weaponry Ameno Ohbari]","무기의 극한을 깨우친 자여. 그 보물상자를 열어도 좋다. \\nw[무기의 신 아메노오하바리]","窮究武器之人啊，打開那個寶箱吧。 \\nw[武器之神 天之尾羽張]","穷究武器之人啊，打开那个宝箱吧。 \\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["また会ったな。武器を極めし者よ。 \\nw[武器の神アメノオハバリ]","We meet again, Master of Weapons... \\nw[God of Weaponry Ameno Ohbari]","또 만났구나. 무기의 극한을 깨우친 자여. \\nw[무기의 신 아메노오하바리]","又見到妳了呢，窮究武器之人。 \\nw[武器之神 天之尾羽張]","又见到你了呢，穷究武器之人。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["ここまで来るということは・・・。 己の強さをもう一度試したいと申すのか？\\!  \\nw[武器の神アメノオハバリ]","Shall I assume you would like to test how strong you've become?\\!  \\nw[God of Weaponry Ameno Ohbari]","여기까지 찾아왔다는 건・・・ 자신의 실력을 다시 시험하고 싶어서인가?\\!  \\nw[무기의 신 아메노오하바리]","妳會來這裡……代表妳想要再測試一次自己有多強嗎？\\!  \\nw[武器之神 天之尾羽張]","你来这里……是想要再测试一次自己有多强吗？\\! \\nw[武器之神 天之尾羽张]"]);
_tr_m.push([">はい",">Yes",">예","＞是","＞是"]);
_tr_m.push(["いいだろう・・・！ \\nw[武器の神アメノオハバリ]","Well said! \\nw[God of Weaponry Ameno Ohbari]","좋다・・・! \\nw[무기의 신 아메노오하바리]","好……！ \\nw[武器之神 天之尾羽張]","好……！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["そなたの力！ ここに示してみせよ！！ \\nw[武器の神アメノオハバリ]","Then show me! Let us see just how powerful you are! \\nw[God of Weaponry Ameno Ohbari]","그대의 힘을 증명해 보아라!! \\nw[무기의 신 아메노오하바리]","就在這裡展現妳的力量吧！！ \\nw[武器之神 天之尾羽張]","就在这里展现你的力量吧！！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["\\{\\{武器の神アメノオハバリ\\|\\|\\|\\^","\\{\\{God of Weaponry Ameno Ohbari\\|\\|\\|\\^","\\>\\{\\{\\<무기의 신 아메노오하바리\\|\\|\\|\\^","\\{\\{武器之神 天之尾羽張\\|\\|\\|\\^","\\{\\{武器之神 天之尾羽张\\|\\|\\|\\^"]);
_tr_m.push([">いいえ",">No",">아니요","＞否","＞否"]);
_tr_m.push(["そうか。 \\nw[武器の神アメノオハバリ]","Very well. \\nw[God of Weaponry Ameno Ohbari]","그렇군. \\nw[무기의 신 아메노오하바리]","這樣啊。 \\nw[武器之神 天之尾羽張]","这样啊。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["・・・そなたは父の教えを忠実に守り、 本当になんでも武器にしてしまう。 \\nw[武器の神アメノオハバリ]","I see you have heeded your father's words and can use anything as a weapon. \\nw[God of Weaponry Ameno Ohbari]","・・・그대는 아버지의 가르침을 충실히 따랐고 무엇이든지 무기로 삼을 수 있다. \\nw[무기의 신 아메노오하바리]","……妳忠實遵守父親的教誨，真的把任何東西都當成武器。 \\nw[武器之神 天之尾羽張]","……你谨遵父亲的教诲，真的把任何东西都当成了武器。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["武器の神である我すら驚くほどだ。 \\nw[武器の神アメノオハバリ]","You have surprised even me, the god of weaponry. \\nw[God of Weaponry Ameno Ohbari]","무기의 신인 나마저 놀랄 정도다. \\nw[무기의 신 아메노오하바리]","甚至連我這個武器之神都大吃一驚。 \\nw[武器之神 天之尾羽張]","甚至连我这个武器之神都大吃一惊。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["もしもそなたが、己の限界に挑戦したいと申すのなら・・・。 \\nw[武器の神アメノオハバリ]","If you truly wish to test your limits... \\nw[God of Weaponry Ameno Ohbari]","만약 그대가 스스로의 한계에 도전하고자 한다면・・・ \\nw[무기의 신 아메노오하바리]","如果妳想要挑戰自己的極限…… \\nw[武器之神 天之尾羽張]","如果你想要挑战自己的极限……\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["\\n[1]よ。 すべての武器を集めてみてはどうだろうか。 \\nw[武器の神アメノオハバリ]","\\n[1], try to collect all the weapons in the world! \\nw[God of Weaponry Ameno Ohbari]","\\n[1]여. 모든 무기를 모아 보면 어떻겠느냐. \\nw[무기의 신 아메노오하바리]","\\n[1]啊，要不要試著收集所有的武器呢？ \\nw[武器之神 天之尾羽張]","\\n[1]啊，要不要试着收集所有的武器呢？\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["それは圧倒的に無謀な挑戦だ。 \\nw[武器の神アメノオハバリ]","This undertaking may just become your undoing. \\nw[God of Weaponry Ameno Ohbari]","그것은 압도적이며 무모한 도전이니라. \\nw[무기의 신 아메노오하바리]","這是一個非常有勇無謀的挑戰。 \\nw[武器之神 天之尾羽張]","这是一个非常有勇无谋的挑战。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["しかし、武器を司る武器の神として・・・ 武器というものの「可能性」を見てみたいのだ。 \\nw[武器の神アメノオハバリ]","However, as the god of weaponry who governs all weapons, I wish to see their potential. \\nw[God of Weaponry Ameno Ohbari]","하지만 무기를 관장하는 무기의 신으로서・・・ 무기의 「가능성」이 어디까지인지 보고 싶구나. \\nw[무기의 신 아메노오하바리]","不過，身為掌管武器的武器之神……我想要見識看看所謂武器的「可能性」。 \\nw[武器之神 天之尾羽張]","不过，身为掌管武器的武器之神……我想要见识看看所谓武器的「可能性」。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["お姉ちゃん！本当に・・・手に入れたんだね！ \\nw[ぽん美]","Weaco! You really got it! \\nw[Ponmi]","언니! 정말로・・・ 찾아냈구나! \\nw[폰미]","姐姐！妳真的……真的拿到了呢！ \\nw[器美]","姐姐！你真的……真的拿到了呢！\\nw[器美]"]);
_tr_m.push(["伝説の武器、ツヨスギテ＝クサハエル・ソード！！ \\nw[ぽん美]","The legendary weapon, the LOL Sword! \\nw[Ponmi]","전설의 무기, 너무강해서 현웃터지는 소드!! \\nw[폰미]","傳說中的武器「暴強‧草叢生劍」！！ \\nw[器美]","传说中的武器「无敌‧草丛生剑」！！\\nw[器美]"]);
_tr_m.push(["わーい！かっこいいー！ \\nw[ぽん美]","Wow! It looks awesome! \\nw[Ponmi]","우와~! 멋지다~! \\nw[폰미]","哇──！好帥喔──！ \\nw[器美]","哇──！好帅喔──！\\nw[器美]"]);
_tr_m.push(["・・・。 \\nw[ぽん美]","... \\nw[Ponmi]","・・・. \\nw[폰미]","…… \\nw[器美]","……\\nw[器美]"]);
_tr_m.push(["お父さんの願い、叶えてあげられたんだね。 お父さん、見てるかな。 \\nw[ぽん美]","You fulfilled dad's dream. I'm sure he's so proud of you. \\nw[Ponmi]","아빠의 소원을 언니가 이뤘네. 아빠도 보고 있을까? \\nw[폰미]","妳實現了爸爸的心願。不知道爸爸是否在看著呢？ \\nw[器美]","你实现了爸爸的心愿。不知道爸爸是否在看着呢？\\nw[器美]"]);
_tr_m.push(["もう一度お父さんに会いたいな・・・。 \\nw[ぽん美]","I wish he were here right now... \\nw[Ponmi]","아빠가 보고 싶어・・・ \\nw[폰미]","真想再見爸爸一面…… \\nw[器美]","真想再见爸爸一面……\\nw[器美]"]);
_tr_m.push(["そのとき！","It was then...","그때였다!","就在此時！","就在此时！"]);
_tr_m.push(["ツヨスギテ＝クサハエルの剣先から 眩い光が放たれた！","A bright light shone from the tip of the LOL Sword!","너무강해서 현웃터지는 소드가 눈부신 빛을 뿜었다!","暴強‧草叢生劍的劍尖發出了耀眼的光芒！","无敌‧草丛生剑的剑尖发出了耀眼的光芒！"]);
_tr_m.push(["！？！？ \\nw[ぽん美]","!?!? \\nw[Ponmi]","!?!? \\nw[폰미]","！？！？ \\nw[器美]","！？！？\\nw[器美]"]);
_tr_m.push(["お・・・。 \\nw[ぽん美]","Da... \\nw[Ponmi]","아・・・ \\nw[폰미]","爸…… \\nw[器美]","爸……\\nw[器美]"]);
_tr_m.push(["お父さん！！ \\nw[ぽん美]","Daaad!!! \\nw[Ponmi]","아빠!! \\nw[폰미]","爸爸！！ \\nw[器美]","爸爸！！\\nw[器美]"]);
_tr_m.push(["そこには・・・死んでしまったはずの 父ウェポチチの姿があった。","It was Dirk, their father who passed away.","그곳에・・・ 세상을 떠났어야 할 웨포파파의 모습이 나타났다.","出現在那裡的……是她們早已死去的爸爸，武父。","出现在那里的……是她们早已死去的爸爸，武父。"]);
_tr_m.push(["娘たちよ。・・・久しぶりだな。 \\nw[ウェポチチ]","Hello, girls. It's been too long. \\nw[Dirk]","딸들아・・・ 오랜만이구나. \\nw[웨포파파]","女兒們……好久不見了。 \\nw[武器父]","女儿们……好久不见了。\\nw[武器父]"]);
_tr_m.push(["お父さんっ！！ \\nw[ぽん美]","Daaad! \\nw[Ponmi]","아빠!! \\nw[폰미]","爸爸！！ \\nw[器美]","爸爸！！\\nw[器美]"]);
_tr_m.push(["わはは。ぽん美。 お前の泣き虫なところは変わらないな。 \\nw[ウェポチチ]","Hahaha. Oh Ponmi, I see you haven't changed much. \\nw[Dirk]","하하하, 폰미. 여전히 울보구나. \\nw[웨포파파]","哇哈哈。器美。妳還是一樣很愛哭呢。 \\nw[武器父]","哇哈哈。器美。你还是一样很爱哭呢。\\nw[武器父]"]);
_tr_m.push(["でも、こんなに大きくなって。 ずっとウチを支えてくれたんだよな。 \\nw[ウェポチチ]","Although, look at how big you've become! You've always helped keep this family together. \\nw[Dirk]","언제 이렇게 컸니. 그동안 집안을 돌보느라 고생했다. \\nw[웨포파파]","不過，妳已經長得這麼大了啊。妳一直在支撐著這個家呢。 \\nw[武器父]","不过，你已经长得这么大了啊。你一直在支撑着这个家呢。\\nw[武器父]"]);
_tr_m.push(["ありがとうな。 \\nw[ウェポチチ]","Thank you. \\nw[Dirk]","고맙다. \\nw[웨포파파]","謝謝妳啊。 \\nw[武器父]","谢谢你啊。\\nw[武器父]"]);
_tr_m.push(["うう・・・。 \\nw[ぽん美]","*sniff* \\nw[Ponmi]","훌쩍・・・ \\nw[폰미]","嗚嗚…… \\nw[器美]","呜呜……\\nw[器美]"]);
_tr_m.push(["\\n[1]。 随分と武器の腕を上げたようだな。見ればわかるさ。 \\nw[ウェポチチ]","\\n[1], I can see that your proficiency with weapons has greatly improved. \\nw[Dirk]","\\n[1], 무기 다루는 실력이 많이 늘었나 보구나. 딱 보니 알겠다. \\nw[웨포파파]","\\n[1]。妳使用武器的功力提升了不少呢。我一看就知道了。 \\nw[武器父]","\\n[1]。你使用武器的功力提升了不少呢。我一看就知道了。\\nw[武器父]"]);
_tr_m.push(["そして、その手に握っている剣。 それが・・・伝説の武器なのだな？ \\nw[ウェポチチ]","And that sword you're holding... That must be the legendary weapon. \\nw[Dirk]","그리고 손에 쥔 그 검. 그것이・・・ 전설의 무기 맞지? \\nw[웨포파파]","然後，妳手上握著的劍……那就是傳說武器吧？ \\nw[武器父]","然后，你手上握着的剑……那就是传说中的武器吧？\\nw[武器父]"]);
_tr_m.push(["俺の果たせなかった夢。お前たちが叶えてくれたんだな。 \\nw[ウェポチチ]","My dream that I was unable to realize... You've fulfilled it for me. \\nw[Dirk]","내가 이루지 못한 꿈을 너희가 이뤄 주었어. \\nw[웨포파파]","我沒能達成的夢想，妳們替我實現了呢。 \\nw[武器父]","我没能达成的梦想，你们替我实现了呢。\\nw[武器父]"]);
_tr_m.push(["ありがとう。 \\nw[ウェポチチ]","Thank you. \\nw[Dirk]","고맙다. \\nw[웨포파파]","謝謝啊。 \\nw[武器父]","谢谢。\\nw[武器父]"]);
_tr_m.push(["お父さん！！ \\nw[ぽん美]","D-Dad!\\nw[Ponmi]","아빠!! \\nw[폰미]","爸爸！！ \\nw[器美]","爸爸！！\\nw[器美]"]);
_tr_m.push(["私たち、ずっと寂しくて・・・。 \\nw[ぽん美]","We've missed you so much... \\nw[Ponmi]","우리 그동안 외로웠어・・・ \\nw[폰미]","我們一直都感到很寂寞…… \\nw[器美]","我们一直都感到很寂寞……\\nw[器美]"]);
_tr_m.push(["もう一度３人で一緒に・・・！！ \\nw[ぽん美]","Finally, we'll be a family again... \\nw[Ponmi]","다시 셋이서 함께・・・!! \\nw[폰미]","讓我們三個人再次一起生活吧……！！ \\nw[器美]","让我们三个人再次一起生活吧……！！\\nw[器美]"]);
_tr_m.push(["・・・ごめんな。 俺もできることならそうしたいが・・・。 \\nw[ウェポチチ]","As much as I want that for us, I'm afraid it can't happen. \\nw[Dirk]","・・・미안하구나. 나도 할 수만 있다면 그러고 싶지만・・・ \\nw[웨포파파]","……抱歉。如果可以的話，我也想這麼做，但…… \\nw[武器父]","……抱歉。如果可以的话，我也想这么做，但……\\nw[武器父]"]);
_tr_m.push(["・・・俺は父親失格だ。 \\nw[ウェポチチ]","I'm a failure of a father. \\nw[Dirk]","・・・난 아버지로서 실격이야. \\nw[웨포파파]","……我不配當一個父親。 \\nw[武器父]","……我不配当一个父亲。\\nw[武器父]"]);
_tr_m.push(["無謀な冒険に中途半端な覚悟で挑み、 かわいい娘二人を残して一人逝ってしまった。 \\nw[ウェポチチ]","I half-heartedly took on an impossible challenge and kicked the bucket, leaving my two precious daughters behind. \\nw[Dirk]","무모한 모험에 어중간한 각오로 도전해서 귀여운 두 딸만 남기고 혼자 세상을 떠났지. \\nw[웨포파파]","我抱持著半弔子的覺悟，挑戰有勇無謀的冒險，留下兩個可愛的女兒，一個人先離開了這個世界。 \\nw[武器父]","我抱持着半吊子的觉悟，挑战有勇无谋的冒险，留下两个可爱的女儿，一个人先离开了这个世界。\\nw[武器父]"]);
_tr_m.push(["しかし、そんな情けない父親の無念をお前たちは晴らしてくれた。 \\nw[ウェポチチ]","But seeing you two all grown up, you've assuaged my final regret. \\nw[Dirk]","하지만 그런 한심한 아비의 원한을 너희가 풀어 주었다. \\nw[웨포파파]","然而，妳們卻實現了我這個沒用的父親的願望。 \\nw[武器父]","然而，你们却替我这个没用的父亲实现了愿望。\\nw[武器父]"]);
_tr_m.push(["そしてこうやって再会を喜んでくれた。 \\nw[ウェポチチ]","And to know that you are simply glad to see me again... \\nw[Dirk]","그리고 이렇게 재회를 기뻐해 주는구나. \\nw[웨포파파]","而且還對這次的重逢感到很開心。 \\nw[武器父]","而且还对这次的重逢感到很开心。\\nw[武器父]"]);
_tr_m.push(["こんなに嬉しいことはない。 \\nw[ウェポチチ]","I could not know a greater happiness. \\nw[Dirk]","이보다 더 기쁜 일이 어디 있겠니. \\nw[웨포파파]","沒有比這更令我高興的事了。 \\nw[武器父]","没有比这更令我高兴的事了。\\nw[武器父]"]);
_tr_m.push(["心の底からお前たちの父親でよかったよ。 \\nw[ウェポチチ]","I'm proud to be your father. \\nw[Dirk]","너희의 아빠라서 진심으로 감사하단다. \\nw[웨포파파]","我打從心底覺得，能成為妳們的父親真是太好了。 \\nw[武器父]","我打从心底觉得，能成为你们的父亲真是太好了。\\nw[武器父]"]);
_tr_m.push(["ありがとう。 \\nw[ウェポチチ]","Thank you.\\nw[Dirk]","고맙다. \\nw[웨포파파]","謝謝。 \\nw[武器父]","谢谢。\\nw[武器父]"]);
_tr_m.push(["おﾞ父さﾞんﾞ～～～～！！！ \\nw[ぽん美]","Daaad!!! \\nw[Ponmi]","아빠~~~~!!! \\nw[폰미]","爸爸～～～～！！！ \\nw[器美]","爸爸～～～～！！！\\nw[器美]"]);
_tr_m.push(["・・・じゃあな。俺はそろそろ行くよ。 \\nw[ウェポチチ]","It looks like this is good-bye. I must be going. \\nw[Dirk]","・・・그럼 난 이만 가야겠구나. \\nw[웨포파파]","……再見。我差不多該走了。 \\nw[武器父]","……再见。我差不多该走了。\\nw[武器父]"]);
_tr_m.push(["大丈夫だ。お前たちは俺を超えたんだ。 どんなことがあっても、乗り越えていけるさ。 \\nw[ウェポチチ]","You've done what I could not. I'm sure you two'll overcome any obstacle that stands in your way. \\nw[Dirk]","괜찮아. 너희는 나를 뛰어넘었어. 어떤 일이든지 극복할 수 있을 거다. \\nw[웨포파파]","沒問題的。妳們已經超越我了。無論發生什麼事，絕對都能度過難關。 \\nw[武器父]","没问题的。你们已经超越我了。无论发生什么事，绝对都能渡过难关。\\nw[武器父]"]);
_tr_m.push(["・・・。 \\nw[ウェポチチ]","... \\nw[Dirk]","・・・. \\nw[웨포파파]","…… \\nw[武器父]","……\\nw[武器父]"]);
_tr_m.push(["\\n[1]、ぽん美。 \\nw[ウェポチチ]","\\n[1]... Ponmi... \\nw[Dirk]","\\n[1], 폰미. \\nw[웨포파파]","\\n[1]，器美。 \\nw[武器父]","\\n[1]，器美。\\nw[武器父]"]);
_tr_m.push(["どんな物でも武器にして戦う。それが真の武器商人だ！！ \\nw[ウェポチチ]","When you fight, arm yourself with anything and everything available to you. That's what it means to be a true weapons merchant! \\nw[Dirk]","무엇이든 무기로 삼아 싸워라. 그래야 진정한 무기 상인이다!! \\nw[웨포파파]","無論什麼東西都能拿來當成武器戰鬥。這才是真正的武器商人！！ \\nw[武器父]","无论什么东西都能拿来当成武器战斗。这才是真正的武器商人！！\\nw[武器父]"]);
_tr_m.push(["伝説の武器を手に入れた姿を、 父に見せることができた二人。","The two girls were able to show their father the legendary weapon.","전설의 무기를 쥔 모습을 아버지에게 보여 드린 두 사람.","讓父親看到自己拿到了傳說武器的兩個女孩。","让父亲看到自己拿到了传说武器的两个女孩。"]);
_tr_m.push(["父からの言葉を胸に、 二人はまた自分の足で歩いていく。","They decided to move forward, cherishing their father's words with pride.","아버지의 말을 가슴에 새기고 자매는 다시 스스로 걸어간다.","她們謹記著父親留下的話語，繼續靠著自己的雙腳邁步前行。","她们谨记着父亲留下的话语，继续靠着自己的双脚迈步前行。"]);
_tr_m.push(["父の死を乗り越え、 その強さを武器にして―――。","They learned to accept their father's death and armed themselves with that strength.","아버지의 죽음을 뛰어넘고, 그 강한 힘을 무기로 삼아・・・","跨越父親的死亡，把這份堅強化為武器───","跨越父亲的死亡，把这份坚强化为武器───"]);
_tr_m.push(["お姉ちゃん。 \\nw[ぽん美]","Weaco... \\nw[Ponmi]","언니. \\nw[폰미]","姐姐。 \\nw[器美]","姐姐。\\nw[器美]"]);
_tr_m.push(["私たち二人なら、なんだってできる気がするよ。 \\nw[ぽん美]","I bet the two of us could accomplish anything. \\nw[Ponmi]","우리 둘이 함께라면 뭐든지 할 수 있을 것 같아. \\nw[폰미]","只要我們兩個在一起，感覺什麼事情都辦得到呢。 \\nw[器美]","只要我们两个在一起，感觉什么事情都办得到呢。\\nw[器美]"]);
_tr_m.push(["なんたって、お父さんの子供なんだからね。 \\nw[ぽん美]","I mean, come on, we ARE Dirk's daughters after all! \\nw[Ponmi]","우린 아빠의 딸이니까. \\nw[폰미]","因為我們是爸爸的孩子啊。 \\nw[器美]","因为我们是爸爸的孩子啊。\\nw[器美]"]);
_tr_m.push(["END 1/4  「願い」","ENDING 1/4 [Dream]","END 1/4  「소원」","END 1/4 「心願」","END 1/4「心愿」"]);
_tr_m.push(["お姉ちゃん！本当に・・・手に入れたんだね！ \\nw[ぽん美]","Oh! You really got it!\\nw[Ponmi]","언니! 정말로・・・ 찾아냈구나! \\nw[폰미]","姐姐！妳真的……真的拿到了呢！ \\nw[器美]","姐姐！你真的……真的拿到了呢！ \\nw[器美]"]);
_tr_m.push(["伝説の武器、ツヨスギテ＝クサハエル・ソード！！ \\nw[ぽん美]","The legendary weapon, the LOL Sword! \\nw[Ponmi]","전설의 무기, 너무강해서 현웃터지는 소드!! \\nw[폰미]","傳說中的武器「暴強‧草叢生劍」！！ \\nw[器美]","传说中的武器「暴强‧草丛生剑」！！ \\nw[器美]"]);
_tr_m.push(["わーい！かっこいいー！ \\nw[ぽん美]","Phew! It looks awesome!\\nw[Ponmi]","우와~! 멋지다~! \\nw[폰미]","哇──！好帥喔──！ \\nw[器美]","哇──！好帅喔──！ \\nw[器美]"]);
_tr_m.push(["ヒュルルル・・・。","Peeeew... *thud*","휘이익・・・","咻嚕嚕嚕……","咻噜噜噜……"]);
_tr_m.push(["\\{\\{ほげーーーっ！！ \\nw[？？？？？]","\\{\\{Oof!!! \\nw[?????]","\\{\\{으아아아아악!! \\nw[?????]","\\{\\{咕哇───！！ \\nw[？？？？？]","\\{\\{咕哇───！！\\nw[？？？？？]"]);
_tr_m.push(["！？ \\nw[ぽん美]","!? \\nw[Ponmi]","!? \\nw[폰미]","！？ \\nw[器美]","！？\\nw[器美]"]);
_tr_m.push(["なんか外から音が！！ \\nw[ぽん美]","That sound came from outside! \\nw[Ponmi]","밖에서 나는 소리야!! \\nw[폰미]","外面好像傳來什麼聲音！！ \\nw[器美]","外面好像传来什么声音！！\\nw[器美]"]);
_tr_m.push(["・・・何これ？ \\nw[ぽん美]","What's going on? \\nw[Ponmi]","・・・이게 뭐지? \\nw[폰미]","……這是什麼？ \\nw[器美]","……这是什么？\\nw[器美]"]);
_tr_m.push(["・・・。 \\nw[勇者シュヴァルディ]","... \\nw[Valiant Hero Schivardi]","・・・. \\nw[용사 슈발디]","…… \\nw[勇者修瓦爾迪]","……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["あ！勇者じゃん！ \\nw[ぽん美]","Why if it isn't mister hero! \\nw[Ponmi]","앗! 용사잖아? \\nw[폰미]","啊！這不是勇者嗎！？ \\nw[器美]","啊！这不是勇者吗！？\\nw[器美]"]);
_tr_m.push(["よくうちの看板前につっ立ってる 勇者のお兄ちゃんじゃん！！ \\nw[ぽん美]","That guy that likes to loiter in front of our shop's sign! \\nw[Ponmi]","종종 우리 가게 간판 앞에 서 있는 용사 오빠네!! \\nw[폰미]","是常常站在我們家招牌面前的勇者大哥嘛！ \\nw[器美]","是常常站在我们家招牌面前的勇者大哥嘛！\\nw[器美]"]);
_tr_m.push(["どうしたの！？ \\nw[ぽん美]","What happened? \\nw[Ponmi]","무슨 일이야!? \\nw[폰미]","怎麼了嗎！？ \\nw[器美]","这是怎么了！？\\nw[器美]"]);
_tr_m.push(["・・・・・。 \\nw[勇者シュヴァルディ]","・・... \\nw[Valiant Hero Schivardi]","・・・・・. \\nw[용사 슈발디]","・・…… \\nw[勇者修瓦爾迪]","……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["魔王に負けた・・・。 \\nw[勇者シュヴァルディ]","I...lost to the Demon Lord. \\nw[Valiant Hero Schivardi]","마왕에게 졌어・・・ \\nw[용사 슈발디]","我輸給魔王了…… \\nw[勇者修瓦爾迪]","我输给魔王了……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["負けてぶっ飛ばされてきた・・・。 \\nw[勇者シュヴァルディ]","He sent me flying... \\nw[Valiant Hero Schivardi]","패배한 나를 여기로 날려 보냈어・・・ \\nw[용사 슈발디]","輸給魔王後，被打飛到這裡來…… \\nw[勇者修瓦爾迪]","输给魔王后，被打飞到这里来……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["何アレ・・・あんなの無理じゃん・・・。 \\nw[勇者シュヴァルディ]","How can anyone defeat such a monster? \\nw[Valiant Hero Schivardi]","대체 뭐야・・・ 그런 놈이랑 어떻게 싸워・・・ \\nw[용사 슈발디]","那是怎樣……那麼強我不可能打得贏啊…… \\nw[勇者修瓦爾迪]","怎么回事……那么强我不可能打得赢啊……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["勝てるわけないじゃん！！ 絶対バランス調整ミスじゃん！！ \\nw[勇者シュヴァルディ]","It's pretty much impossible! Someone really messed up the game balance! \\nw[Valiant Hero Schivardi]","절대로 이길 수 없다고!! 분명히 이건 밸런스 조절을 잘못한 거야!! \\nw[용사 슈발디]","怎麼可能打得贏！！一定是平衡度調整有問題啦！！ \\nw[勇者修瓦爾迪]","怎么可能打得赢！！一定是平衡度调整有问题啦！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["ちょっと！勇者のお兄ちゃん！発言がメタいよ！！ \\nw[ぽん美]","Whoa there, mister hero. That's too meta! \\nw[Ponmi]","잠깐! 용사 오빠! 그거 누구한테 하는 소리야!? \\nw[폰미]","等一下！勇者大哥！你這是後設發言啊！！ \\nw[器美]","等一下！勇者大哥！不要自己组织对话啊！！\\nw[器美]"]);
_tr_m.push(["くそっ！俺はなんか悔しいぞ！！ \\nw[勇者シュヴァルディ]","Dammit! This is really getting to me! \\nw[Valiant Hero Schivardi]","젠장! 분하고 원통하다!! \\nw[용사 슈발디]","可惡！我感覺超不甘心的！！ \\nw[勇者修瓦爾迪]","可恶！我非常不甘心啊！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["魔王討伐なんてダルいと思ってたが・・・ 実際ボロ負けしてみると悔しいぞ！ \\nw[勇者シュヴァルディ]","Taking down the Demon Lord seemed like a boring quest, but after being one-shotted...it's got me triggered! \\nw[Valiant Hero Schivardi]","마왕 토벌 따위는 시시할 줄 알았는데・・・ 실제로는 엉망진창으로 졌어! \\nw[용사 슈발디]","我原本覺得要去討伐魔王很煩……不過實際上慘敗之後，真的很不甘心！ \\nw[勇者修瓦爾迪]","我原本觉得，去讨伐魔王是件很烦人的事……不过实际上惨败之后，真的很不甘心！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["もっとこう・・・アレだ！ 仲間を集めて、レベル上げしないと！！ \\nw[勇者シュヴァルディ]","I need to...I know! Get a party together and grind some levels! \\nw[Valiant Hero Schivardi]","좀 더 뭐랄까・・・ 그래! 동료를 모아서 레벨을 올려야 해!! \\nw[용사 슈발디]","應該是要……就是那樣啊！去募集伙伴，再想辦法提升等級！！ \\nw[勇者修瓦爾迪]","应该是要……就是那样啊！去募集伙伴，再想办法提升等级！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["よっしゃ！！なんかやる気が湧いてきた！！ \\nw[勇者シュヴァルディ]","Alright! Now I'm feeling up to the task! \\nw[Valiant Hero Schivardi]","좋았어!! 갑자기 의욕이 끓어오른다!! \\nw[용사 슈발디]","好！！我感覺到幹勁湧現了！！ \\nw[勇者修瓦爾迪]","好！！我感觉浑身都充满了干劲！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["・・・なんだかんだで、\\n[1]ちゃんのお陰だな。 キミのお陰でやる気が出たよ。ありがとな！！ \\nw[勇者シュヴァルディ]","I think I owe it all to you, \\n[1]. You've got me pumped up and ready for action! \\nw[Valiant Hero Schivardi]","・・・어쨌든 다 \\n[1] 덕분이네. 네 덕분에 의욕이 생겼어. 고마워!! \\nw[용사 슈발디]","……說起來，這都是多虧了\\n[1]的鼓勵。多虧有妳，我才能拿出幹勁。謝啦！！ \\nw[勇者修瓦爾迪]","……说起来，这都是多亏了\\n[1]的鼓励。多亏有你，我才能拿出干劲。谢啦！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["では、行ってくるよ！ \\nw[勇者シュヴァルディ]","Well then, I'm off! \\nw[Valiant Hero Schivardi]","그럼 다녀올게! \\nw[용사 슈발디]","那麼，我要出發了！ \\nw[勇者修瓦爾迪]","那么，我要出发了！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["あ、ちょっとまってよ！ \\nw[ぽん美]","Hey, wait up! \\nw[Ponmi]","아, 잠깐만! \\nw[폰미]","啊，等一下！ \\nw[器美]","啊，等一下！\\nw[器美]"]);
_tr_m.push(["お姉ちゃんのお陰でやる気が出たなら、 お礼があってもいいんじゃない？ \\nw[ぽん美]","If you are truly thankful to my sister for motivating you, then why not give us something in return? \\nw[Ponmi]","언니 덕분에 의욕이 생겼다면 뭔가 보답을 해도 좋지 않을까? \\nw[폰미]","你說你是多虧了姐姐才拿出幹勁的，那是不是該給她一點謝禮？ \\nw[器美]","你说你是多亏了姐姐才拿出干劲的，那是不是该给她一点谢礼？\\nw[器美]"]);
_tr_m.push(["例えば・・・勇者のお兄ちゃん、勇者だし知名度高いでしょ。 ウチの武器屋の宣伝とかしてくれてもいいんだよ。 \\nw[ぽん美]","Let's see... You are a warrior renowned throughout the land. You could promote our shop to everyone you know! \\nw[Ponmi]","예를 들면・・・ 용사 오빠는 용사니까 유명하겠지? 우리 무기점을 홍보해 주면 좋겠는데. \\nw[폰미]","比如說……勇者大哥，你畢竟是個勇者，也很有名氣對吧？可以替我們的武器店做點宣傳啊。 \\nw[器美]","比如说……勇者大哥，你毕竟是个勇者，也很有名气对吧？可以替我们的武器店做点宣传啊。\\nw[器美]"]);
_tr_m.push(["・・・。 \\nw[勇者シュヴァルディ]","... \\nw[Valiant Hero Schivardi]","・・・. \\nw[용사 슈발디]","…… \\nw[勇者修瓦爾迪]","……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["（なんだこの打算的な少女は・・・。） \\nw[勇者シュヴァルディ]","(What a shrewd young girl...) \\nw[Valiant Hero Schivardi]","(뭐지, 이 약삭빠른 소녀는・・・) \\nw[용사 슈발디]","（這個愛計較的少女是怎麼回事……） \\nw[勇者修瓦爾迪]","（这个爱计较的少女是怎么回事……）\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["まあいいよ。 最初、そんな約束した気もするし。 \\nw[勇者シュヴァルディ]","Very well. I feel like that was a part of our deal anyway. \\nw[Valiant Hero Schivardi]","그래, 좋다. 처음부터 그런 약속을 했던 것도 같고. \\nw[용사 슈발디]","嗯，也是可以啦。反正我一開始好像也跟妳們約好了。 \\nw[勇者修瓦爾迪]","嗯，也是可以啦。反正我一开始好像也跟你们约好了。\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["でも、んーっと、そうだな・・・。 \\nw[勇者シュヴァルディ]","Although... I guess... \\nw[Valiant Hero Schivardi]","그나저나 흐음・・・ \\nw[용사 슈발디]","不過，嗯……我想想…… \\nw[勇者修瓦爾迪]","不过，嗯……我想想……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["じゃあ、「勇者のやる気を出した武器屋」ってことで 広めといてあげるよ！ \\nw[勇者シュヴァルディ]","Okay, I'll spread the word. You'll be known as the \"shop that inspired the great hero\"! \\nw[Valiant Hero Schivardi]","그럼 「용사가 의욕을 얻은 무기점」이라고 소문을 내 줄게! \\nw[용사 슈발디]","那麼，我就以「讓勇者拿出幹勁的武器店」之名，替妳們宣傳吧！ \\nw[勇者修瓦爾迪]","那么，我就以「让勇者拿出干劲的武器店」之名，替你们宣传吧！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["やったー！ \\nw[ぽん美]","Yaaay! \\nw[Ponmi]","야호~! \\nw[폰미]","太好了──！ \\nw[器美]","太好了──！\\nw[器美]"]);
_tr_m.push(["あんまよく意味わかんないけど、なんかすごそう！ \\nw[ぽん美]","Not sure if people will understand, but it's better than nothing! \\nw[Ponmi]","무슨 소린지 잘 모르겠지만 엄청나다! \\nw[폰미]","雖然搞不太懂那是什麼意思，但感覺好像很厲害！ \\nw[器美]","虽然搞不太懂那是什么意思，但感觉好像很厉害！\\nw[器美]"]);
_tr_m.push(["でもボクは本当に感謝してるよ。ありがとう！ \\nw[勇者シュヴァルディ]","I truly am grateful for what you've done! \\nw[Valiant Hero Schivardi]","아무튼 감사하고 있다는 뜻이야. 고마워! \\nw[용사 슈발디]","不過，我真的很感謝妳。謝謝啊！ \\nw[勇者修瓦爾迪]","不过，我真的很感谢你。谢谢啊！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["さて、次こそは倒してやるぞ！ 魔王ーーっ！！！ \\nw[勇者シュヴァルディ]","Just you wait, Demon Lord...you're going down! \\nw[Valiant Hero Schivardi]","자, 이번에는 쓰러뜨려 주마! 마왕!!! \\nw[용사 슈발디]","那麼，魔王──！下次我一定要打倒你！！！ \\nw[勇者修瓦爾迪]","那么，魔王──！下次我一定要打倒你！！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["一週間後。","One week later...","1주일 후","一週後。","一周后。"]);
_tr_m.push(["勇者の広めた「勇者のやる気を出した武器屋」の噂は またたく間に広がり・・・。","In the blink of an eye, rumors of the \"shop that inspired the great hero\" spread.","용사가 퍼뜨린 「용사가 의욕을 얻은 무기점」 소문은 눈 깜짝할 사이에 퍼져서・・・","勇者四處宣傳「讓勇者拿出幹勁的武器店」，消息一下子就傳開了……","勇者四处宣传「让勇者拿出干劲的武器店」，消息一下子就传开了……"]);
_tr_m.push(["\\v[20]は大繁盛していた。","Business was booming for \\v[20].","\\v[20](은)는 큰 성황을 이뤘다.","\\v[20]生意變得相當好。","\\v[20]生意变得相当好。"]);
_tr_m.push(["ここが勇者のやる気を出したという 噂の武器屋か！ \\nw[人々]","So this is the famous shop that inspired the great hero! \\nw[people]","용사가 의욕을 얻었다는 무기점이 여기구나! \\nw[사람들]","這裡就是傳聞中能夠「讓勇者拿出幹勁」的武器店啊！ \\nw[人]","这里就是传闻中能够「让勇者拿出干劲」的武器店啊！\\nw[人]"]);
_tr_m.push(["すごい・・・さすがは話題の武器屋！ 品揃えも品質もすばらしい！！ \\nw[人々]","This is the weapons shop everyone's talking about! What a wide selection of quality arms! \\nw[people]","멋지다・・・ 역시 소문난 무기점이야! 물건도 좋고 품질도 훌륭해!! \\nw[사람들]","好厲害……真不愧是蔚為話題的武器店！品項齊全，品質也很精良！！ \\nw[人]","好厉害……真不愧是大家都在讨论的武器店！品项齐全，品质也很精良！！\\nw[人]"]);
_tr_m.push(["・・・うむ。この武器は、本物だ。 \\nw[人々]","Yes... This weapon is the real deal. \\nw[people]","・・・흐음, 이 무기는 명품이군. \\nw[사람들]","……嗯。這些武器真的是好東西。 \\nw[人]","……嗯。这些武器真的是好东西。\\nw[人]"]);
_tr_m.push(["やったね！大繁盛だよ、お姉ちゃん！ \\nw[ぽん美]","Just look at how well we're doing, Sis! \\nw[Ponmi]","성공이야, 언니! 엄청난 인기라구! \\nw[폰미]","太好了！生意興隆呢！姐姐！ \\nw[器美]","太好了！生意兴隆呢！姐姐！\\nw[器美]"]);
_tr_m.push(["\\n[1]のがんばりによって、やる気を出した勇者。 そのお陰で大繁盛した、\\v[20]。","\\n[1] worked hard to motivate the hero and \\v[20] flourished as a result.","\\n[1]의 노력 덕분에 의욕을 얻은 용사. 그 덕분에 큰 인기를 얻게 된 \\v[20].","由於\\n[1]的努力，讓勇者拿出了幹勁。也因此使得\\v[20]的生意變得相當興隆。","由于\\n[1]的努力，让勇者拿出了干劲。也因此使得\\v[20]的生意变得相当兴隆。"]);
_tr_m.push(["きっと\\n[1]のがんばりを、 天国のウェポチチも見ていることだろう―――。","Dirk must be watching from above, proud of \\n[1]'s efforts.","\\n[1]가 열심히 일하는 모습을 천국의 웨포파파도 지켜보고 있을 것이다.","\\n[1]所做的這些努力，在天國的武父應該也都看在眼裡吧───","\\n[1]所做的这些努力，在天上的武父应该也都看在眼里吧───"]);
_tr_m.push(["そういえば、勇者のお兄ちゃんは魔王を倒したのかな？ \\nw[ぽん美]","I wonder if that hero guy defeated the Demon Lord... \\nw[Ponmi]","그나저나 용사 오빠는 마왕을 쓰러뜨렸을까? \\nw[폰미]","對了，不知道勇者大哥打倒魔王了沒？ \\nw[器美]","对了，不知道勇者大哥打倒魔王了没？\\nw[器美]"]);
_tr_m.push(["ヒュルルル・・・。","Peeeew...","휘이익・・・","咻嚕嚕嚕……","咻噜噜噜……"]);
_tr_m.push(["\\{\\{ほげーーーっ！！ \\nw[？？？？？]","\\{\\{Oof!!!\\nw[?????]","\\{\\{으아아아아악!! \\nw[?????]","\\{\\{咕哇───！！ \\nw[？？？？？]","\\{\\{咕哇───！！\\nw[？？？？？]"]);
_tr_m.push(["END 2/4  「やる気の出る武器屋」","ENDING 2/4 [Inspirational Weapons Shop]","END 2/4  「의욕이 생기는 무기점」","END 2/4 「讓人拿出幹勁的武器店」","END 2/4「让人拿出干劲的武器店」"]);
_tr_m.push(["お姉ちゃん！本当に・・・手に入れたんだね！ \\nw[ぽん美]","Oh! You really got it!\\nw[Ponmi]","언니! 정말로・・・ 찾아냈구나! \\nw[폰미]","姐姐！妳真的……真的拿到了呢！ \\nw[器美]","姐姐！你真的……真的拿到了呢！ \\nw[器美]"]);
_tr_m.push(["伝説の武器、ツヨスギテ＝クサハエル・ソード！！ \\nw[ぽん美]","The legendary weapon, the LOL Sword! \\nw[Ponmi]","전설의 무기, 너무강해서 현웃터지는 소드!! \\nw[폰미]","傳說中的武器「暴強‧草叢生劍」！！ \\nw[器美]","传说中的武器「暴强‧草丛生剑」！！ \\nw[器美]"]);
_tr_m.push(["わーい！かっこいいー！ \\nw[ぽん美]","Phew! It looks awesome!\\nw[Ponmi]","우와~! 멋지다~! \\nw[폰미]","哇──！好帥喔──！ \\nw[器美]","哇──！好帅喔──！ \\nw[器美]"]);
_tr_m.push(["\\n[1]は、ついでに 魔王を倒したこともぽん美に伝えた！","\\n[1] told Ponmi that she defeated the Demon Lord!","\\n[1]는 마왕까지 쓰러뜨렸다는 소식도 폰미에게 전했다!","\\n[1]也把她順便打倒魔王的事告訴了器美！","\\n[1]也把她顺便打倒魔王的事告诉了器美！"]);
_tr_m.push(["・・・え？ \\nw[ぽん美]","What?! \\nw[Ponmi]","・・・뭐? \\nw[폰미]","……咦？ \\nw[器美]","……咦？\\nw[器美]"]);
_tr_m.push(["ま、魔王を倒したのーー！？ \\nw[ぽん美]","You defeated the Demon Lord?! \\nw[Ponmi]","마, 마왕을 쓰러뜨렸다고!? \\nw[폰미]","妳、妳打倒了魔王──！？ \\nw[器美]","你，你打倒了魔王──！？\\nw[器美]"]);
_tr_m.push(["なにしてるのお姉ちゃん！？ \\nw[ぽん美]","What do you think you're doing?! \\nw[Ponmi]","무슨 짓을 한 거야, 언니!? \\nw[폰미]","妳在做什麼啊，姐姐！？ \\nw[器美]","你在做什么啊，姐姐！？\\nw[器美]"]);
_tr_m.push(["いや、すごいけどさー！！ \\nw[ぽん美]","I mean that's wonderful! \\nw[Ponmi]","아니, 대단하긴 한데!! \\nw[폰미]","呃，妳這樣是很厲害啦，可是……！！ \\nw[器美]","呃，你这样是很厉害啦，可是……！！\\nw[器美]"]);
_tr_m.push(["ガヤガヤ・・・。","*noisy chatter*","웅성웅성・・・","吵雜吵雜……","人声嘈杂……"]);
_tr_m.push(["あっ！外が騒がしい！！ お姉ちゃん、様子を見に行こうよ！ \\nw[ぽん美]","Something's going on outside. Let's go take a look! \\nw[Ponmi]","앗! 바깥이 소란스럽잖아!! 언니, 무슨 일인지 보러 가자! \\nw[폰미]","啊！外面怎麼那麼吵！！姐姐，我們去外面看一下情況吧！ \\nw[器美]","啊！外面怎么那么吵！！姐姐，我们去外面看一下情况吧！\\nw[器美]"]);
_tr_m.push(["\\n[1]様！ありがとう！ あなたはこの世界の救世主だわ！！ \\nw[人々]","Lady \\n[1]! You are the world's savior! \\nw[people]","\\n[1] 님! 고마워요! 당신은 이 세계의 구세주예요!! \\nw[사람들]","\\n[1]大人！感謝您！您是這個世界的救世主！！ \\nw[人]","\\n[1]大人！感谢您！您是这个世界的救世主！！\\nw[人]"]);
_tr_m.push(["いやー。まさかキミみたいなおチビちゃんが 魔王を倒すなんてな。 \\nw[人々]","Who would've thought such a petite little girl would be the one to defeat the Demon Lord! \\nw[people]","우와~ 설마 너 같은 꼬마가 마왕을 해치울 줄이야. \\nw[사람들]","哎呀～沒想到像妳這樣的小姑娘，竟然能打倒魔王。 \\nw[人]","哎呀～没想到像你这样的小姑娘，竟然能打倒魔王。\\nw[人]"]);
_tr_m.push(["\\n[1]様が、真の勇者様に違いない！！ \\nw[人々]","Lady \\n[1], you are the heroine this land needs! \\nw[people]","\\n[1] 님은 진정한 용사님이 틀림없어!! \\nw[사람들]","\\n[1]大人確實是真正的勇者沒錯！！ \\nw[人]","\\n[1]大人确实是真正的勇者没错！！\\nw[人]"]);
_tr_m.push(["すごーい！！ お姉ちゃん、ヒーローじゃん！ \\nw[ぽん美]","Amazing! They're all calling you a heroine! \\nw[Ponmi]","굉장하다~!! 언니, 완전 영웅이야! \\nw[폰미]","好厲害！！ 姐姐，妳變成英雄了呢！ \\nw[器美]","好厉害！！姐姐，你变成英雄了呢！\\nw[器美]"]);
_tr_m.push(["はいはーい。大勇者\\n[1]の姿を見にきた人は ぜひ「\\v[20]」で武器を買っていってね！ \\nw[ぽん美]","Come one, come all! Those who have come to see our great heroine, \\n[1], please purchase a weapon from her shop, \\v[20]! \\nw[Ponmi]","자~! 위대한 용사 \\n[1]를 보러 오신 분들은 「\\v[20]」에서 무기도 사 가세요! \\nw[폰미]","大家注意！來看大勇者\\n[1]的人，請務必要到「\\v[20]」買個武器再回去喔！ \\nw[器美]","大家注意！来看大勇者\\n[1]的人，请务必要到「\\v[20]」买件武器再回去喔！\\nw[器美]"]);
_tr_m.push(["おい！！\\n[1]ちゃーーん！！！ \\nw[勇者シュヴァルディ]","Ahoy! \\n[1]! \\nw[Valiant Hero Schivardi]","이봐!! \\n[1]~!!! \\nw[용사 슈발디]","喂！！\\n[1]───！！！ \\nw[勇者修瓦爾迪]","喂！！\\n[1]───！！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["何やってんだよーー！！！ \\nw[勇者シュヴァルディ]","What do you think you're doing? \\nw[Valiant Hero Schivardi]","이게 무슨 짓이야~!!! \\nw[용사 슈발디]","看妳做了什麼好事──！！！ \\nw[勇者修瓦爾迪]","看你做了什么好事──！！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["キミが魔王を倒しちゃったせいで、 ボクの面目丸潰れじゃーん！！ \\nw[勇者シュヴァルディ]","Now that you've defeated the Demon Lord, people have lost their faith in me! \\nw[Valiant Hero Schivardi]","네가 마왕을 쓰러뜨리는 바람에 내 체면이 완전히 구겨졌잖아!! \\nw[용사 슈발디]","都是因為妳打倒了魔王，害我沒臉見人了啦！！ \\nw[勇者修瓦爾迪]","都是因为你打倒了魔王，害我没脸见人了啦！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["あっ。よくうちの看板前につっ立ってる変な人じゃん！！ \\nw[ぽん美]","You're that kook that likes to loiter in front of our shop's sign! \\nw[Ponmi]","앗, 종종 우리 가게 간판 앞에 서 있는 이상한 사람이네!! \\nw[폰미]","啊。你是常常站在我們家招牌前的那個怪人嘛！！ \\nw[器美]","啊。你是常常站在我们家招牌前的那个怪人嘛！！\\nw[器美]"]);
_tr_m.push(["変な人ゆーな！！！ ボクは由緒正しき勇者の血を引く大勇者様だ！！！ \\nw[勇者シュヴァルディ]","I'm no kook! I'm a great hero coming from an ancient and honorable line of heroes! \\nw[Valiant Hero Schivardi]","누구더러 이상한 사람이래!!! 난 유서 깊은 용사의 혈통을 이은 대용사님이시다!!! \\nw[용사 슈발디]","不要叫我怪人！！！我可是繼承了正統勇者血脈的大勇者大人！！！ \\nw[勇者修瓦爾迪]","不要叫我怪人！！！我可是继承了正统勇者血脉的大勇者大人！！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["え、でも・・・。 \\nw[ぽん美]","Sure, but... \\nw[Ponmi]","앗, 근데・・・ \\nw[폰미]","咦？可是…… \\nw[器美]","咦？可是……\\nw[器美]"]);
_tr_m.push(["魔王倒してないじゃん。 \\nw[ぽん美]","You didn't defeat the Demon Lord. \\nw[Ponmi]","마왕을 못 쓰러뜨렸잖아. \\nw[폰미]","你又沒打倒魔王。 \\nw[器美]","你又没打倒魔王。\\nw[器美]"]);
_tr_m.push(["うるせーーーー！！！ \\nw[勇者シュヴァルディ]","Shut up! \\nw[Valiant Hero Schivardi]","시끄러워~~~!!! \\nw[용사 슈발디]","煩死了────！！！ \\nw[勇者修瓦爾迪]","真是多嘴────！！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["クッソ！！なんだキミは！！ アッタマきた！！ \\nw[勇者シュヴァルディ]","Dammit! What is wrong with you?! Now I'm triggered! \\nw[Valiant Hero Schivardi]","젠장!! 너 뭐야!! 사람 열 받게!! \\nw[용사 슈발디]","可惡！！妳到底是怎樣啦！！氣死我了！！ \\nw[勇者修瓦爾迪]","可恶！！你到底要怎样！！气死我了！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["こうなったら・・・。 \\nw[勇者シュヴァルディ]","You leave me no choice. \\nw[Valiant Hero Schivardi]","그렇다면・・・ \\nw[용사 슈발디]","既然如此…… \\nw[勇者修瓦爾迪]","既然如此……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["なんと勇者シュヴァルディは ぽん美の喉もとに剣をあてがった！","The Valiant Hero Schivardi pulled out his sword and pointed it at Ponmi's throat!","용사 슈발디의 칼끝이 폰미의 목을 겨눴다!","勇者修瓦爾迪竟然把劍抵在器美的喉嚨上！","勇者修瓦尔迪竟然把剑抵在器美的喉咙上！"]);
_tr_m.push(["！？ \\nw[ぽん美]","!? \\nw[Ponmi]","!? \\nw[폰미]","！？ \\nw[器美]","！？\\nw[器美]"]);
_tr_m.push(["こ、怖いよーー！！ たすげでーーーーーー！！！ \\nw[ぽん美]","I-I'm scared! Help!!! \\nw[Ponmi]","무, 무서워~~!! 살려 줘~~~~~!!! \\nw[폰미]","好、好可怕啊──！！救救我─────！！！ \\nw[器美]","好，好可怕啊──！！救救我─────！！！\\nw[器美]"]);
_tr_m.push(["ギャハハハ！この生意気な娘の命が惜しければァ！！ \\nw[勇者シュヴァルディ]","Gwahaha! If you value this little harpy's life... \\nw[Valiant Hero Schivardi]","크하하하! 이 건방진 계집애의 목숨을 구하고 싶으면!! \\nw[용사 슈발디]","哇哈哈哈！如果還想要這個囂張的小姑娘活命……！！ \\nw[勇者修瓦爾迪]","哇哈哈哈！如果还想要这个嚣张的小姑娘活命……！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["\\n[1]よ！！ 魔王討伐の手柄をボクによこせぇーーっ！！！ \\nw[勇者シュヴァルディ]","\\n[1]! You will relinquish that title of Demon Lord Slayer to me! \\nw[Valiant Hero Schivardi]","\\n[1]!! 마왕을 토벌한 공을 나에게 넘겨라~~!!! \\nw[용사 슈발디]","\\n[1]啊！妳就把討伐魔王的功勞讓給我吧──！！！ \\nw[勇者修瓦爾迪]","\\n[1]啊！你就把讨伐魔王的功劳让给我吧──！！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["\\n[1]は「ええーーーー！？」と思った！","\\n[1] couldn't believe her ears.","\\n[1]는 깜짝 놀랐다! 「뭐라고~~~!?」","\\n[1]心想：「咦咦────！？」！","\\n[1]心想：「咦咦────！？」！"]);
_tr_m.push(["しかし！\\n[1]は魔王の言葉を思い出した！","But then she remembered what the Demon Lord said...","그러나 \\n[1]는 마왕의 말을 떠올렸다!","然而！\\n[1]想起了魔王說過的話！","然而！\\n[1]想起了魔王说过的话！"]);
_tr_m.push(["\\c[2]少女よ。我はまた封印されて、 しばらくこの世界に魔王という存在は無くなるだろう。 \\nw[魔王ドルハーバ]","\\c[2]If I'm locked away, the world will be without a Demon Lord for a while. \\nw[Demon Lord Dolhabach]","\\c[2]소녀여, 내가 다시 봉인되면 한동안 이 세계에 마왕이란 존재는 사라질 것이다. \\nw[마왕 도르하바]","\\c[2]少女啊。我再次遭到封印後，這個世界應該會暫時沒有魔王存在吧。 \\nw[魔王多爾哈巴]","\\c[2]少女啊。我再次遭到封印后，这个世界应该会暂时没有魔王存在吧。\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]魔王がいない平和な世界は、 人類の誰もが望むすばらしい世界だ！ \\nw[魔王ドルハーバ]","\\c[2]A peaceful world without a Demon Lord... It's all humankind could ever wish for! \\nw[Demon Lord Dolhabach]","\\c[2]마왕이 없는 평화로운 세계는 모든 인류가 바라는 훌륭한 세계다! \\nw[마왕 도르하바]","\\c[2]沒有魔王的和平世界，是每個人類都期盼的美好世界！ \\nw[魔王多爾哈巴]","\\c[2]没有魔王的和平世界，是每个人类都期盼的美好世界！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]しかし・・・先程述べたように、魔王の消えた平和な世界では 人類は平和ボケし堕落してしまう可能性がある！！ \\nw[魔王ドルハーバ]","\\c[2]However! As I explained earlier, a peaceful world without a Demon Lord will cause the people to become complacent! \\nw[Demon Lord Dolhabach]","\\c[2]그러나・・・ 앞서 말했다시피 마왕이 사라진 평화로운 세계에서 인류는 평화에 도취되어 타락할 가능성이 있다!! \\nw[마왕 도르하바]","\\c[2]然而……如同我剛剛所說的，魔王消失後的和平世界，有可能會讓人類墮落，沉溺於和平之中！！ \\nw[魔王多爾哈巴]","\\c[2]然而……如同我刚刚所说的，魔王消失后的和平世界，有可能会让人类堕落，沉溺于和平之中！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]それを防ぐのがキミだ！キミは魔王を倒した英雄として 人類を引っ張って行かなくてはならない！ \\nw[魔王ドルハーバ]","\\c[2]The only one who can prevent that is you! You've got to become a moral compass for the people! \\nw[Demon Lord Dolhabach]","\\c[2]그걸 막을 사람이 바로 너다! 너는 마왕을 쓰러뜨린 영웅으로서 인류를 이끌어야 한다! \\nw[마왕 도르하바]","\\c[2]為了防止這一點，就必須要靠妳！妳必須以「打倒魔王的英雄」這個身份，引導人類前進！ \\nw[魔王多爾哈巴]","\\c[2]为了防止这一点，就必须要靠你！你必须以「打倒魔王的英雄」这个身份，引导人类前进！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]キミが正しく導いてくれたとき・・・ 世界に「真の平和」は訪れる！！ \\nw[魔王ドルハーバ]","\\c[2]When you have led them down the righteous path is when the world will truly become peaceful! \\nw[Demon Lord Dolhabach]","\\c[2]네가 올바른 길로 이끌었을 때・・・ 세계에 「진정한 평화」가 찾아올 것이다!! \\nw[마왕 도르하바]","\\c[2]當妳引導人類朝正確方向前進之時……這個世界才會獲得「真正的和平」！！ \\nw[魔王多爾哈巴]","\\c[2]当你引导人类朝正确方向前进之时……这个世界才会获得「真正的和平」！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]キミにまかせるぞ！！この世界を・・・守ってくれよな！！ \\nw[魔王ドルハーバ]","\\c[2]I'm entrusting this duty to you! Guide and protect this world! \\nw[Demon Lord Dolhabach]","\\c[2]너를 믿으마!! 이 세계를・・・ 지켜 다오!! \\nw[마왕 도르하바]","\\c[2]一切就交給妳了！！妳要……好好保護這個世界！！ \\nw[魔王多爾哈巴]","\\c[2]一切就交给你了！！你要……好好保护这个世界！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]うおおおおお！！！ \\nw[魔王ドルハーバ]","\\c[2]Graaagh!!! \\nw[Demon Lord Dolhabach]","\\c[2]으아아아아악!!! \\nw[마왕 도르하바]","\\c[2]唔喔喔喔喔喔！！！ \\nw[魔王多爾哈巴]","\\c[2]唔喔喔喔喔喔！！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\n[1]は思った。","\\n[1] had a thought.","\\n[1]는 생각했다.","\\n[1]心想：","\\n[1]心想："]);
_tr_m.push(["「人間は愚かな生き物である。高い知性を持ちながら、 　放っておけば傷つけ合い、勝者を決めたがる。」","(Humans are foolish creatures. Despite being intelligent, we crave to rule over each other.)","「인간은 어리석은 생물이야. 높은 지성을 갖췄지만 내버려 두면 서로 상처를 입히고 승자를 정하고 싶어 해.」","「人類是一種愚蠢的生物。雖然擁有高等智慧，但若放任不管，就會想互相傷害，決定誰才是勝利者。」","「人类是一种愚蠢的生物。虽然拥有高等智慧，但若放任不管，就会想互相伤害，决定谁才是胜利者。」"]);
_tr_m.push(["「人間には、人間を正しく導く者が必要だ。」","(Humans need proper guidance.)","「인간에게는 인간을 올바른 길로 이끌 사람이 필요해.」","「人類需要一個能引導他們朝正確方向前進的人。」","「人类需要一个能引导他们朝正确方向前进的人。」"]);
_tr_m.push(["「私が・・・私が導かなければ！！ 　真の平和を守るため！！」","(I'll do it. I'll lead them! I will defend true peace!)","「내가・・・ 내가 이끌어야 해!! 진정한 평화를 지키기 위해!!」","「就由我……必須由我來引導！！為了守護真正的和平！！」","「就由我……必须由我来引导！！为了守护真正的和平！！」"]);
_tr_m.push(["「それが、魔王の願いなのだから・・・！」と。","(That was what the Demon Lord entrusted me to do!)","「그것이 마왕의 바람이니까・・・!」","「因為這是魔王的心願……！」","「因为这是魔王的心愿……！」"]);
_tr_m.push(["勇者シュヴァルディよ。 \\nw[うぇ子]","Valiant Hero Schivardi... \\nw[Weaco]","용사 슈발디여. \\nw[웨코]","勇者修瓦爾迪啊。 \\nw[武子]","勇者修瓦尔迪啊\\nw[武子]"]);
_tr_m.push(["！？！？ \\nw[ぽん美]","!?!? \\nw[Ponmi]","!?!? \\nw[폰미]","！？！？ \\nw[器美]","！？！？\\nw[器美]"]);
_tr_m.push(["な・・・なんだよ！！ \\nw[勇者シュヴァルディ]","Wh-What now?! \\nw[Valiant Hero Schivardi]","뭐・・・ 뭐야!! \\nw[용사 슈발디]","怎……怎樣啦！！ \\nw[勇者修瓦爾迪]","干……干什么！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["感情的にならずに、心を落ち着けなさい。 \\nw[うぇ子]","I want you to quiet your heart and your mind. \\nw[Weaco]","감정을 억누르고 마음을 진정시키세요. \\nw[웨코]","你不要如此情緒化，讓你的心靈冷靜下來。 \\nw[武子]","你不要如此激动，让你的心灵冷静下来。\\nw[武子]"]);
_tr_m.push(["冷静に・・・冷静になるのです。 そして、客観的に見つめるのです。 \\nw[うぇ子]","Calm your nerves and let go. Take a step back and look at yourself. \\nw[Weaco]","냉정・・・ 냉정을 찾으세요. 그리고 객관적으로 보세요. \\nw[웨코]","冷靜……讓心靈冷靜。然後，再以客觀的角度仔細看看。 \\nw[武子]","冷静……让心灵冷静。然后，再以客观的角度仔细看看。\\nw[武子]"]);
_tr_m.push(["今のあなたの姿を！！ \\nw[うぇ子]","See what you've become. \\nw[Weaco]","지금 당신의 모습을!! \\nw[웨코]","看看你現在的樣子！！ \\nw[武子]","看看你现在的样子！！\\nw[武子]"]);
_tr_m.push(["・・・。 \\nw[勇者シュヴァルディ]","... \\nw[Valiant Hero Schivardi]","・・・. \\nw[용사 슈발디]","…… \\nw[勇者修瓦爾迪]","……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["・・・・・。 \\nw[勇者シュヴァルディ]","...... \\nw[Valiant Hero Schivardi]","・・・・・. \\nw[용사 슈발디]","……\\nw[勇者修瓦爾迪]","……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["はっ！！ \\nw[勇者シュヴァルディ]","Agh! \\nw[Valiant Hero Schivardi]","앗!! \\nw[용사 슈발디]","啊！！ \\nw[勇者修瓦爾迪]","啊！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["・・・そうです。気づいたようですね。 \\nw[うぇ子]","It seems you have come around full circle. \\nw[Weaco]","・・・그래요. 이제 깨달았군요. \\nw[웨코]","……沒錯。看來你也已經察覺了呢。 \\nw[武子]","……没错。看来你也已经察觉了呢。\\nw[武子]"]);
_tr_m.push(["今のあなたは、いい歳こいて怠けて若者に先を越され、 腹いせに少女を人質にとっているただのゴミクズです。 \\nw[うぇ子]","You've become lazy at your age. You were shown up by someone younger, and now you're taking it out on a mere girl.<br>You are scum. \\nw[Weaco]","지금 당신은 나이를 먹을 만큼 먹은 주제에 게을러빠져서는 젊은 사람이 자기를 앞질렀다고 소녀를 인질 삼아 화풀이하는 쓰레기예요. \\nw[웨코]","你年紀已經不小了，卻太過怠惰，而被年輕人超越。為了洩憤，你抓了一個少女當成人質。這樣的你只不過是個廢物而已。 \\nw[武子]","你年纪已经不小了，却太过怠惰，而被年轻人超越。为了泄愤，你抓了一个少女当成人质。这样的你只不过是个废物而已。\\nw[武子]"]);
_tr_m.push(["\\{\\{\\{うわあああああ！！！ \\nw[勇者シュヴァルディ]","\\{\\{\\{Auggghhh! \\nw[Valiant Hero Schivardi]","\\{\\{\\{으아아아아아아!!! \\nw[용사 슈발디]","\\{\\{\\{嗚哇啊啊啊啊啊！！！ \\nw[勇者修瓦爾迪]","\\{\\{\\{呜哇啊啊啊啊啊！！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["しかし、まだ遅くありません。 あなたはこれから変わることができます。 \\nw[うぇ子]","It is not too late for you, however. You can still change. \\nw[Weaco]","하지만 아직 늦지 않았어요. 당신은 앞으로 변할 수 있어요. \\nw[웨코]","不過，現在還不算太遲。還有機會可以改變。 \\nw[武子]","不过，现在还不算太迟。还有机会可以改变。\\nw[武子]"]);
_tr_m.push(["人はみな、手を取り合って生きていけるはずです。 \\nw[うぇ子]","We survive by helping each other out. \\nw[Weaco]","인간은 누구나 손을 맞잡고 살아갈 수 있어요. \\nw[웨코]","每個人應該都能互助合作，一起生活在這個世界上。 \\nw[武子]","每个人应该都能互助合作，一起生活在这个世界上。\\nw[武子]"]);
_tr_m.push(["さあ！あなたも、人を傷つけるようなマネはやめて・・・ みんなで仲良くしましょう！ \\nw[うぇ子]","Now, stop this violent outrage and let's be friends! \\nw[Weaco]","자! 당신도 남을 상처 입히지 말고・・・ 모두 사이좋게 지내요! \\nw[웨코]","來吧！別再做出這種會傷害到別人的事……讓大家一起和平共處吧！ \\nw[武子]","来吧！别再做出这种会伤害到别人的事……让大家一起和平共处吧！\\nw[武子]"]);
_tr_m.push(["うう・・・うわあ・・・。 \\nw[勇者シュヴァルディ]","*sniff* *sniff* \\nw[Valiant Hero Schivardi]","으윽・・・ 으아아・・・ \\nw[용사 슈발디]","嗚嗚……嗚哇啊…… \\nw[勇者修瓦爾迪]","呜呜……呜哇啊……\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["うわあああああ！！！ \\nw[勇者シュヴァルディ]","Waaahhh! \\nw[Valiant Hero Schivardi]","으아아아아아아!!! \\nw[용사 슈발디]","嗚哇啊啊啊啊啊！！！ \\nw[勇者修瓦爾迪]","呜哇啊啊啊啊啊！！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["ボクが悪かった・・・ごめんなさい・・・！！ \\nw[勇者シュヴァルディ]","What have I done?! I'm so sorry! \\nw[Valiant Hero Schivardi]","내가 잘못했어・・・ 미안해・・・!! \\nw[용사 슈발디]","都是我的錯……對不起……！！ \\nw[勇者修瓦爾迪]","都是我的错……对不起……！！\\nw[勇者修瓦尔迪]"]);
_tr_m.push(["勇者シュヴァルディは 警察に連行されていった！","The police took custody of the Valiant Hero Schivardi!","용사 슈발디는 경찰에 연행되었다!","勇者修瓦爾迪被警察帶走了！","勇者修瓦尔迪被警察带走了！"]);
_tr_m.push(["ううっ、怖かった。 お姉ちゃん、助けてくれてありがとう。 \\nw[ぽん美]","*sniff* I was so scared. I'm glad you were there to save me. \\nw[Ponmi]","으으, 무서웠어. 언니, 구해 줘서 고마워. \\nw[폰미]","嗚哇，好可怕喔。姐姐，謝謝妳救了我。 \\nw[器美]","呜哇，好可怕喔。姐姐，谢谢你救了我。\\nw[器美]"]);
_tr_m.push(["それにしても、お姉ちゃん・・・ 人を諭す才能もあったんだね。見直したよ。 \\nw[ぽん美]","But that was something else; the way you persuaded him... I didn't know you had it in you. \\nw[Ponmi]","그런데 언니・・・ 사람의 마음을 움직이는 재능도 있었구나. 다시 봤어. \\nw[폰미]","不過，姐姐……想不到妳有勸人向善的天分呢。我真是對妳刮目相看了。 \\nw[器美]","不过，姐姐……想不到你有劝人向善的天分呢。我真是对你刮目相看了。\\nw[器美]"]);
_tr_m.push(["それで、ちょっと提案があるんだけど・・・。 \\nw[ぽん美]","I've got an idea... \\nw[Ponmi]","그래서 제안하고 싶은 게 있는데・・・ \\nw[폰미]","然後呢，我有個提議…… \\nw[器美]","然后呢，我有个提议……\\nw[器美]"]);
_tr_m.push(["その後、ぽん美の提案で、\\n[1]は 「\\v[20]教」という新興宗教を立ち上げた。","As suggested by Ponmi, \\n[1] established a new denomination, the Order of the \\v[20].","그 후, 폰미의 제안으로 \\n[1]는 「\\v[20]교」라는 신흥 종교를 설립했다.","之後，根據器美的提議，\\n[1]創立了名為「\\v[20]教」的新興宗教。","之后，根据器美的提议，\\n[1]创立了名为「\\v[20]教」的新兴宗教。"]);
_tr_m.push(["みなさん。「\\v[20]教」の基本理念は 「みんななかよく」です。 \\nw[うぇ子]","Welcome to the Order of the \\v[20]. Our golden rule is to be kind to all. \\nw[Weaco]","여러분, 「\\v[20]교」의 기본 이념은 「모두 사이좋게」예요. \\nw[웨코]","各位。「\\v[20]教」的基本理念是「大家要和平共處」。 \\nw[武子]","各位。「\\v[20]教」的基本理念是「大家要和平共处」。\\nw[武子]"]);
_tr_m.push(["みんなが仲良くすれば、みんな楽しいし、 世界は平和になります！ \\nw[うぇ子]","If we are kind to each other, everyone will be able to enjoy themselves and the world will be at peace. \\nw[Weaco]","모두 사이좋게 지내면 모두가 즐겁고 세계는 평화로워질 거예요! \\nw[웨코]","只要大家能和平共處，大家就都能獲得快樂，世界也能獲得和平！ \\nw[武子]","只要大家能和平共处，大家就都能获得快乐，世界也能获得和平！\\nw[武子]"]);
_tr_m.push(["魔王がいない今・・・ みんなで仲良くし、みんなでワイワイしましょう！ \\nw[うぇ子]","With the Demon Lord gone, let us enjoy each other's company and have a good time! \\nw[Weaco]","마왕이 사라진 지금・・・ 모두 사이좋게, 모두 왁자지껄하게 지내도록 해요! \\nw[웨코]","現在沒了魔王……大家就來和平共處，過著快樂的生活吧！ \\nw[武子]","现在没了魔王……大家就来和平共处，快乐地生活吧！\\nw[武子]"]);
_tr_m.push(["うおおおおおおお！！！ \\nw[人々]","Hear, hear! \\nw[people]","믿습니다~~~!!! \\nw[사람들]","唔喔喔喔喔喔！！！ \\nw[人]","唔喔喔喔喔喔！！！\\nw[人]"]);
_tr_m.push(["教祖様バンザイ！！教祖様バンザイ！！ \\nw[人々]","Praise be the high priestess! Praise be the high priestess! \\nw[people]","교주님 만세!! 교주님 만세!! \\nw[사람들]","教祖大人萬歲！！教祖大人萬歲！！ \\nw[人]","教主大人万岁！！教主大人万岁！！\\nw[人]"]);
_tr_m.push(["はい！というわけで、教祖様の話は終わりでーす。 解散！！ \\nw[ぽん美]","There you have it. That's all folks. You are dismissed! \\nw[Ponmi]","자, 오늘 교주님의 말씀은 여기까지입니다~ 해산!! \\nw[폰미]","好！事情就是這樣，教祖大人的演講結束了。解散！！ \\nw[器美]","好！事情就是这样，教主大人的演讲结束了。解散！！\\nw[器美]"]);
_tr_m.push(["あっ！ついでに信者は「\\v[20]」で お買い物してってねー！ \\nw[ぽん美]","Ah! Before you go, we ask that you show your faith by purchasing something from \\v[20]! \\nw[Ponmi]","앗! 그리고 신자 여러분은 「\\v[20]」에서 쇼핑하고 가세요~! \\nw[폰미]","啊！各位信徒記得要順便到「\\v[20]」買東西喔～！ \\nw[器美]","啊！各位信徒记得要顺便到「\\v[20]」买东西喔～！\\nw[器美]"]);
_tr_m.push(["魔王を倒し、さらには宗教を立ち上げ 人々を導いていく道を選んだ\\n[1]。","\\n[1] defeated the Demon Lord and decided to lead the people through her teachings.","마왕을 쓰러뜨리고 또 종교를 만들어 사람들을 이끄는 길을 선택한 \\n[1].","\\n[1]打倒魔王後，創立了宗教，選擇讓自己來引導眾人前進的道路。","\\n[1]打倒魔王后，创立了宗教，选择让自己来引导众人前进的道路。"]);
_tr_m.push(["\\n[1]の導きのもと、人間が作り上げていく平和な世界。 きっと魔王も笑顔で見守っていることだろう―――。","\\n[1] achieved peace by guiding the people. Surely, the Demon Lord is smiling, wherever he may be.","\\n[1]의 인도 아래, 인간이 만들어 가는 평화로운 세계. 분명 마왕도 웃는 얼굴로 지켜보고 있겠지.","人類在\\n[1]的引導之下，建立了和平的世界。想必魔王一定也是笑著在看顧這個世界吧────","人类在\\n[1]的引导之下，建立了和平的世界。想必魔王一定也会满意地看着这个世界吧────"]);
_tr_m.push(["END 3/4  「みんななかよく」","ENDING 3/4 [Be Kind & Friendly]","END 3/4  「모두 사이좋게」","END 3/4 「大家和平共處」","END 3/4「大家和平共处」"]);
_tr_m.push(["お姉ちゃん！本当に・・・手に入れたんだね！ \\nw[ぽん美]","Oh! You really got it!\\nw[Ponmi]","언니! 정말로・・・ 찾아냈구나! \\nw[폰미]","姐姐！妳真的……真的拿到了呢！ \\nw[器美]","姐姐！你真的……真的拿到了呢！ \\nw[器美]"]);
_tr_m.push(["伝説の武器、ツヨスギテ＝クサハエル・ソード！！ \\nw[ぽん美]","The legendary weapon, the LOL Sword! \\nw[Ponmi]","전설의 무기, 너무강해서 현웃터지는 소드!! \\nw[폰미]","傳說中的武器「暴強‧草叢生劍」！！ \\nw[器美]","传说中的武器「暴强‧草丛生剑」！！ \\nw[器美]"]);
_tr_m.push(["わーい！かっこいいー！ \\nw[ぽん美]","Phew! It looks awesome!\\nw[Ponmi]","우와~! 멋지다~! \\nw[폰미]","哇──！好帥喔──！ \\nw[器美]","哇──！好帅喔──！ \\nw[器美]"]);
_tr_m.push(["それにしても。随分色んな武器集めてたねー。 \\nw[ぽん美]","But that's not all! You've gathered so many other weapons too! \\nw[Ponmi]","근데 무기를 참 다양하게 모았구나~ \\nw[폰미]","不過，妳還收集了真多武器呢。 \\nw[器美]","不过，你收集的武器还真多呢。\\nw[器美]"]);
_tr_m.push(["何種類ぐらいあるの？ \\nw[ぽん美]","How many have you got? \\nw[Ponmi]","몇 가지나 있어? \\nw[폰미]","大概有多少種啊？ \\nw[器美]","大概有多少种啊？\\nw[器美]"]);
_tr_m.push(["\\n[1]はぽん美に完成した武器図鑑を見せた！","\\n[1] showed Ponmi her completed Weapons Gallery!","\\n[1]는 폰미에게 완성된 무기 도감을 보여 주었다!","\\n[1]把完成的武器圖鑑拿給器美看！","\\n[1]把完成的武器图鉴拿给器美看！"]);
_tr_m.push(["えっ・・・なにこれ！多っ！！ \\nw[ぽん美]","Amazing! Look at all of them! \\nw[Ponmi]","우와・・・ 이게 뭐야! 정말 많네!! \\nw[폰미]","咦……這是怎麼回事？好多喔！！ \\nw[器美]","咦……这是怎么回事？好多喔！！\\nw[器美]"]);
_tr_m.push(["これ全部集めたの！？お姉ちゃん何者！？ \\nw[ぽん美]","You got ALL of these? What kind of weapon hoarder are you?! \\nw[Ponmi]","이걸 전부 모았다구!? 언니, 정체가 뭐야!? \\nw[폰미]","這些全部是妳收集的！？姐姐妳到底是何方神聖！？ \\nw[器美]","这些全部是你收集的！？姐姐你到底是何方神圣！？\\nw[器美]"]);
_tr_m.push(["あ、お客さんかな。はーい！！ \\nw[ぽん美]","Oh, looks like we have a customer. How may I help you? \\nw[Ponmi]","앗, 손님인가? 어서 오세요!! \\nw[폰미]","啊，好像有客人。馬上來──！！ \\nw[器美]","啊，好像有客人。马上来──！！\\nw[器美]"]);
_tr_m.push(["あ、どうも。神です。 \\nw[武器の神アメノオハバリ]","Hey! It's me, the god of weaponry. \\nw[God of Weaponry Ameno Ohbari]","아, 안녕하세요. 신입니다. \\nw[무기의 신 아메노오하바리]","啊，妳好，我是神。 \\nw[武器之神 天之尾羽張]","啊，你好，我是神。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["\\nw[ぽん美]","\\nw[Ponmi]","\\nw[폰미]","\\nw[器美]","\\nw[器美]"]);
_tr_m.push(["\\n[1]よ。そなたは、すべての武器を集めるという 偉業を成し遂げてしまった。 \\nw[武器の神アメノオハバリ]","\\n[1]! You have managed to collect all of the weapons. \\nw[God of Weaponry Ameno Ohbari]","\\n[1]여. 그대는 모든 무기를 모으는 위업을 달성하였다. \\nw[무기의 신 아메노오하바리]","\\n[1]啊。妳完成了收集所有武器的偉大功績。 \\nw[武器之神 天之尾羽張]","\\n[1]啊。你完成了收集所有武器的伟大功绩。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["そなたはこの世の武器を知り尽くし、 武器の持つ可能性を最大まで高めた。 \\nw[武器の神アメノオハバリ]","You have discovered every weapon and their capabilities, maximizing the potential of each one. \\nw[God of Weaponry Ameno Ohbari]","그대는 이 세계의 무기를 샅샅이 파헤쳐 무기가 가진 가능성을 최대로 끌어올렸다. \\nw[무기의 신 아메노오하바리]","妳熟知這個世界上所有的武器，並將武器擁有的潛能發揮到極限。 \\nw[武器之神 天之尾羽張]","你熟知这个世界上所有的武器，并将武器拥有的潜能发挥到极限。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["まさに、「武器を司る」者に相応しい。 \\nw[武器の神アメノオハバリ]","I hereby deem you worthy to govern all weapons. \\nw[God of Weaponry Ameno Ohbari]","그야말로 「무기를 관장하는」 자로서 적합하다. \\nw[무기의 신 아메노오하바리]","非常適合擔任「掌管武器」的人。 \\nw[武器之神 天之尾羽張]","非常适合担任「掌管武器」的人。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["我も長いこと神をやってきた。 そろそろ引退時と考えている・・・。 \\nw[武器の神アメノオハバリ]","I have been at this for too long. It's about time I retire... \\nw[God of Weaponry Ameno Ohbari]","나도 오랫동안 신이었지. 곧 은퇴를 생각해야 할 시기다・・・ \\nw[무기의 신 아메노오하바리]","我當神也很久了，差不多也該考慮退休一事…… \\nw[武器之神 天之尾羽張]","我当了很久的神了，差不多也该考虑退休一事……\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["それって・・・。 \\nw[ぽん美]","Don't tell me... \\nw[Ponmi]","그 말은 설마・・・ \\nw[폰미]","這話的意思是…… \\nw[器美]","这话的意思是……\\nw[器美]"]);
_tr_m.push(["\\n[1]を「二代目　武器の神」として認定する！！ \\nw[武器の神アメノオハバリ]","\\n[1]! I hereby recognize you as the second coming of the god of weaponry! \\nw[God of Weaponry Ameno Ohbari]","\\n[1]를 「제2대 무기의 신」으로 인정한다!! \\nw[무기의 신 아메노오하바리]","\\n[1]，我認定妳為「第二代 武器之神」！！　\\nw[武器之神 天之尾羽張]","\\n[1]，我认定你为「第二代 武器之神」！！　\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["\\{\\{ええーーーっ！！？？ \\nw[ぽん美]","\\{\\{Whaaat?! \\nw[Ponmi]","\\{\\{뭐라고~~~!!?? \\nw[폰미]","\\{\\{咦咦───！！？？ \\nw[器美]","\\{\\{咦咦───！！？？\\nw[器美]"]);
_tr_m.push(["お姉ちゃんが神に！？！？ \\nw[ぽん美]","She's gonna become...a god? \\nw[Ponmi]","언니를 신으로!?!? \\nw[폰미]","姐姐要變成神！？！？ \\nw[器美]","姐姐要变成神！？！？\\nw[器美]"]);
_tr_m.push(["そ、そうは言っても・・・ 神って実際何をすればいいんですか？ \\nw[ぽん美]","That aside... What is it that a god needs to do anyway? \\nw[Ponmi]","하, 하지만・・・ 신은 실제로 무슨 일을 하죠? \\nw[폰미]","就、就算你這麼說……神實際上到底要做些什麼事啊？ \\nw[器美]","就，就算你这么说……神实际上到底要做些什么事啊？\\nw[器美]"]);
_tr_m.push(["・・・武器の神の仕事。 それは、武器の発展を影で支えることだ。 \\nw[武器の神アメノオハバリ]","The god of weaponry must support the development of weapons from the shadows. \\nw[God of Weaponry Ameno Ohbari]","・・・무기의 신이 하는 일. 그것은 무기의 발전을 남몰래 응원하는 것이다. \\nw[무기의 신 아메노오하바리]","……武器之神的工作，就是在暗地裡支持武器的發展。 \\nw[武器之神 天之尾羽張]","……武器之神的工作，就是在暗地里支持武器的发展。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["へ？ \\nw[ぽん美]","Huh? \\nw[Ponmi]","네? \\nw[폰미]","咦？ \\nw[器美]","咦？\\nw[器美]"]);
_tr_m.push(["例えば今回の「伝説の武器」のようなものだ。 \\nw[武器の神アメノオハバリ]","Take this legendary weapon for example. \\nw[God of Weaponry Ameno Ohbari]","예를 들면 이번 「전설의 무기」도 마찬가지다. \\nw[무기의 신 아메노오하바리]","比如說，像是這次的「傳說武器」。 \\nw[武器之神 天之尾羽張]","比如说，像是这次的「传说武器」。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["ああ、言っていなかったか。 実は伝説の武器のウワサは我が流したのだよ。 \\nw[武器の神アメノオハバリ]","Oh, I guess I haven't told you yet. I was the one who spread the rumor of the legendary weapon. \\nw[God of Weaponry Ameno Ohbari]","아, 내가 말 안 했나? 사실 전설의 무기에 관한 소문은 내가 흘린 거야. \\nw[무기의 신 아메노오하바리]","啊，我沒跟妳提過嗎？其實，傳說武器的消息就是我流傳出去的。 \\nw[武器之神 天之尾羽張]","啊，我没跟你提过吗？其实，传说武器的消息就是我流传出去的。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["そうすることで皆、武器を求めて奮起する。 武器文化の発展につながるからな。 \\nw[武器の神アメノオハバリ]","With a desire to find it, people would seek out stronger weapons, advancing the development of arms. \\nw[God of Weaponry Ameno Ohbari]","그렇게 하면 모두 무기를 열심히 찾아다니겠지. 그것이 무기 문화의 발전으로 이어질 테고. \\nw[무기의 신 아메노오하바리]","如此一來，大家就會努力地想要尋求武器。也有助於武器文化的發展。 \\nw[武器之神 天之尾羽張]","如此一来，大家就会努力地想要寻求武器。也有助于武器文化的发展。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["まあ、まさか本当に伝説の武器を手に入れる者が 現れるとは思わなかったがな。 \\nw[武器の神アメノオハバリ]","I never imagined someone capable would actually appear and acquire it, though. \\nw[God of Weaponry Ameno Ohbari]","뭐, 사실 진짜로 전설의 무기를 손에 넣는 사람이 나타날 줄은 몰랐지만. \\nw[무기의 신 아메노오하바리]","不過，我還真沒想到會有人拿到這把傳說武器。 \\nw[武器之神 天之尾羽張]","不过，我还真没想到会有人拿到这把传说武器。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["・・・。 \\nw[ぽん美]","... \\nw[Ponmi]","・・・. \\nw[폰미]","…… \\nw[器美]","……\\nw[器美]"]);
_tr_m.push(["なんか、予想してた神とは違うような・・・。 \\nw[ぽん美]","You seem a little different from what I'd call a deity... \\nw[Ponmi]","뭔가 내가 상상하던 신과 이미지가 달라・・・ \\nw[폰미]","感覺跟我想像中的神不太一樣… \\nw[器美]","感觉跟我想像中的神不太一样…\\nw[器美]"]);
_tr_m.push(["いいや。これも立派な神としての役割である。 \\nw[武器の神アメノオハバリ]","How so? I serve a magnificent purpose in this role. \\nw[God of Weaponry Ameno Ohbari]","아니, 이것도 어엿한 신의 역할이다. \\nw[무기의 신 아메노오하바리]","不。這也是神該做的神聖任務之一。 \\nw[武器之神 天之尾羽張]","不。这也是神该做的神圣任务之一。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["魔王のいない平和な世界では、戦いの必要性も薄れ、 いずれ「武器」の文化は廃れてしまう。 \\nw[武器の神アメノオハバリ]","With the Demon Lord vanquished, a world without war would prosper in peace, but the culture of weapons would be lost. \\nw[God of Weaponry Ameno Ohbari]","마왕이 없는 평화로운 세계에서는 싸울 필요가 없으니 「무기」 문화는 결국 쇠퇴하겠지. \\nw[무기의 신 아메노오하바리]","在沒有魔王的和平世界中，戰爭的必要性也會減低，總有一天「武器」文化會消失。 \\nw[武器之神 天之尾羽張]","在没有魔王的和平世界中，战争的必要性也会减低，总有一天「武器」文化会消失。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["しかし武器とは古代から存在する歴史ある文化。 簡単に終わらせてはならない。 \\nw[武器の神アメノオハバリ]","But weapons have a long history since the first days of man. They cannot be allowed to disappear. \\nw[God of Weaponry Ameno Ohbari]","하지만 무기란 고대부터 존재하는 역사가 깊은 문화다. 그렇게 쉽게 끝낼 수는 없어. \\nw[무기의 신 아메노오하바리]","然而，武器是自古以來就存在的文化，擁有漫長的歷史。不能讓這種文化輕易消失。 \\nw[武器之神 天之尾羽張]","然而，武器是自古以来就存在的文化，拥有漫长的历史。不能让这种文化轻易消失。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["なぜなら！！ \\nw[武器の神アメノオハバリ]","Do you wish to know why? \\nw[God of Weaponry Ameno Ohbari]","왜냐하면!! \\nw[무기의 신 아메노오하바리]","因為……！！ \\nw[武器之神 天之尾羽張]","因为……！！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["武器は「ロマン」だからだ！！ \\nw[武器の神アメノオハバリ]","Because weapons are the essence of adventure! \\nw[God of Weaponry Ameno Ohbari]","무기는 「로망」이니까!! \\nw[무기의 신 아메노오하바리]","武器就是「浪漫」啊！！ \\nw[武器之神 天之尾羽張]","武器就是「浪漫」啊！！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["古代から人々の中二病を刺激し続ける！！ それが武器だ！！ \\nw[武器の神アメノオハバリ]","Weapons have always brought people back to their adolescence! \\nw[God of Weaponry Ameno Ohbari]","고대부터 사람들의 중2병을 끊임없이 자극하는 것!! 그것이 바로 무기다!! \\nw[무기의 신 아메노오하바리]","自古以來，持續搔動著人類中二病的心靈！！這就是武器！！ \\nw[武器之神 天之尾羽張]","自古以来，持续搔动着人类中二病的心灵！！这就是武器！！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["我はその「ロマン」を未来永劫守るため、 神となり武器を守り続けているのだ。 \\nw[武器の神アメノオハバリ]","To protect this sense of adventure, I became a god to guard weapons for all time. \\nw[God of Weaponry Ameno Ohbari]","나는 그 「로망」을 영원히 지키기 위해 신이 되어 무기를 지키고 있는 것이지. \\nw[무기의 신 아메노오하바리]","我為了守護這種「浪漫」直到永遠，於是成為了神，一直在守護著武器。 \\nw[武器之神 天之尾羽張]","我为了永远守护这种「浪漫」，于是成为了神，一直在守护着武器。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["・・・。 \\nw[ぽん美]","... \\nw[Ponmi]","・・・. \\nw[폰미]","…… \\nw[器美]","……\\nw[器美]"]);
_tr_m.push(["「ロマン」か・・・。 \\nw[ぽん美]","\"Adventure,\" huh... \\nw[Ponmi]","「로망」이라・・・ \\nw[폰미]","「浪漫」啊…… \\nw[器美]","「浪漫」啊……\\nw[器美]"]);
_tr_m.push(["私にはよくわからないけど。 \\nw[ぽん美]","I'm not sure I get it... \\nw[Ponmi]","난 잘 모르겠지만 \\nw[폰미]","雖然我不太懂那種情懷，不過…… \\nw[器美]","虽然我不太懂那种情怀，不过……\\nw[器美]"]);
_tr_m.push(["お姉ちゃんはそういうの、大好きだもんね。 \\nw[ぽん美]","But I think my sister does. \\nw[Ponmi]","언니는 그런 거 아주 좋아하지? \\nw[폰미]","姐姐非常喜歡那種東西呢。 \\nw[器美]","姐姐非常喜欢那种东西呢。\\nw[器美]"]);
_tr_m.push(["神になるんでしょ、お姉ちゃん。がんばってね！！ \\nw[ぽん美]","So you'll become a god, huh? Good luck with that! \\nw[Ponmi]","신이 되겠구나. 언니, 열심히 해!! \\nw[폰미]","姐姐，妳應該會成為神吧？要加油喔！！ \\nw[器美]","姐姐，你应该会成为神吧？要加油喔！！\\nw[器美]"]);
_tr_m.push(["二代目武器の神、\\n[1]よ。 \\nw[武器の神アメノオハバリ]","\\n[1], the second coming of the god of weaponry... \\nw[God of Weaponry Ameno Ohbari]","제2대 무기의 신, \\n[1]여. \\nw[무기의 신 아메노오하바리]","第二代武器之神，\\n[1]啊。 \\nw[武器之神 天之尾羽張]","第二代武器之神，\\n[1]啊。\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["神として、今後の武器の発展のため 誠心誠意尽くすように！！ \\nw[武器の神アメノオハバリ]","I implore you to devote yourself to the continued development of weapons as a god! \\nw[God of Weaponry Ameno Ohbari]","신으로서 앞으로 무기의 발전을 위해 성심과 성의를 다하거라!! \\nw[무기의 신 아메노오하바리]","妳要以神的身份，致力於武器今後的發展！！ \\nw[武器之神 天之尾羽張]","你要以神的身份，致力于武器今后的发展！！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["「ロマン」を守るためにな！！ \\nw[武器の神アメノオハバリ]","To protect the spirit of adventure! \\nw[God of Weaponry Ameno Ohbari]","「로망」을 지키기 위해서!! \\nw[무기의 신 아메노오하바리]","這也是為了要守護「浪漫」！！ \\nw[武器之神 天之尾羽張]","这也是为了要守护「浪漫」！！\\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["こうして、\\n[1]は神となった。","Thus, \\n[1] became a god.","이리하여 \\n[1]는 신이 되었다.","就這樣，\\n[1]成為了神。","就这样，\\n[1]成为了神。"]);
_tr_m.push(["\\n[1]は人間を超越した存在となったのだ。","\\n[1] transcended the realm of mortality.","\\n[1]는 인간을 초월한 존재가 된 것이다.","\\n[1]成為了超越人類的存在。","\\n[1]成为了超越人类的存在。"]);
_tr_m.push(["武器の神\\n[1]は、今日も世界を見守り続けている。","And as the god of weaponry, \\n[1] continues to watch over the world and its armaments to this day.","무기의 신 \\n[1]는 오늘도 세계를 지켜보고 있다.","武器之神\\n[1]，今天也會繼續看顧著這個世界。","武器之神\\n[1]，今天也会继续照看着这个世界。"]);
_tr_m.push(["「ロマン」を守るため―――。","To protect its spirit of adventure...","「로망」을 지키기 위해.","為了要守護「浪漫」────","为了守护「浪漫」────"]);
_tr_m.push(["END 4/4  「\\n[1]、神となる」","ENDING 4/4 \"\\n[1] the Divine\"","END 4/4  「\\n[1], 신이 되다」","END 4/4 「\\n[1]，成為神」","END 4/4「\\n[1]，成为神」"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["\\i[141]アルマの石を手に入れた！","You got an \\i[141]Arma Stone!","\\i[141]알마의 돌을 획득했다!","獲得了\\i[141]魂石！","获得了\\i[141]魂石！"]);
_tr_m.push(["\\i[261]薬草を手に入れた！","You got an \\i[261]herb!","\\i[261]약초를 획득했다!","獲得了 \\i[261]藥草！","获得了\\i[261]药草！"]);
_tr_m.push(["金床だ。 ここで武器の手入れができそうだ。","It's an anvil. You can use it to maintain your weapons.","모루다. 여기에서 무기를 손질할 수 있을 것 같다.","這是鐵砧。看來可以在這裡保養武器。","這是鐵砧。看來可以在這裡保養武器。"]);
_tr_m.push(["金床だ。ここで武器の手入れができそうだ。 （\\i[141]所持数 ✕ \\v[8] ）\\! ","It's an anvil. You can use it to maintain your weapons.<br>(\\i[141] Owned: \\v[8])\\! ","모루다. 여기에서 무기를 손질할 수 있을 것 같다.<br>(\\i[141]소지 수 ✕ \\v[8])\\! ","這是鐵砧。看來可以在這裡保養武器。<br>（\\i[141]持有數✕ \\v[8]）\\! ","这是铁砧。可以在这里保养武器。<br>（\\i[141]持有数✕\\v[8]）\\! "]);
_tr_m.push([">修理する",">Repair",">수리하기","＞修理","＞修理"]);
_tr_m.push(["武器はすべて新品のようにピカピカだ。","Makes your weapons like new.","무기는 전부 새것처럼 반짝거린다.","武器全都變得像新的一樣，閃閃發亮。","武器全都变得像新的一样，闪闪发亮。"]);
_tr_m.push(["持っている武器をすべて修理した！ 耐久値が回復した！","All of your weapons have been repaired! They've regained their durability!","갖고 있는 무기를 모두 수리했다! 내구도가 회복되었다!","持有的武器全都修理完畢！耐久值恢復了！","持有的武器全都修理完毕！耐久值恢复了！"]);
_tr_m.push([">\\v[3]",">\\v[3]",">\\v[3]",">\\v[3]",">\\v[3]"]);
_tr_m.push(["\\i[\\v[3]]\\v[4] を研ぎますか？ \\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! ","Sharpen the \\i[\\v[3]]\\v[4]? <br>\\>(\\i[141] x\\v[8] → \\v[5])\\! ","\\i[\\v[3]]\\v[4](을)를 예리 있습니까? <br>\\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! ","要研磨\\i[\\v[3]]\\v[4]嗎？ <br>\\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! ","\\i[\\v[3]]\\v[4]嗎？ <br>\\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! "]);
_tr_m.push([">はい",">Yes",">예","＞是","＞是"]);
_tr_m.push(["\\i[\\v[3]]\\v[4]を研いだ！ しばらくの間斬れ味が上昇する！","Sharpened the \\i[\\v[3]]\\v[4]! It'll be sharper than usual for a while!","\\i[\\v[3]]\\v[4](을)를 연마했다! 한동안 예리도가 상승한다!","研磨了\\i[\\v[3]]\\v[4]！暫時提升了鋒利程度！","研磨了\\i[\\v[3]]\\v[4]！暂时提升了锋利程度！"]);
_tr_m.push([">いいえ",">No",">아니요","＞否","＞否"]);
_tr_m.push([">\\v[4]",">\\v[4]",">\\v[4]",">\\v[4]",">\\v[4]"]);
_tr_m.push(["\\i[\\v[3]]\\v[4] にメッキをしますか？ \\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! ","Galvanize the \\i[\\v[3]]\\v[4]? <br>\\>(\\i[141] x\\v[8] → \\v[5])\\! ","\\i[\\v[3]]\\v[4](을)를 도금 하시겠습니까? <br>\\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! ","要鍍金\\i[\\v[3]]\\v[4]嗎？ <br>\\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! ","\\i[\\v[3]]\\v[4]嗎？ <br>\\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! "]);
_tr_m.push([">はい",">Yes",">예","＞是","＞是"]);
_tr_m.push(["\\i[\\v[3]]\\v[4]をメッキした！ 耐久値が減らなくなった！","Galvanized the \\i[\\v[3]]\\v[4]! Durability will not be lost!","\\i[\\v[3]]\\v[4](을)를 도금했다! 내구도가 줄지 않게 되었다!","鍍金了\\i[\\v[3]]\\v[4]！耐久值不會下降了！","给\\i[\\v[3]]\\v[4]镀了金！耐久值不会下降了！"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push([">\\v[5]",">\\v[5]",">\\v[5]",">\\v[5]",">\\v[5]"]);
_tr_m.push(["\\i[\\v[3]]\\v[4]を鍛えますか？ \\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! ","Temper the \\i[\\v[3]]\\v[4]? <br>\\>(\\i[141] x\\v[8] → \\v[5])\\! ","\\i[\\v[3]]\\v[4](을)를 단련하시겠습니까? <br>\\>(\\i[141] ✕ \\v[8] → \\v[5])\\! ","要鍛造\\i[\\v[3]]\\v[4]嗎？ <br>\\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! ","要鍛造\\i[\\v[3]]\\v[4]嗎？ <br>\\>（\\i[141] ✕ \\v[8] → \\v[5] ）\\! "]);
_tr_m.push([">はい",">Yes",">예","＞好","＞是"]);
_tr_m.push(["\\i[\\v[3]]\\v[4]を鍛えた！ 攻撃力が上昇した！","Tempered the \\i[\\v[3]]\\v[4]! ATK increased!","\\i[\\v[3]]\\v[4](을)를 단련했다! 공격력이 상승했다!","鍛造了\\i[\\v[3]]\\v[4]！攻擊力上升了！","锻造了\\i[\\v[3]]\\v[4]！攻击力上升了！"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push([">説明を聞く",">Explanation",">설명 듣기","＞聆聽說明","＞聆聽說明"]);
_tr_m.push([">やめる",">Cancel",">그만둔다","＞算了","＞算了"]);
_tr_m.push(["返事がない。ただのしかばねのようだ。","There's no response. It's just a corpse.","대답이 없다. 그냥 시체인 모양이다.","沒有回應。看來像是一具普通的屍體。","没有回应。看来像是一具普通的尸体。"]);
_tr_m.push(["返事がない。ただのしかばねのようだ。","There's no response. It's just a corpse.","대답이 없다. 그냥 시체인 모양이다.","沒有回應。看來像是一具普通的屍體。","沒有回應。看來像是一具普通的屍體。"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["金床だ。 ここで武器の手入れができそうだ。","An anvil. You could maintain your weapons here.","모루다. 여기에서 무기를 손질할 수 있을 것 같다.","這是鐵砧。看來可以在這裡保養武器。","這是鐵砧。看來可以在這裡保養武器。"]);
_tr_m.push(["金床だ。ここで武器の修理ができそうだ。\\! ","It's an anvil. You can use it to repair your weapons.\\! ","모루다. 여기에서 무기를 수리할 수 있을 것 같다.\\! ","這是鐵砧。看來可以在這裡修理武器。\\! ","這是鐵砧。看來可以在這裡修理武器。\\! "]);
_tr_m.push([">修理する",">Repair",">수리하기","＞修理","＞修理"]);
_tr_m.push(["武器はすべて新品のようにピカピカだ。","Makes your weapons like new.","무기는 전부 새것처럼 반짝거린다.","武器全都變得像新的一樣，閃閃發亮。","武器全都變得像新的一樣，閃閃發亮。"]);
_tr_m.push(["持っている武器をすべて修理した！ 耐久値が回復した！","All of your weapons have been repaired! They've regained their durability!","갖고 있는 무기를 모두 수리했다! 내구도가 회복되었다!","持有的武器全都修理完畢！耐久值恢復了！","持有的武器全都修理完畢！耐久值恢復了！"]);
_tr_m.push([">やめる",">Cancel",">그만둔다","＞算了","＞算了"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["「ここはヘイワ村 　→食堂　↑武器屋」","[Welcome to Peaceville  →Cafe  ↑Weapons Shop]","「여기는 평화 마을 　→식당　↑무기점」","「這裡是和平村 →餐廳　↑武器店」","「这里是和平村 →餐厅 ↑武器店」"]);
_tr_m.push(["墓には何も書かれていない・・・。","There's nothing written on the tombstone...","무덤에는 아무것도 적혀 있지 않다・・・","墓碑上什麼都沒寫……","墓碑上什么都没写……"]);
_tr_m.push(["プルルル！ガチャ！ \\nw[鍛冶屋]","Riiing! Riiing! \\nw[blacksmith]","뚜루루루! 철컥! \\nw[대장장이]","嘟嚕嚕嚕！喀！ \\nw[鐵匠]","嘟噜噜噜！喀！\\nw[铁匠]"]);
_tr_m.push(["いらっしゃいませ！ こちら鍛冶屋電話サービス「kazizon」です！ \\nw[鍛冶屋]","Hello there! This is Mr. Smith, the source for all your smithing needs! \\nw[blacksmith]","안녕하세요! 대장간 전화 서비스 「dodream」입니다! \\nw[대장장이]","歡迎光臨！這裡是鐵匠鋪電話服務「kazizon」！ \\nw[鐵匠]","欢迎光临！这里是铁匠铺电话服务「kazizon」！\\nw[铁匠]"]);
_tr_m.push(["あらまあ、\\n[1]さん！ いつもごひいきにしていただいております！ \\nw[鍛冶屋]","Why if it isn't, \\n[1]! I'm always happy to hear from my regular customers! \\nw[blacksmith]","아이고, \\n[1] 씨! 항상 애용해 주셔서 감사합니다! \\nw[대장장이]","哎呀，是\\n[1]啊！謝謝您平常的照顧！ \\nw[鐵匠]","哎呀，是\\n[1]啊！谢谢您平常的照顾！\\nw[铁匠]"]);
_tr_m.push(["あの武器を今すぐ欲しいけど、探しに行くのがめんどくさい。 そんなときってありますよね！ \\nw[鍛冶屋]","Don't you find yourself wanting a certain weapon, but can't be bothered to go find it? \\nw[blacksmith]","당장 그 무기가 필요한데 찾으러 가기 귀찮다! 그럴 때가 있으시죠? \\nw[대장장이]","有時候您會立即很想要某件武器，但又懶得去找。這種事爾偶爾會發生對吧！ \\nw[鐵匠]","有时候您会突然很想要某件武器，但又懒得去找。这种事偶尔会发生对吧！\\nw[铁匠]"]);
_tr_m.push(["そんなとき、ウチに電話していただければ！ 光の速さで作ってお届けに参ります！ \\nw[鍛冶屋]","If that's ever your situation, give me a call! I'll make you one faster than the speed of light! \\nw[blacksmith]","바로 그럴 때, 저희에게 전화를 주시면! 빛의 속도로 만들어 배달해 드립니다! \\nw[대장장이]","這種時候，只要打電話給我們！我們就會用光速把武器做好送到您府上！ \\nw[鐵匠]","这种时候，只要打电话给我们！我们就会用光速把武器做好送到您府上！\\nw[铁匠]"]);
_tr_m.push(["吉●屋よりも早く！す●屋よりも早く！ それが鍛冶屋「kazizon」のモットーでございます！ \\nw[鍛冶屋]","Faster than McDowell's! Faster than Burger Queen! Mr. Smith always delivers! \\nw[blacksmith]","맥●●드보다 빠르게! 도●노보다 빠르게! 그것이 바로 대장간 「dodream」의 좌우명입니다! \\nw[대장장이]","比吉●家還快！比Su●ya還快！這就是鐵匠鋪「kazizon」的守則！ \\nw[鐵匠]","比吉●家还快！比Su●ya还快！这就是铁匠铺「kazizon」的守则！\\nw[铁匠]"]);
_tr_m.push(["では改めまして・・・。 \\nw[鍛冶屋]","Okay, I'm done with my spiel now. \\nw[blacksmith]","자, 그럼 다시 정식으로・・・ \\nw[대장장이]","那麼，就再次向您打聲招呼…… \\nw[鐵匠]","那么，再次向您打声招呼……\\nw[铁匠]"]);
_tr_m.push(["いらっしゃいませ！ こちら鍛冶屋電話サービス「kazizon」です！\\!  \\nw[鍛冶屋]","Hello there! This is Mr. Smith, the source for all your smithing needs!\\!  \\nw[blacksmith]","안녕하세요! 대장간 전화 서비스 「dodream」입니다!\\!  \\nw[대장장이]","歡迎光臨！這裡是鐵匠鋪電話服務「kazizon」！\\!  \\nw[鐵匠]","欢迎光临！这里是铁匠铺电话服务「kazizon」！\\! \\nw[铁匠]"]);
_tr_m.push([">武器を作成したい",">Make a weapon",">무기 주문하기","＞想要製作武器","＞想要制作武器"]);
_tr_m.push(["ご注文ありがとうございます！ 今から作って発送いたしますねー！ \\nw[鍛冶屋]","Thank you very much! I'll make it right away and ship it straight to your door! \\nw[blacksmith]","주문해 주셔서 감사합니다! 즉시 제작해서 배달해 드리겠습니다~! \\nw[대장장이]","謝謝您的訂購！我們立刻進行打造，完成後馬上寄出！ \\nw[鐵匠]","谢谢您的订购！我们立刻进行打造，完成后会马上寄出！\\nw[铁匠]"]);
_tr_m.push(["作成した武器が届いた！ \\v[3]","Your weapon order arrived! \\v[3]","주문한 무기가 도착했다! \\v[3]","打造完成的武器送到了！ \\v[3]","打造完成的武器送到了！\\v[3]"]);
_tr_m.push(["まだ武器はナニモツクレナイ。 \\nw[鍛冶屋]","I can't make you a weapon right now. \\nw[blacksmith]","아직은 무기를 만들 수 없다. \\nw[대장장이]","目前無法製作任何武器。 \\nw[鐵匠]","目前无法制作任何武器。\\nw[铁匠]"]);
_tr_m.push([">説明を聞く",">Listen to Explanation",">설명 듣기","＞聆聽說明","＞聆听说明"]);
_tr_m.push([">切る",">Hang Up",">끊기","＞掛斷電話","＞挂断电话"]);
_tr_m.push(["「ウェポチチ ここに眠る」 と書かれている・・・。","It reads \"R.I.P. Dirk\"...","「웨포파파 여기에 잠들다」라고 적혀 있다・・・","上面寫著「武父沉眠於此」……","上面写着「武父沉眠于此」……"]);
_tr_m.push(["「ブレイグの妻 ここに眠る」 と書かれている・・・。","It reads \"R.I.P. Wife of Blaig\"...","「브레이그의 아내 여기에 잠들다」라고 적혀 있다・・・","上面寫著「布雷格之妻沉眠於此」……","上面写着「布雷格之妻沉眠于此」……"]);
_tr_m.push(["『冒険の手引き』と書かれた本がある。\\! ","It is a book entitled \"The Adventurer's Handbook\".\\! ","『모험의 길잡이』라고 적힌 책이 있다.\\! ","有一本書，封面寫著「冒險指南」。\\! ","有一本书，封面写着「冒险指南」。\\! "]);
_tr_m.push([">読む",">Read",">읽기","＞閱讀","＞阅读"]);
_tr_m.push(["「その一・・・何でも拾って試してみよ。 　この世には見かけによらず強力な武器も多い。」 \\nw[『冒険の手引き』]","\"Rule 1: Pick up everything. There are some inconspicuous but powerful weapons in this world.\" \\nw[\"The Adventurer's Handbook\"]","「하나・・・ 뭐든지 주워서 시험해 보아라. 이 세상에는 겉보기와 다르게 강력한 무기도 많다.」 \\nw[『모험의 길잡이』]","「其一……什麼東西都要撿起來試試看。這個世界有許多從外觀看不出來的強大武器。」 \\nw[「冒險指南」]","「其一……什么东西都要捡起来试试看。这个世界有许多从外观看不出来的强大武器。」\\nw[「冒险指南」]"]);
_tr_m.push(["「その二・・・死にそうになったら薬草を飲め。 　薬草はたくさん持っていると安心だ。」 \\nw[『冒険の手引き』]","\"Rule 2: Use an herb when you're about to die. The more you carry, the safer you are.\" \\nw[\"The Adventurer's Handbook\"]","「둘・・・ 죽을 것 같으면 약초를 먹어라. 약초는 많이 갖고 다니면 든든하다.」 \\nw[『모험의 길잡이』]","「其二……快死的時候就服用藥草。攜帶大量藥草會更加安心。」 \\nw[「冒險指南」]","「其二……快死的时候就服用药草。携带大量药草会更加安心。」\\nw[「冒险指南」]"]);
_tr_m.push(["「その三・・・金床を使いこなせ。 　金床では武器の手入れや強化ができる。」 \\nw[『冒険の手引き』]","\"Rule 3: Make good use of the anvil. You can maintain and upgrade weapons with it.\" \\nw[\"The Adventurer's Handbook\"]","「셋・・・ 모루를 활용하라. 모루에서는 무기를 손질하거나 강화할 수 있다.」 \\nw[『모험의 길잡이』]","「其三……善用鐵砧。鐵砧能夠保養及強化武器。」 \\nw[「冒險指南」]","「其三……善用铁砧。铁砧能够保养及强化武器。」\\nw[「冒险指南」]"]);
_tr_m.push(["「武器の神の祝福があらんことを―――。」 \\nw[『冒険の手引き』]","\"May the god of weaponry be with you...\" \\nw[\"The Adventurer's Handbook\"]","「무기의 신의 축복이 함께하기를.」 \\nw[『모험의 길잡이』]","「願武器之神的祝福與你同在────」 \\nw[「冒險指南」]","「愿武器之神的祝福与你同在────」\\nw[「冒险指南」]"]);
_tr_m.push(["筆者の名は記されていない。 誰が書いた本なのだろうか。","The author's name isn't anywhere to be seen. Who could've written it?","저자의 이름은 적혀 있지 않다. 누가 쓴 책일까?","書上並未記載作者的姓名。不知道這本書是誰寫的呢？","书上并未记载作者的姓名。不知道这本书是谁写的呢？"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push(["\\n[1]は井戸の中を覗いてみた。 しかし底は暗く何も見えない・・・。","\\n[1] peeked into the well. It's too deep and dark to see anything.","\\n[1]는 우물 안을 들여다보았다. 하지만 바닥은 캄캄해서 아무것도 보이지 않는다・・・","\\n[1]探頭往井裡看去。然而井底一片黑暗，什麼也看不見……","\\n[1]探头往井里看去。然而井底一片黑暗，什么也看不见……"]);
_tr_m.push(["なんと井戸の中から 眩い光が放たれている・・・！","What's this?! A bright light shines from within the well!","오늘부터 편리 명령","井裡突然散發出一陣耀眼的光芒……！","井里突然散发出一阵耀眼的光芒……！"]);
_tr_m.push(["井戸チャレンジに挑戦しますか？\\! ","Take the Well Challenge?\\! ","우물 챌린지에 도전하시겠습니까?\\! ","要進行水井挑戰嗎？\\! ","要进行水井挑战吗？\\! "]);
_tr_m.push([">挑戦する",">Yes",">도전하기","＞挑戰","＞挑战"]);
_tr_m.push([">説明を聞く",">Listen to Explanation",">설명 듣기","＞聆聽說明","＞聆听说明"]);
_tr_m.push([">やめる",">Cancel",">그만둔다","＞算了","＞算了"]);
_tr_m.push(["挑戦するチャレンジを選んでください。\\! ","Select the Challenge you wish to attempt.\\! ","도전할 종목을 선택하세요.\\! ","請選擇要挑戰的選項。\\! ","请选择要挑战的选项。\\! "]);
_tr_m.push([">生き延びろ！サバイバル島",">Try to Survive! Survival Island",">살아남아라! 서바이벌 아일랜드","＞活下去！生存島","＞活下去！生存岛"]);
_tr_m.push(["生き延びろ！サバイバル島に挑戦しますか？ （最高スコア：\\v[31]）\\! ","Attempt the \"Try to Survive! Survival Island\" challenge?<br>(High Score: \\v[31])\\! ","살아남아라! 서바이벌 아일랜드에 도전하시겠습니까?<br>(최고 점수: \\v[31])\\! ","要挑戰「活下去！生存島」嗎？<br>（最高分數：\\v[31]）\\! ","挑战「活下去！生存岛」吗？<br>（最高分数：\\v[31]）\\! "]);
_tr_m.push([">挑戦する！",">Take the Challenge!",">도전한다!","＞挑戰！","＞挑战"]);
_tr_m.push(["\\n[1]は井戸の中に飛び込んだ！","\\n[1] jumped into the well!","\\n[1]는 우물 안으로 뛰어들었다!","\\n[1]跳進了井裡！","\\n[1]跳进了井里！"]);
_tr_m.push([">ランキングを見る",">See Rankings",">랭킹 보기","＞檢視排行榜","＞檢視排行榜"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["ランキング機能は RPGアツマール版のみで使用できます。 申し訳ございません。","Sorry, but the rankings feature is only available in the RPG Atsumaru version.","죄송하지만 랭킹 기능은 RPG 콜렉션 버전에서만 사용 가능합니다.","排行榜功能僅限「RPGアツマール版」使用。敬請見諒。","排行榜功能僅限「RPGアツマール版」使用。敬請見諒。"]);
_tr_m.push([">説明を聞く",">Explanation",">설명 듣기","＞聆聽說明","＞聆听说明"]);
_tr_m.push([">やめる",">Cancel",">그만둔다","＞算了","＞算了"]);
_tr_m.push([">やめる",">Cancel",">그만둔다","＞算了","＞算了"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","宝箱里似乎空无一物。"]);
_tr_m.push(["料理のような何かが置いてある。 \\n[1]は震えている！","There's something resembling food. \\n[1] shudders!","요리 같은 것이 놓여 있다. \\n[1]는 떨고 있다!","裡面放著看似料理的東西。 \\n[1]開始顫抖！","里面放着看似料理的东西。\\n[1]开始颤抖！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["\\n[1]は冷蔵庫の中を覗いてみた。","\\n[1] peeks into the fridge.","\\n[1]는 냉장고 안을 들여다보았다.","\\n[1]往冰箱裡看去。","\\n[1]往冰箱里看去。"]);
_tr_m.push(["色々な食材が所狭しと並べられている。","It is packed with groceries.","다양한 식재료가 빼곡하게 늘어서 있다.","裡面塞滿了各式各樣的食材。","里面塞满了各式各样的食材。"]);
_tr_m.push(["『今日から私も便利コマンド』と書かれた本がある。\\! ","It is a book entitled \"Useful Commands 101\".\\! ","『오늘부터 나도 간편 명령』이라고 적힌 책이 있다.\\! ","有一本書，封面寫著「從今天起我也會使用快捷指令」。\\! ","有一本书，封面写着「从今天起我也会使用快捷指令」。\\! "]);
_tr_m.push([">読む",">Read",">읽기","＞閱讀","＞阅读"]);
_tr_m.push(["「この世界に隠された「便利コマンド」。 　使いこなすことであなたの冒険が微妙に楽になるでしょう。」 \\nw[『今日から私も便利コマンド』]","\"There are useful commands hidden in this world. Mastering them will slightly improve your experience.\" \\nw[\"Useful Commands 101\"]","「이 세계에 숨겨진 「간편 명령」. 잘 활용하면 당신의 모험이 왠지 모르게 편해질 것입니다.」 \\nw[『오늘부터 나도 간편 명령』]","「隱藏在這個世界中的『快捷指令』。如果能善用快捷指令，就能讓您的冒險變得稍微輕鬆一點。」 \\nw[「從今天起我也會使用快捷指令」]","「隐藏在这个世界中的『快捷指令』。如果能善用快捷指令，就能让您的冒险变得稍微轻松一点。」\\nw[「从今天起我也会使用快捷指令」]"]);
_tr_m.push(["「便利コマンドは、キーボードでしか使えない。 　マウスやスマートフォンの方、ごめんなさい！」 \\nw[『今日から私も便利コマンド』]","\"Useful commands can only be used via the keyboard. To those of you using a mouse or smartphone...sorry!\" \\nw[\"Useful Commands 101\"]","「간편 명령은 키보드로만 사용할 수 있습니다. 마우스 및 스마트폰을 이용하시는 여러분, 죄송합니다!」 \\nw[『오늘부터 나도 간편 명령』]","「快捷指令只能透過鍵盤使用。用滑鼠或手機遊玩的玩家，敬請見諒！」 \\nw[「從今天起我也會使用快捷指令」]","「快捷指令只能借助键盘使用。用鼠标或手机的玩家，敬请见谅！」\\nw[「从今天起我也会使用快捷指令」]"]);
_tr_m.push(["しかし\\n[1]には書いてある意味がわからなかった！","However, \\n[1] does not understand a word of it!","그러나 \\n[1]는 무슨 뜻인지 이해할 수 없었다!","然而，\\n[1]卻看不懂書上在寫什麼！","然而，\\n[1]却看不懂书上写了些什么！"]);
_tr_m.push([">いいえ",">No",">아니요","＞否","＞否"]);
_tr_m.push(["『アルマの石についての本だよ』と書かれた本がある。\\! ","It is a book entitled \"All About Arma Stones\".\\! ","『알마의 돌에 대한 책이야』라고 적힌 책이 있다.\\! ","有一本書，封面寫著「這是跟魂石有關的書喔」。\\! ","有一本书，封面写着「这是跟魂石有关的书喔」。\\! "]);
_tr_m.push([">読む",">Read",">읽기","＞閱讀","＞阅读"]);
_tr_m.push(["「武器は『研ぐ』『メッキする』『鍛える』ことで 　強化することができるが、」 \\nw[『アルマの石についての本だよ』]","\"Weapons can be upgraded by sharpening them, galvanizing them, or tempering them.\" \\nw[\"All About Arma Stones\"]","「무기는 『연마』 『도금』 『단련』으로 강화할 수 있는데」 \\nw[『알마의 돌에 대한 책이야』]","「武器可以藉由『研磨』『鍍金』『鍛造』等方式加以強化，」 \\nw[「這是跟魂石有關的書喔」]","「武器可以借由『研磨』『镀金』『锻造』等方式加以强化，」\\nw[「这是跟魂石有关的书喔」]"]);
_tr_m.push(["「その強化に必要なのが『\\i[141]アルマの石』。 　武器の神の力を宿す石だ。」 \\nw[『アルマの石についての本だよ』]","\"\\i[141]Arma Stones are required to upgrade any weapon. They contain the power of the god of weaponry.\" \\nw[\"All About Arma Stones\"]","「이때 필요한 것이 『\\i[141]알마의 돌』. 바로 무기의 신이 깃든 돌이다.」 \\nw[『알마의 돌에 대한 책이야』]","「但強化時需要使用『\\i[141]魂石』。這是寄宿著武器之神力量的石頭。」 \\nw[「這是跟魂石有關的書喔」]","「但强化时需要使用『\\i[141]魂石』。这是寄宿着武器之神力量的石头。」\\nw[「这是跟魂石有关的书喔」]"]);
_tr_m.push(["「みんなも\\i[141]アルマの石を集めて 　楽しい武器強化ライフを送ろう！」 \\nw[『アルマの石についての本だよ』]","\"Find some \\i[141]Arma Stones and have fun upgrading your weapons!\" \\nw[\"All About Arma Stones\"]","「모두 \\i[141]알마의 돌을 모아 무기 강화 라이프를 즐기자!」 \\nw[『알마의 돌에 대한 책이야』]","「大家也來收集\\i[141]魂石，過著快樂的強化武器生活吧！」 \\nw[「這是跟魂石有關的書喔」]","「大家都来收集\\i[141]魂石，享受快乐的强化武器生活吧！」\\nw[「这是跟魂石有关的书喔」]"]);
_tr_m.push([">いいえ",">No",">아니요","＞否","＞否"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。","\\n[1] looked in the pot.","\\n[1]는 항아리 안을 들여다보았다.","\\n[1]探頭往壺裡看去。","\\n[1]探头往壶里看去。"]);
_tr_m.push(["しかしフタがついていて覗けなかった・・・。","It has a lid, so nothing can be seen.","하지만 뚜껑이 있어 안을 볼 수 없었다・・・","然而壺上有個蓋子，看不到裡面的狀況……","然而壶上有个盖子，看不到里面的状况……"]);
_tr_m.push(["高級なワインが置いてある。 しかし\\n[1]は未成年だった！","There's a bottle of expensive wine. Unfortunately, \\n[1] is underage!","고급 와인이 놓여 있다. 하지만 \\n[1]는 미성년자였다!","裡面放著高級的葡萄酒。然而\\n[1]還未成年！","里面放着高级的葡萄酒。然而\\n[1]还未成年！"]);
_tr_m.push(["美味しそうな生姜焼き定食が置いてある。 \\n[1]はガン見している！","There's a delicious-looking plate of stir-fried pork. \\n[1] stares intently.","맛있어 보이는 돼지고기 생강구이 정식이 놓여 있다. \\n[1]가 뚫어져라 쳐다보고 있다!","裡面放著看起來很好吃的薑燒豬肉定食。 \\n[1]緊盯著那份食物不放！","里面放着看起来很好吃的姜烧猪肉套餐。\\n[1]紧盯着那份食物不放！"]);
_tr_m.push(["熱々の野菜炒めセットが置いてある。 \\n[1]はお腹が減ってきた！","There's a piping hot stir-fried veggie plate. \\n[1] is hungry!","따끈따끈한 채소 볶음 세트가 놓여 있다. \\n[1]는 배가 고프기 시작했다!","裡面放著熱騰騰的炒青菜套餐。 \\n[1]感到肚子餓了！","里面放着热腾腾的炒青菜套餐。\\n[1]感到肚子饿了！"]);
_tr_m.push(["豪快な魚の丸焼きが置いてある。 \\n[1]はよだれをたらした！","There's a magnificent grilled fish. \\n[1] is drooling!","푸짐한 생선 통구이가 놓여 있다. \\n[1]는 군침을 흘렸다!","裡面放著豪邁的整隻烤魚。 \\n[1]開始流口水了！","里面放着豪迈的整只烤鱼。\\n[1]开始流口水了！"]);
_tr_m.push(["そこのピランカという女性の日記だ。 \\nw[name]","It's the diary of a woman named Piranka. \\nw[name]","Piranka라는 여자의 일기다. \\nw[name]","那是一位名叫Piranka的女性寫的日記。 \\nw[name]","那是一位名叫Piranka的女性写的日记。 \\nw[name]"]);
_tr_m.push(["ハロー！私はキャサリン！情熱的でラブリーなクマよ！ \\nw[name]","Howdy! I'm Catherine! I'm a passionate and lovely bear! \\nw[name]","헬로! 난 Catherine! 정열적이고 러블리한 곰이야! \\nw[name]","哈囉！我是Catherine！是一隻熱情又可愛的熊！ \\nw[name]","哈罗！我是Catherine！是一只热情又可爱的熊！\\nw[name]"]);
_tr_m.push(["武器商人のお姉さん！ 突然だけどキャサリン、見せてほしい武器があるの！ \\nw[name]","So, Miss Weapons Merchant! I know it's sudden, but there's a weapon I want you to show me! \\nw[name]","무기 상인 언니! 갑작스럽겠지만 캐서린이 보고 싶은 무기가 있어! \\nw[name]","武器商人大姐！抱歉突然打擾妳，我想要看某種武器！ \\nw[name]","武器商人大姐！抱歉突然打扰你，我想要看某种武器！\\nw[name]"]);
_tr_m.push(["私の見たい武器は・・・燃える炎のように情熱的な ズバリ「炎の出る武器」！ \\nw[name]","The weapon I want to see is fiery and passionate, a \"weapon that shoots flames\"! \\nw[name]","내가 보고 싶은 무기는・・・ 타오르는 불꽃처럼 정열적인, 바로 「불꽃이 나오는 무기」! \\nw[name]","我想看的武器是……有如燃燒的火焰般熱情……也就是「會發出火焰的武器」！ \\nw[name]","我想看的武器是……有如燃烧的火焰般热情……也就是「会发出火焰的武器」！\\nw[name]"]);
_tr_m.push(["そんな武器を見つけたら、装備して話しかけてね！ \\nw[name]","If you see something like that, equip it and come talk to me again! \\nw[name]","그런 무기를 발견하면 장비한 후에 말을 걸어 줘! \\nw[name]","如果妳找到了那種武器，就裝備在身上再跟我說話吧！ \\nw[name]","如果你找到了那种武器，就装备在身上再跟我说话吧！\\nw[name]"]);
_tr_m.push(["それじゃあ、よろしくね！ \\nw[name]","I hope you find one! \\nw[name]","그럼 부탁할게! \\nw[name]","那麼，就麻煩妳囉！ \\nw[name]","那么，就麻烦你咯！\\nw[name]"]);
_tr_m.push(["あら！「炎の出る武器」、持ってきたようね！ \\nw[name]","Whoa! You found a \"weapon that shoots flames\"! \\nw[name]","어머! 「불꽃이 나오는 무기」를 가져왔나 보네! \\nw[name]","哎呀！妳好像把「會發出火焰的武器」帶來了呢！ \\nw[name]","哎呀！你好像把「会发出火焰的武器」带来了呢！\\nw[name]"]);
_tr_m.push(["ん・・・♡すごい熱気！ \\nw[name]","Phew! It's a hot one! \\nw[name]","음・・・♡ 엄청난 열기! \\nw[name]","嗯……♡好強烈的熱氣！ \\nw[name]","嗯……♡好强烈的热气！\\nw[name]"]);
_tr_m.push(["そう・・・私の求める恋とは、 この武器のように熱く激しい恋！！ \\nw[name]","That's the kind of love I seek. Hot and passionate like this weapon! \\nw[name]","그래・・・ 내가 바라는 사랑은 이 무기처럼 뜨겁고 격렬한 사랑!! \\nw[name]","沒錯……我所追求的戀愛，就是像這把武器一樣熱情激烈的戀愛！！ \\nw[name]","没错……我所追求的恋爱，就是像这把武器一样热情激烈的恋爱！！\\nw[name]"]);
_tr_m.push(["\\{\\{あん♡あん♡あん♡あん♡あん♡ \\nw[name]","\\{\\{Ungh... Augh... Ough... \\nw[name]","\\{\\{앙♡ 앙♡ 앙♡ 앙♡ 앙♡ \\nw[name]","\\{\\{啊嗯♡ 啊嗯♡ 啊嗯♡ 啊嗯♡ 啊嗯♡ \\nw[name]","\\{\\{啊嗯♡ 啊嗯♡ 啊嗯♡ 啊嗯♡ 啊嗯♡\\nw[name]"]);
_tr_m.push(["・・・ごめん。取り乱したわ。 \\nw[name]","Oh, excuse me. I got all worked up. \\nw[name]","・・・미안, 나도 모르게 그만. \\nw[name]","……抱歉。我一時失態了。 \\nw[name]","……抱歉。我一时失态了。\\nw[name]"]);
_tr_m.push(["ご褒美をあげるわ。はい、どうぞ！ \\nw[name]","Let me give you a reward. Here you go! \\nw[name]","상을 줄게. 자, 받아! \\nw[name]","我給妳獎勵吧。來，請收下！ \\nw[name]","这是给你的奖励。来，请收下！\\nw[name]"]);
_tr_m.push(["世界には私を含めて４匹のクマがいるわ。 すべてのクマに認められるようがんばって！ \\nw[name]","Including me, there are four of us bears in the world. Try to get their approval too! \\nw[name]","이 세계에는 나를 포함해서 4마리의 곰이 있어. 모든 곰에게 인정받을 수 있도록 열심히 해 봐! \\nw[name]","這個世界上包含我在內總共有四隻熊。妳要努力獲得每一隻熊的認同喔！ \\nw[name]","这个世界上包含我在内总共有四只熊。你要努力获得每一只熊的认同喔！\\nw[name]"]);
_tr_m.push(["私の見たい武器は・・・燃える炎のように情熱的な ズバリ「炎の出る武器」！ \\nw[name]","The weapon I want to see is fiery and passionate, a \"weapon that shoots flames\"! \\nw[name]","내가 보고 싶은 무기는・・・ 타오르는 불꽃처럼 정열적인, 바로 「불꽃이 나오는 무기」! \\nw[name]","我想看的武器是……有如燃燒的火焰般熱情……也就是「會發出火焰的武器」！ \\nw[name]","我想看的武器是……有如燃烧的火焰般热情……也就是「会发出火焰的武器」！\\nw[name]"]);
_tr_m.push(["そんな武器を見つけたら、装備して話しかけてね！ \\nw[name]","If you see something like that, equip it and come talk to me again! \\nw[name]","그런 무기를 발견하면 장비한 후에 말을 걸어 줘! \\nw[name]","如果妳找到了那種武器，就裝備在身上再跟我說話吧！ \\nw[name]","如果你找到了那种武器，就装备在身上再跟我说话吧！\\nw[name]"]);
_tr_m.push(["返事がない。ただのぬいぐるみのようだ。","There's no response. It's just a stuffed animal.","대답이 없다. 그냥 인형인 모양이다.","沒有回應。看來像是一具普通的布偶。","没有回应。看上去像是一具普通的布偶。"]);
_tr_m.push(["世界には私を含めて４匹のクマがいるわ。 すべてのクマに認められるようがんばって！ \\nw[name]","Including me, there are four of us bears in the world. Try to get their approval too!\\nw[name]","이 세계에는 나를 포함해서 4마리의 곰이 있어. 모든 곰에게 인정받을 수 있도록 열심히 해 봐! \\nw[name]","這個世界上包含我在內總共有四隻熊。妳要努力獲得每一隻熊的認同喔！ \\nw[name]","这个世界上包含我在内总共有四只熊。你要努力获得每一只熊的认同喔！\\nw[name]"]);
_tr_m.push(["あら、\\n[1]ちゃんじゃない。 武器屋さんは繁盛してる？ \\nw[name]","Well if it isn't \\n[1]! I take it business is good? \\nw[name]","어머나, \\n[1] 아니야? 무기점은 잘되고? \\nw[name]","哎呀，這不是\\n[1]嗎？妳的武器店生意應該不錯吧？ \\nw[name]","哎呀，这不是\\n[1]吗？你的武器店生意应该不错吧？\\nw[name]"]);
_tr_m.push(["え、伝説の武器を探しに旅に出るって？ すごいじゃな～い！がんばって！！ \\nw[name]","You're on an adventure to find the legendary weapon? Good luck! \\nw[name]","뭐? 전설의 무기를 찾아 여행을 떠난다고? 대단하다~! 열심히 해!! \\nw[name]","咦？妳要踏上旅途，尋找傳說武器？真厲害～！要加油喔！！ \\nw[name]","咦？你要踏上旅途，寻找传说武器？真厉害～！要加油喔！！\\nw[name]"]);
_tr_m.push(["あ、そうだ。 この前なんか強そうな武器もらったんだけど、ほしい？ \\nw[name]","By the by...I got a weapon that looks quite powerful. Want it? \\nw[name]","아, 맞다. 요전에 강해 보이는 무기를 받았는데 줄까? \\nw[name]","啊，對了。我之前拿到一把看起來很強的武器，妳想要嗎？ \\nw[name]","啊，对了。我之前拿到一把看起来很强的武器，你想要吗？\\nw[name]"]);
_tr_m.push(["私じゃ難しくて使えそうにないのよ～。 \\n[1]ちゃんの旅の力になれればと思って！ \\nw[name]","It's probably too much for me to handle. But it might help you on your adventure, \\n[1]! \\nw[name]","나는 어려워서 못 쓸 것 같아~ \\n[1]가 여행하는 데 힘이 되면 좋겠어! \\nw[name]","這太難了，我不會用～。希望這對 \\n[1]妳的旅途有所幫助！ \\nw[name]","这太难了，我不会用～希望这对\\n[1]你的旅途有所帮助！\\nw[name]"]);
_tr_m.push(["でもその代わり、ちょっとおつかいを頼みたいんだけど・・・。\\!  \\nw[name]","In return, I do have a favor to ask, though...\\!  \\nw[name]","하지만 그 대신, 심부름을 하나 부탁하고 싶은데・・・\\!  \\nw[name]","不過相對的，我有件小事想請妳幫忙……\\!  \\nw[name]","不过相对的，我有件小事想请你帮忙……\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞好"]);
_tr_m.push([">いいえ",">No",">아니요","＞不要","＞不要"]);
_tr_m.push(["あら、忙しいかしら？また時間が空いたらお願いね。 \\nw[name]","Too busy, eh? Well, whenever you got the time then. \\nw[name]","어머, 바쁘니? 나중에 시간이 생기면 부탁할게. \\nw[name]","哎呀，妳很忙嗎？那等妳有空的時候我再拜託妳吧。 \\nw[name]","哎呀，你很忙吗？那等你有空的时候我再拜托你吧。\\nw[name]"]);
_tr_m.push(["あら、ありがとう！  \\nw[name]","Really? Cool beans! \\nw[name]","어머, 고마워!  \\nw[name]","哎呀，謝謝妳！ \\nw[name]","哎呀，谢谢你！\\nw[name]"]);
_tr_m.push(["あのね。うちの右側に雑草が生えすぎて困ってるのよ。 その雑草を全部掃除してほしいの！ \\nw[name]","Weeds have taken over the land to the right of my house. I'd like you to clear them all out! \\nw[name]","저기, 우리 집 오른쪽에 잡초가 많이 자라서 큰일이야. 그 잡초를 전부 뽑아 줬으면 해! \\nw[name]","我跟妳說，我們家右邊的雜草長太多了，讓我很困擾。希望妳可以把那些雜草全部除掉！ \\nw[name]","我跟你说，我们家右边的杂草长太多了，让我很困扰。希望你可以把那些杂草全部除掉！\\nw[name]"]);
_tr_m.push(["それじゃあ、がんばってね！ \\nw[name]","Okay then! You know what to do! \\nw[name]","그럼 열심히 해! \\nw[name]","那麼，除草加油喔！ \\nw[name]","那么，请加油除草吧！\\nw[name]"]);
_tr_m.push(["ちょっと、\\n[1]ちゃん！ 雑草と間違えて\\i[1041]きれいな花も抜いちゃったでしょ！ \\nw[name]","\\n[1]! You mistook some \\i[1041]Beautiful Flowers for a weed, didn't you! \\nw[name]","잠깐만, \\n[1]! 잡초인 줄 알고 \\i[1041]예쁜 꽃까지 뽑아 버렸지? \\nw[name]","等一下，\\n[1]！妳把花跟雜草搞錯了，連\\i[1041]漂亮的花也被妳拔掉了吧！ \\nw[name]","等一下，\\n[1]！你把花跟杂草搞混了，连\\i[1041]漂亮的花也被你拔掉了吧！\\nw[name]"]);
_tr_m.push(["あの花、きれいだから残しておきたかったのよ。 戻してきてくれると助かるわ～！ \\nw[name]","I wanted to leave them because they were so beautiful. Return them, will you? \\nw[name]","워낙 예쁜 꽃이라서 남겨 두고 싶었단 말이야. 돌려주면 고맙겠어~! \\nw[name]","因為那朵花很漂亮，我想要留下來。如果妳能把花種回去那就太好了～！ \\nw[name]","因为那朵花很漂亮，我想要留下来。如果你能把花种回去那就太好了～！\\nw[name]"]);
_tr_m.push(["メニューから\\i[1041]きれいな花を「破棄」した後、 いったん村の外にでも出れば復活すると思うわ！ \\nw[name]","If you go to the menu, \"Discard\" the \\i[1041]Beautiful Flowers and leave town, they should come back! \\nw[name]","메뉴에서 \\i[1041]예쁜 꽃을 「버리기」한 후에 마을 밖에 나갔다 오면 원래 있던 자리에 다시 나타나! \\nw[name]","從選單中「捨棄」\\i[1041]漂亮的花之後，離開村子一趟再回來，花應該就會復活了！ \\nw[name]","从选单中「舍弃」\\i[1041]漂亮的花之后，离开村子一趟再回来，花应该就会复活了！\\nw[name]"]);
_tr_m.push(["申し訳ないけど、よろしくね～。 \\nw[name]","Sorry for the hassle! \\nw[name]","미안한데 부탁 좀 할게~ \\nw[name]","不好意思，那就麻煩妳啦～ \\nw[name]","不好意思，那就麻烦你啦～\\nw[name]"]);
_tr_m.push(["うちの右側に雑草が生えすぎて困ってるのよ。 その雑草を全部掃除してほしいの！ \\nw[name]","Listen carefully. Weeds have taken over the land to the right of my house. I'd like you to clear them all out! \\nw[name]","집 오른쪽에 잡초가 많이 자라서 큰일이야. 그 잡초를 전부 뽑아 줬으면 해! \\nw[name]","我們家右邊的雜草長太多了，讓我很困擾。希望妳可以把那些雜草全部除掉！ \\nw[name]","我们家右边的杂草长太多了，让我很困扰。希望你可以把那些杂草全部除掉！\\nw[name]"]);
_tr_m.push(["のこりの雑草の数は \\v[3]本 みたいよ。がんばって！ \\nw[name]","Just \\v[3] more and you'll be finished weeding. You can do it! \\nw[name]","잡초는 \\v[3]개 남은 것 같아. 힘내! \\nw[name]","大約還剩下 \\v[3]根雜草。加油吧！ \\nw[name]","大约还剩下\\v[3]根杂草。加油吧！\\nw[name]"]);
_tr_m.push(["おつかれさま～。きれいさっぱり、雑草抜いてきてくれたようね！ \\nw[name]","Good work out there! Now I've got a beautiful and weed-free patch of land! \\nw[name]","수고했어~ 잡초를 깔끔하게 다 뽑아 줬구나! \\nw[name]","辛苦了～妳把雜草拔得很乾淨呢！ \\nw[name]","辛苦了～你把杂草拔得很干净呢！\\nw[name]"]);
_tr_m.push(["ありがとう！これはお礼よ。 \\nw[name]","Thank you so much! Here's your reward. \\nw[name]","고마워! 이거 받아. \\nw[name]","謝謝！這是我的謝禮。 \\nw[name]","谢谢！这是我的谢礼。\\nw[name]"]);
_tr_m.push(["それじゃあ私が預かってるわ。欲しくなったらまた来てね！ \\nw[name]","Then I'll hold onto it for now. Let me know when you want it! \\nw[name]","그럼 내가 갖고 있을게. 나중에 다시 받으러 와! \\nw[name]","那麼，我就先幫妳保管囉。有需要的話再來找我拿吧！ \\nw[name]","那么，我就先帮你保管咯。有需要的话再来找我拿吧！\\nw[name]"]);
_tr_m.push(["強そうな杖でしょ！ 私にはちょっと装備できないみたいなの。 \\nw[name]","Doesn't it look like a powerful staff? I can't equip it, though. \\nw[name]","강해 보이는 지팡이지? 나는 장비할 수 없는 것 같아. \\nw[name]","這把杖看起來很強吧！不過我好像沒辦法裝備。 \\nw[name]","这根手杖看起来很强吧！不过我好像没办法装备。\\nw[name]"]);
_tr_m.push(["聞いた話だと、振るだけで誰でも魔法が使える！だって。 便利な世の中ね～！ \\nw[name]","They say whoever swings it can use magic! Everything is so convenient these days! \\nw[name]","내가 듣기로는 누구나 휘두르기만 해도 마법을 쓸 수 있대. 참 편리하겠지~!? \\nw[name]","聽說只要揮動這把杖，任何人都能使用魔法！這個世界還真是方便呢～！ \\nw[name]","听说只要挥动这根手杖，任何人都能使用魔法！这个世界还真是方便呢～！\\nw[name]"]);
_tr_m.push(["杖を受け取りに来たのね！ \\nw[name]","You've come for the staff! \\nw[name]","지팡이를 받으러 왔구나! \\nw[name]","妳來拿杖了啊！ \\nw[name]","你来拿手杖了啊！\\nw[name]"]);
_tr_m.push(["それじゃあ私が預かってるわ。欲しくなったらまた来てね！ \\nw[name]","Then I'll hold onto it for now. Let me know when you want it!\\nw[name]","그럼 내가 갖고 있을게. 나중에 다시 받으러 와! \\nw[name]","那麼，我就先幫妳保管囉。有需要的話再來找我拿吧！ \\nw[name]","那么，我就先帮你保管咯。有需要的话再来找我拿吧！\\nw[name]"]);
_tr_m.push(["\\n[1]ちゃんは強いもの。 伝説の武器だって夢じゃないわ。がんばってね～！ \\nw[name]","You are a strong one, \\n[1]! I don't doubt you'll actually find the legendary weapon! \\nw[name]","\\n[1]는 강하잖아. 전설의 무기도 분명 찾을 수 있을 거야. 열심히 해~! \\nw[name]","因為\\n[1]很強嘛。要拿到傳說武器也不是夢呢。加油吧～！ \\nw[name]","因为\\n[1]很强嘛。要拿到传说武器也不是梦呢。加油吧～！\\nw[name]"]);
_tr_m.push(["おーい！！息子よ！ どこ行った！おーーい！！！ \\nw[name]","Heeey! Where are you, boy?! Where'd you go?! \\nw[name]","어이~!! 아들아! 어디 갔니? 어이~~!!! \\nw[name]","喂──！！兒子啊！你跑哪去了！喂──！！　\\nw[name]","喂──！！儿子啊！你跑哪去了！喂──！！\\nw[name]"]);
_tr_m.push(["こらこら。二人とも、あんまり はしゃぎすぎるなよっ。 \\nw[name]","You two! Don't be causing a ruckus, y'hear? \\nw[name]","어허! 너희 둘, 너무 시끄럽게 떠들면 못써. \\nw[name]","喂喂！你們兩個別太興奮了。 \\nw[name]","喂喂！你们两个别太兴奋了。\\nw[name]"]);
_tr_m.push(["うわーすげぇ！ そのほうき、いかにも空飛びそう！ \\nw[name]","That is awesome! That broom lets you fly! \\nw[name]","우와~ 대단하다! 그 빗자루, 금방이라도 하늘을 날 것 같아! \\nw[name]","嗚哇！好厲害！那把掃帚感覺可以在天空中飛翔呢！ \\nw[name]","呜哇！好厉害！感觉那把扫帚可以在天空中飞翔呢！\\nw[name]"]);
_tr_m.push(["いいなーいいなー！ \\nw[name]","How cool is that?! I want one! \\nw[name]","좋겠다~ 멋있다~! \\nw[name]","真好─真好─！　\\nw[name]","真好─真好─！　\\nw[name]"]);
_tr_m.push(["でも、まじょさんは無くして 困ってるんじゃないかなあ？ \\nw[name]","I wonder if the witch can do without it though. \\nw[name]","그런데 마녀님은 빗자루를 잃어버려서 큰일 나지 않았을까? \\nw[name]","不過，魔女小姐的掃帚不見了，應該會很困擾吧？ \\nw[name]","不过，魔女小姐的扫帚不见了，应该会很难过吧？\\nw[name]"]);
_tr_m.push(["空飛ぶほうきって知ってるか？ まじょは空飛ぶほうきでスイスイ移動するんだ！ \\nw[name]","Do you know what a flying broom is? The witch uses one to zoom around! \\nw[name]","하늘을 나는 빗자루에 대해 알아? 마녀는 하늘을 나는 빗자루를 타고 빠르게 이동하거든! \\nw[name]","你知道飛天掃帚嗎？魔女就是靠飛天掃帚快速移動的！ \\nw[name]","你知道飞天扫帚吗？魔女就是靠飞天扫帚快速移动的！\\nw[name]"]);
_tr_m.push(["オレもほしいな！空飛ぶほうき！ \\nw[name]","I wish I had one! I want a flying broom! \\nw[name]","나도 갖고 싶어! 하늘을 나는 빗자루! \\nw[name]","我也想要一把飛天掃帚！ \\nw[name]","我也想要一把飞天扫帚！\\nw[name]"]);
_tr_m.push(["ぽん美のやつが、持ってるとか言ってたけど どうせウソだろうな！ \\nw[name]","Ponmi said she has one, but I know she's lying! \\nw[name]","폰미가 갖고 있다던데 어차피 거짓말이겠지! \\nw[name]","器美說她有一把，不過一定是騙人的！ \\nw[name]","器美说她有一把，不过一定是骗人的！\\nw[name]"]);
_tr_m.push(["わーい！わーい！ \\nw[name]","Wooo! Wooo! \\nw[name]","와아~! 와아~! \\nw[name]","耶──！耶──！ \\nw[name]","耶──！耶──！\\nw[name]"]);
_tr_m.push(["おかしいわね。家はどこかしら？ さっきまで住んでたはずなのに・・・。 \\nw[name]","That's odd. Where'd my house go? I was just in it a few moments ago... \\nw[name]","이상하네. 집이 어디지? 방금 전까지 살고 있었는데・・・ \\nw[name]","真奇怪。我的家跑哪去了？剛才我還住在裡面才對啊…… \\nw[name]","真奇怪。我的家跑哪去了？刚才我还住在里面的啊……\\nw[name]"]);
_tr_m.push(["今コーヒーを飲んでたはずなのに。 おかしいわねえ・・・。 \\nw[name]","I was just sipping on some coffee. How strange... \\nw[name]","방금 커피를 마셨는데 이상하네・・・ \\nw[name]","剛剛明明還在喝咖啡呢。真奇怪…… \\nw[name]","刚刚明明还在喝咖啡呢。真奇怪……\\nw[name]"]);
_tr_m.push(["あら。\\n[1]ちゃん、いらっしゃい。 今は夫と息子たちが外で遊んでいるのよ。 \\nw[name]","Oh! Hey there, \\n[1]. My husband is playing with our sons outside. \\nw[name]","어머, \\n[1]. 어서 와. 지금은 남편이랑 아들들이 밖에서 놀고 있어. \\nw[name]","哎呀。\\n[1]，歡迎妳來。現在我先生跟孩子們正在外面玩呢。 \\nw[name]","哎呀。\\n[1]，欢迎你来。现在我先生跟孩子们正在外面玩呢。\\nw[name]"]);
_tr_m.push(["うちは外の風に当たりながら食事ができるんだ。 斬新だろう。 \\nw[name]","You can feel the outside breeze while enjoying a meal here. Novel, isn't it? \\nw[name]","우리 집은 바깥바람을 맞으면서 식사를 할 수 있어. 참신하지? \\nw[name]","我們這裡可以一面吹著戶外的風一邊吃飯。很新穎吧？ \\nw[name]","我们这里可以一面吹着户外的风一边吃饭。很棒吧？\\nw[name]"]);
_tr_m.push(["いらっしゃい。 うちの料理は絶品だよ。 \\nw[name]","Welcome. The food here is exquisite. \\nw[name]","어서 와. 우리 집 요리는 일품이야. \\nw[name]","歡迎光臨。我們店的料理都非常美味喔。 \\nw[name]","欢迎光临。我们店的料理都非常美味喔。\\nw[name]"]);
_tr_m.push(["大将！料理が来てないぞ！ \\nw[name]","Chef! Where's my food?! \\nw[name]","주인장! 요리가 안 나왔어! \\nw[name]","老闆！料理還沒來啊！ \\nw[name]","老板！料理还没来啊！\\nw[name]"]);
_tr_m.push(["ここのメシは中毒性があってウマいんだ！ 毎日通いたくなってしまうよ！ \\nw[name]","Once you eat here, you can't go anywhere else! \\nw[name]","여기 음식은 중독성이 있고 맛있어! 매일 오고 싶어진다니까! \\nw[name]","這裡的料理會讓人上癮，非常好吃！我甚至想每天都來吃呢！ \\nw[name]","这里的料理会让人上瘾，非常好吃！我甚至想每天都来吃呢！\\nw[name]"]);
_tr_m.push(["大将！カタメコイメトッピングヤサイマシマシアブラマシマシ ニンニクマシマシカラメマシマシマヨマシマシで！ \\nw[name]","Chef! I'll have mine al dente with extra veggies, a healthy serving of meat, a heap of garlic, a glop of mayo, and a touch of spice! \\nw[name]","주인장! 면발 단단하게, 국물 진국, 토핑 채소 곱빼기, 기름 곱빼기, 마늘 곱빼기, 고춧가루 곱빼기, 마요네즈 곱빼기로! \\nw[name]","老闆！麵硬湯濃蔬菜加倍加倍豬油加倍加倍大蒜加倍加倍醬油加倍加倍美乃滋加倍加倍！ \\nw[name]","老板！面硬汤浓蔬菜加倍加倍猪油加倍加倍大蒜加倍加倍酱油加倍加倍蛋黄酱加倍加倍！\\nw[name]"]);
_tr_m.push(["どんな料理か楽しみだ。 \\nw[name]","I'm really looking forward to my next meal. \\nw[name]","어떤 요리가 나올지 기대된다. \\nw[name]","真期待會是什麼樣的料理呢。 \\nw[name]","会是什么样的料理呢，真让人期待。\\nw[name]"]);
_tr_m.push(["ふむ。隠し味が効いていておいしいね。 \\nw[name]","Yum. The secret ingredient is really shining through. \\nw[name]","흐음, 비밀 조미료가 들어가서 맛있는걸. \\nw[name]","嗯。提味的食材很出色，相當好吃。 \\nw[name]","嗯。提味的食材很出色，相当好吃。\\nw[name]"]);
_tr_m.push(["私の妻は美しい物が好きでね。 豪華な墓を建ててやれなくて申し訳ないよ。 \\nw[name]","My wife loved beautiful things. I wish I could have built her a more dignified grave. \\nw[name]","내 아내는 아름다운 걸 좋아했지. 화려한 무덤을 만들어 주지 못해서 마음이 아파. \\nw[name]","我太太喜歡美麗的事物，沒辦法替她蓋一座豪華的墳墓，我實在是覺得很對不起她。 \\nw[name]","我太太喜欢美丽的事物，没能替她盖一座豪华的坟墓，我实在是觉得很对不起她。\\nw[name]"]);
_tr_m.push(["私の妻は寂しがり屋だからね。 毎日顔を見せてやりに来ているんだ。 \\nw[name]","My wife couldn't stand being alone. She'd always come and visit. \\nw[name]","내 아내는 외로움을 많이 탔거든. 그래서 매일 얼굴을 보여 주러 오는 거야. \\nw[name]","我太太很怕孤單，所以我才每天都來看她。 \\nw[name]","我太太很怕孤单，所以我才每天都来看她。\\nw[name]"]);
_tr_m.push(["今日もいい天気だわ～。 \\nw[name]","What a beautiful day it is outside. \\nw[name]","오늘도 날씨가 좋네~ \\nw[name]","今天天氣也很好呢～ \\nw[name]","今天天气也很好呢～\\nw[name]"]);
_tr_m.push(["井戸がないとなんだか落ち着かないわ～。 \\nw[name]","I'm sad that the well's gone. \\nw[name]","우물이 없으니까 왠지 마음이 불안해~ \\nw[name]","少了水井，感覺會讓人靜不下心來呢～ \\nw[name]","少了水井，感觉会让人静不下心来呢～\\nw[name]"]);
_tr_m.push(["やだ～。井戸光ってるんですけど～！ ちょっと眩しいんですけど～。 \\nw[name]","What the?! There's a light coming from the well! It's so blinding! \\nw[name]","어머나~ 우물이 빛나고 있어~! 너무 눈부시네~ \\nw[name]","討厭～水井正在發光耶～！有點刺眼耶～ \\nw[name]","哎呀～水井正在发光～！有点刺眼啊～\\nw[name]"]);
_tr_m.push(["あらやだ～。はやく最強の勇者様が 魔王をやっつけてくれないかしら。 \\nw[name]","Oh heavens. Won't the legendary hero hurry and defeat the Demon Lord already? \\nw[name]","아유, 참~ 빨리 최강의 용사님이 마왕을 해치워 주면 좋겠어. \\nw[name]","哎呀，真是的～最強的勇者大人能不能快點去打倒魔王啊。 \\nw[name]","哎呀，真是的～最强的勇者大人能不能快点去打倒魔王啊。\\nw[name]"]);
_tr_m.push(["ふう～。今日もいい天気ね。 \\nw[name]","Wow! Great weather, huh? \\nw[name]","휴우~ 오늘도 날씨가 좋구나. \\nw[name]","呼～今天天氣也很好呢～ \\nw[name]","呼～今天天气也很好呢～\\nw[name]"]);
_tr_m.push(["なんか、井戸がないと落ち着かないわね・・・。 \\nw[name]","I wish the well was still there... \\nw[name]","우물이 없으니까 괜히 불안해・・・ \\nw[name]","總覺得少了水井，讓人靜不下心來呢～ \\nw[name]","少了水井，感觉会让人静不下心来呢～\\nw[name]"]);
_tr_m.push(["なんか井戸すっごい光ってるわね・・・。 すごい主張してきてるわね・・・。 \\nw[name]","That's some crazy light coming from the well! It's so...assertive! \\nw[name]","우물이 엄청나게 빛나고 있어・・・ 존재감이 어마어마해・・・ \\nw[name]","水井好像發出了很強烈的光芒呢……感覺是拚了命在表現自己的存在…… \\nw[name]","水井好像发出了很强烈的光芒呢……感觉是拼了命在表现自己的存在……\\nw[name]"]);
_tr_m.push(["村の外はモンスターがたくさんで危ないわよね。 なんでも、魔王の仕業らしいよ。 \\nw[name]","There are so many monsters beyond the village. It's all the Demon Lord's fault! \\nw[name]","마을 밖은 몬스터가 많아서 위험해. 아무래도 마왕의 소행 같아. \\nw[name]","村外有許多怪物，很危險呢。據說這都是魔王幹的好事。 \\nw[name]","村外有许多怪物，很危险呢。据说这都是魔王干的好事。\\nw[name]"]);
_tr_m.push(["お主、知っておるか？\\!  \\nw[name]","Do you already know what I know?\\!  \\nw[name]","자네, 알고 있나?\\!  \\nw[name]","妳知道嗎？\\!  \\nw[name]","你知道吗？\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞知道","＞知道"]);
_tr_m.push(["じゃあ話しかけんじゃねえ！ \\nw[name]","Then leave me alone! \\nw[name]","그럼 말을 걸지 마! \\nw[name]","既然知道就不要來找我問話嘛！ \\nw[name]","既然知道就不要来找我问话嘛！\\nw[name]"]);
_tr_m.push([">いいえ",">No",">아니요","＞不知道","＞不知道"]);
_tr_m.push(["この大陸には、４つの洞窟がある。 しかし魔物の巣窟となっており人が近づくことはないのじゃ。 \\nw[name]","There are four caves in these lands. They are crawling with monsters, so nobody dares approach them. \\nw[name]","이 대륙에는 4개의 동굴이 있네. 하지만 마물들의 소굴이라 사람들은 가까이 가지 않아. \\nw[name]","這片大陸上有四個洞窟。然而那些地方都成了魔物的巢穴，人類無法靠近。 \\nw[name]","这片大陆上有四个洞窟。然而那些地方都成了魔物的巢穴，人类无法靠近。\\nw[name]"]);
_tr_m.push(["どうじゃ、ためになっただろう！ \\nw[name]","Useful information, right? \\nw[name]","어때, 유익한 정보였지? \\nw[name]","怎麼樣！有幫上妳的忙吧！ \\nw[name]","怎么样！是不是帮上你的忙了！\\nw[name]"]);
_tr_m.push(["「探し物があったらとりあえず洞窟が怪しい。」 これはおじさんの故郷に古くから伝わる伝承のひとつだよ。 \\nw[name]","\"Search the caves and ye shall find what you seek.\" That saying has been passed down through the generations. \\nw[name]","「찾는 물건이 있다면 일단 동굴부터 의심해라.」 이건 이 아저씨의 고향에서 옛날부터 전해 내려오는 전설 중 하나란다. \\nw[name]","「如果要找東西，總之就先懷疑在洞窟裡就對了。」這是叔叔我故鄉自古以來傳承的話語之一。 \\nw[name]","「如果要找东西，总之先怀疑在洞窟里就对了。」这是叔叔我的故乡自古以来传承的话语之一。\\nw[name]"]);
_tr_m.push(["お姉ちゃん、おかえりなさい！ 疲れてるようだね、ちょっと休んでいきなよ！ \\nw[name]","Welcome home! Why don't you rest up? \\nw[name]","언니, 어서 와! 피곤해 보이네. 좀 쉬었다 가! \\nw[name]","姐姐，妳回來啦！妳看起來很累呢。先稍微休息一下吧！ \\nw[name]","姐姐，你回来啦！你看起来很累呢。先稍微休息一下吧！\\nw[name]"]);
_tr_m.push(["ぽん美に\\i[1009]ツヨスギテ＝クサハエルを見せますか？\\! ","Show the \\i[1009]LOL Sword to Ponmi?\\! ","폰미에게 \\i[1009]너무강해서 현웃터지는 소드를 보여 주시겠습니까?\\! ","要把\\i[1009]暴強‧草叢生劍拿給器美看嗎？\\! ","要把\\i[1009]无敌‧草丛生剑拿给器美看吗？\\! "]);
_tr_m.push([">はい",">Yes",">예","＞要","＞要"]);
_tr_m.push(["エンディングを選択してください。\\! ","Choose an ending.\\! ","엔딩을 선택하세요.\\! ","請選擇結局。\\! ","请选择结局。\\! "]);
_tr_m.push([">\\i[90]伝説のトロフィーEND",">\\i[90]Legendary Trophy Ending",">\\i[90]전설의 트로피 END","＞\\i[90]傳說的獎杯結局","＞\\i[90]傳說的獎杯結局"]);
_tr_m.push([">\\i[90]商人王のトロフィーEND",">\\i[90]Tycoon Trophy Ending",">\\i[90]상인왕의 트로피 END","＞\\i[90]商人王的獎杯結局","＞\\i[90]商人王的獎杯結局"]);
_tr_m.push([">\\i[90]勇者のトロフィーEND",">\\i[90]Hero Trophy Ending",">\\i[90]용사의 트로피 END","＞\\i[90]勇者的獎杯結局","＞\\i[90]勇者的獎杯結局"]);
_tr_m.push([">\\i[90]感謝のトロフィーEND",">\\i[90]Gratitude Trophy Ending",">\\i[90]감사의 트로피 END","＞\\i[90]感謝的獎杯結局","＞\\i[90]感謝的獎杯結局"]);
_tr_m.push([">いいえ",">No",">아니요","＞不要","＞不要"]);
_tr_m.push(["お姉ちゃん、ごくろうさま！ 何か手伝おっか？\\!  \\nw[name]","It's tough out there! Need help with anything?\\!  \\nw[name]","언니, 고생이 많지? 내가 좀 도와줄까?\\!  \\nw[name]","姐姐，辛苦妳了！有什麼要我幫忙的嗎？\\!  \\nw[name]","姐姐，辛苦你了！有什么要我帮忙的吗？\\! \\nw[name]"]);
_tr_m.push([">手伝ってもらう",">Ask for Help",">도움을 받는다","＞請妹妹幫忙","＞请妹妹帮忙"]);
_tr_m.push(["でも、もうお姉ちゃんに手伝ってあげられることも 見つからないなあ・・・。 \\nw[name]","Actually, I don't think there's anything I can do for you anymore. \\nw[name]","근데 내가 언니를 도와줄 수 있는 일을 못 찾겠네・・・ \\nw[name]","不過，我找不到什麼可以幫姐姐做的事了…… \\nw[name]","不过，我找不到什么可以帮姐姐做的事了……\\nw[name]"]);
_tr_m.push(["そうだ！ \\nw[name]","I know! \\nw[name]","아, 그래! \\nw[name]","對了！ \\nw[name]","对了！\\nw[name]"]);
_tr_m.push(["今後は特別に、\\i[141]アルマの石を売ってあげるよ！ きれいだから集めてたんだー。 \\nw[name]","How about I sell you \\i[141]Arma Stones from now on? I collect them because they're so pretty. \\nw[name]","앞으로는 특별히 \\i[141]알마의 돌을 팔아 줄게! 예뻐서 모아 뒀거든~ \\nw[name]","從現在開始，我就特別把\\i[141]魂石賣給妳吧！因為魂石很漂亮，我收集了不少。 \\nw[name]","从现在开始，我就把\\i[141]魂石卖给你吧！因为魂石很漂亮，我收集了不少。\\nw[name]"]);
_tr_m.push(["ただし、ちょっと値は張るけどね。 \\nw[name]","They don't come cheap though! \\nw[name]","다만 가격은 만만치 않을 거야. \\nw[name]","不過，價格會有點貴喔。 \\nw[name]","不过，价格会有点贵喔。\\nw[name]"]);
_tr_m.push(["というわけで・・・。 \\nw[name]","Let's get down to business. \\nw[name]","그래서 말인데・・・ \\nw[name]","所以…… \\nw[name]","所以……\\nw[name]"]);
_tr_m.push(["\\i[141]お姉ちゃん。アルマの石はいかが？ （所持金：\\v[3]\\g　\\i[141]所持数 ✕ \\v[4]）\\!  \\nw[name]","Hey, Sis. Want some \\i[141]Arma Stones?<br> (Money: \\v[3]\\g, \\i[141] Owned: \\v[4])\\!  \\nw[name]","언니, \\i[141]알마의 돌을 사 갈래? <br>(소지금: \\v[3]\\g　\\i[141]소지 수 ✕ \\v[4])\\!  \\nw[name]","姐姐，要買\\i[141]魂石嗎？ <br>（持有金錢：\\v[3]\\g \\i[141]持有數 ✕ \\v[4]）\\!  \\nw[name]","姐姐，要买\\i[141]魂石吗？<br>（持有金钱：\\v[3]\\g\\i[141]持有数 ✕ \\v[4]）\\! \\nw[name]"]);
_tr_m.push([">",">",">",">",">"]);
_tr_m.push(["まいどあり！ （\\i[141]所持数 ✕ \\v[4]） \\nw[ぽん美]","Always a pleasure! (\\i[141] Owned: \\v[4]) \\nw[Ponmi]","고마워! (\\i[141]소지 수 ✕ \\v[4]) \\nw[폰미]","謝謝惠顧！（\\i[141]持有數 ✕ \\v[4]） \\nw[器美]","谢谢惠顾！（\\i[141]持有数 ✕ \\v[4]）\\nw[器美]"]);
_tr_m.push(["何を手伝おっか？ （所持金：\\v[3]\\g）\\!  \\nw[ぽん美]","Need help with anything? (Money: \\v[3]\\g)\\!  \\nw[Ponmi]","뭘 도와줄까? (소지금: \\v[3]\\g)\\!  \\nw[폰미]","需要我幫忙嗎？（持有金錢：\\v[3]\\g）\\!  \\nw[器美]","需要我帮忙吗？（持有金钱：\\v[3]\\g）\\! \\nw[器美]"]);
_tr_m.push([">",">",">",">",">"]);
_tr_m.push(["やっぱりやめるんだね。 \\nw[ぽん美]","All finished, huh? \\nw[Ponmi]","그만두는구나. \\nw[폰미]","果然不需要啊。 \\nw[器美]","果然不需要啊。\\nw[器美]"]);
_tr_m.push(["\\v[3] Lv.\\v[4] だね。 お代は \\v[5]\\g だよ。\\!  \\nw[ぽん美]","\\v[3] Lv. \\v[4], eh? It'll cost you \\v[5]\\g.\\!  \\nw[Ponmi]","\\v[3] Lv.\\v[4]. 그럼 \\v[5]\\g야.\\!  \\nw[폰미]","\\v[3] Lv.\\v[4] 是吧。費用是 \\v[5]\\g。\\!  \\nw[器美]","\\v[3]等级\\v[4]是吧。费用是\\v[5]\\g。\\! \\nw[器美]"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞好"]);
_tr_m.push([">いいえ",">No",">아니요","＞先等等","＞先等等"]);
_tr_m.push(["まいどあり！ \\nw[ぽん美]","Always a pleasure! \\nw[Ponmi]","고마워! \\nw[폰미]","謝謝惠顧！ \\nw[器美]","谢谢惠顾！\\nw[器美]"]);
_tr_m.push(["\\i[261]薬草を買いたい","Buy \\i[261]herbs","\\i[261]약초를 사고 싶다","想要購買\\i[261]藥草","想要购买\\i[261]药草"]);
_tr_m.push(["\\i[261]薬草だね。いくつほしい？ （所持金：\\v[3]\\g　\\i[261]所持数：\\v[4]/ \\v[5]）\\!  \\nw[name]","You want some \\i[261]herbs, eh? How many?<br>(Money: \\v[3]\\g, \\i[261] Owned: \\v[4]/\\v[5])\\!  \\nw[name]","\\i[261]약초 말이구나. 몇 개 필요해?<br>(소지금: \\v[3]\\g　\\i[261]소지 수: \\v[4]/ \\v[5])\\!  \\nw[name]","\\i[261]藥草是吧。想要買幾個？<br>（持有金錢：\\v[3]\\g \\i[261]持有數：\\v[4]／　\\v[5]）\\! 　\\nw[name]","\\i[261]药草是吧。想要买几个？<br>（持有金钱：\\v[3]\\g\\i[261]持有数：\\v[4]／\\v[5]）\\! \\nw[name]"]);
_tr_m.push([">",">",">",">",">"]);
_tr_m.push(["もっと\\i[261]薬草ほしい？ （所持金：\\v[3]\\g　\\i[261]所持数：\\v[4]/ \\v[5]）\\!  \\nw[name]","You want some more \\i[261]herbs? (Money: \\v[3]\\g, \\i[261] Owned: \\v[4]/\\v[5])\\!  \\nw[name]","\\i[261]약초 더 필요해? (소지금: \\v[3]\\g　\\i[261]소지 수: \\v[4]/ \\v[5])\\!  \\nw[name]","想要更多\\i[261]藥草是嗎？（持有金錢：\\v[3]\\g \\i[261]持有數：\\v[4]／　\\v[5]）\\! 　\\nw[name]","想要更多\\i[261]药草是吗？（持有金钱：\\v[3]\\g\\i[261]持有数：\\v[4]／\\v[5]）\\! 　\\nw[name]"]);
_tr_m.push([">",">",">",">",">"]);
_tr_m.push(["まいどあり！ （\\i[261]所持数：\\v[4]/\\v[5]） \\nw[ぽん美]","Always a pleasure! (\\i[261] Owned: \\v[4]/\\v[5]) \\nw[Ponmi]","고마워! (\\i[261]소지 수: \\v[4]/\\v[5]) \\nw[폰미]","謝謝惠顧！（\\i[261]持有數：\\v[4]/\\v[5]） \\nw[器美]","谢谢惠顾！（\\i[261]持有数：\\v[4]/\\v[5]）\\nw[器美]"]);
_tr_m.push([">説明を聞く",">Explanation",">설명 듣기","＞聆聽說明","＞聆听说明"]);
_tr_m.push([">いいえ",">No",">아니요","＞否","＞否"]);
_tr_m.push(["\\i[1238]空飛ぶほうきが立てかけてある！","The \\i[1238]Flying Broom is leaning against the wall!","\\i[1238]하늘을 나는 빗자루가 세워져 있다!","\\i[1238]飛天掃帚靠在牆上！","\\i[1238]飞天扫帚靠在墙上！"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。","\\n[1] looked in the pot.","\\n[1]는 항아리 안을 들여다보았다.","\\n[1]探頭往壺裡看去。","\\n[1]探头往壶里看去。"]);
_tr_m.push(["なんとツボの中には\\i[141]アルマの石が入っていた！ \\i[141]アルマの石を手に入れた！","What's this? There's an \\i[141]Arma Stone inside! You got an \\i[141]Arma Stone!","항아리 안에는 \\i[141]알마의 돌이 들어 있었다! \\i[141]알마의 돌을 획득했다!","壺裡竟然有\\i[141]魂石！獲得了 \\i[141]魂石！","壶里竟然有\\i[141]魂石！获得了\\i[141]魂石！"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the pot. Unfortunately, it's empty.","\\n[1]는 항아리 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……","\\n[1]探头往壶里看去。但壶里什么都没有……"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the pot. Unfortunately, it's empty.","\\n[1]는 항아리 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the pot. Unfortunately, it's empty.","\\n[1]는 항아리 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the pot. Unfortunately, it's empty.","\\n[1]는 항아리 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","寶箱裡似乎空無一物。"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the pot. Unfortunately, it's empty.","\\n[1]는 항아리 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。","\\n[1] looked in the pot.","\\n[1]는 항아리 안을 들여다보았다.","\\n[1]探頭往壺裡看去。","\\n[1]探頭往壺裡看去。"]);
_tr_m.push(["なんとツボの中には\\i[261]薬草が入っていた！ \\i[261]薬草を手に入れた！","What's this? There's an \\i[261]herb inside! You got an \\i[261]herb!","항아리 안에는 \\i[261]약초가 들어 있었다! \\i[261]약초를 획득했다!","壺裡竟然有\\i[261]藥草！獲得了 \\i[261]藥草！","壺裡竟然有\\i[261]藥草！獲得了 \\i[261]藥草！"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the pot. Unfortunately, it's empty.","\\n[1]는 항아리 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……"]);
_tr_m.push(["\\n[1]はタルの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the barrel. Unfortunately, it's empty.","\\n[1]는 통 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……"]);
_tr_m.push(["\\n[1]はタルの中を覗いてみた。","\\n[1] looked in the barrel.","\\n[1]는 통 안을 들여다보았다.","\\n[1]探頭往桶子裡看去。","\\n[1]探頭往桶子裡看去。"]);
_tr_m.push(["なんとタルの中には 110\\g が入っていた！ 110\\g を手に入れた！","What's this? There's 110\\g! You got 110\\g!","통 안에는 무려 110\\g가 들어 있었다! 110\\g를 획득했다!","桶子裡竟然有110\\g！獲得了110\\g！","桶子裡竟然有110\\g！獲得了110\\g！"]);
_tr_m.push(["\\n[1]はタルの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the barrel. Unfortunately, it's empty.","\\n[1]는 통 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……"]);
_tr_m.push(["\\n[1]はタルの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the barrel. Unfortunately, it's empty.","\\n[1]는 통 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……"]);
_tr_m.push(["\\n[1]はタルの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the barrel. Unfortunately, it's empty.","\\n[1]는 통 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……"]);
_tr_m.push(["\\n[1]はタルの中を覗いてみた。","\\n[1] looked in the barrel.","\\n[1]는 통 안을 들여다보았다.","\\n[1]探頭往桶子裡看去。","\\n[1]探頭往桶子裡看去。"]);
_tr_m.push(["なんとタルの中には\\i[141]アルマの石が入っていた！ \\i[141]アルマの石を手に入れた！","What's this? There's an \\i[141]Arma Stone inside! You got an \\i[141]Arma Stone!","통 안에는 \\i[141]알마의 돌이 들어 있었다! \\i[141]알마의 돌을 획득했다!","桶子裡竟然有\\i[141]魂石！獲得了 \\i[141]魂石！","桶子裡竟然有\\i[141]魂石！獲得了 \\i[141]魂石！"]);
_tr_m.push(["\\n[1]はタルの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the barrel. But it's empty.","\\n[1]는 통 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……","\\n[1]探頭往桶子裡看去。但桶子裡什麼都沒有……"]);
_tr_m.push(["ンゴロンゴロ。  \\nw[name]","Meow! \\nw[name]","가르르릉.  \\nw[name]","恩戈羅恩戈羅。 \\nw[name]","恩戈罗恩戈罗。\\nw[name]"]);
_tr_m.push(["宅配便が届いている！ \\n[1]は箱を開けてみた！","You received a delivery! \\n[1] opened the package!","택배가 도착했다! \\n[1]는 상자를 열어 보았다!","宅配送到了！ \\n[1]打開了箱子！","快递到了！\\n[1]打开了箱子！"]);
_tr_m.push(["「ヘイワ食堂　２４時間営業！」","\"Peace Pub Now Open 24/7!\"","「평화 식당 24시간 영업!」","「和平餐廳 24小時營業！」","「和平餐厅24小时营业！」"]);
_tr_m.push(["「\\v[20]」の売上成績の本がある。","It's a ledger of the sales made by \\v[20].","「\\v[20]」의 매출 실적이 적힌 책이 있다.","這本書記載著「\\v[20]」的銷售成績。","这本书记载着「\\v[20]」的销售成绩。"]);
_tr_m.push(["\\v[20]は、これまでに武器を \\v[3]個、 合計で \\v[4]\\g 売り上げたようだ。","It says that \\v[20] has sold \\v[3] weapons for \\v[4]\\g.","\\v[20](은)는 지금까지 무기 \\v[3]개를 판매해서 총 \\v[4]\\g의 매출을 기록했다.","\\v[20]至今為止賣出了 \\v[3]個武器，總營業額是 \\v[4]\\g。","\\v[20]至今为止卖出了\\v[3]个武器，总营业额是\\v[4]\\g。"]);
_tr_m.push(["目指せ、合計売上 1000000\\g！！","Go for 1,000,000\\g in total sales!","목표는 총매출 1000000\\g!!","目標是總營業額 1000000\\g！！","总营业额目标是1000000\\g！！"]);
_tr_m.push(["1000000\\g突破おめでとう！ 限界を目指そう！！","You reached 1,000,000\\g in sales! Keep going! The sky's the limit!","1000000\\g 돌파를 축하합니다! 한계에 도전해 보세요!","恭喜總營業額突破1000000\\g！以突破極限為目標吧！！","恭喜总营业额突破1000000\\g！以突破极限为目标吧！！"]);
_tr_m.push(["「\\v[20]」\\! ","「\\v[20]」\\! ","「\\v[20]」\\! ","「\\v[20]」\\! ","「\\v[20]」\\! "]);
_tr_m.push([">店名を変える",">Rename Shop",">이름 바꾸기","＞更改店名","＞更改店名"]);
_tr_m.push(["\\n[1]の武器屋の名前は 今日から「\\v[20]」になった！","\\n[1] renamed the weapons shop to \"\\v[20]\"!","\\n[1]의 무기점 이름은 오늘부터 「\\v[20]」(이)다!","\\n[1]的武器店名字，從今天開始就變成「\\v[20]」了！","\\n[1]的武器店，从今天开始就叫「\\v[20]」了！"]);
_tr_m.push([">いいえ",">No",">아니요","＞不改","＞不改"]);
_tr_m.push(["・・・。 \\nw[？？？？？]","... \\nw[?????]","・・・. \\nw[?????]","…… \\nw[？？？？？]","……\\nw[？？？？？]"]);
_tr_m.push(["「\\v[20]」。 ここが最近口コミで話題の武器屋かあ・・・。 \\nw[？？？？？]","\"\\v[20]\"... So this is the weapons shop everyone's talking about? \\nw[?????]","「\\v[20]」. 여기가 요새 소문이 자자한 무기점인가・・・ \\nw[?????]","「\\v[20]」。這就是最近蔚為話題的武器店嗎…… \\nw[？？？？？]","「\\v[20]」。这就是最近广为谈论的武器店吗……\\nw[？？？？？]"]);
_tr_m.push(["ん？どうしたのキミ？ そろそろお家に帰りなさい。 \\nw[？？？？？]","What're you doing here, little girl? Run along home now. \\nw[?????]","응? 넌 여기서 뭐 해? 얼른 집에 가라. \\nw[?????]","嗯？妳怎麼了？時候不早了，快點回家吧。 \\nw[？？？？？]","嗯？你怎么了？时候不早了，快点回家吧。\\nw[？？？？？]"]);
_tr_m.push(["・・・え？ここの店主の\\n[1]？ へー、随分ちっちゃい店主なんだね。 \\nw[？？？？？]","...What? You own this place? At such a young age? \\nw[?????]","・・・뭐? 여기 주인인 \\n[1]라고? 오오~ 이렇게 쪼끄만 주인도 있구나. \\nw[?????]","……咦？妳是這裡的店長\\n[1]？哦─還真是一位嬌小的店長呢。 \\nw[？？？？？]","……咦？你是这里的店长\\n[1]？哦─还真是一位娇小的店长呢。 \\nw[？？？？？]"]);
_tr_m.push(["ボクかい？ふふ、何を隠そう・・・ \\nw[？？？？？]","Me? Heh, I have nothing to hide! \\nw[?????]","나 말이야? 후후, 그래・・・ \\nw[?????]","我嗎？呵呵，我就老實說吧…… \\nw[？？？？？]","我吗？呵呵，我就老实说吧……\\nw[？？？？？]"]);
_tr_m.push(["ボクはいずれ魔王を倒し英雄となる男！ 勇者シュヴァルディだ！！ \\nw[name]","I am the Valiant Hero Schivardi! The one destined to defeat the Demon Lord! \\nw[name]","나는 언젠가 마왕을 쓰러뜨리고 영웅이 될 남자! 용사 슈발디야!! \\nw[name]","我是將來會打倒魔王成為英雄的男人！勇者修瓦爾迪！！ \\nw[name]","我是将来会打倒魔王成为英雄的男人！勇者修瓦尔迪！！\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["とは言っても、正直魔王を倒すのってダルいんだよねー。 レベル上げにも飽きたし。 \\nw[name]","But to be honest, defeating the Demon Lord is such a chore. I'm tired of grinding for levels too. \\nw[name]","하지만 솔직히 마왕을 쓰러뜨리는 건 지루해~ 레벨 올리는 것도 지겹고. \\nw[name]","話雖如此，老實說我還蠻懶得去打倒魔王的。練等級也已經練到膩了。 \\nw[name]","话虽如此，老实说，打不打倒魔王对我来说都无所谓。练等级也已经练到腻了。\\nw[name]"]);
_tr_m.push(["しかも、最近魔王おとなしいじゃん？ 特に危害加えてこなければこのままでいいと思うんよね。 \\nw[name]","Besides, the Demon Lord isn't even causing that much trouble. If he doesn't hurt anyone, I don't see the harm in letting him be. \\nw[name]","게다가 요즘 마왕이 잠잠하잖아? 딱히 피해만 주지 않으면 이대로도 상관없지 않겠어? \\nw[name]","而且，最近魔王好像都很安分不是嗎？如果沒有特別產生什麼危害，我覺得放著不管也行吧。 \\nw[name]","而且，最近魔王好像都很安分不是吗？如果没有产生什么实质性的危害，我觉得放着不管也行吧。\\nw[name]"]);
_tr_m.push(["けど、みんな、オマエは勇者なんだから倒しに行けって。 ほんと嫌んなるよね。みんな本質が見えてないんだよ。 \\nw[name]","But everyone is telling me that it's my duty as a hero to go take him down. They just don't get me. \\nw[name]","근데 사람들이 나더러 용사니까 쓰러뜨리러 가라잖아. 아, 짜증 나. 왜 다들 본질을 못 보냐고. \\nw[name]","不過大家都對我說「因為你是勇者，所以快點去打倒魔王」。真的很討厭呢。大家都沒有看清楚事情的本質。 \\nw[name]","不过大家都对我说「因为你是勇者，所以快点去打倒魔王」。真是烦人。大家都没有看清楚事情的本质。\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["てことで、ボクとしてはなるべく楽に魔王を倒したいワケ。 \\nw[name]","Therefore, I want to take him down without having to work too hard. \\nw[name]","아무튼 그래서 내 입장에서는 되도록 편하게 마왕을 쓰러뜨리고 싶다는 거야. \\nw[name]","因此，我想要盡可能輕鬆地打倒魔王。 \\nw[name]","因此，我想要尽可能轻松地打倒魔王。\\nw[name]"]);
_tr_m.push(["で、最強の武器でもあればサクッと倒せるじゃん！と思って、 ここに来てみたワケよ。 \\nw[name]","That brings me to why I'm here. With an ultimate weapon, he'd go down in a heartbeat! \\nw[name]","최강의 무기라도 있으면 쉽게 쓰러뜨릴 수 있잖아? 그래서 여기에 와 봤어. \\nw[name]","然後，我就想說，如果有最強武器的話，就能輕鬆打倒魔王了！所以我才會來這裡。 \\nw[name]","然后，我就想说，如果有最强武器的话，就能轻松打倒魔王了！所以我才会来这里。\\nw[name]"]);
_tr_m.push(["でもあれだな。キミの武器屋・・・。 勢いはすごいけど、まだまだ発展途上って感じだね。 \\nw[name]","This shop is quite bustling, but by the looks of things, you girls are still growing your business. \\nw[name]","근데 너희 무기점・・・ 기세는 엄청난데 아직 갈 길이 먼 느낌이네. \\nw[name]","不過該怎麼說呢。妳的武器店……雖然氣勢很驚人，但感覺還在發展途中。 \\nw[name]","不过该怎么说呢。你的武器店……虽然气势很惊人，但感觉还在发展中。\\nw[name]"]);
_tr_m.push(["ほら。どうせ最強の武器を買うなら、それなりに すごい武器屋で買いたいみたいな所あるじゃん。 \\nw[name]","I mean, if I want an ultimate weapon, I'd want to get it from a big weapons shop. \\nw[name]","기왕 무기를 살 거면 그래도 근사한 무기점에서 사고 싶기 마련이잖아. \\nw[name]","妳想想，反正都要買最強武器，應該會想要到夠厲害的武器店去買吧？ \\nw[name]","你想想，反正都要买最强武器，应该会想要到够厉害的武器店去买吧？\\nw[name]"]);
_tr_m.push(["例えば・・・ 「今までの総売上１００万\\g！！」みたいな武器屋。 \\nw[name]","You know, maybe one with \"1 million \\g in sales\" or something like that. \\nw[name]","예를 들면・・・ 「현재까지 총매출 100만 \\g!!」 이런 무기점 말이야. \\nw[name]","比如說……「至今為止的總營業額到達100萬\\g！！」這樣的武器店。 \\nw[name]","比如说……「至今为止的总营业额到达100万\\g！！」这样的武器店。\\nw[name]"]);
_tr_m.push(["てことで。この店を、ボクに認められるような 武器屋にしてみてよ、\\n[1]ちゃん！！ \\nw[name]","There you have it. That's what it'll take to impress me, \\n[1]! \\nw[name]","그러니까 이곳을 내가 인정할 만한 무기점으로 만들어 봐, \\n[1]!! \\nw[name]","所以呢，妳就想辦法把這間店經營成能讓我認同的武器店吧，\\n[1]！！ \\nw[name]","所以呢，你就想办法把这间店经营成能让我认同的武器店吧，\\n[1]！！\\nw[name]"]);
_tr_m.push(["そしたら「魔王を倒した武器をここで買いました」って、 キミの武器屋を広めてあげるからさ。 \\nw[name]","And I'll be sure to tell people where I got the weapon that took down the Demon Lord! \\nw[name]","그럼 「마왕을 쓰러뜨린 무기를 여기서 샀습니다!」 하고 너희 무기점을 소문내 줄게. \\nw[name]","這麼一來，我就可以說「我打倒魔王的武器就是在這裡買的」，幫妳宣傳這間武器店。 \\nw[name]","这么一来，我就可以说「我打倒魔王的武器就是在这里买的」，帮你宣传这家武器店。\\nw[name]"]);
_tr_m.push(["よろしく頼んだよ、\\n[1]ちゃん！ 世界の平和のために！ \\nw[name]","I look forward to that day, \\n[1]! It'll be a great moment for world peace! \\nw[name]","잘 부탁해, \\n[1]! 세계 평화를 위해서! \\nw[name]","那就拜託妳啦，\\n[1]！這也是為了世界和平！ \\nw[name]","那就拜托你啦，\\n[1]！这也是为了世界和平！\\nw[name]"]);
_tr_m.push(["さ、帰ってゲームしよ。 \\nw[name]","Well, I'm going home to get my game on. \\nw[name]","자, 난 가서 게임이나 해야겠다. \\nw[name]","好，回去打電動吧。 \\nw[name]","好了，回去打游戏吧。\\nw[name]"]);
_tr_m.push(["勇者シュヴァルディに認められるため、 「総売上１００万\\g」を目指そう！","Aim for \"1 million \\g in sales\" to impress the Valiant Hero Schivardi!","용사 슈발디에게 인정받을 수 있도록 「총매출 100만 \\g」를 노려 보자!","為了獲得勇者修瓦爾迪的認同，請以「總營業額100萬\\g」為目標努力吧！","为了获得勇者修瓦尔迪的认同，请以「总营业额100万\\g」为目标努力吧！"]);
_tr_m.push(["やあ！また会ったね、\\n[1]ちゃん。 ゲームにも飽きたし、様子を見に来たよ。 \\nw[name]","Greetings! We meet again, \\n[1]. I got bored, so I came to check in on things. \\nw[name]","안녕? 또 만났네, \\n[1]? 게임도 지겨워져서 어떻게 지내나 보러 왔어. \\nw[name]","嗨！又見面了呢，\\n[1]。我打電動也打膩了，就來這裡看看狀況。 \\nw[name]","嗨！又见面了呢，\\n[1]。我游戏也玩腻了，就来这里看看状况。\\nw[name]"]);
_tr_m.push(["しかしすごいじゃん。ちょっと前に来た時とはなんつーか。 オーラが格段に違うよね。 \\nw[name]","But boy, how things have changed. Look at this place! It's like a completely different store! \\nw[name]","근데 대단하다. 얼마 전에 왔을 때랑 뭐랄까, 아우라가 차원이 달라. \\nw[name]","不過妳還真是厲害。跟我前一陣子來這裡時相比，該怎麼說呢……店裡的氣氛完全不同了呢。 \\nw[name]","不过你还真是厉害。跟我前一阵子来这里时相比，该怎么说呢……店里的气氛完全不同了呢。\\nw[name]"]);
_tr_m.push(["やっぱり魔王を倒す武器を買う武器屋はここしかないな！ \\nw[name]","This will definitely be the shop with the weapon that'll defeat the Demon Lord! \\nw[name]","역시 마왕을 쓰러뜨릴 무기를 살 만한 무기점은 여기뿐이야! \\nw[name]","要買用來打倒魔王的武器，果然就是要來這間武器店！ \\nw[name]","要买用来打倒魔王的武器，果然就是要来这间武器店！\\nw[name]"]);
_tr_m.push(["てことで。もっともっと武器を売ってさ、 ボクにふさわしい武器屋になってくれよ！ \\nw[name]","So keep selling even more weapons and become a shop worthy of selling me my weapon! \\nw[name]","그러니까 더 무기를 많이 팔아서 나한테 어울리는 무기점으로 만들어 줘! \\nw[name]","所以呢，妳就盡量賣出更多武器，讓這裡成為一間跟我相配的武器店吧！ \\nw[name]","所以呢，你就尽量卖出更多武器，让这里成为一家跟我相配的武器店吧！\\nw[name]"]);
_tr_m.push(["そういえば、これ拾ったんだ。受け取ってよ！ \\nw[name]","One more thing! I found these somewhere. They're all yours! \\nw[name]","참, 이런 걸 주웠어. 네가 가져! \\nw[name]","對了，我撿到了這個。妳就收下吧！ \\nw[name]","对了，我捡到了这个。你就收下吧！\\nw[name]"]);
_tr_m.push(["\\i[141]アルマの石を 8個 手に入れた！","You got 8 \\i[141]Arma Stones!","\\i[141]알마의 돌 8개를 획득했다!","獲得8顆\\i[141]魂石！","获得8颗\\i[141]魂石！"]);
_tr_m.push(["てことで、今後も頑張ってね！ \\nw[name]","Keep working on growing your business! \\nw[name]","앞으로도 열심히 해! \\nw[name]","那麼，接下來也要繼續努力喔！ \\nw[name]","那么，接下来也要继续努力喔！\\nw[name]"]);
_tr_m.push(["さ、帰ってニコ生見よ。 \\nw[name]","Well, I'm going home to watch some niconico live. \\nw[name]","자, 집에 가서 먹방이나 봐야지. \\nw[name]","好，我要回去看NICO直播了。 \\nw[name]","好，我要回去看NICO直播了。\\nw[name]"]);
_tr_m.push(["おつかれ！\\n[1]ちゃん。 ニコ生も見尽くしたし、様子を見に来たよ。 \\nw[name]","Hey there! We meet again, \\n[1]. I finished watching niconico live, so I came to check in on things. \\nw[name]","안녕, \\n[1]? 먹방이란 먹방은 다 봐서, 어떻게 지내나 보러 왔어. \\nw[name]","辛苦了！\\n[1]。我把NICO直播全都看完了，就來這裡看看狀況。 \\nw[name]","辛苦了！\\n[1]。我把NICO直播全都看完了，就来这里看看状况。\\nw[name]"]);
_tr_m.push(["いやー。それにしても、 かなり武器屋としてたくましくなったね！ \\nw[name]","Wow! There's a part of me that can't believe how big this shop has become. \\nw[name]","와~ 근데 엄청 듬직한 무기점이 되었구나! \\nw[name]","哎呀─想不到這裡已經變成一間很雄偉的武器店了呢！ \\nw[name]","哎呀─想不到这里已经变成一家很壮观的武器店了呢！\\nw[name]"]);
_tr_m.push(["この調子なら本当に・・・。 まさかねー・・・。 \\nw[name]","The way things are going, you just might... \\nw[name]","이대로라면 정말로・・・ 설마・・・ \\nw[name]","照這樣下去，真的就能夠……實在想不到呢…… \\nw[name]","照这样下去，真的就能够……实在想不到呢……\\nw[name]"]);
_tr_m.push(["でも、\\n[1]ちゃんならあり得るかもね。 これからもバンバン武器を売ってくれよ！ \\nw[name]","Actually, I know you can do it, \\n[1]. Keep on selling those weapons! \\nw[name]","그래도 \\n[1]라면 가능할지도 몰라. 앞으로도 무기를 팍팍 팔아 줘! \\nw[name]","不過，如果是\\n[1]的話，或許真的有可能辦到。接下來妳也要盡量多賣一點武器喔！ \\nw[name]","不过，如果是\\n[1]的话，或许真的有可能办到。接下来你也要尽量多卖一点武器喔！\\nw[name]"]);
_tr_m.push(["そういえば、これ拾ったんだ。受け取ってよ！ \\nw[name]","One more thing! I found these somewhere. They're all yours! \\nw[name]","참, 이런 걸 주웠어. 네가 가져! \\nw[name]","對了，我撿到了這個。妳就收下吧！ \\nw[name]","对了，我捡到了这个。你就收下吧！\\nw[name]"]);
_tr_m.push(["\\i[141]アルマの石を 12個 手に入れた！","You got 12 \\i[141]Arma Stones!","\\i[141]알마의 돌 12개를 획득했다!","獲得12顆\\i[141]魂石！","获得12颗\\i[141]魂石！"]);
_tr_m.push(["てことで、まあ頑張ってよね！ \\nw[name]","Keep up the good work! \\nw[name]","뭐, 아무튼 앞으로도 열심히 해! \\nw[name]","那麼，妳也要繼續努力喔！ \\nw[name]","那么，你也要继续努力喔！\\nw[name]"]);
_tr_m.push(["さ、帰ってYou◯tube見よ。 \\nw[name]","Well, I'm going home to watch some more videos. \\nw[name]","자, 난 집에 가서 You◯tube나 봐야지. \\nw[name]","好，我要回去看You◯tube了。 \\nw[name]","好，我要回去看You◯tube了。\\nw[name]"]);
_tr_m.push(["おう！\\n[1]ちゃん！ 様子を見に来たよ。 \\nw[name]","Yo, \\n[1]! I came to check on your progress. \\nw[name]","안녕, \\n[1]? 어떻게 지내나 보러 왔어. \\nw[name]","哦！\\n[1]。我來這裡看看狀況了。 \\nw[name]","哦！\\n[1]。我来这里看看状况了。\\nw[name]"]);
_tr_m.push(["いやー。ほんと、すごい勢いだよね！ \\nw[name]","The shop's got some serious patronage! \\nw[name]","우와~ 기세가 어마어마하네! \\nw[name]","哎呀─妳真的經營得很棒呢！ \\nw[name]","哎呀─你真的经营得很棒呢！\\nw[name]"]);
_tr_m.push(["このままなら本当に・・・達成できちゃうかもね。 \\nw[name]","The way things are going, you might actually pull this off... \\nw[name]","이대로라면 정말・・・ 달성할 수도 있겠어. \\nw[name]","這樣下去的話……或許真的能達成目標。 \\nw[name]","这样下去的话……或许真的能达成目标。\\nw[name]"]);
_tr_m.push(["・・・でも、いいかい？ 無理は禁物だよ。 \\nw[name]","But, uh, don't push yourself TOO hard, you hear? \\nw[name]","・・・근데 알지? 무리하면 안 돼. \\nw[name]","……不過，妳要注意喔！不可以太勉強。 \\nw[name]","……不过，你要注意喔！不可以太勉强。\\nw[name]"]);
_tr_m.push(["何事も程々が一番。頑張りすぎて身体を壊したりなんかしたら 意味がないからね。急がば回れだよ。 \\nw[name]","Do everything in moderation. There's no point in killing yourself over anything. And haste makes waste. \\nw[name]","뭐든지 적당한 게 최고야. 너무 애쓰다가 몸이 상하면 의미가 없잖아. 급할수록 돌아가라는 말도 있고. \\nw[name]","無論做什麼事，適度才是最好的。萬一太過努力而弄壞身體，那就沒意義了。欲速則不達啊。 \\nw[name]","无论做什么事，适度才是最好的。万一太过努力而弄坏身体，那就没意义了。欲速则不达啊。\\nw[name]"]);
_tr_m.push(["それを肝に念じてね！ \\nw[name]","Just keep that in mind! \\nw[name]","내 말 명심해! \\nw[name]","妳要好好記住這一點喔！ \\nw[name]","你要好好记住这一点喔！\\nw[name]"]);
_tr_m.push(["そういえば、これ拾ったんだ。受け取ってよ！ \\nw[name]","One more thing! I found these somewhere. They're all yours! \\nw[name]","참, 이런 걸 주웠어. 네가 가져! \\nw[name]","對了，我撿到了這個。妳就收下吧！ \\nw[name]","对了，我捡到了这个。你就收下吧！\\nw[name]"]);
_tr_m.push(["\\i[141]アルマの石を 20個 手に入れた！","You got 20 \\i[141]Arma Stones!","\\i[141]알마의 돌 20개를 획득했다!","獲得20顆\\i[141]魂石！","获得20颗\\i[141]魂石！"]);
_tr_m.push(["てことで、無理しすぎない程度に頑張ってよね！ \\nw[name]","Keep it up, but don't overdo it either! \\nw[name]","그러니까 무리하지 않는 선에서 열심히 해! \\nw[name]","那麼，妳就以不要太勉強的程度繼續努力吧！ \\nw[name]","那么，你就以不要太勉强的程度继续努力吧！\\nw[name]"]);
_tr_m.push(["さ、帰ってＲＰＧツクールしよ。 \\nw[name]","Well, I'm going home to play RPG Maker. \\nw[name]","자, 난 가서 RPG 메이커나 해야겠다. \\nw[name]","好，回去玩RPG製作大師吧。 \\nw[name]","好，回去玩RPG制作大师吧。\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["ほ、本当に達成しちゃったの？ 売上げ、１００万\\g。 \\nw[name]","Y-You actually did it?! 1 million \\g in sales! \\nw[name]","지, 진짜로 달성한 거야? 총매출 100만 \\g? \\nw[name]","妳、妳真的達成了嗎？營業額100萬\\g。 \\nw[name]","你、你真的达成了吗？营业额100万\\g。\\nw[name]"]);
_tr_m.push(["そ、そっかー。ふーん。 \\nw[name]","Hmm... I see... \\nw[name]","그, 그렇구나. 흐음~ \\nw[name]","這、這樣啊。哦…… \\nw[name]","这、这样啊。哦……\\nw[name]"]);
_tr_m.push(["お、おめでとう！！ \\nw[name]","C-Congratulations! \\nw[name]","추, 축하해!! \\nw[name]","恭、恭喜妳！！ \\nw[name]","恭、恭喜你！！\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["やだやだ！ なんで達成すんのもうー！！ \\nw[name]","You really shouldn't have done that... \\nw[name]","아, 뭐야~! 왜 막 달성하고 그래!! \\nw[name]","討厭啦討厭啦！妳為什麼要達成嘛！真是的！！ \\nw[name]","讨厌啦讨厌啦！你为什么要达成嘛！真是的！！\\nw[name]"]);
_tr_m.push(["キミに無理難題を押し付けて キミのせいにしてサボろうと思ったのに！！！ \\nw[name]","The purpose of giving you an impossible task was to make you give up from the start! \\nw[name]","너한테 어려운 과제를 던져 주고, 그 동안에 난 땡땡이칠 생각이었는데!! \\nw[name]","我本來想丟一個不可能的難題給妳，然後用妳還沒達成當做藉口，繼續偷懶下去的說！！ \\nw[name]","我本来想丢一个不可能的难题给你，然后用你还没达成当做借口，继续偷懒下去的呢！！\\nw[name]"]);
_tr_m.push(["なんか嫌な予感はしたけど・・・ まさか本当に達成するとは思わなかったよー！！ \\nw[name]","I suppose it was always a possibility, but I didn't really expect you to reach the goal! \\nw[name]","불길한 예감이 들긴 했지만・・・ 설마 진짜로 달성할 줄은 몰랐단 말이야~!! \\nw[name]","雖然我之前就有不好的預感……但沒想到妳真的達成了！！ \\nw[name]","虽然我之前就有不好的预感……但没想到你真的达成了！！\\nw[name]"]);
_tr_m.push(["何やってんだよー！！ 何、偉業成し遂げちゃってんだよー！！！ \\nw[name]","Why?! Why would you ever want to accomplish such a ridiculous feat?! \\nw[name]","나한테 왜 이래~!! 왜 위업을 달성하고 난리냐고~!!! \\nw[name]","看妳做了什麼好事─！！為什麼要完成這種豐功偉業啊！！！ \\nw[name]","看你做了什么好事─！！为什么要完成这种丰功伟业啊！！！\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["・・・・・。 \\nw[name]","...... \\nw[name]","・・・・・. \\nw[name]","………… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["何だよ、その腐った底辺ゴミクズを見るような目は・・・。 \\nw[name]","Stop looking at me like that... \\nw[name]","뭐지, 그 썩어빠진 밑바닥 쓰레기를 보는 듯한 눈빛은・・・ \\nw[name]","什麼啊？妳那種像是在看腐爛垃圾廢物般的眼神是怎樣…… \\nw[name]","什么啊？你那种像是在看腐烂垃圾废物般的眼神是怎么回事……\\nw[name]"]);
_tr_m.push(["はいはい！行けばいいんでしょ、行けば！！！ \\nw[name]","Fine! I just have to get it over with, right? I'll go then! \\nw[name]","알았어, 알았어! 가면 되잖아, 가면!!! \\nw[name]","好啦好啦！我去就行了吧！要我去就去啊！！！ \\nw[name]","好啦好啦！我去就行了吧！要我去就去啊！！！\\nw[name]"]);
_tr_m.push(["くっそー、なんでボクがこんな目に・・・。 \\nw[name]","Damn! Why does this always have to happen to me... \\nw[name]","젠장, 왜 내가 이런 꼴을・・・ \\nw[name]","可惡！為什麼我要遇到這種事…… \\nw[name]","可恶！为什么我要遇到这种事……\\nw[name]"]);
_tr_m.push(["勇者シュヴァルディは不満そうに 魔王城へ向かっていった！","The Valiant Hero Schivardi reluctantly headed to the Demonic Castle!","용사 슈발디는 불만스러운 얼굴로 마왕성을 향해 출발했다!","勇者修瓦爾迪心不甘情不願地朝魔王城出發！","勇者修瓦尔迪心不甘情不愿地朝魔王城出发！"]);
_tr_m.push(["トロフィーが飾ってある！\\! ","You earned a trophy!\\! ","트로피가 놓여 있다!\\! ","這裡裝飾著獎杯！\\! ","这里摆放着奖杯！\\! "]);
_tr_m.push([">\\i[90]伝説のトロフィー",">\\i[90]Legendary Trophy",">\\i[90]전설의 트로피","＞\\i[90]傳說的獎杯","＞\\i[90]传说的奖杯"]);
_tr_m.push(["\\i[90]「伝説のトロフィー」 試練を乗り越え、伝説の武器を手にした者に送られるトロフィー。","\\i[90]Legendary Trophy: Celebrates the champion who overcame adversity to acquire the legendary weapon.","\\i[90]「전설의 트로피」 시련을 뛰어넘어 전설의 무기를 손에 넣은 자에게 수여되는 트로피.","\\i[90]「傳說的獎杯」：通過試煉獲得傳說武器之一，將會得到這個獎杯。","\\i[90]「传说的奖杯」：通过试炼获得传说武器之一，将会得到这个奖杯。"]);
_tr_m.push(["伝説の武器を手に入れよう！","Get the legendary weapon!","전설의 무기를 찾아내자!","想辦法拿到傳說武器吧！","想办法拿到传说武器吧！"]);
_tr_m.push([">\\i[90]商人王のトロフィー",">\\i[90]Tycoon Trophy",">\\i[90]상인왕의 트로피","＞\\i[90]商人王的獎杯","＞\\i[90]商人王的奖杯"]);
_tr_m.push(["\\i[90]「商人王のトロフィー」 総額１００万\\g売り上げた、武器商人の王に送られるトロフィー。","\\i[90]Tycoon Trophy: Celebrates the weapons merchant who has achieved 1 million \\g in sales.","\\i[90]「상인왕의 트로피」 100만 \\g의 총매출을 기록한 무기 상인의 왕에게 수여되는 트로피.","\\i[90]「商人王的獎杯」：武器店營業額到達100萬\\g的武器商人之王，將會得到這個獎杯。","\\i[90]「商人王的奖杯」：武器店营业额到达100万\\g的武器商人之王，将会得到这个奖杯。"]);
_tr_m.push(["武器を総額１００万\\g売り上げよう！","Sell 1 million \\g in weapons!","무기를 판매하여 100만 \\g의 총매출을 달성하자!","想辦法賣掉總額100萬\\g的武器吧！","想办法卖掉总价值100万\\g的武器吧！"]);
_tr_m.push([">\\i[90]勇者のトロフィー",">\\i[90]Hero Trophy",">\\i[90]용사의 트로피","＞\\i[90]勇者的獎杯","＞\\i[90]勇者的奖杯"]);
_tr_m.push(["\\i[90]「勇者のトロフィー」 魔王を討伐した、勇敢で屈強な者に送られるトロフィー。","\\i[90]Hero Trophy: Celebrates the brave and mighty warrior who defeated the Demon Lord.","\\i[90]「용사의 트로피」 마왕을 토벌한, 용감하고 굳센 자에게 수여되는 트로피.","\\i[90]「勇者的獎杯」：成功討伐魔王，勇敢又堅強之人，將會得到這個獎杯。","\\i[90]「勇者的奖杯」：成功讨伐魔王，勇敢又坚强之人，将会得到这个奖杯。"]);
_tr_m.push(["魔王城で魔王を倒そう！","Defeat the Demon Lord in his castle!","마왕성에서 마왕을 쓰러뜨리자!","到魔王城打倒魔王吧！","到魔王城打倒魔王吧！"]);
_tr_m.push([">\\i[90]感謝のトロフィー",">\\i[90]Gratitude Trophy",">\\i[90]감사의 트로피","＞\\i[90]感謝的獎杯","＞\\i[90]感谢的奖杯"]);
_tr_m.push(["\\i[90]「感謝のトロフィー」 武器図鑑をすべて埋めた、想像を超える者に送られるトロフィー。","\\i[90]Gratitude Trophy: Celebrates the amazing collector who completely filled in the Weapons Gallery.","\\i[90]「감사의 트로피」 무기 도감을 전부 완성한, 상상을 초월하는 자에게 수여되는 트로피.","\\i[90]「感謝的獎杯」：填滿武器圖鑑，超乎想像之人，將會得到這個獎杯。","\\i[90]「感谢的奖杯」：填满武器图鉴，超乎想像之人，将会得到这个奖杯。"]);
_tr_m.push(["このゲームを遊び尽くしてくれて、 本当にありがとう！（作者）","Thank you so much for playing my game! -Creator","이 게임을 열심히 플레이해 주셔서 정말 감사합니다! (제작자)","真的非常感謝您將這款遊戲玩得如此徹底！（From作者）","真的非常感谢您将这款游戏玩得如此彻底！（From作者）"]);
_tr_m.push(["武器図鑑を完成させよう！","Complete the Weapons Gallery!","무기 도감을 완성하자!","想辦法完成武器圖鑑吧！","想办法完成武器图鉴吧！"]);
_tr_m.push([">やめる",">Cancel",">그만둔다","＞算了","＞算了"]);
_tr_m.push(["宅配便が届いている！ \\n[1]は箱を開けてみた！","A delivery has been made! \\n[1] opened the package!","택배가 도착했다! \\n[1]는 상자를 열어 보았다!","宅配送到了！ \\n[1]打開了箱子！","快递到了！\\n[1]打开了箱子！"]);
_tr_m.push(["バウワウ！ \\nw[name]","Bow-wow! \\nw[name]","멍멍! \\nw[name]","汪汪！ \\nw[name]","汪汪！\\nw[name]"]);
_tr_m.push(["\\i[1363]ムシとりアミが立てかけてある！","An \\i[1363]Insect Net is leaning against the wall!","\\i[1363]곤충 채집망이 세워져 있다!","\\i[1363]捕蟲網靠在牆上！","\\i[1363]捕虫网靠在墙上！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["「ここはヘイワ村 　注：平和なので魔物は立入禁止！」","\"Welcome to Peaceville. Note: To maintain the peace, no monsters allowed!\"","「여기는 평화 마을 　주의: 평화로우므로 마물은 출입 금지!」","「這裡是和平村 註：因為這裡很和平，所以禁止魔物進入！」","「这里是和平村 注：因为这里很和平，所以禁止魔物进入！」"]);
_tr_m.push(["「↑森　↓荒れ地 　←海　→村」","\"↑Forest  ↓Wasteland  ←Ocean  →Village\"","「↑숲　↓황무지 　←바다　→마을」","「↑森林　↓荒地　←海　→村子」","「↑森林　↓荒地　←海　→村子」"]);
_tr_m.push(["「この先↑魔王城」","\"↑Demonic Castle ahead\"","「↑전방에 마왕성」","「前方↑魔王城」","「前方↑魔王城」"]);
_tr_m.push(["「バザー開催中！ 　掘り出し物があるかも！？」","\"Bazaar Sale! You might find a real treasure!\"","「시장이 열렸습니다! 진흙 속의 진주를 찾아 보세요!」","「跳蚤市場舉辦中！或許能挖到寶喔！？」","「跳蚤市场举办中！或许能挖到宝喔！？」"]);
_tr_m.push(["「～今日の豆知識～ 　枯れ木で殴っても人は死なない！」","\"Daily Trivia: Humans can't die from being hit with a Withered Tree!\"","「~오늘의 힌트~ 　메마른 나무로 때려도 사람은 죽지 않는다!」","「～今天的小知識～ 用枯木打人，人也不會死！」","「～今天的小知识～用枯木打人，人也不会死！」"]);
_tr_m.push(["返事がない。ただのかかしのようだ。 \\nw[かかし]","There's no response. It's just a scarecrow. \\nw[scarecrow]","대답이 없다. 그냥 허수아비인 모양이다. \\nw[허수아비]","沒有回應。看來像是一具普通的屍體。 \\nw[稻草人]","没有回应。看来像是一具普通的尸体。\\nw[稻草人]"]);
_tr_m.push(["強そうな戦士像が置いてある・・・。","It's a statue of a mighty warrior.","강해 보이는 전사의 동상이 있다・・・","放著一尊看起來很強的戰士雕像……","放着一尊看起来很强的战士雕像……"]);
_tr_m.push(["「弱者の洞窟」","\"Cave of the Weak\"","「약자의 동굴」","「弱者洞窟」","「弱者洞窟」"]);
_tr_m.push(["「コンビニエンスストア『魔王城前』 　２４時間営業！」","\"Convenience Store - Demonic Castle branch: We're open 24/7!\"","「편의점 『마왕성 앞』 　24시간 영업!」","「便利商店『魔王城前』 24小時營業！」","「便利商店『魔王城前』 24小时营业！」"]);
_tr_m.push(["「霊魂の洞窟」","\"Cave of Lost Souls\"","「영혼의 동굴」","「靈魂洞窟」","「灵魂洞窟」"]);
_tr_m.push(["「幻術の洞窟」","\"Cave of Illusion\"","「환술의 동굴」","「幻術洞窟」","「幻术洞窟」"]);
_tr_m.push(["「鋼鉄の洞窟」","\"Cave of Iron\"","「강철의 동굴」","「鋼鐵洞窟」","「钢铁洞窟」"]);
_tr_m.push(["うんちだ。","It's a pile of poop.","똥이다.","是大便。","是大便。"]);
_tr_m.push(["俺はジェームズ。 ・・・青いクマだ。 \\nw[name]","I'm James. I'm a blue bear. \\nw[name]","난 James. ・・・파란색 곰이지. \\nw[name]","我叫James……是一隻藍熊。 \\nw[name]","我叫James……是一只蓝熊。\\nw[name]"]);
_tr_m.push(["お前、武器商人だな。 俺の指定する武器を持ってくれば、褒美をやろう。 \\nw[name]","You're a weapons merchant, right? I'll give you a reward if you bring me the weapon I tell you to. \\nw[name]","넌 무기 상인이군. 내가 지정한 무기를 가져오면 상을 주지. \\nw[name]","妳是武器商人吧？如果妳把我指定的武器帶過來，我就會給妳獎勵。 \\nw[name]","你是武器商人吧？如果你把我指定的武器带过来，我就会给你奖励。\\nw[name]"]);
_tr_m.push(["よし、そうだな・・・「威力が 32 以上の武器」を持って来い。 \\nw[name]","Okay... Bring me a weapon with an \"ATK value of at least 32.\" \\nw[name]","어디 보자・・・ 「위력이 32 이상인 무기」를 가져와. \\nw[name]","好，我想想……把「威力 32 以上的武器」帶來吧。 \\nw[name]","好，我想想……把「威力32以上的武器」带来吧。\\nw[name]"]);
_tr_m.push(["俺は強い武器が大好きでな。 持ってきたら装備して話しかけろ。 \\nw[name]","I simply love powerful weapons. Equip one and come talk to me again! \\nw[name]","난 강한 무기가 좋거든. 가져오면 장비한 후에 나한테 말을 걸어. \\nw[name]","我很喜歡強大的武器。如果妳帶來了，就裝備在身上來跟我說話吧！ \\nw[name]","我很喜欢强大的武器。如果你带来了，就装备在身上来跟我说话吧！\\nw[name]"]);
_tr_m.push(["ふん・・・健闘を祈る。 \\nw[name]","I wish you the best of luck. \\nw[name]","흥・・・ 건투를 빈다. \\nw[name]","哼……祝妳順利。 \\nw[name]","哼……祝你顺利。\\nw[name]"]);
_tr_m.push(["「威力が 32 以上の武器」。持ってきたようだな。 \\nw[name]","Alright! You found a weapon with an \"ATK value of at least 32\"! \\nw[name]","「위력이 32 이상인 무기」를 가져왔나 보군. \\nw[name]","妳似乎把「威力 32 以上的武器」帶來了啊。 \\nw[name]","你似乎把「威力32以上的武器」带来了啊。\\nw[name]"]);
_tr_m.push(["なるほど・・・なかなかいい武器だ。 \\nw[name]","Oh my... What a magnificent weapon! \\nw[name]","그래・・・ 상당히 좋은 무기네. \\nw[name]","原來如此……真是一把不錯的武器呢。 \\nw[name]","原来如此……真是一把不错的武器呢。\\nw[name]"]);
_tr_m.push(["しかしこうも威力が高いと・・・ これで斬られたら即死ってことか？ \\nw[name]","With something so powerful...whatever it hits might die in a single stroke! \\nw[name]","그나저나 이렇게 위력이 높으면・・・ 이거에 베이면 즉사하는 건가? \\nw[name]","不過，威力這麼強大……被這把武器砍到的話，應該會立刻死亡吧？ \\nw[name]","不过，威力这么强大……被这把武器砍到的话，应该会立刻死亡吧？\\nw[name]"]);
_tr_m.push(["う、うわー・・・なんか怖くなってきた・・・。 ガクガクブルブル・・・。 \\nw[name]","Yikes! That's a scary thought... \\nw[name]","우, 우와・・・ 갑자기 무섭다・・・ 후들후들 덜덜덜・・・ \\nw[name]","嗚、嗚哇……我開始害怕了……顫抖顫抖顫抖…… \\nw[name]","呜、呜哇……我开始害怕了……颤抖颤抖颤抖……\\nw[name]"]);
_tr_m.push(["まあいい。合格だ。褒美をやろう。ガクブル・・・。 \\nw[name]","Anyhow, this is exactly what I asked for. Your reward, as promised. \\nw[name]","좋아, 합격이다. 상을 주마. 후들덜덜・・・ \\nw[name]","沒問題，妳合格了。我就給妳獎勵吧。顫抖…… \\nw[name]","没问题，你合格了。我就给你奖励吧。颤抖……\\nw[name]"]);
_tr_m.push(["この世界には俺を含めて４匹のクマがいる。 すべてのクマに認められてみよガクブル。 \\nw[name]","There are four of us bears in the world, including myself. Try to get their approval too! \\nw[name]","이 세계에는 나를 포함해서 4마리의 곰이 있다. 모든 곰에게 인정받아 봐. 후들덜덜. \\nw[name]","這個世界上包含我總共有四隻熊。妳就試著想辦法獲得每一隻熊的認同吧！顫抖…… \\nw[name]","包括我在内，这个世界上总共有四只熊。你就试着想办法获得每一只熊的认同吧！颤抖……\\nw[name]"]);
_tr_m.push(["「威力が 32 以上の武器」を持って来い。 \\nw[name]","Bring me a weapon with an \"ATK value of at least 32.\" \\nw[name]","「위력이 32 이상인 무기」를 가져와. \\nw[name]","把「威力 32 以上的武器」帶來吧。 \\nw[name]","把「威力32以上的武器」带来吧。\\nw[name]"]);
_tr_m.push(["俺は強い武器が大好きでな。 持ってきたら装備して話しかけろ。 \\nw[name]","I simply love powerful weapons. Equip one and come talk to me again! \\nw[name]","난 강한 무기가 좋거든. 가져오면 장비한 후에 나한테 말을 걸어. \\nw[name]","我很喜歡強大的武器。如果妳帶來了，就裝備在身上來跟我說話吧！ \\nw[name]","我很喜欢强大的武器。如果你带来了，就装备在身上来跟我说话吧！\\nw[name]"]);
_tr_m.push(["返事がない。ただのぬいぐるみのようだ。","There's no response. It's just a stuffed animal.","대답이 없다. 그냥 인형인 모양이다.","沒有回應。看來像是一具普通的布偶。","没有回应。看上去像是一具普通的布偶。"]);
_tr_m.push(["この世界には俺を含めて４匹のクマがいる。 すべてのクマに認められてみよガクブル。 \\nw[name]","There are four of us bears in the world, including myself. Try to get their approval too! \\nw[name]","이 세계에는 나를 포함해서 4마리의 곰이 있다. 모든 곰에게 인정받아 봐. 후들덜덜. \\nw[name]","這個世界上包含我總共有四隻熊。妳就試著想辦法獲得每一隻熊的認同吧！顫抖…… \\nw[name]","包括我在内，这个世界上总共有四只熊。你就试着想办法获得每一只熊的认同吧！颤抖……\\nw[name]"]);
_tr_m.push(["なんかよくわからないけど 魔導書っぽい本がたくさんある・・・！","For some reason or another, there are a lot of magic tomes!","잘 모르겠지만 마도서 같은 책이 잔뜩 있다・・・!","雖然不清楚是怎麼回事，但有好多看起來像是魔導書的書籍……！","虽然不清楚是怎么回事，但有好多看起来像是魔导书的书籍……！"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。","\\n[1] looked in the pot.","\\n[1]는 항아리 안을 들여다보았다.","\\n[1]探頭往壺裡看去。","\\n[1]探頭往壺裡看去。"]);
_tr_m.push(["なんとツボの中には\\i[261]薬草が入っていた！ \\i[261]薬草を手に入れた！","What's this? There's an \\i[261]herb inside! You got an \\i[261]herb!","항아리 안에는 \\i[261]약초가 들어 있었다! \\i[261]약초를 획득했다!","壺裡竟然有\\i[261]藥草！獲得了 \\i[261]藥草！","壺裡竟然有\\i[261]藥草！獲得了 \\i[261]藥草！"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the pot. Unfortunately, it's empty.","\\n[1]는 항아리 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……"]);
_tr_m.push(["魔王城に挑戦しますか？\\! ","Take on the Demonic Castle?\\! ","마왕성에 도전하시겠습니까?\\! ","要挑戰魔王城嗎？\\! ","要挑战魔王城吗？\\! "]);
_tr_m.push([">挑戦する",">Yes",">도전하기","＞挑戰","＞挑战"]);
_tr_m.push(["魔王城に挑戦するには 武器を８つ以上持って行かなくてはならない！","You must have at least eight weapons in your possession to take on the Demonic Castle.","마왕성에 도전하려면 무기를 8개 이상 가져가야 한다!","若要挑戰魔王城，必須攜帶八項以上的武器！","若要挑战魔王城，必须携带八种以上的武器！"]);
_tr_m.push(["もう最上階に魔王は出現しません。 よろしいですか？\\! ","The Demon Lord will not appear on the top floor. Proceed anyway?\\! ","이제 최상층에 마왕은 출현하지 않습니다. 그래도 도전하시겠습니까?\\! ","魔王只會出現在最上層。確定要挑戰嗎？\\! ","魔王只会出现在最上层。确定要挑战吗？\\! "]);
_tr_m.push([">はい",">Yes",">예","＞是","＞是"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push([">説明を聞く",">Explanation",">설명 듣기","＞聆聽說明","＞聆聽說明"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push(["なんと宝箱の中には \\i[141]アルマの石が 10個 入っていた！","There are 10 \\i[141]Arma Stones in the chest!","보물상자 안에는 무려 \\i[141]알마의 돌 10개가 들어 있었다!","寶箱裡竟然有10顆 \\i[141]魂石！","寶箱裡竟然有10顆 \\i[141]魂石！"]);
_tr_m.push(["\\i[141]アルマの石を 10個 手に入れた！","You got 10 \\i[141]Arma Stones!","\\i[141]알마의 돌 10개를 획득했다!","獲得10顆 \\i[141]魂石！","獲得10顆 \\i[141]魂石！"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","寶箱裡似乎空無一物。"]);
_tr_m.push(["\\n[1]は 「まずはスライムたちを倒すのが先やな」 と思った！","\\n[1] thought, \"I'd best defeat those slimes first!\"","\\n[1]는 「우선 슬라임들부터 해치워야 해」라고 생각했다!","\\n[1]心想：「先從史萊姆開始打倒吧」！","\\n[1]心想：「先从打倒史莱姆开始吧」！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["戦いを終えて疲れた\\n[1]は いったん自宅に帰ることにした。","Tired from the fight, \\n[1] headed home.","배틀을 마치고 지친 \\n[1]는 일단 집에 돌아가기로 했다.","\\n[1]在戰鬥結束後感到十分疲累，決定先回家一趟。","\\n[1]在战斗结束后感到十分疲累，决定先回家一趟。"]);
_tr_m.push(["こんなところにスコップが落ちている・・・。","There's a shovel lying on the ground.","이런 곳에 삽이 떨어져 있네・・・","有一把鐵鍬掉在這裡……","有一把铁锹掉在这里……"]);
_tr_m.push(["スコップが落ちている。","Someone left a shovel lying about.","삽이 떨어져 있다.","一把鐵鍬掉在這裡。","一把铁锹掉在这里。"]);
_tr_m.push(["\\n[1]はツボの中を覗いてみた。 しかし何もなかった・・・。","\\n[1] looked in the pot. Unfortunately, it's empty.","\\n[1]는 항아리 안을 들여다보았다. 그러나 아무것도 없었다・・・","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……","\\n[1]探頭往壺裡看去。但壺裡什麼都沒有……"]);
_tr_m.push(["オレはキカンジ！ 伝説の武器を求めて冒険してるんだ！ \\nw[name]","The name's Kikangji! I'm on an adventure to find the legendary weapon! \\nw[name]","내 이름은 Kikangji! 전설의 무기를 찾아 모험 중이야! \\nw[name]","我是Kikangji！為了尋找傳說武器，正在進行冒險之旅！ \\nw[name]","我是Kikangji！为了寻找传说武器，正在进行冒险之旅！\\nw[name]"]);
_tr_m.push(["しかし手がかりが全然得られないんだ。 オレ的には洞窟とかが怪しいと思うんだけどな。 \\nw[name]","Unfortunately, I have zero leads. The caves seem like a good place to start... \\nw[name]","그런데 도무지 단서를 찾을 수가 없어. 개인적으로는 동굴이 의심스럽지만 말이야. \\nw[name]","然而，我卻完全找不到線索。我個人覺得洞窟還滿可疑的。 \\nw[name]","然而，我却完全找不到线索。我个人觉得洞窟很可疑。\\nw[name]"]);
_tr_m.push(["だけど、怪しい所には入っちゃダメって 母ちゃんに教わったからな・・・。 \\nw[name]","But my mommy always told me to stay away from shady places like that. \\nw[name]","하지만 우리 엄마가 의심스러운 데는 가지 말라고 그랬거든・・・ \\nw[name]","不過，媽媽告訴過我不可以去可疑的地方…… \\nw[name]","不过，妈妈告诉过我不可以去可疑的地方……\\nw[name]"]);
_tr_m.push(["ワッハッハ！池の水は気持ちいいわい！ \\nw[name]","Wahaha! The water in this pond feels great! \\nw[name]","와하핫! 연못 물이 시원하군! \\nw[name]","哇哈哈！池子裡的水好舒服啊！ \\nw[name]","哇哈哈！池子里的水好舒服啊！\\nw[name]"]);
_tr_m.push(["お嬢ちゃんも浴びていかないかい？ 気持ちいいぞい！ウッヒョ！ \\nw[name]","What about you, young lady? Wanna jump in? It's so soothing! \\nw[name]","너도 들어와 봐! 기분이 얼마나 좋다고! 야호! \\nw[name]","小妹妹，妳要不要也來泡個水？很舒服喔！嗚呵！ \\nw[name]","小妹妹，你要不要也来泡下看看？很舒服喔！呜呵！\\nw[name]"]);
_tr_m.push(["ここから西は岩の転がる荒れ地だ。 変な洞窟しかないし、近寄らないほうがいいよ。 \\nw[name]","To the west is the rocky wasteland. There is a strange cave there, so keep your distance. \\nw[name]","여기서부터 서쪽은 바위가 굴러다니는 황무지야. 이상한 동굴밖에 없으니까 가까이 가지 마. \\nw[name]","這裡往西邊走，是一片岩石遍佈的荒地。那裡只有奇怪的洞窟，還是別靠近比較好。 \\nw[name]","这里往西边走，是一片岩石遍布的荒地。那里只有奇怪的洞窟，还是别靠近比较好。\\nw[name]"]);
_tr_m.push(["なんでも、そこに住む岩の魔物はめちゃくちゃ固いとか。 \\nw[name]","They say the stone monsters there are super tough, and you'll have a hard time fighting them. \\nw[name]","듣자 하니 거기에 사는 바위 마물은 엄청나게 단단하대. \\nw[name]","聽說住在那裡的岩石魔物十分堅硬呢。 \\nw[name]","听说住在那里的岩石魔物十分坚硬呢。\\nw[name]"]);
_tr_m.push(["ここから眺める海は綺麗ね。 ソロキャンプするにはぴったりの場所だわ。 \\nw[name]","The open sea is such a beautiful sight. Perfect for camping on your own. \\nw[name]","여기서는 아름다운 바다가 보여. 혼자 캠핑하기에는 딱 좋은 곳이지. \\nw[name]","從這裡看過去的海洋非常漂亮呢。是個很適合獨自來露營的地方。 \\nw[name]","从这里看过去的海洋非常漂亮呢。是个很适合独自来露营的地方。\\nw[name]"]);
_tr_m.push(["まあ・・・魔物はうじゃうじゃいるし 隣のはしゃいでる男はうるさいけど・・・。 \\nw[name]","Well, there are monsters roaming about and that noisy fella next door... \\nw[name]","뭐・・・ 마물은 우글거리지, 옆에 시끄러운 남자가 있기는 하지만・・・ \\nw[name]","不過……這裡常有魔物出沒，在旁邊吵鬧的男人也很煩…… \\nw[name]","不过……这里常有魔物出没，在旁边吵闹的男人也很烦……\\nw[name]"]);
_tr_m.push(["キャンプファイヤーが消えた！わーいわーい！ \\nw[name]","The campfire went out! Yaaay! \\nw[name]","캠프파이어가 꺼졌다! 만세~ 만세~! \\nw[name]","營火消失了！耶─！耶─！ \\nw[name]","营火消失了！耶─！耶─！\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["なんか悲しくなってきた・・・。 \\nw[name]","It feels a bit lonely... \\nw[name]","왠지 슬퍼졌어・・・ \\nw[name]","感覺有點開始難過了呢…… \\nw[name]","感觉开始有点难过了呢…… \\nw[name]"]);
_tr_m.push(["キャンプファイヤーだ！わーいわーい！ \\nw[name]","It's a campfire! Yaaay! \\nw[name]","캠프파이어다! 만세~ 만세~! \\nw[name]","是營火耶！耶─！耶─！ \\nw[name]","是营火耶！耶─！耶─！\\nw[name]"]);
_tr_m.push(["こんにちは、旅の方。 えっ、洞窟をお探しなのですか？ \\nw[name]","Hello, traveler. Are you looking for a cave? \\nw[name]","안녕하세요, 여행자님. 앗, 동굴을 찾으시나요? \\nw[name]","妳好啊，旅行者。咦？妳要到洞窟探險嗎？ \\nw[name]","你好啊，旅行者。咦？你要到洞窟探险吗？\\nw[name]"]);
_tr_m.push(["聞いた話ではこの道を東に辿った先に 洞窟があるとかないとか・・・。 \\nw[name]","I've been told if you head east, there's one at the end of this path. \\nw[name]","제가 듣기로는 이 길을 동쪽으로 따라가면 동굴이 있다던데・・・ \\nw[name]","我聽說沿著這條路往東邊走會抵達一個洞窟，但又好像不會…… \\nw[name]","我听说沿着这条路往东边走会抵达一个洞窟，但又好像不会……\\nw[name]"]);
_tr_m.push(["しかし、洞窟には魔物が住み着き危険と聞きます。 目的はわかりませんが、無理をならさぬよう。 \\nw[name]","I've also been told that the cave is infested with monsters. I don't know what you're after, but try not to bite off more than you can chew. \\nw[name]","그런데 동굴에는 마물이 살아서 위험하대요. 목적이 뭔지는 몰라도 무리하지 마시길. \\nw[name]","不過，我聽說洞窟裡住著危險的魔物。雖然我不知道妳要去做什麼，但還是別太勉強比較好。 \\nw[name]","不过，我听说洞窟里住着危险的魔物。虽然我不知道你要去做什么，但最好不要太勉强。\\nw[name]"]);
_tr_m.push(["オレは冒険者フルイレ。 伝説の武器を探して旅をしているんだ！ \\nw[name]","I'm Fureil the Adventurer. I'm on a quest to find the legendary weapon! \\nw[name]","나는 Fureil는 모험가야. 전설의 무기를 찾아 여행 중이지! \\nw[name]","我是冒險者Fureil。為了尋找傳說武器，正在進行冒險之旅！ \\nw[name]","我是冒险者Fureil。为了寻找传说武器，正在进行冒险之旅！\\nw[name]"]);
_tr_m.push(["この先の毒沼地帯に行くのか？ 気をつけたほうがいいぞ。 \\nw[name]","Are you heading to the Poisonous Swamp? You'd better be careful. \\nw[name]","이 앞의 독의 늪 지대에 가려고? 조심하는 게 좋을걸. \\nw[name]","妳要去前面的毒沼地帶嗎？還是小心一點比較好喔。 \\nw[name]","你要去前面的毒沼地带吗？还是小心一点比较好喔。\\nw[name]"]);
_tr_m.push(["そうだ、これ持っていけよ！ \\nw[name]","You know what? Here, take this! \\nw[name]","아, 이걸 가져가라! \\nw[name]","對了，妳帶著這個去吧！ \\nw[name]","对了，你带着这个去吧！\\nw[name]"]);
_tr_m.push(["薬草を手に入れた！","You got an herb!","약초를 획득했다!","獲得了藥草！","获得了药草！"]);
_tr_m.push(["この先は毒の魔物もいるからな。 薬草があれば、毒もきれいさっぱりだ。 \\nw[name]","There are poisonous monsters up ahead. You can use herbs to counteract the poison, though. \\nw[name]","이 앞에는 독성이 있는 마물도 있거든. 약초가 있으면 독 때문에 걱정할 필요 없어. \\nw[name]","畢竟前方會有用毒的魔物啊。但有了藥草，就能輕鬆解毒了！ \\nw[name]","毕竟前方会有用毒的魔物啊。但有了药草，就能轻松解毒了！\\nw[name]"]);
_tr_m.push(["なんだ。気が変わったのか？ 仕方ない・・・もう一度聞くぞ。 \\nw[name]","You're back. You change your mind or something? Well, I'll ask again... \\nw[name]","뭐야, 마음이 바뀌었어? 어쩔 수 없지・・・ 다시 물어볼게. \\nw[name]","怎麼，妳改變主意了嗎？真沒辦法……我再問妳一次吧。 \\nw[name]","怎么，你改变主意了吗？真没办法……我再问你一次吧。\\nw[name]"]);
_tr_m.push(["・・・こんな古ぼけた小屋に一体なんのようじゃ。\\!  \\nw[name]","What business do you have in my humble abode?\\!  \\nw[name]","・・・이런 낡아빠진 오두막에 무슨 볼일이냐.\\!  \\nw[name]","……妳來這棟老舊的小屋，到底想做什麼？\\!  \\nw[name]","……你来这栋老旧的小屋，到底想做什么？\\! \\nw[name]"]);
_tr_m.push([">特に用はない",">Nothing",">딱히 볼일은 없다","＞沒什麼特別的事","＞没什么特别的事"]);
_tr_m.push(["ふん。 こんなところにいないで、さっさと家に帰るがいい。 \\nw[name]","Hmph. In that case, stop loitering around here and go back home. \\nw[name]","흥, 이런 데서 어슬렁거리지 말고 빨리 집에 가라. \\nw[name]","哼。妳別待在這裡了，快點回家吧。 \\nw[name]","哼。你别待在这里了，快点回家吧。\\nw[name]"]);
_tr_m.push([">せっかくなので何かください",">I've come all this way, give me something!",">이것도 인연인데 뭔가 주세요","＞難得都來了，請給我一點東西吧","＞难得我来了，请给我一点东西吧"]);
_tr_m.push(["いや、何が「せっかく」じゃい！！ \\nw[name]","\"Come all this way,\" you say? \\nw[name]","「인연」 좋아하시네!! \\nw[name]","呃，什麼叫「難得」啊！！ \\nw[name]","呃，什么叫「难得」啊！！\\nw[name]"]);
_tr_m.push(["・・・まあ、いいだろう。 ただし、こちらの条件ものんでもらうぞ。 \\nw[name]","Okay, fine. But I've got conditions of my own. \\nw[name]","・・・뭐, 좋다. 단, 조건이 있어. \\nw[name]","……算了，要給妳東西也是可以。不過，妳得先達成我提出的條件才行。 \\nw[name]","……算了，要给你东西也可以。不过，你得先达成我提出的条件才行。\\nw[name]"]);
_tr_m.push(["ワシの名はウォンマー。木こりをやっておる。 \\nw[name]","I'm Wonmar the Woodcutter. \\nw[name]","내 이름은 Wonmar. 나무꾼이다. \\nw[name]","我叫Wonmar，是個樵夫。 \\nw[name]","我叫Wonmar，是个樵夫。\\nw[name]"]);
_tr_m.push(["ワシはこれまでずっと、ダーガと二人で暮らしておった。 \\nw[name]","I've lived here for a long time along with Dagger. \\nw[name]","나는 지금까지 쭉 Dagger와 둘이서 살았지. \\nw[name]","一直以來，我都是跟Dagger兩個人住在這裡生活。 \\nw[name]","一直以来，我都是跟Dagger两个人住在这里生活。\\nw[name]"]);
_tr_m.push(["ダーガはまだ幼いうちに事故で両親を無くしてな・・・。 その後、ワシが引き取った。 \\nw[name]","Dagger lost both her parents to an accident when she was young, so I adopted her as my own. \\nw[name]","Dagger는 어릴 때 사고로 부모님을 잃어서・・・ 그 후로 내가 돌봤어. \\nw[name]","Dagger的父母在她小時候意外過世……之後，就由我收養了她。 \\nw[name]","Dagger的父母在她小时候意外过世……之后，就由我收养了她。\\nw[name]"]);
_tr_m.push(["孫娘のように可愛がって、 ずっと世話してやったというのに・・・。 \\nw[name]","She's like a granddaughter to me, and she was just the cutest thing. \\nw[name]","내 손녀처럼 아끼고 보살펴 주었는데・・・ \\nw[name]","我把她當成孫女來疼愛，一直照顧著她。但是…… \\nw[name]","我把她当成孙女来疼爱，一直照顾着她。但是……\\nw[name]"]);
_tr_m.push(["あいつは段々反抗期というやつで、 生意気な口ばかり聞くようになってな。 \\nw[name]","But these days she only knows how to rebel and talk back with some serious attitude. \\nw[name]","점점 반항기가 되면서 온갖 트집을 잡으며 내 말을 안 들었지. \\nw[name]","她漸漸來到了反抗期，講話變得越來越沒大沒小。 \\nw[name]","她渐渐变得叛逆起来，讲话越来越没大没小。\\nw[name]"]);
_tr_m.push(["やれイビキがうるさいとか、 一緒に服を洗濯しないでとか言い出しおった。 \\nw[name]","She complains about my snoring, and demands that I do our laundry separately. \\nw[name]","코 고는 소리가 시끄럽다는 둥, 자기 옷과 내 옷을 함께 빨지 말라는 둥. \\nw[name]","說什麼我打鼾很吵，不准把我和她的衣服混在一起洗之類的。 \\nw[name]","说什么我打鼾很吵，不准把我和她的衣服混在一起洗之类的。\\nw[name]"]);
_tr_m.push(["そしてこの前・・・ついに家出しおったのじゃ！ \\nw[name]","Just recently, she even ran away from home! \\nw[name]","그러더니 얼마 전에・・・ 결국 가출을 했어! \\nw[name]","然後，前一陣子……她終於離家出走了！ \\nw[name]","然后，前一阵子……她终于离家出走了！\\nw[name]"]);
_tr_m.push(["しかも「もう二度と帰らねえよクソジジイ！」 と言い放ってな！！ \\nw[name]","She said she'd never return and even called me a stinking geezer! \\nw[name]","게다가 「다시는 안 돌아올 거야! 이 꼰대야!」라는 거야!! \\nw[name]","而且走之前還對我放話說「死老頭，我不會再回來了！」！！ \\nw[name]","而且走之前还对我放话说「死老头，我不会再回来了！」！！\\nw[name]"]);
_tr_m.push(["ワシも「うるせぇクソガキ！」と言おうとしたが アイツは既に飛び出して行きおった！ \\nw[name]","Before I could call her a spoiled brat, she'd already darted out the door! \\nw[name]","나도 「시끄러워, 이 못된 녀석아!」라고 말하려 했는데 어느새 집을 뛰쳐나갔어! \\nw[name]","我本來也想回她「妳這個死小鬼，煩死了！」，但她早就跑走了！ \\nw[name]","我本来也想回她「你这个死小鬼，烦死了！」，但她早就跑走了！\\nw[name]"]);
_tr_m.push(["チキショウ！ ワシもあんな生意気なガキの顔なんかもう見たくないがな！ \\nw[name]","Flubbin' finickers! I don't care if I ever see that brazen little devil again! \\nw[name]","젠장! 나도 그런 건방진 녀석은 꼴도 보기 싫어! \\nw[name]","可惡！我也不太想再見到那個任性的臭小子！ \\nw[name]","可恶！我也不想再见到那个任性的臭小鬼！\\nw[name]"]);
_tr_m.push(["でも最後に「うるせぇクソガキ！」と 一発言ってやらねえと気がすまねえんじゃ！ \\nw[name]","But I can't rest unless I at least get in the last word and call her a spoiled brat! \\nw[name]","하지만 마지막에 「시끄러워, 이 못된 녀석아!」라고 따끔하게 말하지 못한 게 마음에 걸려! \\nw[name]","不過在那最後沒能向她說「妳這個死小鬼，煩死了！」，總覺得心裡不太舒坦！ \\nw[name]","不过在那最后没能向她说「你这个死小鬼，烦死了！」，总觉得心里不太舒坦！\\nw[name]"]);
_tr_m.push(["ゼエ、ゼエ・・・というわけでだ。 \\nw[name]","*wheeze* ...And that's my story. \\nw[name]","씩씩・・・ 아무튼 그렇게 된 거야. \\nw[name]","呼……呼……事情就是這樣。 \\nw[name]","呼……呼……事情就是这样。\\nw[name]"]);
_tr_m.push(["旅人よ、ダーガを無理矢理でもここに連れてきてくれんか？ その代わり、ワシのとっておきのモンをやろう。\\!  \\nw[name]","Young lady, will you drag her sorry butt back here? I've got something special for you if you do.\\!  \\nw[name]","여행자 양반, 억지로라도 Dagger를 데려와 주지 않겠어? 그러면 내가 엄청난 물건을 줄게.\\!  \\nw[name]","旅人啊，妳可以使出強硬手段，幫我把Dagger帶來這裡嗎？為了感謝妳，我會把我珍藏的寶物送給妳。\\!  \\nw[name]","旅人啊，你可以使出强硬手段，帮我把Dagger带来这里吗？为了感谢你，我会把我珍藏的宝物送给你。\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞好"]);
_tr_m.push([">いいえ",">No",">아니요","＞不要","＞不要"]);
_tr_m.push(["ふん。ならばお前にやるモンはないぞ。 \\nw[name]","Then there's nothing you can do for me. \\nw[name]","흥, 그렇다면 아무것도 못 줘. \\nw[name]","哼。那我就沒什麼東西可以給妳了。 \\nw[name]","哼。那我就没什么东西可以给你了。\\nw[name]"]);
_tr_m.push(["ふん。交渉成立だ。 \\nw[name]","Great. We've got a deal. \\nw[name]","흥, 협상 성립이군. \\nw[name]","哦。交涉成立。 \\nw[name]","哦。交易达成。\\nw[name]"]);
_tr_m.push(["ダーガは真っ赤な髪の色をした女性だ。 よろしく頼んだぞ。 \\nw[name]","Dagger is a girl with bright red hair. I hope you find her. \\nw[name]","Dagger는 머리가 새빨간 여자아이야. 잘 부탁해. \\nw[name]","Dagger是一位有著鮮紅色頭髮的女性。那這件事就拜託妳啦。 \\nw[name]","Dagger是一位有着鲜红色头发的女性。那这件事就拜托你啦。\\nw[name]"]);
_tr_m.push(["ダーガをここに連れてきてくれたら、 代わりにとっておきのモンをやろう。 \\nw[name]","Bring Dagger back and you'll have something special waiting for you. \\nw[name]","Dagger를 여기로 데려와 주면 그 대가로 엄청난 물건을 줄게. \\nw[name]","如果妳能把Dagger帶來這裡，我就把我珍藏的寶物送給妳。 \\nw[name]","如果你能把Dagger带来这里，我就把我珍藏的宝物送给你。\\nw[name]"]);
_tr_m.push(["ダーガは真っ赤な髪の色をした女性だ。 どこにいるかはわからん。 \\nw[name]","Dagger is a girl with bright red hair. I have no idea where she might've gone. \\nw[name]","Dagger는 머리가 새빨간 여자아이야. 어디 있는지는 몰라. \\nw[name]","Dagger是一位有著鮮紅色頭髮的女性。我不知道她在哪裡。 \\nw[name]","Dagger是一位有着鲜红色头发的女性。我不知道她在哪里。\\nw[name]"]);
_tr_m.push(["ダーガをウォンマーに会わせますか？ （\\i[1347]冒険者 ダーガ を持ち物から失います）\\! ","Let Dagger meet Wonmar? (You will lose the \\i[1347]Adventurous Dagger from your inventory.)\\! ","Dagger를 Wonmar와 만나게 하시겠습니까? (\\i[1347]모험가 Dagger는 소지품에서 사라집니다.)\\! ","要讓Dagger去見Wonmar嗎？ （\\i[1347]冒險者 Dagger會從攜帶物當中消失）\\! ","要让Dagger去见Wonmar吗？（\\i[1347]冒险者 Dagger会从携带物当中消失）\\! "]);
_tr_m.push([">はい",">Yes",">예","＞好","＞是"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push(["！！！ \\nw[name]","!!! \\nw[name]","!!! \\nw[name]","！！！ \\nw[name]","！！！\\nw[name]"]);
_tr_m.push(["・・・ダーガ。 \\nw[name]","...Dagger! \\nw[name]","・・・Dagger. \\nw[name]","……Dagger。 \\nw[name]","……Dagger。\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[ダーガ]","... \\nw[Dagger]","・・・. \\nw[Dagger]","…… \\nw[Dagger]","……\\nw[Dagger]"]);
_tr_m.push(["お前というやつは！！！ \\nw[name]","You insolent little... \\nw[name]","너, 이 녀석!!! \\nw[name]","妳這傢伙！！！ \\nw[name]","你这家伙！！！\\nw[name]"]);
_tr_m.push(["育ての親同然の男に！！ 「クソジジイ」などと暴言を吐き、出ていきやがって！！！ \\nw[name]","I raised you like my own! How dare you call me a stinking geezer and run out like that! \\nw[name]","부모나 다름없는 나를 「꼰대」라고 부르고 가출까지 하다니!!! \\nw[name]","竟然對養育妳長大的男人！說出「死老頭」這種沒禮貌的話！然後就離家出走了！！！ \\nw[name]","竟然对养育你长大的男人！说出「死老头」这种没礼貌的话！然后就离家出走了！！！\\nw[name]"]);
_tr_m.push(["それだけではない！！ おじいはイビキがうるさいだの何だのいっつもケチつけて！！ \\nw[name]","And to top it off, you complain about my snoring. You ingrate! \\nw[name]","그게 다가 아니지!! 할아버지는 코 고는 소리가 시끄럽네 어쩌네 트집을 잡고!! \\nw[name]","不僅如此！！妳還一直抱怨我的鼾聲很吵！！ \\nw[name]","不仅如此！！你还一直抱怨我的鼾声很吵！！\\nw[name]"]);
_tr_m.push(["育ての親に感謝もせずに、お前というやつは！！ \\nw[name]","How could you be so selfish?! \\nw[name]","너란 녀석은 키워 준 은혜도 모르는 거냐!? \\nw[name]","妳這傢伙竟然對養育妳長大的人也不懂得感謝！！ \\nw[name]","你这家伙竟然对养育你长大的人也不懂得感谢！！\\nw[name]"]);
_tr_m.push(["おじい！！ \\nw[ダーガ]","Gramps! \\nw[Dagger]","할아버지!! \\nw[Dagger]","爺爺！！ \\nw[Dagger]","爷爷！！\\nw[Dagger]"]);
_tr_m.push(["お前にクソジジイと言われた時！ ワシはお前に言ってやろうと思ったわい！！ \\nw[name]","When you called me a geezer, there was something I wanted to say to you! \\nw[name]","네가 나더러 꼰대라고 했을 때 난 너한테 이 말을 해 주려고 했어!! \\nw[name]","妳罵我死老頭的時候！我有句話一直想對妳說！！ \\nw[name]","你骂我死老头的时候！我有句话一直想对你说！！\\nw[name]"]);
_tr_m.push(["「うるせぇクソガキ！！」となあ！！！ \\nw[name]","I wanted to call you a spoiled brat! \\nw[name]","「시끄러워, 이 못된 녀석아!!」라고!! \\nw[name]","「妳這個死小鬼，煩死了！」！！！ \\nw[name]","「你这个死小鬼，烦死了！」！！！\\nw[name]"]);
_tr_m.push(["お前のほうこそクソガキだ！！ そう！！お前のほうがクソガキ！！！ \\nw[name]","You're a good-for-nothing little runt, and I can't believe I raised you! \\nw[name]","이 못된 녀석!! 그래!! 넌 못된 녀석이야!!! \\nw[name]","妳才是死小鬼！！沒錯！！妳才是死小鬼！！！ \\nw[name]","你才是死小鬼！！没错！！你才是死小鬼！！！\\nw[name]"]);
_tr_m.push(["おじいったらあ！！！！ \\nw[ダーガ]","Hey, Gramps! \\nw[Dagger]","할아버지!!!! \\nw[Dagger]","爺爺！！！！ \\nw[Dagger]","爷爷！！！！\\nw[Dagger]"]);
_tr_m.push(["おじい。人の話を聞いて。 \\nw[ダーガ]","Gramps! You gotta learn to listen. \\nw[Dagger]","할아버지, 내 말부터 들어 줘. \\nw[Dagger]","爺爺，你聽我說話啦。 \\nw[Dagger]","爷爷，你听我说话啦。\\nw[Dagger]"]);
_tr_m.push(["たしかに私は、口ではクソジジイと言ったわ・・・。 \\nw[ダーガ]","Maybe I did call you a stinkin' geezer... \\nw[Dagger]","내가 꼰대라고 부른 건 사실이야・・・ \\nw[Dagger]","我嘴巴上確實是叫你死老頭…… \\nw[Dagger]","我嘴巴上确实是叫你死老头……\\nw[Dagger]"]);
_tr_m.push(["そして実際に、心の中でクソジジイと思っていたわ！ \\nw[ダーガ]","And maybe I do think you really are a stinkin' geezer, too... \\nw[Dagger]","그리고 실제로 마음속으로 꼰대라고 생각했어! \\nw[Dagger]","在我心目中也確實認為你是個死老頭！ \\nw[Dagger]","在我心目中也确实认为你是个死老头！\\nw[Dagger]"]);
_tr_m.push(["テメェ！！このやろう！！！！ \\nw[name]","There you go again, you little rascal! \\nw[name]","이 녀석이!!!! \\nw[name]","妳！！妳這混帳！！！！ \\nw[name]","你！！你这混帐！！！！\\nw[name]"]);
_tr_m.push(["でも一つだけ訂正させて！ \\nw[ダーガ]","But you got one thing wrong! \\nw[Dagger]","하지만 이것만은 알아 줘! \\nw[Dagger]","不過，有一件事情我要更正！ \\nw[Dagger]","不过，有一件事情我要更正！\\nw[Dagger]"]);
_tr_m.push(["今まで育ててくれて・・・ 感謝してないわけないじゃない！ \\nw[ダーガ]","I've never once been ungrateful for you raising me! \\nw[Dagger]","지금까지 키워 줘서・・・ 감사하고 있어! \\nw[Dagger]","你養育我這麼長一段時間……我並不是不感謝你！ \\nw[Dagger]","你养育我这么长一段时间……我并不是不感谢你！\\nw[Dagger]"]);
_tr_m.push(["！！！ \\nw[name]","!!! \\nw[name]","!!! \\nw[name]","！！！ \\nw[name]","！！！\\nw[name]"]);
_tr_m.push(["何年もずーっと一人で、仕事して、家事もして。 料理も掃除もぜーんぶして、遊んでくれて。 \\nw[ダーガ]","For years, you worked and did all the chores. You cooked, cleaned, and even played with me. \\nw[Dagger]","오랫동안 혼자 일하면서 집안일도 하고. 요리도 청소도 전부 해 주고 나랑 놀아 줬지. \\nw[Dagger]","這些年來你都是一個人在努力，要工作，還要做家事。料理和打掃也都是由你一個人包辦，你甚至還會陪我玩。 \\nw[Dagger]","这些年来你都是一个人在努力，要工作，还要做家事。料理和打扫也都是由你一个人包办，你甚至还会陪我玩。\\nw[Dagger]"]);
_tr_m.push(["そのくせ弱い部分は少しも出さず・・・ 何もかも私のために尽くしてくれて。 \\nw[ダーガ]","You did all that by yourself without even a peep. And I know everything you did, you did for me. \\nw[Dagger]","그러면서 한 번도 약한 모습도 안 보이고・・・ 나를 위해 뭐든지 해 줬어. \\nw[Dagger]","儘管如此，你卻完全沒有露出軟弱的一面……為我付出了一切。 \\nw[Dagger]","尽管如此，你却完全没有露出软弱的一面……为我付出了一切。 \\nw[Dagger]"]);
_tr_m.push(["たとえクソジジイでも・・・ そんな親に、感謝しないわけないじゃない。 \\nw[ダーガ]","You may be a stinkin' geezer, but there's no way I'd be ungrateful for everything you've given me. \\nw[Dagger]","아무리 꼰대라고 해도・・・ 그런 사람에게 어떻게 감사하지 않을 수 있겠어. \\nw[Dagger]","就算你是個死老頭……我怎麼會不感謝這樣的親人呢？ \\nw[Dagger]","就算你是个死老头……我怎么会不感谢这样的亲人呢？\\nw[Dagger]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["言ってなかったけど・・・私、今仕事を探してるの。 \\nw[ダーガ]","I failed to mention it, but I'm out job hunting. \\nw[Dagger]","그동안 말하지 못했지만・・・ 난 지금 일자리를 찾고 있어. \\nw[Dagger]","雖然我沒跟你說……但我現在正在找工作。 \\nw[Dagger]","虽然我没跟你说……但我现在正在找工作。\\nw[Dagger]"]);
_tr_m.push(["か、勘違いしないでよ！！ 別におじいが楽になるようにとかじゃないんだから！！ \\nw[ダーガ]","But don't misunderstand! I'm not trying to earn money so you can take it easy or nothing! \\nw[Dagger]","차, 착각하지 마!! 딱히 할아버지 편하라고 그러는 건 아니니까!! \\nw[Dagger]","你、你可別誤會囉！！我並不是想讓爺爺過得輕鬆一點！！ \\nw[Dagger]","你，你可别误会！！我并不是想让爷爷过得轻松一点！！\\nw[Dagger]"]);
_tr_m.push(["ダーガ・・・。 \\nw[name]","Dagger... \\nw[name]","Dagger・・・ \\nw[name]","Dagger…… \\nw[name]","Dagger……\\nw[name]"]);
_tr_m.push(["じゃあもう私行くから！！ \\nw[ダーガ]","Well then, I'm going to go now! \\nw[Dagger]","그럼 난 그만 가 볼게!! \\nw[Dagger]","那麼，我要走囉！！ \\nw[Dagger]","那么，我要走了！！\\nw[Dagger]"]);
_tr_m.push(["・・・ダーガ。ちょっと待て。 \\nw[name]","Hold your horses there, Dagger. \\nw[name]","・・・Dagger, 기다려라. \\nw[name]","……Dagger，等一下。 \\nw[name]","……Dagger，等一下。\\nw[name]"]);
_tr_m.push(["な、なに！ \\nw[ダーガ]","What is it now? \\nw[Dagger]","뭐, 뭔데! \\nw[Dagger]","怎、怎樣啦！ \\nw[Dagger]","干，干什么！\\nw[Dagger]"]);
_tr_m.push(["その・・・あのな。 \\nw[name]","Um... How do I say this... \\nw[name]","저기・・・ 말이다. \\nw[name]","那個……就是…… \\nw[name]","那个……就是……\\nw[name]"]);
_tr_m.push(["いつでも戻ってこいよ。 \\nw[name]","You've always got a home here. \\nw[name]","언제든지 돌아와도 좋다. \\nw[name]","這個家，隨時都歡迎妳回來。 \\nw[name]","这个家，随时都欢迎你回来。\\nw[name]"]);
_tr_m.push(["・・・！ \\nw[ダーガ]","...! \\nw[Dagger]","・・・! \\nw[Dagger]","……！ \\nw[Dagger]","……！\\nw[Dagger]"]);
_tr_m.push(["・・・うん。 \\nw[ダーガ]","...Thanks. \\nw[Dagger]","・・・응. \\nw[Dagger]","……嗯。 \\nw[Dagger]","……嗯。\\nw[Dagger]"]);
_tr_m.push(["見苦しいところを見せたな。 \\nw[name]","I'm sorry you had to see that. \\nw[name]","창피한 모습을 보였군. \\nw[name]","抱歉，讓妳見到了難堪的一面。 \\nw[name]","抱歉，让你见到了难堪的一面。\\nw[name]"]);
_tr_m.push(["まあ、ダーガを連れてきてくれたからな。これをやろう。 \\nw[name]","You brought her back like you promised. Here, take this. \\nw[name]","뭐, Dagger를 데려와 줬으니 이걸 주마. \\nw[name]","不過，多虧了妳把Dagger帶過來。這個送妳吧。 \\nw[name]","不过，多亏了你把Dagger带过来。这个送你吧。\\nw[name]"]);
_tr_m.push(["ふん、ならば欲しいときにまた取りに来い。 \\nw[name]","Oh? Then come and get it whenever you want. \\nw[name]","흥, 그럼 언제든지 필요할 때 받으러 와라. \\nw[name]","哦，既然如此，就等妳想要的時候再來拿吧。 \\nw[name]","哦，既然如此，就等你想要的时候再来拿吧。\\nw[name]"]);
_tr_m.push(["それはワシの知る限り最強の斧・・・。 ワシには使いこなせんからの。持っていけ。 \\nw[name]","This is the strongest axe I know of. But it's much too big for these old bones. Take it. \\nw[name]","내가 아는 한, 그건 최강의 도끼다・・・ 나는 쓸 줄 모르니까 네가 가져가거라. \\nw[name]","這是我所知最強的斧頭……我沒辦法善用它，妳就帶走吧。 \\nw[name]","这是我所知最强的斧头……我没办法善用它，你就带走吧。\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["感謝したいのはワシのほうじゃ。 \\nw[name]","Obviously, I couldn't tell her that I'm the one who's grateful. \\nw[name]","고마워해야 할 사람은 나다. \\nw[name]","應該是我要感謝妳才對。 \\nw[name]","应该是我要感谢你才对。\\nw[name]"]);
_tr_m.push(["ふふ。 \\nw[name]","Hehe... \\nw[name]","후후. \\nw[name]","呵呵。 \\nw[name]","呵呵。\\nw[name]"]);
_tr_m.push(["！？ \\nw[name]","!? \\nw[name]","!? \\nw[name]","！？ \\nw[name]","！？\\nw[name]"]);
_tr_m.push(["旅人よ！まだいたのか！！ もう用は済んだはずじゃ。さっさと行けい！！ \\nw[name]","Young lady! We've got no more business here. Now go! \\nw[name]","여행자 양반! 아직도 있었나! 볼일은 다 끝났어. 어서 가!! \\nw[name]","旅人啊！妳怎麼還在這裡！？妳的事情應該辦完了吧。快點回去啦！！ \\nw[name]","旅人啊！你怎么还在这里！？你的事情应该办完了吧。快点回去啦！！\\nw[name]"]);
_tr_m.push(["ギガントアックスを取りに来たのだな。 \\nw[name]","So you've come for the Giant Axe... \\nw[name]","기간트 액스를 받으러 왔군. \\nw[name]","妳是來拿巨人斧頭的吧。 \\nw[name]","你是来拿巨人斧头的吧。\\nw[name]"]);
_tr_m.push(["ふん、ならば欲しいときにまた取りに来い。 \\nw[name]","Oh? Then come and get it whenever you want.\\nw[name]","흥, 그럼 언제든지 필요할 때 받으러 와라. \\nw[name]","哼，既然如此，就等妳想要時再來拿吧。　\\nw[name]","哦，既然如此，就等你想要的时候再来拿吧。\\nw[name]"]);
_tr_m.push(["おお、お前か。 旅は順調か？ \\nw[name]","It's you again. How goes your travels? \\nw[name]","오오, 자네로군. 여행은 잘되어 가나? \\nw[name]","喔喔，是妳啊。妳的旅行還順利吧？ \\nw[name]","喔喔，是你啊。你的旅行还顺利吧？\\nw[name]"]);
_tr_m.push(["・・・お前、武器商人だったのか。 ならばワシの斧も使いこなせるであろう。 \\nw[name]","So you're a weapons merchant? Then you must be able to wield my axe. \\nw[name]","・・・자네, 무기 상인이었나? 그럼 내 도끼도 잘 쓸 수 있겠군. \\nw[name]","……原來妳是武器商人啊。既然如此，應該能好好善用我的斧頭吧。 \\nw[name]","……原来你是武器商人啊。既然如此，应该能好好利用我的斧头吧。\\nw[name]"]);
_tr_m.push(["ワシはこれからもこのボロ家に住み続けるわい。 いつでもダーガが帰ってこられるように・・・。 \\nw[name]","I've got to maintain this shabby old house in case Dagger ever wants to come back. \\nw[name]","나는 앞으로도 이 낡은 오두막에서 살 거야. 언제든지 Dagger가 돌아올 수 있도록・・・ \\nw[name]","我之後也會繼續住在那間破爛房子裡。這樣Dagger隨時都可以回來…… \\nw[name]","我之后也会继续住在那间破烂房子里。这样Dagger随时都可以回来……\\nw[name]"]);
_tr_m.push(["い、いや！！きこりの仕事があるからな！！ \\nw[name]","Ahem! Well these trees aren't going to cut themselves! \\nw[name]","아, 아 참!! 난 나무를 베러 가야 해서!! \\nw[name]","不、不對！！是因為我要砍柴才會住在那裡的！！ \\nw[name]","不，不对！！是因为我要砍柴才会住在那里的！！\\nw[name]"]);
_tr_m.push(["旅の方よ。 ここを西に進むと森があるぞ。 \\nw[name]","Oh, that reminds me. There is a forest west of here. \\nw[name]","여행자여, 여기서 서쪽으로 가면 숲이 있다네. \\nw[name]","旅人啊。從這裡往西前進，會遇到一片森林。 \\nw[name]","旅人啊。从这里往西前进，会遇到一片森林。\\nw[name]"]);
_tr_m.push(["森には不思議な洞窟があるが、 何やら得体の知れないものが入り口を塞いでおる・・・。 \\nw[name]","There's a mysterious cave among the trees. Something strange blocks its entrance though. \\nw[name]","숲에는 신기한 동굴이 있는데 정체를 알 수 없는 뭔가가 입구를 막고 있어・・・ \\nw[name]","森林裡有個神秘的洞窟，但似乎被不明生物把洞口塞住了…… \\nw[name]","森林里有个神秘的洞窟，但似乎被不明生物把洞口塞住了……\\nw[name]"]);
_tr_m.push(["そなた、冒険初心者であるな？ この一流冒険者の私がアドバイスしてやろう。 \\nw[name]","You're still a novice when it comes to adventuring, right? Well, I'm as seasoned as they come, so heed my words. \\nw[name]","그대는 초보 모험가로군? 일류 모험가인 이 몸이 충고를 해 주지. \\nw[name]","妳是冒險的新手吧？就由我這個一流冒險者來給妳建議吧。 \\nw[name]","你是冒险的新手吧？就由我这个一流冒险者来给你建议吧。\\nw[name]"]);
_tr_m.push(["敵を拾いたくなったら、弱らせることだ。 体力が存分に余っていると振り払われてしまうぞ。 \\nw[name]","If you want to pick up an enemy, you must weaken them first. If they're healthy, they'll just dodge your attempt. \\nw[name]","적을 줍고 싶으면 일단 약하게 만들어야 해. 체력이 많이 남아 있으면 적이 그대를 밀어낼 테니까. \\nw[name]","如果妳想撿起敵人，就要先讓敵人變得虛弱。如果敵人還有很多體力，妳就會被對方甩開。 \\nw[name]","如果你想捡起敌人，就要先让敌人变得虚弱。如果敌人还有很多体力，你就会被对方甩开。\\nw[name]"]);
_tr_m.push(["少女はうなされているようだ・・・。","The girl appears to be in distress...","소녀는 괴로워 보인다・・・","少女似乎在夢囈……","少女似乎在梦呓……"]);
_tr_m.push(["う、う～ん。 \\nw[？？？？？]","Ngh... Ungh... \\nw[?????]","으, 으음~ \\nw[?????]","唔……唔嗯…… \\nw[？？？？？]","唔……唔嗯……\\nw[？？？？？]"]);
_tr_m.push(["はあ、はあ。あんたが助けてくれたのね・・・。 \\nw[？？？？？]","Oh, so you're the one who saved me... \\nw[?????]","허억, 허억. 네가 날 구해 주었구나・・・ \\nw[?????]","呼、呼……是妳救了我啊…… \\nw[？？？？？]","呼，呼……是你救了我啊……\\nw[？？？？？]"]);
_tr_m.push(["ありがとう。これはお礼よ。 \\nw[？？？？？]","Thank you. Let me give you this. \\nw[?????]","고마워. 이걸 받아 줘. \\nw[?????]","謝謝妳。這是我的謝禮。 \\nw[？？？？？]","谢谢你。这是我的谢礼。\\nw[？？？？？]"]);
_tr_m.push(["\\i[141]アルマの石を 5個 手に入れた！","You got 5 \\i[141]Arma Stones!","\\i[141]알마의 돌 5개를 획득했다!","獲得5顆 \\i[141]魂石！","获得5颗\\i[141]魂石！"]);
_tr_m.push(["私は魔女クレモア。 あんたは・・・\\n[1]っていうのね。 \\nw[name]","I'm Cremon the Witch. And you must be \\n[1]. \\nw[name]","난 마녀 크레모아야. 네 이름은・・・ \\n[1]구나. \\nw[name]","我是魔女Cremon。 妳是……\\n[1]對吧。 \\nw[name]","我是魔女Cremon。你是……\\n[1]对吧。\\nw[name]"]);
_tr_m.push(["数日前、急にさっきの気持ち悪い触手が床から生えてきたの。 \\nw[name]","That disgusting tentacle sprouted up from the floor a few days ago. \\nw[name]","며칠 전에 갑자기, 아까 그 기분 나쁜 촉수가 바닥에서 솟아났어. \\nw[name]","數天前，突然從地板上長出了剛才那種噁心的觸手。 \\nw[name]","数天前，地板上突然长出了刚才那种恶心的触手。\\nw[name]"]);
_tr_m.push(["もちろん私は抵抗したんだけど、身体に巻き付かれて・・・ 私の魔力を全部吸い取られてしまったの。 \\nw[name]","I fought at first, but it somehow grabbed me and now I'm no longer able to use my magic. \\nw[name]","물론 나는 맞서 싸웠지만 내 몸을 칭칭 감더니・・・ 내 마력을 전부 흡수해 버렸어. \\nw[name]","我當然有試著抵抗，但觸手纏繞著我的身體……把我的魔力全都吸走了。 \\nw[name]","我试着抵抗，但触手缠绕着我的身体……把我的魔力全都吸走了。\\nw[name]"]);
_tr_m.push(["この触手の本体がどこかにいるはずだわ。 そいつを倒せば、私の魔力ももとに戻るはず。 \\nw[name]","I don't know where the creature itself is... But my magic should return if it's defeated. \\nw[name]","이 촉수의 본체가 어딘가에 있을 거야. 그걸 쓰러뜨리면 내 마력도 원래대로 돌아오겠지. \\nw[name]","這些觸手的本體一定就潛藏在某處。如果能打倒牠，我的魔力應該就會恢復了。 \\nw[name]","这些触手的本体一定就潜藏在某处。如果能打倒它，我的魔力应该就会恢复了。\\nw[name]"]);
_tr_m.push(["でも、魔力がない状態の私じゃ戦えない・・・。 \\nw[name]","A witch without her magic is like a sitting duck. \\nw[name]","하지만 나는 마력 없이는 싸울 수 없어・・・ \\nw[name]","不過，我現在沒有魔力，無法戰鬥…… \\nw[name]","不过，我现在没有魔力，无法战斗……\\nw[name]"]);
_tr_m.push(["そこで・・・あのね、助けてもらっておいてこんなこと言うのは 図々しいんだけど・・・。 \\nw[name]","I know it's a lot to ask since you just saved me, but... \\nw[name]","그래서 말인데・・・ 방금 도움을 받고 이런 말을 하기는 뻔뻔하지만・・・ \\nw[name]","因此……雖然我被妳所救，還向妳提出這種請求感覺實在是很厚臉皮，不過…… \\nw[name]","因此……虽然我被你所救，还向你提出这种请求感觉实在是很厚脸皮，不过……\\nw[name]"]);
_tr_m.push(["お願い！触手の本体を倒してきてほしいの！ \\nw[name]","Please! Defeat whatever it is behind that tentacle! \\nw[name]","부탁이야! 촉수의 본체를 쓰러뜨려 줘! \\nw[name]","拜託！希望妳去幫我打倒觸手的本體！ \\nw[name]","拜托！希望你去帮我打倒触手的本体！\\nw[name]"]);
_tr_m.push(["\\n[1]、あんた見かけによらず強そうだからね！ \\nw[name]","Despite your appearance, \\n[1], I know you're strong! \\nw[name]","\\n[1], 너는 보기보다 강할 것 같아! \\nw[name]","\\n[1]，妳雖然看起來不起眼，但感覺其實還滿強的吧！ \\nw[name]","\\n[1]，你虽然看起来不起眼，但感觉其实还挺强的吧！\\nw[name]"]);
_tr_m.push(["・・・え？なんでこんな森に一人で住んでるのかって？ \\nw[name]","Hm? You want to know why I live alone in the forest? \\nw[name]","・・・응? 왜 이런 숲 속에 혼자 사냐고? \\nw[name]","……咦？妳問我為什麼會一個人住在這種森林裡？ \\nw[name]","……咦？你问我为什么会一个人住在这种森林里？\\nw[name]"]);
_tr_m.push(["ふふん。魔女は人前に 姿を見せちゃいけない決まりになってるのよ。 \\nw[name]","Hehe... There's an unwritten rule saying witches can't be seen by the people. \\nw[name]","후훗. 마녀는 남들 앞에 모습을 드러내서는 안 돼. \\nw[name]","哼哼。這當然是因為魔女不能在別人面前露出真面目啊。 \\nw[name]","哼哼。这当然是因为魔女不能在别人面前露出真面目啊。\\nw[name]"]);
_tr_m.push(["だから、友達がいないわけじゃないわよ。 いい？友達がいないわけじゃないわよ。 \\nw[name]","That doesn't mean I don't have any friends, okay? I DO have friends. \\nw[name]","그렇다고 친구가 없는 건 아니야. 알겠지? 친구가 없는 건 아니라구. \\nw[name]","所以，我並不是沒有朋友喔。聽好了，我並不是沒有朋友喔。 \\nw[name]","所以，我并不是没有朋友喔。听好了，我并不是没有朋友喔。\\nw[name]"]);
_tr_m.push(["・・・そんなことどうでもいいから、 触手の本体を倒してきてよ！お願いね！ \\nw[name]","Anyway, that's not important. Defeat the creature controlling those tentacles. Please! \\nw[name]","・・・아무튼 그건 상관없는 일이니까 촉수의 본체를 쓰러뜨려 줘! 부탁해! \\nw[name]","……這種事情不重要啦，請妳去幫我打倒觸手的本體吧！拜託妳了！ \\nw[name]","……这种事情不重要啦，请你去帮我打倒触手的本体吧！拜托你了！\\nw[name]"]);
_tr_m.push(["私的には近くの洞窟が死ぬほど怪しいと思ってるわ。 \\nw[name]","I'd bet my crystal ball you'll find it in the nearby cave. \\nw[name]","내가 보기에는 근처의 동굴이 아주아주 수상해. \\nw[name]","我覺得附近那個洞窟超級可疑。 \\nw[name]","我觉得附近那个洞窟超级可疑。\\nw[name]"]);
_tr_m.push(["触手の本体を探して、倒してきてほしいの！ お願いね！ \\nw[name]","Find the main body controlling the tentacles and defeat it. Please! \\nw[name]","촉수의 본체를 찾아서 쓰러뜨려 줘! 부탁할게! \\nw[name]","希望妳能找出觸手的本體，並幫我打倒牠！拜託了！ \\nw[name]","希望你能找出触手的本体，并帮我打倒它！拜托了！\\nw[name]"]);
_tr_m.push(["私的には近くの洞窟が死ぬほど怪しいと思ってるわ。 \\nw[name]","I'd bet my crystal ball you'll find it in the nearby cave. \\nw[name]","내가 보기에는 근처의 동굴이 아주아주 수상해. \\nw[name]","我覺得附近那個洞窟超級可疑。 \\nw[name]","我觉得附近那个洞窟超级可疑。\\nw[name]"]);
_tr_m.push(["あなた誰？そんなことより聞いてよ！！ \\nw[name]","Who are you? Never mind, something amazing just happened! \\nw[name]","넌 누구야? 아니, 그것보다 내 말 좀 들어 봐!! \\nw[name]","妳是誰？先別管那個，妳聽我說！！ \\nw[name]","你是谁？先别管那个，你听我说！！\\nw[name]"]);
_tr_m.push(["さっき、失われていた私の魔力が戻ったの！！ \\nw[name]","My magical powers returned to me mere moments ago! \\nw[name]","잃어버렸던 내 마력이 방금 돌아왔어!! \\nw[name]","就在剛才，我失去的魔力回來了！！ \\nw[name]","就在刚才，我失去的魔力回来了！！\\nw[name]"]);
_tr_m.push(["さっきまで、変なピンクの触手みたいなやつに 魔力を吸い取られて動けなかったんだけど・・・。 \\nw[name]","There was this pink tentacle absorbing all my magic and I was paralyzed until just now. \\nw[name]","조금 전까지 이상한 핑크색 촉수 같은 게 내 마력을 전부 흡수해서 움직일 수 없었는데・・・ \\nw[name]","直到剛才為止，我一直被奇怪的粉紅色觸手吸取魔力，無法動彈。然而…… \\nw[name]","直到刚才为止，我一直被奇怪的粉红色触手吸取魔力，无法动弹。然而……\\nw[name]"]);
_tr_m.push(["いきなり触手が消滅して。 私の魔力も回復したの！ \\nw[name]","All of a sudden, the tentacle disappeared and my magic was restored! \\nw[name]","갑자기 촉수가 사라지고 내 마력도 돌아왔어! \\nw[name]","觸手突然消失，我的魔力也恢復了！ \\nw[name]","触手突然消失，我的魔力也恢复了！\\nw[name]"]);
_tr_m.push(["きっと誰かが、触手の本体を やっつけてくれたんだと思うわ！！ \\nw[name]","Somebody must've defeated the creature's main body! \\nw[name]","분명 누군가가 촉수의 본체를 해치워 준 거야!! \\nw[name]","一定是有人消滅了觸手的本體！！ \\nw[name]","一定是有人消灭了触手的本体！！\\nw[name]"]);
_tr_m.push(["・・・え？本体を倒したのはあなただって？ \\nw[name]","What? You're the one that saved me? \\nw[name]","・・・뭐? 네가 본체를 쓰러뜨렸다고? \\nw[name]","……咦？妳說打倒本體的人就是妳？ \\nw[name]","……咦？你说打倒本体的人就是你？\\nw[name]"]);
_tr_m.push(["本当に！？恩人だわ！！ \\nw[name]","Really? I owe you my life! \\nw[name]","정말!? 그럼 네가 내 은인이구나!! \\nw[name]","真的嗎！？妳是我的恩人！！ \\nw[name]","真的吗！？你是我的恩人！！\\nw[name]"]);
_tr_m.push(["名前はなんていうの？ ・・・\\n[1]っていうのね！ \\nw[name]","What is your name, by the way? \\n[1]? \\nw[name]","이름이 뭐야? ・・・\\n[1]라고? \\nw[name]","請問妳怎麼稱呼？……妳叫做\\n[1]啊！ \\nw[name]","请问你怎么称呼？……你叫做\\n[1]啊！\\nw[name]"]);
_tr_m.push(["あ！\\n[1]！ 本体を倒してくれたのね！？ \\nw[name]","Ah, \\n[1]! You defeated the creature, didn't you?! \\nw[name]","아! \\n[1]! 본체를 쓰러뜨렸구나!? \\nw[name]","啊！\\n[1]！妳幫我打倒本體了吧！？ \\nw[name]","啊！\\n[1]！你帮我打倒本体了吧！？\\nw[name]"]);
_tr_m.push(["聞いてよ！私の魔力が戻ったの！！ \\nw[name]","Look! My magic has returned! \\nw[name]","내 얘기 좀 들어 봐! 내 마력이 돌아왔어!! \\nw[name]","妳聽我說！我的魔力恢復了！！ \\nw[name]","你听我说！我的魔力恢复了！！\\nw[name]"]);
_tr_m.push(["\\n[1]のおかげだわ！本当にありがとう！！ \\nw[name]","It's all thanks to you, \\n[1]! I don't know how to repay you! \\nw[name]","\\n[1] 덕분이야! 정말 고마워!! \\nw[name]","都是多虧了\\n[1]妳的幫忙！真的很謝謝妳！！ \\nw[name]","都是多亏了\\n[1]你的帮忙！真的很谢谢你！！\\nw[name]"]);
_tr_m.push(["これ、お礼よ。受け取って！ \\nw[name]","I know! Take this! \\nw[name]","이건 내 답례야. 받아 줘! \\nw[name]","這是我的謝禮，請收下吧！ \\nw[name]","这是我的谢礼，请收下吧！\\nw[name]"]);
_tr_m.push(["今は持っていかないのね。 私が持ってるから、いつでも取りに来てよね。 \\nw[name]","You won't accept it now? Well, I'll hold on to it for you then. \\nw[name]","지금은 못 가져가는구나. 내가 갖고 있을 테니까 언제든지 받으러 와. \\nw[name]","妳現在沒有立刻要拿嗎？那就由我保管吧，妳隨時可以來拿。 \\nw[name]","你现在不拿吗？那就由我保管吧，你随时可以来拿。\\nw[name]"]);
_tr_m.push(["その武器はすごいわよ。世界に数本しかない最強の杖！ 降れば素人でも最強呪文をドカスカ打てちゃう！ \\nw[name]","That weapon is quite amazing. It's a mighty staff, one of only several in this world! Even a total novice can use it to cast a powerful spell! \\nw[name]","그건 엄청난 무기야. 이 세계에 몇 개밖에 없는 최강의 지팡이! 누구라도 휘두르면 최강의 주문을 마구마구 쓸 수 있어! \\nw[name]","這個武器很厲害喔。是全世界只有幾把的最強之杖！只要揮動這把杖，就算是外行人也能不間斷地使出最強咒語！ \\nw[name]","这个武器很厉害喔。是全世界只有几把的最强手杖！只要挥动这把手杖，就算是外行人也能不间断地使出最强咒语！\\nw[name]"]);
_tr_m.push(["だからまあ・・・絶対に人に向かって打たないようにね。 \\nw[name]","Careful though... Don't swing it when facing another person! \\nw[name]","그러니까・・・ 절대로 아무한테나 쓰지 마. \\nw[name]","所以……妳絕對不可以朝著人揮動喔。 \\nw[name]","所以……你绝对不可以朝着人挥喔。\\nw[name]"]);
_tr_m.push(["え？こんなすごそうなの、いいのかって？ \\nw[name]","Huh? Are you questioning my generosity for giving you such a weapon? \\nw[name]","뭐? 이렇게 대단한 걸 받아도 되냐고? \\nw[name]","咦？妳說我要把這麼厲害的東西送給妳真的好嗎？ \\nw[name]","咦？你说我要把这么厉害的东西送给你真的好吗？\\nw[name]"]);
_tr_m.push(["ふふふ・・・ 見た所、あんた、武器を集めてるんでしょ？ \\nw[name]","Hehehe... From the looks of it, you've got quite a collection going. \\nw[name]","후후후・・・ 딱 보니까 너, 무기를 모으고 있지? \\nw[name]","呵呵呵……就我看來，妳正在收集武器吧？ \\nw[name]","呵呵呵……就我看来，你正在收集武器吧？\\nw[name]"]);
_tr_m.push(["助けてもらったんだから、協力してあげるのは当然よ！ \\nw[name]","You helped me, so it's only natural I help you too! \\nw[name]","날 구해 줬으니 나도 널 돕는 게 당연하지! \\nw[name]","因為妳救了我，我也當然要協助妳啊！ \\nw[name]","因为你救了我，我也当然要协助你啊！\\nw[name]"]);
_tr_m.push(["それに。そんな杖なくても、 私は最強の魔女だから大丈夫なの。ふふん！ \\nw[name]","Besides, I'm the world's most powerful witch. I've no use for that staff! \\nw[name]","그리고 그 지팡이가 없어도 나는 최강의 마녀라서 괜찮아. 후훗! \\nw[name]","而且，就算我沒有那把杖，我還是最強的魔女，所以我沒問題的。哼哼！ \\nw[name]","而且，就算我没有那把手杖，我也还是最强的魔女，所以我没问题的。哼哼！\\nw[name]"]);
_tr_m.push(["さあ。武器集めの旅、がんばってね！\\n[1]！！ \\nw[name]","Good luck on your mission to collect weapons! Take care, \\n[1]! \\nw[name]","자, 그럼 앞으로도 열심히 무기를 모아 봐! \\n[1]!! \\nw[name]","那麼，妳收集武器的旅程就好好加油吧，\\n[1]！！ \\nw[name]","那么，你收集武器的旅程就好好加油吧，\\n[1]！！\\nw[name]"]);
_tr_m.push(["あなた誰？そんなことより聞いてよ！！ \\nw[name]","Who are you? Never mind, something amazing just happened! \\nw[name]","넌 누구야? 아니, 그것보다 내 말 좀 들어 봐!! \\nw[name]","妳是誰？先別管那個，妳聽我說！！ \\nw[name]","妳是誰？先別管那個，妳聽我說！！ \\nw[name]"]);
_tr_m.push(["さっき、失われていた私の魔力が戻ったの！！ \\nw[name]","My magical powers returned to me mere moments ago! \\nw[name]","잃어버렸던 내 마력이 방금 돌아왔어!! \\nw[name]","就在剛才，我失去的魔力回來了！！ \\nw[name]","就在剛才，我失去的魔力回來了！！ \\nw[name]"]);
_tr_m.push(["さっきまで、変なピンクの触手みたいなやつに 魔力を吸い取られて動けなかったんだけど・・・。 \\nw[name]","There was this pink tentacle absorbing all my magic and I was paralyzed until just now. \\nw[name]","조금 전까지 이상한 핑크색 촉수 같은 게 내 마력을 전부 흡수해서 움직일 수 없었는데・・・ \\nw[name]","直到剛才為止，我一直被奇怪的粉紅色觸手吸取魔力，無法動彈。然而…… \\nw[name]","直到剛才為止，我一直被奇怪的粉紅色觸手吸取魔力，無法動彈。然而…… \\nw[name]"]);
_tr_m.push(["いきなり触手が消滅して。 私の魔力も回復したの！ \\nw[name]","All of a sudden, the tentacle disappeared and my magic was restored! \\nw[name]","갑자기 촉수가 사라지고 내 마력도 돌아왔어! \\nw[name]","觸手突然消失，我的魔力也恢復了！ \\nw[name]","觸手突然消失，我的魔力也恢復了！ \\nw[name]"]);
_tr_m.push(["きっと誰かが、触手の本体を やっつけてくれたんだと思うわ！！ \\nw[name]","Somebody must've defeated the creature's main body! \\nw[name]","분명 누군가가 촉수의 본체를 해치워 준 거야!! \\nw[name]","一定是有人消滅了觸手的本體！！ \\nw[name]","一定是有人消滅了觸手的本體！！ \\nw[name]"]);
_tr_m.push(["・・・え？本体を倒したのはあなただって？ \\nw[name]","What? You're the one that saved me? \\nw[name]","・・・뭐? 네가 본체를 쓰러뜨렸다고? \\nw[name]","……咦？妳說打倒本體的人就是妳？ \\nw[name]","……咦？妳說打倒本體的人就是妳？ \\nw[name]"]);
_tr_m.push(["本当に！？恩人だわ！！ \\nw[name]","Really? I owe you my life! \\nw[name]","정말!? 그럼 네가 내 은인이구나!! \\nw[name]","真的嗎！？妳是我的恩人！！ \\nw[name]","真的嗎！？妳是我的恩人！！ \\nw[name]"]);
_tr_m.push(["名前はなんていうの？ ・・・\\n[1]っていうのね！ \\nw[name]","What is your name, by the way? \\n[1]? \\nw[name]","이름이 뭐야? ・・・\\n[1]라고? \\nw[name]","請問妳怎麼稱呼？……妳叫做\\n[1]啊！ \\nw[name]","請問妳怎麼稱呼？……妳叫做\\n[1]啊！ \\nw[name]"]);
_tr_m.push(["あ！\\n[1]！ 本体を倒してくれたのね！？ \\nw[name]","Ah, \\n[1]! You defeated the creature, didn't you?! \\nw[name]","아! \\n[1]! 본체를 쓰러뜨렸구나!? \\nw[name]","啊！\\n[1]！妳幫我打倒本體了吧！？ \\nw[name]","啊！\\n[1]！妳幫我打倒本體了吧！？ \\nw[name]"]);
_tr_m.push(["聞いてよ！私の魔力が戻ったの！！ \\nw[name]","Look! My magic has returned! \\nw[name]","내 얘기 좀 들어 봐! 내 마력이 돌아왔어!! \\nw[name]","妳聽我說！我的魔力恢復了！！ \\nw[name]","妳聽我說！我的魔力恢復了！！ \\nw[name]"]);
_tr_m.push(["\\n[1]のおかげだわ！本当にありがとう！！ \\nw[name]","It's all thanks to you, \\n[1]! I don't know how to repay you! \\nw[name]","\\n[1] 덕분이야! 정말 고마워!! \\nw[name]","都是多虧了\\n[1]妳的幫忙！真的很謝謝妳！！ \\nw[name]","都是多虧了\\n[1]妳的幫忙！真的很謝謝妳！！ \\nw[name]"]);
_tr_m.push(["これ、お礼よ。受け取って！ \\nw[name]","I know! Take this! \\nw[name]","이건 내 답례야. 받아 줘! \\nw[name]","這是我的謝禮，請收下吧！ \\nw[name]","這是我的謝禮，請收下吧！ \\nw[name]"]);
_tr_m.push(["今は持っていかないのね。 私が持ってるから、いつでも取りに来てよね。 \\nw[name]","You won't accept it now? Well, I'll hold on to it for you then. \\nw[name]","지금은 못 가져가는구나. 내가 갖고 있을 테니까 언제든지 받으러 와. \\nw[name]","妳現在沒有立刻要拿嗎？那就由我保管吧，妳隨時可以來拿。 \\nw[name]","妳現在沒有立刻要拿嗎？那就由我保管吧，妳隨時可以來拿。 \\nw[name]"]);
_tr_m.push(["その武器はすごいわよ。世界に数本しかない最強の杖！ 降れば素人でも最強呪文をドカスカ打てちゃう！ \\nw[name]","That weapon is quite amazing. It's a mighty staff, one of only several in this world! Even a total novice can use it to cast a powerful spell! \\nw[name]","그건 엄청난 무기야. 이 세계에 몇 개밖에 없는 최강의 지팡이! 누구라도 휘두르면 최강의 주문을 마구마구 쓸 수 있어! \\nw[name]","這個武器很厲害喔。是全世界只有幾把的最強之杖！只要揮動這把杖，就算是外行人也能不間斷地使出最強咒語！ \\nw[name]","這個武器很厲害喔。是全世界只有幾把的最強之杖！只要揮動這把杖，就算是外行人也能不間斷地使出最強咒語！ \\nw[name]"]);
_tr_m.push(["だからまあ・・・絶対に人に向かって打たないようにね。 \\nw[name]","Careful though... Don't swing it when facing another person! \\nw[name]","그러니까・・・ 절대로 아무한테나 쓰지 마. \\nw[name]","所以……妳絕對不可以朝著人揮動喔。 \\nw[name]","所以……妳絕對不可以朝著人揮動喔。 \\nw[name]"]);
_tr_m.push(["え？こんなすごそうなの、いいのかって？ \\nw[name]","Huh? Are you questioning my generosity for giving you such a weapon? \\nw[name]","뭐? 이렇게 대단한 걸 받아도 되냐고? \\nw[name]","咦？妳說我要把這麼厲害的東西送給妳真的好嗎？ \\nw[name]","咦？妳說我要把這麼厲害的東西送給妳真的好嗎？ \\nw[name]"]);
_tr_m.push(["ふふふ・・・ 見た所、あんた、武器を集めてるんでしょ？ \\nw[name]","Hehehe... From the looks of it, you've got quite a collection going. \\nw[name]","후후후・・・ 딱 보니까 너, 무기를 모으고 있지? \\nw[name]","呵呵呵……就我看來，妳正在收集武器吧？ \\nw[name]","呵呵呵……就我看來，妳正在收集武器吧？ \\nw[name]"]);
_tr_m.push(["助けてもらったんだから、協力してあげるのは当然よ！ \\nw[name]","You helped me, so it's only natural I help you too! \\nw[name]","날 구해 줬으니 나도 널 돕는 게 당연하지! \\nw[name]","因為妳救了我，我也當然要協助妳啊！ \\nw[name]","因為妳救了我，我也當然要協助妳啊！ \\nw[name]"]);
_tr_m.push(["それに。そんな杖なくても、 私は最強の魔女だから大丈夫なの。ふふん！ \\nw[name]","Besides, I'm the world's most powerful witch. I've no use for that staff! \\nw[name]","그리고 그 지팡이가 없어도 나는 최강의 마녀라서 괜찮아. 후훗! \\nw[name]","而且，就算我沒有那把杖，我還是最強的魔女，所以我沒問題的。哼哼！ \\nw[name]","而且，就算我沒有那把杖，我還是最強的魔女，所以我沒問題的。哼哼！ \\nw[name]"]);
_tr_m.push(["さあ。武器集めの旅、がんばってね！\\n[1]！！ \\nw[name]","Good luck on your mission to collect weapons! Take care, \\n[1]! \\nw[name]","자, 그럼 앞으로도 열심히 무기를 모아 봐! \\n[1]!! \\nw[name]","那麼，妳收集武器的旅程就好好加油吧，\\n[1]！！ \\nw[name]","那麼，妳收集武器的旅程就好好加油吧，\\n[1]！！ \\nw[name]"]);
_tr_m.push(["杖を取りに来たのね！ \\nw[name]","So you've come for the staff! \\nw[name]","지팡이를 받으러 왔구나! \\nw[name]","妳來拿杖了啊！ \\nw[name]","你来拿手杖了啊！\\nw[name]"]);
_tr_m.push(["今は持っていかないのね。 私が持ってるから、いつでも取りに来てよね。 \\nw[name]","You won't accept it now? Well, I'll hold onto it for you then.\\nw[name]","지금은 못 가져가는구나. 내가 갖고 있을 테니까 언제든지 받으러 와. \\nw[name]","妳現在沒有立刻要拿嗎？那就由我保管吧，妳隨時可以來拿。 \\nw[name]","你现在不拿吗？那就由我保管吧，你随时可以来拿。\\nw[name]"]);
_tr_m.push(["いざ森から出ようとなると、名残惜しいわね。 もうちょっとここでゆっくりしていくことにするわ。 \\nw[name]","I'm reluctant to leave the forest now. I'll stay just a bit longer. \\nw[name]","막상 숲에서 나가려고 하니까 아쉽네. 조금 더 여기서 편하게 지내다 갈래. \\nw[name]","一旦決定要離開森林，總覺得很捨不得呢。我還想稍微在這裡悠閒地待上一陣子。 \\nw[name]","一旦决定要离开森林，总觉得很舍不得呢。我还想稍微在这里悠闲地待上一阵子。\\nw[name]"]);
_tr_m.push(["\\n[1]、あんたも無理せずがんばりなさい。 \\nw[name]","\\n[1], be sure to always give it your all, but take care of yourself too. \\nw[name]","\\n[1], 너도 무리하지 말고 힘내. \\nw[name]","\\n[1]，妳也別太勉強，繼續努力吧。 \\nw[name]","\\n[1]，你也别太勉强，继续努力吧。\\nw[name]"]);
_tr_m.push(["ゆっくり空でも飛んで見渡してみたら、 案外見えてくるものもあるわよ。 \\nw[name]","If you look carefully when flying across the sky, you may see things you've never noticed before. \\nw[name]","느긋하게 하늘을 날면서 둘러보면 의외의 것이 보이기도 해. \\nw[name]","只要緩緩地飛在天空中俯瞰大地，有時也會意外地看到許多東西喔。 \\nw[name]","只要缓缓地飞在天空中俯瞰大地，有时也会意外地看到许多东西喔。\\nw[name]"]);
_tr_m.push(["あっ！ \\nw[name]","Ah! \\nw[name]","앗! \\nw[name]","啊！ \\nw[name]","啊！\\nw[name]"]);
_tr_m.push(["そのほうき！！私の\\i[1238]空飛ぶほうきじゃない！！ どうしてあんたが持ってるの！？ \\nw[name]","That broom! That's my \\i[1238]Flying Broom! Why do you have it? \\nw[name]","그 빗자루는 내 \\i[1238]하늘을 나는 빗자루잖아!! 그걸 네가 왜 갖고 있어!? \\nw[name]","那根掃帚！！那不是我的\\i[1238]飛天掃帚嗎！！為什麼會在妳手上！？ \\nw[name]","那根扫帚！！那不是我的\\i[1238]飞天扫帚吗！！为什么会在你手上！？\\nw[name]"]);
_tr_m.push(["もしかして、見つけてくれたのね！？ 助かった～、これでこの森から出られるわ！！ \\nw[name]","Don't tell me...you found it for me?! Phew! Now I can finally leave this forest! \\nw[name]","혹시 나 대신 찾아 준 거야!? 고마워~ 이제 숲에서 나갈 수 있게 됐어!! \\nw[name]","該不會是妳幫我找到的吧！？太好了～這樣我就能離開這片森林了！！ \\nw[name]","该不会是你帮我找到的吧！？太好了～这样我就能离开这片森林了！！\\nw[name]"]);
_tr_m.push(["さあ、はやくよこしなさい！\\!  \\nw[name]","Hurry up and give it to me!\\!  \\nw[name]","자, 빨리 이리 줘!\\!  \\nw[name]","來，快點給我！\\!  \\nw[name]","来，快点给我！\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞好"]);
_tr_m.push([">いいえ",">No",">아니요","＞不要","＞不要"]);
_tr_m.push(["ええ・・・私のなのに・・・。 \\nw[name]","But...it belongs to me... \\nw[name]","에이・・・ 그거 내 건데・・・ \\nw[name]","咦咦……那明明是我的東西耶…… \\nw[name]","咦咦……那明明是我的东西耶……\\nw[name]"]);
_tr_m.push(["\\n[1]は魔女クレモアに \\i[1238]空飛ぶほうきを渡した！","\\n[1] handed the \\i[1238]Flying Broom to Cremon.","\\n[1]는 마녀 Cremon에게 \\i[1238]하늘을 나는 빗자루를 건넸다!","\\n[1]把 \\i[1238]飛天掃帚交給了魔女Cremon！","\\n[1]把\\i[1238]飞天扫帚交给了魔女Cremon！"]);
_tr_m.push(["ふふふ・・・やっと私のほうきが帰ってきたわ。 \\nw[name]","Hehehe! Finally, I got my broom back. \\nw[name]","후후후・・・ 드디어 내 빗자루가 돌아왔네. \\nw[name]","呵呵呵……我的掃帚終於回來了。 \\nw[name]","呵呵呵……我的扫帚终于回来了。\\nw[name]"]);
_tr_m.push(["ん？何よ、そのお礼が欲しそうな目は。 \\nw[name]","Hey, wipe that look off your face. It's screaming that you want something in return. \\nw[name]","응? 그 답례를 바라는 눈빛은 뭐야? \\nw[name]","嗯？什麼啊，妳那看起來很想要謝禮的眼神是怎樣？ \\nw[name]","嗯？什么啊，你那看起来很想要谢礼的眼神是怎么回事？\\nw[name]"]);
_tr_m.push(["しょうがないわね。これでどう？ \\nw[name]","I guess I could give you this. \\nw[name]","어쩔 수 없지. 이거면 되겠어? \\nw[name]","真拿妳沒辦法。這個怎麼樣？ \\nw[name]","真拿你没办法。这个怎么样？\\nw[name]"]);
_tr_m.push(["なんと魔女クレモアは\\n[1]に魔法をかけた！","Cremon casts her magic on \\n[1]!","마녀 Cremon는 \\n[1]에게 마법을 걸었다!","魔女Cremon竟然對\\n[1]施放了魔法！","魔女Cremon竟然对\\n[1]施放了魔法！"]);
_tr_m.push(["ふっふっふ。これであんたは 一生私の言うなりになることしかできないわ・・・。 \\nw[name]","Hehe... Now you must obey my every command for the rest of your days. \\nw[name]","후후후. 이제 넌 평생 내가 시키는 대로 해야 해・・・ \\nw[name]","呵呵呵，這樣一來，妳一輩子就只能乖乖對我言聽計從了…… \\nw[name]","呵呵呵，这样一来，你一辈子就只能乖乖对我言听计从了……\\nw[name]"]);
_tr_m.push(["まんまとひっかかったわね。 ふははははは！！！ \\nw[name]","Muahaha! You really fell for it! \\nw[name]","완전히 걸려들었군. 푸하하하하하!!! \\nw[name]","妳完全中計了呢。呵哈哈哈哈哈！！！ \\nw[name]","你完全中计了呢。呵哈哈哈哈哈！！！\\nw[name]"]);
_tr_m.push(["\\n[1]は「クソッ！ハメられたッ！！」と思った！","\\n[1] is afraid she's been deceived!","\\n[1]는 「이런! 당했다!!」라고 생각했다!","\\n[1]心想：「可惡！我中計了！！」。","\\n[1]心想：「可恶！我中计了！！」。"]);
_tr_m.push(["・・・冗談よ。 \\nw[name]","I'm just teasing you. \\nw[name]","・・・농담이야. \\nw[name]","……開玩笑的啦。 \\nw[name]","……开玩笑的啦。\\nw[name]"]);
_tr_m.push(["なんとなんと！ \\nw[name]","You see... \\nw[name]","이번엔 진짜니까 잘 들어 봐! \\nw[name]","其實……！ \\nw[name]","其实……！\\nw[name]"]);
_tr_m.push(["今の魔法で、あんたは \\i[1238]空飛ぶほうきがなくても空を飛べるようになったわ！ \\nw[name]","That spell allows you to fly even without a \\i[1238]Flying Broom! \\nw[name]","방금 그 마법으로 넌 \\i[1238]하늘을 나는 빗자루가 없어도 하늘을 날 수 있게 되었어! \\nw[name]","剛剛的魔法，讓妳就算沒有 \\i[1238]飛天掃帚，也能在天空飛了！ \\nw[name]","有了刚刚的魔法，你就算没有\\i[1238]飞天扫帚，现在也能在天空飞了！\\nw[name]"]);
_tr_m.push(["なんでもいいから、 ほうき代わりの媒体となるものが必要だけどね。 \\nw[name]","You will need something to act as a charm in its place, though. \\nw[name]","뭐든지 상관없긴 하지만 그래도 빗자루를 대신할 물건은 필요할 거야. \\nw[name]","但還是需要一個媒介來代替掃帚，無論是什麼東西都行。 \\nw[name]","但还是需要一个媒介来代替扫帚，无论是什么东西都行。\\nw[name]"]);
_tr_m.push(["ふふん。けっこうすごいでしょ！ これに関しては私もすごいと思うわ！ \\nw[name]","Aren't I amazing? I like to think so, too! \\nw[name]","후훗, 멋지지? 나도 이것만큼은 대단하다고 생각해! \\nw[name]","哼哼，我很厲害吧！關於這件事，我也覺得自己很厲害！ \\nw[name]","哼哼，我很厉害吧！关于这件事，我也觉得自己很厉害！\\nw[name]"]);
_tr_m.push(["ふっふっふ。どうよ、この魔法！ さすが天才魔女クレモア様ね。 \\nw[name]","Well, what do you think? I am truly a genius, no? \\nw[name]","후후후. 이 마법 어때? 역시 천재 마녀 Cremon답지? \\nw[name]","呵呵呵。這個魔法怎麼樣？不愧是天才魔女Cremon大人對吧？ \\nw[name]","呵呵呵。这个魔法怎么样？不愧是天才魔女Cremon大人对吧？ \\nw[name]"]);
_tr_m.push(["え？ 自分にその魔法をかければ森から出られたんじゃ、って？ \\nw[name]","You want to know why didn't I just use this magic to fly out of the forest on my own? \\nw[name]","뭐? 진작 스스로 그 마법을 걸었으면 숲에서 나갈 수 있지 않았냐고? \\nw[name]","咦？妳說我只要對自己施放這個魔法，就可以離開這片森林了？ \\nw[name]","咦？你说我只要对自己施放这个魔法，就可以离开这片森林了？ \\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["・・・・・。 \\nw[name]","...... \\nw[name]","・・・・・. \\nw[name]","……… \\nw[name]","………\\nw[name]"]);
_tr_m.push(["ふ、ふん。 ちょっと頭がきれるからって調子に乗らないでよね！ \\nw[name]","Hmph! You think you're so clever! \\nw[name]","흐, 흥. 좀 똑똑하다고 잘난 척 하지 마! \\nw[name]","哼、哼！妳不要因為自己頭腦稍微聰明了點，就得意忘形了！ \\nw[name]","哼，哼！你不要因为自己头脑稍微聪明了点，就得意忘形了！\\nw[name]"]);
_tr_m.push(["あら、\\n[1]じゃない。 ふふん、私の杖は役に立ってる？ \\nw[name]","Hey there, \\n[1]! Is my staff proving useful? Hehe... \\nw[name]","어머, \\n[1]. 후훗, 내 지팡이는 잘 쓰고 있어? \\nw[name]","哎呀，這不是\\n[1]嗎？呵呵，我的杖有幫上妳的忙嗎？ \\nw[name]","哎呀，这不是\\n[1]吗？呵呵，我的手杖有帮上你的忙吗？\\nw[name]"]);
_tr_m.push(["・・・あんたなら高みを目指せる気がするわ。 武器集めの旅、がんばってね！\\n[1]！！ \\nw[name]","I know you'll reach even greater heights, \\n[1]. Good luck on your weapons collecting journey! \\nw[name]","・・・너라면 정상을 노릴 수 있을 거야. 앞으로도 열심히 무기를 모아 봐! \\n[1]!! \\nw[name]","……如果是妳，應該能達成更高的目標呢。妳的武器收集之旅就好好加油吧！\\n[1]！！ \\nw[name]","……如果是你，应该能达成更高的目标呢。你的武器收集之旅就好好加油吧！\\n[1]！！\\nw[name]"]);
_tr_m.push(["それにしても・・・私の空飛ぶほうき。 どこ行っちゃったのかしら・・・。 \\nw[name]","But you know, my flying broom is gone. I don't know where it went... \\nw[name]","그나저나・・・ 내 하늘을 나는 빗자루는 어디로 갔을까・・・ \\nw[name]","不過……我的飛天掃帚跑哪去了呢…… \\nw[name]","不过……我的飞天扫帚跑哪去了呢……\\nw[name]"]);
_tr_m.push(["あのほうきがないと この森から出られないのよね・・・。 \\nw[name]","I can't leave the forest without it... \\nw[name]","그 빗자루가 없으면 이 숲에서 나갈 수가 없는데・・・ \\nw[name]","沒有那把掃帚，我就無法離開這片森林了…… \\nw[name]","没有那把扫帚，我就无法离开这片森林了……\\nw[name]"]);
_tr_m.push(["この東でバザーが開かれているそうよ。 旅人たちが集まって店を開いているんだって。 \\nw[name]","I heard there's a bazaar in the east. Travelers from all over will be peddling their goods. \\nw[name]","여기 동쪽에 시장이 열렸다나 봐. 여행자들이 모여서 가게를 열었대. \\nw[name]","這裡的東邊似乎正在舉行跳蚤市場。聽說許多旅人聚集在那裡做生意。 \\nw[name]","这里的东边似乎正在举行跳蚤市场。听说许多旅人聚集在那里做生意。\\nw[name]"]);
_tr_m.push(["私も行ってみようと思うの！ \\nw[name]","I think I'm going to see it for myself! \\nw[name]","나도 가 볼 생각이야! \\nw[name]","我也想要去看看！ \\nw[name]","我也想要去看看！\\nw[name]"]);
_tr_m.push(["商品を受け取りに来たんですね！ \\nw[name]","So you're here for your item! \\nw[name]","상품을 받으러 오셨군요! \\nw[name]","妳是來拿商品的吧！ \\nw[name]","你是来拿商品的吧！\\nw[name]"]);
_tr_m.push(["荷物を整理して、また受け取りに来てくださいね。 お待ちしております！ \\nw[name]","Check your inventory and make room for it for next time. I'll be waiting for you! \\nw[name]","짐을 정리한 후에 다시 받으러 오세요. 기다리고 있을게요! \\nw[name]","等妳整理好包包之後再來拿吧。我會等妳的！ \\nw[name]","等你整理好包包之后再来拿吧。我会等你的！\\nw[name]"]);
_tr_m.push(["モーニングスター、買ってくださってありがとうございます！ \\nw[name]","Thank you for purchasing the Morning Star! \\nw[name]","모닝 스타를 구입해 주셔서 감사합니다! \\nw[name]","感謝您購買晨星鎚！ \\nw[name]","感谢您购买晨星锤！\\nw[name]"]);
_tr_m.push(["１年間、深夜の通販番組を見続けた甲斐がありました！ やっぱり努力って大切ですね！！ \\nw[name]","Studying infomercials late at night for an entire year is finally paying off! Hard work always does! \\nw[name]","1년 동안 심야 홈쇼핑을 계속 본 보람이 있었네요! 역시 노력이 중요하죠!! \\nw[name]","持續觀看深夜的購物頻道一整年，終於有回報了！努力果然很重要呢！！ \\nw[name]","持续观看深夜的购物频道一整年，终于有回报了！努力果然很重要呢！！\\nw[name]"]);
_tr_m.push(["らっしゃいらっしゃい！ モーニングスターはいかが！？ \\nw[name]","Welcome, my friend! How would you like to be a proud owner of a Morning Star?! \\nw[name]","어서 오세요! 모닝 스타는 어떠십니까!? \\nw[name]","歡迎光臨！要不要買晨星鎚啊！？ \\nw[name]","欢迎光临！要不要买晨星锤啊！？\\nw[name]"]);
_tr_m.push(["最近肩こりが酷い！腰が痛くて曲がってきた！ そんなときにはモーニングスター！ \\nw[name]","Are your shoulders stiff? Is your lower back in pain? If so, this Morning Star is the tool for you! \\nw[name]","요즘 어깨가 결리거나 허리가 아파서 굽지 않았나요? 그럴 때는 모닝 스타! \\nw[name]","最近如果感到肩膀僵硬！腰痛得直不起來！這種時候需要的就是晨星鎚！ \\nw[name]","最近如果感到肩膀僵硬！腰痛得直不起来！这种时候需要的就是晨星锤！\\nw[name]"]);
_tr_m.push(["モーニングスターでマッサージすれば、 ほーら！肩も腰も軽くなりすっきり爽快！ \\nw[name]","Use the Morning Star to pound your stress away! Your entire body will feel lighter instantly! \\nw[name]","모닝 스타로 마사지하면 짜잔~! 어깨도 허리도 가뿐하고 상쾌해집니다! \\nw[name]","只要拿晨星鎚來按摩……看吧！肩膀和腰部都變輕鬆了，非常輕盈舒爽！ \\nw[name]","只要拿晨星锤来按摩……看吧！肩膀和腰部都变轻松了，非常轻盈舒爽！\\nw[name]"]);
_tr_m.push(["―――農家のサチコさん（83）は 最近腰が痛くて農作業も一苦労・・・。 \\nw[name]","Mrs. Sciatica the Farmer is 83 years old and had a bad back. She found it hard to plow the fields... \\nw[name]","농부인 사치코 씨(83)는 요즘 들어 허리가 아파서 농사일도 힘들다고 합니다労・・・ \\nw[name]","────以務農為生的幸子女士（83）最近常腰痛，讓她下田工作時很辛苦…… \\nw[name]","────以务农为生的幸子女士（83）最近常腰痛，让她下田工作时很辛苦……\\nw[name]"]);
_tr_m.push(["サチコ「私も昔は元気に走り回りながら 　農作業していたんですがねえ。」 \\nw[name]","Mrs. Sciatica: \"When I was a young lass, I'd plow those fields like there was no tomorrow!\" \\nw[name]","사치코 「나도 한때는 훨훨 날아 다니면서 농사일을 했는데 말이죠.」 \\nw[name]","幸子「從前我也是精神飽滿地四處奔跑，整天都在田裡努力工作。」 \\nw[name]","幸子「从前我也是精神饱满地四处奔跑，整天都在田里努力工作。」\\nw[name]"]);
_tr_m.push(["サチコ「もうそろそろ引退ですかねえ・・・。」 \\nw[name]","Mrs. Sciatica: \"However, tomorrow is starting to feel like yesterday. Maybe it's time to retire...\" \\nw[name]","사치코 「슬슬 은퇴해야 할까 봐요・・・」 \\nw[name]","幸子「看來，我也差不多該退休了呢…」 \\nw[name]","幸子「看来，我也差不多该退休了呢…」\\nw[name]"]);
_tr_m.push(["腰腰大学医学部教授「歳を取ると、加齢性の変化により 　背骨が変形して曲がってくるのです。」 \\nw[name]","Professor at Lumbar University of Medicine: \"With age comes age-related problems. Your bones begin to change.\" \\nw[name]","허리허리대학 의대 교수 「나이를 먹으면 몸이 변화하면서 척추가 변형되어 구부러지죠.」 \\nw[name]","腰腰大學醫學系教授「年紀漸長後，高齡導致的退化會讓脊椎骨變形彎曲。」 \\nw[name]","腰腰大学医学系教授「年纪渐长后，高龄导致的退化会让脊椎骨变形弯曲。」\\nw[name]"]);
_tr_m.push(["腰の痛みに悩まされるサチコさん。 こういう方、けっこう多いハズ・・・。 \\nw[name]","There are many like Mrs. Sciatica who suffer the same symptoms. \\nw[name]","허리 통증이 고민인 사치코 씨. 이런 분들이 많으실 텐데요・・・ \\nw[name]","腰痛讓幸子女士十分痛苦。像她這種人應該不在少數…… \\nw[name]","腰痛让幸子女士十分痛苦。像她这种人应该不在少数……\\nw[name]"]);
_tr_m.push(["そこで登場！モーニングスター！ \\nw[name]","And that's where the Morning Star comes in! \\nw[name]","그런 분들을 위해! 모닝 스타! \\nw[name]","這時，就輪到晨星鎚登場了！ \\nw[name]","这时，就轮到晨星锤登场了！\\nw[name]"]);
_tr_m.push(["モーニングスターの先端の突起が、 ツボを刺激して骨を若返らせる！ \\nw[name]","The spikes of the Morning Star will dig in to your flesh and stimulate your muscles. You'll feel like a teenager again! \\nw[name]","모닝 스타 끝부분의 돌기가 혈자리를 자극해서 뼈에 청춘을 되찾아 줍니다! \\nw[name]","晨星鎚前端的突起部份，能夠刺激穴道，讓骨頭恢復青春活力！ \\nw[name]","晨星锤前端的突起部份，能够刺激穴道，让骨头恢复青春活力！ \\nw[name]"]);
_tr_m.push(["サチコさんにモーニングスターを使ってもらい、 その効果を検証することに！ \\nw[name]","Mrs. Sciatica will now use the Morning Star to see just how effective it is for herself! \\nw[name]","사치코 씨가 직접 모닝 스타를 사용하면서 그 효과를 검증해 보았습니다! \\nw[name]","讓幸子女士使用晨星鎚，驗證看看效果如何！ \\nw[name]","让幸子女士使用晨星锤，验证看看效果如何！\\nw[name]"]);
_tr_m.push(["サチコ「（ブスブス）あ～すっごい効く。 　簡単なので毎日続けられそう！」 \\nw[name]","Mrs. Sciatica: \"Mmm...? Ah, that hits the spot. It's so easy. I could use it everyday!\" \\nw[name]","사치코 「(퍽퍽)아이고~ 시원하다. 간편해서 매일 할 수 있겠네요!」 \\nw[name]","幸子「（刺入聲）哦～很有效呢。因為很簡單，感覺我可以每天都做！」 \\nw[name]","幸子「（刺入声）哦～很有效呢。因为很简单，感觉我可以每天都做！」\\nw[name]"]);
_tr_m.push(["そして一ヶ月後、サチコさんの自宅を訪ねると・・・。 \\nw[name]","I visited Mrs. Sciatica at her home one month later... \\nw[name]","그리고 한 달 후, 사치코 씨의 집을 방문해 보니・・・ \\nw[name]","過了一個月後，我們再次來到幸子女士的家…… \\nw[name]","过了一个月后，我们再次来到幸子女士的家……\\nw[name]"]);
_tr_m.push(["なんとそこには、背筋をピンと伸ばして 元気に農作業するサチコさんの姿が！！ \\nw[name]","And I found that she was standing up straight and plowing the fields like she was a teenager again! \\nw[name]","아니, 글쎄! 척추를 곧게 펴고 열심히 농사를 짓는 사치코 씨의 모습이!! \\nw[name]","在那裡，竟然看見了幸子女士挺直腰桿，精神百倍地在務農！！ \\nw[name]","在那里，竟然看见了幸子女士挺直腰杆，精神百倍地在务农！！ \\nw[name]"]);
_tr_m.push(["軽快なステップでにんじんを収穫するサチコさん。 その表情は満面の笑み。一ヶ月前とは大違いです！ \\nw[name]","She harvests carrots as if she was dancing on clouds! And look at that smile! In just one month, she's become a totally different person! \\nw[name]","경쾌한 스텝으로 당근을 수확하는 사치코 씨. 얼굴에 함박웃음을 띠고 있네요. 한 달 전의 모습과는 딴판입니다! \\nw[name]","幸子女士以輕快地腳步正在採收紅蘿蔔。她臉上浮現出滿滿的笑容。跟一個月前完全不一樣！ \\nw[name]","幸子女士正迈着轻快地脚步采收红萝卜。她脸上浮现出满满的笑容。跟一个月前完全不一样！\\nw[name]"]);
_tr_m.push(["サチコ「モーニングスターを使ってから腰も身体も絶好調！ 　毎日笑顔が止まりませんわ。」 \\nw[name]","Mrs. Sciatica: \"After using the Morning Star, my back...no, my entire body feels great! I can't stop smiling!\" \\nw[name]","사치코 「모닝 스타 덕분에 허리도 몸도 최고 상태예요! 매일 웃음이 그치지 않는답니다.」 \\nw[name]","幸子「自從我開始使用晨星鎚之後，腰和身體的狀況就變得非常好！每天都笑呵呵的。」 \\nw[name]","幸子「自从我开始使用晨星锤之后，腰和身体的状况就变得非常好！每天都笑呵呵的。」\\nw[name]"]);
_tr_m.push(["サチコ「私の農家人生、まだまだこれからですねえ！」 \\nw[name]","Mrs. Sciatica: \"I thought my farming days were over. But they've only just begun!\" \\nw[name]","사치코 「제 농부 인생은 지금부터가 시작이에요!」 \\nw[name]","幸子「我的農業人生才正要開始呢！」 \\nw[name]","幸子「我的农业人生才正要开始呢！」\\nw[name]"]);
_tr_m.push(["サチコさんの人生を変えたモーニングスター。 でも、お高いんでしょ？そんなことはありません！！ \\nw[name]","The Morning Star turned her life around. Now, you're probably thinking such an amazing device might be expensive. That's absolutely not the case! \\nw[name]","사치코 씨의 인생을 바꾼 모닝 스타. 네? 비싸지 않냐고요? 아니요, 그렇지 않습니다!! \\nw[name]","改變幸子女士人生的晨星鎚。您或許會想說這一定不便宜吧？不會的！！ \\nw[name]","改变幸子女士人生的晨星锤。您或许会想说这一定不便宜吧？不会的！！\\nw[name]"]);
_tr_m.push(["今回は特別に！モーニングスターを、 定価 800,000\\g のところ・・・。 \\nw[name]","As part of a special, limited-time offer, this Morning Star which normally retails for 800,000\\g... \\nw[name]","모닝 스타의 정가는 800,000\\g지만・・・ 이번에는 특별히! \\nw[name]","這次特別回饋大家！將原本定價800,000\\g 的晨星鎚…… \\nw[name]","这次特别回馈大家！将原本定价800,000\\g的晨星锤……\\nw[name]"]);
_tr_m.push(["なんと・・・９９％ＯＦＦでご提供！ 8,000\\g！ 8,000\\g でお買い求めいただけます！ \\nw[name]","Can be yours today for 99% off the regular price! Just 8,000\\g! That's right! Only 8,000\\g! \\nw[name]","무려・・・ 99% 할인가로 모시겠습니다! 8,000\\g! 8,000\\g에 구입 가능! \\nw[name]","竟然…以0.1折販售！只要8,000\\g！只要8,000\\g 就可以買到了！ \\nw[name]","竟然…以0.1折贩售！只要8,000\\g！只要8,000\\g就可以买到了！\\nw[name]"]);
_tr_m.push(["ええっ！？そんなに割引して大丈夫なんですか！？ \\nw[name]","What's that? You want to know how we can stay in business at these prices?! \\nw[name]","어머나!? 그렇게 싸게 팔아도 되는 건가요!? \\nw[name]","咦咦！？您問說打這麼多折沒問題嗎！？ \\nw[name]","咦咦！？您问打这么多折没问题吗！？\\nw[name]"]);
_tr_m.push(["もちろん今回限り！こんなチャンスはめったにありません！！ \\nw[name]","Well, it's a one-time limited offer! You'll never see this deal again! \\nw[name]","물론 기회는 이번뿐입니다! 날이면 날마다 오는 기회가 아니에요!! \\nw[name]","當然是僅限這次！機會難得，錯過就沒有了！！ \\nw[name]","当然是仅限这次！机会难得，错过就没有了！！\\nw[name]"]);
_tr_m.push(["さあ！そこのあなた！ \\nw[name]","Hey, you there! \\nw[name]","자! 거기 손님! \\nw[name]","來吧！那邊的客人！ \\nw[name]","来吧！那边的客人！\\nw[name]"]);
_tr_m.push(["モーニングスター、今だけ 8000\\g！買いますか！？ （所持金：\\v[3]\\g）\\!  \\nw[name]","The Morning Star can be yours for 8,000\\g! Well, what do you say?  (Money: \\v[3]\\g)\\!  \\nw[name]","모닝스타가 이번만 특별히 8000\\g! 구입하시겠습니까!?<br>(소지금: \\v[3]\\g)\\!  \\nw[name]","晨星鎚現在只要8,000\\g！要買嗎！？<br>（持有金錢：\\v[3]\\g）\\!  \\nw[name]","晨星锤现在只要8,000\\g！要买吗！？<br>（持有金钱：\\v[3]\\g）\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞是","＞是"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push(["あ、そうですか・・・。 \\nw[name]","Oh, I see... \\nw[name]","아, 네・・・ \\nw[name]","啊，這樣啊…… \\nw[name]","啊，这样啊……\\nw[name]"]);
_tr_m.push(["やったー！ありがとうございます！！ \\nw[name]","Great! Thank you so much! \\nw[name]","오예~! 감사합니다!! \\nw[name]","太好了！感謝購買！ \\nw[name]","太好了！感谢购买！\\nw[name]"]);
_tr_m.push(["荷物を整理して、また受け取りに来てくださいね。 お待ちしております！ \\nw[name]","Check your inventory and make room for it for next time. I'll be waiting for you! \\nw[name]","짐을 정리한 후에 다시 받으러 오세요. 기다리고 있을게요! \\nw[name]","等妳整理好包包之後再來拿吧。我會等妳的！ \\nw[name]","等你整理好包包之后再来拿吧。我会等你的！\\nw[name]"]);
_tr_m.push(["いやー、よかったー。売れてよかった！！ \\nw[name]","Phew! I'm just glad it finally sold! \\nw[name]","우와~ 다행이다~ 팔려서 다행이야!! \\nw[name]","哎呀～太好了～能賣掉真是太好了！！ \\nw[name]","哎呀～太好了～能卖掉真是太好了！！\\nw[name]"]);
_tr_m.push(["１年間、深夜の通販番組を見続けた甲斐がありました！ やっぱり努力って大切ですね！！ \\nw[name]","Watching infomercials late at night for an entire year is now paying off! I knew it would!\\nw[name]","1년 동안 심야 홈쇼핑을 계속 본 보람이 있었네요! 역시 노력이 중요하죠!! \\nw[name]","持續觀看深夜的購物頻道一整年，終於有回報了！努力果然很重要呢！！ \\nw[name]","持续观看深夜的购物频道一整年，终于有回报了！努力果然很重要呢！！\\nw[name]"]);
_tr_m.push(["坊や。この像にあまり近づくんじゃないよ。 \\nw[name]","Kid, you shouldn't get too close to this statue. \\nw[name]","이봐, 이 동상에 너무 가까이 오지 마. \\nw[name]","小子，妳不要太靠近那座雕像。 \\nw[name]","小子，你不要太靠近那座雕像。\\nw[name]"]);
_tr_m.push(["これは世界にたった一つしかない、 類まれなる戦士像である！ \\nw[name]","This is a unique warrior statue, the only one in the entire world! \\nw[name]","이건 세상에 하나뿐인 진귀한 전사의 동상이다! \\nw[name]","這是全世界只有一座，非常稀有的戰士像！ \\nw[name]","这是全世界只有一座，非常稀有的战士像！\\nw[name]"]);
_tr_m.push(["世界にたった一つだぞ！！こんなに貴重な品はない！！ \\nw[name]","It's a one-of-a-kind! There is nothing more precious! \\nw[name]","세상에 단 하나뿐이라고!! 이렇게 귀중한 걸 또 어디 가서 보겠어!! \\nw[name]","全世界只有一座喔！！沒有比這個更貴重的東西了！！ \\nw[name]","全世界只有一座喔！！没有比这个更贵重的东西了！！\\nw[name]"]);
_tr_m.push(["さあさあ！欲しい者は大金をもってこい！！ \\nw[name]","Come one, come all! If you want it, you'd better have deep pockets! \\nw[name]","자! 갖고 싶으면 거금을 가져와라!! \\nw[name]","來吧來吧！想要的人就拿大錢來買吧！！ \\nw[name]","来吧来吧！想要的人就拿大钱来买吧！！\\nw[name]"]);
_tr_m.push(["商品を受け取りに来たのだな。 \\nw[name]","I see you've come for your item! \\nw[name]","상품을 받으러 왔군. \\nw[name]","妳來拿商品了啊。 \\nw[name]","你来拿商品了啊。\\nw[name]"]);
_tr_m.push(["そうか。 荷物を整理して、また取りに来てくれ。 \\nw[name]","Very well. Check your inventory and make room for it for next time. \\nw[name]","그래. 짐을 정리한 후에 다시 받으러 와라. \\nw[name]","這樣啊。等妳整理好包包之後再來拿吧。 \\nw[name]","这样啊。等你整理好包包之后再来拿吧。\\nw[name]"]);
_tr_m.push(["にんじんを買ってくれてありがとう。 きっとおばあちゃんも喜ぶに違いない。 \\nw[name]","Thank you for buying our carrots. Grammy will definitely be happy. \\nw[name]","당근을 사 줘서 고마워. 분명 할머니도 기뻐할 거야. \\nw[name]","謝謝妳買了紅蘿蔔。奶奶一定也會很高興的。 \\nw[name]","谢谢你买了红萝卜。奶奶一定也会很高兴的。\\nw[name]"]);
_tr_m.push(["私はレイピア。祖母に頼まれて野菜を売りに来た。 \\nw[name]","I'm Rapier. My grammy asked me to help sell her produce. \\nw[name]","내 이름은 Rapier. 할머니의 부탁으로 채소를 팔러 왔어. \\nw[name]","我叫Rapier。奶奶請我來幫她賣菜。 \\nw[name]","我叫Rapier。奶奶请我来帮她卖菜。\\nw[name]"]);
_tr_m.push(["そこの君。採れたてピチピチのにんじん、１本 5000\\g だ。 買って行かないか？　（所持金：\\v[3]\\g）\\!  \\nw[name]","You there. How about a fresh and shiny carrot for 5,000\\g. Whaddya say? (Money: \\v[3]\\g)\\!  \\nw[name]","거기 너. 5000\\g에 갓 뽑은 신선한 당근 1개를 살 수 있어. 어때?　(소지금: \\v[3]\\g)\\!  \\nw[name]","客人，這是剛採收下來的新鮮紅蘿蔔，一根5000\\g。要不要買一根呢？（持有金錢：\\v[3]\\g）\\!  \\nw[name]","客人，这是刚采收下来的新鲜红萝卜，一根5000\\g。要不要买一根呢？（持有金钱：\\v[3]\\g）\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞好"]);
_tr_m.push([">いいえ",">No",">아니요","＞不要","＞不要"]);
_tr_m.push(["そうか。いや、無理にとは言わない。 しかし、さっぱり売れないな・・・。 \\nw[name]","Well, I won't press the issue. I just wonder why they're not selling... \\nw[name]","그래? 아니, 억지로 사라는 건 아니야. 근데 어떻게 하나도 안 팔리냐・・・ \\nw[name]","這樣啊。不，我不會勉強妳的。不過，生意實在是很差啊…… \\nw[name]","这样啊。不，我不会勉强你的。不过，生意实在是很差啊……\\nw[name]"]);
_tr_m.push(["おお・・・！買ってくれるのか！ まいどあり！ \\nw[name]","Ooh! You'll take one? Excellent! \\nw[name]","오오・・・! 사 줄 거야? 고마워! \\nw[name]","哦哦……！妳願意買嗎！謝謝惠顧！ \\nw[name]","哦哦……！你愿意买吗！谢谢惠顾！\\nw[name]"]);
_tr_m.push(["後日受け取りだな。 荷物を整理して、また取りに来てくれ。 \\nw[name]","So you'll get it some other time? Well, come when you've sorted through your inventory! \\nw[name]","나중에 받으러 와. 짐을 정리한 후에 다시 오면 돼. \\nw[name]","妳要之後再來拿啊？那等妳整理好包包之後再來拿吧。 \\nw[name]","你要之后再来拿啊？那等你整理好包包之后再来拿吧。\\nw[name]"]);
_tr_m.push(["お、それは当たりだ。一本当たりが混ざってると言っていたが、 それのことだろう。よかったな。 \\nw[name]","It looks like you hit the jackpot! I was told there was an exceptional one mixed in. Good for you! \\nw[name]","오, 당첨됐네. 당첨 1개가 섞여 있다고 했는데 아마 그거겠지. 좋겠군. \\nw[name]","哦，那根是極品紅蘿蔔。奶奶說有一根特別好的混在裡面，應該就是那根吧。真是太好了呢 \\nw[name]","哦，那根是极品红萝卜。奶奶说有一根特别好的混在里面，应该就是那根吧。真是太好了呢\\nw[name]"]);
_tr_m.push(["見た目もかっこいいし、きっとおいしいんだろう。 味わって食べてくれ。 \\nw[name]","It looks perfect and simply decadent. Be sure to enjoy every last bite. \\nw[name]","보기에도 멋있고 분명 맛있을 거야. 먹어 봐. \\nw[name]","外觀看起來也很完美，一定很好吃。妳就吃吃看吧。 \\nw[name]","外观看起来也很完美，一定很好吃。你就吃吃看吧。\\nw[name]"]);
_tr_m.push(["あんた、武器商人ウェポチチって知ってるか？ \\nw[name]","Have you heard of Dirk the weapons merchant? \\nw[name]","당신, 웨포파파라는 무기 상인을 알아? \\nw[name]","妳認識武器商人武父嗎？ \\nw[name]","你认识武器商人武父吗？\\nw[name]"]);
_tr_m.push(["なんでも、物凄い商売の腕を持つ武器商人だったらしいぞ。 武器の扱いの腕もすごかったとか。 \\nw[name]","They say he was a businessman like no other. He could even wield a multitude of weapons. \\nw[name]","장사 실력이 엄청난 무기 상인이래. 무기를 다루는 실력도 출중하다나? \\nw[name]","聽說他是個很會賣東西的武器商人呢。而且使用武器的技術也很強。 \\nw[name]","听说他是个很会卖东西的武器商人呢。而且使用武器的技术也很强。\\nw[name]"]);
_tr_m.push(["いいよな！オレもそんな商才があれば このバザーに出店して大儲けしてやったのになあ。 \\nw[name]","I'm so jealous! If I was that savvy, I'd set up shop at this bazaar and make a killing! \\nw[name]","멋지지? 나도 그런 능력이 있으면 이 시장에 나와서 큰돈을 벌었을 텐데. \\nw[name]","真羨慕！如果我也有那種商業才能的話，就能來這個跳蚤市場擺攤大賺一筆了。 \\nw[name]","真羡慕！如果我也有那种商业才能的话，就能来这个跳蚤市场摆摊大赚一笔了。\\nw[name]"]);
_tr_m.push(["色んな商品があって・・・ どれにしようか迷っちゃう～！ \\nw[name]","There are so many things to buy... How much did I bring again? \\nw[name]","상품이 다양해서・・・ 뭘 사야 할지 모르겠어~! \\nw[name]","這裡有很多商品……不知道該買哪個才好呢～！ \\nw[name]","这里有很多商品……不知道该买哪个才好呢～！\\nw[name]"]);
_tr_m.push(["そういえば全然関係ないけど・・・。 \\nw[name]","Speaking of which! This is totally unrelated, but... \\nw[name]","그러고 보니 전혀 상관없는 얘기인데・・・ \\nw[name]","對了，雖然是完全無關的事，不過…… \\nw[name]","对了，虽然是完全无关的事，不过……\\nw[name]"]);
_tr_m.push(["大きい武器は強いけど、 そのぶん視界が遮られて見え辛いらしいわよ。 \\nw[name]","Big weapons are powerful, but they also block your field of vision. \\nw[name]","커다란 무기는 강하지만 그만큼 시야를 가려서 앞이 잘 안 보인대. \\nw[name]","巨大的武器雖然強，但容易遮住視野，讓人難以看清楚前方。 \\nw[name]","巨大的武器虽然强，但容易遮住视野，让人难以看清楚前方。\\nw[name]"]);
_tr_m.push(["中々よさそうな像に見えるけどな。 どうも怪しいんだよな。 \\nw[name]","It looks like a mighty fine statue, but something about it feels off. \\nw[name]","제법 근사한 동상 같은데. 뭔가 수상하단 말이지. \\nw[name]","這座雕像看起來不錯。不過感覺很可疑呢。 \\nw[name]","这座雕像看起来不错。不过感觉很可疑呢。\\nw[name]"]);
_tr_m.push(["おい！像なんてどこにもねえじゃねぇか！！ \\nw[name]","Hey! Where the heck is the statue?! \\nw[name]","이봐! 동상 같은 건 아무 데도 없잖아!! \\nw[name]","喂！到處都找不到雕像啊！！ \\nw[name]","喂！到处都找不到雕像啊！！\\nw[name]"]);
_tr_m.push(["この詐欺師が！訴えてやる！！ \\nw[name]","You scam artist! I'll sue your pants off! \\nw[name]","이런 사기꾼! 고소할 거야!! \\nw[name]","你這個騙子！我要告你！！ \\nw[name]","你这个骗子！我要告你！！\\nw[name]"]);
_tr_m.push(["おい！何が世界にたった一つだ！！ テントに隠し持ってんじゃねぇか！！ \\nw[name]","Hey! What's this only one in the world business? You've got more in the tent! \\nw[name]","어이! 뭐가 세상에 하나뿐이라는 거야!! 텐트 안에 숨겨 놨네!! \\nw[name]","喂！什麼叫做全世界只有一個啊！！帳棚裡明明還有啊！ \\nw[name]","喂！什么叫做全世界只有一个啊！！帐棚里明明还有啊！\\nw[name]"]);
_tr_m.push(["この詐欺師が！訴えてやる！！ \\nw[name]","You scam artist! I'll sue your pants off! \\nw[name]","이런 사기꾼! 고소할 거야!! \\nw[name]","你這個騙子！我要告你！！ \\nw[name]","你这个骗子！我要告你！！\\nw[name]"]);
_tr_m.push(["ふむ・・・この像はすばらしい。 さすがは世界に一つしか存在しない像だ・・・。 \\nw[name]","Mm... What a wonderful statue! I can see why there's only one of them in the world. \\nw[name]","흐음・・・ 이 동상은 훌륭하군. 역시 세상에 하나뿐인 동상이야・・・ \\nw[name]","嗯……這雕像真不錯。真不愧是全世界只有一尊的雕像…… \\nw[name]","嗯……这雕像真不错。真不愧是全世界只有一尊的雕像……\\nw[name]"]);
_tr_m.push(["こんな貴重な品を仕入れるとは・・・ この商人、なかなかのやり手と見た。 \\nw[name]","A merchant with something so rare in stock is a merchant I approve of. \\nw[name]","이런 귀중한 물건을 들여오다니・・・ 이 상인은 수완이 상당한 게 틀림없어. \\nw[name]","竟然能進到這麼棒的貨……這個商人看起來相當厲害。 \\nw[name]","竟然能进到这么棒的货……这个商人看起来相当厉害。\\nw[name]"]);
_tr_m.push(["うむ・・・この商人、嘘をついていたのだな。 思っていたとおりだ。 \\nw[name]","Hmph! This merchant was lying about everything. I knew it. \\nw[name]","흐음・・・ 이 상인은 거짓말을 했군. 내 예상대로야. \\nw[name]","嗯……這個商人果然在說謊。跟我想的一樣。 \\nw[name]","嗯……这个商人果然在说谎。跟我想的一样。\\nw[name]"]);
_tr_m.push(["うむ・・・この商人、嘘をついていたのだな。 思っていたとおりだ。 \\nw[name]","Hmph! This merchant was lying about everything. I knew it. \\nw[name]","흐음・・・ 이 상인은 거짓말을 했군. 내 예상대로야. \\nw[name]","嗯……這個商人果然在說謊。跟我想的一樣。 \\nw[name]","嗯……这个商人果然在说谎。跟我想的一样。\\nw[name]"]);
_tr_m.push(["私はここの教会の牧師。そして 「アーメン教」の教徒でもある、ファルシオだ。アーメン。 \\nw[name]","I'm the priest of this church. This is Falchion, an acolyte of the Armen Sect. Armen. \\nw[name]","나는 이 교회의 목사이자 「아멘교」의 신도인 팔시오야. 아멘. \\nw[name]","我是這間教會的牧師，同時也是「阿門教」的信徒。我叫法爾吉歐。阿門。 \\nw[name]","我是这间教会的牧师，同时也是「阿门教」的信徒。我叫法尔吉欧。阿门。\\nw[name]"]);
_tr_m.push(["アーメン教の理念は、「とりあえずアーメンって言えば なんか神に救われるんじゃね？」である。アーメン。 \\nw[name]","The philosophy of the Armen Sect is that god will help us as long as we say \"armen.\" Armen to that! \\nw[name]","아멘교의 교리는 「일단 아멘이라고 하면 신이 구원해 주지 않을까?」야. 아멘. \\nw[name]","阿門教的理念是：「總之只要說了『阿門』，應該就能受到神的拯救吧？」阿門。 \\nw[name]","阿门教的理念是：「总之只要说了『阿门』，应该就能受到神的拯救吧？」阿门。\\nw[name]"]);
_tr_m.push(["そう。アーメンとは、人々を 正しく導いてくれるのだ。 \\nw[name]","Indeed. The power of armen will lead people down the righteous path. \\nw[name]","그래. 아멘은 사람들을 올바른 길로 이끌어 줄 거야. \\nw[name]","沒錯。「阿門」能夠引領人們走向正途。 \\nw[name]","没错。「阿门」能够引领人们走向正途。\\nw[name]"]);
_tr_m.push(["汝にも神のご加護のあらんことを。 アーメン。 \\nw[name]","May the lord watch over you as well. Armen. \\nw[name]","너에게도 신의 가호가 함께하길. 아멘. \\nw[name]","願神保佑你。阿門。 \\nw[name]","愿神保佑你。阿门。\\nw[name]"]);
_tr_m.push(["ここは海沿いの教会でございます。 \\nw[name]","This is the Seaside Sect. \\nw[name]","여기는 해안의 교회입니다. \\nw[name]","這裡是海邊的教會。 \\nw[name]","这里是海边的教会。\\nw[name]"]);
_tr_m.push(["あなたは、この大陸に住む魔王の話を知っていますか？\\!  \\nw[name]","Do you know about the Demon Lord that lives in these lands?\\!  \\nw[name]","당신은 이 대륙에 사는 마왕의 이야기를 알고 있습니까?\\!  \\nw[name]","妳知道住在這片大陸上的魔王的故事嗎？\\!  \\nw[name]","你知道住在这片大陆上的魔王的故事吗？\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞知道","＞知道"]);
_tr_m.push(["そうですか。 旅のお方、どうかご無理をなさらずに。 \\nw[name]","I see. Be safe during your travels, pilgrim. \\nw[name]","그렇군요. 여행자여, 무리하지 마시길. \\nw[name]","這樣啊。旅行者，請不要太勉強自己了。 \\nw[name]","这样啊。旅行者，请不要太勉强自己了。\\nw[name]"]);
_tr_m.push([">いいえ",">No",">아니요","＞不知道","＞不知道"]);
_tr_m.push(["魔王はかつてこの大陸に魔物を呼び出した 邪悪の根源・・・。 \\nw[name]","The Demon Lord is the source of all evil. He's the one responsible for the monsters you see. \\nw[name]","마왕은 과거에 이 대륙에 마물을 불러들인 악의 근원・・・ \\nw[name]","魔王是從前把魔物召喚到這個大陸的邪惡根源…… \\nw[name]","魔王是从前把魔物召唤到这个大陆的邪恶根源……\\nw[name]"]);
_tr_m.push(["一度は勇敢な勇者によって倒れ、封印されました。 \\nw[name]","Once in times past, a brave knight defeated him and locked him away. \\nw[name]","한때는 용감한 용사의 손에 쓰러져 봉인당했죠. \\nw[name]","勇敢的勇者曾打倒魔王，並將其封印。 \\nw[name]","勇敢的勇者曾打倒魔王，并将其封印。\\nw[name]"]);
_tr_m.push(["しかし、長い封印から目覚めた魔王は、 この大陸の城を滅ぼしてしまったのです。 \\nw[name]","However, the Demon Lord escaped from those shackles after many years and destroyed this country's castle. \\nw[name]","하지만 오랜 봉인에서 깨어난 마왕은 이 대륙의 성을 멸망시키고 말았습니다. \\nw[name]","然而，魔王從漫長的封印中覺醒後，就毀滅了這片大陸上的城堡。 \\nw[name]","然而，魔王从漫长的封印中觉醒后，就毁灭了这片大陆上的城堡。\\nw[name]"]);
_tr_m.push(["今は魔王城でおとなしくしていますが、 いつまた暴れだしてもおかしくありません。 \\nw[name]","He currently lies dormant in the Demonic Castle, but who knows when he'll decide to go on a rampage. \\nw[name]","지금은 마왕성에서 얌전히 지내지만 언제 다시 난동을 피워도 이상하지 않은 일이죠. \\nw[name]","現在魔王雖然正安分地待在魔王城裡，但就算某一天他會開始作亂也不奇怪。 \\nw[name]","现在，虽然魔王正安分地待在魔王城里，但就算某一天他会开始作乱也不奇怪。\\nw[name]"]);
_tr_m.push(["世界は危機的状況にあるのです。 ああ、神よ、どうかお救いください・・・！ \\nw[name]","The world is in peril. Oh lord, please save us!\\nw[name]","이 세계는 위태롭습니다. 아아, 신이시여. 구원해 주소서・・・! 아멘! \\nw[name]","世界正處於危機之中。啊，神啊，請拯救我們吧……！阿門！ \\nw[name]","世界正处于危机之中。啊，神啊，请拯救我们吧……！阿门！\\nw[name]"]);
_tr_m.push(["ああ、アーメン！ アーメン教万歳！アーメン！ \\nw[name]","A-Armen! Praise be to those who say Armen! \\nw[name]","아아, 아멘! 아멘교 만세! \\nw[name]","啊，阿門！阿門教萬歲！ \\nw[name]","啊，阿门！阿门教万岁！\\nw[name]"]);
_tr_m.push(["来年の入試で合格できるよう、お祈りに来たぞ・・・！ \\nw[name]","I've come to pray for my school entrance exams! \\nw[name]","내년 입시에 합격시켜 달라고 기도하러 왔어・・・! \\nw[name]","我來祈禱了，希望能順利通過明年的入學考試……！ \\nw[name]","我来祈祷了，希望能顺利通过明年的入学考试……！\\nw[name]"]);
_tr_m.push(["どうか補欠でもいいので合格できますように！ \\nw[name]","Please let me get into an academy! It can be one of my safety schools for all I care! \\nw[name]","대기자여도 좋으니 꼭 합격하게 해 주세요! \\nw[name]","就算只是備取上榜也好，希望能順利通過！\\nw[name]","就算只是候补上榜也好，希望能顺利通过！\\nw[name]"]);
_tr_m.push(["ここは今は廃墟だが、かつては大きな城だった。 なんでも、復活した魔王に攻め滅ぼされたって話だ。 \\nw[name]","These ruins used to be a grand castle. They say the revived Demon Lord attacked and obliterated it. \\nw[name]","여기가 지금은 폐허지만 한때는 커다란 성이었어. 듣자 하니 부활한 마왕에게 멸망당했다더라고. \\nw[name]","這裡現在雖然是廢墟，但從前是一座很大的城堡。據說是遭到復活後的魔王攻擊，才變成這副模樣。 \\nw[name]","这里现在虽然是废墟，但从前是一座很大的城堡。据说是遭到复活后的魔王攻击，才变成这副模样。\\nw[name]"]);
_tr_m.push(["その無念な結末ゆえ、亡霊の姿が目撃されているとか・・・。 ああ、恐ろしい。 \\nw[name]","They also say that victims of that mortifying defeat can be seen as ghosts. Scary, no?! \\nw[name]","그런 슬픈 결말 때문인지 망령이 종종 목격된다나・・・ 아이고, 무서워. \\nw[name]","由於有過這種慘痛的事件，才會有人目睹到幽靈……哎唷，真是恐怖。 \\nw[name]","由于有过这种惨痛的事件，才会有人目睹到幽灵……哎唷，真是恐怖。\\nw[name]"]);
_tr_m.push(["あれ！？ 海は！？！？！？！？！？ \\nw[name]","Whoa! What happened to the ocean?! \\nw[name]","어!? 바다는!?!?!?!?!?!? \\nw[name]","咦！？海呢！？！？！？！？！？ \\nw[name]","咦！？海呢！？！？！？！？！？\\nw[name]"]);
_tr_m.push(["ワシはここで何年も釣りをしておるが、 一向に釣れる気配がないわい・・・。 \\nw[name]","I've been fishing here for years, but I'm starting to get the feeling I won't catch anything here. \\nw[name]","나는 여기에서 몇 년째 낚시 중인데 도무지 잡힐 것 같지 않아・・・ \\nw[name]","我在這裡釣魚好幾年了，但總是沒有釣到魚過…… \\nw[name]","我在这里钓鱼好几年了，但总是没有收获……\\nw[name]"]);
_tr_m.push(["だが、いいんじゃ。この果てしなく大きな海を見つめ、 静かに想いを馳せる・・・それが幸せなんじゃ。 \\nw[name]","But that's all right. My imagination runs wild just looking into the vast expanse of the sea...and that's enough for me. \\nw[name]","그래도 상관없어. 이 끝없이 드넓은 바다를 바라보며 조용히 생각에 잠기는 것・・・ 그게 행복이지. \\nw[name]","不過，這樣也好。看著這片無垠無界的大海，靜靜地讓思考馳騁……這樣就很幸福了。 \\nw[name]","不过，这样也好。看着这片无垠无界的大海，静静地让思考驰骋……这样就很幸福了。\\nw[name]"]);
_tr_m.push(["見てみぃ。こうしてこの位置から海を眺めていると・・・。 \\nw[name]","Take a look. If you stare into the horizon from this spot... \\nw[name]","보거라. 이렇게 여기에서 바다를 바라보면・・・ \\nw[name]","妳看，從這個位置遠眺海洋的話…… \\nw[name]","你看，从这个位置远眺海洋的话……\\nw[name]"]);
_tr_m.push(["まるで、この海すら・・・ 自分の物のような、そんな気がしてこんか？ \\nw[name]","It makes it feel like the sea belongs to you. Don't you think so too? \\nw[name]","마치 이 바다도・・・ 내 것처럼 느껴지지 않아? \\nw[name]","這整片海洋彷彿就像……是自己的東西一樣。妳不這麼覺得嗎？ \\nw[name]","这整片海洋彷佛就像……是自己的东西一样。你不这么觉得吗？ \\nw[name]"]);
_tr_m.push(["なんつってな。はっはっは。 \\nw[name]","Pardon my rambling. Hahaha! \\nw[name]","농담이야. 하하하. \\nw[name]","開玩笑的啦。哈哈哈。 \\nw[name]","开玩笑的啦。哈哈哈。\\nw[name]"]);
_tr_m.push(["旅人さん・・・。 \\nw[name]","Traveler... \\nw[name]","여행자님・・・ \\nw[name]","旅人小姐…… \\nw[name]","旅人小姐……\\nw[name]"]);
_tr_m.push(["その・・・おじいのこと、ありがとうね。 \\nw[name]","I, uh, wanted to thank you. For Gramps and all. \\nw[name]","저기・・・ 할아버지 일은 고마워. \\nw[name]","那個……爺爺的事，謝謝妳的幫忙。 \\nw[name]","那个……爷爷的事，谢谢你的帮忙。\\nw[name]"]);
_tr_m.push(["私とおじいは、恥ずかしがり屋な所がそっくりなの。 なかなか本音を言えなくて。 \\nw[name]","We're both kinda reserved I guess. Don't like to wear our hearts on our sleeves, y'know? \\nw[name]","나랑 할아버지는 쑥스러워하는 성격이 똑같아. 서로 진심을 잘 말하지 않거든. \\nw[name]","我和爺爺都很容易害羞，實在很難說出真心話。 \\nw[name]","我和爷爷都很容易害羞，实在很难说出真心话。\\nw[name]"]);
_tr_m.push(["出ていく時だってね。クソジジイなんて言うつもりなかったのに。 何言ってんだろ私、ってなっちゃった。 \\nw[name]","When I ran away, I didn't mean to call him a stinkin' geezer. It was the only way to hold back the tears. \\nw[name]","집을 나올 때도 말이지. 꼰대라고 할 생각까지는 없었는데. 내가 무슨 소리를 하는지 나도 잘 몰랐어. \\nw[name]","我離家出走時，原本並沒打算要罵他死老頭的。結果……我怎麼會說出那種話呢。 \\nw[name]","我离家出走时，原本并没打算要骂他死老头的。结果……我怎么会说出那种话呢。\\nw[name]"]);
_tr_m.push(["でも、あなたが会わせてくれたおかげで。 やっと感謝の気持ちを言えたわ。 \\nw[name]","But I was able to tell him how grateful I was thanks to you reuniting us. \\nw[name]","하지만 당신이 만나게 해 준 덕분에 내가 감사하는 마음을 겨우 전할 수 있었어. \\nw[name]","不過，多虧妳讓我見到他，我才終於能說出對他的感謝之意。 \\nw[name]","不过，多亏你让我见到他，我才终于能说出对他的感谢之意。\\nw[name]"]);
_tr_m.push(["おじいのために・・・ はやく仕事見つけなくっちゃね！ \\nw[name]","I have to find a job and soon! For Gramps's sake. \\nw[name]","할아버지를 위해・・・ 빨리 일자리를 찾아야 해! \\nw[name]","為了爺爺……我要快點找到工作才行！ \\nw[name]","为了爷爷……我要快点找到工作才行！\\nw[name]"]);
_tr_m.push(["今更、おじいの所になんて 戻れるわけないじゃない・・・。 \\nw[name]","There's no way I could see him again the way things are. \\nw[name]","이제 와서 할아버지 집으로 돌아갈 수는 없어・・・ \\nw[name]","事到如今，我也不能再回去爺爺那邊了…… \\nw[name]","事到如今，我也不能再回去爷爷那边了……\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["こんにちは、旅人さん。 \\nw[name]","Hello, traveler. \\nw[name]","여행자님, 안녕? \\nw[name]","妳好啊，旅人。 \\nw[name]","你好啊，旅人。\\nw[name]"]);
_tr_m.push(["あーあ。はやく何か仕事見つからないかなあ。 \\nw[name]","Sigh... Why can't finding a job be easier? \\nw[name]","아아~ 빨리 일자리를 구해야 하는데. \\nw[name]","唉～真希望能快點找到工作啊。 \\nw[name]","唉～真希望能快点找到工作啊。\\nw[name]"]);
_tr_m.push(["え？ダーガという人を探しているって？ \\nw[name]","What? You're looking for someone named Dagger? \\nw[name]","뭐? Dagger라는 사람을 찾는다고? \\nw[name]","咦？妳在找一個叫做Dagger的人？ \\nw[name]","咦？你在找一个叫做Dagger的人？\\nw[name]"]);
_tr_m.push(["わ、私だけど・・・。 \\nw[name]","Th-That's me... \\nw[name]","내, 내가 Dagger인데・・・ \\nw[name]","我、我就是Dagger…… \\nw[name]","我，我就是Dagger……\\nw[name]"]);
_tr_m.push(["え？ きこりのウォンマーとかいう人の家に戻れだって！？ \\nw[name]","Huh? You want me to go back to Wonmar's house?! \\nw[name]","뭐? Wonmar라는 나무꾼의 집으로 돌아가라고!? \\nw[name]","咦？妳叫我回到樵夫Wonmar的家！？ \\nw[name]","咦？你叫我回到樵夫Wonmar的家！？\\nw[name]"]);
_tr_m.push(["嫌よ！！ 一度家出しておいて、どんな顔して戻ればいいのよ！！ \\nw[name]","Forget it! I ran away, how could I show my face there?! \\nw[name]","싫어!! 이미 가출했는데 어떻게 돌아가라는 거야!! \\nw[name]","不要！！我都已經離家出走了，哪有臉再回去啊！！ \\nw[name]","不要！！我都已经离家出走了，哪有脸再回去啊！！\\nw[name]"]);
_tr_m.push(["しかも私、出ていくとき おじいに向かって「クソジジイ」って言ったのよ！ \\nw[name]","I even called him a stinkin' geezer when I ran out the door! \\nw[name]","게다가 나는 집을 나오면서 할아버지한테 「꼰대」라고 했단 말이야! \\nw[name]","而且我離家出走時，還罵爺爺說「你這個死老頭」耶！ \\nw[name]","而且我离家出走时，还骂爷爷说「你这个死老头」耶！\\nw[name]"]);
_tr_m.push(["きっと怒ってるに違いないわ！ \\nw[name]","I bet he's still angry about that! \\nw[name]","분명 화가 머리끝까지 났을 거야! \\nw[name]","他一定很生氣！ \\nw[name]","他一定很生气！\\nw[name]"]);
_tr_m.push(["今更戻れるわけないじゃない・・・。 \\nw[name]","There's no way I could go back the way things are... \\nw[name]","이제 와서 어떻게 돌아가・・・ \\nw[name]","事到如今，我怎麼能再回去呢…… \\nw[name]","事到如今，我怎么能再回去呢……\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["商品をお受け取りに来られたのですね。 \\nw[name]","I see you've come for the item. \\nw[name]","상품을 받으러 오셨군요. \\nw[name]","妳來拿商品了啊。 \\nw[name]","你来拿商品了啊。\\nw[name]"]);
_tr_m.push(["荷物を整理してまた取りにいらしてください。 \\nw[name]","Make some space in your inventory and come for it later. \\nw[name]","짐을 정리한 후에 다시 받으러 오세요. \\nw[name]","那等妳整理好包包之後再來拿吧。 \\nw[name]","那等你整理好包包之后再来拿吧。\\nw[name]"]);
_tr_m.push(["ありがとうございました！ またのご来店お待ちしております！ \\nw[name]","Thank you very much! Please come again! \\nw[name]","감사합니다! 다음에 또 오세요! \\nw[name]","謝謝惠顧！希望您下次再來！ \\nw[name]","谢谢惠顾！希望您下次再来！\\nw[name]"]);
_tr_m.push(["いらっしゃいませ！ コンビニエンスストア『魔王城前』です！\\!  \\nw[name]","Welcome to the Demonic Castle branch of our convenience store!\\!  \\nw[name]","어서 오세요! 편의점 『마왕성 앞』입니다!\\!  \\nw[name]","歡迎光臨！這裡是便利商店『魔王城前』！\\!  \\nw[name]","欢迎光临！这里是便利商店『魔王城前』！\\! \\nw[name]"]);
_tr_m.push([">商品を見る",">View Items",">상품 확인","＞查看商品","＞查看商品"]);
_tr_m.push([">いいえ",">No",">아니요","＞否","＞否"]);
_tr_m.push(["ありがとうございました！ またのご来店お待ちしております！ \\nw[name]","Thank you very much! Please come again! \\nw[name]","감사합니다! 다음에 또 오세요! \\nw[name]","謝謝惠顧！希望您下次再來！ \\nw[name]","谢谢惠顾！希望您下次再来！\\nw[name]"]);
_tr_m.push(["どの商品にしましょうか？ （所持金：\\v[3]\\g）\\!  \\nw[name]","What would you like? (Money: \\v[3]\\g)\\!  \\nw[name]","어느 상품을 선택하시겠습니까? (소지금: \\v[3]\\g)\\!  \\nw[name]","請問要買哪一項商品呢？（持有金錢：\\v[3]\\g）\\!  \\nw[name]","请问要买哪一样商品呢？（持有金钱：\\v[3]\\g）\\! \\nw[name]"]);
_tr_m.push([">\\i[261]薬草✕3 　    200\\g",">\\i[261]Herbs x3       200\\g",">\\i[261]약초✕3 　    200\\g",">\\i[261]藥草✕3      200\\g",">\\i[261]药草✕3      200\\g"]);
_tr_m.push(["\\i[261]薬草を３つですね！ 200\\g になりますがよろしいですか？\\!  \\nw[name]","Oh, excellent choice! You want 3 \\i[261]herbs for 200\\g?\\!  \\nw[name]","\\i[261]약초 3개 말씀이시죠! 200\\g인데 괜찮으시겠습니까?\\!  \\nw[name]","\\i[261]3個藥草是吧！總共是200\\g，可以嗎？\\!  \\nw[name]","\\i[261]3个药草是吧！总共是200\\g，可以吗？\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞是"]);
_tr_m.push(["ありがとうございます！ こちら、温めますか？\\!  \\nw[name]","Thank you very much! Would you like to warm up your purchase?\\!  \\nw[name]","감사합니다! 데워 드릴까요?\\!  \\nw[name]","謝謝惠顧！請問要幫您加熱嗎？\\!  \\nw[name]","谢谢惠顾！请问要帮您加热吗？\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞好"]);
_tr_m.push(["では少々お待ち下さい。 \\nw[name]","One moment please. \\nw[name]","잠시만 기다려 주세요. \\nw[name]","那麼，請您稍等一下。 \\nw[name]","那么，请您稍等一下。\\nw[name]"]);
_tr_m.push(["ホカホカに温まりました！ \\nw[name]","There you go! Nice and toasty! \\nw[name]","따끈따끈 데워졌습니다! \\nw[name]","幫您加熱了，熱呼呼的！ \\nw[name]","帮您加热了，热呼呼的！\\nw[name]"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞不用"]);
_tr_m.push(["ありがとうございました！ またのご来店お待ちしております！ \\nw[name]","Thank you very much! Hope to see you again soon!\\nw[name]","감사합니다! 다음에 또 오세요! \\nw[name]","謝謝惠顧！希望您下次再來！ \\nw[name]","谢谢惠顾！希望您下次再来！\\nw[name]"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push([">\\i[1011]巨人の大剣　\\v[4]",">\\i[1011]Giant's Sword  \\v[4]",">\\i[1011]거인의 대검　 \\v[4]","＞\\i[1011]巨人的大劍   \\v[4]","＞\\i[1011]巨人的大劍   \\v[4]"]);
_tr_m.push(["\\i[1011]巨人の大剣を１つですね！ 10000\\g になりますがよろしいですか？\\!  \\nw[name]","Excellent choice! Purchase a \\i[1011]Giant's Sword for 10,000\\g?\\!  \\nw[name]","\\i[1011]거인의 대검 1개 말씀이시죠! 10000\\g인데 괜찮으시겠습니까?\\!  \\nw[name]","\\i[1011]1個巨人的大劍是吧！總共是10000\\g，可以嗎？\\!  \\nw[name]","\\i[1011]1把巨人的大剑是吧！总共是10000\\g，可以吗？\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞是","＞是"]);
_tr_m.push(["ありがとうございます！ こちら、温めますか？\\!  \\nw[name]","Thank you very much! Would you like to warm up your purchase?\\!  \\nw[name]","감사합니다! 데워 드릴까요?\\!  \\nw[name]","謝謝惠顧！請問要幫您加熱嗎？\\!  \\nw[name]","谢谢惠顾！请问要帮您加热吗？\\! \\nw[name]"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞好"]);
_tr_m.push(["では少々お待ち下さい。 \\nw[name]","One moment please. \\nw[name]","잠시만 기다려 주세요. \\nw[name]","那麼，請您稍等一下。 \\nw[name]","那么，请您稍等一下。\\nw[name]"]);
_tr_m.push(["ホカホカに温まりました！ \\nw[name]","There you go! Nice and toasty! \\nw[name]","따끈따끈 데워졌습니다! \\nw[name]","幫您加熱了，熱呼呼的！ \\nw[name]","帮您加热了，热呼呼的！\\nw[name]"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞不用"]);
_tr_m.push(["後日受け取りですね。 荷物を整理して取りにいらしてください。 \\nw[name]","You'll pick it up later? Just be sure you've got room in your inventory when you come back for it! \\nw[name]","나중에 받으시겠다고요? 짐을 정리한 후에 다시 받으러 와 주세요. \\nw[name]","要之後再來拿是吧。那等您整理好包包後再來拿吧。 \\nw[name]","要之后再来拿是吧。那等您整理好包包后再来拿吧。\\nw[name]"]);
_tr_m.push(["ありがとうございました！ またのご来店お待ちしております！ \\nw[name]","Thank you very much! Hope to see you again soon!\\nw[name]","감사합니다! 다음에 또 오세요! \\nw[name]","謝謝惠顧！希望您下次再來！ \\nw[name]","谢谢惠顾！希望您下次再来！\\nw[name]"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push([">スマイル           0\\g",">Smile             0\\g",">스마일           0\\g","＞微笑            0\\g","＞微笑            0\\g"]);
_tr_m.push(["その商品はマニュアルに載っていないため ご提供することができません！ \\nw[name]","I'm sorry, that item's not in our employee manual so I can't sell it to you. \\nw[name]","그 상품은 매뉴얼에 적혀 있지 않아서 드릴 수가 없습니다! \\nw[name]","這項商品並沒有在教學手冊中，因此無法提供！ \\nw[name]","这项商品并没有在教学手册中，因此无法提供！\\nw[name]"]);
_tr_m.push([">やめる",">Cancel",">그만둔다","＞算了","＞算了"]);
_tr_m.push(["ありがとうございました！ またのご来店お待ちしております！ \\nw[name]","Thank you very much! Hope to see you again soon!\\nw[name]","감사합니다! 다음에 또 오세요! \\nw[name]","謝謝惠顧！希望您下次再來！ \\nw[name]","谢谢惠顾！希望您下次再来！\\nw[name]"]);
_tr_m.push(["コンビニは便利だなあ！ \\nw[name]","Convenience stores sure are convenient! \\nw[name]","편의점은 참 편리해! \\nw[name]","便利商店還真是方便啊！ \\nw[name]","便利商店还真是方便啊！\\nw[name]"]);
_tr_m.push(["・・・。 \\nw[name]","... \\nw[name]","・・・. \\nw[name]","…… \\nw[name]","……\\nw[name]"]);
_tr_m.push(["何見てんだゴルァ？ \\nw[name]","Whatchu lookin' at? \\nw[name]","뭘 봐, 임마? \\nw[name]","妳看什麼看啊？ \\nw[name]","你看什么看啊？\\nw[name]"]);
_tr_m.push(["俺ァヤンキーだゴラァ！ ヤンキーつったらコンビニ前だろうがァ！！ \\nw[name]","I'm a hoodlum! And hoodlums hang in front of convenience stores! \\nw[name]","나는 깡패다, 이 자식아! 편의점 앞엔 원래 깡패가 있는 거야!! \\nw[name]","我是不良少年！不良少年就是要坐在便利商店前！！ \\nw[name]","我是不良少年！不良少年就是要坐在便利商店前！！\\nw[name]"]);
_tr_m.push(["俺はここからどかねぇ！ なぜならヤンキーであるから！！ \\nw[name]","I ain't movin' an inch! Why not? Cuz I'm a hoodlum! \\nw[name]","난 여기서 안 비킬 거다! 왜냐하면 깡패니까!! \\nw[name]","我不會讓開的！因為我是不良少年！！ \\nw[name]","我不会让开的！因为我是不良少年！！\\nw[name]"]);
_tr_m.push(["冒険者か？この辺りの魔物は強い、気をつけろ。 この大陸は北に向かうほど強い魔物が出現する。 \\nw[name]","You an adventurer? The monsters here are tough. The further north you go, the stronger they get. \\nw[name]","모험가인가? 이 부근의 마물은 강하니까 조심해라. 이 대륙은 북쪽으로 갈수록 강한 마물이 나타나지. \\nw[name]","妳是冒險者嗎？這附近的魔物很強，妳要小心點。這片大陸越往北走，就會有越強的魔物出現。 \\nw[name]","你是冒险者吗？这附近的魔物很强，你要小心点。这片大陆越往北走，就会有越强的魔物出现。\\nw[name]"]);
_tr_m.push(["そして最北には魔王城がそびえ立つ。 魔王は今は大人しいが、いつ暴れだすのやら・・・。 \\nw[name]","At the northern tip stands the Demonic Castle. The Demon Lord hasn't stirred up any trouble, but who knows when he'll change his mind. \\nw[name]","그리고 가장 북쪽에는 마왕성이 우뚝 서 있어. 지금이야 마왕이 잠잠하지만 언제 날뛰기 시작할지・・・ \\nw[name]","而最北邊聳立著魔王城。雖然魔王現在很安分，但不知道他何時會開始作亂啊…… \\nw[name]","而最北边耸立着魔王城。虽然魔王现在很安分，但不知道他何时会开始作乱啊……\\nw[name]"]);
_tr_m.push(["４つの試練をクリアしたことで、 \\n[1]の力が覚醒した！","By completing the four challenges, \\n[1]'s strength has awakened!","4개의 시련을 클리어하여 \\n[1]의 힘이 각성했다!","\\n[1]通過了四個試煉之後，力量就此覺醒！","\\n[1]通过了四个试炼之后，力量就此觉醒！"]);
_tr_m.push(["なんと、フィールド内の武器を すべて拾えるようになった！","All weapons in the field can now be picked up!","팔드 안의 무기를 전부 주울 수 있게 되었다!","竟然可以拾取地圖上所有的武器了！","竟然可以拾取地图上所有的武器了！"]);
_tr_m.push(["さらに、洞窟のボスと再戦できるようになった！","You can also face all of the cave bosses again!","그리고 동굴의 보스와 다시 싸울 수 있게 되었다!","此外，還能再次跟洞窟的頭目戰鬥！","此外，还能再次跟洞窟的头目战斗！"]);
_tr_m.push(["準備を整えて、最終試練の行われる \\c[14]ヘイワ村の北の洞窟\\c[0]へ向かおう！","Make any necessary preparations and head for the final challenge in the cave north of \\c[14]Peaceville\\c[0]!","준비를 갖추고 마지막 시련의 장소인 \\c[14]평온한 마을 북쪽 동굴\\c[0]로 가자!","準備好之後，就到 \\c[14]和平村北方的洞窟\\c[0]，接受最終試煉吧！","准备好之后，就到\\c[14]和平村北方的洞窟\\c[0]，接受最终试炼吧！"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","寶箱裡似乎空無一物。"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","寶箱裡似乎空無一物。"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","寶箱裡似乎空無一物。"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","寶箱裡似乎空無一物。"]);
_tr_m.push(["強そうな戦士像が置いてある・・・。","It's a statue of a mighty warrior.","강해 보이는 전사의 동상이 있다・・・","放著一尊看起來很強的戰士雕像……","放着一尊看起来很强的战士雕像……"]);
_tr_m.push(["宝箱はカラのようだ。","The chest is empty.","보물상자는 텅 비어 있다.","寶箱裡似乎空無一物。","寶箱裡似乎空無一物。"]);
_tr_m.push(["強そうな戦士像が置いてある・・・。","It's a statue of a mighty warrior.","강해 보이는 전사의 동상이 있다・・・","放著一尊看起來很強的戰士雕像……","放著一尊看起來很強的戰士雕像……"]);
_tr_m.push(["強そうな戦士像が置いてある・・・。","It's a statue of a mighty warrior.","강해 보이는 전사의 동상이 있다・・・","放著一尊看起來很強的戰士雕像……","放著一尊看起來很強的戰士雕像……"]);
_tr_m.push(["「魔女の家」","\"Witch's House\"","「마녀의 집」","「魔女家」","「魔女家」"]);
_tr_m.push(["俺はトム！体も腹の中もまっ黒なクマだ！ \\nw[name]","I'm Tom! I'm a black bear that's as dark as night! \\nw[name]","내 이름은 Tom! 몸도 마음도 새까만 곰이다! \\nw[name]","我叫Tom！是一頭熊！身體是黑的，內心也是黑的！ \\nw[name]","我叫Tom！是一头熊！身体是黑的，内心也是黑的！\\nw[name]"]);
_tr_m.push(["お前、武器商人だな！ 突然だが、俺の好みの武器を見せてくれたらいいもんやるぜ。 \\nw[name]","I see you're a weapons merchant! That's right, I want you to find me a weapon I favor! \\nw[name]","넌 무기 상인이로군? 갑작스럽겠지만 내가 좋아하는 무기를 보여 주면 좋은 걸 주지. \\nw[name]","妳是武器商人吧！恕我冒昧，如果妳帶我喜歡的武器來給我看，我就會給妳好東西。 \\nw[name]","你是武器商人吧！恕我冒昧，如果你带我喜欢的武器来给我看，我就会给你好东西。\\nw[name]"]);
_tr_m.push(["俺の見たい武器はズバリ！「黒い武器だ」！！ \\nw[name]","The weapon I want to see is a \"black weapon\"! \\nw[name]","내가 보고 싶은 무기는 바로! 「검은 무기」다!! \\nw[name]","我想看的武器，簡單來說就是「黑色的武器」！！ \\nw[name]","我想看的武器，简单来说就是「黑色的武器」！！\\nw[name]"]);
_tr_m.push(["黒けりゃなんでもいい！ 黒い武器を見つけたら、装備して話しかけてくれ！ \\nw[name]","It can be anything, so long as it's black! Equip one and come talk to me again! \\nw[name]","검기만 하면 뭐든 상관없어! 검은 무기를 발견하면 장비한 후에 말을 걸어 줘! \\nw[name]","只要是黑色的，什麼都行！如果妳找到了黑色武器，就裝備在身上來跟我說話吧！ \\nw[name]","只要是黑色的，什么都行！如果你找到了黑色武器，就装备在身上来跟我说话吧！\\nw[name]"]);
_tr_m.push(["ということで、よろしくな！ \\nw[name]","I look forward to seeing it! \\nw[name]","아무튼 잘 부탁해! \\nw[name]","事情就是這樣，麻煩妳囉！ \\nw[name]","事情就是这样，麻烦你咯！\\nw[name]"]);
_tr_m.push(["おっ！「黒い武器」持ってきたようだな。 \\nw[name]","I see you brought a \"black weapon\" with you. \\nw[name]","오! 「검은 무기」를 가져왔나 보구나. \\nw[name]","哦！妳似乎把「黑色武器」帶來了啊。 \\nw[name]","哦！你似乎把「黑色武器」带来了啊。\\nw[name]"]);
_tr_m.push(["うむ。この黒さといい、ここの黒といい。 全体的なブラック感といい、中々の黒さだ。 \\nw[name]","Yes, this shade of black...this dark color... My fur pales in comparison. \\nw[name]","으음. 이 검은색도, 이쪽의 검은색도 또 전체적인 블랙스러움이 아주 까맣군. \\nw[name]","嗯。無論是這黑色的程度，或者是這裡的黑色，以及整體的黑色感，都是很棒的黑色。 \\nw[name]","嗯。无论是这黑色的程度，或者是这里的黑色，以及整体的黑色感，都是很棒的黑色。\\nw[name]"]);
_tr_m.push(["よし！合格としよう！！これを持っていけ！ \\nw[name]","Okay! This will do! Take this with you! \\nw[name]","좋아! 합격이다!! 이걸 가져가라! \\nw[name]","很好！妳合格了！把這個帶走吧！ \\nw[name]","很好！你合格了！把这个带走吧！\\nw[name]"]);
_tr_m.push(["世界にはこの俺を含めて４匹のクマがいる。 すべてのクマに認められたらいいことあるかもな！ \\nw[name]","You'll find four of us bears in the world, including myself. See if you can't get their approval, too! \\nw[name]","이 세계에는 나를 포함해서 4마리의 곰이 있어. 모든 곰에게 인정받으면 좋은 일이 생길지도 몰라! \\nw[name]","這個世界上包含我總共有四隻熊。妳如果能獲得每一隻熊的認同，或許會有好事發生喔！ \\nw[name]","包括我在内，这个世界上总共有四只熊。如果你能获得每一只熊的认同，或许会有好事发生哦！\\nw[name]"]);
_tr_m.push(["俺の見たい武器はズバリ！「黒い武器だ」！！ \\nw[name]","The weapon I want to see is a \"black weapon\"! \\nw[name]","내가 보고 싶은 무기는 바로! 「검은 무기」다!! \\nw[name]","我想看的武器，簡單來說就是「黑色的武器」！！ \\nw[name]","我想看的武器，简单来说就是「黑色的武器」！！\\nw[name]"]);
_tr_m.push(["黒けりゃなんでもいい！ 黒い武器を見つけたら、装備して話しかけてくれ！ \\nw[name]","It can be anything, so long as it's black! Equip one and come talk to me again! \\nw[name]","검기만 하면 뭐든 상관없어! 검은 무기를 발견하면 장비한 후에 말을 걸어 줘! \\nw[name]","只要是黑色的，什麼都行！如果妳找到了黑色武器，就裝備在身上來跟我說話吧！ \\nw[name]","只要是黑色的，什么都行！如果你找到了黑色武器，就装备在身上来跟我说话吧！\\nw[name]"]);
_tr_m.push(["返事がない。ただのぬいぐるみのようだ。","There's no response. It's just a stuffed animal.","대답이 없다. 그냥 인형인 모양이다.","沒有回應。看來像是一具普通的布偶。","没有回应。看上去像是一具普通的布偶。"]);
_tr_m.push(["世界にはこの俺を含めて４匹のクマがいる。 すべてのクマに認められたらいいことあるかもな！ \\nw[name]","You'll find four of us bears in the world, including myself. See if you can't get their approval, too! \\nw[name]","이 세계에는 나를 포함해서 4마리의 곰이 있어. 모든 곰에게 인정받으면 좋은 일이 생길지도 몰라! \\nw[name]","這個世界上包含我總共有四隻熊。妳如果能獲得每一隻熊的認同，或許會有好事發生喔！ \\nw[name]","包括我在内，这个世界上总共有四只熊。如果你能获得每一只熊的认同，或许会有好事发生哦！\\nw[name]"]);
_tr_m.push(["やあ！僕はブラウン。キュートなクマさんだよ！ \\nw[name]","Hey there! My name's Brown. I'm the cutest little brown bear you've ever seen! \\nw[name]","안녕! 내 이름은 Brown. 큐트한 곰이야! \\nw[name]","嗨！我是Brown，是一隻可愛的熊喔！ \\nw[name]","嗨！我是Brown，是一只可爱的熊喔！\\nw[name]"]);
_tr_m.push(["君、武器を集めるのが好きそうだね！ ねえねえ、見せてほしい武器があるんだ！ \\nw[name]","It seems you like to collect weapons! There's one weapon I'd really love to see! \\nw[name]","너는 무기를 모으는 걸 좋아하나 보네! 있잖아, 나 보고 싶은 무기가 있어! \\nw[name]","妳看起來很喜歡收集武器呢！對了對了，我希望妳能給我看一種武器！ \\nw[name]","你看起来很喜欢收集武器呢！对了对了，我希望你能给我看一种武器！\\nw[name]"]);
_tr_m.push(["僕の見せてほしい武器は 「トイレットペーパー代わりになりそうな武器」だよ！ \\nw[name]","The weapon I wish to see is \"something that can be used as toilet paper\"! \\nw[name]","내가 보고 싶은 무기는 「화장실 휴지 대신 쓸 수 있는 무기」야! \\nw[name]","我想看的武器是「感覺能代替衛生紙的武器」！ \\nw[name]","我想看的武器是「感觉能代替卫生纸的武器」！\\nw[name]"]);
_tr_m.push(["いざというとき、おしりが拭けると安心だからね！ 備えあれば憂いなし、ってね！ \\nw[name]","You never know when you have to go! As such, it's important to have something to wipe my bear butt with--better safe than sorry! \\nw[name]","여차할 때 엉덩이를 닦을 수 있으면 마음이 놓이잖아! 유비무환이란 말도 있고! \\nw[name]","如果臨時有需要的時候就能拿來擦屁股，可以令人感到放心！這就是所謂的有備無患啊！ \\nw[name]","如果临时有需要的时候就能拿来擦屁股，可以令人感到放心！这就是所谓的有备无患啊！\\nw[name]"]);
_tr_m.push(["見つけてきたら装備して話しかけてね。待ってるよ！ \\nw[name]","When you find something, equip it and come talk to me! I'll be waiting! \\nw[name]","발견하면 장비한 후에 말을 걸어 줘. 기다리고 있을게! \\nw[name]","如果妳找到了那種武器，就裝備在身上來跟我說話吧。我會等妳的！ \\nw[name]","如果你找到了那种武器，就装备在身上来跟我说话吧。我会等你的！\\nw[name]"]);
_tr_m.push(["お！「トイレットペーパー代わりになりそうな武器」を 持ってきてくれたんだね！ \\nw[name]","I see you brought me \"something that can be used as toilet paper\"! \\nw[name]","아! 「화장실 휴지 대신 쓸 수 있는 무기」를 가져왔구나! \\nw[name]","啊！妳把「感覺能代替衛生紙的武器」帶來啦！ \\nw[name]","啊！你把「感觉能代替卫生纸的武器」带来啦！\\nw[name]"]);
_tr_m.push(["うんうん。これならしっかりふけるし、 おしりへの刺激も少ないから、痔にならなくてすむね！ \\nw[name]","Yes... This'll be good for wiping, and it's soft on the skin too. We wouldn't want to cause a rash now! \\nw[name]","그래. 이거라면 엉덩이에 자극도 별로 없으면서 잘 닦일 테니까 치질 걱정도 없을 거야! \\nw[name]","嗯嗯。這個看來能順利擦屁股，對屁股的刺激也不會太強，這樣就不會長痔瘡了！ \\nw[name]","嗯嗯。这个看来能顺利擦屁股，对屁股的刺激也不会太强，这样就不会长痔疮了！\\nw[name]"]);
_tr_m.push(["今度から、その武器を持ち歩くようにするよ！ そうすれば安心だね！ \\nw[name]","Having this on me will give me ease of mind as well as ease of behind! \\nw[name]","앞으로는 그 무기를 들고 다닐게! 그러면 안심해도 되겠지! \\nw[name]","接下來我會隨身攜帶這個武器的！這麼一來就可以放心了！ \\nw[name]","接下来我会随身携带这个武器的！这么一来就可以放心了！\\nw[name]"]);
_tr_m.push(["いいものを見せてもらったお礼に、これをあげるよ！ \\nw[name]","Thank you! Here, you can have this! \\nw[name]","좋은 무기를 가져왔으니까 답례로 이걸 줄게! \\nw[name]","謝謝妳讓我看到了好東西，這個送妳！ \\nw[name]","谢谢你让我看到了好东西，这个送你！\\nw[name]"]);
_tr_m.push(["世界には僕みたいなクマが４匹いるよ。 みんなに認められるといいことがあるかも！ \\nw[name]","There are four of us bears in the world. Go get their approval too! \\nw[name]","이 세계에는 나를 포함해서 곰 4마리가 있어. 모두에게 인정받으면 좋은 일이 생길지도 몰라! \\nw[name]","這個世界上總共有四隻像我這樣的熊。妳如果能獲得大家認同，或許會有好事發生喔！ \\nw[name]","这个世界上总共有四只像我这样的熊。你如果能获得大家认同，或许会有好事发生喔！\\nw[name]"]);
_tr_m.push(["僕の見せてほしい武器は 「トイレットペーパー代わりになりそうな武器」だよ！ \\nw[name]","The weapon I wish to see is \"something that can be used as toilet paper\"! \\nw[name]","내가 보고 싶은 무기는 「화장실 휴지 대신 쓸 수 있는 무기」야! \\nw[name]","我想看的武器是「感覺能代替衛生紙的武器」！ \\nw[name]","我想看的武器是「感觉能代替卫生纸的武器」！\\nw[name]"]);
_tr_m.push(["いざというとき、おしりが拭けると安心だからね！ 備えあれば憂いなし、ってね！ \\nw[name]","You never know when you have to go! As such, it's important to have something to wipe my bear butt with--better safe than sorry! \\nw[name]","여차할 때 엉덩이를 닦을 수 있으면 마음이 놓이잖아! 유비무환이란 말도 있고! \\nw[name]","如果臨時有需要的時候就能拿來擦屁股，可以令人感到放心！這就是所謂的有備無患啊！ \\nw[name]","如果临时有需要的时候就能拿来擦屁股，可以令人感到放心！这就是所谓的有备无患啊！\\nw[name]"]);
_tr_m.push(["見つけてきたら装備して話しかけてね。待ってるよ！ \\nw[name]","When you find something, equip it and come talk to me! I'll be waiting! \\nw[name]","발견하면 장비한 후에 말을 걸어 줘. 기다리고 있을게! \\nw[name]","如果妳找到了那種武器，就裝備在身上來跟我說話吧。我會等妳的！ \\nw[name]","如果你找到了那种武器，就装备在身上来跟我说话吧。我会等你的！\\nw[name]"]);
_tr_m.push(["返事がない。ただのぬいぐるみのようだ。","There's no response. It's just a stuffed animal.","대답이 없다. 그냥 인형인 모양이다.","沒有回應。看來像是一具普通的布偶。","没有回应。看上去像是一具普通的布偶。"]);
_tr_m.push(["世界には僕みたいなクマが４匹いるよ。 みんなに認められるといいことがあるかも！ \\nw[name]","There are four of us bears in the world. Go get their approval too! \\nw[name]","이 세계에는 나를 포함해서 곰 4마리가 있어. 모두에게 인정받으면 좋은 일이 생길지도 몰라! \\nw[name]","這個世界上總共有四隻像我這樣的熊。妳如果能獲得大家認同，或許會有好事發生喔！ \\nw[name]","这个世界上总共有四只像我这样的熊。你如果能获得大家认同，或许会有好事发生喔！\\nw[name]"]);
_tr_m.push(["「魔王城 　※封印されたため現在留守にしております。魔王より」","\"Demonic Castle - NOTE: I'm not here right now because I've been locked away. -Demon Lord\"","「마왕성 　※봉인당해서 현재 부재중입니다. 마왕백」","「魔王城　 ※由於我目前遭到封印，所以不在家。魔王留」","「魔王城　 ※由于我目前遭到封印，所以不在家。魔王留」"]);
_tr_m.push(["「魔王城 　２４時間営業！」","\"Demonic Castle - Always open!\"","「마왕성 24시간 영업!」","「魔王城　 24小時營業！」","「魔王城　 24小时营业！」"]);
_tr_m.push([">はい",">Yes",">예","＞好","＞是"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push(["ここからは海がよく見渡せる。","You have a nice panoramic view of the sea.","여기서는 바다가 잘 보여.","從這裡能夠清楚看見一整片海洋。","从这里能够清楚看见一整片海洋。"]);
_tr_m.push(["ヒヒーン。 \\nw[name]","Neeeigh. \\nw[name]","히힝~ \\nw[name]","嘶嘶── \\nw[name]","嘶嘶── \\nw[name]"]);
_tr_m.push(["ヒヒーン。 \\nw[name]","Neeeigh.\\nw[name]","히힝~ \\nw[name]","嘶嘶── \\nw[name]","嘶嘶── \\nw[name]"]);
_tr_m.push(["・・・オレはすごい馬。 最近人間の言葉を覚えた。 \\nw[すごい馬]","I'm a wild stallion. I recently learned to speak your human language. \\nw[amazing horse]","・・・나는 대단한 말이야. 최근에 인간의 말을 배웠다. \\nw[대단한 말]","……我是一匹很厲害的馬。最近學會了人類的語言。 \\nw[很厲害的馬]","……我是一匹很厉害的马。最近学会了人类的语言。\\nw[很厉害的马]"]);
_tr_m.push(["オレにかかれば・・・お前のレベルアップボーナスの 再振り分けを行うことができる。 \\nw[すごい馬]","I even have the power to let you redistribute your level-up bonuses. \\nw[amazing horse]","나에게 말을 걸면・・・ 너의 레벨업 보너스를 다시 분배할 수 있다. \\nw[대단한 말]","只要由我出馬……就能讓妳重新分配升級獎勵。 \\nw[很厲害的馬]","只要由我出马……就能让你重新分配升级奖励。\\nw[很厉害的马]"]);
_tr_m.push(["ヒヒーン。20000\\g でレベルアップボーナスを 再振り分けしてやろう。（所持金：\\v[3]\\g）\\!  \\nw[すごい馬]","Neeeigh! You can redistribute your level-up bonuses for 20,000\\g. (Money: \\v[3]\\g)\\!  \\nw[amazing horse]","히힝~ 20000\\g에 레벨업 보너스를 다시 분배해 줄게.<br>(소지금: \\v[3]\\g)\\!  \\nw[대단한 말]","嘶嘶──只要給我20000\\g ，我就能讓妳重新分配升級獎勵。（持有金錢：\\v[3]\\g）\\!  \\nw[很厲害的馬]","嘶嘶──只要给我20000\\g，我就能让你重新分配升级奖励。（持有金钱：\\v[3]\\g）\\! \\nw[很厉害的马]"]);
_tr_m.push([">再振り分けする",">Redistribute",">다시 분배한다","＞重新分配","＞重新分配"]);
_tr_m.push(["では・・・まずは下げる能力を一つ選ぶがよい。\\!  \\nw[すごい馬]","Go ahead and choose a stat you'd like to lower.\\!  \\nw[amazing horse]","그럼・・・ 먼저 낮출 능력을 하나 선택해라.\\!  \\nw[대단한 말]","那麼……首先請妳選擇一項要降低的能力。\\!  \\nw[很厲害的馬]","那么……首先请你选择一项要降低的能力。\\! \\nw[很厉害的马]"]);
_tr_m.push([">最大ＨＰアップ",">Max HP",">최대 HP 상승","＞提升最大HP","＞提升最大HP"]);
_tr_m.push([">攻撃力アップ",">ATK",">공격력 상승","＞提升攻擊力","＞提升攻击力"]);
_tr_m.push([">運アップ",">LCK",">운 상승","＞提升運氣","＞提升运气"]);
_tr_m.push([">やっぱやめる",">Cancel",">역시 그만둔다","＞算了","＞算了"]);
_tr_m.push(["\\v[3]を下げるのだな。 では、代わりに上げる能力を一つ選ぶがよい。\\!  \\nw[すごい馬]","I'll lower you \\v[3] stat. Now choose a stat you wish to increase.\\!  \\nw[amazing horse]","\\v[3] 말이지? 그럼 그 대신 높일 능력을 하나 선택해라.\\!  \\nw[대단한 말]","妳要降低\\v[3]啊。那麼，請妳再選擇一個要提高的能力。\\!  \\nw[很厲害的馬]","你要降低\\v[3]啊。那么，请你再选择一个要提高的能力。\\! \\nw[很厉害的马]"]);
_tr_m.push([">最大ＨＰアップ",">Max HP",">최대 HP 상승","＞提升最大HP","＞提升最大HP"]);
_tr_m.push([">攻撃力アップ",">ATK",">공격력 상승","＞提升攻擊力","＞提升攻擊力"]);
_tr_m.push([">運アップ",">LCK",">운 상승","＞提升運氣","＞提升運氣"]);
_tr_m.push([">やっぱやめる",">Cancel",">역시 그만둔다","＞算了","＞算了"]);
_tr_m.push(["\\v[3]を下げ、\\v[4]を上げるのだな。 よかろう！！ではいくぞ！！！ \\nw[すごい馬]","Very well! I'll lower your \\v[3] and raise your \\v[4]. It shall be done! \\nw[amazing horse]","\\v[3](을)를 낮추고 \\v[4](을)를 높이는구나. 좋다!! 그럼 간다!!! \\nw[대단한 말]","降低\\v[3]，提升\\v[4]，對吧。很好！！那我要開始囉！！！ \\nw[很厲害的馬]","降低\\v[3]，提升\\v[4]，对吧。很好！！那我要开始咯！！！\\nw[很厉害的马]"]);
_tr_m.push(["\\n[1]の最大ＨＰが下がった！ 最大ＨＰ \\v[3] → \\c[14]\\v[4]","\\n[1]'s Max HP has decreased!<br>Max HP:\\v[3] → \\c[14]\\v[4]","\\n[1]의 최대 HP가 감소했다!<br>최대 HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP下降了！<br>最大HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP下降了！<br>最大HP\\v[3]→\\c[14]\\v[4]"]);
_tr_m.push(["\\n[1]の攻撃力が下がった！ \\i[469]上昇値 \\v[3] → \\c[14]\\v[4]","\\n[1]'s ATK has decreased!<br>\\i[469] Boost: \\v[3] → \\c[14]\\v[4]","\\n[1]의 공격력이 감소했다!<br>\\i[469]상승치 \\v[3] → \\c[14]\\v[4]","\\n[1]的攻擊力下降了！<br>\\i[469]上升值 \\v[3] → \\c[14]\\v[4]","\\n[1]的攻击力下降了！<br>\\i[469]上升值\\v[3]→\\c[14]\\v[4]"]);
_tr_m.push(["武器の\\i[470]耐久値減少確率が上がった！ \\i[470]減少確率 \\v[3] → \\c[14]\\v[4]","The chances of weapon \\i[470]durability loss have increased!<br>Chance of \\i[470]durability loss: \\v[3] → \\c[14]\\v[4]","무기의 \\i[470] 내구도 감소 확률이 상승했다!<br>\\i[470]감소 확률 \\v[3] → \\c[14]\\v[4]","武器的\\i[470]耐久值減少機率上升了！<br>\\i[470]減少機率 \\v[3] → \\c[14]\\v[4]","武器的\\i[470]耐久值减少机率上升了！<br>\\i[470]减少机率\\v[3]→\\c[14]\\v[4]"]);
_tr_m.push(["\\n[1]の回避率が下がった！ 回避率 \\v[3] → \\c[14]\\v[4]","\\n[1]'s Evasion ability has decreased!<br>Evasion: \\v[3] → \\c[14]\\v[4]","\\n[1]의 회피율이 감소했다!<br>회피율 \\v[3] → \\c[14]\\v[4]","\\n[1]的回避率下降了！<br>回避率 \\v[3] → \\c[14]\\v[4]","\\n[1]的回避率下降了！<br>回避率\\v[3]→\\c[14]\\v[4]"]);
_tr_m.push(["\\n[1]の最大ＨＰが上がった！ 最大ＨＰ \\v[3] → \\c[14]\\v[4]","\\n[1]'s Max HP has increased!<br>Max HP: \\v[3] → \\c[14]\\v[4]","\\n[1]의 최대 HP가 상승했다!<br>최대 HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP上升了！<br>最大HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP上升了！<br>最大HP\\v[3]→\\c[14]\\v[4]"]);
_tr_m.push(["\\n[1]の攻撃力が上がった！ \\i[469]上昇値 \\v[3] → \\c[14]\\v[4]","\\n[1]'s ATK has increased!<br>\\i[469] Boost: \\v[3] → \\c[14]\\v[4]","\\n[1]의 공격력이 상승했다!<br>\\i[469]상승치 \\v[3] → \\c[14]\\v[4]","\\n[1]的攻擊力上升了！<br>\\i[469]上升值 \\v[3] → \\c[14]\\v[4]","\\n[1]的攻击力上升了！<br>\\i[469]上升值\\v[3]→\\c[14]\\v[4]"]);
_tr_m.push(["武器の\\i[470]耐久値減少確率が下がった！ \\i[470]減少確率 \\v[3] → \\c[14]\\v[4]","The chances of weapon \\i[470]durability loss have decreased!<br>Chance of \\i[470]durability loss: \\v[3] → \\c[14]\\v[4]","무기의 \\i[470] 내구도 감소 확률이 감소했다!<br>\\i[470]감소 확률 \\v[3] → \\c[14]\\v[4]","武器的\\i[470]耐久值減少機率下降了！<br>\\i[470]減少機率 \\v[3] → \\c[14]\\v[4]","武器的\\i[470]耐久值减少机率下降了！<br>\\i[470]减少机率\\v[3]→\\c[14]\\v[4]"]);
_tr_m.push(["\\n[1]の回避率が上がった！ 回避率 \\v[3] → \\c[14]\\v[4]","\\n[1]'s Evasion ability has increased!<br>Evasion: \\v[3] → \\c[14]\\v[4]","\\n[1]의 회피율이 상승했다!<br>회피율 \\v[3] → \\c[14]\\v[4]","\\n[1]的回避率上升了！<br>回避率 \\v[3] → \\c[14]\\v[4]","\\n[1]的回避率上升了！<br>回避率\\v[3]→\\c[14]\\v[4]"]);
_tr_m.push(["たしかに 20000\\g 受け取った。 \\nw[すごい馬]","That'll be 20,000\\g. \\nw[amazing horse]","정확하게 20000\\g 받았다. \\nw[대단한 말]","我確實收到20000\\g 了。 \\nw[很厲害的馬]","我确实收到20000\\g了。\\nw[很厉害的马]"]);
_tr_m.push([">説明を聞く",">Explanation",">설명 듣기","＞聆聽說明","＞聆聽說明"]);
_tr_m.push([">やめる",">Cancel",">그만둔다","＞算了","＞算了"]);
_tr_m.push(["そうか。ではまた。 \\nw[すごい馬]","Nay? Maybe another time, then. \\nw[amazing horse]","그래? 그럼 잘 가라. \\nw[대단한 말]","這樣啊。那就下次吧。 \\nw[很厲害的馬]","这样啊。那就下次吧。\\nw[很厉害的马]"]);
_tr_m.push(["どうもこんにちは。私はすごい羊です。 \\nw[すごい羊]","Good day to you. I'm an amazing sheep. \\nw[amazing sheep]","안녕하세요. 나는 대단한 양입니다. \\nw[대단한 양]","妳好，我是很厲害的羊。 \\nw[很厲害的羊]","你好，我是很厉害的羊。\\nw[很厉害的羊]"]);
_tr_m.push(["以後私に話しかけるたび、 あなたがいまだ未発見の武器のヒントを差し上げます。 \\nw[すごい羊]","Each time you speak to me, I'll give you a hint about an undiscovered weapon. \\nw[amazing sheep]","앞으로 나한테 말을 걸 때마다 당신이 아직 발견하지 못한 무기의 힌트를 드립니다. \\nw[대단한 양]","以後如果妳來找我說話，我會給妳有關於妳尚未發現的武器線索。 \\nw[很厲害的羊]","以后如果你来找我说话，我会给你有关于你尚未发现的武器线索。\\nw[很厉害的羊]"]);
_tr_m.push(["なぜなら、私はすごい羊であるからです。 \\nw[すごい羊]","How do I know about weapons, you ask? Simple. It's because I'm amazing. \\nw[amazing sheep]","왜냐하면 나는 대단한 양이니까요. \\nw[대단한 양]","要問為什麼的話，因為我是很厲害的羊。 \\nw[很厲害的羊]","要问为什么的话，因为我是很厉害的羊。\\nw[很厉害的羊]"]);
_tr_m.push(["こんにちは。すごい羊です。 \\nw[すごい羊]","Good day to you. It's me, the amazing sheep. \\nw[amazing sheep]","안녕하세요. 대단한 양입니다. \\nw[대단한 양]","妳好，我是很厲害的羊。 \\nw[很厲害的羊]","你好，我是很厉害的羊。\\nw[很厉害的羊]"]);
_tr_m.push(["あなたはすべての武器を集めたようです。 \\nw[すごい羊]","It seems you've got every weapon there is to have. \\nw[amazing sheep]","당신은 모든 무기를 모으셨나 보군요. \\nw[대단한 양]","妳似乎收集到了所有武器。 \\nw[很厲害的羊]","你似乎收集到了所有武器。\\nw[很厉害的羊]"]);
_tr_m.push(["すごいのは私ではなく、 あなたでしたか・・・。 \\nw[すごい羊]","I guess you are the amazing one. \\nw[amazing sheep]","대단한 건 내가 아니라 당신이었네요・・・ \\nw[대단한 양]","原來很厲害的不是我，而是妳啊…… \\nw[很厲害的羊]","原来很厉害的不是我，而是你啊……\\nw[很厉害的羊]"]);
_tr_m.push(["こりゃ、一本とられましたね！ \\nw[すごい羊]","Not baaaad at all, I must say! \\nw[amazing sheep]","제가 졌습니다! \\nw[대단한 양]","這次是我輸了！ \\nw[很厲害的羊]","这次是我输了！\\nw[很厉害的羊]"]);
_tr_m.push(["あなたはすべての武器を集めたようです。 しかし、まだ未解禁の武器があるようですよ。 \\nw[すごい羊]","It seems you've got every weapon there is to have. However, you have not used all of them yet. \\nw[amazing sheep]","당신은 모든 무기를 모으셨나 보군요. 하지만 아직 미확인 상태인 무기가 있는 것 같아요. \\nw[대단한 양]","妳似乎收集到了所有武器。不過，似乎還有未解禁的武器哦。 \\nw[很厲害的羊]","你似乎收集到了所有武器。不过，似乎还有未解锁的武器哦。\\nw[很厉害的羊]"]);
_tr_m.push(["図鑑に目を凝らして・・・ 半透明になっている武器を探すのです・・・！！ \\nw[すごい羊]","Take a peek into your Weapons Gallery and look for the ones that are translucent! \\nw[amazing sheep]","도감을 뚫어져라 쳐다보며・・・ 반투명하게 보이는 무기를 찾으세요・・・!! \\nw[대단한 양]","妳仔細觀察圖鑑……去尋找圖鑑上半透明的武器吧……！！ \\nw[很厲害的羊]","你仔细观察图鉴……去寻找图鉴上半透明的武器吧……！！\\nw[很厉害的羊]"]);
_tr_m.push(["残り一息、がんばってください。 \\nw[すごい羊]","You're almost there. Keep up the good work! \\nw[amazing sheep]","얼마 안 남았으니까 힘내세요. \\nw[대단한 양]","只剩下一點了，繼續努力吧。 \\nw[很厲害的羊]","只剩下一点了，继续努力吧。\\nw[很厉害的羊]"]);
_tr_m.push(["あなたが未発見の武器は・・・ 「\\v[3]」から始まるこのシルエット！\\!\\>（所持金：\\v[4]\\g） \\nw[すごい羊]","Here's a weapon you haven't found yet. The weapon behind this silhouette starts with the letter \"\\v[3]\"!\\! \\>(Money: \\v[4]\\g) \\nw[amazing sheep]","당신이 아직 찾지 못한 무기는・・・ 「\\v[3]」(으)로 시작되는 이 실루엣!\\! \\>(소지금: \\v[4]\\g) \\nw[대단한 양]","妳尚未發現的武器是……由「\\v[3]」開始的這個剪影！\\! \\>（持有金錢：\\v[4]\\g） \\nw[很厲害的羊]","你尚未发现的武器是……由「\\v[3]」开始的这个剪影！\\! \\>（持有金钱：\\v[4]\\g）\\nw[很厉害的羊]"]);
_tr_m.push([">ありがとう",">Thanks",">고마워","＞謝謝","＞谢谢"]);
_tr_m.push([">もう少し教えて　 1000\\g",">Tell me more    1,000\\g",">조금 더 가르쳐 줘　 1000\\g","＞再多告訴我一點　 1000\\g","＞再多告诉我一点　1000\\g"]);
_tr_m.push(["仕方ないですね。 \\nw[すごい羊]","Since you're nagging nicely... \\nw[amazing sheep]","어쩔 수 없죠. \\nw[대단한 양]","真拿妳沒辦法。 \\nw[很厲害的羊]","真拿你没办法。\\nw[很厉害的羊]"]);
_tr_m.push(["その武器の名は・・・ 「\\v[3]」でございます。 \\nw[すごい羊]","It's called the \"\\v[3]\". \\nw[amazing sheep]","그 무기의 이름은・・・ 「\\v[3]」입니다. \\nw[대단한 양]","那個武器的名字就叫做……「\\v[3]」。 \\nw[很厲害的羊]","那个武器的名字就叫做……「\\v[3]」。\\nw[很厉害的羊]"]);
_tr_m.push([">すべて吐け　　　10000\\g",">Spill the beans    10,000\\g",">전부 불어라　　　10000\\g","＞說出一切吧　　　10000\\g","＞说出一切吧　10000\\g"]);
_tr_m.push(["すべて教えますので・・・お許しを・・・。 \\nw[すごい羊]","Okay! I'll tell you... Forgive me... \\nw[amazing sheep]","전부 가르쳐 드릴 테니・・・ 제발 목숨만은・・・ \\nw[대단한 양]","我會說出一切的…請饒了我…… \\nw[很厲害的羊]","我会说出一切的…请饶了我……\\nw[很厉害的羊]"]);
_tr_m.push(["この武器の名は「\\v[3]」。 \\v[4] \\nw[すごい羊]","This weapon is called the \"\\v[3]\".<br> \\v[4] \\nw[amazing sheep]","이 무기의 이름은 「\\v[3]」. <br>\\v[4] \\nw[대단한 양]","這個武器的名字叫做「\\v[3]」。<br>\\v[4] \\nw[很厲害的羊]","那个武器的名字就叫做……「\\v[3]」。<br>\\v[4]\\nw[很厉害的羊]"]);
_tr_m.push(["がんばってください。 \\nw[すごい羊]","Good luck! \\nw[amazing sheep]","건투를 빕니다. \\nw[대단한 양]","請多多努力吧。 \\nw[很厲害的羊]","请多多努力吧。\\nw[很厉害的羊]"]);
_tr_m.push(["「↑ヘイワ村 　←↓→ヘイワ村の外」","\"↑Peaceville 　←↓→Not Peaceville\"","「↑평화 마을 　←↓→평화 마을 외곽」","「↑和平村 　←↓→和平村外」","「↑和平村　←↓→和平村外」"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["魔王城から出ますか？\\! ","Exit the Demonic Castle?\\! ","마왕성에서 나가시겠습니까?\\! ","要離開魔王城嗎？\\! ","要离开魔王城吗？\\! "]);
_tr_m.push([">はい",">Yes",">예","＞好","＞是"]);
_tr_m.push([">いいえ",">No",">아니요","＞不用","＞否"]);
_tr_m.push(["\\c[2]ふはははは！我は魔王なり！！ \\nw[魔王]","\\c[2]Mwahaha! I am the Demon Lord! \\nw[Demon Lord]","\\c[2]푸하하하하! 나는 마왕이다!! \\nw[마왕]","\\c[2]呼哈哈哈哈！我就是魔王！！ \\nw[魔王]","\\c[2]呼哈哈哈哈！我就是魔王！！\\nw[魔王]"]);
_tr_m.push(["\\c[2]どうだこのいかにも悪っぽい外見は！！ 我こそ最強の魔王である！ \\nw[魔王]","\\c[2]How do you like my evil-looking clothes? I am the strongest Demon Lord ever! \\nw[Demon Lord]","\\c[2]어떠냐, 너무나도 못된 이 모습이!! 나야말로 최강의 마왕이다! \\nw[마왕]","\\c[2]怎麼樣，我這個一看就覺得很邪惡的外觀很棒吧！！我就是最強的魔王！ \\nw[魔王]","\\c[2]怎么样，我这个看起来就很邪恶的外观很棒吧！！我就是最强的魔王！\\nw[魔王]"]);
_tr_m.push(["\\c[2]・・・ってええ！？ なんか小さくてかわいいのが来た！ \\nw[魔王]","\\c[2]What the?! What is someone so small and cute doing here?! \\nw[Demon Lord]","\\c[2]・・・가만!? 뭔가 작고 귀여운 게 왔잖아! \\nw[마왕]","\\c[2]……咦咦！？有個嬌小可愛的傢伙來了！ \\nw[魔王]","\\c[2]……咦咦！？有个娇小可爱的家伙来了！\\nw[魔王]"]);
_tr_m.push(["\\c[2]キミ、勇者じゃないよね！？ なんでここまで来れたの！？すご！！！ \\nw[魔王]","\\c[2]You're not some kind of heroine, are you?! How did you even get here? Mind. Blown. \\nw[Demon Lord]","\\c[2]너, 용사가 아니지!? 여기까지 어떻게 왔어!? 장난 아니다!!! \\nw[마왕]","\\c[2]妳不是勇者吧！？為什麼能來到這裡！？太厲害了！！！ \\nw[魔王]","\\c[2]你不是勇者吧！？为什么能来到这里！？太厉害了！！！\\nw[魔王]"]);
_tr_m.push(["\\c[2]・・・まあいい！ここまで来たということは、 この魔王に挑戦する権利があるということだ。 \\nw[魔王]","\\c[2]Well, whatever. You being here gives you the right to challenge moi, the Demon Lord! \\nw[Demon Lord]","\\c[2]・・・뭐, 아무튼! 여기까지 왔다는 건 이 마왕에게 도전할 권리가 있다는 뜻이다. \\nw[마왕]","\\c[2]……算了！既然妳能來到這裡，就代表有挑戰我這魔王的權利。 \\nw[魔王]","\\c[2]……算了！既然你能来到这里，就代表有挑战我这魔王的权利。\\nw[魔王]"]);
_tr_m.push(["\\c[2]でも。 その前に、我の自己紹介をしようではないか！ \\nw[魔王]","\\c[2]How rude of me. I haven't even introduced myself yet. \\nw[Demon Lord]","\\c[2]하지만 그 전에 내가 자기 소개부터 해야겠다! \\nw[마왕]","\\c[2]不過，在那之前，我要先自我介紹一下！ \\nw[魔王]","\\c[2]不过，在那之前，我要先自我介绍一下！\\nw[魔王]"]);
_tr_m.push(["\\c[2]我は魔王ドルハーバ！ 魔王歴１０００年のベテラン魔王だ！ \\nw[魔王ドルハーバ]","\\c[2]I am the Demon Lord Dolhabach! I'm a seasoned demon lord with a demonic history of 1,000 demon years! \\nw[Demon Lord Dolhabach]","\\c[2]나는 마왕 도르하바! 마왕 경력 1000년을 자랑하는 베테랑 마왕이다! \\nw[마왕 도르하바]","\\c[2]我是魔王多爾哈巴！是擔任魔王1000年的資深魔王！ \\nw[魔王多爾哈巴]","\\c[2]我是魔王多尔哈巴！是担任魔王1000年的资深魔王！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]一度勇者に敗れて封印され、その後復活し・・・ この大陸の城を滅ぼしたりした魔王！それが我だ！！ \\nw[魔王ドルハーバ]","\\c[2]I was defeated by a hero and locked away once, but I have been restored. I am the very Demon Lord that destroyed this kingdom's \\c[2]castle! \\nw[Demon Lord Dolhabach]","\\c[2]한때 용사에게 패해 봉인당했으나 그 후에 부활하여・・・ 이 대륙의 성을 멸망시켰던 마왕! 그게 바로 나다!! \\nw[마왕 도르하바]","\\c[2]曾有一位魔王敗給勇者，遭到封印，之後又再度復活……摧毀了這片大陸上的城堡！那個魔王就是我！！ \\nw[魔王多爾哈巴]","\\c[2]曾有一位魔王败给勇者，遭到封印，之后又再度复活……摧毁了这片大陆上的城堡！那个魔王就是我！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]しかし、魔王だからと言って 悪さばかりしてるわけじゃあないぞ！ \\nw[魔王ドルハーバ]","\\c[2]But a Demon Lord does much more than commit crimes against humanity! \\nw[Demon Lord Dolhabach]","\\c[2]하지만 마왕이라고 해서 나쁜 짓만 하는 건 아니야! \\nw[마왕 도르하바]","\\c[2]不過，雖然我是魔王，但我可沒有老是在做壞事喔！ \\nw[魔王多爾哈巴]","\\c[2]不过，虽然我是魔王，但我可没有老是在做坏事喔！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]我は人類に危害を与えすぎない程度に魔物を送り込み、 人類と魔物の勢力の均衡を保っているのだ！ \\nw[魔王ドルハーバ]","\\c[2]I maintain the balance between man and monsters! I make sure those critters don't cause too much harm to humankind. \\nw[Demon Lord Dolhabach]","\\c[2]나는 인류에게 지나친 위해를 가하지 않을 정도로만 마물을 보내서, 인류와 마물 사이의 균형을 유지하는 것이다! \\nw[마왕 도르하바]","\\c[2]我努力保持著不會對人類造成危害的頻率，把魔物送出城，維持人類與魔物的勢力均衡！ \\nw[魔王多爾哈巴]","\\c[2]我努力保持着不会对人类造成危害的频率，把魔物送出城，维持人类与魔物的势力均衡！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]魔物のいない平和な世界では、 人類は平和ボケし堕落してしまうからな！ \\nw[魔王ドルハーバ]","\\c[2]However, humankind will become complacent in a peaceful world without monsters! \\nw[Demon Lord Dolhabach]","\\c[2]마물이 없는 평화로운 세계에서 인류는 평화에 도취되어 타락해 버릴 테니까! \\nw[마왕 도르하바]","\\c[2]因為沒有魔物的和平世界，會讓人類墮落，沉溺於和平之中！！ \\nw[魔王多爾哈巴]","\\c[2]因为没有魔物的和平世界，会让人类堕落，沉溺于和平之中！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]我の存在により人類は一致団結し発展するのだ！ つまり、これも大事な仕事というわけだな！ \\nw[魔王ドルハーバ]","\\c[2]My very presence helps people come together and spurs their growth! Do you see the importance of my job? \\nw[Demon Lord Dolhabach]","\\c[2]나의 존재로 인해 인류는 일치단결하며 발전하는 것이다! 즉, 중요한 역할을 한다는 뜻이지! \\nw[마왕 도르하바]","\\c[2]我的存在能讓人類團結一致，進步發展！也就是說，這是一件很重要的工作！ \\nw[魔王多爾哈巴]","\\c[2]我的存在能让人类团结一致，进步发展！也就是说，这是一件很重要的工作！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]というわけで、簡単にキミたちに滅ぼされたら 困るのだ！人類にとってもな！！ \\nw[魔王ドルハーバ]","\\c[2]That means I can't just lose to the likes of you or anyone else! Mankind would be lost without me! \\nw[Demon Lord Dolhabach]","\\c[2]그러니 너희에게 쉽게 패배하면 곤란하다! 인류 입장에서도 말이야!! \\nw[마왕 도르하바]","\\c[2]因此，如果我輕易就被你們給打倒，那可就麻煩了！這對人類來說也會是件很困擾的事！！ \\nw[魔王多爾哈巴]","\\c[2]因此，如果我轻易就被你们给打倒，那可就麻烦了！这对人类来说也会是件很困扰的事！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]だから魔王城には強い魔物をたくさん雇って、 かなり難易度高くしたはずなのに・・・。 \\nw[魔王ドルハーバ]","\\c[2]I've employed the biggest and baddest of monsters at the castle, and yet here you stand. \\nw[Demon Lord Dolhabach]","\\c[2]그래서 마왕성에 강력한 마물을 잔뜩 고용해서 난이도를 실컷 높여 놨는데・・・ \\nw[마왕 도르하바]","\\c[2]所以魔王城才會雇用許多強大的魔物，把難度調整得很高。然而…… \\nw[魔王多爾哈巴]","\\c[2]所以魔王城才会雇用许多强大的魔物，把难度调整得很高。然而……\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]まさかキミみたいな少女に突破されちゃうとはな！ 一度、難易度設定を見直さなくては！ \\nw[魔王ドルハーバ]","\\c[2]I can't believe you defeated them all! I'll have to remember to adjust the difficulty setting later. \\nw[Demon Lord Dolhabach]","\\c[2]설마 너 같은 소녀가 돌파할 줄이야! 다시 난이도 설정을 검토해 봐야겠군! \\nw[마왕 도르하바]","\\c[2]想不到會被妳這種少女給突破！我必須要重新設定難度才行！ \\nw[魔王多爾哈巴]","\\c[2]想不到会被你这种少女给突破！我必须要重新设定难度才行！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]まあいい！キミが何者であろうと、 この魔王城をここまで登ってきたことは事実だ！ \\nw[魔王ドルハーバ]","\\c[2]Anyways! Nothing changes the fact that you stand at the top of the Demonic Castle! \\nw[Demon Lord Dolhabach]","\\c[2]좋다! 네 정체가 무엇이든 이 마왕성을 여기까지 올라온 건 사실이지! \\nw[마왕 도르하바]","\\c[2]算了！無論妳是什麼人，妳的確已經進到魔王城，還來到了這裡！ \\nw[魔王多爾哈巴]","\\c[2]算了！无论你是什么人，你的确已经进到魔王城，还来到了这里！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]我は全力でキミと戦い、 魔王としての役目をまっとうするのみ！ \\nw[魔王ドルハーバ]","\\c[2]I will grant you the battle you desire, and show you the full power of the Demon Lord! \\nw[Demon Lord Dolhabach]","\\c[2]나는 전력을 다해 너와 싸워서 마왕의 역할을 다하면 그만이다! \\nw[마왕 도르하바]","\\c[2]我要使出全力跟妳戰鬥，完成身為魔王的任務！ \\nw[魔王多爾哈巴]","\\c[2]我要使出全力跟你战斗，完成身为魔王的任务！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]ちなみに！もし負けて再挑戦しに来た時には このテキストはスキップできるから安心してくれよな！ \\nw[魔王ドルハーバ]","\\c[2]Know that I am kind enough to let you skip all this text if you happen to lose (you will) and decide to fight me again (you \\c[2]probably won't)! \\nw[Demon Lord Dolhabach]","\\c[2]참고로! 만약 져서 재도전하러 왔을 때는 이 텍스트를 건너뛸 수 있으니까 걱정하지 말고! \\nw[마왕 도르하바]","\\c[2]附帶一提，如果妳輸了之後想要再次挑戰，這段對話可以跳過，所以妳可以稍微放心！ \\nw[魔王多爾哈巴]","\\c[2]附带一提，如果你输了之后想要再次挑战，这段对话可以跳过，所以你可以稍微放心！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]さあ、少女よ！ 貴様の力を見せてもらおうではないか！！ \\nw[魔王ドルハーバ]","\\c[2]Now, young lady! Show me the extent of your strength! \\nw[Demon Lord Dolhabach]","\\c[2]자, 소녀여! 네 힘을 보여 다오!! \\nw[마왕 도르하바]","\\c[2]來吧，少女啊！讓我見識一下妳的力量吧！！ \\nw[魔王多爾哈巴]","\\c[2]来吧，少女啊！让我见识一下你的力量吧！！ \\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]言っておくが・・・我は強いぞ！！！ \\nw[魔王ドルハーバ]","\\c[2]I warn you... I am more powerful than you can possibly imagine! \\nw[Demon Lord Dolhabach]","\\c[2]미리 말해 두지만・・・ 나는 강하다!!! \\nw[마왕 도르하바]","\\c[2]我話說在前頭……我可是很強的喔！！！ \\nw[魔王多爾哈巴]","\\c[2]我话说在前头……我可是很强的喔！！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\{\\{魔王ドルハーバ\\|\\|\\|\\^","\\{\\{Demon Lord Dolhabach\\|\\|\\|\\^","\\{\\{마왕 도르하바\\|\\|\\|\\^","\\{\\{魔王多爾哈巴\\|\\|\\|\\^","\\{\\{魔王多尔哈巴\\|\\|\\|\\^"]);
_tr_m.push(["\\c[2]ふはははは！我は魔王なり！！ \\nw[魔王ドルハーバ]","\\c[2]Mwahaha! I am the Demon Lord! \\nw[Demon Lord Dolhabach]","\\c[2]푸하하하하! 나는 마왕이다!! \\nw[마왕 도르하바]","\\c[2]呼哈哈哈哈！我就是魔王！！ \\nw[魔王多爾哈巴]","\\c[2]呼哈哈哈哈！我就是魔王！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]・・・ってええ！？ またキミ来たの！？ \\nw[魔王ドルハーバ]","\\c[2]...Agh! It's you again?! \\nw[Demon Lord Dolhabach]","\\c[2]・・・앗!? 또 네가 왔어!? \\nw[마왕 도르하바]","\\c[2]……咦咦！？妳又來了嗎！？ \\nw[魔王多爾哈巴]","\\c[2]……咦咦！？你又来了吗！？\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]ちょっと来すぎじゃなあい！？ もしかして暇つぶしみたいなノリで来てない！？！？ \\nw[魔王ドルハーバ]","\\c[2]I think I've been seeing a little too much of you lately. Don't you have anything better to do? \\nw[Demon Lord Dolhabach]","\\c[2]너무 자주 오는 것 아니냐!? 혹시 시간을 때우려고 오는 거냐!?!? \\nw[마왕 도르하바]","\\c[2]妳也來太多次了吧！？妳該不會是來打發時間的！？！？ \\nw[魔王多爾哈巴]","\\c[2]你也来太多次了吧！？你该不会是来打发时间的！？！？\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]え？なんで封印されたのにここにいるのかって？ まあ、形式的には封印されたんだが・・・。 \\nw[魔王ドルハーバ]","\\c[2]You're wondering how I'm here after being locked away? Technically, the seal is still intact... \\nw[Demon Lord Dolhabach]","\\c[2]뭐? 왜 봉인됐는데도 여기에 있냐고? 뭐, 형식적으로는 봉인됐지만・・・ \\nw[마왕 도르하바]","\\c[2]咦？妳問我為什麼被封印了還待在這裡？這個，形式上我是被封印了沒錯啦，不過…… \\nw[魔王多爾哈巴]","\\c[2]咦？你问我为什么被封印了还待在这里？这个，形式上我是被封印了没错啦，不过……\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]別にわざわざクリスタルの中とかに封印されなくても、 ちゃんとここでおとなしくしてれば実はオーケーなのだよ。 \\nw[魔王ドルハーバ]","\\c[2]Being locked away doesn't mean I have to spend my days in a crystal ball or anything. I can also be confined to this castle, \\c[2]too. \\nw[Demon Lord Dolhabach]","\\c[2]굳이 크리스털 안에 봉인당하지 않아도 그냥 여기서 얌전히 있으면 사실 상관없잖아. \\nw[마왕 도르하바]","\\c[2]但其實我也不用特地被封印在水晶之類的東西裡，只要乖乖待在這，其實就OK囉。 \\nw[魔王多爾哈巴]","\\c[2]但其实我也不用特地被封印在水晶之类的东西里，只要乖乖待在这，其实就OK啰。\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]封印中はちゃんと封印手当もつくから、 部下に食材とか買いに行かせれば生活には困らないしな。 \\nw[魔王ドルハーバ]","\\c[2]I collect Demon Lord unemployment and my underlings do my shopping for me, so being under castle arrest really isn't that bad at all. \\nw[Demon Lord Dolhabach]","\\c[2]봉인된 동안에는 봉인 수당도 받으니까 부하를 시켜서 음식도 사 오면 사는 데는 별 지장이 없거든. \\nw[마왕 도르하바]","\\c[2]我在被封印的期間可以領取封印津貼，只要派部下去買食材回來，生活也不會有什麼問題。 \\nw[魔王多爾哈巴]","\\c[2]我在被封印的期间可以领取封印津贴，只要派部下去买食材回来，生活也不会有什么问题。\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]つまりしばらくはここで引きこもり生活ってワケだ。 けっこういい仕事なんだ、魔王っていうのは！ふははは！ \\nw[魔王ドルハーバ]","\\c[2]I guess you could say I'm working from home. I actually kind of like it, being the Demon Lord. Mwahaha! \\nw[Demon Lord Dolhabach]","\\c[2]즉, 한동안은 여기에 은둔해서 살 생각이야. 마왕이란 게 은근히 꿀 빠는 직업이야! 푸하하하! \\nw[마왕 도르하바]","\\c[2]也就是說，我只要在這段期間內宅在這裡過日子就好。魔王還真是一件好工作啊！呼哈哈哈！ \\nw[魔王多爾哈巴]","\\c[2]也就是说，我只要在这段期间内宅在这里过日子就好。魔王还真是一件好工作啊！呼哈哈哈！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]・・・ともかく！またここまで来たということは、 この魔王に挑戦する権利があるということだ。 \\nw[魔王ドルハーバ]","\\c[2]Anyhoo! You being here again gives you the right to challenge moi, the Demon Lord! \\nw[Demon Lord Dolhabach]","\\c[2]・・・뭐, 아무튼! 여기까지 또 왔다는 건 이 마왕에게 도전할 권리가 있다는 뜻이다. \\nw[마왕 도르하바]","\\c[2]……總之！既然妳又來到了這裡，就代表妳獲得了挑戰我這個魔王的權利。 \\nw[魔王多爾哈巴]","\\c[2]……总之！既然你又来到了这里，就代表你获得了挑战我这个魔王的权利。\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]我は全力でキミと戦い、 魔王としての役目をまっとうするのみ！ \\nw[魔王ドルハーバ]","\\c[2]I will grant you the battle you desire, and show you the full power of the Demon Lord! \\nw[Demon Lord Dolhabach]","\\c[2]나는 전력을 다해 너와 싸워서 마왕의 역할을 다하면 그만이다! \\nw[마왕 도르하바]","\\c[2]我要使出全力跟妳戰鬥，完成身為魔王的任務！ \\nw[魔王多爾哈巴]","\\c[2]我要使出全力跟你战斗，完成身为魔王的任务！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]ちなみに！もし負けて再挑戦しに来た時には このテキストはスキップできるから安心してくれよな！ \\nw[魔王ドルハーバ]","\\c[2]Know that I am kind enough to let you skip all this text if you happen to lose (you will) and decide to fight me again (you \\c[2]probably won't)! \\nw[Demon Lord Dolhabach]","\\c[2]참고로! 만약 져서 재도전하러 왔을 때는 이 텍스트를 건너뛸 수 있으니까 걱정하지 말고! \\nw[마왕 도르하바]","\\c[2]附帶一提，如果妳輸了之後想要再次挑戰，這段對話可以跳過，所以妳可以稍微放心！ \\nw[魔王多爾哈巴]","\\c[2]附带一提，如果你输了之后想要再次挑战，这段对话可以跳过，所以你可以稍微放心！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]さあ、少女よ！ 貴様の力を再度、見せてもらおうではないか！！ \\nw[魔王ドルハーバ]","\\c[2]Now, young lady! Show me the extent of your strength once again! \\nw[Demon Lord Dolhabach]","\\c[2]자, 소녀여! 네 힘을 다시 보여 다오!! \\nw[마왕 도르하바]","\\c[2]來吧，少女啊！就讓我再次見識妳的力量吧！！ \\nw[魔王多爾哈巴]","\\c[2]来吧，少女啊！就让我再次见识你的力量吧！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]言っておくが・・・ 我は前より数段強いぞ！！！ \\nw[魔王ドルハーバ]","\\c[2]I warn you... I'm more powerful than last time! \\nw[Demon Lord Dolhabach]","\\c[2]미리 말해 두지만・・・ 나는 전보다 훨씬 강하다!!! \\nw[마왕 도르하바]","\\c[2]先說清楚……我可是比之前還要強上許多喔！！！ \\nw[魔王多爾哈巴]","\\c[2]先说清楚……我可是比之前还要强上许多喔！！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\{\\{魔王ドルハーバ\\|\\|\\|\\^","\\{\\{Demon Lord Dolhabach\\|\\|\\|\\^","\\{\\{마왕 도르하바\\|\\|\\|\\^","\\{\\{魔王多爾哈巴\\|\\|\\|\\^","\\{\\{魔王多尔哈巴\\|\\|\\|\\^"]);
_tr_m.push(["\\n[1]は再び 魔王城の最上階までたどり着いた！","\\n[1] reached the top of the Demonic Castle once more!","\\n[1]는 다시 마왕성의 최상층에 도착했다!","\\n[1]再次抵達了魔王城的最上層！","\\n[1]再次抵达了魔王城的最顶层！"]);
_tr_m.push(["なんと空からご褒美が降ってきた！","A reward fell from the sky!","갑자기 하늘에서 보상이 내리기 시작했다!","獎勵竟然從天而降了！","奖励竟然从天而降了！"]);
_tr_m.push(["\\i[141]アルマの石を 77個 手に入れた！","You got 77 \\i[141]Arma Stones!","\\i[141]알마의 돌 77개를 획득했다!","獲得77顆 \\i[141]魂石！","获得77颗\\i[141]魂石！"]);
_tr_m.push(["魔王ドルハーバを倒した！\\|\\|\\|\\|\\|\\|\\|\\|\\^","You defeated the Demon Lord Dolhabach!\\|\\|\\|\\|\\|\\|\\|\\|\\^","마왕 도르하바를 쓰러뜨렸다!\\|\\|\\|\\|\\|\\|\\|\\|\\^","打倒魔王多爾哈巴了！\\|\\|\\|\\|\\|\\|\\|\\|\\^","打倒魔王多尔哈巴了！\\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["\\c[2]うおおお・・・！ \\nw[魔王ドルハーバ]","\\c[2]Auuuggghhh! \\nw[Demon Lord Dolhabach]","\\c[2]으아아악・・・! \\nw[마왕 도르하바]","\\c[2]唔喔喔喔……！ \\nw[魔王多爾哈巴]","\\c[2]唔喔喔喔……！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]なん・・・だと・・・！？ \\nw[魔王ドルハーバ]","\\c[2]How could this be... \\nw[Demon Lord Dolhabach]","\\c[2]뭐, 뭐・・・야・・・!? \\nw[마왕 도르하바]","\\c[2]什……什麼……！？ \\nw[魔王多爾哈巴]","\\c[2]什……什么……！？\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]キミ強くない・・・？ \\nw[魔王ドルハーバ]","\\c[2]Aren't you a bit overpowered? You could've done this a zillion levels ago. \\nw[Demon Lord Dolhabach]","\\c[2]넌 아주 강하군・・・ \\nw[마왕 도르하바]","\\c[2]妳也太強了吧……？ \\nw[魔王多爾哈巴]","\\c[2]你也太强了吧……？\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]あっ、ちなみに第二形態とかはないぞ！ 我はこの姿が最終形態だ！ \\nw[魔王ドルハーバ]","\\c[2]By the way, I don't have a second form or anything like that. This is my final form! \\nw[Demon Lord Dolhabach]","\\c[2]앗, 참고로 제2 형태 같은 건 없어! 난 이 모습이 최종 형태야! \\nw[마왕 도르하바]","\\c[2]啊，附帶一提，我並沒有什麼第二型態喔！我的這個樣子就是最終型態了！ \\nw[魔王多爾哈巴]","\\c[2]啊，附带一提，我并没有什么第二型态喔！我的这个样子就是最终型态了！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]というわけで・・・。 \\nw[魔王ドルハーバ]","\\c[2]Which means... \\nw[Demon Lord Dolhabach]","\\c[2]결과적으로・・・ \\nw[마왕 도르하바]","\\c[2]因此…… \\nw[魔王多爾哈巴]","\\c[2]因此……\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]我の負けだあああ！！うわああああ！！ \\nw[魔王ドルハーバ]","\\c[2]I lost! Noooooo!!! \\nw[Demon Lord Dolhabach]","\\c[2]내가 졌다!! 으아아아아악!! \\nw[마왕 도르하바]","\\c[2]我輸啦啊啊啊！！嗚哇啊啊啊啊！！ \\nw[魔王多爾哈巴]","\\c[2]我输啦啊啊啊！！呜哇啊啊啊啊！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]身体が滅びていくうう！！ \\nw[魔王ドルハーバ]","\\c[2]My physical form is being destroyed! \\nw[Demon Lord Dolhabach]","\\c[2]내 몸이 사라지고 있어!! \\nw[마왕 도르하바]","\\c[2]身體漸漸消滅啦！！ \\nw[魔王多爾哈巴]","\\c[2]身体渐渐消失啦！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]熱い！！熱いいい！！ \\nw[魔王ドルハーバ]","\\c[2]Melting... I'm melting!!! \\nw[Demon Lord Dolhabach]","\\c[2]뜨거워!! 뜨겁다고!! \\nw[마왕 도르하바]","\\c[2]好燙！！好燙燙燙！！ \\nw[魔王多爾哈巴]","\\c[2]好烫！！好烫烫烫！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]あっ！うるさくてごめんな！魔王は滅びるときには 叫ばなくちゃいけない決まりになってるんだ！ \\nw[魔王ドルハーバ]","\\c[2]I apologize if this is annoying. It's part of the job description to scream and wail when a demon lord is defeated. \\nw[Demon Lord Dolhabach]","\\c[2]아! 시끄럽지? 미안! 마왕은 멸망할 때 마구 절규하기로 되어 있거든! \\nw[마왕 도르하바]","\\c[2]啊！抱歉我這麼吵！因為有規定說魔王在毀滅時必須要大聲吶喊才行！ \\nw[魔王多爾哈巴]","\\c[2]啊！抱歉我这么吵！因为有规定说魔王在毁灭时必须要大声呐喊才行！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]そうだ。折角魔王を倒したのだからな。 何か褒美を渡さなければ。 \\nw[魔王ドルハーバ]","\\c[2]That reminds me... You went through the trouble of defeating me. Let me reward you. \\nw[Demon Lord Dolhabach]","\\c[2]참, 다른 상대도 아닌 마왕을 쓰러뜨렸으니까 뭔가 보상을 줘야지. \\nw[마왕 도르하바]","\\c[2]對了，妳難得都打倒了魔王，必須要給妳一點獎勵才行。 \\nw[魔王多爾哈巴]","\\c[2]对了，难得你打倒了魔王，必须要给你一点奖励才行。\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]うむ・・・ではキミに、我の剣をやろう。 どうも私は武器よりも拳の戦いが向いているのでな。 \\nw[魔王ドルハーバ]","\\c[2]Let's see... How about I give you my sword? I'm not much of a sword guy anyway. I like to fight with my fists. \\nw[Demon Lord Dolhabach]","\\c[2]흐음・・・ 그럼 너에게 나의 검을 주마. 아무래도 나는 무기보다 맨주먹이 더 적성에 맞나 봐. \\nw[마왕 도르하바]","\\c[2]嗯……那麼，就把我的劍給妳吧。看來比起使用武器，我更適合使用拳頭戰鬥。 \\nw[魔王多爾哈巴]","\\c[2]嗯……那么，就把我的剑给你吧。看来比起使用武器，我更适合使用拳头战斗。\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]ならばキミの家に宅配で送っておくぞ。 送料はもちろん元払いでこちら負担にするから心配するな！ \\nw[魔王ドルハーバ]","\\c[2]I'll make sure it gets delivered to your house. Don't worry, I'll cover the shipping, so it won't cost you a thing! \\nw[Demon Lord Dolhabach]","\\c[2]그럼 너희 집에 택배로 보낼게. 배송비는 선불로 내가 낼 테니까 걱정하지 말고! \\nw[마왕 도르하바]","\\c[2]既然如此，我就用宅配送到妳家吧。郵寄費用就由我全額負擔，妳不用擔心！ \\nw[魔王多爾哈巴]","\\c[2]既然如此，我就用快递送到你家吧。邮寄费用就由我全额负担，你不用担心！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]少女よ。我はまた封印されて、 しばらくこの世界に魔王という存在は無くなるだろう。 \\nw[魔王ドルハーバ]","\\c[2]If I'm locked away, the world will be without a Demon Lord for a while. \\nw[Demon Lord Dolhabach]","\\c[2]소녀여, 내가 다시 봉인되면 한동안 이 세계에 마왕이란 존재는 사라질 것이다. \\nw[마왕 도르하바]","\\c[2]少女啊。我再次遭到封印後，這個世界應該會暫時沒有魔王存在吧。 \\nw[魔王多爾哈巴]","\\c[2]少女啊。我再次遭到封印后，这个世界应该会暂时没有魔王存在吧。\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]魔王がいない平和な世界は、 人類の誰もが望むすばらしい世界だ！ \\nw[魔王ドルハーバ]","\\c[2]A peaceful world without a Demon Lord... It's all humankind could ever wish for! \\nw[Demon Lord Dolhabach]","\\c[2]마왕이 없는 평화로운 세계는 모든 인류가 바라는 훌륭한 세계다! \\nw[마왕 도르하바]","\\c[2]沒有魔王的和平世界，是每個人類都期盼的美好世界！ \\nw[魔王多爾哈巴]","\\c[2]没有魔王的和平世界，是每个人类都期盼的美好世界！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]しかし・・・先程述べたように、魔王の消えた平和な世界では 人類は平和ボケし堕落してしまう可能性がある！！ \\nw[魔王ドルハーバ]","\\c[2]However! As I explained earlier, a peaceful world without a Demon Lord will cause the people to become complacent! \\nw[Demon Lord Dolhabach]","\\c[2]그러나・・・ 앞서 말했다시피 마왕이 사라진 평화로운 세계에서 인류는 평화에 도취되어 타락할 가능성이 있다!! \\nw[마왕 도르하바]","\\c[2]然而……如同我剛剛所說的，魔王消失後的和平世界，有可能會讓人類墮落，沉溺於和平之中！！ \\nw[魔王多爾哈巴]","\\c[2]然而……如同我刚刚所说的，魔王消失后的和平世界，有可能会让人类堕落，沉溺于和平之中！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]それを防ぐのがキミだ！キミは魔王を倒した英雄として 人類を引っ張って行かなくてはならない！ \\nw[魔王ドルハーバ]","\\c[2]The only one who can prevent that is you! You've got to become a moral compass for the people! \\nw[Demon Lord Dolhabach]","\\c[2]그걸 막을 사람이 바로 너다! 너는 마왕을 쓰러뜨린 영웅으로서 인류를 이끌어야 한다! \\nw[마왕 도르하바]","\\c[2]為了防止這一點，就必須要靠妳！妳必須以「打倒魔王的英雄」這個身份，引導人類前進！ \\nw[魔王多爾哈巴]","\\c[2]为了防止这一点，就必须要靠你！你必须以「打倒魔王的英雄」这个身份，引导人类前进！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]キミが正しく導いてくれたとき・・・ 世界に「真の平和」は訪れる！！ \\nw[魔王ドルハーバ]","\\c[2]When you have led them down the righteous path is when the world will truly become peaceful! \\nw[Demon Lord Dolhabach]","\\c[2]네가 올바른 길로 이끌었을 때・・・ 세계에 「진정한 평화」가 찾아올 것이다!! \\nw[마왕 도르하바]","\\c[2]當妳引導人類朝正確方向前進之時……這個世界才會獲得「真正的和平」！！ \\nw[魔王多爾哈巴]","\\c[2]当你引导人类朝正确方向前进之时……这个世界才会获得「真正的和平」！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]キミにまかせるぞ！！この世界を・・・守ってくれよな！！ \\nw[魔王ドルハーバ]","\\c[2]I'm entrusting this duty to you! Guide and protect this world! \\nw[Demon Lord Dolhabach]","\\c[2]너를 믿으마!! 이 세계를・・・ 지켜 다오!! \\nw[마왕 도르하바]","\\c[2]一切就交給妳了！！妳要……好好保護這個世界！！ \\nw[魔王多爾哈巴]","\\c[2]一切就交给你了！！你要……好好保护这个世界！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["\\c[2]うおおおおお！！！ \\nw[魔王ドルハーバ]","\\c[2]Graaagh!!! \\nw[Demon Lord Dolhabach]","\\c[2]으아아아아악!!! \\nw[마왕 도르하바]","\\c[2]唔喔喔喔喔喔！！！ \\nw[魔王多爾哈巴]","\\c[2]唔喔喔喔喔喔！！！\\nw[魔王多尔哈巴]"]);
_tr_m.push(["かくして、世界を脅かす魔王を倒し 魔王から真の平和を託された\\n[1]。","Thus, \\n[1] defeated the menacing Demon Lord and was tasked with maintaining peace throughout the land.","이리하여 세계를 위협하는 마왕을 쓰러뜨리고, 마왕으로부터 진정한 평화를 책임지게 된 \\n[1].","就這樣，\\n[1]打倒了威脅世界的魔王，並受魔王所託要達成「真正的和平」。","就这样，\\n[1]打倒了威胁世界的魔王，并受魔王所托要达成「真正的和平」。"]);
_tr_m.push(["「とりあえず、おなかが空いたので家に帰ろう。」 \\n[1]はそう思った。","\\n[1] thought, \"I'm hungry. I just want to go home.\"","「배가 고프니까 일단 집에 가자.」 \\n[1]는 그렇게 생각했다.","\\n[1]心想：「總之我餓了，就先回家吧」。","\\n[1]心想：「总之，我饿了，就先回家吧。」"]);
_tr_m.push(["魔王城内の武器を すべて拾えるようになった！","All weapons in the Demonic Castle can now be picked up!","마왕성 안의 무기를 전부 주울 수 있게 되었다!","可以拾取魔王城內所有的武器了！","可以拾取魔王城内所有的武器了！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["――少し前のこと。","A short while later...","지금으로부터 조금 전.","──這是一段時間前的事。","──这是之前发生的事。"]);
_tr_m.push(["お姉ちゃん！ \\nw[ぽん美]","Hey, Sis! \\nw[Ponmi]","언니! \\nw[폰미]","姐姐！ \\nw[器美]","姐姐！\\nw[器美]"]);
_tr_m.push(["この間、ウワサで聞いたんだけど・・・。 井戸の底にはいいものが落ちているらしいよ！ \\nw[ぽん美]","I heard an interesting rumor the other day. Apparently there are some valuable items at the bottom of the well! \\nw[Ponmi]","요전에 소문으로 들었는데・・・ 우물 밑바닥에 좋은 물건이 떨어져 있대! \\nw[폰미]","我最近聽說……有個好東西掉在水井裡呢！ \\nw[器美]","我最近听说……有个好东西掉在水井里了呢！\\nw[器美]"]);
_tr_m.push(["いいなー。私も小さめのメダルを集めて、王から ◯◯キンの盾をもらいたいな～！ \\nw[ぽん美]","How cool is that? I wish I could collect small tokens and earn a Hoarder's Shield from the king himself! \\nw[Ponmi]","멋지다~ 나도 작은 메달을 모아서 왕한테 ◯금의 방패를 받고 싶은데~! \\nw[폰미]","真好～我也想要收集小小的徽章，從國王那邊拿到◯◯王之盾啊～！ \\nw[器美]","真好～我也想要收集小小的徽章，从国王那边拿到◯◯王之盾啊～！\\nw[器美]"]);
_tr_m.push(["というわけで、お姉ちゃん！村の井戸の中を見てきてよ！ \\nw[ぽん美]","So that's why you should totally go check out the well! \\nw[Ponmi]","그러니까 언니! 마을의 우물 안을 보고 와! \\nw[폰미]","所以……姐姐！妳去村子的水井裡看看吧！ \\nw[器美]","所以……姐姐！你去村子的水井里看看吧！\\nw[器美]"]);
_tr_m.push(["\\n[1]はぽん美に、そう頼まれた。 しかし \\n[1]は反対であった。","And so, Ponmi made a request of \\n[1]. However, \\n[1] did not agree...","\\n[1]는 폰미에게 부탁을 받았다. 하지만 \\n[1]의 생각은 달랐다.","器美開口拜託了\\n[1]。然而 \\n[1]卻抱持著反對的意見。","器美开口拜托了\\n[1]。然而\\n[1]却抱持着反对的意见。"]);
_tr_m.push(["「交換するならき◯きのつるぎやろ！！」 と、\\n[1]は内心思っていた。","On the inside, \\n[1] was thinking, \"A Stashing Sword would be so much better!\"","「기왕이면 ◯적의 검이 더 낫지!!」라고 \\n[1]는 내심 생각했다.","\\n[1]心想：「真要交換的話，應該是換奇◯之劍吧！！」","\\n[1]心想：「真要交换的话，应该是换奇◯之剑吧！！」"]);
_tr_m.push(["というわけで、期待を胸に井戸に飛び込んだ \\n[1]。","Thus, \\n[1] jumped into the well with high hopes.","그리하여 기대를 품고 우물에 뛰어든 \\n[1].","因此， \\n[1]抱著期待跳進了水井裡。","因此，\\n[1]抱着期待跳进了水井里。"]);
_tr_m.push(["しかし、その先で\\n[1]の見た光景は・・・。","However, what \\n[1] saw there...","하지만 그 끝에서 \\n[1]가 본 풍경은・・・","然而，\\n[1]在那裡看到的光景卻是……","然而，\\n[1]在那里看到的光景却是……"]);
_tr_m.push(["青空。雲一つない青空であった。","...was a clear, blue sky.","파란 하늘. 구름 한 점 없는 파란 하늘이었다.","藍天。萬里無雲的一片藍天。","蓝天。万里无云的一片蓝天。"]);
_tr_m.push(["気がつくと、うぇ子は パラシュートで宙を飛んでいた・・・。","Before she knew it, Weaco was soaring through the sky with a parachute.","정신을 차려 보니 웨코는 낙하산을 매고 하늘을 날고 있었다・・・","回過神來，武子才發現自己正披著降落傘，在空中飄盪……","回过神来，武子才发现自己正披着降落伞，在空中飘荡……"]);
_tr_m.push(["\\n[1]は思った。","\\n[1] thought to herself...","\\n[1]는 생각했다.","\\n[1]心想：","\\n[1]心想："]);
_tr_m.push(["「なんやこれ・・・。 　思ってたのとちゃう・・・。」","\"What's going on? This isn't what I was expecting at all!\"","「이건 뭐지・・・ 내가 생각했던 것과 달라・・・」","「這是怎麼回事……跟我想的不一樣……」","「这是怎么回事……跟我想的不一样……」"]);
_tr_m.push(["「もしかしてこの後100人ぐらいで同じ島に降り立って、 　ドン勝を食うために乱闘を始めるのだろうか・・・」と。","\"Don't tell me 100 of us are going to land on an island somewhere and duke it out in a battle royale...\"","「혹시 이러다가 100명 정도가 같은 섬에 착륙해서 저녁으로 치킨을 먹기 위해 난투를 벌이게 되는 걸까・・・」","「該不會等等會有100人左右降落在同一個島上，為了吃雞肉而展開大逃殺吧……」。","「该不会等下有100人左右降落在同一个岛上，为了吃鸡肉而展开大逃杀吧……」。"]);
_tr_m.push(["\\n[1]は気持ちを切り替えて、 来たるべき戦いに備え精神統一を始めた・・・！","\\n[1] prepared her body and soul for the upcoming battle...","\\n[1]는 마음을 다잡고 앞으로 닥칠 싸움에 대비해 정신을 가다듬었다・・・!","\\n[1]轉換了心情，為了即將來臨的戰鬥做準備，開始冥想集中精神……！","\\n[1]转换了心情，为了即将来临的战斗做准备，开始冥想集中精神……！"]);
_tr_m.push(["そして、風に流され、着陸したその地は・・・。","Carried by the wind, she landed on the ground...","바람에 몸을 맡긴 채 착륙한 그곳은・・・","就這樣，她在風中飄盪，最後降落的地方是……","就这样，她在风中飘荡，最后降落的地方是……"]);
_tr_m.push(["謎の小島だった。 \\n[1]の他には誰もいない・・・。","It's a mysterious little island that appears to be deserted except for \\n[1]...","이상한 새였다. \\n[1] 말고는 아무도 없다・・・","一個神秘的小島。除了 \\n[1]以外，沒有其他人在……","一个神秘的小岛。除了\\n[1]以外，没有其他人在……"]);
_tr_m.push(["しかし、血に飢えた魔物の気配がムンムンとする。","And the numerous bloodthirsty monsters roaming the field, of course.","하지만 피에 굶주린 마물의 기척이 주위에 가득했다.","然而，這裡卻充滿了渴求鮮血的魔物氣息。","然而，这里却充满了渴求鲜血的魔物气息。"]);
_tr_m.push(["\\n[1]は思った。","\\n[1] thought to herself...","\\n[1]는 생각했다.","\\n[1]心想：","\\n[1]心想："]);
_tr_m.push(["普通に帰りてぇ、と・・・。","She just wanted to go home.","그냥 집에 가고 싶다・・・","我想回家……","我想回家……"]);
_tr_m.push(["魔物たちの島で、がんばって生き延びよう！","Fight to survive on this island of monsters!","마물들의 섬에서 어떻게든 살아남자!","努力在魔物之島上活下去吧！","努力在魔物之岛上活下去吧！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["じゃあ、応援してあげるね！ \\nw[ぽん美]","Okay, I'll cheer you on! \\nw[Ponmi]","내가 응원해 줄게! \\nw[폰미]","那我來幫妳加油喔！ \\nw[器美]","那我來幫妳加油喔！\\nw[器美]"]);
_tr_m.push(["ふれー！ふれー！お姉ちゃん！ \\nw[ぽん美]","Hooray! Hooray! Hip hip hooray! \\nw[Ponmi]","잘한다! 잘한다! 우리 언니! \\nw[폰미]","加油！加油！姐姐加油！ \\nw[器美]","加油！加油！姐姐加油！\\nw[器美]"]);
_tr_m.push(["勝てるぞ！勝てるぞ！お姉ちゃん！ \\nw[ぽん美]","Don't worry! You've got this, Sis! \\nw[Ponmi]","힘내라~ 힘! 힘내라~ 힘! \\nw[폰미]","會贏！會贏！姐姐會贏！ \\nw[器美]","会赢！会赢！姐姐会赢！\\nw[器美]"]);
_tr_m.push(["・・・ふぅ！これで応援はおしまい！ \\nw[ぽん美]","Okay! That's enough of that! \\nw[Ponmi]","휴~! 이걸로 응원 끝! \\nw[폰미]","……呼！這樣加油就結束了！ \\nw[器美]","……呼！这样加油就结束了！\\nw[器美]"]);
_tr_m.push(["\\n[1]のもらえる経験値が増加した！ 経験値 \\v[3] → \\c[14]\\v[4]","The EXP that \\n[1] can receive has increased!<br>EXP Bonus: \\v[3] → \\c[14]\\v[4]","\\n[1]의 획득 가능 경험치가 증가했다!<br>경험치 \\v[3] → \\c[14]\\v[4]","\\n[1]可獲得的經驗值增加了！<br>經驗值 \\v[3] → \\c[14]\\v[4]","\\n[1]可获得的经验值增加了！<br>经验值 \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["じゃあ、\\v[3]を拡張しとくね！ \\nw[ぽん美]","Okay! I'll expand the \\v[3]! \\nw[Ponmi]","그럼 \\v[3](을)를 확장해 둘게! \\nw[폰미]","那我去擴增\\v[3]囉！ \\nw[器美]","那我去扩增\\v[3]喽！\\nw[器美]"]);
_tr_m.push(["しといたよー。 \\nw[ぽん美]","Done and done. \\nw[Ponmi]","해 놨어~ \\nw[폰미]","擴增好了喔～ \\nw[器美]","扩增好了喔～\\nw[器美]"]);
_tr_m.push(["武器庫が拡張された！ 武器庫の容量 \\v[3] → \\c[14]\\v[4]","Your Armory has been expanded! Armory Capacity: \\v[3] → \\c[14]\\v[4]","무기고가 확장되었다! 무기고 용량 \\v[3] → \\c[14]\\v[4]","武器庫擴增了！武器庫容量 \\v[3] → \\c[14]\\v[4]","武器库扩增了！武器库容量 \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["商品棚がひとつ増えた！ 商品棚の数 \\v[3] → \\c[14]\\v[4]","Another shop table has been added!<br>Shop Tables: \\v[3] → \\c[14]\\v[4]","상품 선반이 1개 증가했다!<br>상품 선반의 수 \\v[3] → \\c[14]\\v[4]","商品架增加一個了！<br>商品架數量 \\v[3] → \\c[14]\\v[4]","商品架增加一个了！<br>商品架数量 \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["じゃあ、ちょっと呼び込みしてくるね！ \\nw[ぽん美]","Alright! I'll see what I can do! \\nw[Ponmi]","손님 불러 올 테니까 기다려 봐! \\nw[폰미]","那我去招攬一下客人囉！ \\nw[器美]","那我去招揽一下客人喽！\\nw[器美]"]);
_tr_m.push(["らっしゃいらっしゃい！（ぱんぱん） \\nw[ぽん美]","Welcome! *clap clap* \\nw[Ponmi]","골라~! 골라~!(짝짝) \\nw[폰미]","歡迎歡迎！（啪啪） \\nw[器美]","欢迎欢迎！（啪啪）\\nw[器美]"]);
_tr_m.push(["安いよ安いよ！（ぱんぱん） \\nw[ぽん美]","Bargains abound! Deals galore! *clap clap* \\nw[Ponmi]","쌉니다~ 싸요~!!(짝짝) \\nw[폰미]","便宜賣喔，便宜賣喔！（啪啪） \\nw[器美]","便宜卖喔，便宜卖喔！（啪啪）\\nw[器美]"]);
_tr_m.push(["買わなきゃそんそん！（ぱんぱん） \\nw[ぽん美]","Don't miss out! *clap clap* \\nw[Ponmi]","안 사면 손해!(짝짝) \\nw[폰미]","不買可就吃大虧囉！（啪啪） \\nw[器美]","不买可就吃大亏喽！（啪啪）\\nw[器美]"]);
_tr_m.push(["ふぅ！呼び込みしてきたよ！ \\nw[ぽん美]","Phew! That was hard work! \\nw[Ponmi]","휴~! 손님 모아 왔어! \\nw[폰미]","呼！我去招攬客人回來囉！ \\nw[器美]","呼！我去招揽客人回来喽！\\nw[器美]"]);
_tr_m.push(["武器の売れる時間が早まった！ 売れる時間 \\v[3] → \\c[14]\\v[4]","Your weapons will sell even faster now!<br>Time Required: \\v[3] → \\c[14]\\v[4]","무기가 팔리는 시간이 짧아졌다!<br>팔리는 시간 \\v[3] → \\c[14]\\v[4]","武器賣出的時間變快了！<br>賣出時間 \\v[3] → \\c[14]\\v[4]","武器卖出的时间变快了！<br>卖出时间 \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["じゃあ、がんばって勉強するね！ \\nw[ぽん美]","Okay, I'll hit the books! \\nw[Ponmi]","자~ 싸게 줄 때 가져가세요~! \\nw[폰미]","那我要努力學習囉！ \\nw[器美]","那我要努力学习喽！\\nw[器美]"]);
_tr_m.push(["カリカリ・・・カリカリ・・・。 \\nw[ぽん美]","Hmm... Okay... \\nw[Ponmi]","부스럭부스럭, 주섬주섬. \\nw[폰미]","（寫字聲）…… \\nw[器美]","（写字声）……\\nw[器美]"]);
_tr_m.push(["ふむふむ！なるほど～！ \\nw[ぽん美]","Ah, I see! I see! \\nw[Ponmi]","흠흠! 아, 이거요~!? \\nw[폰미]","嗯嗯！原來如此～！ \\nw[器美]","嗯嗯！原来如此～！\\nw[器美]"]);
_tr_m.push(["なんだか商売術がすっごく身についた気がするよ！ \\nw[ぽん美]","I think I'm finally getting the hang of this sales stuff! \\nw[Ponmi]","나 장사 잘하지? 그렇지!? \\nw[폰미]","我覺得好像學到了很多生意經！ \\nw[器美]","我感觉好像学到了很多生意经！\\nw[器美]"]);
_tr_m.push(["武器の売れる値段が増加した！ 売れる値段 \\v[3] → \\c[14]\\v[4]","Your weapons will sell for even more now!<br>Selling Price: \\v[3] → \\c[14]\\v[4]","무기가 팔리는 가격이 상승했다!<br>팔리는 가격 \\v[3] → \\c[14]\\v[4]","武器販售的價格上漲了！<br>販售價格 \\v[3] → \\c[14]\\v[4]","武器贩售的价格上涨了！<br>贩售价格\\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["私からのプレゼントだよ。 お姉ちゃん、受け取って！ \\nw[ぽん美]","I got you a present. Here! Take it! \\nw[Ponmi]","내가 주는 선물이야. 언니, 받아줘! \\nw[폰미]","這是我送的禮物，姐姐妳收下吧！ \\nw[器美]","这是我送的礼物，姐姐你收下吧！\\nw[器美]"]);
_tr_m.push(["この前森にお散歩に出かけたら、落ちてたの。 なんか面白そうだから拾って来ちゃった！ \\nw[ぽん美]","I found it when I went for a walk in the forest. I thought it was interesting enough to bring back with me! \\nw[Ponmi]","얼마 전에 숲으로 산책하러 갔더니 떨어져 있더라구. 재미있을 것 같아서 주워 왔어! \\nw[폰미]","之前我去森林裡散步時，它掉在地上。我覺得好像很有趣，就撿回來了！ \\nw[器美]","之前我去森林里散步时，它掉在地上。我觉得好像很有趣，就捡回来了！\\nw[器美]"]);
_tr_m.push(["私の宝物にしようと思ってたけど・・・。 お姉ちゃんにあげるね！ \\nw[ぽん美]","At first, I thought I'd keep it for myself... But I'd rather you have it! \\nw[Ponmi]","내 보물로 삼으려고 했는데・・・ 언니한테 줄게! \\nw[폰미]","本來是想把它當作我的寶物……但我要送給姐姐！ \\nw[器美]","本来是想把它当作我的宝物……但我要送给姐姐！\\nw[器美]"]);
_tr_m.push(["じゃあ、家の前に立てかけておくね。 \\nw[ぽん美]","Well then, I'll put it out in front of the house. \\nw[Ponmi]","그럼, 집 앞에 세워 놓을게. \\nw[폰미]","那我放在家門口囉。 \\nw[器美]","那我放在家门口咯。\\nw[器美]"]);
_tr_m.push(["なんか不思議な力があるみたいだけど・・・ お姉ちゃんなら使いこなせるんじゃない？ \\nw[ぽん美]","Some sort of strange power seems to be emanating from it... Maybe you can harness it somehow? \\nw[Ponmi]","뭔가 신비로운 힘이 있는 것 같은데・・・ 언니라면 잘 사용할 수 있지 않을까? \\nw[폰미]","感覺它好像有不可思議的力量……姐姐妳應該能活用它吧？ \\nw[器美]","感觉它好像有不可思议的力量……姐姐你应该能活用它吧？\\nw[器美]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["昨日お姉ちゃんのために徹夜で作った、 かわいいポーチだよ！ \\nw[ぽん美]","I stayed up all night making this for you. Isn't it the cutest pouch you've ever seen⁈ \\nw[Ponmi]","어제 언니를 위해 밤새워 만든 귀여운 파우치야! \\nw[폰미]","這是我昨天為了姐姐熬夜製作的可愛袋子喔！ \\nw[器美]","这是我昨天为了姐姐熬夜制作的可爱袋子喔！\\nw[器美]"]);
_tr_m.push(["薬草ポーチを手に入れた！ 薬草の最大所持数が 10 → \\c[14]20\\c[0] にアップした！","You got a medicinal herb pouch! Max Herbs: 10 → \\c[14]20\\c[0]!","약초 파우치를 획득했다! 약초의 최대 소지 수가 10 → \\c[14]20\\c[0]으로 증가했다!","獲得藥草袋！藥草最大持有數從10提升至\\c[14]20\\c[0]了！","获得药草袋！药草最大持有数从10提升至\\c[14]20\\c[0]了！"]);
_tr_m.push(["そういえば、\\c[14]\\i[261]薬草が５個あれば 死んだときに復活することができる\\c[0]らしいよ！ \\nw[ぽん美]","Oh, that reminds me! If you have \\c[14]five \\i[261]herbs, you can apparently resurrect yourself\\c[0] if you die! \\nw[Ponmi]","아 참, \\c[14]\\i[261]약초가 5개 있으면 죽었을 때 부활할 수도\\c[0] 있대! \\nw[폰미]","對了，聽說\\c[14]只要有五棵\\i[261]藥草，就能在死亡時復活\\c[0]喔！ \\nw[器美]","对了，听说\\c[14]只要有五棵\\i[261]药草，就能在死亡时复活\\c[0]喔！\\nw[器美]"]);
_tr_m.push(["今あげたポーチに\\i[261]薬草をたくさん持っておけば 何かあっても安心だね。 \\nw[ぽん美]","Things'll be easier if you store a bunch of \\i[261]herbs in the pouch I gave you. \\nw[Ponmi]","방금 준 파우치에 \\i[261]약초를 많이 넣어 두면 무슨 일이 있어도 안심할 수 있을 거야. \\nw[폰미]","只要在剛才送妳的袋子裡裝上很多\\i[261]藥草，不管發生什麼事都可以放心了呢。 \\nw[器美]","只要在刚才送你的袋子里装上很多\\i[261]药草，不管发生什么事都可以放心了呢。\\nw[器美]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["突然だけど・・・いつもありがとう！ \\nw[ぽん美]","I know this is random, but... Thank you so much! \\nw[Ponmi]","뜬금없지만・・・ 항상 고마워! \\nw[폰미]","雖然很突然……謝謝妳一直以來的照顧！ \\nw[器美]","虽然很突然……谢谢你一直以来的照顾！\\nw[器美]"]);
_tr_m.push(["えへへへ。 お姉ちゃんにはいつもお世話になってるからね。 \\nw[ぽん美]","Hehehe. I can't thank you enough for all you've done. \\nw[Ponmi]","에헤헤헤. 항상 언니한테는 신세만 지고 있으니까. \\nw[폰미]","耶嘿嘿嘿，因為姐姐總是在照顧我嘛。 \\nw[器美]","耶嘿嘿嘿，因为姐姐总是在照顾我嘛。\\nw[器美]"]);
_tr_m.push(["私ね、一生懸命考えたんだ。 お姉ちゃんが本当に欲しい物ってなんなのか・・・。 \\nw[ぽん美]","I really thought long and hard about this. I wanted to come up with the perfect gift for you... \\nw[Ponmi]","내가 정말 열심히 생각했거든. 언니가 정말 가지고 싶어하는 건 무언지 말이야・・・ \\nw[폰미]","我呀，拼命想了很多喔。思考姐姐真正想要的東西會是什麼…… \\nw[器美]","我呀，拼命想了很多喔。思考姐姐真正想要的东西会是什么……\\nw[器美]"]);
_tr_m.push(["それで、やっぱりお姉ちゃんは・・・ 「虫捕り」が好きなんじゃないかって、そう思ったの！ \\nw[ぽん美]","And that's why...I thought you might like catching bugs! \\nw[Ponmi]","그랬더니 역시 언니는・・・ 「곤충 채집」을 좋아하지 않을까 생각했던 거지! \\nw[폰미]","然後我就想到了，姐姐妳……應該會喜歡「抓昆蟲」吧！ \\nw[器美]","然后我就想到了，姐姐你……应该会喜欢「抓昆虫」吧！\\nw[器美]"]);
_tr_m.push(["根拠はないよ！お姉ちゃんが虫、捕りたそうにしてるの 見たことないし、興味を示してるのも見たことがない。 \\nw[ぽん美]","I'm not really sure why! You just seem the type. It's not like I've ever seen anything to confirm my suspicions... \\nw[Ponmi]","근거는 없어! 언니가 곤충 잡는 모습을 본 적도 없고 곤충에 흥미를 느끼는 걸 본 적도 없어 \\nw[폰미]","這個想法沒有任何根據喔！我沒有看過姐姐妳表現出想要抓昆蟲的樣子，也沒看過妳對這件事感興趣。 \\nw[器美]","这个想法没有任何根据喔！我没有看过姐姐你表现出想要抓昆虫的样子，也没看过你对这件事感兴趣。\\nw[器美]"]);
_tr_m.push(["けど、やっぱ姉妹だから、本心がわかるというか。 テレパシー的なのというか。 \\nw[ぽん美]","Sisters just...know...right? It's like telepathy. \\nw[Ponmi]","하지만 역시 자매니까 속마음을 알겠더라구. 이게 텔레파시 아닐까? \\nw[폰미]","但我們畢竟是姐妹，所以能明白彼此的真實想法，或者應該說像是心電感應嗎？ \\nw[器美]","但我们毕竟是姐妹，所以能明白彼此的真实想法，或者应该说像是心电感应吗？\\nw[器美]"]);
_tr_m.push(["「ビビッ」と来ちゃったんだよね。 なんとなくだけど。 \\nw[ぽん美]","Let me guess, this is a bit of a surprise? Just a hunch is all. \\nw[Ponmi]","삐빗 하고 오는 거 말이야. 그런 느낌 알지? \\nw[폰미]","就是會「嗶」一聲地接收到妳的想法呢。雖然只是一種感覺啦。 \\nw[器美]","就是会「哔」一声地接收到你的想法呢。虽然只是一种感觉啦。\\nw[器美]"]);
_tr_m.push(["というわけで、３徹して作ってみたよ。 私お手製の・・・虫取りアミをね！ \\nw[ぽん美]","Anyway, I worked hard making this for you. It's my...handmade Insect Net! \\nw[Ponmi]","그래서 3일간 밤 새워 만들어 봤어. 나만의... 곤충 채집망을! \\nw[폰미]","因此我熬了三個晚上，試著做出來了喔。這是我親手製作的……捕蟲網！ \\nw[器美]","因此我熬了三个晚上，试着做出来了喔。这是我亲手制作的……捕虫网！\\nw[器美]"]);
_tr_m.push(["じゃあ、家の前に立てかけておくね。 \\nw[ぽん美]","Well then, I'll put it out in front of the house. \\nw[Ponmi]","그럼, 집 앞에 세워 놓을게. \\nw[폰미]","那我放在家門口囉。 \\nw[器美]","那我放在家门口咯。\\nw[器美]"]);
_tr_m.push(["これで思う存分、虫捕りにはげんでよね！ \\nw[ぽん美]","Now you can catch all the bugs you want! \\nw[Ponmi]","이걸로 어떤 곤충이든 잡을 수 있을 거야! \\nw[폰미]","妳就盡情地努力用它去抓昆蟲吧！ \\nw[器美]","你就尽情地努力用它去抓昆虫吧！\\nw[器美]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["そのとき！\\n[1]の頭の中に、何かが聞こえてきた！","Suddenly! \\n[1] heard a voice from within her head! \\nw[bears]","그때였다! \\n[1]의 머릿속에서 누군가의 목소리가 들렸다!","就在這時！\\n[1]的腦海裡傳來了一些聲音！ \\nw[Bears]","就在这时！\\n[1]的脑海里传来了一些声音！"]);
_tr_m.push(["・・・私たちは、クマ４兄弟。 あなたの心の中に話しかけています。 \\nw[クマたち]","We're a brethren of four bears. You can hear our voice directly within your heart. \\nw[bears]","・・・우리는 곰 4형제. 당신의 마음속에서 말을 걸고 있습니다. \\nw[곰들]","……我們是熊四兄弟。我們直接把聲音傳到了妳的腦海裡。 \\nw[熊]","……我们是熊四兄弟。我们直接把声音传到了你的脑海里。 \\nw[熊]"]);
_tr_m.push(["あなたはとうとう、この世界のすべてのクマに 認められましたね。 \\nw[クマたち]","The bears of this world appear to have accepted you. \\nw[bears]","당신은 드디어 이 세계의 모든 곰에게 인정을 받았군요. \\nw[곰들]","這世界上所有的熊終於全都認同妳了。 \\nw[熊]","这世界上所有的熊终于全都认同你了。 \\nw[熊]"]);
_tr_m.push(["・・・これはもうとてつもなくすごい。 国民栄誉賞レベルですごいことだ。 \\nw[クマたち]","This is an amazing achievement. You should be proud of what you have accomplished. \\nw[bears]","・・・이건 정말 굉장한 위업이다. 국민훈장을 받을 만한 일이지. \\nw[곰들]","……這實在是太厲害了。厲害到可以拿國民榮譽獎了。 \\nw[熊]","……这实在是太厉害了。厉害到可以拿国民荣誉奖了。 \\nw[熊]"]);
_tr_m.push(["おう、お前ならやってくれると信じていたぞ！ \\nw[クマたち]","We knew you wouldn't let us down! \\nw[bears]","오~ 넌 꼭 해낼 줄 알았어! \\nw[곰들]","是呀，我一直相信妳能做到的！ \\nw[熊]","是呀，我一直相信你能做到的！ \\nw[熊]"]);
_tr_m.push(["そんなすごい君に、クマたちからプレゼントを送るよ！ \\nw[クマたち]","In recognition of your efforts, we have prepared a gift for you! \\nw[bears]","그런 널 위해 곰들이 선물을 주기로 했어! \\nw[곰들]","妳這麼厲害，我們這些熊要送妳禮物喔！ \\nw[熊]","你这么厉害，我们这些熊要送你礼物喔！ \\nw[熊]"]);
_tr_m.push(["では、あなたの家に送っておきます。 \\nw[クマたち]","It will be delivered to your home. \\nw[bears]","그럼 당신 집으로 보내 놓겠습니다. \\nw[곰들]","那麼，禮物就先送到妳家了。 \\nw[熊]","那么，礼物就先送到你家了。 \\nw[熊]"]);
_tr_m.push(["それはクマたちの力を込めたものすごい剣よ！ \\nw[クマたち]","It's an incredible sword imbued with the power of us bears. \\nw[bears]","그것은 곰의 힘을 담은 굉장한 검이야! \\nw[곰들]","那是一把隱藏著熊之力的超強寶劍喔！ \\nw[熊]","那是一把隐藏着熊之力的超强宝剑喔！ \\nw[熊]"]);
_tr_m.push(["クマたちの心はいつでも\\n[1]と共にあるんだ！！ \\nw[クマたち]","Our hearts will always be on your side, \\n[1]! \\nw[bears]","곰들의 마음은 항상 \\n[1] 너와 함께 있어!! \\nw[곰들]","熊的精神會永遠與\\n[1]同在！！ \\nw[熊]","熊的精神会永远与\\n[1]同在！！ \\nw[熊]"]);
_tr_m.push(["・・・ふっ、これでお別れだな、\\n[1]。 \\nw[クマたち]","Hmm... I guess this is farewell, \\n[1]. \\nw[bears]","・・・흐음, 이제 작별해야겠군 \\n[1]. \\nw[곰들]","……呵，那麼就要告別了呢，\\n[1]。 \\nw[熊]","……呵，那么就要告别了呢，\\n[1]。 \\nw[熊]"]);
_tr_m.push(["\\n[1]に・・・クマのご加護があらんことを！ \\nw[クマたち]","May the protection of us bears always be with you! \\nw[bears]","\\n[1]에게・・・ 곰의 가호가 함께하길! \\nw[곰들]","願熊……保佑\\n[1]！ \\nw[熊]","愿熊……保佑\\n[1]！ \\nw[熊]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["\\>武器「\\i[\\v[3]]\\v[4]」を拾った！ \\v[5]","\\>You picked up the \\i[\\v[3]]\\v[4]!<br>\\v[5]","\\>무기 「\\i[\\v[3]]\\v[4]」 획득!<br>\\v[5]","\\>撿到武器「\\i[\\v[3]]\\v[4]」了！<br>\\v[5]","\\>捡到武器「\\i[\\v[3]]\\v[4]」了！<br>\\v[5]"]);
_tr_m.push(["武器「\\i[\\v[3]]\\v[4]」を拾った！ \\v[5]","You picked up the \\i[\\v[3]]\\v[4]!<br>\\v[5]","무기 「\\i[\\v[3]]\\v[4]」 획득!<br>\\v[5]","撿到武器「\\i[\\v[3]]\\v[4]」了！<br>\\v[5]","捡到武器「\\i[\\v[3]]\\v[4]」了！<br>\\v[5]"]);
_tr_m.push(["しかし武器庫がいっぱいでこれ以上拾えなかった。","You can't pick up any more weapons because your Armory is full.","하지만 무기고가 가득 차서 획득하지 못했다.","但是武器庫已經滿了，無法撿起更多武器。","但是武器库已经满了，无法捡起更多武器。"]);
_tr_m.push(["しかし持ち物がいっぱいでこれ以上拾えなかった。","You can't pick up any more weapons because you are at capacity.","하지만 소지품이 가득 차서 획득하지 못했다.","但是攜帶物太多了，無法撿起更多武器。","但是攜帶物太多了，無法撿起更多武器。"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["レベルアップボーナスを一つ選ぼう！\\! ","Choose your level-up bonus!\\! ","레벨업 보너스를 1개 선택하세요!\\! ","選擇一項升級獎勵吧！\\! ","选择一项升级奖励吧！\\! "]);
_tr_m.push(["\\n[1]の最大ＨＰが上がった！ 最大ＨＰ \\v[3] → \\c[14]\\v[4]","\\n[1]\'s Max HP increased!<br>Max HP: \\v[3] → \\c[14]\\v[4]","\\n[1]의 최대 HP가 상승했다!<br>최대 HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP上升了！<br>最大HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP提升了！<br>最大HP \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["\\n[1]の攻撃力が上がった！ \\i[469]上昇値 \\v[3] → \\c[14]\\v[4]","\\n[1]\'s ATK increased!<br>\\i[469] increase: \\v[3] → \\c[14]\\v[4]","\\n[1]의 공격력이 상승했다!<br>\\i[469]공격력 \\v[3] → \\c[14]\\v[4]","\\n[1]的攻擊力上升了！<br>\\i[469]上升值 \\v[3] → \\c[14]\\v[4]","\\n[1]的攻擊力上升了！<br>\\i[469]上升值 \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["武器の\\i[470]耐久値減少確率が下がった！ \\i[470]減少確率 \\v[3] → \\c[14]\\v[4]","The chances of weapon \\i[470]durability loss have been reduced!<br>Chance of \\i[470]durability loss: \\v[3] → \\c[14]\\v[4]","무기의 \\i[470] 내구도 감소 확률이 감소했다!<br>\\i[470]감소 확률 \\v[3] → \\c[14]\\v[4]","武器的\\i[470]耐久值減少機率下降了！<br>\\i[470]減少機率 \\v[3] → \\c[14]\\v[4]","武器的\\i[470]耐久值减少机率下降了！<br>\\i[470]減少機率 \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["\\n[1]の回避率が上がった！ 回避率 \\v[3] → \\c[14]\\v[4]","\\n[1]\'s Evasion ability increased!<br>Evasion: \\v[3] → \\c[14]\\v[4]","\\n[1]의 회피율이 상승했다!<br>회피율 \\v[3] → \\c[14]\\v[4]","\\n[1]的回避率上升了！<br>回避率 \\v[3] → \\c[14]\\v[4]","\\n[1]的回避率上升了！<br>回避率 \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["\\v[3]は全回復した！","\\v[3]\'s HP have been completely restored!","\\v[3](은)는 완전히 회복되었다!","\\v[3]完全恢復了！","\\v[3]完全恢复了！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["武器が売れた！ \\n[1]は合計 \\v[3]\\g 手に入れた！","You made a sale!<br>\\n[1] received a total of \\v[3]\\g!","무기가 팔렸다!<br>\\n[1]는 총 \\v[3]\\g을 획득했다!","武器賣出去了！<br>\\n[1]總共獲得\\v[3]\\g！","武器卖出去了！<br>\\n[1]总共获得\\v[3]\\g！"]);
_tr_m.push(["なんと武器を買ったお客さんが お礼を置いていったようだ！","The customer who made the purchase left you a tip!","무기를 산 손님이 감사 인사를 남긴 모양이다!","購買武器的客人竟然留下了禮物！","购买武器的客人竟然留下了礼物！"]);
_tr_m.push(["\\i[141]アルマの石を \\v[3]個 手に入れた！","You received \\i[141]Arma Stones x\\v[3]!","\\i[141]알마의 돌 \\v[3]개를 획득했다!","獲得\\v[3]顆\\i[141]魂石！","获得\\v[3]颗\\i[141]魂石！"]);
_tr_m.push(["これ以上商品棚に武器を置けない。","You can\'t place any more weapons on the shop table.","더 이상 상품 선반에 무기를 올려놓을 수 없다.","無法放置更多武器至商品架。","无法放置更多武器至商品架。"]);
_tr_m.push(["商品棚に武器を置きますか？\\! ","Place a weapon on the shop table?\\! ","상품 선반에 무기를 올려놓으시겠습니까?\\! ","要放置武器至商品架嗎？\\! ","要放置武器至商品架吗？\\! "]);
_tr_m.push(["これらの武器を商品棚に置きますか？\\! ","Place these weapons on the shop table?\\! ","선택한 무기를 상품 선반에 무기를 올려놓으시겠습니까?\\! ","要放置這些武器至商品架嗎？\\! ","要放置这些武器至商品架吗？\\! "]);
_tr_m.push(["","","","",""]);
_tr_m.push(["今すぐ家に帰りますか？\\! ","Return home right away?\\! ","지금 바로 집으로 돌아가시겠습니까?\\! ","要馬上回家嗎？\\! ","要马上回家吗？\\! "]);
_tr_m.push(["","","","",""]);
_tr_m.push(["どれについて確認しますか？\\! ","Which do you wish to confirm?\\! ","무엇을 확인하겠습니까?\\! ","要確認哪一個項目呢？\\! ","要确认哪一个项目呢？\\! "]);
_tr_m.push(["","","","",""]);
_tr_m.push(["\\n[1]の最大ＨＰが 10 上がった！ 最大ＨＰ \\v[3] → \\c[14]\\v[4]","\\n[1]\'s Max HP increased by 10!<br>MAX HP: \\v[3] → \\c[14]\\v[4]","\\n[1]의 최대 HP가 10 상승했다!<br>최대 HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP上升10了！<br>最大HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP上升10了！<br>最大HP \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["\\n[1]の最大ＨＰが \\v[5] 上がった！ 最大ＨＰ \\v[3] → \\c[14]\\v[4]","\\n[1]\'s Max HP increased by \\v[5]!<br>MAX HP: \\v[3] → \\c[14]\\v[4]","\\n[1]의 최대 HP가 \\v[5] 상승했다!<br>최대 HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP上升\\v[5]了！<br>最大HP \\v[3] → \\c[14]\\v[4]","\\n[1]的最大HP上升\\v[5]了！<br>最大HP \\v[3] → \\c[14]\\v[4]"]);
_tr_m.push(["復活する","Resurrect","부활하기","復活","复活"]);
_tr_m.push(["\\i[261]薬草 5個 で復活できます。 （\\i[261]所持数 ✕ \\v[3] ）\\! ","You can use five \\i[261]herbs to resurrect yourself.<br>(\\i[261] Owned: \\v[3])\\! ","\\i[261]약초 5개로 부활할 수 있습니다.<br>(\\i[261]소지 수 ✕ \\v[3])\\! ","可以使用5棵\\i[261]藥草復活。<br>（\\i[261]持有數 ✕ \\v[3]）\\! ","可以使用5棵\\i[261]药草复活。<br>（\\i[261]持有数 ✕ \\v[3]）\\! "]);
_tr_m.push(["\\n[1]は倒れてしまった・・・。","\\n[1] was defeated...","\\n[1]는 쓰러지고 말았다・・・.","\\n[1]倒下了……","\\n[1]倒下了……"]);
_tr_m.push(["\\n[1]は復活した！","\\n[1] was resurrected!","\\n[1]는 부활했다!","\\n[1]復活了！","\\n[1]复活了！"]);
_tr_m.push(["最終スコア ：\\v[30] 経過ターン数：\\v[3]","Final Score: \\v[30]<br>Turns Elapsed: \\v[3]","최종 점수: \\v[30]<br>경과 턴 수: \\v[3]","最終分數　：\\v[30]<br>經過回合數：\\v[3]","最终分数：\\v[30]<br>经过回合数：\\v[3]"]);
_tr_m.push(["ハイスコアを更新！","New High Score!","최고 기록 갱신!","更新最高分數了！","最高分数更新了！"]);
_tr_m.push(["報酬として \\i[141]アルマの石を \\v[3]個 手に入れた！","In recognition of your achievement, you received \\i[141]Arma Stones x\\v[3]!","보수로 \\i[141]알마의 돌 \\v[3]개를 획득했다!","獲得報酬\\v[3]顆\\i[141]魂石！","獲得報酬\\v[3]顆\\i[141]魂石！"]);
_tr_m.push(["お姉ちゃんなら、もっと生き延びられるはずだよ！ ファイト！ \\nw[ぽん美]","You should surely be able to last longer than that! Give it your all! \\nw[Ponmi]","언니라면 더 오래 살아 남을 수 있어! 화이팅! \\nw[폰미]","姐姐應該可以生存更久的！ 加油！ \\nw[器美]","姐姐应该可以生存更久的！ 加油！ \\nw[器美]"]);
_tr_m.push(["おお～！ お姉ちゃん、なかなかやるね！！ \\nw[ぽん美]","Oh! You\'re really something! \\nw[Ponmi]","오~~! 언니, 좀 하는데!? \\nw[폰미]","喔喔～！ 姐姐妳很厲害嘛！！ \\nw[器美]","喔喔～！ 姐姐你很厉害嘛！！ \\nw[器美]"]);
_tr_m.push(["すごーい！！！ お姉ちゃん強いね！！ \\nw[ぽん美]","Amazing! You\'re SO strong, Sis! \\nw[Ponmi]","굉장해!!! 언니 정말 강하다!! \\nw[폰미]","好厲害！！！ 姐姐好強喔！！ \\nw[器美]","好厉害！！！ 姐姐好强喔！！ \\nw[器美]"]);
_tr_m.push(["お姉ちゃん・・・ もはや、スゴスギテ＝クサハエルだよ！！ \\nw[ぽん美]","Weaco... The LOL Sword is all yours! \\nw[Ponmi]","언니・・・ 조금만 더하면 너무강해서 현웃터지는 소드야!! \\nw[폰미]","姐姐…… 妳根本就是超強‧草叢生了嘛！！ \\nw[器美]","姐姐…… 你根本就是超强‧草丛生了嘛！！ \\nw[器美]"]);
_tr_m.push(["\\n[1]は倒れてしまった・・・。\\! ","\\n[1] was defeated...\\! ","\\n[1]는 쓰러지고 말았다・・・.\\! ","\\n[1]倒下了……\\! ","\\n[1]倒下了……\\! "]);
_tr_m.push([">ヘイワ村から再開",">Restart from Peaceville",">평온한 마을에서 다시 시작",">從和平村重新開始",">从和平村重新开始"]);
_tr_m.push([">タイトルへ戻る",">Return to Title Screen",">타이틀로 돌아가기",">返回標題畫面",">返回标题画面"]);
_tr_m.push([">履歴を見る",">View Log",">이력 확인",">查看履歷",">查看履历"]);
_tr_m.push(["ヘイワ村から再開します。 お金を \\v[3]\\g 失います。（所持金：\\v[4]\\g）\\! ","You will restart from Peaceville. <br>You lost \\v[3]\\g. (Money: \\v[4]\\g)\\! ","평온한 마을에서 다시 시작합니다. <br>\\v[3]\\g의 돈을 잃게 됩니다(소지금: \\v[4]\\g)\\! ","將從和平村重新開始。 <br>失去金錢\\v[3]\\g。（持有金錢：\\v[4]\\g）\\! ","将从和平村重新开始。<br>失去金钱\\v[3]\\g。（持有金钱：\\v[4]\\g）\\! "]);
_tr_m.push(["タイトルへ戻ります。 セーブしていない情報が失われます。\\! ","You will return to the title screen. Any unsaved data will be lost.\\! ","타이틀 화면으로 돌아갑니다. 저장하지 않은 정보는 사라집니다.\\! ","將返回標題畫面。尚未存檔的資料將會消失。\\! ","将返回标题画面。尚未存档的数据将会消失。\\! "]);
_tr_m.push(["","","","",""]);
_tr_m.push(["装備する","Equip","장비하기","裝備","裝備"]);
_tr_m.push(["しまう","Store","집어넣기","收起","收起"]);
_tr_m.push(["元に戻す","Drop","원래대로 되돌리기","放回去","放回去"]);
_tr_m.push(["最大ＨＰアップ","Increase Max HP","최대 HP 상승","提升最大HP","提升最大HP"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["持ち物から選ぶ","Choose from Inventory","소지품에서 고르기","從攜帶物選擇","从携带物选择"]);
_tr_m.push(["武器庫から選ぶ","Choose from Armory","무기고에서 고르기","從武器庫選擇","从武器库选择"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["移動と方向転換","Move and change direction","이동과 방향 전환","移動並改變方向","移动并改变方向"]);
_tr_m.push(["基本操作（画面のボタン）","Basic Controls (Screen Buttons)","기본 조작(화면 버튼)","基本操作（畫面上的按鍵）","基本操作（画面上的按键）"]);
_tr_m.push(["武器の拾い方（基本）","How to Pick Up Weapons","무기 줍는 방법","撿起武器的方式","捡起武器的方式"]);
_tr_m.push(["武器の拾い方（敵の場合）","How to Pick Up Weapons (Enemies)","무기 줍는 방법(적의 경우)","撿起武器的方式（對象為敵人時）","捡起武器的方式（对象为敌人时）"]);
_tr_m.push(["攻撃のしかたと範囲の確認","Attacking & Confirming Attack Range","공격 방법과 범위 확인","攻擊方式與確認範圍","攻击方式与确认范围"]);
_tr_m.push(["武器の耐久値について","Weapon Durability","무기의 내구도에 대하여","關於武器的耐久值","关于武器的耐久值"]);
_tr_m.push(["メニューと装備の変え方","Menu & Changing Equipment","메뉴와 장비 변경 방법","選單與裝備的更改方式","选单与装备的更改方式"]);
_tr_m.push(["武器の破棄","Discarding Weapons","무기 버리기","捨棄武器","舍弃武器"]);
_tr_m.push(["必殺奥義","Special Moves","필살오의","必殺奧義","必杀奥义"]);
_tr_m.push(["金床の説明","Anvils","모루","鐵砧的說明","铁砧的说明"]);
_tr_m.push(["鍛冶屋の説明","Blacksmith","대장간","鐵匠鋪說明","铁匠铺说明"]);
_tr_m.push(["商品棚の説明","Shop Table","상품 선반","商品架說明","商品架说明"]);
_tr_m.push(["妹の説明","Younger Sister","여동생","妹妹的說明","妹妹的说明"]);
_tr_m.push(["魔王城の説明","Demonic Castle","마왕성","魔王城說明","魔王城说明"]);
_tr_m.push(["便利コマンド","Useful Commands (Keyboard)","간편 명령(키보드)","快捷指令（鍵盤）","快捷指令（键盘）"]);
_tr_m.push(["ゲームパッドでの操作","Gamepad Controls","게임 패드 조작 방법","控制器的操作","控制器的操作"]);
_tr_m.push(["武器を投げる","Weapon Throwing","무기 투척","武器投擲","武器投掷"]);
_tr_m.push(["ザ・ミラクル斬り","Miracle Slash","THE 미라클 베기","THE‧奇蹟斬","THE‧奇迹斩"]);
_tr_m.push(["ぐるぐる回転斬り","Spin Attack","빙글빙글 회전베기","轉轉迴旋斬","转转回旋斩"]);
_tr_m.push(["ロック・オン","Lock On","록 온","LOCK‧ON","LOCK‧ON"]);
_tr_m.push(["斬り捨てゴメン","Sorry Not Sorry","정당방위 베기","斬捨御免","斩舍御免"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["１個　","×1 ","×1 ","×1 ","×1 "]);
_tr_m.push(["５個　","×5 ","×5 ","×5 ","×5 "]);
_tr_m.push(["２０個","×20","×20","×20","×20"]);
_tr_m.push(["\\>\\i[\\v[3]]\\v[4]の情報が解禁された！ \\>\\n[1]は図鑑に書き込んだ！","\\>\\i[\\v[3]]\\v[4]\'s information was unlocked! <br>\\>\\n[1] wrote in the Weapons Gallery!","\\>\\i[\\v[3]]\\v[4]의 정보가 해제 된!<br>\\>\\n[1]는 무기 도감에 쓴!","\\>\\i[\\v[3]]\\v[4]資訊已解禁！<br>\\>\\n[1]在武器圖鑑中寫道！","\\>\\i[\\v[3]]\\v[4]资讯已解禁！<br>\\>\\n[1]在武器图鉴中写道！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["床も壁も、一面が鋼鉄でできている。 辺りには、岩と鉄の臭いが漂う・・・。","Here, both the floor and the wall are made of steel. There are smells of rocks and irons around here...","이곳은 바닥도 벽도 모두 강철로되어있다. 근처에는 바위와 철의 냄새가 감도는・・・.","這裡，地板和牆壁都是鋼製的。這裡到處都是岩石和鐵桿的味道……","这里，地板和墙壁都是钢制的。这里到处都是岩石和铁杆的味道……"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["不思議な物体がある。 触れてみますか？\\! ","There is a strange object. Would you like to touch it?\\! ","이상한 물체가있다. 만져보고 있습니까?\\! ","有一個奇怪的對象。你想觸摸嗎？ \\! ","有一个奇怪的对象。你想触摸吗？ \\!"]);
_tr_m.push(["\\n[1]は鍵を手に入れた！","\\n[1] got the key!","\\n[1]은 열쇠를 손에 넣었다!","得到了鑰匙！","得到了钥匙！"]);
_tr_m.push(["\\n[1]はこの洞窟内の どこかの鍵を開けられそうな気がした！","\\n[1] seemed to be able to open the door somewhere in this cave!","\\n[1]은 이 동굴 어딘가에있는 문을 열 수있을 것 같은 기분이 들었다!","似乎能夠在這個洞穴的某個地方打開門！","似乎能够在这个洞穴的某个地方打开门！"]);
_tr_m.push(["鍵がかかっている・・・。\\! ","This door is locked...\\! ","이 문은 잠겨있다・・・.\\! ","這扇門被鎖了……\\! ","这扇门被锁了…… \\!"]);
_tr_m.push([">無理やり開ける",">Forcibly open",">억지로 연다",">強行開放",">强行开放"]);
_tr_m.push([">諦める",">Give up",">포기",">放棄",">放弃"]);
_tr_m.push(["\\n[1]は扉をつかんで力を入れた！","\\n[1] grabbed the iron grid and put her strength!","\\n[1]은 철창을 잡고 힘을 쏟았다!","\\n[1]抓住鐵格並把你的力量！","\\n[1]抓住铁格并把你的力量！"]);
_tr_m.push(["ガチャガチャ！","Getdata!","빙글 빙글!","獲得！","获得！"]);
_tr_m.push(["ガチャガチャ！！","Getdata!!","빙글 빙글! !","獲得！！","获得！ ！"]);
_tr_m.push(["しかし\\n[1]は別に怪力キャラではなかった！ 扉はビクともしなかった！","But \\n[1] was not a stubborn character! The door did not work as well!","그러나 \\n[1]은 괴력 캐릭터가 아니었다! 문은 꿈쩍도하지 않았다!","但是\\n[1]並不是一個頑固的角色！門沒有用！","但是\\n[1]并不是一个顽固的角色！门没有用！"]);
_tr_m.push(["\\n[1]は疲れている！","\\n[1] is tired!","\\n[1]은 피로하다!","\\n[1]累了！","\\n[1]累了！"]);
_tr_m.push(["\\n[1]は諦めた。","\\n[1] gave up everything.","\\n[1]은 모든 것을 포기했다.","\\n[1]放棄了一切。","\\n[1]放弃了一切。"]);
_tr_m.push(["\\n[1]はさっき拾った鍵を使った！","\\n[1] used the key she picked up earlier!","\\n[1]은 아까 주운 열쇠를 사용한!","\\n[1]使用了她之前選擇的鑰匙！","\\n[1]使用了她之前选择的钥匙！"]);
_tr_m.push(["なんと扉が開いた！","How the door opened!","무려 문이 열렸다!","門是怎麼打開的！","门是怎么打开的！"]);
_tr_m.push(["\\n[1]はドヤドヤしている！","\\n[1] is doyaya!","\\n[1]은 우르르있다!","\\n[1]很自豪！","\\n[1]很自豪！"]);
_tr_m.push(["\\{\\{鎧竜エリムサルエ\\|\\|\\|\\^","\\{\\{Armored Dragon Elimsalue\\|\\|\\|\\^","\\{\\{철갑용 엘림살루에\\|\\|\\|\\^","\\{\\{鎧龍艾利姆薩爾耶\\|\\|\\|\\^","\\{\\{铠龙艾利姆萨尔耶\\|\\|\\|\\^"]);
_tr_m.push(["鎧竜エリムサルエを倒した！\\|\\|\\|\\|\\|\\|\\|\\|\\^","You won Armored Dragon Elimsalue!!!\\|\\|\\|\\|\\|\\|\\|\\|\\^","당신은 철갑용 엘림살루에 이겼다!\\|\\|\\|\\|\\|\\|\\|\\|\\^","你贏了鎧龍艾利姆薩爾耶！\\|\\|\\|\\|\\|\\|\\|\\|\\^","你赢了铠龙艾利姆萨尔耶！ \\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["突然現れた鎧竜エリムサルエを、 武器を駆使してなんとか倒した\\n[1]。","She was able to overwhelm Armored Dragon Elimsalue by using the power of weapons.","그녀는 무기의 힘을 사용하는 것으로, 철갑용 엘림살루 를보기 좋게 이길 수 있었다.","她能夠利用武器的力量壓倒鎧龍艾利姆薩爾耶。","她能够利用武器的力量压倒铠龙艾利姆萨尔耶。"]);
_tr_m.push(["鋼鉄の洞窟内の武器を すべて拾えるようになった！","\\n[1] now be able to pick up all the weapons in Cave of Iron!","\\n[1]는 강철의 동굴의 무기를 모두 잡을 수있게되었다!","\\n[1]現在能夠拿起鋼鐵洞窟中的所有武器！","\\n[1]现在能够拿起钢铁洞窟中的所有武器！"]);
_tr_m.push(["洞窟から出ますか？\\! ","Are you going out of the cave? \\! ","동굴에서 나와 있습니까? \\! ","你要離開山洞嗎？ \\! ","你要离开山洞吗？ \\! "]);
_tr_m.push(["","","","",""]);
_tr_m.push(["毒々しく不快な空気が流れている。 嫌な予感のする場所だ・・・。","Here, poisonous and unpleasant air is flowing. It is a place I feel bad feeling...","이곳은 독한 불쾌한 공기가 흐르고있다. 싫은 예감이 드는 곳이다・・・.","在這裡，有毒和令人不快的空氣流動。這是一個讓我心情不好的地方……","在这里，有毒和令人不快的空气流动。这是一个让我心情不好的地方……"]);
_tr_m.push(["\\{\\{怪鳥ヴィジャヤ\\|\\|\\|\\^","\\{\\{Ominous Bird Vijaya\\|\\|\\|\\^","\\{\\{괴조 비자야\\|\\|\\|\\^","\\{\\{怪鳥毘闍耶\\|\\|\\|\\^","\\{\\{怪鸟毗闍耶\\|\\|\\|\\^"]);
_tr_m.push(["怪鳥ヴィジャヤを倒した！\\|\\|\\|\\|\\|\\|\\|\\|\\^","You won Ominous Bird Vijaya!!!\\|\\|\\|\\|\\|\\|\\|\\|\\^","당신은 괴조 비자야에 이겼다!\\|\\|\\|\\|\\|\\|\\|\\|\\^","你贏了怪鳥毘闍耶！\\|\\|\\|\\|\\|\\|\\|\\|\\^","你赢了怪鸟毗闍耶！ \\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["不気味な洞窟の最奥に現れた怪鳥ヴィジャヤを、 武器を駆使してなんとか倒した\\n[1]。","She was able to overwhelm Ominous Bird Vijaya by using the power of weapons.","그녀는 무기의 힘을 사용하는 것으로, 괴조 비자야 를보기 좋게 이길 수 있었다.","她能夠利用武器的力量壓倒怪鳥毘闍耶。","她能够利用武器的力量压倒怪鸟毗闍耶。"]);
_tr_m.push(["弱者の洞窟内の武器を すべて拾えるようになった！","\\n[1] now be able to pick up all the weapons in Cave of the Weak!","\\n[1]는 약자의 동굴의 무기를 모두 잡을 수있게되었다!","\\n[1]現在能夠拿起弱者洞窟中的所有武器！","\\n[1]现在能够拿起弱者洞窟中的所有武器！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["幻想的な雰囲気のその空間には、 強い魔力が張り巡らされているのが感じられる・・・。","In this space of a fantastic atmosphere, it is felt that strong magical power is being stretched...","환상적인 분위기의이 공간은 강한 마력이 깔려있는 것이 느껴진다・・・.","在這個夢幻般的氛圍中，人們感覺到強大的魔法力量正在被拉伸……","在这个梦幻般的氛围中，人们感觉到强大的魔法力量正在被拉伸……"]);
_tr_m.push(["鉄格子はビクともしない。","Iron Bunko does not do it.","철창은 꿈쩍도하지 않는다.","鐵格柵不這樣做。","铁格栅不这样做。"]);
_tr_m.push(["「同じ地を二度踏んではならぬ。 　四つの問に答えよ。されば扉は開かれん。」","\"You must not step on the same place twice. Answer the four questions, and the door will open.\"","「당신은 같은 땅을 두번 밟지한다. 네 질문에 대답하라 그리하면 문이 열린다 것이다.」","「你不能兩次踩到同一個地方。回答這四個問題，門就會打開。」","「你不能两次踩到同一个地方。回答这四个问题，门就会打开。」"]);
_tr_m.push(["なんと鉄格子が開いた！","It seems that Iron Ritsiko has opened!","무려 철창이 열려있는 것 같다!","似乎鐵格柵已經開業了！","似乎铁格栅已经开业了！"]);
_tr_m.push(["\\n[1]はドヤドヤしている！","\\n[1] is doyaya!","\\n[1]는 우르르있다!","\\n[1]很自豪！","\\n[1]很自豪！"]);
_tr_m.push(["\\{\\{魔塊テュルソス\\|\\|\\|\\^","\\{\\{Thyrsos, Grand Demon\\|\\|\\|\\^","\\{\\{마괴 티르소스\\|\\|\\|\\^","\\{\\{魔塊希瑟斯\\|\\|\\|\\^","\\{\\{魔块希瑟斯\\|\\|\\|\\^"]);
_tr_m.push(["魔塊テュルソスを倒した！\\|\\|\\|\\|\\|\\|\\|\\|\\^","You won Thyrsos, Grand Demon!!!\\|\\|\\|\\|\\|\\|\\|\\|\\^","당신은 마괴 티르소스에 이겼다!\\|\\|\\|\\|\\|\\|\\|\\|\\^","你贏了魔塊希瑟斯！\\|\\|\\|\\|\\|\\|\\|\\|\\^","你赢了魔块希瑟斯！ \\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["洞窟の奥に不気味に佇む魔塊テュルソスを、 武器を駆使してなんとか倒した\\n[1]。","She was able to overwhelm Thyrsos, Grand Demon by using the power of weapons.","그녀는 무기의 힘을 사용하는 것으로, 마괴 티르소스 를보기 좋게 이길 수 있었다.","她能夠利用武器的力量壓倒魔塊希瑟斯。","她能够利用武器的力量压倒魔块希瑟斯。"]);
_tr_m.push(["幻術の洞窟内の武器を すべて拾えるようになった！","\\n[1] now be able to pick up all the weapons in Cave of Illusion!","\\n[1]는 환술의 동굴의 무기를 모두 잡을 수있게되었다!","\\n[1]現在能夠拿起幻術洞窟中的所有武器！","\\n[1]现在能够拿起幻术洞窟中的所有武器！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["暗闇に光る青い炎。 そして心なしか感じる不気味な視線。","A blue flame that shines in the dark. And an eerie gaze feels from somewhere.","어둠에 빛나는 푸른 불꽃. 그리고 어딘가에서 느끼는 기분 나쁜 시선.","在黑暗中閃耀的藍色火焰。從某個地方感受到一種令人毛骨悚然的目光。","在黑暗中闪耀的蓝色火焰。从某个地方感受到一种令人毛骨悚然的目光。"]);
_tr_m.push(["見るからに危険な場所である。 \\n[1]は一刻も早く帰りたいと思った。","This is a dangerous place to see. \\n[1] want to go home now.","이곳은보기에도 위험한 장소이다. \\n[1]은 당장 돌아가고 싶었다.","這是一個危險的地方。 \\n[1]現在想回家。","这是一个危险的地方。 \\n[1]现在想回家。"]);
_tr_m.push(["「此の地を彷徨いし魔物、一切の攻撃を受けず。 　青き炎の力を纏い、戦うべし。」","\"The monsters wandering around this land will not suffer any attacks. Wear the power of the blue flame and fight.\"","「이 땅을 방황하는 마물은 일체의 공격을받지 않는다. 푸른 불꽃의 힘을 입고 싸우는 것.」","「在這片土地上游蕩的怪物不會受到任何攻擊，佩戴藍色火焰的力量並進行戰鬥。」","「在这片土地上游荡的怪物不会受到任何攻击，佩戴蓝色火焰的力量并进行战斗。」"]);
_tr_m.push(["不思議な灯籠がある。 ここに青い炎を灯せそうだ。","There is a strange lantern. you can light up a blue flame by this lantern.","이상한 등불이있다. 이 등불에 푸른 불꽃을 피운 것 같다.","有一個奇怪的燈籠。我要在這個燈籠裡點亮藍色的火焰。","有一个奇怪的灯笼。我要在这个灯笼里点亮蓝色的火焰。"]);
_tr_m.push(["不思議な灯籠がある。 ここに青い炎を灯せそうだ。\\! ","There is a strange lantern. you can light up a blue flame by this lantern. \\! ","이상한 등불이있다. 이 등불에 푸른 불꽃을 피운 것 같다. \\! ","有一個奇怪的燈籠。我要在這個燈籠裡點亮藍色的火焰。\\! ","有一个奇怪的灯笼。我要在这个灯笼里点亮蓝色的火焰。 \\! "]);
_tr_m.push(["青い炎が揺れている。 ","A blue flame is shaking.","푸른 불꽃이 흔들리고있다.","藍色的火焰正在顫抖。","蓝色的火焰正在颤抖。"]);
_tr_m.push(["青い炎が揺れている。 なにかに炎をつければ たいまつ代わりに利用できそうだ。","A blue flame is shaking. If you add flames to something, it seems to be able to use like torch.","푸른 불꽃이 흔들리고있다. 뭔가 불꽃을 붙이면 횃불 대신 사용할 수있을 것이다.","藍色的火焰正在顫抖。如果你添加火焰，它似乎可以使用魚雷代替。","蓝色的火焰正在颤抖。如果你添加火焰，它似乎可以使用鱼雷代替。"]);
_tr_m.push(["青い炎が揺れている。 武器に炎をつければ たいまつ代わりに利用できそうだ。\\! ","A blue flame is shaking. If you put a flame on your weapon, it seems to be able to use like torch. \\! ","푸른 불꽃이 흔들리고있다. 무기에 불을 붙이면 횃불 대신 사용할 수있을 것이다. \\! ","藍色的火焰正在顫抖。如果你在你的武器上放一個火焰，你可以使用魚雷代替。\\! ","蓝色的火焰正在颤抖。如果你在你的武器上放一个火焰，你可以使用鱼雷代替。 \\! "]);
_tr_m.push([">炎を灯す",">Light up a flame",">불을 지피는",">點燃火焰",">点燃火焰"]);
_tr_m.push([">炎をつける",">Add a flame",">불꽃을 점화",">添加火焰",">添加火焰"]);
_tr_m.push(["\\{\\{霊騎ダインスレイフ\\|\\|\\|\\^","\\{\\{Ghost Knight Dáinsleif\\|\\|\\|\\^","\\{\\{영혼기사 다인슬라이프\\|\\|\\|\\^","\\{\\{靈騎丹斯雷夫\\|\\|\\|\\^","\\{\\{灵骑丹斯雷夫\\|\\|\\|\\^"]);
_tr_m.push(["霊騎ダインスレイフを倒した！\\|\\|\\|\\|\\|\\|\\|\\|\\^","You won Ghost Knight Dáinsleif!!!\\|\\|\\|\\|\\|\\|\\|\\|\\^","당신은 영혼기사 다인슬라이프에 이겼다!\\|\\|\\|\\|\\|\\|\\|\\|\\^","你贏了靈騎丹斯雷夫！\\|\\|\\|\\|\\|\\|\\|\\|\\^","你赢了灵骑丹斯雷夫！ \\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["暗い廃城の最奥に現れた霊騎ダインスレイフを、 武器を駆使してなんとか倒した\\n[1]。","She was able to overwhelm Ghost Knight Dáinsleif by using the power of weapons.","그녀는 무기의 힘을 사용하는 것으로, 영혼기사 다인슬라이프 를보기 좋게 이길 수 있었다.","她能夠利用武器的力量壓倒靈騎丹斯雷夫。","她能够利用武器的力量压倒灵骑丹斯雷夫。"]);
_tr_m.push(["霊魂の洞窟内の武器を すべて拾えるようになった！","\\n[1] now be able to pick up all the weapons in Cave of Lost Souls!","\\n[1]는 영혼의 동굴의 무기를 모두 잡을 수있게되었다!","\\n[1]現在能夠拿起靈魂洞窟中的所有武器！","\\n[1]现在能够拿起灵魂洞窟中的所有武器！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["なんと宝箱の中には \\i[141]アルマの石が 11個 入っていた！","There are 11 \\i[141]Arma Stones in the chest!","보물상자 안에는 무려 \\i[141]알마의 돌 11개가 들어 있었다!","寶箱裡竟然有11顆 \\i[141]魂石！","寶箱裡竟然有11顆 \\i[141]魂石！"]);
_tr_m.push(["\\i[141]アルマの石を 11個 手に入れた！","You got 11 \\i[141]Arma Stones!","\\i[141]알마의 돌 11개를 획득했다!","獲得11顆 \\i[141]魂石！","獲得11顆 \\i[141]魂石！"]);
_tr_m.push(["武器の神アメノオハバリを倒した！","You defeated the God of Weaponry Ameno Ohbari!","무기의 신 아메노오하바리를 쓰러뜨렸다!","打倒武器之神 天之尾羽張了！","打倒武器之神 天之尾羽张了！\\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["「\\i[90]伝説のトロフィー」を手に入れた！\\|\\|","You got the \"\\i[90]Legendary Trophy\"! \\|\\|","「\\i[90]전설의 트로피」를 획득했다! \\|\\|","獲得了「\\i[90]傳說的獎杯」！ \\|\\|","获得了「\\i[90]传说的奖杯」！ \\|\\|"]);
_tr_m.push(["「\\i[90]商人王のトロフィー」を手に入れた！\\|\\|","You got the \"\\i[90]Tycoon Trophy\"! \\|\\|","「\\i[90]상인왕의 트로피」를 획득했다! \\|\\|","獲得了「\\i[90]商人王的獎杯」！ \\|\\|","获得了「\\i[90]商人王的奖杯」！ \\|\\|"]);
_tr_m.push(["「\\i[90]勇者のトロフィー」を手に入れた！\\|\\|","You got the \"\\i[90]Hero Trophy\"! \\|\\|","「\\i[90]용사의 트로피」를 획득했다! \\|\\|","獲得了「\\i[90]勇者的獎杯」！ \\|\\|","获得了「\\i[90]勇者的奖杯」！ \\|\\|"]);
_tr_m.push(["「\\i[90]感謝のトロフィー」を手に入れた！\\|\\|","You got the \"\\i[90]Gratitude Trophy\"! \\|\\|","「\\i[90]감사의 트로피」를 획득했다! \\|\\|","獲得了「\\i[90]感謝的獎杯」！ \\|\\|","获得了「\\i[90]感谢的奖杯」！ \\|\\|"]);
_tr_m.push(["何も書かれていない。","Nothing is written.","아무것도 쓰여 있지 않은.","什麼都沒寫。","什么都没写。"]);
_tr_m.push(["「中間地点 　ごゆっくりおくつろぎください」","\"Intermediate point<br> Please relax and have a moment. \"","「중간 지점<br>　천천히 휴식하십시오 」","「中間點<br>　請放心吧。」","「中间点<br>　请放心吧。」"]);
_tr_m.push(["なんと宝箱の中には \\i[261]薬草が 3個 入っていた！","There are 3 \\i[261]herbs in the chest!","보물상자 안에는 무려 \\i[261]약초 3개가 들어 있었다!","居然，寶箱裡有3個\\i[261]藥草！","宝箱里居然有3个\\i[261]药草！"]);
_tr_m.push(["\\i[261]薬草を 3個 手に入れた！","You got 3 \\i[261]herbs!","\\i[261]약초 3개를 획득했다!","獲得3個\\i[261]藥草！","获得3个\\i[261]药草！"]);
_tr_m.push(["「\\v[15]階」","\"\\v[15]F\"","「\\v[15] 층」","「\\v[15]樓」","「\\v[15]楼」"]);
_tr_m.push(["どこからスタートしますか？\\! ","Where do you start from?\\! ","어디서부터 시작할까요?\\! ","你從哪裡開始？\\! ","你从哪里开始？ \\! "]);
_tr_m.push([">1階から",">1F",">1 층",">1樓",">1楼"]);
_tr_m.push([">中間地点から",">Intermediate point",">중간 지점",">中間點",">中间点"]);
_tr_m.push([">最上階から",">Top floor",">최상층",">頂樓",">顶楼"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["とにかく色んなものを拾えるだけ拾って、 どんどん使って図鑑を埋めていこう！","Let\'s pick up various things as much as you can, use them, and add many weapon to the Weapons Gallery!","많은 것을 주워 사용하여 무기 도감을 채워 가자!","讓我們盡可能多地拿起各種東西，使用它們，並在武器庫中添加許多武器！","让我们尽可能多地拿起各种东西，使用它们，并在武器库中添加许多武器！"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["え？もう図鑑を完成させた？ \\nw[武器の神アメノオハバリ]","Huh? Have you completed the picture book yet? \\nw[God of Weaponry Ameno Ohabari]","응? 또 도감을 완성시켰다합니까? \\nw[무기의 신 아메노오하바리]","嗯？你完成了武器圖鑑？\\nw[武器之神 天之尾羽張]","嗯？你完成了武器图鉴？ \\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["・・・。 \\nw[武器の神アメノオハバリ]",".... \\nw[God of Weaponry Ameno Ohabari]","···. \\nw[무기의 신 아메노오하바리]",".... \\nw[武器之神 天之尾羽張]",".... \\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["ふっ。やはり我の見る目は正しかったようだな。 \\nw[武器の神アメノオハバリ]","After all, it looks like my eyes were right. \\nw[God of Weaponry Ameno Ohabari]","역시, 우리가 보는 눈은 정확했다 것 같다. \\nw[무기의 신 아메노오하바리]","畢竟，看起來我的眼睛是正確的。\\nw [武器之神 天之尾羽張]","毕竟，看起来我的眼睛是正确的。 \\nw [武器之神 天之尾羽张]"]);
_tr_m.push(["いや・・・むしろ、それを超えてきた。 \\nw[武器の神アメノオハバリ]","No ... Rather, you have exceeded it. \\nw[God of Weaponry Ameno Ohabari]","아니 ... 오히려, 당신은 그것을 넘어왔다. \\nw[무기의 신 아메노오하바리]","不......相反，你超過了它。\\nw[武器之神 天之尾羽張]","不......相反，你超过了它。 \\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["そなたに伝えたいことがある。 自宅にて、妹とともに待っていてくれ。 \\nw[武器の神アメノオハバリ]","I have something to tell you. Wait at home with your sister. \\nw[God of Weaponry Ameno Ohabari]","당신에게 전하고 싶은 것이있다. 집에서 당신의 여동생과 함께 기다리고있어 줘. \\nw[무기의 신 아메노오하바리]","我有話要跟你說。在家等你的妹妹。\\nw[武器之神 天之尾羽張]","我有话要跟你说。在家等你的妹妹。 \\nw[武器之神 天之尾羽张]"]);
_tr_m.push(["","","","",""]);
_tr_m.push(["私はここの教会の牧師。そして 「ブンナゲ教」の教徒でもある、ファルシオだ。 \\nw[name]","I am Falsio. I am a pastor of the church here, and also a \"THROWING\" religionist. \\nw[name]","나는 Falsio. 나는 여기에서 교회의 목사이며, \"투척 교\"교도이기도하다. \\nw[name]","我是Falsio。我是這裡教會的牧師，也是一個“投擲教”的宗教主義者。\\nw[name]","我是Falsio。我是这里教会的牧师，也是一个“投掷教”的宗教主义者。 \\nw[name]"]);
_tr_m.push(["ブンナゲ教の理念は、「なんでも物をぶん投げて ストレスを発散しよう！」である。アーメン。 \\nw[name]","The idea of ​​\"THROWING\" is \"Let\'s throw anything to release stress!\" Amen. \\nw[name]","투척 교 의 이념은\"뭐든지 물건을 ぶん投げ 스트레스를 발산하자! \"이다. 아멘. \\nw[name]","“投擲教”的想法是“讓我們放任何東西來釋放壓力！”Amen。\\nw[name]","“投掷教”的想法是“让我们放任何东西来释放压力！”Amen。 \\nw[name]"]);
_tr_m.push(["汝がここを訪れたのも 神のお導きに違いない。 \\nw[name]","It must be the guidance of God that you have visited here. \\nw[name]","당신이 이곳을 찾은 것도 하나님의 인도에 틀림 없다. \\nw[name]","你去過這裡一定是上帝的指導。\\nw[name]","你去过这里一定是上帝的指导。 \\nw[name]"]);
_tr_m.push(["汝にも我々のぶん投げを伝授しよう！ \\nw[name]","I will teach you our teachings! \\nw[name]","당신도 우리의 가르침을 전수하자! \\nw[name]","我會教你我們的教誨！\\nw[name]","我会教你我们的教诲！ \\nw[name]"]);
_tr_m.push(["\\n[1]は\\c[14]必殺奥義『武器投げ』\\c[0]を覚えた！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1] learned the \\c[14]special move [Weapon Throwing]\\c[0]! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]는 \\c[14]필살오의 『무기 투척』\\c[0]를 익혔다! \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]學會了\\c[14]必殺奧義「武器投擲」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^","\\n[1]学会了\\c[14]必杀奥义「武器投掷」\\c[0]！ \\|\\|\\|\\|\\|\\|\\|\\|\\|\\^"]);
_tr_m.push(["これからはバンバンぶん投げていくように。 神のご加護のあらんことを！ \\nw[name]","Let\'s throw various things from now on. God bless you! \\nw[name]","앞으로는 많은 물건을 던져보십시오. 신의 축복이 있기를! \\nw[name]","上帝保佑你！\\nw[name]讓我們從現在開始拋出各種各樣的東西。","上帝保佑你！ \\nw[name]让我们从现在开始抛出各种各样的东西。"]);
_tr_m.push(["物をぶん投げるのは、気持ちいいわよねえ。 \\nw[name]","It\'s wonderful to throw things. \\nw[name]","물건을 던지는 것은 매우 기분이 좋다. \\nw[name]","扔東西太好了。\\nw[name]","扔东西太好了。 \\nw[name]"]);
_tr_m.push(["人に向かって投げると、危ないから そこだけ気をつけないと。 \\nw[name]","But It\'s dangerous if you throw at people, so just be careful. \\nw[name]","사람을 향해 물건을 던져 위험하니 조심 해야지. \\nw[name]","但是如果你向人們扔東西是危險的，所以要小心。\\nw[name]","但是如果你向人们扔东西是危险的，所以要小心。 \\nw[name]"]);
_tr_m.push(["ああ、ブンナゲ教バンザイ！ \\nw[name]","Oh, \"THROWING\" is great! \\nw[name]","아, \"투척 교\" 최고 다! \\nw[name]","哦，“投擲教”很棒！\\nw[name]","哦，“投掷教”很棒！ \\nw[name]"]);
_tr_m.push(["北東の海沿いにある教会に行ったことはありますか？ なんでも、武器の投げ方を教えてくれるそうですよ。 \\nw[name]","Have you ever been to a church by the northeastern sea? You can be taught how to throw a weapon there. \\nw[name]","북동 해안에있는 교회에 가본 적이 있습니까? 당신은 거기에서 무기 던지는 방법을 배울 수 있다고합니다. \\nw[name]","你有沒有去過東北海的教堂？你可以怎樣在那里扔武器。\\nw[name]","你有没有去过东北海的教堂？你可以怎样在那里扔武器。 \\nw[name]"]);
_tr_m.push(["魔物との戦闘に役立ちそうですよね。 ちょっと怪しい宗教との噂ですが・・・。 \\nw[name]","It may be useful for fighting with monsters. However, they seem to be a little suspicious religion ... \\nw[name]","마물과의 전투에 도움이됩니다. 단지, 그들은 조금 이상한 종교 같습니다만 .... \\nw[name]","然而，他們似乎是一個有點可疑的宗教...... 它可能對與怪物戰鬥有用。\\nw[name]","然而，他们似乎是一个有点可疑的宗教...... 它可能对与怪物战斗有用。\\nw[name]"]);






































_tr_m.push(["","","","",""]);
_tr_m.push(["","","","",""]);

})();
    //・システム
    (function(){
_tr_e.push(["テキスト","英語","韓国語","繁体文字","簡体文字"]);
_tr_e.push(["は毒にかかった。毎ターンダメージを受ける！"," is poisoned and will take damage each turn!","는 독에 걸렸다. 턴마다 대미지를 받는다!","中毒了。每回合都會承受傷害！","中毒了。每回合都会受到伤害！"]);
_tr_e.push(["は毒にかかった。"," is poisoned.","는 독에 걸렸다.","中毒了。","中毒了。"]);
_tr_e.push(["の毒が消えた。"," is cured of the poison.","의 독이 해제되었다.","的中毒狀態解除了。","的中毒状态解除了。"]);
_tr_e.push(["は混乱した。行動が逆になる！"," got confused, and the action flips!","은 혼란했다.","感到困惑。","感到困惑。"]);
_tr_e.push(["は混乱した。"," got confused.","은 혼란했다.","感到困惑。","感到困惑。"]);
_tr_e.push(["は我に返った。"," cured from the confusion.","은 혼란이 나았다.","的混亂治癒了。","的混乱治愈了。"]);
_tr_e.push(["は石化した。しばらく動けない！"," was petrified, and can not act for a while!","은 석화했다.","變成了一塊石頭。","变成了一块石头。"]);
_tr_e.push(["は石化した。"," was petrified.","은 석화했다.","變成了一塊石頭。","变成了一块石头。"]);
_tr_e.push(["の石化が解かれた。"," healed from the petrification.","은 석화가 나았다.","不再是一塊石頭了。","不再是一块石头了。"]);
_tr_e.push(["は感電した。しばらく何もできない！"," got an electric shock, and can not act for a while!","은 감전했다.","麻木了。","麻木了。"]);
_tr_e.push(["は感電した。"," got an electric shock.","은 감전했다.","麻木了。","麻木了。"]);
_tr_e.push(["のしびれがとれた。"," cured from electric shock.","은 감전이 나았다.","的麻木治癒了。","的麻木治愈了。"]);
_tr_e.push(["は眠った。しばらく何もできない！"," slept, and can not act for a while!","은 잤다.","睡了。","睡了。"]);
_tr_e.push(["は眠った。"," slept.","은 잤다.","睡了。","睡了。"]);
_tr_e.push(["は目を覚ました。"," woke up.","은 깨어났다.","醒了。","醒了。"]);
_tr_e.push(["の回避率が上がった。","\'s Evasion increased.","의 회피율이 상승했다.","的回避率上升了。","的回避率上升了。"]);
_tr_e.push(["の回避率が下がった。","\'s Evasion decreased.","의 회피율이 감소했다.","的回避率下降了。","的回避率下降了。"]);
_tr_e.push(["の回避率が元に戻った。","\'s Evasion returned to normal.","의 회피율이 원래대로 되돌아왔다.","的回避率恢復原狀了。","的回避率复原了。"]);
_tr_e.push(["の命中率が上がった。","\'s Accuracy increased.","의 명중률이 상승했다.","的命中率上升了。","的命中率上升了。"]);
_tr_e.push(["の命中率が下がった。","\'s Accuracy decreased.","의 명중률이 감소했다.","的命中率下降了。","的命中率下降了。"]);
_tr_e.push(["の命中率が元に戻った。","\'s Accuracy returned to normal.","의 명중률이 원래대로 되돌아왔다.","的命中率恢復原狀了。","的命中率复原了。"]);
_tr_e.push(["はこのターン行動できない！"," can not act this turn!","은이 턴 행동 할 수 없다!","在本回合不能動作！","在本回合不能动作！"]);
_tr_e.push(["は２ターンの間行動できなくなった！"," can not act for 2 turns!","은 2 턴 동안 행동 할 수 없게되었다!","不能行動2回合！","不能行动2回合！"]);
_tr_e.push(["は３ターンの間行動できなくなった！"," can not act for 3 turns!","은 3 턴 동안 행동 할 수 없게되었다!","不能行動3回合！","不能行动3回合！"]);
_tr_e.push(["は猛毒にかかった。毎ターンダメージを受ける！"," is poisoned and will take damage each turn!","는 독에 걸렸다. 턴마다 대미지를 받는다!","中毒了。每回合都會承受傷害！","中毒了。每回合都会受到伤害！"]);
_tr_e.push(["は猛毒にかかった。"," is poisoned.","는 독에 걸렸다.","中毒了。","中毒了。"]);
_tr_e.push(["の毒が消えた。"," is cured of the poison.","의 독이 해제되었다.","的中毒狀態解除了。","的中毒状态解除了。"]);
_tr_e.push(["は水に濡れて雷耐性が下がった！"," got wet by the water and the lightning resistance decreased.","물에 젖었으며 천둥 내성이 감소했다.","被水弄濕，雷耐性下降了。","被水弄湿，雷耐性下降了。"]);
_tr_e.push(["の雷耐性が元に戻った。","\'s Lightning resistance returned to normal.","의 천둥 내성이 원래대로 되돌아왔다.","的雷耐性恢復原狀了。","的雷耐性恢复原状了。"]);
_tr_e.push(["の炎耐性が下がった！","\'s Flame resistance decreased.","의 불 내성이 감소했다.","的炎耐性下降了。","的炎耐性下降了。"]);
_tr_e.push(["の炎耐性が元に戻った。","\'s Flame resistance returned to normal.","의 불 내성이 원래대로 되돌아왔다.","的炎耐性恢復原狀了。","的炎耐性恢复原状了。"]);
_tr_e.push(["の氷耐性が下がった！","\'s Ice resistance decreased.","의 얼음 내성이 감소했다.","的冰耐性下降了。","的冰耐性下降了。"]);
_tr_e.push(["の氷耐性が元に戻った。","\'s Ice resistance returned to normal.","의 얼음 내성이 원래대로 되돌아왔다.","的冰耐性恢復原狀了。","的冰耐性恢复原状了。"]);
_tr_e.push(["の雷耐性が下がった！","\'s Lightning resistance decreased.","의 천둥 내성이 감소했다.","的雷耐性下降了。","的雷耐性下降了。"]);
_tr_e.push(["の雷耐性が元に戻った。","\'s Lightning resistance returned to normal.","의 천둥 내성이 원래대로 되돌아왔다.","的雷耐性恢復原狀了。","的雷耐性恢复原状了。"]);
_tr_e.push(["は物理攻撃が使えなくなった！"," became impossible to use physical attacks!","는 물리적 공격을 사용할 수 없게되었습니다!","未能使用物理攻擊！","未能使用物理攻击！"]);
_tr_e.push(["は物理攻撃を使えるようになった。"," became possible to use physical attacks.","는 물리적 공격을 사용할 수있게되었습니다.","可以使用物理攻擊。","可以使用物理攻击。"]);
_tr_e.push(["は魔法を使えなくなった！"," became impossible to use magic!","는 마술을 사용할 수 없게되었습니다!","未能使用魔法！","未能使用魔法！"]);
_tr_e.push(["は魔法を使えるようになった。"," became possible to use magic.","는 마술을 사용할 수있게되었습니다.","可以使用魔法。","可以使用魔法。"]);
_tr_e.push(["は通常攻撃しかできなくなった！"," became impossible to use non-standard attacks!","는 비표준 공격을 사용할 수 없게되었습니다!","未能使用非標準攻擊！","未能使用非标准攻击！"]);
_tr_e.push(["は通常攻撃以外も使えるようになった。"," became possible to use non-standard attacks.","는 비표준 공격을 사용할 수있게되었습니다.","可以使用非標準攻擊。","可以使用非标准攻击。"]);
_tr_e.push(["の打撃耐性が下がった！","\'s Slash Resistance decreased.","의 내성 저항이 감소했다.","的斬擊耐性下降了。","的斩击耐性下降了。"]);
_tr_e.push(["の打撃耐性が元に戻った。","\'s Slash Resistance returned to normal.","의 슬래쉬 저항이 원래대로 되돌아왔다.","的斬擊耐性恢復原狀了。","的斩击耐性恢复原状了。"]);
_tr_e.push(["の斬撃耐性が下がった！","\'s Blow Resistance decreased.","의 내 블로 저항이 감소했다.","的打擊耐性下降了。","的打击耐性下降了。"]);
_tr_e.push(["の斬撃耐性が元に戻った。","\'s Blow Resistance returned to normal.","의 불기 저항이 원래대로 되돌아왔다.","的打擊耐性恢復原狀了。","的打击耐性恢复原状了。"]);
_tr_e.push(["は眠らなくなった！"," got insomnia!","는 불면증!","失眠了！","失眠了！"]);
_tr_e.push(["の不眠状態が解除された。","\'s Insomnia has been lifted.","의 불면증이 해제되었습니다.","的失眠已被解除。","的失眠已被解除。"]);
_tr_e.push(["は武器を変えられなくなった！"," became impossible to change weapons!","는 무기를 바꾸는 것이 불가능하게되었습니다!","無法改變武器！","无法改变武器！"]);
_tr_e.push(["は武器を変えられるようになった。"," became possible to change weapons.","는 무기를 바꿀 수있게되었습니다.","成為可能改變武器。","成为可能改变武器。"]);
_tr_e.push(["はロックオン状態になった！"," entered the \"Lock on\"!","이 (가) 『록 온』에 들어갔습니다!","進入「LOCK‧ON」狀態！","进入「LOCK‧ON」状态！"]);
_tr_e.push(["のロックオン状態が解除された。","\'s \"Lock on\" has been lifted.","의 『록 온』이 해제되었습니다.","的「LOCK‧ON」已被取消。","的「LOCK‧ON」已被取消。"]);
_tr_e.push(["は 3ターンの間 自動で回復する！"," will automatically recover for 3 turns!","은 (는) 3 턴 동안 자동으로 회복됩니다!","將自動恢復3回合！","将自动恢复3回合！"]);
_tr_e.push(["の自動回復効果が切れた。","\'s The auto recovery state has been lifted.","s 자동 복구 상태가 해제되었습니다.","的自動恢復狀態已被解除。","的自动恢复状态已被解除。"]);
_tr_e.push(["は貧弱になった。攻撃力が大幅に下がった！"," entered the Enfeeble state! The ATK quite decreased.","이 (가) 빈약에 들어갔습니다! 공격력이 상당히 떨어졌다.","進入虛弱狀態！攻擊力大幅下降了。","进入虚弱状态！攻击力大幅下降了。"]);
_tr_e.push(["の貧弱状態が解除された。","\'s Enfeeble state has been lifted.","의 빈약이 해제되었습니다.","的虛弱狀態已被取消。","的虚弱状态已被取消。"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["の攻撃！"," attacked!","의 공격!","的攻擊！","的攻击！"]);
_tr_e.push(["は様子を見ている。"," is observing the situation.","(은)는 상황을 보고 있다.","在觀察情況。","在观察情况。"]);
_tr_e.push(["は炎をふきだした！"," breathed the fire","은 화염을 닦아 냈다!","會噴出了火焰！","会喷出了火焰！"]);
_tr_e.push(["は【火炎の魔法】を唱えた！"," cast fire magic!","(은)는 【화염 마법】을 외웠다!","吟唱【火焰魔法】了！","吟唱了【火焰魔法】！"]);
_tr_e.push(["は【火炎の魔法・極】を唱えた！"," cast EX fire magic!","(은)는 【궁극의 화염 마법】을 외웠다!","吟唱【火焰魔法‧極】了！","吟唱了【火焰魔法‧极】！"]);
_tr_e.push(["は【氷の魔法】を唱えた！"," cast ice magic!","(은)는 【얼음 마법】을 외웠다!","吟唱【寒冰魔法】了！","吟唱了【寒冰魔法】！"]);
_tr_e.push(["は【氷の魔法・極】を唱えた！"," cast EX ice magic!","(은)는 【궁극의 얼음 마법】을 외웠다!","吟唱【寒冰魔法‧極】了！","吟唱了【寒冰魔法‧极】！"]);
_tr_e.push(["は【雷の魔法】を唱えた！"," cast lightning magic!","(은)는 【천둥 마법】을 외웠다!","吟唱【雷電魔法】了！","吟唱了【雷电魔法】！"]);
_tr_e.push(["は【雷の魔法・極】を唱えた！"," cast EX lightning magic!","(은)는 【궁극의 천둥 마법】을 외웠다!","吟唱【雷電魔法‧極】了！","吟唱了【雷电魔法‧极】！"]);
_tr_e.push(["は【攻撃強化の魔法】を唱えた！"," cast attack buff magic!","(은)는 【공격 강화 마법】을 외웠다!","吟唱【攻擊強化魔法】了！","吟唱了【攻击強化魔法】！"]);
_tr_e.push(["は【攻撃弱体の魔法】を唱えた！"," cast attack weakening magic!","(은)는 【공격 약화 마법】을 외웠다!","吟唱【攻擊虛弱魔法】了！","吟唱了【攻击虚弱魔法】！"]);
_tr_e.push(["は【防御強化の魔法】を唱えた！"," cast defense buff magic!","(은)는 【방어 강화 마법】을 외웠다!","吟唱【防禦強化魔法】了！","吟唱了【防御強化魔法】！"]);
_tr_e.push(["は【防御弱体の魔法】を唱えた！"," cast defense weakening magic!","(은)는 【방어 약화 마법】을 외웠다!","吟唱【防禦虛弱魔法】了！","吟唱了【防御虚弱魔法】！"]);
_tr_e.push(["は【全強化の魔法】を唱えた！"," cast buffs magic!","(은)는 【전체 강화 마법】을 외웠다!","吟唱【全強化魔法】了！","吟唱了【全強化魔法】！"]);
_tr_e.push(["は【全弱体の魔法】を唱えた！"," cast overall weakening magic!","(은)는 【전체 약화 마법】을 외웠다!","吟唱【全虛弱魔法】了！","吟唱了【全虚弱魔法】！"]);
_tr_e.push(["は【闇の魔法】を唱えた！"," cast dark magic!","(은)는 【어둠 마법】을 외웠다!","吟唱【黑暗魔法】了！","吟唱了【黑暗魔法】！"]);
_tr_e.push(["は【闇の魔法・極】を唱えた！"," cast EX dark magic!","(은)는 【궁극의 어둠 마법】을 외웠다!","吟唱【黑暗魔法‧極】了！","吟唱了【黑暗魔法‧极】！"]);
_tr_e.push(["は【行動回数上昇魔法】を唱えた！"," cast stamina down magic!","(은)는 【행동 횟수 증가 마법】을 외웠다!","吟唱【行動次數提升魔法】了！","吟唱了【行动次数提升魔法】！"]);
_tr_e.push(["は【行動回数減少魔法】を唱えた！"," cast stamina up magic!","(은)는 【행동 횟수 감소 마법】을 외웠다!","吟唱【行動次數降低魔法】了！","吟唱了【行动次数降低魔法】！"]);
_tr_e.push(["は必殺奥義『ザ・ミラクル斬り』を放った！"," used the special move [Miracle Slash]!","는 필살오의 『THE 미라클 베기』(을)를 사용했다!","施展出必殺奧義『THE‧奇蹟斬』了！","施展出必杀奥义『THE‧奇迹斩』了！"]);
_tr_e.push(["は必殺奥義『ぐるぐる回転斬り』を放った！"," used the special move [Spin Attack]!","는 필살오의 『빙글빙글 회전베기』(을)를 사용했다!","施展出必殺奧義『轉轉迴旋斬』了！","施展出必杀奥义『转转回旋斩』了！"]);
_tr_e.push(["は必殺奥義『ロック・オン』を放った！"," used the special move [Lock On]!","는 필살오의 『록 온』(을)를 사용했다!","施展出必殺奧義『LOCK‧ON』了！","施展出必杀奥义『LOCK‧ON』了！"]);
_tr_e.push(["は必殺奥義『斬り捨てゴメン』を放った！"," used the special move [Sorry Not Sorry]!","는 필살오의 『정당방위 베기』(을)를 사용했다!","施展出必殺奧義『斬捨御免』了！","施展出必杀奥义『斩舍御免』了！"]);
_tr_e.push(["は火炎をふき出した！"," breathed the fire","은 화염을 닦아 냈다!","會噴出了火焰！","会喷出了火焰！"]);
_tr_e.push(["は棒をブン投げた！"," threw a club!","은 곤봉 붕 던졌다!","扔了一根棍子！","扔了一根棍子！"]);
_tr_e.push(["は斧をブン投げた！"," threw an ax!","은 도끼를 붕 던졌다!","扔了一把斧頭！","扔了一把斧头！"]);
_tr_e.push(["は爆弾をブン投げた！"," threw a bomb!","은 폭탄을 붕 던졌다!","扔了一顆炸彈！","扔了一颗炸弹！"]);
_tr_e.push(["は岩砕きを放った！"," fired rock fragments!","는 바위 깨기를 구사했다!","施展出碎石術了！","施展出碎石术了！"]);
_tr_e.push(["は鎧砕きを放った！"," fired broken pieces of armor!","는 갑옷 깨기를 구사했다!","施展出碎鎧術了！","施展出碎铠术了！"]);
_tr_e.push(["はシビレ攻撃を放った！"," attacked with its tingling arms!","는 독 촉수로 공격했다!","用電人的觸手攻擊了！","用电人的触手攻击了！"]);
_tr_e.push(["はほうしをまき散らした！"," sprinkled spores!","은 포자를 퍼뜨린!","撒上了孢子！","撒上了孢子！"]);
_tr_e.push(["はうじゃうじゃした！"," is squirming around!","가 우글우글한다.","蠕動了！","蠕动了！"]);
_tr_e.push(["は150デシベルの鳴き声をあげた！"," screeched at 150 decibels!","는 150dB의 울음소리를 냈다!","發出150分貝的鳴叫聲了！","发出150分贝的鸣叫声了！"]);
_tr_e.push(["はじっとしている。"," is observing the situation.","(은)는 상황을 보고 있다.","在觀察情況。","在观察情况。"]);
_tr_e.push(["は大爆発した！"," exploded!","는 대폭발을 일으켰다!","大爆炸了！","大爆炸了！"]);
_tr_e.push(["はするどい岩を放った！"," shot a sharp rock!","은 날카로운 바위를 던졌다!","扔了一塊鋒利的岩石！","扔了一块锋利的岩石！"]);
_tr_e.push(["は鋼鉄の爪で切り裂いた！"," slashed away with its claws of steel!","는 강철 발톱으로 할퀴었다!","用鋼鐵爪子劈了又劈！","用钢铁爪子劈了又劈！"]);
_tr_e.push(["は鋼鉄の爪で何度も切り裂いた！"," slashed away with its claws of steel many times!","는 강철 발톱으로 몇 번이고 할퀴었다!","用鋼鐵爪多次子劈了又劈！","用钢铁爪多次子劈了又劈！"]);
_tr_e.push(["は魔触手を生やした！"," grew the tentacles!","은 촉수를 기른!","增長了觸手！","增长了触手！"]);
_tr_e.push(["は毒液を吐き出した！"," spewed poison!","는 독물을 뿜어댔다!","吐出毒液了！","吐出毒液了！"]);
_tr_e.push(["は串刺しにしてきた！"," impaled!","는 꿰뚫어 매달았다!","刺穿了！","刺穿了！"]);
_tr_e.push(["は馬で蹴り飛ばした！"," kicked with the horse!","은 발로 차버렸다!","踢飛了！","踢飞了！"]);
_tr_e.push(["は暗黒の波動を放った！"," released a dark wave!","은 암흑의 파동을 날렸다!","發布了一波黑暗！","发布了一波黑暗！"]);
_tr_e.push(["は武器を研いだ！"," sharpened weapons!","은 무기를 예리하게!","削尖了武器！","削尖了武器！"]);
_tr_e.push(["は武器の流星群を呼び出した・・・！"," called the meteor shower of weapons ...!","은 무기 유성우를 호출했다 ・・・!","稱為了流星雨武器......！","称为了流星雨武器......！"]);
_tr_e.push(["は大賢者の杖をかかげた！"," got the age\'s Staff!","은 대현자의 지팡이 들고 있었다!","拄著拐了大賢者之杖！","拄着拐了大贤者之杖！"]);
_tr_e.push(["は衝撃波を放った！"," released a shock wave!","은 충격파를 발했다!","發布了衝擊波！","发布了冲击波！"]);
_tr_e.push(["は『大神斬』を放った！"," unleashed a Divine Strike!","은 「대신참」을 구사했다!","施展出『大神斬』了！","施展出了『大神斩』！"]);
_tr_e.push(["は笑っている・・・。"," is laughing...","은 웃고있다 ・・・.","笑了......","笑了......"]);
_tr_e.push(["はうじゃうじゃ玉を放った！"," is squirming around!","가 우글우글한다.","蠕動了！","蠕动了！"]);
_tr_e.push(["は「魔界拳骨」を放った！"," used Fists of Hell!","는 「마계권골」을 구사했다!","施展出「魔界拳」了！","施展出了「魔界拳」！"]);
_tr_e.push(["は燃え盛る闇の火炎を吐き出した！"," breathed the burning fire of darkness!","은 타오르는 어둠의 불꽃을 쏟아 냈다!","會噴出了燃燒黑暗的火焰！","会喷出了燃烧黑暗的火焰！"]);
_tr_e.push(["は「凍てつく波動」を放った！"," released \"freezing wave\"!","은 얼어 붙는 파동을 날렸다!","發布了一股冰冷的浪潮！","发布了一股冰冷的浪潮！"]);
_tr_e.push(["は面倒臭そうに剣を振った！"," shook the sword like a mess! ","은 귀찮은 듯이 검을 휘둘러!","像一團糟一樣震動了劍！","像一团糟一样震动了剑！"]);
_tr_e.push(["は\\i[261]薬草を使った！"," used \\i[261] herbs! ","은 \\i[261] 약초를 이용한!","使用了\\i[261]草藥！","使用了\\i[261]草药！"]);
_tr_e.push(["は「魔界二連拳骨」を放った！"," used Two Fists of Hell! ","는 「마계권골×2」을 구사했다!","施展出「魔界二拳」了！","施展出「魔界二拳」了！"]);
_tr_e.push(["は「呪いの拳骨」を放った！"," used Fists of Curse! ","는 「저주의 주먹」을 구사했다!","施展出「拳头的诅咒」了！","施展出「拳头的诅咒」了！"]);
_tr_e.push(["\n[1]の生命力が 半分削られた！"," \\n[1]\'s life force has been cut by half!","\\n[1]의 생명력이 절반 깎인!","\\n[1]生命力減半！","\\n[1]生命力减半！"]);
_tr_e.push(["は激しく燃え盛る闇の豪炎を吐き出した！"," breathed the violent burning fire of darkness!","은 격렬하게 타오르는 어둠의 불길을 쏟아 냈다!","會噴出了一股猛烈燃燒的黑暗之火！","会喷出了一股猛烈燃烧的黑暗之火！"]);
_tr_e.push(["は瞑想を始めた・・・！"," started meditation ...!","은 명상을 시작했다 ・・・!","開始了冥想......！","开始了冥想......！"]);
_tr_e.push([""," ","","",""]);
_tr_e.push(["%1の能力変化が %2にコピーされた！","Capacity change of %1 was copied to %2!","%1의 능력 변화가 %2에 복사 된!","%1的容量更改已復製到%2！","%1的容量更改已复制到%2！"]);
_tr_e.push(["%1は武器を変えられなくなった！","%1 can no longer change weapons!","%1은 무기를 바꿀 수 없게되었다!","%1無法再改變武器！","%1无法再改变武器！"]);
_tr_e.push(["%1は通常攻撃しかできなくなった！","%1 can only attack normally!","%1은 통상 공격 밖에 할 수 없게되었다!","%1只能正常攻擊！","%1只能正常攻击！"]);
_tr_e.push(["%1は消滅した。","%1 has disappeared.","%1은 소멸했다.","%1已經消失。","%1已经消失。"]);
_tr_e.push(["青い炎が変化した！","The blue flame has changed!","푸른 불꽃이 변화했다!","藍色的火焰已經改變了！","蓝色的火焰已经改变了！"]);
_tr_e.push(["%1についていた炎が消えてしまった！","The flame on %1 has disappeared!","%1에 붙어 있던 불길이 사라졌다!","%1上的火焰消失了！","%1上的火焰消失了！"]);
_tr_e.push(["%1の%2が %3 回復した。","%2 of %1 recovered %3.","%1 %2 %3 회복했다.","%1的%2恢復了%3。","%1的%2恢复了%3。"]);
_tr_e.push(["%1は%2の財布からお金を抜き取った！","%1 pulled out money from the %2 wallet!","%1 %2의 지갑에서 돈을 빼낸!","%1從%2錢包裡掏出錢了！","%1从%2钱包里掏出钱了！"]);
_tr_e.push(["\\n[1]は %1\\g 手に入れた！","\\n[1] got %1\\g!","\\n[1] %1\\g 얻었다!","\\n[1]得到了%1\\g！","\\n[1]得到了%1\\g！"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["%1は%2を装備した！","","","",""]);
_tr_e.push(["はぷるんっと体当たりしてきた！"," jiggled as it slammed","은 온몸으로 철퍼덕 부딪혔다!","彈跳著衝撞了！","弹跳着冲撞了！"]);
_tr_e.push(["は岩の腕を叩きつけた！"," attacked with its stone arms.","은 바위 팔을 내리쳤다!","用岩石手臂毆打了！","用岩石手臂殴打了！"]);
_tr_e.push(["の攻撃。"," attacked.","의 공격.","的攻擊。","的攻击。"]);
_tr_e.push(["は骨で殴ってきた！"," attacked with the bone!","은 뼈 때려왔다!","用骨頭打！","用骨头打！"]);
_tr_e.push(["は噛みついてきた！"," bit!","는 물었다!","咬了！","咬了！"]);
_tr_e.push(["持てない武器は武器庫に送った。","Some weapons were sent to your Armory.","일부 무기가 무기고에 보냈습니다.","一些武器被送到了武器庫。","一些武器被送到了武器库。"]);
_tr_e.push(["作成した武器","Weapons created","만든 무기","創造武器","创造武器"]);
_tr_e.push(["研ぐ　　　","Sharpen  ","연마하기","研磨","研磨"]);
_tr_e.push(["メッキする","Galvanize","도금하기","鍍金","鍍金"]);
_tr_e.push(["鍛える　　","Temper   ","단련하기","鍛造","鍛造"]);
_tr_e.push(["%1 (Lv.%2で解放)","%1 (Unlocked at Lv.%2)","%1 (Lv.%2에서 풀린다)","%1 (在Lv.%2解鎖)","%1 (在Lv.%2解锁)"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["応援","Cheering","응원  ","加油","加油"]);
_tr_e.push(["武器庫拡張","Expanding armory","무기고 확장 ","擴大武器庫","扩大武器库"]);
_tr_e.push(["呼び込み","Invitation","고객 유치 ","招攬一下客人","招揽一下客人"]);
_tr_e.push(["商売術の勉強","Studying sales","장사 공부 ","學習生意","学习生意"]);
_tr_e.push(["商品棚拡張","Expanding shop tables","상품 선반 확장","擴大商品架","扩大商品架"]);
_tr_e.push(["妹のプレゼント","A gift from a sister","동생의 선물 ","妹送的禮物","妹送的礼物"]);
_tr_e.push(["最大ＨＰアップ","Increase Max HP","최대 HP 상승","提升最大HP","提升最大HP"]);
_tr_e.push(["攻撃力アップ","Increase ATK","공격력 상승","提升攻擊力","提升攻擊力"]);
_tr_e.push(["運アップ","Increase LCK","운 상승","提升運氣","提升運氣"]);
_tr_e.push(["✕%1","×%1","×%1","✕%1","✕%1"]);
_tr_e.push(["%1個","×%1","×%1","✕%1","✕%1"]);
_tr_e.push(["最大 %1分","Max %1 min","최대 %1 분","最多%1分鐘","最多%1分钟"]);
_tr_e.push(["%1％","%1%","%1％","%1％","%1％"]);
_tr_e.push(["ぽん美に応援してもらう。\nもらえるすべての経験値が増加する。\n経験値 %1 ","I will support Ponmi. All the experience \nvalues gained are increased. \nExp : %1 ","폰미 응원 해달라고한다. \n받을 수있는 모든 경험치가 증가한다.  \n경험치 %1 ","器美歡呼雀躍。\n獲得的所有經驗值都會增加。 \n經驗值 %1 ","器美歡呼雀躍。 \n獲得的所有經驗值都會增加。 \n經驗值 %1 "]);
_tr_e.push(["武器庫を拡張する。\n収納しておける武器の数が増える。\n武器庫の容量 %1 ","Extend the weapon repository. The number\nof weapons that can be stored increases. \nWeapon storage : %1 ","무기고를 확장한다. 수납 해 둘\n수있는 무기의 수가 늘어난다.  \n무기고 용량 %1 ","擴展武器庫。\n可存儲的武器數量增加。 \n武器儲存量 %1 ","擴展武器庫。 \n可存儲的武器數量增加。 \n武器儲存量 %1 "]);
_tr_e.push(["お客さんの呼び込みをしてもらう。\n武器の売れる速度が上がる。\n売れる時間 %1 ","Have Ponmi attract customers. The speed \nat which weapons can be sold rises. \nTime to sell %1 ","폰미에 집객을 받는다. \n무기 팔리는 속도가 오른다.  \n팔리는 시간 %1 ","讓器美吸引顧客。\n武器銷售的速度上升。 \n出售的時間 %1 ","讓器美吸引顧客。 \n武器銷售的速度上升。 \n出售的時間 %1 "]);
_tr_e.push(["ぽん美に商売術を学んでもらう。\n武器の売れる値段が増加する。\n売れる価格 %1 ","Have Ponmi study business. The price\nat which weapons can be sold increases. \nSelling price : %1 ","폰미 장사 기술을 배워달라고한다.  \n무기 팔리는 가격이 증가한다. \n매매가 %1 ","讓器美學習商業技能。\n武器銷售的價格會上漲。 \n銷售價格 %1 ","讓器美學習商業技能。 \n武器銷售的價格會上漲。 \n銷售價格 %1 "]);
_tr_e.push(["商品棚をひとつ増やす。\n一度に売れる武器の数が増える。\n商品棚の数 %1 ","Increase commodity shelf. The number \nof weapons that can be sold at once \nincreases. \nNum of shop tables : %1 ","진열대를 하나 늘린다.\n한 번에 팔리는 무기의 수가 늘어난다. \n진열대의 몇 %1 ","增加商品貨架。\n可以一次出售的武器數量增加。\n產品貨架數量 %1 ","增加商品貨架。 \n可以一次出售的武器數量增加。 \n產品貨架數量 %1 "]);
_tr_e.push(["妹から嬉しいプレゼントを受け取る。\nどんなプレゼントかはお楽しみ。","You receive a nice present from \na sister. \nYou don\'t know what presents are. ","여동생 기쁜 선물을 받는다. \n어떤 선물 모르겠어요. ","你從姐姐那裡得到一份精美的禮物。 \n你不知道它是什麼樣的禮物。 ","你從姐姐那裡得到一份精美的禮物。 \n你不知道它是什麼樣的禮物。"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["武器を研いだ!!","Sharpened!!","연마했다!!","研磨了！","研磨了！"]);
_tr_e.push(["武器をメッキした!!","Galvanized!!","도금했다!!","鍍金了！","镀了金！"]);
_tr_e.push(["武器を鍛えた!!","Tempered!!","단련했다!!","鍛造了！","锻造了！"]);
_tr_e.push(["武器庫","Armory","무기고","武器庫","武器库"]);
_tr_e.push(["商品棚","Shop tables","상품 선반","商品架","商品架"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["%1はがんばってパンチしてみた！","%1 bravely tried to throw a punch!","%1는 용기를 내어 펀치를 날려 보았다!","%1試著使出加油拳法！","%1试着使出加油拳法！"]);
_tr_e.push(["\\n[1]の最大ＨＰがアップ！\n最大ＨＰ %1 ","\\n[1]\'s Max Hp increases! \nMax Hp %1","\\n[1]의 최대 HP가 업! \n최대 HP %1","提升\\n[1]的最大HP！ \n最大HP %1","提升\\n[1]的最大HP！ \n最大HP %1"]);
_tr_e.push(["\\n[1]の攻撃力がアップ！\nすべての武器の\\i[469]威力が上昇する。\n\\i[469]上昇値 %1 ","\\n[1]\'s ATK increases! \n\\i[469]ATK of all weapons increase. \nATK %1","\\n[1]의 공격력이 업! \n모든 무기의 \\i[469] 위력이 상승한다. \n\\i[469] 상승치 %1","提升\\n[1]的攻擊力！ \n所有武器的\\i[469]威力上升。 \n \\i[469]價值上漲 %1","提升\\n[1]的攻击力！ \n所有武器的\\i[469]威力上升。 \n \\i[469]价值上涨 %1"]);
_tr_e.push(["\\n[1]の運がアップ！\n武器の\\i[470]耐久値減少確率が下がる。\nまた、\\n[1]の回避率が上昇する。\n\\i[470]減少確率 %1 \n回避率 %2 ","\\n[1]\'s LCK increases! \nThe chances of weapon \\i[470]durability \nloss decrease. \nAlso \\n[1]\'s Evasion ability increases. \n \\i[470]\'s probability %1 \nEvasion ability %2","\\n[1] 운이 업! \n무기 \\i[470] 내구도 감소 확률이 떨어진다. \n또한 \\n[1]의 회피율이 상승한다. \n \\i[470] 감소 확률 %1 \n회피율 %2 ","提升\\n[1]的運氣！ \n武器的\\i[470]耐久值減少機率下降。\n此外，\\n[1]的迴避率上升。 \n\\i[470]降低概率 %1 \n迴避率 %2","提升\\n[1]的运气！ \n武器的\\i[470]耐久值减少机率下降。\n此外，\\n[1]的回避率上升。 \n\\i[470]降低概率 %1 \n回避率 %2"]);
_tr_e.push(["\\n[1]の最大ＨＰがダウン！\n最大ＨＰ %1 ","\\n[1]\'s Max Hp decreases! \nMax Hp %1","\\n[1]의 최대 HP가 감소! \n최대 HP %1","降低\\n[1]的最大HP！ \n最大HP %1","降低\\n[1]的最大HP！ \n最大HP %1"]);
_tr_e.push(["\\n[1]の攻撃力がダウン！\nすべての武器の\\i[469]威力が低下する。\n\\i[469]上昇値 %1 ","\\n[1]\'s ATK decreases! \n\\i[469]ATK of all weapons decrease. \nATK %1","\\n[1]의 공격력이 감소! \n모든 무기의 \\i[469] 위력이 감소. \n\\i[469] 상승치 %1","降低\\n[1]的攻擊力！ \n所有武器的\\i[469]威力下降。 \n \\i[469]價值上漲 %1","降低\\n[1]的攻击力！ \n所有武器的\\i[469]威力下降。 \n \\i[469]价值上涨 %1"]);
_tr_e.push(["\\n[1]の運がダウン！\n武器の\\i[470]耐久値減少確率が上がる。\nまた、\\n[1]の回避率が低下する。\n\\i[470]減少確率 %1 \n回避率 %2 ","\\n[1]\'s LCK decreases! \nThe chances of weapon \\i[470]durability \nloss increase. \nAlso \\n[1]\'s Evasion ability decreases. \n \\i[470]'s probability %1 \nEvasion ability %2","\\n[1] 운이 감소! \n무기 \\i[470] 내구도 감소 확률이 증가. \n또한 \\n[1]의 회피율이 감소한다. \n \\i[470] 감소 확률 %1 \n회피율 %2","降低\\n[1]的運氣！ \n武器的\\i[470]耐久值減少機率上升。\n此外，\\n[1]的迴避率下降。 \n\\i[470]降低概率 %1 \n迴避率 %2","降低\\n[1]的运气！ \n武器的\\i[470]耐久值减少机率上升。\n此外，\\n[1]的回避率下降。 \n\\i[470]降低概率 %1 \n回避率 %2 "]);
_tr_e.push(["","","","",""]);
_tr_e.push(["のこり\\{%1\\}ターン","\\{%1\\} more turns","나머지\\{%1\\}턴","剩下的\\{%1\\}回合","剩下的\\{%1\\}回合"]);
_tr_e.push(["アップ"," Up"," 상승","升起","升起"]);
_tr_e.push(["ダウン"," Down"," 감소","降低","降低"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["毒","Poison","독","中毒","中毒"]);
_tr_e.push(["混乱","Confuse","혼란","混亂","混乱"]);
_tr_e.push(["石化","","","",""]);
_tr_e.push(["感電","Shock","감전","觸電","触电"]);
_tr_e.push(["睡眠","Sleep","수면","睡眠","睡眠"]);
_tr_e.push(["回避率アップ","Evasion Up","회피율 상승","回避率升起","回避率升起"]);
_tr_e.push(["回避率ダウン","Evasion Down","회피율 감소","回避率降低","回避率降低"]);
_tr_e.push(["命中率アップ","Accuracy Up","명중률 상승","命中率升起","命中率升起"]);
_tr_e.push(["命中率ダウン","Accuracy Down","명중률 감소","命中率降低","命中率降低"]);
_tr_e.push(["行動不能","Immobilize","행동 불능","不能行動","不能行动"]);
_tr_e.push(["猛毒","Deadly Poison","맹독","劇毒","剧毒"]);
_tr_e.push(["びしょぬれ","Soaked","흠뻑","淋濕","淋湿"]);
_tr_e.push(["炎耐性ダウン","Fire Resistance Down","화염 내성 감소","火焰耐性降低","火焰耐性降低"]);
_tr_e.push(["氷耐性ダウン","Ice Resistance Down","얼음 내성 감소","雷電耐性降低","雷电耐性降低"]);
_tr_e.push(["雷耐性ダウン","Lightning Resistance Down","천둥 내성 감소","寒冰耐性降低","寒冰耐性降低"]);
_tr_e.push(["物理攻撃封印","Physical Seal","물리 공격 봉인","物理攻擊印章","物理攻击印章"]);
_tr_e.push(["魔法封印","Magic Seal","마법 봉인","魔法印章","魔法印章"]);
_tr_e.push(["スキル封印","Skill Seal","스킬 봉인","技能印章","技能印章"]);
_tr_e.push(["打撃耐性ダウン","Slash Resistance Down","슬래쉬 내성 감소","斬擊耐性降低","斩击耐性降低"]);
_tr_e.push(["斬撃耐性ダウン","Blow Resistance Down","블로 내성 감소","打擊耐性降低","打击耐性降低"]);
_tr_e.push(["カフェイン","Caffeine","카페인","咖啡因","咖啡因"]);
_tr_e.push(["武器変更不可","Weapon Pinning","무기 고정","武器釘扎","武器钉扎"]);
_tr_e.push(["ロック・オン","Lock On","록 온","LOCK‧ON","LOCK‧ON"]);
_tr_e.push(["自動回復","Auto Recovery","자동 복구","自動恢復","自动恢复"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["%1は\\i[261]薬草を使った！","%1 used \\i[261] herbs! ","%1은 \\i[261] 약초를 이용한!","%1使用了\\i[261]草藥！","%1使用了\\i[261]草药！"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["１個　　","×1   ","×1   ","×1   ","×1   "]);
_tr_e.push(["５個　　","×5   ","×5   ","×5   ","×5   "]);
_tr_e.push(["２０個　","×20  ","×20  ","×20  ","×20  "]);
_tr_e.push(["１つ","×1","×1","×1","×1"]);
_tr_e.push(["３つ","×3","×3","×3","×3"]);
_tr_e.push(["５つ","×5","×5","×5","×5"]);
_tr_e.push(["売り切れ","Sold Out","매진","售罄","售罄"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["\\n[1]は魔物を全滅させた！次の階層へ進めるようになった！","\\n[1] destroyed all monsters! She came to advance to the next floor!","\\n[1]은 몬스터를 모두 박살했다! 다음 층으로 진행되었다!","\\n[1]消滅所有的怪物！ 她可以前往下一層！","\\n[1]消灭所有权怪物！她可以前往下一层！"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["機能を選択","Select Function","기능을 선택","選擇功能","选择功能"]);
_tr_e.push(["ゲームパッドコンフィグ","Gamepad Config","게임 패드 구성","控制器配置","控制器配置"]);
_tr_e.push(["キーコンフィグ","Key Config","키 구성","密鑰配置","密钥配置"]);
_tr_e.push(["設定を保存","Save Setting","설정을 저장","保存設置","保存设置"]);
_tr_e.push(["変更前に戻す","Restore Before Change","변경 전 취소","更改前還原","更改前还原"]);
_tr_e.push(["初期設定に戻す","Reset To Initial State","원래 상태로 복원","回到初始狀態","回到初始状态"]);
_tr_e.push(["消去","Erase","삭제","刪除","删除"]);
_tr_e.push(["決定 / 調べる / 攻撃","Confirm/Attack/Search","결정 / 공격 / 조사","調查 / 決定 / 攻擊","调查 / 决定 / 攻击"]);
_tr_e.push(["キャンセル / メニュー","Cancel/Menu","취소 / 메뉴","取消 / 選單","取消 / 选单"]);
_tr_e.push(["方向転換 / 範囲確認","\\}Change Direction/Confirm Range\\{","방향 전환 / 범위 확인","改變方向 / 確認範圍","改变方向 / 确认范围"]);
_tr_e.push(["武器チェンジ","Change Weapon","무기 변경하기","更換武器","更换武器"]);
_tr_e.push(["武器を拾う","Pick Up Weapon","무기 줍기","撿起武器","捡起武器"]);
_tr_e.push(["待機","Wait","대기","待機","待机"]);
_tr_e.push(["薬草","Use Herb","약초","藥草","药草"]);
_tr_e.push(["空を飛ぶ","Fly","하늘 날기","空中飛翔","空中飞翔"]);
_tr_e.push(["やめる","Cancel","그만둔다","算了","算了"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["メッセージのテキストのアニメーションの ON/OFF を設定します。","Switch the message text animations ON/OFF.","메시지 텍스트의 애니메이션 ON/OFF를 설정합니다.","設定是否顯示對話文字的動畫效果。","设定是否显示对话文字的动画效果。"]);
_tr_e.push(["オートセーブの ON/OFF を設定します。\nONにすると、現在のセーブデータへ定期的にオートセーブを行います。\nセーブ時は少し動作が重くなる可能性があります。","Turn the Auto-Save feature ON/OFF. When ON, your game progress \nwill be automatically saved periodically.\nNote there may be some lag when saving.","자동 저장 ON/OFF를 설정합니다. ON으로 설정하면 \n현재 저장 데이터에 정기적으로 자동 저장을 실행합니다. \n저장 도중에는 동작에 다소 지연이 생길 가능성이 있습니다.","設定是否開啟自動存檔功能。\n若設定為ON，將會定期自動存檔至當前的存檔資料。\n存檔時遊戲畫面可能會略微延遲。","设定是否开启自动存档功能。 \n若设定为ON，将会定期自动存档至当前的存档资料。 \n存档时游戏画面可能会略微延迟。"]);
_tr_e.push(["メッセージログの画面最大表示行数を設定します。","Set the max number of lines in a message window.","메시지 로그 화면의 최대 표시 행수를 설정합니다.","設定訊息履歷畫面顯示的行數上限。","设定讯息履历画面显示的行数上限。"]);
_tr_e.push(["戦闘のテンポの速さを設定します。\nある程度ゲームに慣れてきてから高速にすることをお薦めします。","Set the game\'s battle speed. \nIt is recommended you speed up this setting \nonce you get used to the game.","전투 템포 속도를 설정합니다. \n어느 정도 게임에 익숙해진 후에 고속으로 변경하는 \n것을 추천합니다.","設定戰鬥節奏的速度。\n建議熟悉遊戲到一定程度後再設定為快速。","设定战斗节奏的速度。 \n建议熟悉游戏到一定程度后再设定为快速。"]);
_tr_e.push(["操作ボタンの表示タイプを設定します。\nゲームパッドでのプレイ時は「ゲームパッド」に設定してください。","Configure the controls display type.\nSet this to \"Gamepad\" when playing with one.","조작 버튼의 표시 타입을 설정합니다.\n게임 패드로 플레이할 경우에는 「게임 패드」로 설정하세요.","設定操作按鍵的顯示類型。\n使用控制器遊玩時請設定為「控制器」。","设定操作按键的显示类型。 \n使用控制器游玩时请设定为「控制器」。"]);
_tr_e.push(["表示する言語を変更します。","Change the language used in the game.","게임 내에서 사용 언어를 변경합니다.","更改遊戲中使用的語言。","更改游戏中使用的语言。"]);
_tr_e.push(["言語はタイトル画面でのみ変更できます。\nThe language can only be changed on the title screen.","The language can only be changed on the title screen.","언어는 타이틀 화면에서만 변경할 수 있습니다.\nThe language can only be changed on the title screen.","語言只能在標題屏幕上更改。\nThe language can only be changed on the title screen.","语言只能在标题屏幕上更改。\nThe language can only be changed on the title screen."]);
_tr_e.push(["ゲーム全体の音量を変更します。","Adjust the game\'s overall volume.","게임 전체의 음량을 변경합니다.","變更遊戲整體的音量。","变更游戏整体的音量。"]);
_tr_e.push(["BGMの音量を変更します。","Adjust the volume of the background music.","BGM의 음량을 변경합니다","變更背景音樂的音量。","变更背景音乐的音量。"]);
_tr_e.push(["BGSの音量を変更します。","Adjust the volume of the background sounds.","BGS의 음량을 변경합니다.","變更背景音效的音量。","变更背景音效的音量。"]);
_tr_e.push(["MEの音量を変更します。","Adjust the volume of featured songs.","ME의 음량을 변경합니다.","變更音樂的音量。","变更音乐的音量。"]);
_tr_e.push(["SEの音量を変更します。","Adjust the volume of the sound effects.","SE의 음량을 변경합니다.","變更音效的音量。","变更音效的音量。"]);
_tr_e.push(["ゲームで使用するキーの割り当てを自由に編集することができます。","Edit the assignment of keys used in the game freely.","게임에서 사용하는 키의 할당을 자유롭게 편집 할 수 있습니다.","自由編輯遊戲中使用的鍵的分配。","自由编辑游戏中使用的键的分配。"]);
_tr_e.push(["ゲームパッドボタンの割り当てを自由に編集することができます。","Edit the assignment of gamepad buttons freely.","게임 패드 버튼의 할당을 자유롭게 편집 할 수 있습니다.","自由編輯遊戲手柄按鈕的分配。","自由编辑游戏手柄按钮的分配。"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["最大ＨＰ","Max HP","최대 HP","最大HP","最大HP"]);
_tr_e.push(["攻撃力","ATK","공격력","攻擊力","攻擊力"]);
_tr_e.push(["運","LCK","운","運氣","運氣"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["セーブしました。","saved a file.","저장되었습니다.","保存了一個文件。","保存了一个文件。"]);
_tr_e.push(["%1は%2を投げた！","%1 threw %2!","%1은 %2를 던졌다!","%1扔了%2！","%1扔了%2！"]);
_tr_e.push(["","","","",""]);
_tr_e.push(["","","","",""]);
_tr_e.push(["","","","",""]);
_tr_e.push(["","","","",""]);
})();
    //・データベース用語
    (function(){
_tr_s["ID"]=["テキスト","英語","韓国語","繁体文字","簡体文字"];
_tr_s["0"]=["レベル","Level","Lv","Lv","Lv"];
_tr_s["1"]=["Lv","Lv","Lv","Lv","Lv"];
_tr_s["2"]=["ＨＰ","HP","HP","ＨＰ","ＨＰ"];
_tr_s["3"]=["HP","HP","HP","HP","HP"];
_tr_s["4"]=["スタミナ","","","",""];
_tr_s["5"]=["ｽﾀﾐﾅ","","","",""];
_tr_s["6"]=["ＴＰ","","","",""];
_tr_s["7"]=["TP","","","",""];
_tr_s["8"]=["経験値","EXP","EXP","EXP","EXP"];
_tr_s["9"]=["EXP","EXP","EXP","EXP","EXP"];
_tr_s["10"]=["最大ＨＰ","Max HP","최대 HP","最大HP","最大HP"];
_tr_s["11"]=["最大ＭＰ","Max MP","최대 MP","最大MP","最大MP"];
_tr_s["12"]=["攻撃力","ATK","공격력","攻擊力","攻击力"];
_tr_s["13"]=["防御力","DEF","방어력","防禦力","防御力"];
_tr_s["14"]=["魔法力","MAT","마법력","魔法攻擊","魔法攻击"];
_tr_s["15"]=["魔法防御","MDE","마법 방어","魔法防禦","魔法防御"];
_tr_s["16"]=["行動回数","STAMINA","행동 횟수","行動數量","行动数量"];
_tr_s["17"]=["運","LUCK","운","運氣","运气"];
_tr_s["18"]=["命中率","DEX","명중률","準確性","准确性"];
_tr_s["19"]=["回避率","EVA","회피율","逃避率","逃避率"];
_tr_s["20"]=["戦う","Fight","싸움","戰鬥","战斗"];
_tr_s["21"]=["逃げる","Escape","도망","逃跑","逃跑"];
_tr_s["22"]=["攻撃","Attack","공격","攻擊","攻击"];
_tr_s["23"]=["防御","Defense","방어","防禦","防御"];
_tr_s["24"]=["アイテム","Item","아이템","項目","项目"];
_tr_s["25"]=["スキル","Skill","스킬","技能","技能"];
_tr_s["26"]=["装備","Equipment","장비","設備","设备"];
_tr_s["27"]=["ステータス","Status","상태","狀態","状态"];
_tr_s["28"]=["並び替え","Sort","정렬","分類","分类"];
_tr_s["29"]=["セーブ","Save","저장","保存","保存"];
_tr_s["30"]=["ゲーム終了","Game end","게임 종료","遊戲結束","游戏结束"];
_tr_s["31"]=["オプション","Settings","옵션","選項","选项"];
_tr_s["32"]=["武器","Weapon","무기","武器","武器"];
_tr_s["33"]=["防具","Armor","방어구","裝甲","装甲"];
_tr_s["34"]=["大事なもの","Important","중요한 것","重要的事情","重要的事情"];
_tr_s["35"]=["装備","Equipment","장비","設備","设备"];
_tr_s["36"]=["最強装備","Strongest equipment","최강 장비","最強的設備","最强的设备"];
_tr_s["37"]=["全て外す","Remove all","모두 제거","全部刪除","全部删除"];
_tr_s["38"]=["ニューゲーム","New game","뉴 게임","新遊戲","新游戏"];
_tr_s["39"]=["コンティニュー","Continue","컨티뉴","繼續","继续"];
_tr_s["40"]=["null","null","null","無效","无效"];
_tr_s["41"]=["タイトルへ","Go to Title","제목에","標題","标题"];
_tr_s["42"]=["やめる","Quit","종료","放棄","放弃"];
_tr_s["43"]=["null","null","null","無效","无效"];
_tr_s["44"]=["購入する","Buy","구매하기","購買","购买"];
_tr_s["45"]=["売却する","Sell","판매","賣","卖"];
_tr_s["actionFailure"]=["しかし %1には効かなかった。","%1 was not affected.","%1은 효과를받지 않았다.","%1未受影響。","%1未受影响。"];
_tr_s["actorDamage"]=["%1は %2 のダメージを受けた。","%1 incurred %2 damage.","%1(은)는 %2의 대미지를 받았다.","%1受到傷害%2了。","%1受到了%2伤害。"];
_tr_s["actorDrain"]=["%1は%2を %3 奪われた。","%1 lifted the %2's wallet and stole %3 from it.","%1는 %2(을)를 %3 빼앗겼다.","%1的%2被奪走%3了。","%1的%2被夺走了%3。"];
_tr_s["actorGain"]=["%1の%2が %3 回復した。","%1's %2 recovered by %3.","%1의 %2(이)가 %3 회복되었다.","%1的%2恢復%3了。","%1的%2恢复了%3。"];
_tr_s["actorLoss"]=["%1の%2が %3 減った。","The %1's %2 was reduced by %3.","%1의 %2(이)가 %3 감소했다.","%1的%2減少%3了。","%1的%2减少了%3。"];
_tr_s["actorNoDamage"]=["%1には効いていない。","%1 was not affected.","%1은 효과를받지 않았다.","%1未受影響。","%1未受影响。"];
_tr_s["actorNoHit"]=["しかし %1には当たらなかった。","However, %1 was not hit.","그러나 %1에게는 맞지 않았다.","但是，沒有命中%1。","但是，没有命中%1。"];
_tr_s["actorRecovery"]=["%1の%2が %3 回復した。","%1's %2 recovered by %3.","%1의 %2(이)가 %3 회복되었다.","%1的%2恢復%3了。","%1的%2恢复了%3。"];
_tr_s["alwaysDash"]=["常時ダッシュ","","","",""];
_tr_s["bgmVolume"]=["BGM 音量","BGM volume","BGM 볼륨","BGM 音量","BGM 音量"];
_tr_s["bgsVolume"]=["BGS 音量","BGS volume","BGS 볼륨","BGS 音量","BGS 音量"];
_tr_s["buffAdd"]=["%1の%2が上がった。","%1's %2 increased.","%1의 %2(이)가 상승했다.","%1的%2上升了。","%1的%2上升了。"];
_tr_s["buffRemove"]=["%1の%2が元に戻った。","%1's %2 returned to normal.","%1의 %2(이)가 원래대로 되돌아왔다.","%1的%2恢復原狀了。","%1的%2复原了。"];
_tr_s["commandRemember"]=["コマンド記憶","","","",""];
_tr_s["counterAttack"]=["%1の反撃！","","","",""];
_tr_s["criticalToActor"]=["痛恨の一撃！！","Critical hit!","회심의 일격!","重擊！","重击！"];
_tr_s["criticalToEnemy"]=["会心の一撃！！","Critical hit!","회심의 일격!","重擊！","重击！"];
_tr_s["debuffAdd"]=["%1の%2が下がった。","%1's %2 decreased.","%1의 %2(이)가 감소했다.","%1的%2下降了。","%1的%2下降了。"];
_tr_s["defeat"]=["","","","",""];
_tr_s["emerge"]=["","","","",""];
_tr_s["enemyDamage"]=["%1は %2 のダメージを受けた。","%1 incurred %2 damage.","%1(은)는 %2의 대미지를 받았다.","%1受到傷害%2了。","%1受到了%2伤害。"];
_tr_s["enemyDrain"]=["%1は%2を %3 奪われた。","%1 lifted the %2's wallet and stole %3 from it.","%1는 %2(을)를 %3 빼앗겼다.","%1的%2被奪走%3了。","%1的%2被夺走了%3。"];
_tr_s["enemyGain"]=["%1の%2が %3 回復した。","%1's %2 recovered by %3.","%1의 %2(이)가 %3 회복되었다.","%1的%2恢復%3了。","%1的%2恢复了%3。"];
_tr_s["enemyLoss"]=["%1の%2が %3 減った。","The %1's %2 was reduced by %3.","%1의 %2(이)가 %3 감소했다.","%1的%2減少%3了。","%1的%2减少了%3。"];
_tr_s["enemyNoDamage"]=["しかし %1には当たらなかった。","However, %1 was not hit.","그러나 %1에게는 맞지 않았다.","但是，沒有命中%1。","但是，没有命中%1。"];
_tr_s["enemyNoHit"]=["","","","",""];
_tr_s["enemyRecovery"]=["%1の%2が %3 回復した。","%1's %2 recovered by %3.","%1의 %2(이)가 %3 회복되었다.","%1的%2恢復%3了。","%1的%2恢复了%3。"];
_tr_s["escapeFailure"]=["","","","",""];
_tr_s["escapeStart"]=["","","","",""];
_tr_s["evasion"]=["%1は攻撃をかわした。","%1 dodged the attack.","%1(은)는 공격을 피했다.","%1閃掉攻擊了。","%1避开了攻击。"];
_tr_s["expNext"]=["次の%1まで","","","",""];
_tr_s["expTotal"]=["現在の%1","","","",""];
_tr_s["file"]=["ファイル","File","파일","文件","文件"];
_tr_s["levelUp"]=["%1は%2 %3 に上がった！","%1's %2 went up to %3!","%1의 %2이 (가) %3까지 올라갔습니다!","%1的%2升至%3！","%1的%2升至%3！"];
_tr_s["loadMessage"]=["どのファイルをロードしますか？","Choose a file to load","어느 파일을 불러오시겠습니까?","要讀取哪一個檔案呢？","要读取哪一个存档呢？"];
_tr_s["magicEvasion"]=["%1は魔法を打ち消した。","","","",""];
_tr_s["magicReflection"]=["%1は魔法を跳ね返した。","","","",""];
_tr_s["meVolume"]=["ME 音量","ME volume","ME 볼륨","ME 音量","ME 音量"];
_tr_s["obtainExp"]=["%1 の%2を手に入れた。","You earned %1 %2!","\\n[1]는 경험치 %1(을)를 획득했다!","\\n[1]獲得經驗值 %1 了！","\\n[1]获得%1经验值！"];
_tr_s["obtainGold"]=["お金を %1\G 手に入れた。","You got %1\\g!","%1\\g의 돈을 획득했다!","獲得金錢 %1\\g 了！","获得%1\\g金钱！"];
_tr_s["obtainItem"]=["%1を入手した。","","","",""];
_tr_s["obtainSkill"]=["%1を覚えた！","","","",""];
_tr_s["partyName"]=["%1たち","","","",""];
_tr_s["possession"]=["持っている数","","","",""];
_tr_s["preemptive"]=["","","","",""];
_tr_s["saveMessage"]=["どのファイルにセーブしますか？","Choose a save slot","어느 파일에 저장하시겠습니까?","要儲存在哪一個檔案呢？","要储存在哪一个存档呢？"];
_tr_s["seVolume"]=["SE 音量","SE volume","SE 볼륨","SE 音量","SE 音量"];
_tr_s["substitute"]=["%1が%2をかばった。","","","",""];
_tr_s["surprise"]=["","","","",""];
_tr_s["useItem"]=["%1は%2を使った！","","","",""];
_tr_s["victory"]=["","","","",""];
})();
    //・ヘルプ
    (function(){

_tr_h[0]=["ー移動と方向転換ー\n\n\\c[14]移動\\c[0]\n　←↑↓→\n\\c[14]方向転換\\c[0]\n　Shiftキー + ←↑↓→\n　または \\n[1]をタップして引っ張る","-Move and change direction-\n\n\\c[14]Move\\c[0]\n←↑↓→\n\\c[14]Change Direction\\c[0]\n[Shift key] + ←↑↓→\nOr tap on \\n[1] and drag her to where you want to move.","-이동과 방향 전환-\n\n\\c[14]이동\\c[0]\n←↑↓→\n\\c[14]방향 전환\\c[0]\nShift키 + ←↑↓→\n또는 \\n[1]를 선택한 채로 드래그한다.","―移動並改變方向―\n\n\\c[14]移動\\c[0]\n←↑↓→\n\\c[14]改變方向\\c[0]\nShift鍵 + ←↑↓→\n或者是點擊 \\n[1]進行拖曳","─移动并改变方向─\n\n\\c[14]移动\\c[0]\n←↑↓→\n\\c[14]改变方向\\c[0]\nShift键 + ←↑↓→\n或者是点击\\n[1]进行拖曳"];
_tr_h[1]=["ー基本操作（画面のボタン）ー\n\n※画面に表示されるため、覚える必要はありません。\n\\c[14]武器チェンジ（Fキー）\\c[0]\n武器を素早く切り替えることができる\n\\c[14]武器を拾う（Dキー）\\c[0]\n目の前または足元の武器を拾う\n\\c[14]待機（Sキー）\\c[0]\nそのターン何もせずに過ごす　長押しで連続\n\\c[14]薬草（Aキー）\\c[0]\n持っている薬草を１つ使う　ＨＰや状態異常を回復する","-Basic Controls (Screen Buttons)-\n\n*Since they\'re displayed on the screen, you don\'t need to memorize them. \n\\c[14]Change Weapon (F key)\\c[0]\nQuickly switch between your weapons.\n\\c[14]Pick Up Weapon (D key)\\c[0]\nPick up weapons in front of you.\n\\c[14]Wait (S key)\\c[0]\nDo nothing on your turn. Hold down to wait multiple turns.\n\\c[14]Use Herb (A key)\\c[0]\nUse an herb to restore your HP and cure any status ailments.","-기본 조작(화면 버튼)-\n\n※화면에 표시되기 때문에 기억하지 않아도 좋다.\n\\c[14]무기 변경하기(F키)\\c[0]\n무기를 재빠르게 변경할 수 있다.\n\\c[14]무기 줍기(D키)\\c[0]\n눈앞 또는 발밑의 무기를 줍는다.\n\\c[14]대기(S키)\\c[0]\n해당 턴에 아무 행동도 취하지 않고 기다린다. 길게 누르면 연속 대기 가능.\n\\c[14]약초(A키)\\c[0]\n갖고 있는 약초를 1개 사용한다. HP 및 상태 이상을 회복한다.","─基本操作（畫面上的按鍵）─\n\n※它不需要记忆，因为它显示在屏幕上。\n\\c[14]更換武器（F鍵）\\c[0]\n可以迅速更換武器\n\\c[14]撿起武器（D鍵）\\c[0]\n撿起眼前或腳邊的武器\n\\c[14]待機（S鍵）\\c[0]\n這一回合什麼都不做 長按不放\n\\c[14]藥草（A鍵）\\c[0]\n使用一個持有的藥草，可恢復HP和異常狀態。","─基本操作（画面上的按键）─\n\n※它不需要记忆，因为它显示在屏幕上。\n\\c[14]更换武器（F键）\\c[0]\n可以迅速更换武器\n\\c[14]捡起武器（D键）\\c[0]\n捡起眼前或脚边的武器\n\\c[14]待机（S键）\\c[0]\n这一回合什么都不做 长按不放\n\\c[14]药草（A键）\\c[0]\n使用一个持有的药草，可恢复HP和异常状态。"];
_tr_h[2]=["ー武器の拾い方ー\n\n「Dキー」または「武器を拾うボタン」を押すと\n目の前または足元の武器を拾います。","-How to Pick Up Weapons-\n\nPress the D key or Pick Up Weapon button to\npick up the weapon in front of you.","-무기 줍는 방법-\n\n「D키」 또는 「무기 줍기 버튼」을 누르면\n눈앞 또는 발밑의 무기를 줍습니다.","─撿起武器的方式─\n\n按下「D鍵」或是「撿起武器按鍵」，\n就能撿起眼前或腳邊的武器。","─捡起武器的方式─\n\n按下「D键」或是「捡起武器按键」，\n就能捡起眼前或脚边的武器。"];
_tr_h[3]=["ー武器の拾い方（敵の場合）ー\n\n敵を武器として拾う場合、\n普通に拾おうとしても振り払われてしまい拾えません。\n\n敵は、\\c[14]ＨＰを減らして弱らせることで拾うことができます\\c[0]。\nＨＰが少なくなるほど、拾える確率がアップします。","-How to Pick Up Weapons (Enemies)-\n\nWhen picking up an enemy to use as a weapon,\nthey will dodge your attempt if you try to pick them up right away.\n\n\\c[14]First, lower their HP, then pick them up.\\c[0]\nThe lower HP, the better the probability of picking up.","-무기 줍는 방법(적의 경우)-\n\n적을 무기로 주울 경우,\n다른 물건을 줍듯이 주울 수는 없습니다.\n\n\\c[14]적은 HP를 줄여서 약하게 만든 후에 주울 수 있습니다.\\c[0]\nHP가 적을수록 주울 확률이 올라갑니다.","─撿起武器的方式（對象為敵人時）─\n\n若要把敵人當作武器撿起，\n就算用普通的方式撿起，也會被甩開而無法撿起。\n\n\\c[14]必須要先削弱敵人的HP，才能撿起。\\c[0]\nHP越低，拾取的可能性越大。","─捡起武器的方式（对象为敌人时）─\n\n若要把敌人当作武器捡起，\n就算用普通的方式捡起，也会被甩开而无法捡起。\n\n\\c[14]必须要先削弱敌人的HP，才能捡起。\\c[0]\nHP越低，拾取的可能性越大。"];
_tr_h[4]=["ー攻撃のしかたと範囲の確認ー\n\n\\c[14]攻撃\\c[0]\n「Enterキー」または「攻撃ボタン」または\n「敵をタップ」することで攻撃します。\n\\c[14]攻撃範囲の確認\\c[0]\n「Shiftキー」または「\\n[1]をタップ」することで\n装備中の武器の攻撃範囲を確認できます。\n\n\n\n初めて入手した武器は、どんな武器かがわかりません。\n（未解禁状態）\n\n未解禁状態の武器で敵を一度でも攻撃すると、\n威力や効果などの詳細な情報がわかるようになります。","-Attacking & Confirming Attack Range-\n\n\\c[14]Attack\\c[0]\nPress the Enter key, Attack button,\nor tap on the enemy to attack them.\n\\c[14]Confirming Attack Range\\c[0]\nPress the Shift key or tap on \\n[1] to\ncheck the attack range of your equipped weapon.\n\n\n\nThe attack range is unknown for newly obtained weapons.\n(Unused Status)\n\nDetails about an unused weapon, such as its ATK and effects,\nwill be revealed once it is used against an enemy.","-공격 방법과 범위 확인-\n\n\\c[14]공격\\c[0]\n「Enter키」 또는 「공격 버튼」 또는\n「적을 선택」하면 공격할 수 있습니다.\n\\c[14]공격 범위 확인\\c[0]\n「Shift키」 또는 「\\n[1]를 선택」하면\n장비 중인 무기의 공격 범위를 확인할 수 있습니다.\n\n\n\n처음으로 획득한 무기는 어떤 무기인지 알 수 없습니다.\n(미확인 상태)\n\n아직 확인되지 않은 무기로 적을 한 번이라도 공격하면\n위력과 효과 등 자세한 정보를 알 수 있게 됩니다.","─攻擊方式與確認範圍─\n\n\\c[14]攻擊\\c[0]\n按下「Enter鍵」或「攻擊按鍵」或\n「點擊敵人」，即可發動攻擊。\n\\c[14]─確認攻擊範圍─\\c[0]\n按下「Shift鍵」或「點擊\\n[1]」\n即可確認裝備中武器的攻擊範圍。\n\n\n\n第一次拿到的武器，會不知道是什麼樣的武器。\n（未解禁狀態）\n\n拿未解禁狀態的武器攻擊過敵人之後，\n就能知道武器的威力、效果等詳細資訊。","─攻击方式与确认范围─\n\n\\c[14]攻击\\c[0]\n按下「Enter键」或「攻击按键」或\n「点击敌人」，即可发动攻击。\n\\c[14]─确认攻击范围─\\c[0]\n按下「Shift键」或「点击\\n[1]」\n即可确认装备中武器的攻击范围。\n\n\n\n第一次拿到的武器，你会对其一无所知。\n（未解禁状态）\n\n拿未解禁状态的武器攻击敌人之后，\n就能知道武器的威力，效果等详细信息。"];
_tr_h[5]=["ー武器の耐久値についてー\n\n武器にはそれぞれ耐久値（\\i[470]）が存在します。\n耐久値は敵に武器を当てるたび 1 ずつ減少し、\n0 になると武器が壊れてなくなってしまいます。\n\\c[14]壊れる直前の攻撃は、必ず会心の一撃\\c[14]となります。\n\n壊れた武器は持ち物から消え、\nもし拾ったものであれば拾った場所に復活します。\nただし、耐久値や金床での強化はリセットされます。","-Weapon Durability-\n\nEvery weapon has a durability (\\i[470]) value.\nThis value goes down by 1 for every hit landed on an enemy,\nand the weapon will break when its durability reaches 0.\nAn attack just before the weapon breaks is always a critical hit.\n\nBroken weapons will disappear from your inventory.\nIf it's a weapon you picked up, it will rematerialize in the place you found\nit. Its durability and any anvil enhancements will be reset, however.","-무기의 내구도에 대하여-\n\n무기에는 각각 내구도(\\i[470])가 존재합니다.\n내구도는 적에게 무기를 맞힐 때마다 1씩 줄어들며\n0이 되면 무기가 부서져 사라집니다.\n무기가 손상 직전의 공격은 반드시 크리티컬 히트가됩니다.\n\n부서진 무기는 소지품에서 사라지며,\n주운 무기의 경우는 주웠던 장소에 다시 나타납니다.\n단, 내구도와 모루에서 강화한 내용은 모두 초기화됩니다.","──關於武器的耐久值──\n\n每種武器都存在著耐久值（\\i[470]）。\n每當武器擊中敵人時，耐久值就會減1，\n當耐久值歸0後，武器就會壞掉消失。\n武器爆發前的攻擊始終是一個重擊。\n\n壞掉的武器會從攜帶物中消失，\n如果是撿到的武器，會復活在撿到它的地方。\n但是耐久值和使用鐵砧進行的強化都會重置。","──关于武器的耐久值──\n\n每种武器都有耐久值（\\i[470]）。\n每当武器击中敌人时，耐久值就会减1，\n当耐久值归0后，武器就会坏掉消失。\n武器爆发前的攻击始终是一个重击。\n\n坏掉的武器会从携带物中消失，\n如果是捡到的武器，会在捡到它的地方重新出现。\n但是耐久值和使用铁砧进行的强化属性都会重置。"];
_tr_h[6]=["ーメニューと装備の変え方ー\n\n「Escキー」または「メニューボタン」を押すことで\nメニューを開くことができます。\n\nメニューでは様々な操作が行えます。\n「武器一覧」では武器の管理や装備の変更を行えます。\n\n武器の装備の変更は、通常時に\n「Fキー」または「武器チェンジボタン」を押すことでも\n素早く行うことができます。","-Menu & Changing Equipment-\n\nPress the Esc key or Menu button to\nopen the menu.\n\nYou can perform various actions from the menu.\nWith Inventory, you can manage your current weapons or equip a different one.\n\nYou can also quickly change weapons at anytime by\npressing the F key or the Change Weapon\nbutton.","-메뉴와 장비 변경 방법-\n\n「Esc키」 또는 「메뉴 버튼」을 누르면\n메뉴를 열 수 있습니다.\n\n메뉴에서는 다양한 조작을 할 수 있습니다.\n「무기 목록」에서는 무기를 관리하거나 장비를 변경할 수 있습니다.\n\n무기 변경은\n「F키」 또는 「무기 변경 버튼」을 눌러서\n빠르게 실행할 수도 있습니다.","─選單與裝備的更改方式─\n\n按下「Esc鍵」或是「選單按鍵」，\n即可打開選單。\n\n在選單中可以進行各種操作。\n在「武器一覽」中可以管理武器或變更裝備。\n\n平時要變更武器裝備時，\n也可以透過按下「F鍵」或是「更換武器按鍵」，\n即可迅速更換武器。","─选单与装备的更改方式─\n\n按下「Esc键」或是「选单按键」，\n即可打开选单。\n\n在菜单中可以进行各种操作。\n在「武器一览」中可以管理武器或变更装备。\n\n平时要变更武器装备时，\n也可以按下「F键」或是「更换武器按键」，\n即可迅速更换武器。"];
_tr_h[7]=["ー武器の破棄ー\n\nメニューの「武器一覧」から\n武器を破棄することができます。\n\n破棄した武器は持ち物から消え、\nもし拾ったものであれば拾った場所に復活します。\nただし、耐久値や金床での強化はリセットされます。","-Discarding Weapons-\n\nYou can discard weapons from\nthe Inventory option in the menu.\n\nDiscarded weapons will disappear from your inventory.\nIf it's a weapon you picked up, it will rematerialize in the place you found\nit. Its durability and any anvil enhancements will be reset, however.","-무기 버리기-\n\n메뉴의 「무기 목록」에서\n무기를 버릴 수 있습니다.\n\n버린 무기는 소지품에서 사라지며,\n주운 무기의 경우는 주웠던 장소에 다시 나타납니다.\n단, 내구도와 모루에서 강화한 내용은 모두 초기화됩니다.","─捨棄武器─\n\n從選單的「武器一覽」中，\n可以選擇捨棄武器。\n\n捨棄的武器會從攜帶物中消失，\n如果是撿到的武器，會復活在撿到它的地方。\n但是耐久值和使用鐵砧進行的強化都會重置。","─舍弃武器─\n\n从菜单的「武器一览」中，\n可以选择舍弃武器。\n\n舍弃的武器会从携带物中消失，\n如果是捡到的武器，会在捡到它的地方重新出现。\n但是耐久值和使用铁砧进行的强化属性都会重置。"];
_tr_h[8]=["ー必殺奥義ー\n\n必殺奥義を覚えると、敵に攻撃を当てるたび\n少しずつ必殺奥義ゲージが溜まっていきます。\n\n必殺奥義ゲージが最大のとき\n「Enterキー」または「攻撃ボタン」を長押しすることで、\n装備中の武器を使って強力な必殺奥義を\n発動することができます。\n必殺奥義では、武器の耐久値は減りません。","-Special Moves-\n\nIf you have learned a special move, the SP Gauge\nwill be gradually filled with every successful attack.\n\nWhen the SP Gauge is full, press and hold the\nEnter key or Attack button\nto unleash a powerful attack\nwith your current weapon.\nEven if you use the Special Move, the durability of your weapon\nwill not decrease.","-필살오의-\n\n필살오의를 익히면 적에게 공격을 맞힐 때마다\n필살오의 게이지가 조금씩 쌓입니다.\n\n필살오의 게이지가 가득 찼을 때\n「Enter키」 또는 「공격 버튼」을 길게 누르면\n장비 중인 무기를 사용하여 강력한 필살오의를\n발동할 수 있습니다.\n필살오의를 사용해도 무기의 내구도가 감소하지 않습니다.","─必殺奧義─\n\n學會必殺奧義後，每當攻擊命中敵人\n就會逐漸累積必殺奧義量表。\n\n當必殺奧義量表累積到最大時，\n長按「Enter鍵」或「攻擊按鍵」，\n即可使用裝備中的武器\n發動強大的必殺奧義。","─必杀奥义─\n\n学会必杀奥义后，每当攻击命中敌人\n就会逐渐累积必杀奥义量表。\n\n当必杀奥义量表累积到最大时，\n长按「Enter键」或「攻击按键」，\n即可使用装备中的武器\n发动强大的必杀奥义。"];
_tr_h[9]=["ー金床の説明ー\n\n各地に置かれている金床を調べることで\n武器の手入れをしたり強化したりできます。\n\n\\c[14]修理する\\c[0]\n手持ちの武器すべての耐久値が最大まで回復します。\n\\c[14]研ぐ\\c[0]\n武器の威力と会心率が上昇します。\n10回使うと効果が切れます。重ねがけはできません。\n\n\\c[14]メッキする\\c[0]\n武器の耐久値が減らなくなります。\n5回耐久値減少を防ぐと効果が切れます。\nメッキは何回も重ねがけすることができます。\n\\c[14]鍛える\\c[0]\n武器を鍛えることでレベルを上げ、\n武器の威力を永久的に上昇させます。\n\n修理する以外の操作には「\\i[141]アルマの石」が必要です。\n\\i[141]アルマの石は魔物を倒したときに手に入る他、\n様々な場面で手に入ります。","-Anvils-\n\nUsing anvils placed in various parts of the world\nallows you to service and upgrade your weapons.\n\n\\c[14]Repair\\c[0]\nRestores the durability of your current weapon to its maximum value.\n\\c[14]Sharpen\\c[0]\nIncreases your weapon's ATK and Critical Hit Rate.\nIf you attack 10 times, the effect will expire. It can not be overlaid.\n\n\\c[14]Galvanize\\c[0]\nPrevents your weapon's durability from decreasing.\nThe effect will be lost if the durability value decrease is prevented\nfive times. Galvanizing can be stacked many times.\n\\c[14]Temper\\c[0]\nTemper a weapon to raise its level.\nIt will permanently boost the weapon's ATK value.\n\nAny action besides repairing your weapons requires the use of \\i[141]Arma Stones.\n\\i[141]Arma Stones can be acquired by defeating enemies\nand can also be found in various other locations.","-모루-\n\n각지에 놓여 있는 모루를 조사하면\n무기를 손질하거나 강화할 수 있습니다.\n\n\\c[14]수리하기\\c[0]\n가지고 있는 모든 무기의 내구도가 최대로 회복됩니다.\n\\c[14]연마하기\\c[0]\n무기의 위력과 크리티컬 확률이 상승합니다.\n10 번 공격하면 효과가 꺼집니다. 거듭 벼랑 수 없습니다.\n\n\\c[14]도금하기\\c[0]\n무기의 내구도가 줄어들지 않게 됩니다.\n5 회 내구도 감소를 방지하는 효과가 꺼집니다.\n도금은 여러 번 거듭 벼랑 수 있습니다.\n\\c[14]단련하기\\c[0]\n무기를 단련하면 레벨이 상승하며\n무기의 위력이 영구적으로 상승합니다.\n\n수리하기 이외의 조작은 「\\i[141]알마의 돌」이 필요합니다.\n\\i[141]알마의 돌은 마물을 쓰러뜨렸을 때 이외에도\n다양한 장면에서 획득할 수 있습니다.","──鐵砧的說明──\n\n調查位於各地的鐵砧，\n即可保養或強化武器。\n\n\\c[14]修理\\c[0]\n完全恢復身上所有武器的耐久值。\n\\c[14]研磨\\c[0]\n提升武器的威力與暴擊機率。\n如果你攻擊10次，效果將會到期。它不能覆蓋。\n\n\\c[14]鍍金\\c[0]\n讓武器的耐久值不會下降。\n如果防止耐久性值降低五次，則效果將會喪失。\n鍍金可以堆疊多次。\n\\c[14]鍛造\\c[0]\n藉由鍛造提升武器等級，\n讓武器的威力永久提升。\n\n除了修理以外的動作，都需要使用「\\i[141]魂石」。\n\\i[141]魂石除了打倒魔物時可以獲得，\n也能在許多場面中獲得。","──铁砧的说明──\n\n调查位于各地的铁砧，\n即可保养或强化武器。\n\n\\c[14]修理\\c[0]\n完全恢复身上所有武器的耐久值。\n\\c[14]研磨\\c[0]\n提升武器的威力与暴击机率。\n使用数次后效果就会消失。\n\n\\c[14]鍍金\\c[0]\n让武器的耐久值不会下降。\n使用数次后效果就会消失。\n\\c[14]鍛造\\c[0]\n借由锻造提升武器等级，\n让武器的威力永久提升。\n\n除了修理以外的动作，都需要使用「\\i[141]魂石」。\n\\i[141]魂石除了打倒魔物时可以获得，\n也能在许多场面中获得。"];
_tr_h[10]=["ー鍛冶屋の説明ー\n\n自宅の電話から鍛冶屋に電話をかけることができます。\n\n鍛冶屋では、発見済みの武器を作成してもらうことができます。\n未解禁状態の（使ったことがない）武器でも可能です。\n作成した武器はすぐに受け取ることができます。\n\n作成には、作成費用（お金）が必要です。\nまた、武器ごとに作成できる数に限りがあります。","-Blacksmith-\n\nYou can use your home telephone to contact the blacksmith.\n\nThe blacksmith allows you to forge weapons that you've previously discovered.\nYou can even forge weapons that you have found but not used yet.\nThe weapons you forge are available immediately.\n\nYou simply need to cover the forging cost (money).\nHowever, there is a limit to the number of each weapon you can forge.","-대장간-\n\n집에서 전화로 대장간에 전화를 걸 수 있습니다.\n\n대장간에서는 이미 발견한 무기를 제작할 수 있습니다.\n미확인 상태의(사용한 적이 없는) 무기도 가능합니다.\n제작한 무기는 바로 받을 수 있습니다.\n\n제작하려면 제작 비용(돈)이 필요합니다.\n또한, 무기마다 제작할 수 있는 횟수가 제한되어 있습니다.","──鐵匠鋪說明──\n\n可以從家裡打電話給鐵匠鋪。\n\n已經發現的武器可以請鐵匠鋪打造出來。\n即使是尚未解禁（不曾使用過）的武器也可以打造。\n打造好的武器可以馬上領取。\n\n想要打造武器，必須花費打造費用（錢）。\n此外，每種武器可以打造的數量有限。","──铁匠铺说明──\n\n可以从家里打电话给铁匠铺。\n\n已经发现的武器可以请铁匠铺打造出来。\n即使是尚未解禁（不曾使用过）的武器也可以打造。\n打造好的武器可以马上领取。\n\n想要打造武器，必须花费打造费用（钱）。\n此外，每种武器可以打造的数量有限。"];
_tr_h[11]=["ー商品棚の説明ー\n\n不要な武器を商品棚に置くことで、武器を売ることができます。\n\n商品棚一つにつき武器は５個まで置くことができます。\n置いた武器は、現実時間で 10～20分後 ぐらいに売れます。\nゲームを閉じている間も武器は自動で売れます。\n\n売れた武器は無くなってしまいます。\nもし拾ったものであれば拾った場所に復活します。\nただし、耐久値や金床での強化はリセットされます。","-Shop Table-\n\nYou can sell weapons by placing them on the shop table.\n\nUp to five weapons can be placed on a single table.\nOnce placed, the weapons are sold in approximately 10 to 20 minutes of\nreal time. The weapons will automatically be sold even if you close the game.\n\nYou will lose the weapons once they are sold.\nIf it's a weapon you picked up, it will rematerialize in the place you found\nit. Durability and anvil enhancements will be reset, however.","-상품 선반-\n\n불필요한 무기를 상품 선반에 놓아 두면 무기를 팔 수 있습니다.\n\n하나의 상품 선반에 5개의 무기까지 올려 놓을 수 있습니다.\n올려 놓은 무기는 현실 시간으로 10~20분 정도에 팔립니다.\n게임을 꺼 두어도 무기는 자동으로 팔립니다.\n\n팔린 무기는 사라져 버립니다.\n만일 주운 무기라면 주웠던 장소에 다시 나타납니다.\n단, 내구도와 모루에서 강화한 내용은 모두 초기화됩니다.","──商品架說明──\n\n將不要的武器放在商品架上，即可販售武器。\n\n每一個商品架最多可以放置五項武器。\n上架的武器在經過現實時間10～20分鐘後會賣出。\n武器在遊戲關閉的期間也會自動賣出。\n\n賣出的武器會消失。\n如果是撿到的武器，會復活在撿到的地方。\n但是耐久值和使用鐵砧進行的強化都會重置。","──商品架说明──\n\n将不要的武器放在商品架上，即可贩售武器。\n\n每一个商品架最多可以放置五件武器。\n上架的武器会在现实时间10～20分钟后卖出。\n武器在游戏关闭的期间也会自动卖出。\n\n卖出的武器会消失。\n如果是捡到的武器，会在捡到它的地方重新出现。\n但是耐久值和使用铁砧进行的强化属性都会重置。"];
_tr_h[12]=["ー妹の説明ー\n\n妹のぽん美に話しかけることで、\n\\n[1]のＨＰや状態異常を全回復することができます。\nさらに、お金を払って様々な強化を行ったり、\n\\i[261]薬草を買うことができます。\n\n\\c[14]手伝ってもらう\\c[0]\n店や武器庫の拡張など、様々なことをしてもらえます。\n手伝ってもらうにはお金が必要です。\n\n\\c[14]\\i[261]薬草を買いたい\\c[0]\n\\i[261]薬草を好きなだけ買うことができます。\n薬草の代金は買うたびにほんの少しずつ上がっていきます。","-Younger Sister-\n\nYou can talk to your younger sister, Ponmi,\nto restore \\n[1]'s HP and cure any status ailments.\nYou can also buy various upgrades\nand \\i[261]herbs.\n\n\\c[14]Ask for Help\\c[0]\nYou can receive various forms of assistance, such as expansions to\nthe Shop orArmory. Note that receiving such assistance will cost\nyou money.\n\n\\c[14]Buy \\i[261]Herbs\\c[0]\nYou can buy \\i[261]herbs from her at any time.\nThe cost of herbs will rise ever so slightly with each purchase.","-여동생-\n\n여동생 폰미에게 말을 걸면\n\\n[1]의 HP와 상태 이상을 모두 회복할 수 있습니다.\n그리고 돈을 지불해 다양한 강화를 하거나\n\\i[261]약초를 살 수도 있습니다.\n\n\\c[14]도움을 받는다\\c[0]\n가게나 무기고 확장 등 다양한 도움을 받을 수 있습니다.\n도움을 받으려면 돈이 필요합니다.\n\n\\c[14]\\i[261]약초를 사고 싶다\\c[0]\n\\i[261]약초를 원하는 만큼 살 수 있습니다.\n약초 값은 살 때마다 조금씩 올라갑니다.","──妹妹的說明──\n\n和妹妹器美說話，\n即可完全恢復\\n[1]的HP和異常狀態。\n此外，還能付錢進行各種強化，\n或是購買\\i[261]藥草。\n\n\\c[14]請妹妹幫忙\\c[0]\n可以請妹妹幫忙做各種事情，比如擴增商店或武器庫等等。\n若要請她幫忙必須花錢。\n\n\\c[14]想要購買\\i[261]藥草\\c[0]\n可以盡情購買\\i[261]藥草。\n每購買一次藥草，價格會逐漸上漲。","──妹妹的说明──\n\n和妹妹器美说话，\n即可完全恢复\\n[1]的HP和异常状态。\n此外，还能付钱进行各种强化，\n或是购买\\i[261]药草。\n\n\\c[14]请妹妹帮忙\\c[0]\n可以请妹妹帮忙做各种事情，比如扩展商店或武器库等等。\n若要请她帮忙必须花钱。\n\n\\c[14]想要购买\\i[261]药草\\c[0]\n可以尽情购买\\i[261]药草。\n每购买一次药草，价格便会上涨一次。"];
_tr_h[13]=["ー魔王城の説明ー\n\n魔王城は集めた武器を駆使して戦う高難易度ステージです。\n階層の魔物を全滅させることで次の階層へと進むことができます。\n\n魔王城に挑むには、武器を８つ以上持っていく必要があります。\n階層ごとに、手持ちの武器の中からランダムで４つの武器が選ばれます。\n選ばれた武器のみで魔物と戦わなくてはなりません。\n\n最上階で待ち構えている魔王を倒すため、\n強力な武器を揃えて挑みましょう。\n\nなお、詳細なルールは以下のとおりです。\n\n・中間地点以外ではセーブができない。\n・魔王城では武器庫を開けない。\n・魔王城内で耐久値が減少したり壊れたりした武器は、\n魔王城を出るともとに戻る。\n・魔王城内で武器を拾うことはできるが、武器庫に送られる。","-Demonic Castle-\n\nThe Demonic Castle is a difficult stage that allows you to fight\nwith the weapons you've gathered.\nDefeating all of the enemies on a floor will allow you to proceed\nto the next floor.\n\nYou will need to have at least eight weapons in your inventory to take\non the Demonic Castle.\nOn each floor, four of your weapons will be chosen at random.\nYou will be forced to fight the monsters you encounter with these weapons.\nBe sure to gather an array of weapons that will allow you to emerge\nvictorious against the Demon Lord on the top floor.\n\nThe rules for the Demonic Castle are as follows.\n\n- The Armory cannot be accessed while in the Demonic Castle.\n- Weapons that suffer durability reductions or are broken\nwill return to normal once you leave the Demonic Castle.\n- Any weapons you pick up in the Demonic Castle will be sent to the Armory.","-마왕성-\n\n마왕성은 수집한 무기를 활용하여 싸우는 높은 난이도의 스테이지입니다.\n해당 층의 마물을 전멸시키면 다음 층으로 올라갈 수 있습니다.\n\n마왕성에 도전하려면 무기를 8개 이상 가지고 있어야 합니다.\n층별로 가지고 있는 무기 중에서 무작위로 4개의 무기가 선택됩니다.\n선택된 무기로만 마물과 싸워야 합니다.\n\n최상층에서 기다리는 마왕을 쓰러뜨리기 위해\n강력한 무기를 모아서 도전해 보세요.\n\n자세한 규칙은 아래와 같습니다.\n\n・마왕성에서는 무기고를 열 수 없다.\n・ 마왕성 안에서 내구도가 감소하거나 망가진 무기는\n마왕성을 나오면 원래대로 돌아온다.\n・ 마왕성 내에서 무기를 주울 수는 있지만 바로 무기고로 보내진다.","──魔王城說明──\n\n魔王城是高難度關卡，必須運用蒐集的武器戰鬥。\n殲滅樓層裡的魔物便能前往下一樓層。\n\n若要挑戰魔王城，必須攜帶八項以上的武器。\n每抵達一層樓，會從攜帶武器裡隨機挑選出四項武器。\n只能使用挑選出的武器與魔物戰鬥。\n\n為了打倒在頂樓等待的魔王，\n湊齊強大的武器前去挑戰吧。\n\n此外，詳細規則如下：\n\n‧在魔王城無法開啟武器庫。\n‧武器若在魔王城內減少耐久值或是毀損，\n離開魔王城後便會恢復原狀。\n‧在魔王城內可以撿武器，但會收進武器庫。","──魔王城说明──\n\n魔王城是高难度关卡，必须运用搜集的武器战斗。\n歼灭楼层里的魔物便能前往下一楼层。\n\n若要挑战魔王城，必须携带八种以上的武器。\n每抵达一层楼，会从携带武器里随机挑选出四种武器。\n只能使用挑选出的武器与魔物战斗。\n\n为了打倒在顶楼等待的魔王，\n凑齐强大的武器前去挑战吧。\n\n此外，详细规则如下：\n\n‧在魔王城无法开启武器库。\n‧武器若在魔王城内减少耐久值或是毁损，\n离开魔王城后便会恢复原状。\n‧在魔王城内可以捡武器，但会收进武器库。"];
_tr_h[13]=["ー魔王城の説明ー\n\n魔王城は、武器を駆使して戦う高難易度ステージです。\n階層の魔物を全滅させることで次の階層へと進むことができます。\n\n魔王城には武器を持ち込むことができますが、\n最初は持ち込んだ武器を使うことができません。\n階層を進んでいくと、威力が低い順に1つずつ使えるようになります。\n\n魔王城の各階層には、ランダムで武器が落ちているので、\n最初はそれらを駆使して戦わなくてはなりません。\n魔王城は20階まであり、途中にセーブができる中間地点があります。\n最上階で待ち構えている魔王を倒すため、がんばりましょう。\n\nなお、詳細なルールは以下のとおりです。\n\n・中間地点以外ではセーブができない。\n・魔王城内では武器庫への出し入れができない。\n・持ち込んだ武器は、耐久値が減少したり壊れたりしても\n　魔王城を出るとすべてもとにもどる。\n・魔王城内で拾った武器は、魔王城を出ると武器庫に送られる。\n　ただし、武器庫がいっぱいの場合は自動で破棄されてしまう。","-Demonic Castle-\n\nThe Demonic Castle is a difficult stage that allows you to fight\nwith the weapons you've gathered. Defeating all of the enemies\non a floor will allow you to proceed o the next floor.\n\nYou can bring weapons to the Demonic Castle, but you can\'t\nuse weapons at first. As you progress through the hierarchy,\nyou will be able to use one by one in order of weak power.\n\n\nThere are some random weapons in the Demonic Castle, so you\nmust fight with them at first. The Demonic Castle has up to the\n20th floor and has an intermediate point where has an Anvil.\n\nDo your best to defeat the Demon Lord waiting at the top floor.\n\nThe detailed rules are as follows.\n\n· Saves can not be made except at intermediate points.\n· In the Demonic Castle, you can\'t put weapons in / out of the armory.\n· Even if the durability of weapons you brought decreases or they break,\n  they returns to the original all the way when leaving the Demon Castle.\n· Weapons picked up in the Demonic Castle are sent to the armory\n  when they leave Demonic Castle.\n  However, if the armory is full, they will be automatically discarded.","-마왕성-\n\n마왕성은 수집한 무기를 활용하여 싸우는 높은 난이도의 스테이지입니다.\n해당 층의 마물을 전멸시키면 다음 층으로 올라갈 수 있습니다.\n\n마왕성에는 무기를 반입 할 수 있지만 처음에는 가져온 무기를\n사용할 수 없습니다.\n계층을 나가면 위력이 낮은 순서대로 하나씩 사용할 수있게됩니다.\n\n마왕성 각 계층은 랜덤으로 무기가 떨어져 있기 때문에 처음에는\n그들을 이용하여 싸우지 않으면 안됩니다.\n마왕성 20 층까지 있으며, 길가 세이브있는 중간 지점이 있습니다.\n최상층에서 기다리고있는 마왕을 물리 치기 위해, 힘냅시다.\n\n또한 상세한 규칙은 다음과 같습니다.\n\n· 중간 지점 이외의 장소에서 저장 수 없다.\n· 마왕성내에서 무기고에 무기를 출입 할 수 없다.\n· 들여온 무기는 내구도가 감소되거나 손상도 마왕성을 나오면 모든\n　근원으로 돌아갑니다.\n· 마왕성내에서 주운 무기는 마왕성을 나오면 무기고에 보내진다.\n　그러나 무기고가 꽉 차면 자동으로 파기되어 버린다.","──魔王城說明──\n\n魔王城是高難度關卡，必須運用蒐集的武器戰鬥。\n殲滅樓層裡的魔物便能前往下一樓層。\n\n你可以把武器帶到魔王城堡，但你不能首先使用武器。\n當你走上樓，你將能夠以弱電的順序逐一使用。\n\n魔王城堡中有一些隨機武器，所以你一開始必須和他們一起戰鬥。\n魔王城堡已經達到了20樓，有一個中間點，有一個鐵砧。\n\n盡力打敗在頂層等候的惡魔王。\n\n詳細規則如下。\n\n·除中間點外不能保存。\n·在魔王城堡中，你不能將武器放入/放出武器庫。\n·即使你帶來的武器的耐久性降低或者它們破壞，\n 它們你在離開魔王城堡時也會一直恢復原狀。\n·在魔王城堡中拾取的武器你在離開魔王城堡時被送往武器庫。\n 但是，如果武器庫已滿，它們將被自動丟棄。","──魔王城说明──\n\n魔王城是高难度关卡，必须运用搜集的武器战斗。\n歼灭楼层里的魔物便能前往下一楼层。\n\n你可以把武器带到魔王城堡，但你不能首先使用武器。\n当你走上楼，你将能够以弱电的顺序逐一使用。\n\n魔王城堡中有一些随机武器，所以你一开始必须和他们一起战斗。\n魔王城堡已经达到了20楼，有一个中间点，有一个铁砧。\n\n尽力打败在顶层等候的恶魔王。\n\n详细规则如下。\n\n·除中间点外不能保存。\n·在魔王城堡中，你不能将武器放入/放出武器库。\n·即使你带来的武器的耐久性降低或者它们破坏，\n 它们你在离开魔王城堡时也会一直恢复原状。\n·在魔王城堡中拾取的武器你在离开魔王城堡时被送往武器库。\n 但是，如果武器库已满，它们将被自动丢弃。"];
_tr_h[14]=["ー便利コマンド（キーボード）ー\n\nキーボード操作では、知っておくと便利なコマンドがあります。\n\n\\c[14]通常時「Fキー + 決定」\\c[0]\n手持ちの武器の中からランダムで装備します。\n\\c[14]通常時「Fキー + キャンセル」\\c[0]\n武器の装備を外します。\n\n\n\n\\c[14]武器一覧表示時「Fキー + 決定」\\c[0]\n空いているスペースに、武器庫内の武器を\nランダムで配置します。\n\\c[14]武器一覧表示時「Fキー + キャンセル」\\c[0]\n手持ちの武器をすべて武器庫に送ります。","-Useful Commands (Keyboard)-\n\nThere are several useful commands that will make controlling the game\nwith a keyboard easier.\n\n\\c[14]Standard: F key + Enter\\c[0]\nRandomly equips a weapon in your inventory.\n\\c[14]Standard: F key + Cancel\\c[0]\nUnequips your weapon.\n\n\n\\c[14]While viewing Inventory: F key + Enter\\c[0]\nRandomly moves a weapon from the Armory\nto one of your empty Inventory slots.\n\\c[14]While viewing Inventory: F key + Cancel\\c[0]\nSends all of the weapons in your Inventory to the Armory.","-간편 명령(키보드)-\n\n알아 두면 편리한 키보드 조작 명령이 있습니다.\n\n\\c[14]평상시 「F키+결정」\\c[0]\n가지고 있는 무기 중에서 무작위로 장비합니다.\n\\c[14]평상시 「F키+취소」\\c[0]\n장비한 무기를 해제합니다.\n\n\n\n\\c[14]무기 목록 표시 시 「F키+결정」\\c[0]\n빈 공간에 무기고 내의 무기를\n무작위로 배치합니다.\n\\c[14]무기 목록 표시 시 「F키+취소」\\c[0]\n가지고 있는 무기를 모두 무기고로 보냅니다.","──快捷指令（鍵盤）──\n\n若是使用鍵盤操作，事先知道以下指令會很方便。\n\n\\c[14]平時「F鍵+決定」\\c[0]\n隨機裝備一項攜帶的武器。\n\\c[14]平時「F鍵+取消」\\c[0]\n卸下武器。\n\n\n\n\\c[14]顯示武器一覽時「F鍵+決定」\\c[0]\n在空位裡隨機放置武器庫裡的\n武器。\n\\c[14]顯示武器一覽時「F鍵+取消」\\c[0]\n將攜帶武器全部收進武器庫。","──快捷指令（键盘）──\n\n若是使用键盘操作，则可先了解下列指令，方便操作。\n\n\\c[14]平时「F键+决定」\\c[0]\n随机装备一项携带的武器。\n\\c[14]平时「F键+取消」\\c[0]\n卸下武器。\n\n\n\n\\c[14]显示武器一览时「F键+决定」\\c[0]\n在空位里随机放置武器库里的\n武器。\n\\c[14]显示武器一览时「F键+取消」\\c[0]\n将携带武器全部收进武器库。"];
_tr_h[15]=["ーゲームパッドでの操作ー\n\n\\c[14]攻撃・調べる・決定\\c[0]　　　A\n\\c[14]メニュー・キャンセル\\c[0]　　B\n\\c[14]武器チェンジ\\c[0]　　　　　　X\n\\c[14]武器を拾う（Dキー）\\c[0]　　 Y\n\\c[14]待機（Sキー）\\c[0]　　　　　 R1\n\\c[14]薬草（Aキー）\\c[0]　　　　　 L1\n\\c[14]方向転換・攻撃範囲確認\\c[0]　(L2 or R2) + ←↑↓→\n\\c[14]空を飛ぶ\\c[0]　　　　　　　　SELECT","-Gamepad Controls-\n\n\\c[14]Attack/Search/Confirm                   \\c[0] A\n\\c[14]Menu/Cancel                             \\c[0] B\n\\c[14]Change Weapon                           \\c[0] X\n\\c[14]Pick Up Weapon (D key)                  \\c[0] Y\n\\c[14]Wait (S key)                            \\c[0] R1\n\\c[14]Use Herb (A key)                        \\c[0] L1\n\\c[14]Change Direction/Confirm Attack Range   \\c[0] (L2 or R2) + ←↑↓→\n\\c[14]Fly                                     \\c[0] SELECT","-게임 패드 조작 방법-\n\n\\c[14]공격, 조사, 결정           \\c[0]  A\n\\c[14]메뉴, 취소                 \\c[0]  B\n\\c[14]무기 변경하기              \\c[0]  X\n\\c[14]무기 줍기(D키)             \\c[0]  Y\n\\c[14]대기(S키)                  \\c[0]  R1\n\\c[14]약초(A키)                  \\c[0]  L1\n\\c[14]방향 전환・공격 범위 확인  \\c[0]  (L2 or R2) + ←↑↓→\n\\c[14]하늘 날기                  \\c[0]  SELECT","──控制器的操作──\n\n\\c[14]攻擊、調查、決定\\c[0] 　　　A\n\\c[14]選單、取消\\c[0] 　　　　　　B\n\\c[14]更換武器\\c[0] 　　　　　　　X\n\\c[14]撿起武器（D鍵）\\c[0] 　　　 Y\n\\c[14]待機（S鍵）\\c[0] 　　　　　 R1\n\\c[14]藥草（A鍵）\\c[0] 　　　　　 L1\n\\c[14]改變方向、確認攻擊範圍\\c[0] (L2 or R2) + ←↑↓→\n\\c[14]空中飛翔\\c[0] 　　　　　　　SELECT","──控制器的操作──\n\n\\c[14]攻擊、調查、決定\\c[0] 　　　A\n\\c[14]選單、取消\\c[0] 　　　　　　B\n\\c[14]更換武器\\c[0] 　　　　　　　X\n\\c[14]撿起武器（D鍵）\\c[0] 　　　 Y\n\\c[14]待機（S鍵）\\c[0] 　　　　　 R1\n\\c[14]藥草（A鍵）\\c[0] 　　　　　 L1\n\\c[14]改變方向、確認攻擊範圍\\c[0] (L2 or R2) + ←↑↓→\n\\c[14]空中飛翔\\c[0] 　　　　　　　SELECT"];
_tr_h[101]=["ー空飛ぶほうきー\n\n武器\\i[1238]空飛ぶほうきを装備しているとき、\n「Gキー」または「空を飛ぶボタン」を押すことで\n空を飛ぶことができます。\n\n空を飛ぶことで、敵に見つからず素早く移動できます。\n\nただし、屋内や洞窟の中では空を飛ぶことができません。","-Flying Broom-\n\nEquip the \\i[1238]Flying Broom\nand press the G key or Fly button\nto rise up into the sky.\n\nFlying lets you move quickly across the map and avoid enemy\nencounters.\n\nHowever, flying is not possible in rooms or caves.","-하늘을 나는 빗자루-\n\n무기 \\i[1238]하늘을 나는 빗자루를 장비하고 있을 때\n「G키」 또는 「하늘 날기 버튼」을 누르면\n하늘을 날 수 있습니다.\n\n하늘을 날면 적에게 발견되지 않고 신속하게 이동할 수 있습니다.\n\n단, 실내 또는 동굴 안에서는 사용할 수 없습니다.","──飛天掃帚──\n\n裝備武器\\i[1238]飛天掃帚時，\n按下「G鍵」或是「空中飛翔按鍵」，\n即可在空中飛翔。\n\n在空中飛翔可以不被敵人發現而快速移動。\n\n但是在屋內或洞窟內無法飛翔。","──飛天掃帚──\n\n裝備武器\\i[1238]飛天掃帚時，\n按下「G鍵」或是「空中飛翔按鍵」，\n即可在空中飛翔。\n\n在空中飛翔可以不被敵人發現而快速移動。\n\n但是在屋內或洞窟內無法飛翔。"];
_tr_h[102]=["ー空飛ぶ武器ー\n\n魔女クレモアの魔法により、\nすべての武器で空を飛べるようになりました。\n\n好きな武器を装備中に\n「Gキー」または「空を飛ぶボタン」を押すことで、\nその武器にまたがって空を飛ぶことができます。","-Flying Weapons-\n\nThanks to the magic of Claymore the Witch,\nevery weapon now gives you the ability to fly.\n\nEquip any weapon and press\nthe G key or Fly button to\nmount the weapon and rise up into the sky.","-하늘을 나는 무기-\n\n마녀 크레모아의 마법에 의해\n모든 무기로 하늘을 날 수 있게 되었습니다.\n\n원하는 무기를 장비한 상태에서\n「G키」 또는 「하늘 날기 버튼」을 누르면\n그 무기를 타고 하늘을 날 수 있습니다.","──飛天武器──\n\n因為魔女克蕾摩爾的魔法，\n所有武器都變得可以在空中飛翔了。\n\n裝備好喜歡的武器，\n按下「G鍵」或是「空中飛翔按鍵」，\n即可騎乘該武器在空中飛翔。","──飛天武器──\n\n因為魔女克蕾摩爾的魔法，\n所有武器都變得可以在空中飛翔了。\n\n裝備好喜歡的武器，\n按下「G鍵」或是「空中飛翔按鍵」，\n即可騎乘該武器在空中飛翔。"];
_tr_h[103]=["ーボスとの再戦ー\n\n洞窟に出現するボスを武器として拾っていない場合、\n再び洞窟を訪れることでボスと再戦することができます。\n\nなお、一度でも拾うと再戦できなくなります。","-Boss Rematches-\n\nIf you did not pick up the bosses that appear in the caves\nto use as weapons,\nyou can revisit the caves to fight those same bosses again.\n\nYou will no longer be able to fight them once you have picked\nthem up, however.","-보스 재도전-\n\n동굴에 출현하는 보스를 무기로서 줍지 못한 경우\n다시 동굴에 가면 보스에게 재도전할 수 있습니다.\n\n한 번이라도 줍게 되면 재도전할 수 없습니다.","──重新與頭目戰鬥──\n\n如果沒有成功將洞窟裡出現的頭目撿來當武器，\n可以再次造訪洞窟，重新與頭目戰鬥。\n\n此外，只要曾經撿來當武器一次，便無法重新戰鬥。","──重新與頭目戰鬥──\n\n如果沒有成功將洞窟裡出現的頭目撿來當武器，\n可以再次造訪洞窟，重新與頭目戰鬥。\n\n此外，只要曾經撿來當武器一次，便無法重新戰鬥。"];
_tr_h[104]=["ー魔王城クリア後ー\n\n魔王城クリア後も、魔王は２０階で待ち構えています。\n何度でも挑戦することができます。\nただし、出現する魔物が強くなっているので注意。","-After Completing the Demonic Castle-\n\nThe Demon Lord will still await you on the 20th floor even after completing the Demonic Castle.\nYou can challenge him as many times as you wish.\nJust be aware that the monsters that appear will be even stronger.","-마왕성 클리어 후-\n\n마왕성을 클리어 하면 마왕은 20층으로 이동합니다.\n몇 번이고 도전할 수 있습니다.\n단, 출현하는 마물이 강해지므로 조심하십시오.","──魔王城通關後──\n\n即使在魔王城通關後，魔王依然會在20樓備戰。\n可以挑戰魔王無數次。\n但是出現的魔物會變得更強，請務必小心。","──魔王城通關後──\n\n即使在魔王城通關後，魔王依然會在20樓備戰。\n可以挑戰魔王無數次。\n但是出現的魔物會變得更強，請務必小心。"];
_tr_h[104]=["ー魔王城クリア後ー\n\n魔王城クリア後も、魔王城に挑むことができます。\nその際、以下の地点からスタートできます。\n・1階から\n・中間地点から\n・最上階から\n最上階には、再び魔王が待ち構えているかも・・・？","-After Completing the Demonic Castle-\n\nEven after clearing the Demonic Castle, you can challenge here again.\nAt that time, you can start from the following points.\n· From the 1F\n· From the intermediate point\n· From the top floor\nOn the top floor, the Demon Lord may be waiting again ...?","-마왕성 클리어 후-\n\n마왕 성을 클리어 후에도 다시 여기에 도전 할 수 있습니다.\n그 때, 다음 지점에서 시작할 수 있습니다.\n· 1 층에서\n· 중간 지점에서\n· 최상층에서\n최상층에는 다시 마왕이 기다리고 있을지・・・?","──魔王城通關後──\n\n即使在魔王城通關後，魔王依然會在20樓備戰。\n可以挑戰魔王無數次。\n但是出現的魔物會變得更強，請務必小心。","──魔王城通關後──\n\n即使在魔王城通關後，魔王依然會在20樓備戰。\n可以挑戰魔王無數次。\n但是出現的魔物會變得更強，請務必小心。"];
_tr_h[105]=["ートロフィーについてー\n\nものすごい功績を挙げると、知らない誰かから\nトロフィーをもらえることがあります。\n\nもらったトロフィーは、自宅で確認することができます。\n目指せ、トロフィーコンプリート！","-Trophies-\n\nAchieving incredible feats in the game will earn\nyou trophies from an unknown individual.\n\nThe trophies you receive will be put on display at your home.\nTry to get 'em all!","-트로피-\n\n공적을 많이 쌓으면 정체 불명의 누군가로부터\n트로피를 받을 수 있습니다.\n\n받은 트로피는 집에서 확인할 수 있습니다.\n트로피 컬렉션을 완성해 보세요!","──關於獎杯──\n\n立下極為了不起的功績，\n就會有不知名人士贈送獎杯。\n\n可以在家裡確認獲得的獎杯。\n目標是蒐集完所有獎杯！","──關於獎杯──\n\n立下極為了不起的功績，\n就會有不知名人士贈送獎杯。\n\n可以在家裡確認獲得的獎杯。\n目標是蒐集完所有獎杯！"];
_tr_h[106]=["ー洞窟についてー\n\n洞窟の中では、「武器庫」が\n使用できなくなるので注意しましょう。\n\nなお、拾った武器を武器庫に送ることは可能です。","-Caves-\n\nBe careful while in caves, as you will not have\naccess to your Armory while inside them.\n\nHowever, you are able to send weapons you pick up to the Armory.","-동굴-\n\n동굴 안에서는 「무기고」를\n사용할 수 없으므로 주의하십시오.\n\n하지만 주운 무기를 무기고로 보낼 수는 있습니다.","──關於洞窟──\n\n在洞窟裡無法使用「武器庫」，\n請務必多加留意。\n\n但是，可以把撿到的武器收進武器庫。","──關於洞窟──\n\n在洞窟裡無法使用「武器庫」，\n請務必多加留意。\n\n但是，可以把撿到的武器收進武器庫。"];
_tr_h[107]=["ームシとりアミー\n\n武器\\i[1363]ムシとりアミは特殊な武器です。\n振って敵に当ててもダメージを与えることはできませんが、\n振ることで、目の前の武器を捕まえることができます。\n（「武器を拾う」と同効果です）\n\nそして\\i[1363]ムシとりアミは、通常の「武器を拾う」に比べて\n敵を拾うときの成功確率が３倍になっています。\n\n敵を拾いたいときに有効活用しましょう。","-Insect Net-\n\nThe \\i[1363]Insect Net is a special weapon.\nYou cannot inflict damage on enemies by swinging it at them,\nbut you can use it to pick up any weapons you encounter.\n(It offers the same effect as the Pick Up Weapon command.)\n\nThe \\i[1363]Insect Net offers triple the success\nrate of the normal Pick Up Weapon option.\n\nUse this weapon to increase your chances of picking up enemies.","-곤충 채집망-\n\n무기 \\i[1363]곤충 채집망은 특수한 무기입니다.\n휘둘러서 적을 때려도 대미지를 줄 수는 없지만,\n휘둘러서 눈 앞의 무기를 획득할 수 있습니다.\n(「무기 줍기」와 같은 효과)\n\n그리고 \\i[1363]곤충 채집망은 「무기 줍기」에 비해\n적을 주울 때의 성공 확률이 3배 상승합니다.\n\n적을 줍고 싶을 때 효과적으로 활용해 보세요.","──捕蟲網──\n\n武器\\i[1363]捕蟲網是一項特殊的武器。\n揮動後即使擊中敵人也無法造成傷害，\n但可以捕捉面前的武器。\n（效果與「撿起武器」相同。）\n\n而且\\i[1363]捕蟲網與平常的「撿起武器」相比，\n撿起敵人的成功率高出了三倍。\n\n想要撿起敵人時就有效活用這項武器吧。","──捕蟲網──\n\n武器\\i[1363]捕蟲網是一項特殊的武器。\n揮動後即使擊中敵人也無法造成傷害，\n但可以捕捉面前的武器。\n（效果與「撿起武器」相同。）\n\n而且\\i[1363]捕蟲網與平常的「撿起武器」相比，\n撿起敵人的成功率高出了三倍。\n\n想要撿起敵人時就有效活用這項武器吧。"];
_tr_h[108]=["ーエンディングについてー\n\n「\\i[1009]ツヨスギテ＝クサハエルを装備した状態で、ぽん美に話しかける」\nことで、エンディングへと移行します。\n\nエンディングは、トロフィーを集めることで種類が増えます。\n気が向いたらトロフィーを集めてみましょう。","-Endings-\n\nSpeaking to Ponmi with the \\i[1009]LOL Sword equipped\nwill initiate the game's ending.\n\nEarning trophies will increase the number of endings available.\nTry collecting additional trophies if it suits your fancy.","-엔딩-\n\n「\\i[1009]너무강해서 현웃터지는 소드를 장비한 상태에서 폰미에게\n　말을 건다」\n그러면 엔딩이 시작됩니다.\n\n엔딩은 트로피 수집 갯수에 따라 종류가 늘어납니다.\n트로피를 더 많이 모아 보세요.","──關於結局──\n\n「裝備\\i[1009]暴強‧草叢生劍的狀態下和器美說話」，\n便會進入結局。\n\n結局種類會因為蒐集獎杯而增加。\n若是感興趣，就去蒐集獎杯看看吧。","──关于结局──\n\n「装备\\i[1009]无敌‧草丛生剑的状态下和器美说话」，\n便会进入结局。\n\n结局种类会因为搜集到的奖杯个数而增加。\n若是感兴趣，就去搜集奖杯看看吧。"];
_tr_h[109]=["ー敵の強化ー\n\n洞窟をクリアするたび、そこらへんの魔物が強化されます。\n\n名前や見た目は変わりませんが、ＨＰや攻撃力が上がります。\nそのぶん、獲得経験値量が大きく増加します。","-Formidable Foes-\n\nThe monsters in a cave will grow more formidable each time you\ncomplete that cave.\n\nTheir name and appearance will remain the same, but\ntheir HP and ATK will rise.\nThey will also provide far more EXP than they did previously.","- 강해지는 적-\n\n동굴을 클리어 할 때마다 주변의 마물이 강해집니다.\n\n이름이나 외모는 변하지 않지만, HP와 공격력이 상승합니다.\n그만큼 획득할 수 있는 경험치는 크게 증가합니다.","──敵人的強化──\n\n洞窟每通關一次，那裡的魔物會變得更強。\n\n雖然名字和外觀不會改變，但是HP和攻擊力會上升。\n同時也會大幅增加獲得的經驗值。","──敌人的强化──\n\n洞窟每通关一次，那里的魔物会变得更强。\n\n虽然名字和外观不会改变，但是HP和攻击力会上升。\n同时也会大幅增加获得的经验值。"];
_tr_h[110]=["ー基本的な流れー\n\\c[14]\n①武器を拾う\n↓\n②拾った武器で敵と戦い、武器の情報を解禁する（図鑑を埋める）\n↓\n③武器を売る\n\\c[0]\nこれを繰り返し、経験値を稼いで強くなりながら\n伝説の武器を探して冒険しよう！","-Basic Game Flow-\n\\c[14]\n1. Pick up weapons\n↓\n2. Fight enemies with the weapons you pick up and unlock info about them\n(add to the Weapons Gallery)\n↓\n3. Sell weapons\n\\c[0]\nRepeat the process to earn EXP, grow stronger,\nand find the legendary weapon you seek!","-기본적인 흐름-\n\\c[14]\n①무기 줍기\n↓\n②주운 무기로 적과 싸워 무기 정보 획득(도감 등록)\n↓\n③무기 팔기\n\\c[0]\n이것을 반복하여 경험치를 쌓아 강해지면서\n전설의 무기를 찾아 모험을 떠나세요!","──基本流程──\n\\c[14]\n①撿起武器\n↓\n②使用撿到的武器與敵人戰鬥，解禁武器資訊（蒐集圖鑑）\n↓\n③販售武器\n\\c[0]\n重複這些步驟，賺取經驗值來增強實力，\n並同時前往冒險，尋找傳說中的武器吧！","──基本流程──\n\\c[14]\n①捡起武器\n↓\n②使用捡到的武器与敌人战斗，解禁武器信息（搜集图鉴）\n↓\n③贩售武器\n\\c[0]\n重复这些步骤，赚取经验值来增强实力，\n并同时前往冒险，寻找传说中的武器吧！"];
_tr_h[111]=["ーすごい馬ー\n\nすごい馬に話しかけると、\nレベルアップボーナスの再振り分けができます。\n「ＨＰ」「攻撃力」「運」のうち 一つの能力を1段階アップさせ、\n代わりに他の一つを1段階もとにもどします。\n\nただし、大金が必要です。","-Wild Stallion-\n\nYou can talk to the wild stallion to\nredistribute your level-up bonuses.\nYou can boost one level of your HP, ATK, or LCK\nstats by reducing the level of another stat by one.\n\nHowever, this process requires a great deal of money.","-대단한 말-\n\n'대단한 말'에게 말을 걸면\n레벨업 보너스를 다시 분배할 수 있습니다.\n「HP」 「공격력」 「운」 중 하나의 능력을 1단계 향상시키고\n그 대신 다른 하나는 1단계가 내려갑니다.\n\n다만, 많은 비용이 필요합니다.","──壯馬──\n\n和壯馬說話，\n便能重新分配升級獎勵。\n可以提升一階「HP」、「攻擊力」、「運氣」裡的一項能力，\n同時降低一階另外一項能力。\n\n但是，需要花費大把金錢。","──壯馬──\n\n和壯馬說話，\n便能重新分配升級獎勵。\n可以提升一階「HP」、「攻擊力」、「運氣」裡的一項能力，\n同時降低一階另外一項能力。\n\n但是，需要花費大把金錢。"];
_tr_h[112]=["ー井戸チャレンジー\n\n光り輝く井戸を調べることで\n高難易度チャレンジに挑むことができます。\n高スコアを狙ってがんばりましょう。\nまた、アルマの石が手に入りやすくなっています。\n\n井戸チャレンジは今後もアップデートで\n追加されていくかもしれません。","-Well Challenges-\n\nCheck shining wells to take on a\ndifficult challenge within them.\nDo your best to achieve a high score.\nIt's also an easier way to get Arma Stones.\n\nMore Well Challenges might be added\nin future updates.","-우물에 도전하자-\n\n빛나는 우물을 조사하면\n높은 난이도의 미션에 도전할 수 있습니다.\n고득점을 노려보세요!\n그리고 알마의 돌을 획득하기 쉬워집니다.\n\n우물 도전은 앞으로 업데이트되어\n계속 추가될 지도 모릅니다.","──水井挑戰──\n\n調查閃閃發光的水井\n即可進行高難度挑戰。\n努力獲得高分吧。\n此外，在挑戰裡可以比較容易獲得魂石。\n\n日後或許也會在更新時，\n追加水井挑戰的內容。","──水井挑戰──\n\n調查閃閃發光的水井\n即可進行高難度挑戰。\n努力獲得高分吧。\n此外，在挑戰裡可以比較容易獲得魂石。\n\n日後或許也會在更新時，\n追加水井挑戰的內容。"];
_tr_h[113]=["ー生き延びろ！サバイバル島ー\n\n無数の魔物が出現する島で、1ターンでも長く生き延びましょう。\n\\n[1]が死んだ時点で終了です。\n\n・武器を 16個まで持ち込むことが可能。\n・「ターン経過」、「魔物を倒す」ことでスコアが増加。\n・薬草は初期状態で5個所持。魔物が落とすこともある。\n・チャレンジ中は魔物を倒した際に経験値とお金が手に入らない。\n　また、チャレンジ中のセーブは不可。","-Try to Survive! Survival Island-\n\nTry to survive as many turns as possible on an island with countless\nmonsters.It's over as soon as \\n[1] dies.\n\n- You can take up to 16 weapons with you.\n- Boost your score by surviving turns and defeating monsters.\n- Initially, you\'ll be able to carry five herbs. Monsters may also\n  drop them.\n- You will not receive any EXP or money for defeating monsters.\n  You will not be able to save, either.","-살아남아라! 서바이벌 아일랜드-\n\n무수히 많은 마물이 출현하는 섬. 1턴이라도 오래 살아 남아야 합니다.\n\\n[1]의 사망 시점에 종료됩니다.\n\n・무기를 16개까지 가지고 갈 수 있습니다.\n・「턴 경과」 「마물 퇴치」로 점수를 획득합니다.\n・약초는 5개만 가지고 시작하지만 마물이 약초를 떨어뜨릴 때도 있습니다.\n・도전 중에는 마물을 쓰러뜨려도 경험치와 돈을 획득할 수 없습니다.\n　도전 중에는 게임 저장이 불가능합니다.","──活下去！生存島──\n\n在會出現無數魔物的島嶼上，一回合也好，盡可能地生存下去吧。\n當\\n[1]死亡時，挑戰便會結束。\n\n‧最多可以攜帶16項武器。\n‧分數會透過「經過回合」、「打倒魔物」來增加。\n‧藥草在初期狀態下擁有五棵。打倒魔物也會掉落藥草。\n‧在挑戰中打倒魔物不會獲得經驗值與金錢。\n 此外，挑戰中不可存檔。","──活下去！生存島──\n\n在會出現無數魔物的島嶼上，一回合也好，盡可能地生存下去吧。\n當\\n[1]死亡時，挑戰便會結束。\n\n‧最多可以攜帶16項武器。\n‧分數會透過「經過回合」、「打倒魔物」來增加。\n‧藥草在初期狀態下擁有五棵。打倒魔物也會掉落藥草。\n‧在挑戰中打倒魔物不會獲得經驗值與金錢。\n 此外，挑戰中不可存檔。"];
_tr_h[114]=["ー武器投げー\n\n必殺奥義「武器投げ」を使うと、装備中の武器を直線上に投擲します。\n武器を失う代わりに、必ず会心攻撃となり、\n武器の追加効果の発生率が 2倍 になります。\n\n武器投げは必殺奥義ゲージを消費しないため\nいつでも使うことができます。\n\n投げた武器は、もし拾ったものであれば拾った場所に復活します。\nただし、耐久値や金床での強化はリセットされます。","-Weapon throwing-\n\nIf you use the Weapon Throwing, you can throw a weapon  you\'re\nequipping on a straight line. You will loss a weapon, but hitting it\nagainst the enemy always results in a critical hit, and doubling the\nincidence of additional effects on the weapon.\nWeapon Throwing can be used at any time, as they do not consume\nthe SP Gauge.\n\nIf it\'s a weapon you picked up, it will rematerialize in the place you found\nit. Durability and anvil enhancements will be reset, however.","-자 무기 던지고 자-\n\n필살 오의「무기 던지기」를 사용하면 장비중인 무기를\n직선 상에 투척합니다. 무기를 잃어 버립니다 만, 반드시\n회심 공격이 무기의 추가 효과의 발생률이 2 배가됩니다.\n\n무기 던지고는 필살 오의 게이지를 소비하지 않기 때문에\n언제든지 사용할 수 있습니다.\n\n주운 무기의 경우는 주웠던 장소에 다시 나타납니다.\n단, 내구도와 모루에서 강화한 내용은 모두 초기화됩니다.","──武器投擲──\n\n如果你使用「武器投擲」，你可以在一條直線上\n投擲你正在裝備的武器。你會失去一把武器，\n但它總會導致一次重擊，並使武器的額外效果加倍。\n\n武器投擲可以隨時使用，因為它們不使用必殺奧義量表。\n\n如果是撿到的武器，會復活在撿到它的地方。\n但是耐久值和使用鐵砧進行的強化都會重置。","──武器投掷──\n\n如果你使用「武器投掷」，你可以在一条直线上\n投掷你正在装备的武器。你会失去一把武器，\n但它总会导致一次重击，并使武器的额外效果加倍。\n\n武器投掷可以随时使用，因为它们不使用必杀奥义量表。\n\n如果是捡到的武器，会复活在捡到它的地方。\n但是耐久值和使用铁砧进行的强化都会重置。"];
})();
    //・武器文章
    (function(){


_tr_w["eng"][Number('4') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('5') || 0]=['10','斬撃','前３','3','5','200','A unique debugging weapon created by the god of weaponry, Buki.','%1 swung the %2!','The %2 hit the %3 in the head!','The %2\'s aura stimulated the %3!','Fire shot forth from the %2!','Debug Vase','','','3x Attack','It doesn\'t actually exist.','3 squares in front'];
_tr_w["eng"][Number('6') || 0]=['10','斬撃','','','30','1000','A mass-produced copper sword. Anyone can use it, but no one should ever rely on it.','%1 swung the %2!','','','','Traveler\'s Sword','','','','It rests inside the treasure chest in your home.',''];
_tr_w["eng"][Number('7') || 0]=['20','斬撃 / 炎','','','30','2100','A magical sword imbued with the power of fire. One swing is more than enough to fire things up.','%1 swung the %2!','','','The tip of the sword is in flames!','Hellfire Sword','','','Emits fire','It rests inside a treasure chest in the south.',''];
_tr_w["eng"][Number('8') || 0]=['25','斬撃 / 雷','','','30','7500','A legendary sword known far and wide. It emits lightning with every slash.','%1 swung the %2!','','','Lightning shot out from the sword\'s tip!','Excalibur','','','Emits lightning','It rests inside a treasure chest.',''];
_tr_w["eng"][Number('9') || 0]=['60','斬撃','','','999','19999','A golden sword forged by the god of weaponry. It is a legendary weapon that can cut through anything that has and ever will exist.','%1 swung the %2!','','','','LOL Sword','','','','Sorry, but not even I know the answer to that...',''];
_tr_w["eng"][Number('10') || 0]=['15','斬撃','直線２','','20','1700','A huge iron sword with a massive blade. It\'s lighter than it looks and easily swung.','%1 swung the %2!','','','','Big Blade','','','','It rests inside a treasure chest in the north.','2 squares straight line'];
_tr_w["eng"][Number('11') || 0]=['27','斬撃','直線２','','20','3600','A huge sword loved by an ancient race of giants. It can slice an entire continent in half.','%1 swung the %2!','','','','Giant\'s Sword','','','','A convenience store has it in stock.','2 squares straight line'];
_tr_w["eng"][Number('12') || 0]=['12','斬撃','','','30','1200','A short blade made for an assassin. It requires incredible skill to hit the target\'s vital points.','%1 stabbed with the %2!','','','','Assassin\'s Dagger','','','Instant Kill (10%)','It rests inside a treasure chest in the village.',''];
_tr_w["eng"][Number('13') || 0]=['14','斬撃 / 闇','','2','30','4666','A knife cursed by a demon. It can cut up opponents in a flash.','%1 stabbed with the %2!','','','','Demonic Knife','','','2x Attack,Poison (30%)','It rests inside a treasure chest on the northeast shoreline.',''];
_tr_w["eng"][Number('14') || 0]=['16','打撃','','','20','1500','A hammer designed for destruction. It does what\'s asked of it.','%1 swung the %2!','','','','Devastating Hammer','','','Confuse (20%)','It rests inside a treasure chest in the northeast of the village.',''];
_tr_w["eng"][Number('15') || 0]=['28','打撃','','','20','8000','A mace that resembles a star. Whatever you do, don\'t use it for massages.','%1 swung the %2!','','','','Morning Star','','','Confuse (20%)','It\'s sold in the bazaar to the east.',''];
_tr_w["eng"][Number('16') || 0]=['13','斬撃 / 氷','直線３','','30','1800','A spear wielded by the god of the sea. Its three sharp points can pierce nearly anything.','%1 stabbed with the %2!','','','','Trident','','','','It rests inside a treasure chest found within a cave in the Poisonous Swamp.','3 squares straight line'];
_tr_w["eng"][Number('17') || 0]=['26','斬撃 / 炎','直線３','','30','7600','A lance from the depths of Hell. It engulfs everything in flames.','%1 stabbed with the %2!','','','','Hellish Lance','','','','It\'s sold in the bazaar to the east.','3 squares straight line'];
_tr_w["eng"][Number('18') || 0]=['15','斬撃','','','20','2000','A sharp and heavy axe. It can damage even the most heavily armored foes.','%1 swung the %2!','','','','Steel Axe','','','Defense Down (30%)','It rests inside a treasure chest in the northeast.',''];
_tr_w["eng"][Number('19') || 0]=['28','斬撃','','','20','9500','An axe that has no equal. It was once the weapon of a fierce god.','%1 swung the %2!','','','','Giant Axe','','','Defense Down (30%)','It belongs to a man living in a cabin.',''];
_tr_w["eng"][Number('20') || 0]=['8','打撃 / 炎','','','20','2100','A staff loved by a wizard for ages. It burns everything to a crisp.','%1 swung the %2!','','','Fire shot forth from the tip of the staff!','Wizard\'s Staff','','','Casts fire magic','It rests inside a treasure chest found within a cave made of iron.',''];
_tr_w["eng"][Number('21') || 0]=['8','打撃 / 雷','','','20','2500','A scepter imbued with the power of the god of lightning. Swing it to make lightning strike, even in the same place twice.','%1 swung the %2!','','','Lightning shot forth from the tip of the staff!','Lightning Scepter','','','Casts lightning magic','It can be obtained by granting a villager\'s request.',''];
_tr_w["eng"][Number('22') || 0]=['8','打撃 / 氷','','','20','2600','A staff once possessed by an ice witch. It has the power to bring about another ice age.','%1 swung the %2!','','','Ice shot forth from the tip of the staff!','Ice Witch\'s Staff','','','Casts ice magic','It rests inside a treasure chest found in a magical cave.',''];
_tr_w["eng"][Number('23') || 0]=['18','打撃 / 闇','','','20','6666','A staff forged in the fires of Hell. It can summon the powers of darkness.','%1 swung the %2!','','','Darkness shot forth from the tip of the staff!','Devilish Staff','','','Casts EX dark magic','It rests inside a treasure chest found in the Demon Lord\'s castle.',''];
_tr_w["eng"][Number('24') || 0]=['18','打撃 / 光','','','20','9999','A staff a powerful sage once used to seal away the darkness. It possesses holy powers.','%1 swung the %2!','','','Holy energy shot forth from the tip of the staff!','Sage\'s Staff','','','Casts random EX magic','It is owned by a witch living in the woods.',''];
_tr_w["eng"][Number('25') || 0]=['20','斬撃','','','20','1500','A tool for climbing walls. It can be used as a weapon as well.','%1 swung the %2!','','','','Ice Axe','','','Defense Down (50%)','It can be found in the southwest.',''];
_tr_w["eng"][Number('26') || 0]=['17','斬撃','','','20','1300','A tool used to dig holes. It can also be used as a weapon.','%1 swung the %2!','The %2 landed a direct hit on the %3!','It also dug a hole underneath the %3!','','Spade','','','Immobilize - 1 turn (30%)','It can be found laying on the ground in the south.',''];
_tr_w["eng"][Number('27') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('28') || 0]=['7','打撃','前３','','3','300','A typical tree found throughout the land. Overharvesting is becoming a problem lately.','%1 swung the %2!','The %2 hit the %3 in the head!','','','Tree','','','','It can be found pretty much anywhere.','3 squares in front'];
_tr_w["eng"][Number('29') || 0]=['6','打撃','前６','4','3','1000','A bundle of four trees. It packs four times the punch.','%1 swung four trees at once!','The trees hit the %3 in the head!','','','Tree x4','','','4x Attack','It can be found pretty much anywhere.','6 squares in front'];
_tr_w["eng"][Number('30') || 0]=['3','打撃','','','3','50','A small, withered tree. All that lives must eventually die.','%1 swung the %2!','The %2 hit the %3 with a loud cracking sound!','','','Small Withered Tree','','','Guaranteed to leave 1 HP','It can be found in the southeast.',''];
_tr_w["eng"][Number('31') || 0]=['5','打撃','直線２','','3','200','A tree that hasn\'t seen life in ages. There is something kind of sad about it.','%1 swung the %2!','The %2 hit the %3 with a loud cracking sound!','','','Withered Tree','','','Guaranteed to leave 1 HP','It can be found in the southeast.','2 squares straight line'];
_tr_w["eng"][Number('32') || 0]=['9','打撃','','','5','200','A stump that would make a great chair. Just seeing it makes you want to sit down.','%1 swung the %2 with her hands!','The %2 hit the %3 in the head!','The %3 sat down to rest on the %2!','','Stump','','','Immobilize - 1 turn (75%)','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('33') || 0]=['8','打撃','','','2','100','A fallen and decayed tree. It is but a mere decoration to the forest now.','%1 swung the %2!','The %2 hit the %3 in the head!','The %3 was knocked off balance!','','Fallen Tree','','','Immobilize - 1 turn (90%)','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('34') || 0]=['10','打撃','直線２','','6','700','A tree that grows in the south. It reminds you of the tropics.','%1 merrily swung the %2!','The %2 hit the %3 in the head!','','','Palm Tree','','','','It can be found along the southern coast.','2 squares straight line'];
_tr_w["eng"][Number('35') || 0]=['4','斬撃','','','3','100','An orthodox grassy plant in both shape and color. B and C versions also exist.','%1 swung the %2 with her hands!','The %3\'s left index finger was cut by the %2!','','','Grass A','','','','It can be found pretty much anywhere.',''];
_tr_w["eng"][Number('36') || 0]=['5','斬撃','','','3','100','Grass that\'s relatively larger than Grass A. Its size gives it a satisfying heft.','%1 swung the %2 with her hands!','The %3\'s left ring finger was cut by the %2!','','','Grass B','','','','It can be found pretty much anywhere.',''];
_tr_w["eng"][Number('37') || 0]=['4','斬撃','','3','3','150','Bigger than both Grass A and Grass B. It is the big brother of all the grasses.','%1 swung the %2 with her hands!','The %3\'s right thumb, index finger, and ring finger were cut by the %2!','','','Grass C','','','3x Attack','It can be found pretty much anywhere.',''];
_tr_w["eng"][Number('38') || 0]=['2','斬撃','','','2','50','Withered, dead grass. It is not easily used as a weapon.','%1 swung the %2 with her hands!','The %2 hit the %3 with a loud cracking sound!','','','Withered Grass','','','Guaranteed to leave 1 HP','It can be found in the southeast.',''];
_tr_w["eng"][Number('39') || 0]=['8','斬撃','','','3','200','Lovely white flowers in full bloom. Seeing them is enough to soothe your soul.','%1 swung the %2 with her hands!','The %3\'s left pinky was cut by the flower petals!','The %3 was soothed by the %2 and lost the will to fight!','','White Flowers','','','Attack Down (50%)','It can be found pretty much anywhere.',''];
_tr_w["eng"][Number('40') || 0]=['9','斬撃','','','3','250','Passionate red flowers. Their beauty steals a foe\'s will to fight.','%1 swung the %2 with her hands!','The %3\'s right middle finger was cut by the flower petals!','The %3 was soothed by the %2 and lost the will to fight!','','Red Flowers','','','Attack Down (40%)','It can be found pretty much anywhere.',''];
_tr_w["eng"][Number('41') || 0]=['6','斬撃','','','3','400','Lovely flowers of type unknown to you. Gazing upon them is enough to lower your blood pressure.','%1 swung the %2 with her hands!','The %3\'s left middle finger was cut by the flower petals!','The %3 was soothed by the %2 and lost the will to fight!','','Beautiful Flowers','','','Attack Down (75%)','They can be found growing in the village and the forest.',''];
_tr_w["eng"][Number('42') || 0]=['20','打撃','','','6','900','A flower that gives off a repulsive odor. You would have preferred to have not picked it if at all possible.','%1 swung the %2 while pinching her nose!','The %2 hit near the %3\'s nostrils!','The %3 appears sickened by the smell!','','Smelly Flower','','','Poison (100%)','It can be found in the forest to the west.',''];
_tr_w["eng"][Number('43') || 0]=['6','打撃','前６','3','4','900','A grassy patch that looks like it could fly up at you. We must not speak of it any further.','%1 swung the %2!','Wild creatures emerge from the %2 to attack the %3!','','','Grassy Patch','','','3x Attack','It can be found pretty much anywhere.','6 squares in front'];
_tr_w["eng"][Number('44') || 0]=['10','斬撃','前９','3','4','1800','A grassy patch with blades of glass that look like swords. They seem pretty sharp, too.','%1 swung the %2!','The %3 was cut by the patch of sword-like grass!','','','Intimidating Grassy Patch','','','3x Attack','It grows to the west of the village.','9 squares in front'];
_tr_w["eng"][Number('45') || 0]=['7','打撃','前６','3','4','1500','A grassy patch that looks like it\'s hiding something rare. The color, for instance, is... Uh oh. Someone\'s coming.','%1 swung the %2!','Vividly colored wild creatures emerged from the %2 to attack the %3!','','','Golden Grassy Patch','','','3x Attack','It can be found to the east of the village.','6 squares in front'];
_tr_w["eng"][Number('46') || 0]=['7','打撃','','','5','300','A cleanly trimmed thicket. It looks fairly harmless.','%1 swung the %2 with her hands!','The %2 hit the %3 with a soft swooshing sound!','','','Thicket','','','','It can be found decorating people\'s homes.',''];
_tr_w["eng"][Number('47') || 0]=['7','打撃','','','5','600','A beautiful thicket with flowers in full bloom. It can make any town prettier.','%1 swung the %2 with her hands!','The %2 hit the %3 with a soft swooshing sound!','','','Flowered Thicket','','','','It can be found near your home.',''];
_tr_w["eng"][Number('48') || 0]=['6','斬撃','','3','4','200','A carpet of fallen autumn leaves. There is something romantic about how they flutter to the ground.','%1 swung the %2 with her hands!','Many of the %2 cut into the %3!','','','Fallen Leaves','','','3x Attack','They can be found in abundance to the east of the village.',''];
_tr_w["eng"][Number('49') || 0]=['5','打撃','','','5','500','Zebra grass with a golden hue. It sways beautifully in the wind.','%1 quickly swung the %2!','The %3 was tickled by the %2!','The %3 looks ready to pass out from laughing so hard!','','Zebra Grass','','','Confuse (100%)','It can be found to the east of the village.',''];
_tr_w["eng"][Number('50') || 0]=['6','打撃','','','6','400','It looks like a poisonous mushroom. Although, you can\'t judge a mushroom by its appearance.','%1 swung the %2 with her hands!','The %2 hit the %3 in the face!','Pieces of the %2 flew into the %3\'s mouth!','','Mushroom','','','Poison (75%)','It can be found in the forest to the west.',''];
_tr_w["eng"][Number('51') || 0]=['8','打撃','','','6','300','You can immediately tell what this mushroom is like. It\'s as delicious as it is deadly.','%1 swung the %2 with her hands!','The %2 hit the %3 in the face!','Pieces of the %2 flew into the %3\'s mouth!','','Ominous Mushroom','','','Deadly Poison (50%)','It can be found in the forest to the west.',''];
_tr_w["eng"][Number('52') || 0]=['14','打撃','直線３','','6','700','A plant found growing on walls. It looks like it could make a great whip.','%1 swung the %2!','The %2 lashed out like a whip and hit the %3!','','','Ivy','','','','It grows on a cliff to the south.','3 squares straight line'];
_tr_w["eng"][Number('53') || 0]=['12','斬撃','直線３','','6','800','A plant that grows around things. It\'s long and can be used to attack from afar.','%1 swung the %2!','The %2 flew out sharply and stabbed the %3!','','','Vine','','','','It grows on a cliff to the south.','3 squares straight line'];
_tr_w["eng"][Number('54') || 0]=['16','斬撃','','','5','1200','A prickly plant found in the desert. It\'s clearly painful to the touch.','%1 swung the %2 while fighting through the pain!','The %2\'s spines dig into the %3\'s body!','','','Cactus','','','','It can be found in the southwest.',''];
_tr_w["eng"][Number('55') || 0]=['15','打撃','','','4','600','A strange grass of an unknown type. Its composition is shrouded in mystery.','%1 swung the %2 with her hands!','The %3\'s left pinky was cut by the %2!','The %2 has an unusual effect on the %3!','','Strange Grass','','','Defense Down (100%)','It can be found in the forest to the west.',''];
_tr_w["eng"][Number('56') || 0]=['13','斬撃','','','4','300','A grass often found growing on walls. It contains sleep-inducing compounds.','%1 swung the %2 with her hands!','The back of the %3\'s right hand was cut by the %2!','The %2 contained sleep-inducing compounds!','','Wall Grass','','','Sleep (25%)','It can be found in a cave made of iron.',''];
_tr_w["eng"][Number('57') || 0]=['10','打撃','','','5','200','An incredibly suspicious-looking sprout. Its roots apparently possess a mysterious power.','%1 swung the %2 with her hands!','The %2 repeatedly stabbed the %3!','The %2 absorbed the %3\'s status!','','Suspicious Sprout','','','Copies status changes','It can be found in a cave located in a poisonous swamp.',''];
_tr_w["eng"][Number('58') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('59') || 0]=['3','打撃','','','3','50','Dirt and pebbles that litter the ground. You wouldn\'t want to get it in your eyes.','%1 threw a handful of %2!','The %3 is covered from head to toe in %2!','The %3\'s eyes are full of dirt!','','Dirt','','','Accuracy Down (100%)','It can be found pretty much anywhere.',''];
_tr_w["eng"][Number('60') || 0]=['6','打撃','','','4','200','A small mound made of dirt. You are horrified by the thought of it covering you.','%1 swung the %2!','The %3 is covered from head to toe in %2!','The %3\'s eyes are full of dirt!','','Dirt Mound','','','Accuracy Down (50%)','It can be found in the southwest.',''];
_tr_w["eng"][Number('61') || 0]=['14','打撃','前６','','5','400','A huge mound made of dirt. Even dust can become a mountain if enough of it piles up.','%1 swung the %2!','The %3 is covered from head to toe in %2!','The %3\'s eyes are full of dirt!','','Large Dirt Mound','','','Accuracy Down (50%)','It can be found in the southwest.','6 squares in front'];
_tr_w["eng"][Number('62') || 0]=['11','打撃','','','3','300','Rock A is distinguished by its brown color. B and C versions also exist.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Rock A','','','','It can be found in abundance in the southwest.',''];
_tr_w["eng"][Number('63') || 0]=['12','打撃','','','3','400','Rock B is black with a well-defined shape. The pointy part makes it easy to swing.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Rock B','','','','It can be found in abundance in the southwest.',''];
_tr_w["eng"][Number('64') || 0]=['15','打撃','','','3','500','Rock C is distinguished by its rugged, white body. It\'s the strongest of Rocks A through C.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Rock C','','','','It can be found in abundance in the southwest.',''];
_tr_w["eng"][Number('65') || 0]=['12','打撃','直線２','','3','700','A massive and imposing rock. It\'s softer than it looks.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Huge Rock','','','','It can be found in abundance in the southwest.','2 squares straight line'];
_tr_w["eng"][Number('66') || 0]=['14','打撃','','','3','200','A rock of deep green. Its color is the only thing that separates it from ordinary rocks.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Green Rock','','','','It can be found in a cave located in a poisonous swamp.',''];
_tr_w["eng"][Number('67') || 0]=['14','打撃','直線２','','3','600','A larger version of the Green Rock. Swing it at things.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Huge Green Rock','','','','It can be found in a cave located in a poisonous swamp.','2 squares straight line'];
_tr_w["eng"][Number('68') || 0]=['11','打撃','','','1','100','A crumbling rock covered in moss. One touch looks like it could cause you to crumble as well.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3\'s defenses have crumbled!','','Crumbling Rock','','','Defense Down (50%)','It can be found in abundance in the northeast.',''];
_tr_w["eng"][Number('69') || 0]=['7','打撃','','','3','50','A rock fragment found on the ground. It\'s brittle and doesn\'t make for a very reliable weapon.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','','','Rock Fragment','','','','It can be found in abundance in the southwest.',''];
_tr_w["eng"][Number('70') || 0]=['13','斬撃','','','3','700','A crystal with a beautiful glitter to it. The pointy part looks really painful.','%1 swung the %2!','The %2\'s tip pierced the %3!','','Magic shot forth from the %2!','Crystal','','','Casts random offensive magic','It can be found in abundance in a magical cave.',''];
_tr_w["eng"][Number('71') || 0]=['15','斬撃','直線２','','3','1200','A huge crystal that glitters intensely. Its magic is evident to the touch.','%1 swung the %2!','The %2\'s tip pierced the %3!','','Magic shot forth from the %2!','Huge Crystal','','','Casts random offensive magic','It can be found in abundance in a magical cave.','2 squares straight line'];
_tr_w["eng"][Number('72') || 0]=['6','斬撃','','','3','300','A piece broken off of a crystal. It seems to have lost its magical powers.','%1 swung the %2 with her hands!','The %2\'s tip pierced the %3!','','','Crystal Fragment','','','','It can be found in abundance in a magical cave.',''];
_tr_w["eng"][Number('73') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('74') || 0]=['13','打撃','前９','','3','800','An imposing cliff that reaches high into the sky. It looks like it packs a serious punch.','%1 swung the %2!','The %2 hit the %3 hard!','','','Cliff','','','','It can be found in abundance outside of the village.','9 squares in front'];
_tr_w["eng"][Number('75') || 0]=['9','なし','','','7','200','A spider web like those found in the corners of a room. It\'s very sticky and difficult to escape from.','%1 reluctantly grabbed the %2 and swung it.','The %3 was hit in the face by the %2!','The %2 stuck to the %3\'s feet as well!','','Spider Web','','','Stamina Down (60%)','It can be found in a cave filled with wandering souls.',''];
_tr_w["eng"][Number('76') || 0]=['5','なし','','','4','400','A fissure formed in the ground. It looks like it could swallow an enemy whole.','%1 swung the %2 with her hands!','The %3 was hit by the %2!','The %3 fell into the fissure!','','Fissure','','','Defense Down (100%)','It can be found in the southwest.',''];
_tr_w["eng"][Number('77') || 0]=['23','打撃','','','5','600','A deep pitfall trap dug by someone. It is great for stopping enemies in their tracks.','%1 swung the %2 with her hands!','The %3 was hit by the %2!','The %3 fell right into the %2!','','Pitfall','','','Immobilize - 2 turns (25%)','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('78') || 0]=['28','打撃','前９','','5','9500','A huge hole too deep to see where it ends. Anything that falls into it isn\'t getting out.','%1 swung the %2 with her hands!','The %3 was hit by the %2!','The %3 was swallowed up by the %2!','','Huge Hole','','','Immobilize - 3 turns (20%)','It can be found in a cave made of iron.','9 squares in front'];
_tr_w["eng"][Number('79') || 0]=['14','なし','前６','','5','2000','A small pond full of fish and frogs. Swinging it can cause a flood.','%1 swung the %2 with her hands!','','','The pond\'s water caused a flood!','Small Pond','','','Emits water,Lightning Resistance Down (100%)','It can be found pretty much anywhere.','6 squares in front'];
_tr_w["eng"][Number('80') || 0]=['20','なし','前６','','5','2500','A lovely pond full of lotus leaves. The lotuses make it more powerful than a normal pond.','%1 swung the %2 with her hands!','','','The pond\'s water caused a flood!','Lotus Pond','','','Emits water,Lightning Resistance Down (100%)','It can be found south of the village.','6 squares in front'];
_tr_w["eng"][Number('81') || 0]=['16','なし','前６','','5','1400','A dangerous swamp full of deadly poisons. Swinging it is sure to bring about great calamity.','%1 swung the %2 with her hands!','','','The poisonous water caused a flood!','Poisonous Swamp','','','Emits poisonous water,Poison (80%)','It can be found in abundance in the southeast.','6 squares in front'];
_tr_w["eng"][Number('82') || 0]=['45','なし','前９','','6','19999','The wellspring of life that covers 70% of the planet\'s surface. Swinging it can cause a tidal wave.','%1 swung the %2 with her hands!','','','It caused a tidal wave!','Ocean','','','Emits a tidal wave,Lightning Resistance Down (100%)','It can be seen from the cape in the northwest.','9 squares in front'];
_tr_w["eng"][Number('83') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('84') || 0]=['16','打撃','','','6','800','A sign that provides information to passersby. Getting hit by its corners would hurt pretty badly.','%1 swung the %2!','The %2\'s corner hit the %3!','','','Sign','','','','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('85') || 0]=['24','斬撃','','','6','1000','A sign designed to show the way. It\'s disliked due to how hard it is to understand.','%1 swung the %2!','The %2\'s sharp edge cut into the %3!','The %3 was confused by the confusing road sign!','','Road Sign','','','Confuse (50%)','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('86') || 0]=['18','打撃','','','7','500','A piece of wood used to start fires. It\'s thick and it would hurt if you were hit with it.','%1 swung the %2!','The thick %2 hit the %3!','The %3 can now burn as easily as firewood!','','Firewood','','','Fire Resistance Down (100%)','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('87') || 0]=['28','打撃','','','5','1000','A hangout spot for old women. It\'d be hard to get out if you happened to fall in.','%1 swung the %2!','The %2\'s wall hit the %3!','The %3 fell to the bottom of the well!','','Well','','','Immobilize - 1 turn (25%)','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('88') || 0]=['17','打撃 / 炎','直線３','','5','2000','A streetlight that gently illuminates the streets. Its long reach makes it an excellent weapon.','%1 swung the %2!','The %2 lit up brightly as it hit the %3!','','','Streetlight','','','','It can be found in the south.','3 squares straight line'];
_tr_w["eng"][Number('89') || 0]=['16','打撃','','','5','1200','An unsung hero that protects the fields from crows. Even humans are afraid of its expressionless face.','%1 swung the %2!','The %2 hit the %3 with a blank look on its face!','The %3 is terrified of the %2!','','Scarecrow','','','Immobilize - 1 turn (40%)','It can be found in the east.',''];
_tr_w["eng"][Number('90') || 0]=['15','打撃','','','5','400','A fashionable thing placed in front of a house. It\'s made of stone, so it will hurt if you hit it.','%1 swung the %2 with her hands!','The %2\'s relatively sharp edge hit the %3!','','','Walkway','','','','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('91') || 0]=['14','打撃','','','4','200','A wooden bucket with nothing inside. It makes a nice sound when tapped.','%1 swung the %2!','The %2 hit the %3 with a bang!','','','Empty Bucket','','','','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('92') || 0]=['14','打撃','','','5','500','A bucket with water in it. Use it to douse things with water.','%1 swung the %2!','The %2 hit the %3 with a bang!','','Water flowed out of the %2!','Water Bucket','','','Emits water,Lightning Resistance Down (100%)','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('93') || 0]=['12','打撃','','','5','800','It\'s just like doing the laundry. Attack by swinging it and its detergent at foes.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','Detergent flew out from the %2!','Laundry Set','','','Emits detergent,Ice Resistance Down (100%)','It can be found to the south of the village.',''];
_tr_w["eng"][Number('94') || 0]=['7','打撃','','','4','100','An ordinary wooden board. Hitting things is the only use you can imagine for it.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Wooden Board','','','','It can be found in the southwest.',''];
_tr_w["eng"][Number('95') || 0]=['13','打撃','直線２','','3','700','The unsung hero that props up buildings. Its shape lends itself to swinging.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Pillar','','','','It can be found in the northeast.','2 squares straight line'];
_tr_w["eng"][Number('96') || 0]=['9','斬撃','','','3','600','A broken pillar. The broken section is pretty jagged.','%1 swung the %2!','The %2\'s jagged section pierced the %3!','','','Broken Pillar','','','','It can be found in the northeast.',''];
_tr_w["eng"][Number('97') || 0]=['13','打撃','直線２','','3','400','A crumbling pillar covered in moss. You can tell many years have passed since it was made.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3\'s defenses have crumbled!','','Crumbling Pillar','','','Defense Down (50%)','It can be found in the northeast.','2 squares straight line'];
_tr_w["eng"][Number('98') || 0]=['9','斬撃','','','3','200','A crumbling and broken pillar. Its days as a support column are long behind it.','%1 swung the %2!','The %2\'s jagged section pierced the %3!','The %3\'s defenses have crumbled!','','Broken & Crumbling Pillar','','','Defense Down (50%)','It can be found in the northeast.',''];
_tr_w["eng"][Number('99') || 0]=['12','打撃','','','6','2000','A stone monument with something important written on it. The letters are extremely hard to make out, however.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 is desperately trying to read the monument!','','Stone Monument','','','Confuse (70%)','It can be found somewhere in a cave.',''];
_tr_w["eng"][Number('100') || 0]=['13','斬撃','','','5','900','A simple grave to enshrine the dead. Using it with reckless abandon invites divine punishment.','%1 swung the %2!','The tip of the %2 pierced the %3!','Divine punishment was inflicted upon the %3...','','Wooden Grave','','','Instant Kill (5%)','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('101') || 0]=['17','打撃','','','5','2400','A stone grave to enshrine the dead. Using it with reckless abandon invites divine punishment.','%1 swung the %2!','The %2 hit the %3 with a punishing blow!','Divine punishment was inflicted upon the %3...','','Stone Grave','','','Instant Kill (10%)','It can be found in a cave filled with wandering souls.',''];
_tr_w["eng"][Number('102') || 0]=['12','打撃','直線２','3','6','3200','A machine that conveniently allows you to buy various drinks. It only has hot drinks for some reason.','%1 swung the %2!','A hot drink fell on the %3\'s head!','','','Vending Machine','','','3x Attack','-','2 squares straight line'];
_tr_w["eng"][Number('103') || 0]=['13','打撃','','','6','800','A can used to separate garbage. You can\'t swing it without spewing trash everywhere.','%1 swung the %2!','The %2 spewed garbage as it hit the %3!','The %3 inhaled the disgusting odor of the garbage!','','Garbage Can','','','Poison (100%)','It can be found near a convenience store.',''];
_tr_w["eng"][Number('104') || 0]=['14','打撃','','','7','1100','A durable, handmade fire pit made of stone. It can easily be lit with lighter fluid.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 was doused in lighter fluid!','','Fire Pit','','','Fire Resistance Down (100%)','It can be found laying on the ground in the south.',''];
_tr_w["eng"][Number('105') || 0]=['25','炎','','','6','2200','A raging fire. Pick it up, and you\'re bound to get burnt.','%1 swung the %2 while withstanding the burning pain!','The %3 lit up like a campfire!','','','Fire','','','','It can be found laying on the ground in the south.',''];
_tr_w["eng"][Number('106') || 0]=['30','打撃','直線４','','7','8500','A mysterious steel plate that sometimes floats into the air and rotates. The rotation can likely be weaponized.','%1 swung the %2!','The %2 rotated as it hit the %3!','','','Mysterious Steel Plate','','','','It can be found in a cave made of iron.','4 squares straight line'];
_tr_w["eng"][Number('107') || 0]=['25','打撃','','','7','4000','Something seems to rotate whenever it\'s used. It\'s surprisingly hard.','%1 swung the %2!','The %2 shifted as it hit the %3!','','','Mysterious Switch','','','','It can be found in a cave made of iron.',''];
_tr_w["eng"][Number('108') || 0]=['30','打撃','直線２','','7','3800','A mysterious bridge that changes direction each time someone crosses it. It can also be used as a weapon.','%1 swung the %2!','The %2 changed directions as it hit the %3!','','','Mysterious Bridge','','','','It can be found in a cave located in a poisonous swamp.','2 squares straight line'];
_tr_w["eng"][Number('109') || 0]=['28','打撃','前９','','7','6000','A mysterious panel that lights up when stimulated. Its puzzle is hard to solve.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 cannot find the correct route!','','Mysterious Panel','','','Confuse (25%)','It can be found in a magical cave.','9 squares in front'];
_tr_w["eng"][Number('110') || 0]=['32','打撃 / 炎','','','7','5500','A lantern with a mystical aura to it. Once lit, its fire will burn forever.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','The %2 ignited on its own for some reason!','Mysterious Lantern','','','Emits fire','It can be found in a cave filled with wandering souls.',''];
_tr_w["eng"][Number('111') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('112') || 0]=['12','打撃','','','1','600','A pot the likes of which can be seen in many homes. Striking something with it will result in a satisfying smash.','%1 swung the %2!','The %2 slammed into the %3!','','','Pot','','','Guarantees a Critical Hit','It can be found in various houses.',''];
_tr_w["eng"][Number('113') || 0]=['12','打撃','','','1','1200','A suspicious pot with its contents hidden by a lid. What in the world could be inside...?','%1 swung the %2!','The %2 slammed into the %3!','Horse dung flew out of the pot and into the %3\'s mouth!','','Covered Pot','','','Guarantees a Critical Hit,Poison (100%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('114') || 0]=['15','打撃','','','3','700','An ordinary barrel seen just about everywhere. It can be used as a weapon against enemies.','%1 swung the %2!','The %2 slammed into the %3!','','','Barrel','','','','It can be found somewhere in the village.',''];
_tr_w["eng"][Number('115') || 0]=['11','斬撃','','','6','600','A small, square table. It\'d be a nice place to put something.','%1 swung the %2!','The corner of the %2 stabbed the %3!','','','Miniature Table','','','','It can be found in a cabin somewhere.',''];
_tr_w["eng"][Number('116') || 0]=['8','打撃','','','6','600','A small, cute table that happens to be round. Its lack of corners negatively impacts its damage.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Round Table','','','','It can be found in a cabin somewhere.',''];
_tr_w["eng"][Number('117') || 0]=['13','斬撃','直線２','','6','900','A square table that is just the right size. Use its corners to slash, thrust, and stab your way through the opposition.','%1 swung the %2!','The corner of the %2 stabbed the %3!','','','Medium Table','','','','It can be found in a home somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('118') || 0]=['14','斬撃','前６','','6','2000','A table made to seat several people. It can be wielded by only one person, however.','%1 swung the %2!','The corner of the %2 stabbed the %3!','','','Large Table','','','','It can be found in various houses.','9 squares in front'];
_tr_w["eng"][Number('119') || 0]=['13','斬撃','直線３','','6','1400','A table that is kind of long. Striking with its corners will kind of hurt.','%1 swung the %2!','The corner of the %2 stabbed the %3!','','','Somewhat Long Table','','','','It can be found in a home somewhere in the village.','3 squares straight line'];
_tr_w["eng"][Number('120') || 0]=['15','斬撃','直線４','','6','2500','A table that boasts considerable length. Its reach makes it a formidable weapon.','%1 swung the %2!','The corner of the %2 stabbed the %3!','','','Long Table','','','','It can be found in a home somewhere in the village.','4 squares straight line'];
_tr_w["eng"][Number('121') || 0]=['12','斬撃','','','6','1100','A table with a flower vase on it. Just the sight of it puts your heart at ease.','%1 swung the %2!','The corner of the %2 stabbed the %3!','The %3 was soothed by the sight of the vase and lost the will to fight!','','Vase Table','','','Attack Down (60%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('122') || 0]=['15','斬撃','','','6','1200','A table with a mirror attached. The mirror only reflects the truth.','%1 swung the %2!','The corner of the %2 stabbed the %3!','The light reflected from the mirror hit the %3!','','Mirror Table','','','Resets status buffs','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('123') || 0]=['17','斬撃','直線２','','6','3600','A table like one you\'d find in a church. It is imbued with the mysterious power of the scriptures.','%1 swung the %2!','The corner of the %2 stabbed the %3!','Some sort of holy power worked its magic on the %3!','','Church Table','','','Fire Resistance Down (50%),Water Resistance Down (50%),Lightning Resistance Down (50%)','It can be found in a church somewhere.','2 squares straight line'];
_tr_w["eng"][Number('124') || 0]=['17','斬撃','','','6','4000','A table featuring a strange crystal. You feel power emanating from it.','%1 swung the %2!','The corner of the %2 stabbed the %3!','','Magic shot forth from the crystal!','Crystal Table','','','Casts random offensive magic','It can be found in a strange house somewhere.',''];
_tr_w["eng"][Number('125') || 0]=['8','打撃','','','6','300','A cheap-looking stool that\'s easy to sit on. You probably shouldn\'t expect much out of it as a weapon.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 sat down to rest on the %2!','','Stool','','','Immobilize - 1 turn (50%)','It can be found in abundance in a house somewhere in the village.',''];
_tr_w["eng"][Number('126') || 0]=['10','斬撃','','','6','300','A small, square chair. Its four corners make it stronger than a stool.','%1 swung the %2!','The corner of the %2 stabbed the %3!','The %3 sat down to rest on the %2!','','Square Chair','','','Immobilize - 1 turn (50%)','It can be found in abundance in a house somewhere in the village.',''];
_tr_w["eng"][Number('127') || 0]=['12','斬撃','直線２','','6','1500','A stylish, high-end chair. It offers superior comfort.','%1 swung the %2!','The corner of the %2\'s backrest stabbed the %3!','The %3 sat down and relaxed on the %2!','','Stylish Chair','','','Immobilize - 1 turn (80%)','It can be found in a cabin somewhere.','2 squares straight line'];
_tr_w["eng"][Number('128') || 0]=['8','打撃','直線２','3','6','1000','A bookshelf lined with various books. Use it by hitting enemies with the books that fall from it.','%1 swung the %2!','Countless books rained down upon the %3\'s head!','The %3 read various books and grew smarter!','','Bookshelf','','','3x Attack,Enfeeble (30%)','It can be found in various houses.','2 squares straight line'];
_tr_w["eng"][Number('129') || 0]=['30','斬撃','直線２','','6','1500','A cabinet with clothes in it. One hit on a left pinky toe would inflict pain worse than death.','%1 swung the %2!','The %3\'s left pinky toe was struck by the corner of the %2!','The %3\'s left pinky toe looks to be in agonizing pain!','','Dresser','','','Stamina Down (30%)','It can be found in a home somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('130') || 0]=['9','打撃','直線２','3','6','1200','A place where you hang clothes. Its contents are what make it so amazing.','%1 swung the %2!','Countless garments rained down upon the %3\'s head!','The garments make it hard for the %3 to move!','','Closet','','','3x Attack,Evasion Down (100%)','It can be found in a home somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('131') || 0]=['7','打撃','直線２','3','6','1400','A shelf with bread on it for whatever reason. French bread is hard enough to be weaponized.','%1 swung the %2!','Countless loaves of French bread rained down upon the %3\'s head!','The %3 began eating bread as if in a daze!','','Bread Shelf','','','3x Attack,Incapacitates for 1 turn (50%)','It can be found in a cabin somewhere.','2 squares straight line'];
_tr_w["eng"][Number('132') || 0]=['8','打撃','直線２','3','6','900','A shelf full of unusual medicines and trinkets. Give it a good swing and see what happens.','%1 swung the %2!','Countless oddities rained down upon the %3\'s head!','The %3 is clearly afraid of them!','','Confusing Shelf','','','3x Attack,Confuse (50%)','It can be found in a cabin somewhere.','2 squares straight line'];
_tr_w["eng"][Number('133') || 0]=['12','打撃','','','6','600','A bookstand attached to a wall. Nobody can reach it directly.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','%3 read a book and grew smarter!','','Bookstand','','','Enfeeble (30%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('134') || 0]=['13','打撃','','','6','800','A rack that allows convenient access to various spices and ingredients. Just be sure to keep an eye on the expiration dates.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','The ingredients are two years past their expiration dates!','','Spice Rack','','','Poison (30%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('135') || 0]=['12','打撃','','','6','700','A vase with a cute flower in it. A beautiful flower requires a beautiful vase.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','%3 was soothed by the sight of the flower and lost the will to fight!','','Flower Vase','','','Attack Down (50%)','It can be found in a church somewhere.',''];
_tr_w["eng"][Number('136') || 0]=['14','打撃','','','5','400','A wooden box to be filled with things. It can also be used to hit things too.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','','','Wooden Box','','','','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('137') || 0]=['6','打撃','前６','5','3','1800','A towering stack of wooden boxes. Use it to bring them crashing down on the enemy\'s head.','%1 swung the %2!','Countless wooden boxes rained down upon the %3\'s head!','','','Wooden Box Tower','','','5x Attack','It can be found in a cabin somewhere.','6 squares in front'];
_tr_w["eng"][Number('138') || 0]=['8','打撃','','','3','300','A wooden box that has seen better days. It\'s neither strong nor particularly useful.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','','','Worn Wooden Box','','','','It can be found in a cabin somewhere.',''];
_tr_w["eng"][Number('139') || 0]=['18','打撃','直線２','','5','700','A wooden box big enough to hold a body. Each swing will have some serious power behind it.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Big Wooden Box','','','','It can be found somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('140') || 0]=['12','打撃','','','5','500','A full bag of rice. Be grateful to the farmer each and every time you swing it.','%1 swung the %2 with her hands!','The full %2 landed a direct hit on the %3!','','','Rice Bag','','','','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('141') || 0]=['9','打撃','直線２','3','5','1400','A towering stack of full bags of rice. Use it to bring them crashing down on the enemy\'s head.','%1 swung the %2!','Countless rice bags rained down upon the %3\'s head!','','','Rice Bag Tower','','','3x Attack','It can be found in a home somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('142') || 0]=['5','打撃','','5','6','1100','A bushel of delicious-looking apples. Although...','%1 swung the %2 with her hands!','The %2 rained down upon the %3\'s head!','All of the apples were poisonous!','','Bunch of Apples','','','5x Attack,Poison (60%)','It can be found in a strange house somewhere.',''];
_tr_w["eng"][Number('143') || 0]=['16','打撃','直線２','','5','1400','A large kitchen that\'s clean as a whistle. It has excellent range.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Kitchen','','','','It can be found in a home somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('144') || 0]=['16','打撃','','','5','900','A clean gas stove without a speck of oil on it. Its use of fire makes it a formidable weapon.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','Be sure to wait until the flame reaches an amber hue!','Gas Stove','','','Emits fire','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('145') || 0]=['18','斬撃','','','6','900','A somewhat dirty kitchen fan. Its rotating blades make for a deadly weapon.','%1 swung the %2 with her hands!','The %3 got caught in the %2\'s spinning blades!','','','Kitchen Fan','','','','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('146') || 0]=['23','斬撃','','','5','800','A knife and cutting board combo for cutting things. The kitchen knife is pretty sharp.','%1 swung the %2!','The %3 landed on the cutting board and was chopped into little pieces!','','','Cutting Board Set','','','Slash Resistance Down (100%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('147') || 0]=['11','打撃','','3','6','700','A frying pan, ladle, and other cooking utensils. Of course, they can be used as weapons, too.','%1 swung the %2!','You hit the %3 with various cooking utensils!','','','Cooking Set','','','3x Attack','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('148') || 0]=['20','打撃 / 氷','直線２','','6','1900','A refrigerator full of leftovers. Its cold can be used to freeze enemies.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','Its door opened and cold air leaked out!','Refrigerator','','','Emits cold air','It can be found in a home somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('149') || 0]=['12','打撃','','','6','700','A fish grilled from head to tail. It\'s served up whole, bones and all.','%1 swung the %2! *You shouldn\'t play with your food!','The %2 slammed hard into the %3!','The fish\'s bones scratched the %3\'s throat!','','Grilled Fish','','','Mute (50%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('150') || 0]=['14','打撃','','','6','600','A simple vegetable stir-fry. It\'s hot enough to burn your tongue.','%1 swung the %2! *You shouldn\'t play with your food!','You attacked the %3 with piping-hot vegetables!','','','Stir-Fried Vegetable Set','','','','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('151') || 0]=['12','打撃','','','6','750','A special lunch set with all-you-can-eat rice. The pork still looks a little pink.','%1 swung the %2! *You shouldn\'t play with your food!','The %2 slammed hard into the %3!','The %2 was only half-cooked. The %3 got diarrhea!','','Stir-Fried Pork Set','','','Poison (30%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('152') || 0]=['7','打撃','','','6','320','A cup of coffee to get you through the afternoon. The caffeine will help shrug off any drowsiness.','%1 proposed a toast with the %2!','%3 \"Cheers!\"','The %3\'s eyes look wide awake!','','Cup of Coffee','','','Sleep Resistance Up (100%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('153') || 0]=['12','打撃','','','6','3000','A full-bodied wine with a rich favor. One whiff is enough to brighten your mood.','%1 proposed a toast with the %2!','%3 \"Cheers!\"','The %3 got drunk on the sent of the wine!','','Fine Wine','','','Prevents non-standard attack(50%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('154') || 0]=['21','打撃','','','6','50','Is it food or not? Why was it even served...?','%1 swung the %2 with her hands!','The %2 flew into the %3\'s mouth!','The %3 has clearly gotten food poisoning!','','Mystery Meat','','','Poison (75%),Mute (50%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('155') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('156') || 0]=['14','打撃','直線２','','6','2400','A fluffy bed that\'s comfortable to sleep on. It makes enemies ready for bed.','%1 swung the %2!','The %2 hit the %3 with a soft thud!','The %3 was swallowed up by the %2!','','Bed','','','Sleep (55%)','It can be found in a home somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('157') || 0]=['18','打撃','前６','','6','7000','A double bed with room for two to sleep. It offers the ultimate in comfort.','%1 swung the %2!','The %2 hit the %3 with a soft thud!','The %3 was swallowed up by the %2!','','Double Bed','','','Sleep (90%)','It can be found in a home somewhere in the village.','6 squares in front'];
_tr_w["eng"][Number('158') || 0]=['7','打撃','直線２','','4','300','A simple, uncomfortable-looking futon. Sleepers can expect sore joints the next morning.','%1 swung the %2!','The %2 hit the %3 with a dull thud!','The %3 has decided to make due with the %2 for tonight!','','Worn Futon','','','Sleep (25%)','It can be found in a cabin somewhere.','2 squares straight line'];
_tr_w["eng"][Number('159') || 0]=['18','打撃','直線２','','6','1400','A splendid fireplace. Wield it from the thin part on top.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','The %2 ignited on its own for some reason!','Fireplace','','','Emits fire','It can be found in a home somewhere in the village.','2 squares straight line'];
_tr_w["eng"][Number('160') || 0]=['11','斬撃','','','5','400','A diary written by someone. Reading it reminds you of your dark past.','%1 swung the %2 with her hands!','The %3\'s left thumb was cut by the %2\'s pages!','The %3 was reminded of a dark and troubled past.','','Diary','','','Confuse (50%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('161') || 0]=['5','斬撃','','','4','200','Paper with something written on it. It could give you a nasty papercut.','%1 swung the %2 with her hands!','The back of the %3\'s left hand was cut by the %2\'s edge!','','','Piece of Paper','','','','It can be found in a cabin somewhere.',''];
_tr_w["eng"][Number('162') || 0]=['13','打撃','','','4','500','A bag with something in it. Contents unknown. Hitting things is the only thing it\'s good for.','%1 swung the %2 with her hands!','The %2 hit the %3! However, the contents are still unknown.','','','Random Bag','','','','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('163') || 0]=['27','打撃','前６','','6','3700','A beautifully polished, antique piano. Make it sing as you bash the enemy with it.','%1 swung the %2!','The %2\'s keyboard made some strange sounds upon hitting the %3!','The %3 was left unsettled by the dissonant chords!','','Piano','','','Resets status buffs','It can be found in a home somewhere in the village.','6 squares in front'];
_tr_w["eng"][Number('164') || 0]=['15','打撃','','','6','1000','A gift box with adorable wrapping. You sense a magical presence inside of it.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','Magic shot forth from the %2!','Gift Box','','','Casts random offensive magic','It can be found in a strange house somewhere.',''];
_tr_w["eng"][Number('165') || 0]=['12','打撃','','','5','1500','A dress found in a witch\'s home. You feel a little bit guilty for taking it.','%1 swung the %2! A lovely scent fills the air!','The %2 hit the %3 in the face!','','','Witch\'s Dress','','','','It can be found in a strange house somewhere.',''];
_tr_w["eng"][Number('166') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('167') || 0]=['11','打撃','前６','3','6','1400','A shelf with an array of convenience store products. It doesn\'t count as shoplifting if you\'re using it as a weapon.','%1 swung the %2!','Various products rained down upon the head of the %3!','All of the products were defective!','','Convenience Store Shelf','','','3x Attack,Confuse (50%)','It can be found in a convenience store somewhere.','6 squares in front'];
_tr_w["eng"][Number('168') || 0]=['10','打撃 / 氷','前６','3','6','2200','A corner shelf featuring various frozen foods. Its freezing cold air makes for a great weapon.','%1 swung the %2!','Frozen foods rained down upon the %3!','','Cold air leaked out!','Frozen Food Corner','','','3x Attack,Emits cold air','It can be found in a convenience store somewhere.','6 squares in front'];
_tr_w["eng"][Number('169') || 0]=['12','打撃','前６','3','6','1800','A corner shelf featuring an array of boxed lunches. All of them have \"half-off\" stickers attached.','%1 swung the %2!','Various boxed lunches rained down upon the head of the %3!','All of the boxed lunches were past their expiration dates!','','Boxed Lunch Corner','','','3x Attack,Poison (50%)','It can be found in a convenience store somewhere.','6 squares in front'];
_tr_w["eng"][Number('170') || 0]=['13','打撃','','','7','3000','My one and only home telephone. We need to watch out for phone scams.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 fell for a phone scam!','','Telephone','','','Confuse (100%)','It can be found in your home.',''];
_tr_w["eng"][Number('171') || 0]=['9','打撃','前９','','6','400','A very tattered carpet. A strange odor also emanates from it.','%1 swung the %2!','The %3 slipped on the %2 and fell hard to the ground!','The %3 slipped and is slowed down!','','Tattered Carpet','','','Evasion Down (75%)','It can be found in a cabin somewhere.','9 squares in front'];
_tr_w["eng"][Number('172') || 0]=['14','打撃','前９','','6','800','A lovely, blue carpet. It is smooth to the touch.','%1 swung the %2!','The %3 slipped on the %2 and fell hard to the ground!','The %3 slipped and is slowed down!','','Blue Carpet','','','Evasion Down (50%)','It can be found in a home somewhere in the village.','9 squares in front'];
_tr_w["eng"][Number('173') || 0]=['18','打撃','前９','','6','2500','An elegant, red carpet. It was sewn from the finest materials.','%1 swung the %2!','The %3 slipped on the %2 and fell hard to the ground!','The %3 slipped and is slowed down!','','Elegant Carpet','','','Evasion Down (25%)','It can be found in a church somewhere.','9 squares in front'];
_tr_w["eng"][Number('174') || 0]=['13','打撃','前９','','6','1100','A carpet with a mysterious air to it. The materials used seem unusually cheap.','%1 swung the %2!','The %3 slipped on the %2 and fell hard to the ground!','The %3 slipped and is slowed down!','','Suspicious Carpet','','','Evasion Down (100%)','It can be found in a strange house somewhere.','9 squares in front'];
_tr_w["eng"][Number('175') || 0]=['16','打撃','','','5','800','A clock that is accurate down to the minute. It\'d be fun to hit something with it.','%1 swung the %2 with her hands!','The %2 hit the %3 as the clock hand moved!','The space-time continuum was altered for the %3!','','Clock','','','Stamina Down (50%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('176') || 0]=['21','斬撃','','','7','1200','A pair of replica swords mounted on a wall. They look unusually sharp for a replica.','%1 swung the %2!','The %2 sliced through the %3 like butter!','','','Mounted Swords','','','','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('177') || 0]=['17','打撃','','','7','900','A shield replica mounted on a wall. It seems sturdier than your average replica.','%1 swung the %2!','The %2 hit the %3 hard!','','','Mounted Shield','','','Defense Down (50%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('178') || 0]=['24','斬撃','','','7','1500','A pair of replica axes mounted on a wall. They\'re as heavy as the real thing.','%1 swung the %2!','The %2 sliced through the %3 like butter!','','','Mounted Axes','','','','It can be found in a cabin somewhere.',''];
_tr_w["eng"][Number('179') || 0]=['16','斬撃','前３','','6','600','A hand-drawn map. No adventurer should leave home without it.','%1 swung the %2 with her hands!','The %2 hit the %3!','The %3 had trouble reading the map!','','Map','','','Confuse (75%)','It can be found in a cabin somewhere.','3 squares in front'];
_tr_w["eng"][Number('180') || 0]=['14','斬撃','','','5','700','A painting of exquisite flowers. The frame makes it heavy enough to weaponize.','%1 swung the %2 with her hands!','The frame landed a direct hit on the %3!','The %3 was soothed by the exquisite flowers and lost the will to fight!','','Flower Painting','','','Attack Down (60%)','It can be found in a strange house somewhere.',''];
_tr_w["eng"][Number('181') || 0]=['15','斬撃','','','5','800','The face of an unknown individual. Who is it? Art is hard.','%1 swung the %2 with her hands!','The frame landed a direct hit on the %3!','The %3 began to think of art in a new way!','','Self-Portrait','','','Confuse (40%)','It can be found in a strange house somewhere.',''];
_tr_w["eng"][Number('182') || 0]=['13','斬撃','直線２','','5','1000','A red banner that appears to be a nation\'s flag. It looks pretty cool, if not all that strong.','%1 swung the %2!','The %2 fluttered as it hit the %3!','','','Red Banner','','','','It can be found in a cave filled with wandering souls.','2 squares straight line'];
_tr_w["eng"][Number('183') || 0]=['7','斬撃','直線２','','3','300','A tattered banner. It\'ll be found in a trash can before long.','%1 swung the %2!','The %2 fluttered as it hit the %3!','','','Tattered Banner','','','','It can be found in a cave filled with wandering souls.','2 squares straight line'];
_tr_w["eng"][Number('184') || 0]=['16','斬撃','前６','','5','900','A crimson curtain that makes you doubt your fashion sense. It seems like it\'d provide a lot of shade.','%1 swung the %2 with her hands!','The %2 hit the %3 with a flutter!','The %3\'s line of sight was blocked by the %2!','','Crimson Curtain','','','Accuracy Down (90%)','It can be found in a strange house somewhere.','6 squares in front'];
_tr_w["eng"][Number('185') || 0]=['12','打撃','','','5','700','A window wiped clean. Its double-pane construction is more durable than it looks.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','','','Window','','','','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('186') || 0]=['12','打撃','','','5','900','A window with green curtains. It can block the enemy\'s line of sight.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','The %3\'s line of sight was blocked by the curtains!','','Green Curtained Window','','','Accuracy Down (100%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('187') || 0]=['13','打撃','','','5','1000','A fairly large window with red curtains. It\'s durable enough to deliver a hit without breaking.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','The %3\'s line of sight was blocked by the curtains!','','Red Curtained Window','','','Accuracy Down (100%)','It can be found in a home somewhere in the village.',''];
_tr_w["eng"][Number('188') || 0]=['15','打撃','','','5','1600','A window with beautiful colored glass panes. It can often be seen in churches.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','','','Stained Glass','','','','It can be found in a church somewhere.',''];
_tr_w["eng"][Number('189') || 0]=['20','打撃','','','4','800','A door offering questionable security. It makes for an even worse weapon.','%1 swung the %2 with a loud bang!','The %2 landed a direct hit on the %3!','','','Door','','','','It can be found at the entrance to almost any home.',''];
_tr_w["eng"][Number('190') || 0]=['16','打撃','前６','','6','1200','A sturdy jail cell made of iron. A hit from it could serve as a form of capital punishment.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 feels a prisoner!','','Jail Cell','','','Immobilize - 1 turn (30%)','It can be found in a cave made of iron.','6 squares in front'];
_tr_w["eng"][Number('191') || 0]=['17','打撃','直線２','','6','900','A sturdy jail cell door that requires a key to open. Good luck breaking out of here.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 feels a prisoner!','','Jail Cell Door','','','Immobilize - 1 turn (20%)','It can be found in a cave made of iron.','2 squares straight line'];
_tr_w["eng"][Number('192') || 0]=['28','打撃','直線２','','7','6000','A statue of a warrior with a stoic face. He\'s not the only one who can make use of that sword.','%1 swung the %2!','The %2\'s sword cut into the %3!','','','Warrior Statue','','','','It can be found in the east.','2 squares straight line'];
_tr_w["eng"][Number('193') || 0]=['24','打撃','直線２','','7','4500','A strange statue that\'s always facing right. It has a holy power to thank for its excellent durability.','%1 swung the %2!','The %2 landed a headbutt on the %3!','Some sort of holy power worked its magic on the %3!','','Right-Facing Goddess','','','Enfeeble (15%)','It can be found in a church somewhere.','2 squares straight line'];
_tr_w["eng"][Number('194') || 0]=['24','打撃','直線２','','7','4500','A strange statue that\'s always facing left. It looks like it could perform one heck of a headbutt.','%1 swung the %2!','The %2 landed a headbutt on the %3!','Some sort of holy power worked its magic on the %3!','','Left-Facing Goddess','','','Mute (50%)','It can be found in a church somewhere.','2 squares straight line'];
_tr_w["eng"][Number('195') || 0]=['40','斬撃 / 炎','前３','','7','18000','An intimidating statue of a ferocious dragon. A dragon\'s soul rests inside of it.','%1 swung the %2!','The %2\'s fangs stabbed the %3!','','The %2 breathed fire!','Dragon Statue','','','Emits fire','It can be found in a cave filled with wandering souls.','3 squares in front'];
_tr_w["eng"][Number('196') || 0]=['18','打撃','','','6','3000','A treasure chest that appears to contain a rare item. You see little choice but to use it as a weapon.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','','','Treasure Chest','','','','It can be found pretty much anywhere.',''];
_tr_w["eng"][Number('197') || 0]=['19','打撃','直線２','','4','1800','A pillar of dazzling gold. You probably shouldn\'t look directly at it.','%1 swung the %2!','The %2 shined brightly as it hit the %3!','The %3 looked directly at the dazzling gold shine!','','Golden Pillar','','','Accuracy Down (50%)','It can be found in a cave of glittering gold.','2 squares straight line'];
_tr_w["eng"][Number('198') || 0]=['15','斬撃','','','4','900','A broken, golden pillar that still shines brightly. It is surprisingly sturdy for gold.','%1 swung the %2!','The %2\'s jagged section pierced the %3!','The %3 looked directly at the dazzling gold shine!','','Broken Golden Pillar','','','Accuracy Down (30%)','It can be found in a cave of glittering gold.',''];
_tr_w["eng"][Number('199') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('200') || 0]=['20','打撃','','','7','500','A lovely, red stuffed bear. It\'s a stuffed animal...isn\'t it?','%1 swung the %2!','The %2 landed a headbutt on the %3!','','','Red Bear','Catherine','','','It is sitting in one of the houses in the village.',''];
_tr_w["eng"][Number('201') || 0]=['20','打撃','','','7','500','A cool, blue stuffed bear. It\'s a stuffed animal...isn\'t it?','%1 swung the %2!','The %2 landed a headbutt on the %3!','','','Blue Bear','James','','','It is sitting near the cape to the south.',''];
_tr_w["eng"][Number('202') || 0]=['20','打撃','','','7','500','A dark, black stuffed bear. It\'s a stuffed animal...isn\'t it?','%1 swung the %2!','The %2 landed a headbutt on the %3!','','','Black Bear','Tom','','','It can be found in the east.',''];
_tr_w["eng"][Number('203') || 0]=['20','打撃','','','7','500','A cute, brown stuffed bear. It\'s a stuffed animal...isn\'t it?','%1 swung the %2!','The %2 landed a headbutt on the %3!','','','Brown Bear','Brown','','','It can be found in the northwest.',''];
_tr_w["eng"][Number('204') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('205') || 0]=['30','打撃','前９','','7','2500','The right side of my beloved house. It feels right at home when using it as a weapon.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','House (Right)','','','','It forms the right side of your house.','9 squares in front'];
_tr_w["eng"][Number('206') || 0]=['30','打撃','前９','','7','3200','The left side of my house, which includes the armory. It feels right at home when using it as a weapon.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','House (Left)','','','','It forms the left side of your house.','9 squares in front'];
_tr_w["eng"][Number('207') || 0]=['29','打撃','前９','','7','2200','A complete stranger\'s house. Their lack of objection to its weaponization is disturbing.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Private House','','','','Many of them can be found throughout the village.','9 squares in front'];
_tr_w["eng"][Number('208') || 0]=['27','打撃','前９','','7','1900','A worn-down house owned by a lumberjack. Its condition and filth do little to discourage its weaponization.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Lumberjack\'s House','','','','It can be found in the west.','9 squares in front'];
_tr_w["eng"][Number('209') || 0]=['25','打撃','前９','','7','2800','A strange house inhabited by a witch. Its gaudy roof is its most prominent feature.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Witch\'s House','','','','It\'s hidden in the forest to the west.','9 squares in front'];
_tr_w["eng"][Number('210') || 0]=['32','打撃','前９','','7','3300','A holy church built on the coast. Praise the gods as you swing it.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','Some sort of holy power worked its magic on the %3!','','Church','','','Resets status buffs','It can be found on the northeast coast.','9 squares in front'];
_tr_w["eng"][Number('211') || 0]=['31','打撃','前９','','7','3000','A store that sells a little of everything. It can be conveniently used as a weapon as well.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Convenience Store','','','','It can be found in the north.','9 squares in front'];
_tr_w["eng"][Number('212') || 0]=['25','打撃','前９','','7','1500','A simple tent that anyone can set up. It\'s a convenient weapon that is easy to carry.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Tent','','','','It can be found in the east.','9 squares in front'];
_tr_w["eng"][Number('213') || 0]=['33','打撃','前９','','7','4500','A cave filled with rock-hard enemies. Its walls are also hard, making them good weapons.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Cave of Iron','','','','It can be found in the southwest.','9 squares in front'];
_tr_w["eng"][Number('214') || 0]=['24','打撃','前９','','7','4400','A cave filled with weak but intelligent enemies. Its poisonous color is its most prominent feature.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Cave of the Weak','','','','It can be found in the southeast.','9 squares in front'];
_tr_w["eng"][Number('215') || 0]=['28','打撃','前９','','7','4800','A cave inhabited by magical enemies. The cave itself is also imbued with magic.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Cave of Illusion','','','Casts random EX magic','It can be found in the forest to the west.','9 squares in front'];
_tr_w["eng"][Number('216') || 0]=['26','打撃','前９','','7','5200','Lost souls possessed an abandoned castle, turning it into a cave. The cave has been consumed by a terrible curse.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Cave of Lost Souls','','','Instant Kill (5%)','It can be found in the northeast.','9 squares in front'];
_tr_w["eng"][Number('217') || 0]=['35','打撃','前９','','7','7500','A golden cave that houses a legendary weapon. The cave itself can also be used as a weapon.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Cave of Legends','','','','You sense its presence to the north of the village.','9 squares in front'];
_tr_w["eng"][Number('218') || 0]=['38','打撃','前９','','7','8700','An evil castle ruled by the Demon Lord. This castle\'s power as a weapon is equal to that of the lord himself.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Demonic Castle','','','','It can be found at the northernmost point of the map.','9 squares in front'];
_tr_w["eng"][Number('219') || 0]=['45','打撃','前９','','7','9800','A village that\'s always peaceful. You have lived in this beloved village your entire life.','%1 swung the %2 down with a bang!','The %2 slammed into the %3!','','','Peaceville','','','','It is the village you call home.','9 squares in front'];
_tr_w["eng"][Number('220') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('221') || 0]=['8','打撃','直線２','','5','600','A crack that is larger than normal. As usual, the bigger, the better.','%1 swung the %2 with her hands!','The %3 was hit by the %2!','The %3 fell into the fissure!','','Large Crack','','','Defense Down (100%)','It can be found in a cave made of iron.','2 squares straight line'];
_tr_w["eng"][Number('222') || 0]=['6','打撃','','','5','300','A hole in the lower part of a wall. Maybe it\'s due to aging?','%1 swung the %2 with her hands!','The %3 was hit by the %2!','The %3 fell into the fissure!','','Wall Crack','','','Defense Down (70%)','It can be found in a cave made of iron.',''];
_tr_w["eng"][Number('223') || 0]=['4','打撃','','','5','500','A broken wall with its inner layer exposed. Its fragility makes it hard to use as a weapon.','%1 swung the %2 with her hands!','The %3 was hit by the %2!','','','Bare Wall','','','','It can be found in a cave filled with wandering souls.',''];
_tr_w["eng"][Number('224') || 0]=['2','打撃','','','3','100','A dark patch on the ground. It looks dirty, and you\'re reluctant to touch it.','%1 swung the %2 with her hands!','The %3 is shrouded in darkness!','A bit of darkness got into the %3\'s eyes!','','Darkness','','','Accuracy Down (100%)','It can be found on the ground at the northernmost point of the map.',''];
_tr_w["eng"][Number('225') || 0]=['2','打撃','','','3','100','A patch of green earth. Much of it is damp.','%1 swung the %2 with her hands!','The %3 grew a little greener!','The %3\'s feet got stuck!','','Green Ground','','','Evasion Down (100%)','It can be found in a cave made of iron.',''];
_tr_w["eng"][Number('226') || 0]=['2','打撃','','','3','200','Just a commonplace road. It makes walking easier.','%1 swung the %2 with her hands!','The %3 was hit by the %2!','','','Road','','','','It can be found pretty much anywhere.',''];
_tr_w["eng"][Number('227') || 0]=['4','斬撃','','','3','400','A white tile made of stone. Four tiles make a single set.','%1 swung the %2 with her hands!','The corner of the %2 cut into the %3!','','','White Tile','','','','It can be found in a convenience store somewhere.',''];
_tr_w["eng"][Number('228') || 0]=['3','斬撃','','','3','200','A white tile worn down by age. It\'s in pretty bad shape.','%1 swung the %2 with her hands!','The corner of the %2 cut into the %3!','','','Cracked White Tile','','','','It can be found in a cave filled with wandering souls.',''];
_tr_w["eng"][Number('229') || 0]=['5','斬撃','','','3','500','A blue tile made of stone. It\'s a bit harder than the white tile.','%1 swung the %2 with her hands!','The corner of the %2 cut into the %3!','','','Blue Tile','','','','It can be found in the northeast.',''];
_tr_w["eng"][Number('230') || 0]=['3','斬撃','','','3','250','A blue tile that is old and cracked. It has clearly seen better days.','%1 swung the %2 with her hands!','The corner of the %2 cut into the %3!','','','Cracked Blue Tile','','','','It can be found in the northeast.',''];
_tr_w["eng"][Number('231') || 0]=['5','斬撃','','','3','400','A beautiful wooden floor. It looks like it\'s been well-maintained.','%1 swung the %2 with her hands!','The corner of the %2 cut into the %3!','','','Wooden Floor','','','','It can be found in various houses.',''];
_tr_w["eng"][Number('232') || 0]=['8','斬撃','','','3','500','A tile with a flashy checkered pattern. I kind of like...uh...never mind.','%1 swung the %2 with her hands!','The corner of the %2 cut into the %3!','','','Checkered Tile','','','','It can be found in a strange house somewhere.',''];
_tr_w["eng"][Number('233') || 0]=['6','斬撃','','','3','300','A wooden church floor. It has a natural feel to it.','%1 swung the %2 with her hands!','The corner of the %2 cut into the %3!','','','Church Floor','','','','It can be found in a church somewhere.',''];
_tr_w["eng"][Number('234') || 0]=['12','打撃','','','6','1500','An anvil you can use to maintain your weapons. It sits upon a large stump.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','','','Anvil','','','','It can be found in various locations.',''];
_tr_w["eng"][Number('235') || 0]=['7','なし','','','2','600','A portion of the huge poisonous swamp found in the Cave of the Weak.','%1 swung the %2 with her hands!','','','','Poisonous Swamp (Small)','','','Poison (100%)','It can be found in a cave located in a poisonous swamp.',''];
_tr_w["eng"][Number('236') || 0]=['15','打撃','','','4','2000','A wicked lantern forged in the depths of Hell. Its purple flame has an evil feel to it.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','Fire erupted from the %2!','Hell\'s Lantern','','','Emits fire','It can be found in the Demon Lord\'s castle.',''];
_tr_w["eng"][Number('237') || 0]=['12','斬撃','直線２','4','20','15000','A mysterious sword received from the bears. Holding it reminds you of them.','%1 swung the %2!','','','','Grizzly Sword','','','4x Attack','It can be obtained from the four bears.','2 squares straight line'];
_tr_w["eng"][Number('238') || 0]=['11','打撃','直線２','','15','9000','An incredible broom that allows you to fly. It\'s just like one a witch might use.','%1 swung the %2!','The %2 landed a direct hit on the %3!','','','Flying Broom','','','Enables flight','Your younger sister is hiding it somewhere.','2 squares straight line'];
_tr_w["eng"][Number('239') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('240') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('241') || 0]=['13','打撃','','','3','499','Somebody\'s corpse. It\'ll never talk back to you.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 was cursed!','','Corpse','','','Instant Kill (5%)','It can be found to the south of the village.',''];
_tr_w["eng"][Number('242') || 0]=['13','打撃','','','3','399','Some sort of animal\'s corpse. You won\'t be teaching it any tricks.','%1 swung the %2!','The %2 landed a direct hit on the %3!','The %3 was cursed!','','Animal Corpse','','','Instant Kill (5%)','It can be found in the southwest.',''];
_tr_w["eng"][Number('243') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('244') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('245') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('246') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('247') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('248') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('249') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('250') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('251') || 0]=['en','打撃','','','6','500','A mysterious slime that roams the wild. It attacks people on sight for no reason.','Go forth, %2!','The %2 jiggled as it slammed into the %3!','','','Slime','','','','It often appears near the village.',''];
_tr_w["eng"][Number('252') || 0]=['en','打撃','','','6','700','A subspecies of slime that is red in color. It\'s more ferocious than its more common counterpart.','Go forth, %2!','The %2 jiggled as it slammed into the %3!','','','Red Slime','','','','It appears in the north.',''];
_tr_w["eng"][Number('253') || 0]=['en','打撃','','','6','2000','A rare slime of shimmering gold. It looks like it might fetch a handsome sum.','Go forth, %2!','The %2 jiggled as it slammed into the %3!','','','Gold Slime','','','','It appears in a cave of glittering gold.',''];
_tr_w["eng"][Number('254') || 0]=['en','打撃','','','6','800','A horse with a flaming body. It has trouble cooling off.','Go forth, %2!','The %2 kicked the %3!','','','Night Mare','','','Breathes fire','It appears to the west of the village.',''];
_tr_w["eng"][Number('255') || 0]=['en','打撃','','','6','1600','A horse wrapped in a flame of a sinister color. It came from the depths of Hell.','Go forth, %2!','The %2 kicked the %3!','','','Hellish Mare','','','Breathes fire','It appears in the Demon Lord\'s castle.',''];
_tr_w["eng"][Number('256') || 0]=['en','打撃','','','6','600','A mischievous monster. It likes to throw things.','Go forth, %2!','The %2 slammed into the %3!','','','Goblin','','','Throws weapons','It appears to the south of the village.',''];
_tr_w["eng"][Number('257') || 0]=['en','打撃','','','6','900','A violent monster that resembles a bull. It likes to throw its many weapons.','Go forth, %2!','The %2 slammed into the %3!','','','Minotaur','','','Throws weapons','It appears in the north.',''];
_tr_w["eng"][Number('258') || 0]=['en','打撃','','','7','500','A large humanoid creature completely made of stone. It doesn\'t fear cuts from blades.','Go forth, %2!','The %2 hit the %3 with its stone arms.','','','Golem','','','','It appears in a cave made of iron.',''];
_tr_w["eng"][Number('259') || 0]=['en','打撃','','','7','700','A large creature whose cells are all made of stone. Its arms are incredibly destructive.','Go forth, %2!','The %2 hit the %3 with its stone arms.','','','Elder Golem','','','','It appears in a cave of glittering gold.',''];
_tr_w["eng"][Number('260') || 0]=['en','打撃','','','6','600','A pest that is incredibly hard. Its shell is hard enough to break stone.','Go forth, %2!','The %2 fired rock fragments!','','','Steel Bug','','','Defense Down (50%)','It appears in a cave made of iron.',''];
_tr_w["eng"][Number('261') || 0]=['en','打撃','','','6','3000','The hardest living thing out there. Physical attacks have no effect on it.','Go forth, %2!','The %2 fired broken pieces of armor!','','','Diamond Bug','','','Defense Down (50%)','It appears in the Demon Lord\'s castle.',''];
_tr_w["eng"][Number('262') || 0]=['en','打撃','','','5','1000','A spirit of fire born from flame. Get too close, and you\'re bound to get burned.','Go forth, %2!','The %2 slammed into the %3!','','','Fire Spirit','','','Casts fire magic','It appears in a magical cave.',''];
_tr_w["eng"][Number('263') || 0]=['en','打撃','','','5','1000','A spirit of ice born from frost. The air around it is always below freezing.','Go forth, %2!','The %2 slammed into the %3!','','','Ice Spirit','','','Casts ice magic','It appears in a magical cave.',''];
_tr_w["eng"][Number('264') || 0]=['en','打撃','','','5','1000','A spirit of lightning born from a thunderous boom. It summons lightning when it\'s angry.','Go forth, %2!','The %2 slammed into the %3!','','','Lightning Spirit','','','Casts lightning magic','It appears in a magical cave.',''];
_tr_w["eng"][Number('265') || 0]=['en','打撃','','','5','1800','A spirit king that rules over all other spirits. Its divine appearance is breathtaking.','Go forth, %2!','The %2 slammed into the %3!','','','Spirit King','','','Casts random EX magic','It appears in a cave of glittering gold.',''];
_tr_w["eng"][Number('266') || 0]=['en','打撃','','','5','800','A creepy monster with countless eyes. It lowers your ATK.','Go forth, %2!','The %2 slammed into the %3!','','','Red Gaze','','','Casts attack weakening magic','It appears in the south.',''];
_tr_w["eng"][Number('267') || 0]=['en','打撃','','','5','700','A creepy monster with a blue body. The accursed stare of its many eyes will leave you defenseless.','Go forth, %2!','The %2 slammed into the %3!','','','Blue Gaze','','','Casts defense weakening magic','It appears in the south.',''];
_tr_w["eng"][Number('268') || 0]=['en','打撃','','','5','1500','A strangely colored gaze dyed red by the blood of its victims. Be wary of its weakening magic.','Go forth, %2!','The %2 slammed into the %3!','','','Blood Gaze','','','Casts overall weakening magic','It appears in the Demon Lord\'s castle.',''];
_tr_w["eng"][Number('269') || 0]=['en','打撃','','','6','900','A jellyfish shrouded in a mysterious veil. Magic doesn\'t work on it at all.','Go forth, %2!','The %2 attacked the %3 with its tingling arms!','','','Magic Jellyfish','','','Shock (25%)','It appears in a magical cave.',''];
_tr_w["eng"][Number('270') || 0]=['en','打撃','','','6','1100','A terrifying monster that invites people to their doom. It is said to be the messenger for the god of death.','Go forth, %2!','The %2 attacked the %3 while laughing wickedly!','','','Death\'s Messenger','','','Casts dark magic','It appears in a cave filled with wandering souls.',''];
_tr_w["eng"][Number('271') || 0]=['en','打撃','','','6','1700','An evil monster that rules over death. It drags its victims down into the depths of Hell.','Go forth, %2!','The %2 attacked the %3 while laughing wickedly!','','','Reaper','','','Casts EX dark magic','It appears in the Demon Lord\'s castle.',''];
_tr_w["eng"][Number('272') || 0]=['en','打撃','','','6','699','A corpse animated by a soul. It cackles as it strikes down foes with a bone.','Go forth, %2!','The %2 attacked the %3 while laughing wickedly!','','','Skeleton','','','','It appears in a cave filled with wandering souls.',''];
_tr_w["eng"][Number('273') || 0]=['en','打撃','','','6','700','An extremely dangerous mushroom that scatters spores as it attacks.','Go forth, %2!','The %2 slammed into the %3!','','','Monstrous Mushroom','','','Scatters spores','It appears in the forest to the west.',''];
_tr_w["eng"][Number('274') || 0]=['en','打撃','','','4','1000','A mushroom with a body that\'s clearly poisonous. Be careful not to mistakenly eat one.','Go forth, %2!','The %2 slammed into the %3!','','','Demonshroom','','','Scatters spores','It appears in the north.',''];
_tr_w["eng"][Number('275') || 0]=['en','打撃','','','6','500','A worm that crawls on the ground. It is as disgusting as it is troublesome.','Go forth, %2!','The %2 is squirming around!','','','Red Worm','','','','It appears in a cave located in a poisonous swamp.',''];
_tr_w["eng"][Number('276') || 0]=['en','打撃','','','3','800','An unbelievably loud bird. Nary a soul could sleep through its song.','Go forth, %2!','The %2 screeched at 150 decibels!','','','Decibel Bird','','','Prevents non-standard attack(50%)','It appears to the west of the village.',''];
_tr_w["eng"][Number('277') || 0]=['en','打撃','','','3','1200','It may be weak and a crybaby, but it\'s also clever. It\'s an irritating monster that likes to buff itself.','Go forth, %2!','The %2 scratched at the %3 while crying!','','','Crybaby Devil','','','Casts a random buff','It appears in a cave located in a poisonous swamp.',''];
_tr_w["eng"][Number('278') || 0]=['en','打撃','','','3','900','A devil strong of both body and mind. It doesn\'t want for intelligence, either.','Go forth, %2!','The %2 bravely scratched at the %3!','','','Brave Devil','','','Casts a random buff','It appears in a cave of glittering gold.',''];
_tr_w["eng"][Number('279') || 0]=['en','打撃','','2','5','600','A bat that bites as it flies around. Beware of its nimble movements.','Go forth, %2!','The %2 quickly bit the %3 twice!','','','Bat','','','2x Attack','It appears near the village.',''];
_tr_w["eng"][Number('280') || 0]=['en','打撃','','2','5','900','A nuisance that attacks under the cover of darkness. Its bite is extremely painful.','Go forth, %2!','The %2 quickly bit the %3 twice!','','','Dark Bat','','','2x Attack','It appears in a cave filled with wandering souls.',''];
_tr_w["eng"][Number('281') || 0]=['30','なし','','','1','1100','A dangerous rock to say the least. It explodes when it is close to death.','Go forth, %2!','The %2 exploded!','','','Ballistic Rock','','','Explodes','It appears in a cave made of iron.',''];
_tr_w["eng"][Number('282') || 0]=['en','打撃','','','6','2200','The king of the worms. It fires a squirming projectile at enemies.','Go forth, %2!','The %2 is squirming around!','','','Blue Worm','','','','It appears in the Demon Lord\'s castle.',''];
_tr_w["eng"][Number('283') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('284') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('285') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('286') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('287') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('288') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('289') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('290') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('291') || 0]=['en','斬撃','','3','8','8500','A huge dragon covered in strong plate-like scales. Its claws can cut through anything.','Go forth, %2!','The %2 slashed away with its claws of steel!','','','Armored Dragon Elimsalue','','','3x Attack','It appears deep inside a cave made of iron.',''];
_tr_w["eng"][Number('292') || 0]=['en','打撃','直線３','','7','7700','A strange bird that appears in the Cave of the Weak. Its tactics revolve around buffing itself.','Go forth, %2!','The %2 spewed poison at its foe!','','','Ominous Bird Vijaya','','','Casts buffs,Poison (50%)','It appears deep inside a cave located in a poisonous swamp.','3 squares straight line'];
_tr_w["eng"][Number('293') || 0]=['en','打撃','前６','','9','9900','A strong-willed creature wrapped in mystery. Its body doesn\'t do much, instead relying on its tentacles to fight.','Go forth, %2!','The %2 leaned heavily on the %3!','','','Thyrsos, Grand Demon','','','','It can be found in abundance in a magical cave.','6 squares in front'];
_tr_w["eng"][Number('294') || 0]=['en','打撃','直線３','','4','2400','A tentacle of a ghastly color. It appears to possess magical powers.','Go forth, %2!','The %2 lashed out like a whip and hit the %3!','','','Demonic Tentacle','','','Casts random offensive magic','It can be found in abundance in a magical cave.','3 squares straight line'];
_tr_w["eng"][Number('295') || 0]=['en','斬撃','','','7','8800','A knight\'s spirit that appears in the Cave of Lost Souls. You can feel its hatred and thirst for vengeance.','Go forth, %2!','%2 impaled the %3!','','','Ghost Knight Dáinsleif','','','Casts EX dark magic','It appears deep inside a cave filled with wandering souls.',''];
_tr_w["eng"][Number('296') || 0]=['en','打撃','直線２','','15','19999','A weapon god that rules over all weapons. This god has no equal.','Go forth, %2!','The %2 unleashed a Divine Strike!','The %3 is in awe of the god\'s majesty.','','God of Weaponry','Ameno Ohbari','','Immobilize - 1 turn (30%)','It appears deep inside a cave of glittering gold.','2 squares straight line'];
_tr_w["eng"][Number('297') || 0]=['en','打撃','','2','12','19999','A demon lord hell-bent on the world\'s destruction. In reality though, it\'s world peace he\'s after.','Go forth, %2!','%2 used Fists of Hell!','','','Demon Lord Dolhabach','','','2x Attack,Confuse (20%)','He appears on the top floor of the Demonic Castle.',''];
_tr_w["eng"][Number('298') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('299') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('300') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('301') || 0]=['14','斬撃','','','5','800','A stray dog that you see wandering the streets. It is friendly and cute.','Go forth, %2!','The %2 bit the %3!','The %2\'s barking scared the %3!','','Dog','','','Immobilize - 1 turn (20%)','He can be found walking around the village.',''];
_tr_w["eng"][Number('302') || 0]=['13','打撃','','','5','800','A stray cat that you see wandering about. It mews and purrs.','Go forth, %2!','The %2 struck the %3 with a vicious cat punch!','The %2 purred as it rubbed up against the %2!','','Cat','','','Attack Down (50%)','He can be found walking around the village.',''];
_tr_w["eng"][Number('303') || 0]=['16','打撃','','','6','1500','A horse owned by someone. Its coat is clean and beautiful.','Go forth, %2!','The %2 kicked the %3!','','','Horse','','','Knock Back','It wanders around the northwest.',''];
_tr_w["eng"][Number('304') || 0]=['19','打撃','','','6','1600','An amazing sheep that gives you hints about weapons. Its fluffy coat is adorable.','Go forth, %2!','The %2 slammed into the %3!','The %2\'s fluffy coat swallowed up the %3!','','Sheep','','','Stamina Down (30%)','It\'s me.',''];
_tr_w["eng"][Number('305') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('306') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('307') || 0]=['18','打撃','','','6','720','A woman who lives alone in Peaceville. She\'s as kind as she is lovely.','Go forth, %2!','The %2 gently headbutted the enemy!','','','Village Woman','Piranka','','','She lives in a house somewhere in the village.',''];
_tr_w["eng"][Number('308') || 0]=['8','打撃','','','6','1050','A kind parent who resides in the village. He\'s always outside, playing with his children.','Go forth, %2!','The %2 unleashed a breadwinner\'s punch!','','','Doting Father','Tel-ho','','','He\'s watching his kids play in the village.',''];
_tr_w["eng"][Number('309') || 0]=['25','打撃','','','3','960','A mother who lives with her family in Peaceville. Her kind exterior hides a ferocious temper.','Go forth, %2!','The %2 attacked with clenched fists!','','The %2 breathed fire!','Kind Mother','Hokkoma','','Breathes fire','She lives in a house somewhere in the village.',''];
_tr_w["eng"][Number('310') || 0]=['12','打撃','','','5','210','The older of two energetic brothers who run around the village. His favorite word is \"Wooo.\"','Go forth, %2!','The %2 punched while shouting \"Wooo!\"','','','Energetic Brother','Kubota','','','He\'s playing somewhere in the village.',''];
_tr_w["eng"][Number('311') || 0]=['6','打撃','','','5','180','The cute younger half of the two brothers. He wants to be an influencer on social media someday.','Go forth, %2!','The %2 unleashed a Deadly Ultra Rider Kick!','','','Energetic Brother','Nagina','','','He\'s playing somewhere in the village.',''];
_tr_w["eng"][Number('312') || 0]=['17','打撃','','','6','1530','The chef that manages the Peaceville Pub. He displays his fancy footwork in battle.','Go forth, %2!','The %2 hit the enemy with a powerful kick!','','','Restaurant Chef','A flavor','','','The chef is working somewhere in the village.',''];
_tr_w["eng"][Number('313') || 0]=['11','打撃','','','6','840','A patron who visits the Peaceville Pub every day. His blood sugar levels have been a bit high lately.','Go forth, %2!','The %2 unleashed a blood sugary punch!','The %3\'s blood sugar levels spiked!','','Restaurant Patron','Dongfa','','Stamina Down (20%)','He is eating a meal somewhere in the village.',''];
_tr_w["eng"][Number('314') || 0]=['13','打撃','','','6','1500','A connoisseur who travels the world in search of culinary delights. He holds the Peaceville Pub in high regard.','Go forth, %2!','The %2 squashed the %3 with his protruding belly!','','','Gourmet','Katsubaruga','','','He is eating a meal somewhere in the village.',''];
_tr_w["eng"][Number('315') || 0]=['4','打撃','','','4','2520','An old man with gentle eyes who inhabits the village. He visits his wife\'s grave every day.','Go forth, %2!','The %2 feebly tackled the enemy!','The %2\'s deceased wife cursed the %3!','','Grieving Old Man','Braig','','Instant Kill (5%)','He\'s visiting a grave somewhere in the village.',''];
_tr_w["eng"][Number('316') || 0]=['25','打撃','','','6','1740','A woman often found gossiping near the well. Once she starts talking, she won\'t stop for hours.','Go forth, %2!','The %2 slapped her target hysterically!','The %3 is in shock at the sight of the hysteric woman!','','Woman by the Well','Fukiya','','Confuse (20%)','She can be found near a well in the village.',''];
_tr_w["eng"][Number('317') || 0]=['15','打撃','','2','6','1680','A woman who loves to gossip. She\'s always lost in conversation near the well.','Go forth, %2!','The %2 unleashed a vicious double slap!','','','Woman by the Well','Illisa','','2x Attack','She can be found near a well in the village.',''];
_tr_w["eng"][Number('318') || 0]=['7','打撃','','','4','2160','An old man who strolls through the village. He\'ll talk your ear off if you let him.','Go forth, %2!','The %2 hit the %3 with his cane!','','','Ambulatory Old Man','Rod','','','He can be found walking around the village.',''];
_tr_w["eng"][Number('319') || 0]=['10','打撃','','','6','1050','A man who strolls through the village. He\'s far from home, trying to earn a living.','Go forth, %2!','The %2 sent the %3 flying with a kick from his powerful legs!','','','Relaxed Man','Ranyapa','','Knock Back','He can be found walking around the village.',''];
_tr_w["eng"][Number('320') || 0]=['','','','','','0','','','','','','','','','','',''];
_tr_w["eng"][Number('321') || 0]=['','','','','','0','','','','','','','','','','',''];
_tr_w["eng"][Number('322') || 0]=['5','打撃','','','6','630','An adventurer who seeks the legendary weapon. He is a real momma\'s boy.','Go forth, %2!','The %2 unleashed a punch that he learned from his mother!','','','Adventurer','Kikangji','','','He can be found walking in the south.',''];
_tr_w["eng"][Number('323') || 0]=['13','打撃','','','9','1470','An old man who bathes in the pond every day. You wish he\'d let you know ahead of time.','Go forth, %2!','The %2 slammed into the %3 while cackling!','The %3 is in shock at the sight of the old creep!','','Creepy Man','Tetsken','','Confuse (40%)','He can found south of the village.',''];
_tr_w["eng"][Number('324') || 0]=['18','打撃','','','6','810','A young man camping by the seaside. He\'s probably more of an indoors person.','Go forth, %2!','The %2 attacked by swinging his camping tools.','','','Camper','Bastar','','','He can be found camping in the south.',''];
_tr_w["eng"][Number('325') || 0]=['15','打撃','','','6','540','A woman who likes to camp alone. Her grandfather got her into camping.','Go forth, %2!','The %2 attacked by swinging his camping tools.','','','Camper','Ikpa','','','He can be found camping in the south.',''];
_tr_w["eng"][Number('326') || 0]=['17','打撃','','','6','960','A camper that likes to amuse himself. He likes fire a little too much.','Go forth, %2!','The %2 attacked by swinging his camping tools.','','The %2 started building a campfire!','Camper','Charak','','Emits fire','He can be found camping in the south.',''];
_tr_w["eng"][Number('327') || 0]=['14','斬撃','','','6','750','A woman on a journey to find herself. She taught you just how dangerous caves can be.','Go forth, %2!','The %2\'s short sword cut into the %3!','','','Traveling Woman','Cathrus','','','She can be found traveling in the south.',''];
_tr_w["eng"][Number('328') || 0]=['22','斬撃','','','6','600','An adventurer who taught you the dangers of the poisonous swamp. This kind soul also gave you an herb.','Go forth, %2!','The %2\'s beloved blade cut into the %3!','','','Adventurer','Fureil','','','He can be found the southeast.',''];
_tr_w["eng"][Number('329') || 0]=['28','斬撃','','','8','1950','A lumberjack who lives alone in an old cabin. He\'s good with an axe.','Go forth, %2!','The %2 swung his axe as if he was chopping down a tree!','','','Lumberjack','Wonmar','','Defense Down (30%)','He lives in a cabin somewhere.',''];
_tr_w["eng"][Number('330') || 0]=['5','打撃','','','4','2310','An old man who likes to take his time when traveling. He\'s still searching for the meaning of life.','Go forth, %2!','The %2 meekly headbutted the enemy!','','','Old Traveler','Brood','','','He can be found to the west of the village.',''];
_tr_w["eng"][Number('331') || 0]=['11','斬撃','','','6','990','A man in pursuit of strength. He acts like a grizzled veteran but isn\'t very tough.','Go forth, %2!','The %2 attacked the %3 with his sword!','','','Adventurer','Esto','','','He can be found walking west of the village.',''];
_tr_w["eng"][Number('332') || 0]=['9','打撃','','','6','5400','A small witch who lives in the forest. She is quite adept at magic by the looks of it.','Go forth, %2!','The %2 attacked with her staff!','','Magic shot out from the tip of the %2\'s staff!','Witch','Cremon','','Casts random EX magic','She lives in a strange house somewhere.',''];
_tr_w["eng"][Number('333') || 0]=['17','打撃','','','6','1260','A woman that walks toward the bazaar every day. She dreams of the day when she\'ll finally get there.','Go forth, %2!','The %2 sent the %3 flying with a kick from her strong legs!','','','Hopeful Woman','Takana','','Knock Back','She can be found walking east of the village.',''];
_tr_w["eng"][Number('334') || 0]=['24','打撃','','','6','630','A woman who runs a shop at the bazaar. She\'s a very hard worker.','Go forth, %2!','The %2 swung her morning star!','','','Merchant Woman','Yawarra','','Confuse (20%)','He can be found selling things in the east.',''];
_tr_w["eng"][Number('335') || 0]=['20','斬撃','','2','6','1020','A man who sells vegetables at the bazaar. He\'s a little too fond of his grandmother.','Go forth, %2!','The %2 sliced the %3 quickly and decisively!','','','Merchant Man','Rapier','','2x Attack','He can be found selling things in the east.',''];
_tr_w["eng"][Number('336') || 0]=['14','打撃','','','7','1650','A swindler if you\'ve ever seen one. You probably shouldn\'t get involved with him.','Go forth, %2!','The %2 squashed the %3 with his protruding belly!','','','Merchant Man','Pusa','','','He can be found selling things in the east.',''];
_tr_w["eng"][Number('337') || 0]=['19','打撃','','','6','720','A red-haired man having fun at the bazaar. He\'s barely got two coins to rub together.','Go forth, %2!','The %2 fired a stiff right jab at the enemy!','','','Bazaar Customer','Shrink','','','He can be found perusing the bazaar in the east.',''];
_tr_w["eng"][Number('338') || 0]=['8','斬撃','','','6','660','A woman visiting the bazaar. She\'s waiting for the half-off sale to begin.','Go forth, %2!','The %2 attacked the %3 with a sword she bought at the bazaar!','','','Bazaar Customer','Scruma','','','He can be found perusing the bazaar in the east.',''];
_tr_w["eng"][Number('339') || 0]=['12','打撃','','','4','1350','A man that came to the bazaar to kill time. He appears to have purchased a suspicious-looking pot.','Go forth, %2!','The %2 hit the %2 with the pot he bought at the bazaar!','','','Bazaar Customer','Bundy','','Guarantees a Critical Hit','He can be found perusing the bazaar in the east.',''];
_tr_w["eng"][Number('340') || 0]=['18','打撃','','2','6','1440','A wealthy bazaar customer. He loves thumbing through his wad of bills.','Go forth, %2!','The %2 took out a wad of money and slapped the enemy with it!','The %3 got excited after seeing all that money!','','Bazaar Customer','Murgiza','','2x Attack,Stamina Up (Opponent)','He can be found perusing the bazaar in the east.',''];
_tr_w["eng"][Number('341') || 0]=['17','打撃','','','5','1230','The pastor of a church along the seaside. He\'s an adherent of the THROWING Sect.','Go forth, %2!','The %2 unleashed an Armen punch!','Some sort of holy power worked its magic on the %3!','','Church Pastor','Falsio','','Resets status buffs','She can be found in a church somewhere.',''];
_tr_w["eng"][Number('342') || 0]=['22','打撃','','','6','1140','A kind and friendly nun. She loves flat noodles.','Go forth, %2!','The %2 slammed hard into the enemy!','','','Church Nun','Lancia','','','She can be found in a church somewhere.',''];
_tr_w["eng"][Number('343') || 0]=['19','打撃','','','5','2460','An old woman praying at the church. She\'s a devoted follower of the THROWING Sect.','Go forth, %2!','The %2 tackled the enemy while she prayed!','','','Devout Old Woman','Versa','','','He can be found praying in a church somewhere.',''];
_tr_w["eng"][Number('344') || 0]=['13','打撃','','','6','450','A young man praying at the church. He is a very serious young man.','Go forth, %2!','The %2 headbutted the enemy as he prayed!','','','Devout Young Man','Irarufu','','','He can be found praying in a church somewhere.',''];
_tr_w["eng"][Number('345') || 0]=['14','打撃','','','6','750','A green-haired adventurer who looks good in glasses. She likes to explore abandoned ruins.','Go forth, %2!','The %2 attacked with her staff!','Magic shot out from the tip of the %2\'s staff!','','Adventurer','Clique','','Casts ice magic','She can be found standing in the northeast.',''];
_tr_w["eng"][Number('346') || 0]=['10','打撃','','','5','2040','An old man who likes fishing by the sea. However, he\'s not very good at it.','Go forth, %2!','The %2 struck the enemy with his fishing pole!','A rotten earthworm was decaying on its hook!','','Fisherman','Phyzigro','','Poison (60%)','He can be found fishing off the cape in the northwest.',''];
_tr_w["eng"][Number('347') || 0]=['12','斬撃','','3','6','510','A strong-willed female adventurer. She\'s been looking for work.','Go forth, %2!','The %2 swiftly slashed at the %3 with her short sword!','','','Adventurer','Dagger','','3x Attack','She can be found walking in the north.',''];
_tr_w["eng"][Number('348') || 0]=['27','打撃','','','6','660','A newly-hired convenience store clerk. The store\'s employee manual is law.','Go forth, %2!','The %2 struck the %3 with a barcode scanner!','','','Store Clerk','Miaota','','','He can be found in a convenience store somewhere.',''];
_tr_w["eng"][Number('349') || 0]=['15','斬撃','','','6','900','A man shopping at the convenience store. He can\'t decide which juice to buy.','Go forth, %2!','The %2 slashed at the %3 with his debit card!','','','Customer','Arsato','','','He can be found in a convenience store somewhere.',''];
_tr_w["eng"][Number('350') || 0]=['33','打撃','','','7','990','A juvenile delinquent loitering in front of the convenience store. You wouldn\'t like him when he\'s angry.','Go forth, %2!','The %2 pounded the snot out of the %3!','The %3 is scared of the %2!','','Hoodlum','Reserville','','Immobilize - 1 turn (20%)','He can be found loitering in front of a convenience store somewhere.',''];
_tr_w["eng"][Number('351') || 0]=['29','斬撃','','','5','2160','A skilled traveler with an injured eye. He looks extremely strong.','Go forth, %2!','The %2 swung his large sword in a heroic fashion!','','','Traveling Man','Gladys','','','He can be found traveling in the north.',''];
_tr_w["eng"][Number('352') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('353') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('354') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('355') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('356') || 0]=['2','打撃','','','8','180','\\n[1]\'s little sister. Small but reliable. She takes care of the house and runs the shop all by herself.','Go forth, %2!','%2 bravely tried to throw a punch!','%2 stole money from the %3\'s wallet!','','Ponmi','','','Earns money','She\'s your little sister.',''];
_tr_w["eng"][Number('357') || 0]=['30','斬撃','','','9','1140','An unmotivated hero. He doesn\'t want to expend much effort to defeat the Demon Lord.','Go forth, %2!','The %2 seemed annoyed as he swung his sword!','The %3 lost motivation after seeing the %2!','','Valiant Hero Schivardi','','','Attack Down (40%)','He\'ll stop by after you expand your shop.',''];
_tr_w["eng"][Number('358') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('359') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('360') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('361') || 0]=['21','斬撃','直線２','','20','18564','A dark sword the Demon Lord gave to you. It\'s extremely heavy.','%1 swung the %2!','','','','Demon Lord\'s Blade','','','Guarantees a Critical Hit','The Demon Lord has it in his possession.','2 squares straight line'];
_tr_w["eng"][Number('362') || 0]=['17','打撃','','','8','1000','A box that holds the trophies you\'ve earned. It feels heavy in your hands.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3!','','','Trophy Case','','','','It can be found in your home.',''];
_tr_w["eng"][Number('363') || 0]=['0','なし','','','20','14500','A net designed to catch all kinds of insects. Its suction power is like that of a black hole.','%1 swung the %2!','','','','Insect Net','','','It can be used to catch things.','Your younger sister is hiding it somewhere.',''];
_tr_w["eng"][Number('364') || 0]=['25','打撃','','','9','2','A fine specimen. It is moist and features a consistent hue.','%1 swung the %2 with her hands!','The %2 landed a direct hit on the %3\'s face!','The %2 smells really bad!','','Poop','','','Attack Down (20%),Defense Down (20%),Stamina Down (20%)','A steaming pile of it can be found in the northeast.',''];
_tr_w["eng"][Number('365') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('366') || 0]=['','','','','','','','%1 bravely tried to throw a punch!','','','','(Unarmed)','','','3x Attack','',''];
_tr_w["eng"][Number('0') || 0]=['','','','','','','','%1 used an \i[261]herb!','','','','(Herb)','','','Emits fire','',''];
_tr_w["eng"][Number('3') || 0]=['','','','','','','An essential item for any adventurer. It restores HP and cures status ailments.','%1 swung the %2!','The %2 is flying into the %3\'s mouth!','','','Herb','','','Recover enemy\'s HP','Sometimes enemies drops.',''];
_tr_w["eng"][Number('4') || 0]=['','','','','','','A mysterious stone that draws out the latent potential of weapons. It is a rare and precious material.','%1 swung the %2!','The corner of the %2 stabbed the %3!','','','Arma Stone','','','','Sometimes enemies drops.',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["eng"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];

})();
    (function(){


_tr_w["kor"][Number('4') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('5') || 0]=['10','斬撃','前３','3','5','200','무기의 신 붓키가 만든 디버그용으로는 유일무이한 무기.','%1는 %2을 휘둘렀다!','%2가 %3의 머리에 정통으로 맞았다!','%2가 뿜어내는 오라가 %3을(를) 자극하였다!','%2 안에서 화염이 튀어나왔다!','디버그 항아리','','','3회 공격','존재하지 않습니다.','3 매스 앞으로'];
_tr_w["kor"][Number('6') || 0]=['10','斬撃','','','30','1000','대량 생산이 가능한 동 재질의 검. 누구나 다루기 쉽지만 위력면에서는 불안감이 있다.','%1는 %2을 휘둘렀다!','','','','여행자의 검','','','','당신 집의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('7') || 0]=['20','斬撃 / 炎','','','30','2100','화염의 마법을 봉인한 마법의 검. 한 번만 휘둘러도 주변은 불바다가 된다.','%1는 %2을 휘둘렀다!','','','검 끝이 화염으로 타오르고 있다!','업화의 검','','','화염 발생','남쪽의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('8') || 0]=['25','斬撃 / 雷','','','30','7500','전 세계에 이름이 알려진 명검. 천둥 망치로 모든 것을 부숴버린다.','%1는 %2을 휘둘렀다!','','','검 끝에 천둥이 일고 있다!','엑스컬리버','','','천둥 발생','보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('9') || 0]=['60','斬撃','','','999','19999','무기의 신이 만든 황금의 검. 만물을 두 동강 낼 수 있는 최강의 무기.','%1는 %2를 휘둘렀다!','','','','너무강해서 현웃터지는 소드','','','','미안하지만 저도 잘 모르겠어요….',''];
_tr_w["kor"][Number('10') || 0]=['15','斬撃','直線２','','20','1700','거대하고 날카로운 강철 재질의 대검. 의외로 가벼워서 마음껏 휘두를 수 있다.','%1는 %2을 휘둘렀다!','','','','빅 블레이드','','','','북녘땅의 보물 상자 안에 잠들어 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('11') || 0]=['27','斬撃','直線２','','20','3600','고대 거인족이 애용하던 대검. 한 번만 휘둘러도 대지가 갈라진다.','%1는 %2을 휘둘렀다!','','','','거인의 대검','','','','근처 편의점에서 팔고 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('12') || 0]=['12','斬撃','','','30','1200','암살용으로 만들어진 단검. 급소를 찌르려면 고도의 기술이 필요하다.','%1는 %2로 찔렀다!','','','','암살자의 단검','','','즉사(10%)','마을 어떤 집의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('13') || 0]=['14','斬撃 / 闇','','2','30','4666','악마의 저주를 받은 단검. 민첩한 움직임으로 순식간에 적을 벤다.','%1는 %2로 찔렀다!','','','','악마의 나이프','','','2회 공격,독(30%)','북동쪽 해안가의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('14') || 0]=['16','打撃','','','20','1500','물건을 부술 목적으로만 만들어진 망치. 그 파괴력은 무시하지 못한다.','%1는 %2을 휘둘렀다!','','','','완전 깨는 망치','','','혼란(20%)','마을 북동쪽의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('15') || 0]=['28','打撃','','','20','8000','손잡이에 별 모양의 장식이 달린 메이스. 마사지용으로는 사용하지 않는 것이 좋다.','%1는 %2을 휘둘렀다!','','','','모닝 스타','','','혼란(20%)','동부 지방의 상인이 팔고 있습니다.',''];
_tr_w["kor"][Number('16') || 0]=['13','斬撃 / 氷','直線３','','30','1800','바다 신의 무기로 잘 알려진 창. 세 갈래의 창끝은 모든 물건을 꿰뚫는다.','%1는 %2으로 찔렀다!','','','','삼지창','','','','독의 늪으로 덮인 동굴의 보물 상자 안에 잠들어 있습니다.','3 매스 직선'];
_tr_w["kor"][Number('17') || 0]=['26','斬撃 / 炎','直線３','','30','7600','지옥에서 온 창. 지옥의 불꽃으로 모든 것을 태워 버린다.','%1는 %2으로 찔렀다!','','','','지옥의 마창','','','','동부 지방의 상인이 팔고 있습니다.','3 매스 직선'];
_tr_w["kor"][Number('18') || 0]=['15','斬撃','','','20','2000','무겁고 날카로운 날을 가진 강철 재질의 도끼. 갑옷을 무시하고 강력한 상처를 입힌다.','%1는 %2을 휘둘렀다!','','','','강철 도끼','','','방어력을 낮춘다(30%)','동북 지방의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('19') || 0]=['28','斬撃','','','20','9500','무엇이든 두 동강 낸다는 천하무쌍의 도끼. 그 옛날 도깨비 왕이 휘둘렀다고 한다.','%1는 %2을 휘둘렀다!','','','','기간트 액스','','','방어력을 낮춘다(30%)','오두막에 사는 사내가 가지고 있습니다.',''];
_tr_w["kor"][Number('20') || 0]=['8','打撃 / 炎','','','20','2100','고대부터 마도사가 애용하던 마법의 지팡이. 불덩이를 날려 모든 것을 불태워 버린다.','%1는 %2을 휘둘렀다!','','','지팡이 끝에서 화염이 튀어나왔다!','마도사의 지팡이','','','【화염 마법】 발동','철로 만들어진 동굴의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('21') || 0]=['8','打撃 / 雷','','','20','2500','뇌신의 힘이 깃든 마법의 지팡이. 휘두르면 천둥 망치가 하늘에서 쏟아진다.','%1는 %2을 휘둘렀다!','','','지팡이 끝에서 천둥이 나갔다!','천둥 망치 지팡이','','','【천둥 마법】 발동','마을 주민의 소원을 들어준다고 합니다.',''];
_tr_w["kor"][Number('22') || 0]=['8','打撃 / 氷','','','20','2600','전설의 얼음 마녀가 가지고 있다는 지팡이. 눈보라를 일으켜 세상을 꽁꽁 얼린다.','%1는 %2을 휘둘렀다!','','','지팡이 끝에서 얼음이 나타났다!','얼음 마녀의 지팡이','','','【얼음 마법】 발동','마력으로 가득 찬 동굴의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('23') || 0]=['18','打撃 / 闇','','','20','6666','지옥에 사는 대악마의 지팡이. 어둠의 힘을 소환한다.','%1는 %2을 휘둘렀다!','','','지팡이 끝에서 어둠의 힘이 방출되었다!','대악마의 지팡이','','','【궁극의 어둠 마법 】 발동','마족의 왕이 사는 성의 보물 상자 안에 잠들어 있습니다.',''];
_tr_w["kor"][Number('24') || 0]=['18','打撃 / 光','','','20','9999','그 옛날 대현자가 어둠을 봉인하기 위해 사용했다는 지팡이. 성스러운 힘이 깃들어 있다.','%1는 %2을 휘둘렀다!','','','지팡이 끝에서 성스러운 힘이 방출되었다!','대현자의 지팡이','','','무작위로 【궁극】 마법 공격이 발동','어딘가에 사는 마녀가 가지고 있습니다.',''];
_tr_w["kor"][Number('25') || 0]=['20','斬撃','','','20','1500','벽을 파는 도구. 무기로 사용할 수도 있다.','%1는 %2을 휘둘렀다!','','','','피켈','','','방어력을 낮춘다(50%)','남서쪽 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('26') || 0]=['17','斬撃','','','20','1300','구멍을 파는 도구. 무기로도 사용 가능.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','땅을 파는 김에 %3의 발 밑에도 웅덩이를 하나 파 두었다!','','삽','','','1턴 행동 불능(30%)','남쪽 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('27') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('28') || 0]=['7','打撃','前３','','3','300','인근 산에서 자라는 나무. 무분별한 벌채로 자연 파괴가 문제가 되고 있다.','%1는 %2을 휘둘렀다!','%2가 %3의 머리에 정통으로 맞았다!','','','나무','','','','주변에 잔뜩 자라고 있습니다.','3 매스 앞으로'];
_tr_w["kor"][Number('29') || 0]=['6','打撃','前６','4','3','1000','나무를 4개 한 묶음으로 주운 것. 평소보다 4배의 이득을 본 느낌이다.','%1는 나무 4개를 동시에 휘둘렀다!','나무가 %3의 머리에 정통으로 맞았다!','','','나무×4','','','4회 공격','주변에 잔뜩 자라고 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('30') || 0]=['3','打撃','','','3','50','인근에 자라나 있던 작고 메마른 나무. 생명이 있는 것은 언젠가 죽기 마련이다.','%1는 %2를 휘둘러 보았다!','%2는 툭 소리를 내며 %3에 닿았다.','','','메마른 작은 나무','','','HP를 반드시 1 남긴다','남동쪽 땅에 잔뜩 자라고 있습니다.',''];
_tr_w["kor"][Number('31') || 0]=['5','打撃','直線２','','3','200','허무하게 메말라 버린 나무. 왠지 쓸쓸함이 느껴진다.','%1는 %2를 휘둘러 보았다!','%2는 툭 소리를 내며 %3에 닿았다.','','','메마른 나무','','','HP를 반드시 1 남긴다','남동쪽 땅에 잔뜩 자라고 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('32') || 0]=['9','打撃','','','5','200','의자로 쓰기에 딱 좋은 그루터기. 보기만 해도 앉고 싶어진다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2가 %3의 머리에 정통으로 맞았다!','%3(은)는 %2에 앉아서 휴식을 취했다!','','그루터기','','','1턴 행동 불능(75%)','마을 어딘가에 있습니다.',''];
_tr_w["kor"][Number('33') || 0]=['8','打撃','','','2','100','쓰러져 메말라 버린 나무. 지금은 숲의 명물로 유명해졌다.','%1는 %2을 휘둘렀다!','%2가 %3의 머리에 정통으로 맞았다!','%3(은)는 균형을 잃고 쓰러지고 말았다!','','쓰러진 나무','','','1턴 행동 불능(90%)','마을 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('34') || 0]=['10','打撃','直線２','','6','700','남쪽 나라에서 자라는 나무. 보고 있으면 남국의 정취가 느껴진다.','%1는 %2를 신나게 휘둘렀다!','%2가 %3의 머리에 정통으로 맞았다!','','','야자나무','','','','남쪽의 해변에서 자라고 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('35') || 0]=['4','斬撃','','','3','100','어딜 봐도 그냥 풀이다. 그 밖에도 B와 C가 있다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 %2에 왼손 검지를 베였다!','','','풀 A','','','','주변에 잔뜩 자라고 있습니다.',''];
_tr_w["kor"][Number('36') || 0]=['5','斬撃','','','3','100','풀 A와 비교하면 조금 큰 풀. 조금 큰 만큼 이득 본 느낌이 든다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 %2에 왼손 약지를 베였다!','','','풀 B','','','','주변에 잔뜩 자라고 있습니다.',''];
_tr_w["kor"][Number('37') || 0]=['4','斬撃','','3','3','150','풀 A나 B보다 더 큰 풀. 풀 중에서 장남 격인 존재이다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 %2에 오른손 엄지, 검자, 약지를 베였다!','','','풀 C','','','3회 공격','주변에 잔뜩 자라고 있습니다.',''];
_tr_w["kor"][Number('38') || 0]=['2','斬撃','','','2','50','메말라서 생을 마친 풀. 무기로 쓸 수 있을 것 같진 않다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 뚝 소리를 내며 %3에 닿았다.','','','메마른 풀','','','HP를 반드시 1 남긴다','남동쪽 땅에 잔뜩 자라고 있습니다.',''];
_tr_w["kor"][Number('39') || 0]=['8','斬撃','','','3','200','애처롭게 피어 있는 하얀 꽃. 보고만 있어도 마음이 깨끗해지는 것 같다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 꽃잎에 왼손 새끼손가락을 베였다!','%3(은)는 %2에 치유되어 전의를 상실했다!','','하얀 꽃','','','공격력을 낮춘다(50%)','주변에 잔뜩 자라고 있습니다.',''];
_tr_w["kor"][Number('40') || 0]=['9','斬撃','','','3','250','정열적인 빨간 꽃. 너무 귀여워 전의까지 상실하게 된다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3은(는) 꽃잎에 오른손 중지를 베였다!','%3(은)는 %2에 치유되어 전의를 상실했다!','','빨간 꽃','','','공격력을 낮춘다(40%)','주변에 잔뜩 자라고 있습니다.',''];
_tr_w["kor"][Number('41') || 0]=['6','斬撃','','','3','400','종류는 모르지만, 예쁜 꽃. 바라보고만 있어도 혈압이 내려가는 것 같다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3은(는) 꽃잎에 왼손 중지를 베였다!','%3(은)는 %2에 치유되어 전의를 상실했다!','','예쁜 꽃','','','공격력을 낮춘다(75%)','마을이나 숲에서 자라고 있습니다.',''];
_tr_w["kor"][Number('42') || 0]=['20','打撃','','','6','900','끔찍한 악취가 나는 꽃. 가능하면 줍고 싶지 않았다.','%1는 코를 막고 %2을 휘둘렀다!','%2이 %3의 콧구멍에 정통으로 맞았다!','%3(은)는 악취가 너무 심해 토할 것 같다!','','냄새 나는 꽃','','','독(100%)','서쪽 숲에서 자라고 있습니다.',''];
_tr_w["kor"][Number('43') || 0]=['6','打撃','前６','3','4','900','뭔가가 불쑥 튀어 나올 것 같은 풀숲. 더 이상의 언급은 피하고 싶다.','%1는 %2을 휘둘렀다!','%2에서 야생 포로롱이 무수히 튀어나와 %3을(를) 습격한다!','','','풀숲','','','3회 공격','주변에 잔뜩 자라고 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('44') || 0]=['10','斬撃','前９','3','4','1800','잘 보면 검의 모양을 한 풀숲. 의외로 예리할 것 같다.','%1는 %2을 휘둘렀다!','잘 보면 검의 모양을 한 풀숲.%3의외로 예리한 듯 하다.','','','강해 보이는 풀숲','','','3회 공격','마을에서 서쪽으로 나가면 잔뜩 자라고 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('45') || 0]=['7','打撃','前６','3','4','1500','레어템이 튀어나올 것 같은 풀숲. 예를 들어 색이 다른, 앗 누군가 온 것 같다.','%1는 %2을 휘둘렀다!','%2에서 색이 다른 포로롱이 무수히 튀어나와 %3을(를) 습격한다!','','','황금 풀숲','','','3회 공격','마을에서 동쪽으로 나가면 잔뜩 자라고 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('46') || 0]=['7','打撃','','','5','300','가지치기가 잘 된 수풀. 부딪혀도 아플 것 같지 않다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 뚝 소리를 내며 %3에 닿았다.','','','우거진 수풀','','','','민가 근처에 있습니다.',''];
_tr_w["kor"][Number('47') || 0]=['7','打撃','','','5','600','꽃이 예쁘게 핀 멋진 꽃밭. 거리 환경 미화에 도움이 된다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 뚝 소리를 내며 %3에 닿았다.','','','꽃밭','','','','당신의 집 근처에 있습니다.',''];
_tr_w["kor"][Number('48') || 0]=['6','斬撃','','3','4','200','가을을 연출하는 낙엽 융단. 흩날리는 모습이 로맨틱하다.','%1는 %2을 손에 들고 휘둘러 보았다!','많은 %2이 %3(을)를 잘게 베었다.','','','낙엽','','','3회 공격','마을에서 동쪽으로 나가면 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('49') || 0]=['5','打撃','','','5','500','황금색으로 빛나는 억새. 바람이 흔들리는 모습이 아름답다.','%1는 %2를 술렁술렁 흔들어 댔다!','%3의 옆구리를 %2가 간지럽힌다!','%3(은)는 너무 웃어서 기절할 것만 같다!','','억새','','','혼란(100%)','마을에서 동쪽으로 나가면 잔뜩 자라고 있습니다.',''];
_tr_w["kor"][Number('50') || 0]=['6','打撃','','','6','400','이래 봬도 독버섯이다. 버섯은 겉모습으로 판단해서는 안 된다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3의 얼굴에 찰싹 부딪혔다!','%2의 파편이 %3의 입에 들어갔다!','','버섯','','','독(75%)','서쪽 숲에서 자라고 있습니다.',''];
_tr_w["kor"][Number('51') || 0]=['8','打撃','','','6','300','보기에도 기분 나쁘게 생긴 버섯. 맹독으로 가득 차 있어 맛있다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3의 얼굴에 찰싹 부딪혔다!','%2의 파편이 %3의 입에 들어갔다!','','험상궂은 버섯','','','맹독(50%)','서쪽 숲에서 자라고 있습니다.',''];
_tr_w["kor"][Number('52') || 0]=['14','打撃','直線３','','6','700','벽을 따라 자라는 식물. 채찍으로 쓸 수 있을 것 같다.','%1는 %2을 휘둘렀다!','%2는 채찍처럼 휘더니 %3에 정통으로 맞았다!','','','덩굴','','','','남쪽의 벼랑에서 자라고 있습니다.','3 매스 직선'];
_tr_w["kor"][Number('53') || 0]=['12','斬撃','直線３','','6','800','뭔가를 감아 올라가며 성장하는 식물. 길어서 원거리 공격이 가능하다.','%1는 %2을 휘둘렀다!','%2은 날카롭게 날아가 %3에 꽂혔다!','','','넝쿨','','','','남쪽의 벼랑에서 자라고 있습니다.','3 매스 직선'];
_tr_w["kor"][Number('54') || 0]=['16','斬撃','','','5','1200','사막에서 자라는 가시가 있는 식물. 가시에 찔리면 아프다.','%1는 아픔을 참고 %2을 휘둘렀다!','%2의 가시가 %3의 온몸에 꽂혔다!','','','선인장','','','','남서쪽 어딘가에서 자라고 있습니다.',''];
_tr_w["kor"][Number('55') || 0]=['15','打撃','','','4','600','이름도 품종도 모르는 이상한 풀. 성분도 비밀에 쌓여 있다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3은(는) %2에 왼손 새끼손가락을 베였다!','%2에 포함된 수수께끼의 성분이 %3에 효과가 있었다!','','이상한 풀','','','방어력을 낮춘다(100%)','서쪽 숲에서 자라고 있습니다.',''];
_tr_w["kor"][Number('56') || 0]=['13','斬撃','','','4','300','벽에서도 잘 자라는 풀. 실은 최면 성분을 포함하고 있다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3은(는) %2에 오른손 손등을 베였다!','설마 %2에 최면 성분이 포함되어 있을 줄이야!','','벽에서 자라는 풀','','','수면(25%)','철로 만들어진 동굴에 있습니다.',''];
_tr_w["kor"][Number('57') || 0]=['10','打撃','','','5','200','딱 보기에도 수상한 식물의 싹. 뿌리에는 신기한 힘이 있는 것 같다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2이 %3에 쑥쑥 박혔다!','놀랍게도 %2은 %3의 능력을 흡수했다!','','수상한 싹','','','능력 변화를 복사한다.','독의 늪으로 덮인 동굴에서 자라고 있습니다.',''];
_tr_w["kor"][Number('58') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('59') || 0]=['3','打撃','','','3','50','지면에 떨어져 있는 흙이나 돌멩이. 눈에 들어가면 아프다.','%1는 %2을 쥐고 뿌려 보았다!','%3은(는) 머리부터 %2를 뒤집어 썼다!','%3의 눈에 흙이 들어갔다!','','흙','','','명중률을 낮춘다(100%)','주변에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('60') || 0]=['6','打撃','','','4','200','흙이 작은 산처럼 쌓인 것. 머리부터 뒤집어 쓰면 정신적 충격이 크다.','%1는 %2을 휘둘렀다!','%3은(는) 머리부터 %2를 뒤집어 썼다!','%3의 눈에 흙이 들어갔다!','','흙더미','','','명중률을 낮춘다(50%)','남서쪽 어딘가에 있습니다.',''];
_tr_w["kor"][Number('61') || 0]=['14','打撃','前６','','5','400','대량의 흙이 산처럼 쌓인 것. 티끌 모아 태산이다.','%1는 %2을 휘둘렀다!','%3은(는) 머리부터 %2를 뒤집어 썼다!','%3의 눈에 흙이 들어갔다!','','커다란 흙더미','','','명중률을 낮춘다(50%)','남서쪽 어딘가에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('62') || 0]=['11','打撃','','','3','300','갈색이 특징인 바위 A. 그 밖에도 바위 B와 바위 C가 있다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','바위 A','','','','남서쪽 어딘가에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('63') || 0]=['12','打撃','','','3','400','모양을 갖춘 새까만 바위 B. 뾰족한 부분을 잡고 휘두르기 쉽다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','바위 B','','','','남서쪽 어딘가에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('64') || 0]=['15','打撃','','','3','500','울퉁불퉁한 하얀 바위 C. A~C 중 가장 강한 강도를 자랑한다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','바위 C','','','','남서쪽 어딘가에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('65') || 0]=['12','打撃','直線２','','3','700','당당하게 우뚝 서 있는 바위. 겉보기보다 의외로 몰랑하다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','커다란 바위','','','','남서쪽 어딘가에 잔뜩 떨어져 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('66') || 0]=['14','打撃','','','3','200','진한 녹색을 띈 바위. 색만 특이할 뿐 그냥 바위다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','녹색 바위','','','','독의 늪으로 덮인 동굴에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('67') || 0]=['14','打撃','直線２','','3','600','진한 녹색을 띈 커다란 바위. 휘둘러 사용한다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','커다란 녹색 바위','','','','독의 늪으로 덮인 동굴에 잔뜩 떨어져 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('68') || 0]=['11','打撃','','','1','100','삭아서 이끼로 뒤덮인 바위. 만지는 쪽도 삭아 버릴 것 같다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3의 방어도 삭아 버렸다!','','삭은 바위','','','방어력을 낮춘다(50%)','북동쪽 어딘가에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('69') || 0]=['7','打撃','','','3','50','큰 바위에서 깨져서 떨어진 조각. 깨지기 쉬워 무기로서는 적합하지 않다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','','','바위 조각','','','','남서쪽 어딘가에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('70') || 0]=['13','斬撃','','','3','700','아름다운 빛을 발하는 신비로운 크리스털. 뾰족한 부분에 찔리면 아프다.','%1는 %2을 휘둘렀다!','%2의 뾰족한 부분이 %3에 꽂혔다!','','갑자기 %2 안에서 마법이 튀어나왔다!','크리스털','','','무작위로 마법 공격 발동','마력으로 가득 찬 동굴에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('71') || 0]=['15','斬撃','直線２','','3','1200','강한 빛을 발하는 거대한 크리스털. 만지면 마력이 느껴진다.','%1는 %2을 휘둘렀다!','%2의 뾰족한 부분이 %3에 꽂혔다!','','갑자기 %2 안에서 마법이 튀어나왔다!','커다란 크리스털','','','무작위로 마법 공격 발동','마력으로 가득 찬 동굴에 잔뜩 떨어져 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('72') || 0]=['6','斬撃','','','3','300','크리스털이 깨진 것. 마력은 남아 있지 않은 것 같다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 뾰족한 부분이 %3에 꽂혔다!','','','크리스털 조각','','','','마력으로 가득 찬 동굴에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('73') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('74') || 0]=['13','打撃','前９','','3','800','하늘 높이 우뚝 선 벼랑. 이것으로 맞으면 상당히 아플 것 같다.','%1는 %2을 휘둘렀다!','%2가 기세 좋게 %3에 정통으로 맞았다!','','','벼랑','','','','마을 밖에 잔뜩 떨어져 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('75') || 0]=['9','なし','','','7','200','방 구석에 자주 생기는 거미집. 점착성이 강해서 들러붙으면 성가시다.','%1은(는) 꺼림칙하지만, %2을(를) 손에 들고 휘둘러 보았다!','%3의 얼굴에 %2이(가) 들러붙었다!','%2은(는) %3의 다리에도 들러붙었다!','','거미집','','','행동 횟수를 낮춘다(60%)','영혼이 떠도는 동굴에 있습니다.',''];
_tr_w["kor"][Number('76') || 0]=['5','なし','','','4','400','땅이 갈라진 금. 반짝이는 금이 아니다. 이것에 맞은 적도 금이 간다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3이(가) %2에 맞았다!','%3에 금이 갔다!','','금','','','방어력을 낮춘다(100%)','남서쪽 어딘가에서 자라고 있습니다.',''];
_tr_w["kor"][Number('77') || 0]=['23','打撃','','','5','600','누군가가 파놓은 깊은 함정. 적의 발을 묶어 놓을 때 편리하다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3이(가) %2에 맞았다!','%3은(는) %2에 멋지게 떨어졌다!','','함정','','','2턴 행동 불능(25%)','마을 어딘가에 있습니다.',''];
_tr_w["kor"][Number('78') || 0]=['28','打撃','前９','','5','9500','바닥이 보이지 않을 정도로 깊고 거대한 구멍. 떨어지면 절대로 올라올 수 없다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3이(가) %2에 맞았다!','%3은(는) %2에 빨려 들어가 버렸다!','','거대한 구멍','','','3턴 행동 불능(20%)','철로 만들어진 동굴에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('79') || 0]=['14','なし','前６','','5','2000','송사리와 개구리가 사는 작은 연못. 휘두르면 범람한다.','%1는 %2을 손에 들고 휘둘러 보았다!','','','연못의 물이 범람했다!','작은 연못','','','물이 천둥 내성을 낮춘다(100%)','주변에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('80') || 0]=['20','なし','前６','','5','2500','연꽃 잎이 떠 있는 경치가 좋은 연못. 연꽃이 있는 만큼 다른 연못 보다 위력이 강하다.','%1는 %2을 손에 들고 휘둘러 보았다!','','','연못의 물이 범람했다!','연꽃 연못','','','물이 천둥 내성을 낮춘다(100%)','마을에서 남쪽으로 나가면 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('81') || 0]=['16','なし','前６','','5','1400','맹독의 액체로 가득 찬 위험한 독의 늪. 흔들면 주변에 참사가 일어난다.','%1는 %2을 손에 들고 휘둘러 보았다!','','','독물이 범람했다!','독의 늪','','','독물 발생,독(80%)','남동쪽 땅에 잔뜩 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('82') || 0]=['45','なし','前９','','6','19999','지구 표면적의 70%를 차지하는 생명의 근원. 손에 쥐고 흔들면 쓰나미가 밀려온다.','%1는 %2을 손에 들고 휘둘러 보았다!','','','놀랍게도 쓰나미가 밀려왔다!','바다','','','쓰나미 발생,천둥 내성을 낮춘다(100%)','북서쪽 곶에 가면 볼 수 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('83') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('84') || 0]=['16','打撃','','','6','800','보는 사람에게 정보를 전달하는 간판. 모서리에 부딪히면 상당히 아프다.','%1는 %2을 휘둘렀다!','%2의 모서리 부분이 %3에 맞았다!','','','간판','','','','마을 어딘가에 있습니다.',''];
_tr_w["kor"][Number('85') || 0]=['24','斬撃','','','6','1000','길의 안내에 특화된 간판. 알아보기 어렵다는 불만이 많다.','%1는 %2을 휘둘렀다!','%2의 뾰족한 부분으로 %3을(를) 잘랐다!','%3은(는) 알아보기 어려운 표지판에 당황하고 있다!','','길안내 표지','','','혼란(50%)','마을 어딘가에 있습니다.',''];
_tr_w["kor"][Number('86') || 0]=['18','打撃','','','7','500','불을 피우는 데 특화된 나무. 굵은 것으로 맞으면 상당히 아프다.','%1는 %2을 휘둘렀다!','굵은 %2(으)로 %3을(를) 맞았다!','%3은(는) 장작처럼 타기 쉬운 체질로 변했다!','','장작','','','불 내성을 낮춘다(100%)','마을 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('87') || 0]=['28','打撃','','','5','1000','아줌마들이 모여 수다를 떠는 곳. 바닥에 떨어지면 나오기 어렵다.','%1는 %2을 휘둘렀다!','%2의 벽으로 %3을(를) 맞았다!','%3은(는) 우물 바닥에 떨어져 버렸다!','','우물','','','1턴 행동 불능(25%)','마을 어딘가에 있습니다.',''];
_tr_w["kor"][Number('88') || 0]=['17','打撃 / 炎','直線３','','5','2000','도시를 밝게 비추는 가로등. 길어서 무기로서 우수하다.','%1는 %2을 휘둘렀다!','%2은(는) 번쩍거리면서 %3에 정통으로 맞았다!','','','가로등','','','','남쪽 어딘가에 있습니다.','3 매스 직선'];
_tr_w["kor"][Number('89') || 0]=['16','打撃','','','5','1200','밭에서 참새를 쫓는 밭의 영웅. 표정 없는 얼굴은 사람이 봐도 무섭다.','%1는 %2을 휘둘렀다!','%2은(는) 무표정인 채로 %3에 정통으로 맞았다!','%3은(는) %2에 겁을 먹었다!','','허수아비','','','1턴 행동 불능(40%)','동쪽 어딘가에 있습니다.',''];
_tr_w["kor"][Number('90') || 0]=['15','打撃','','','5','400','문 앞에 띄엄띄엄 깔아 놓은 돌. 돌로 만들어져서 맞으면 많이 아프다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 비교적 뾰족한 부분에 %3을(를) 맞았다!','','','징검돌','','','','마을 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('91') || 0]=['14','打撃','','','4','200','아무것도 안 든 나무통. 두드리면 좋은 소리가 난다.','%1는 %2을 휘둘렀다!','%2은(는) 퉁 소리를 내며 %3에 부딪혔다.','','','빈 통','','','','마을 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('92') || 0]=['14','打撃','','','5','500','물이 들어 있는 통. 물을 뿌리는 데 쓴다.','%1는 %2을 휘둘렀다!','%2은(는) 퉁 소리를 내며 %3에 부딪혔다.','','%2에서 물이 넘쳤다!','물통','','','물이 천둥 내성을 낮춘다(100%)','마을 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('93') || 0]=['12','打撃','','','5','800','옷을 세탁하고 있는 것 같다. 휘두르면 세제를 뿌려 공격할 수 있다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','%2에서 세제가 사방에 날렸다!','세탁 세트','','','세제가 얼음 내성을 낮춘다(100%)','마을 남쪽으로 나가면 떨어져 있습니다.',''];
_tr_w["kor"][Number('94') || 0]=['7','打撃','','','4','100','평범한 나무판. 휘둘러 때리는 것 외엔 사용 방법이 떠오르지 않는다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','나무판','','','','남서쪽 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('95') || 0]=['13','打撃','直線２','','3','700','건축물을 떠받치고 있다. 휘두르기에 딱 좋은 모양이다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','기둥','','','','북동쪽 어딘가에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('96') || 0]=['9','斬撃','','','3','600','부러져 버린 기둥. 부러진 단면은 꽤 까칠하다.','%1는 %2을 휘둘렀다!','%2이 부러져 까칠한 단면이 %3에 꽂혔다!','','','부러진 기둥','','','','북동쪽 어딘가에 있습니다.',''];
_tr_w["kor"][Number('97') || 0]=['13','打撃','直線２','','3','400','삭아서 이끼가 낀 기둥. 꽤 오래 전에 만들어졌다는 것을 알 수 있다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3의 방어도 삭아 버렸다!','','삭은 기둥','','','방어력을 낮춘다(50%)','북동쪽 어딘가에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('98') || 0]=['9','斬撃','','','3','200','삭은 데다 부러지기까지 한 기둥. 기둥의 역할은 다한 모양이다.','%1는 %2을 휘둘렀다!','%2이 부러져 까칠한 단면이 %3에 꽂혔다!','%3의 방어도 삭아 버렸다!','','삭아서 부러진 기둥','','','방어력을 낮춘다(50%)','북동쪽 어딘가에 있습니다.',''];
_tr_w["kor"][Number('99') || 0]=['12','打撃','','','6','2000','중요한 내용이 쓰여 있는 듯한 비석. 하지만 문자 해석이 매우 곤란하다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3은(는) 비석의 문자 해독에 필사적이다!','','비석','','','혼란(70%)','동굴에 근처에 있습니다.',''];
_tr_w["kor"][Number('100') || 0]=['13','斬撃','','','5','900','죽은 자의 혼을 기리기 위한 간이 무덤. 함부로 다루면 벌을 받는다.','%1는 %2을 휘둘렀다!','%2의 끝이 %3에 박혔다!','%3에게 묘를 함부로 한 천벌이 떨어졌다….','','나무 무덤','','','즉사(5%)','마을 어딘가에 있습니다.',''];
_tr_w["kor"][Number('101') || 0]=['17','打撃','','','5','2400','죽은 자의 혼을 기리기 위한 돌 무덤. 함부로 다루면 벌을 받는다.','%1는 %2을 휘둘렀다!','%2이 격렬한 기세로 %3에 직격했다!','%3에게 묘를 함부로 한 천벌이 떨어졌다….','','돌 무덤','','','즉사(10%)','영혼이 떠도는 동굴에 있습니다.',''];
_tr_w["kor"][Number('102') || 0]=['12','打撃','直線２','3','6','3200','다양한 음료수를 살 수 있는 편리한 기계. 왠지 「따뜻한 음료」 밖에 없다.','%1는 %2을 휘둘렀다!','따뜻한 음료가 %3의 머리 위로 쏟아졌다!','','','자판기','','','3회 공격','-','2 매스 직선'];
_tr_w["kor"][Number('103') || 0]=['13','打撃','','','6','800','분리 수거를 위한 쓰레기통. 휘둘러서 쓰레기를 어지럽혀서는 안 된다.','%1는 %2을 휘둘렀다!','%2은 쓰레기를 흩뿌리며 %3에 정통으로 맞았다!','%3은(는) 쓰레기가 풍기는 악취를 그대로 맡았다!','','쓰레기통','','','독(100%)','근처 편의점 앞에 있습니다.',''];
_tr_w["kor"][Number('104') || 0]=['14','打撃','','','7','1100','돌로 튼튼히 쌓아 올린 부뚜막. 착화제를 사용해 불이 쉽게 붙게 하였다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3에 착화제가 묻었다!','','돌 부뚜막','','','불 내성을 낮춘다(100%)','남쪽 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('105') || 0]=['25','炎','','','6','2200','활활 타오르는 불꽃. 주울 때 화상 좀 입었다.','%1는 화상의 고통을 견디며 %2을 휘둘렀다!','%3은(는) 모닥불처럼 활활 타올랐다!','','','불꽃','','','','남쪽 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('106') || 0]=['30','打撃','直線４','','7','8500','공중에 떠 올라 가끔 회전하는 신기한 철판. 이 회전을 이용하면 무기로 쓸 수 있을 것 같다.','%1는 %2을 휘둘렀다!','%2은 회전하면서 %3에 정통으로 맞았다!','','','신기한 철판','','','','철로 만들어진 동굴에 있습니다.','4 매스 직선'];
_tr_w["kor"][Number('107') || 0]=['25','打撃','','','7','4000','조작할 때마다 뭔가 회전하는 듯한 신기한 스위치. 의외로 딱딱하다.','%1는 %2을 휘둘렀다!','%2은(는) ON/OFF를 반복하며 %3에 정통으로 맞았다!','','','신기한 스위치','','','','철로 만들어진 동굴에 있습니다.',''];
_tr_w["kor"][Number('108') || 0]=['30','打撃','直線２','','7','3800','지날 때마다 방향이 바뀌는 일방통행의 신기한 다리. 무기로도 쓸 수 있을 것 같다.','%1는 %2을 휘둘렀다!','%2는 방향을 바꾸면서 %3에 정통으로 맞았다!','','','신기한 다리','','','','독의 늪으로 덮인 동굴에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('109') || 0]=['28','打撃','前９','','7','6000','자극을 주면 빛나는 신기한 패널. 이걸 이용한 퍼즐은 꽤 어렵다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3은(는) 정답 루트를 알지 못한다!','','신기한 패널','','','혼란(25%)','마력으로 가득 찬 동굴에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('110') || 0]=['32','打撃 / 炎','','','7','5500','신비한 분위기를 자아내는 초롱불. 일단 불이 붙으면 영원히 안 꺼진다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','왠지 %2는 저절로 불이 붙었다!','신기한 초롱불','','','화염 발생','영혼이 떠도는 동굴에 있습니다.',''];
_tr_w["kor"][Number('111') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('112') || 0]=['12','打撃','','','1','600','여느 가정에나 있는 평범한 항아리. 부딪히면 회심의 일격을 가하고 망가진다.','%1는 %2을 휘둘렀다!','%2이(가) %3에 세차게 부딪혔다!','','','항아리','','','반드시 회심의 일격','주변의 집 안에 있습니다.',''];
_tr_w["kor"][Number('113') || 0]=['12','打撃','','','1','1200','뚜껑 때문에 내용물이 보이지 않는 수상한 항아리. 안에는 도대체 무엇이…?','%1는 %2을 휘둘렀다!','%2이(가) %3에 세차게 부딪혔다!','항아리에서 튀어나온 「말똥」이 %3의 입에 들어갔다!','','뚜껑 있는 항아리','','','반드시 회심의 일격,독(100%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('114') || 0]=['15','打撃','','','3','700','어느 곳에나 있는 흔한 통. 무기로 쓸 때는 적에게 던진다.','%1는 %2을 휘둘렀다!','%2이(가) %3에 세차게 부딪혔다!','','','통','','','','마을 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('115') || 0]=['11','斬撃','','','6','600','작은 사각형 테이블. 잡동사니를 잠시 올려 둘 때 유용하다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','','','미니 테이블','','','','어느 오두막 안에 있습니다.',''];
_tr_w["kor"][Number('116') || 0]=['8','打撃','','','6','600','작고 귀여운 둥근 테이블. 모서리가 없어 위력이 없을 것 같다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','둥근 테이블','','','','어느 오두막 안에 있습니다.',''];
_tr_w["kor"][Number('117') || 0]=['13','斬撃','直線２','','6','900','적당한 크기의 사각 테이블. 모서리로 힘껏 찌르면 위협적이다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','','','중간 크기 테이블','','','','마을의 어느 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('118') || 0]=['14','斬撃','前６','','6','2000','여러 명이 사용하기에 편리한 테이블. 이 정도는 무기로 쓸 줄 알아야 한다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','','','큰 테이블','','','','주변의 집 안에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('119') || 0]=['13','斬撃','直線３','','6','1400','미묘한 길이의 미묘한 테이블. 모서리에 꽂히면 미묘하게 아프다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','','','약간 긴 테이블','','','','마을의 어느 집 안에 있습니다.','3 매스 직선'];
_tr_w["kor"][Number('120') || 0]=['15','斬撃','直線４','','6','2500','상당한 길이를 자랑하는 테이블. 공격 범위는 무기로서 최상급.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','','','긴 테이블','','','','마을의 어느 집 안에 있습니다.','4 매스 직선'];
_tr_w["kor"][Number('121') || 0]=['12','斬撃','','','6','1100','꽃병과 세트를 이루는 테이블. 보기만 해도 마음이 편해진다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','%3은(는) 꽃병에 마음이 동화되어 전의를 상실했다!','','꽃병 테이블','','','공격력을 낮춘다(60%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('122') || 0]=['15','斬撃','','','6','1200','거울과 세트를 이루는 테이블. 그 거울은 진실만을 비춘다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','거울에 반사된 빛이 %3을(를) 비췄다!','','화장대','','','능력 강화를 초기화한다','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('123') || 0]=['17','斬撃','直線２','','6','3600','딱 보기에도 교회에 있을 법한 테이블. 성서에 따르면 신기한 힘이 있다고 한다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','%3에 성스러운 힘이 작용했다!','','교회 테이블','','','불 내성을 낮춘다(50%),얼음 내성을 낮춘다(50%),천둥 내성을 낮춘다(50%)','어느 교회 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('124') || 0]=['17','斬撃','','','6','4000','수상한 수정과 세트를 이루는 테이블. 수정은 수상하지만, 마력은 느껴진다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','','갑자기 수정에서 마법이 튀어나왔다!','수정 테이블','','','무작위로 마법 공격 발동','신기하게 생긴 집 안에 있습니다.',''];
_tr_w["kor"][Number('125') || 0]=['8','打撃','','','6','300','싸 보이지만 앉기 편한 둥근 의자. 무기로 쓰기엔 좀 그럴 것 같다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3(은)는 %2에 앉아서 휴식을 취했다!','','둥근 의자','','','1턴 행동 불능(50%)','마을 어딘가의 집 안에 잔뜩 있습니다.',''];
_tr_w["kor"][Number('126') || 0]=['10','斬撃','','','6','300','작은 사각형 의자. 모서리가 있어 둥근 의자보다 강하다.','%1는 %2을 휘둘렀다!','%2의 모서리가 %3에 박혔다!','%3(은)는 %2에 앉아서 휴식을 취했다!','','사각 의자','','','1턴 행동 불능(50%)','마을 어딘가의 집 안에 잔뜩 있습니다.',''];
_tr_w["kor"][Number('127') || 0]=['12','斬撃','直線２','','6','1500','고급스럽고 느낌이 좋은 의자. 앉았을 때 느낌도 최고다!','%1는 %2을 휘둘렀다!','%2의 등받이 모서리가 %3에 박혔다!','%3은(는) %2에 앉아서 편안한 휴식을 취했다!','','느낌이 좋은 의자','','','1턴 행동 불능(80%)','어느 오두막 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('128') || 0]=['8','打撃','直線２','3','6','1000','다양한 책이 정돈된 책장. 책을 적 위로 떨어뜨려 공격한다.','%1는 %2을 휘둘렀다!','수많은 책이 %3 머리 위로 쏟아졌다!','%3은(는) 책을 많이 읽고 똑똑해졌다!','','책장','','','3회 공격,빈약(30%)','주변의 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('129') || 0]=['30','斬撃','直線２','','6','1500','옷을 보관하는 가구. 왼발 새끼발가락을 부딪히면 죽을 만큼 아프다.','%1는 %2을 휘둘렀다!','%3의 왼발 새끼발가락이 %2의 모서리에 부딪혔다!','%3(은)는 왼발 새끼발가락을 부여잡고 괴로워하고 있다!','','옷장','','','행동 횟수를 낮춘다(30%)','마을의 어느 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('130') || 0]=['9','打撃','直線２','3','6','1200','옷을 걸어 두는 가구. 실은 문을 열면 산사태가 난다.','%1는 %2을 휘둘렀다!','수많은 옷이 %3 머리 위로 쏟아졌다!','%3(은)는 옷에 파묻혀 움직임이 둔해졌다!','','장롱','','','3회 공격,회피를 낮춘다(100%)','마을의 어느 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('131') || 0]=['7','打撃','直線２','3','6','1400','선반에 빵은 왜 들어 있을까? 바게트는 딱딱해서 무기가 된다.','%1는 %2을 휘둘렀다!','수많은 바게트가 %3 머리 위로 쏟아졌다!','%3(은)는 게걸스럽게 빵을 먹기 시작했다!','','빵이 들어 있는 선반','','','3회 공격,1턴 행동 불능(50%)','어느 오두막 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('132') || 0]=['8','打撃','直線２','3','6','900','정체 불명의 약이 든 선반. 아무튼 휘둘러 사용해 보자.','%1는 %2을 휘둘렀다!','수많은 정체 불명의 물체가 %3 머리 위로 쏟아졌다!','%3(은)는 정체 불명의 물체에 겁을 먹었다!','','정체 불명의 선반','','','3회 공격,혼란(50%)','어느 오두막 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('133') || 0]=['12','打撃','','','6','600','벽에 달아서 책을 놓는 선반. 여기에 달아 놓으면 책은 어떻게 꺼내나?','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','%3(은)는 책을 읽고 똑똑해졌다!','','벽걸이형 책꽂이','','','빈약(30%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('134') || 0]=['13','打撃','','','6','800','각종 조미료를 올려 놓는 편리한 선반. 유통기한이 지난 조미료에 주의할 것.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','이 조미료의 유통기한은 2년 전이었다!','','조미료 선반','','','독(30%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('135') || 0]=['12','打撃','','','6','700','예쁜 꽃과 꽃병 세트. 아름다운 꽃은 아름다운 꽃병에 꽂아야 한다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','%3은(는) 꽃에 마음이 동화되어 전의를 상실했다!','','꽃병','','','공격력을 낮춘다(50%)','어느 교회 안에 있습니다.',''];
_tr_w["kor"][Number('136') || 0]=['14','打撃','','','5','400','잡동사니를 넣어 두는 나무 상자. 그냥 들고 때리면 된다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','','','나무 상자','','','','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('137') || 0]=['6','打撃','前６','5','3','1800','나무 상자를 잔뜩 쌓아 놓은 탑. 적의 머리를 노리고 무너뜨리면 된다.','%1는 %2을 휘둘렀다!','수많은 나무 상자가 %3 머리 위로 쏟아졌다!','','','나무 상자 탑','','','5회 공격','어느 오두막 안에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('138') || 0]=['8','打撃','','','3','300','낡아빠진 나무 상자. 강도가 약해져 쓸모가 없다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','','','낡은 나무 상자','','','','어느 오두막 안에 있습니다.',''];
_tr_w["kor"][Number('139') || 0]=['18','打撃','直線２','','5','700','어른 키만큼 큰 나무 상자. 휘두르면 굉장한 위력을 발휘한다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','큰 나무 상자','','','','마을 어딘가에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('140') || 0]=['12','打撃','','','5','500','쌀이 가득 차 있는 쌀가마. 농부에게 감사하며 휘두르자.','%1는 %2을 손에 들고 휘둘러 보았다!','가득 들어 있는 %2이 %3에 정통으로 맞았다!','','','쌀가마','','','','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('141') || 0]=['9','打撃','直線２','3','5','1400','쌀이 가득 담긴 쌀가마가 쌓여있다. 적의 머리 위로 쏟아지게 한다.','%1는 %2을 휘둘렀다!','수많은 쌀가마가 %3의 머리 위로 쏟아졌다!','','','쌀가마 더미','','','3회 공격','마을의 어느 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('142') || 0]=['5','打撃','','5','6','1100','맛있게 생긴 사과가 많긴 하지만, 과연…?','%1는 %2을 손에 들고 휘둘러 보았다!','%2가 %3의 머리 위로 쏟아졌다!','설마 모두 독사과였다니!','','수많은 사과','','','5회 공격,맹독(60%)','신기하게 생긴 집 안에 있습니다.',''];
_tr_w["kor"][Number('143') || 0]=['16','打撃','直線２','','5','1400','깨끗하게 정돈된 넓은 부엌. 휘둘러 보니 공격 범위가 상당히 넓다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','부엌','','','','마을의 어느 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('144') || 0]=['16','打撃','','','5','900','기름때 하나 없는 가스레인지 설치대. 불이 나오므로 무기로서 쓸만하다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','강한 불로 갈색이 될 때까지 가열하자!','가스레인지 설치대','','','화염 발생','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('145') || 0]=['18','斬撃','','','6','900','약간 더러운 환풍기. 회전하는 날개는 무기가 될 것 같다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 %2의 회전에 휘말렸다!','','','환풍기','','','','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('146') || 0]=['23','斬撃','','','5','800','뭔가를 자르다 만 도마와 부엌칼. 부엌칼은 정말 예리하다.','%1는 %2을 휘둘렀다!','%3(을)를 올려 놓고 잘게 썰었다!','','','칼도마 세트','','','물리공격 내성을 낮춘다(100%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('147') || 0]=['11','打撃','','3','6','700','프라이팬, 국자 등 다양하다. 물론 무기로도 사용할 수 있다.','%1는 %2을 휘둘렀다!','%3(을)를 다양한 조리기구로 때렸다!','','','조리 기구 세트','','','3회 공격','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('148') || 0]=['20','打撃 / 氷','直線２','','6','1900','남은 음식으로 가득 찬 냉장고. 냉기로 적을 얼리는 전술이 가능하다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','문이 열려 냉기가 새고 있다!','냉장고','','','냉기 발생','마을의 어느 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('149') || 0]=['12','打撃','','','6','700','생선 한 마리를 통째로 구운 일품 요리. 가시가 많다.','%1는 %2을 휘둘러 보았다! ※음식은 소중히 여깁시다!','%2은 %3에 세게 부딪혔다!','%3의 목에 생선 가시가 걸렸다!','','생선 구이','','','마법을 봉인한다(50%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('150') || 0]=['14','打撃','','','6','600','심플하고 맛있는 볶음 요리. 혀에 화상을 입을 정도로 뜨겁다.','%1는 %2을 휘둘러 보았다! ※음식은 소중히 여깁시다!','뜨끈뜨끈한 채소가 %3(을) 공격한다!','','','채소 볶음 세트','','','','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('151') || 0]=['12','打撃','','','6','750','밥은 무한 리필인 정식 메뉴. 하지만 돼지고기가 덜 익은 것 같다.','%1는 %2을 휘둘러 보았다! ※음식은 소중히 여깁시다!','%2은 %3에 세게 부딪혔다!','%2은 돼지고기가 덜 익었다. %3(은)는 배탈이 났다!','','돼지고기 생강구이 정식','','','독(30%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('152') || 0]=['7','打撃','','','6','320','마시면 오후에도 힘이 나는 한잔의 커피. 카페인이 졸음을 날려준다.','%1는 %2으로 건배했다!','%3「건배~!!」','%3의 눈이 번쩍 뜨였다!','','식후의 커피','','','수면 내성을 높인다(100%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('153') || 0]=['12','打撃','','','6','3000','농후한 맛이 특징인 풀바디 와인. 향기만 맡아도 기분이 좋아진다.','%1는 %2으로 건배했다!','%3「건배~!!」','%3(은)는 와인의 향기에 취해 버렸다.','','고급 와인','','','마법 공격을 봉인한다(50%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('154') || 0]=['21','打撃','','','6','50','요리 같기도 하고 아닌 것 같기도 하고…. 어떻게 이런 걸 손님에게 팔지?','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3의 입안으로 빨려 들어가 버렸다!','%3의 모습이 너무 이상하다!','','요리 같은 것','','','독(75%),마법 봉인(50%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('155') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('156') || 0]=['14','打撃','直線２','','6','2400','푹 잠들 수 있는 푹신푹신한 침대. 빨려 들어가 듯이 적을 잠재운다.','%1는 %2을 휘둘렀다!','%2는 퉁 소리를 내며 %3에 닿았다.','%3(은)는 %2에 빨려 들어가 버렸다!','','침대','','','수면(55%)','마을의 어느 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('157') || 0]=['18','打撃','前６','','6','7000','두 명이 함께 잘 수 있는 더블 침대. 숙면을 위한 최고의 침대.','%1는 %2을 휘둘렀다!','%2는 퉁 소리를 내며 %3에 닿았다.','%3(은)는 %2에 빨려 들어가 버렸다!','','더블 침대','','','수면(90%)','마을의 어느 집 안에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('158') || 0]=['7','打撃','直線２','','4','300','잠자리가 불편할 것 같은 침대. 다음날 아침 모든 관절이 쑤신다.','%1는 %2을 휘둘렀다!','%2은 부스럭 소리를 내며 %3에 닿았다.','%3(은)는 오늘 밤만 %2로 견디기로 했다.','','낡은 이불','','','수면(25%)','어느 오두막 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('159') || 0]=['18','打撃','直線２','','6','1400','호화로운 분위기의 난로. 위쪽의 가는 부분을 잡고 휘두른다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','왠지 %2는 저절로 불이 붙었다!','난로','','','화염 발생','마을의 어느 집 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('160') || 0]=['11','斬撃','','','5','400','누군가가 쓴 일기. 읽으면 자신의 흑역사가 되살아난다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 %2에 왼손 엄지손가락을 베였다!','%3(은)는 흑역사를 떠올리고 말았다….','','일기','','','혼란(50%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('161') || 0]=['5','斬撃','','','4','200','뭔가 쓰여 있는 종이. 종이에 손을 베이면 많이 아프다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 %2에 왼손 손등을 베였다!','','','정체 불명의 종이','','','','어느 오두막 안에 있습니다.',''];
_tr_w["kor"][Number('162') || 0]=['13','打撃','','','4','500','뭔가가 들어 있는 봉투. 내용물은 모른다. 그냥 때리는 용도로만 쓴다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2는 %3에 정통으로 맞았다! 내용물이 뭔지는 모른다.','','','정체 불명의 봉투','','','','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('163') || 0]=['27','打撃','前６','','6','3700','관리가 잘 된 엔틱 피아노. 멜로디를 연주하면서 적에게 휘두르자.','%1는 %2을 휘둘렀다!','%2의 건반 부분이 %3에 맞아 불협화음이 흘러나왔다!','%3(은)는 불협화음을 듣고 기분이 나빠졌다!','','피아노','','','능력 강화를 초기화한다','마을의 어느 집 안에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('164') || 0]=['15','打撃','','','6','1000','예쁘게 포장된 상자. 안에서 마력이 느껴진다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','%2 안에서 마법이 튀어나왔다!','선물 상자','','','무작위로 마법 공격 발동','신기하게 생긴 집 안에 있습니다.',''];
_tr_w["kor"][Number('165') || 0]=['12','打撃','','','5','1500','마녀의 집에 있었던 옷. 주울 때 약간의 죄책감을 느꼈다.','%1(은)는 %2을 휘둘러 보았다! 좋은 냄새가 난다!','%2으로 %3의 얼굴을 때렸다.','','','마녀의 옷','','','','신기하게 생긴 집 안에 있습니다.',''];
_tr_w["kor"][Number('166') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('167') || 0]=['11','打撃','前６','3','6','1400','편의점의 일용품이 놓여 있는 진열대. 무기로 주웠으므로 도둑질은 아니다.','%1는 %2을 휘둘렀다!','다양한 상품이 %3의 머리 위로 쏟아졌다!','맙소사 모두 불량품이였다!','','편의점 진열대','','','3회 공격,혼란(50%)','근처 편의점 안에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('168') || 0]=['10','打撃 / 氷','前６','3','6','2200','다양한 냉동식품이 구비된 코너. 이 냉기는 무기로 쓸 수 있을 것 같다.','%1는 %2을 휘둘렀다!','냉동식품이 %3의 머리 위로 쏟아졌다!','','냉기가 새기 시작했다!','냉동식품 코너','','','3회 공격,냉기 발생','근처 편의점 안에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('169') || 0]=['12','打撃','前６','3','6','1800','다양한 도시락이 진열된 코너. 모두 \'반값\' 스티커가 붙어 있다.','%1는 %2을 휘둘렀다!','다양한 도시락이 %3의 머리 위로 쏟아졌다!','맙소사 모든 도시락의 유통기한이 지났다!','','도시락 코너','','','3회 공격,독(50%)','근처 편의점 안에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('170') || 0]=['13','打撃','','','7','3000','우리 집의 소중한 전화기. 보이스 피싱을 조심하자.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3(은)는 보이스 피싱에 걸려들고 말았다!','','전화','','','혼란(100%)','당신의 집 안에 있습니다.',''];
_tr_w["kor"][Number('171') || 0]=['9','打撃','前９','','6','400','상당히 낡은 융단. 약간 이상한 냄새도 난다.','%1는 %2을 휘둘렀다!','%3(은)는 %2에 걸려 넘어져 바닥에 얼굴을 찧었다!','%3(은)는 발이 꼬여 움직임이 둔해졌다!','','낡은 융단','','','회피율을 낮춘다(75%)','어느 오두막 안에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('172') || 0]=['14','打撃','前９','','6','800','파랗고 예쁜 융단. 감촉이 좋다.','%1는 %2을 휘둘렀다!','%3(은)는 %2에 걸려 넘어져 바닥에 얼굴을 찧었다!','%3(은)는 발이 꼬여 움직임이 둔해졌다!','','파란 융단','','','회피율을 낮춘다(50%)','마을의 어느 집 안에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('173') || 0]=['18','打撃','前９','','6','2500','새빨갛고 호화로운 융단. 최고급 원단으로 만들어졌다.','%1는 %2을 휘둘렀다!','%3(은)는 %2에 걸려 넘어져 바닥에 얼굴을 찧었다!','%3(은)는 발이 꼬여 움직임이 둔해졌다!','','호화로운 융단','','','회피율을 낮춘다(25%)','어느 교회 안에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('174') || 0]=['13','打撃','前９','','6','1100','신기한 분위기를 자아내는 융단. 하지만 원단은 의외로 싸구려 같다.','%1는 %2을 휘둘렀다!','%3(은)는 %2에 걸려 넘어져 바닥에 얼굴을 찧었다!','%3(은)는 발이 꼬여 움직임이 둔해졌다!','','수상한 융단','','','회피율을 낮춘다(100%)','신기하게 생긴 집 안에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('175') || 0]=['16','打撃','','','5','800','정확한 시각을 알려 주는 시계. 이걸로 때려 보는 것도 재미있을 것 같다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2는 째깍째깍 소리를 내며 %3에 정통으로 맞았다!','맙소사 %3 주변의 시공간이 변화했다!','','시계','','','행동 횟수를 줄인다(50%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('176') || 0]=['21','斬撃','','','7','1200','벽에 걸어 장식한 모조 검. 하지만 의외로 잘 들 것 같다.','%1는 %2을 휘둘렀다!','%2는 간단히 %3(을)를 베었다!','','','벽걸이 장식용 검','','','','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('177') || 0]=['17','打撃','','','7','900','벽에 걸어 장식한 모조 방패. 모조 방패지만 꽤 단단하다.','%1는 %2을 휘둘렀다!','%2가 기세 좋게 %3에 정통으로 맞았다!','','','벽걸이 장식용 방패','','','방어력을 낮춘다(50%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('178') || 0]=['24','斬撃','','','7','1500','벽에 걸어 장식한 모조 도끼. 진짜 도끼처럼 묵직하다.','%1는 %2을 휘둘렀다!','%2는 간단히 %3(을)를 베었다!','','','벽걸이 장식용 도끼','','','','어느 오두막 안에 있습니다.',''];
_tr_w["kor"][Number('179') || 0]=['16','斬撃','前３','','6','600','손으로 그린 지도. 모험의 필수품이다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2가 %3에 맞았다!','%3(은)는 지도를 잘 읽지 못했다!','','지도','','','혼란(75%)','어느 오두막 안에 있습니다.','3 매스 앞으로'];
_tr_w["kor"][Number('180') || 0]=['14','斬撃','','','5','700','아름다운 꽃 그림. 액자는 딱딱하므로 무기가 된다.','%1는 %2을 손에 들고 휘둘러 보았다!','액자가 %3에 정통으로 맞았다!','%3(은)는 아름다운 꽃에 치유되어 전의를 상실했다!','','꽃 그림','','','공격력을 낮춘다(60%)','신기하게 생긴 집 안에 있습니다.',''];
_tr_w["kor"][Number('181') || 0]=['15','斬撃','','','5','800','모르는 얼굴이다. 넌 누구냐? 예술은 참 어렵다.','%1는 %2을 손에 들고 휘둘러 보았다!','액자가 %3에 정통으로 맞았다!','%3(은)는 예술이란 무엇인지 진지하게 고민하기 시작했다!','','사람 그림','','','혼란(40%)','신기하게 생긴 집 안에 있습니다.',''];
_tr_w["kor"][Number('182') || 0]=['13','斬撃','直線２','','5','1000','어떤 나라의 국기로 보이는 빨간 깃발. 위력은 기대되지 않는다.','%1는 %2을 휘둘렀다!','펄럭거리는 %2이 %3에 맞았다!','','','빨간 깃발','','','','영혼이 떠도는 동굴에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('183') || 0]=['7','斬撃','直線２','','3','300','낡아빠진 깃발. 쓰레기일 뿐이다.','%1는 %2을 휘둘렀다!','펄럭거리는 %2이 %3에 맞았다!','','','낡은 깃발','','','','영혼이 떠도는 동굴에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('184') || 0]=['16','斬撃','前６','','5','900','취향이 의심스러운 새빨간 커튼. 차광성은 높을 것 같다.','%1는 %2을 손에 들고 휘둘러 보았다!','나풀거리는 %2이 %3에 맞았다!','%3의 시야가 %2에 가려졌다!','','새빨간 커튼','','','명중률을 낮춘다(90%)','신기하게 생긴 집 안에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('185') || 0]=['12','打撃','','','5','700','깨끗하게 잘 닦인 창문. 이중창이라 튼튼하다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','','','창문','','','','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('186') || 0]=['12','打撃','','','5','900','녹색 커튼이 달려 있는 창문. 커튼으로 적의 시야를 가릴 수 있다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','%3의 시야가 커튼에 가려졌다!','','녹색 커튼 창문','','','명중률을 낮춘다(100%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('187') || 0]=['13','打撃','','','5','1000','빨간 커튼이 달린 조금 큰 창문. 망치로 쳐도 깨지지 않을 정도로 튼튼하다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','%3의 시야가 커튼에 가려졌다!','','빨간 커튼 창문','','','명중률을 낮춘다(100%)','마을의 어느 집 안에 있습니다.',''];
_tr_w["kor"][Number('188') || 0]=['15','打撃','','','5','1600','예쁜 모양이 그려진 창문. 교회에 가면 볼 수 있다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','','','스테인드글라스','','','','어느 교회 안에 있습니다.',''];
_tr_w["kor"][Number('189') || 0]=['20','打撃','','','4','800','보안 차원에서 약간 불안한 문. 무기로는 쓸만하다.','%1는 %2을 철컥철컥 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','문','','','','주변의 집 입구에 있습니다.',''];
_tr_w["kor"][Number('190') || 0]=['16','打撃','前６','','6','1200','튼튼한 철창으로 만들어진 감옥. 휘두르면 강력한 무기가 된다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3(은)는 감옥에 갇힌 느낌이 들었다!','','감옥','','','1턴 행동 불능(30%)','철로 만들어진 동굴에 있습니다.','6 매스 앞으로'];
_tr_w["kor"][Number('191') || 0]=['17','打撃','直線２','','6','900','감옥 열쇠로 열 수 있는 튼튼한 문. 강도는 검증되었다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3(은)는 감옥에 갇힌 느낌이 들었다!','','감옥의 문','','','1턴 행동 불능(20%)','철로 만들어진 동굴에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('192') || 0]=['28','打撃','直線２','','7','6000','늠름한 얼굴을 한 전사의 조각상. 손에 든 검은 무기로 쓸만해 보인다.','%1는 %2을 휘둘렀다!','%2의 검이 %3(을)를 베었다!','','','전사의 조각상','','','','동쪽 어딘가에 떨어져 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('193') || 0]=['24','打撃','直線２','','7','4500','항상 오른쪽을 바라보는 신기한 여신상. 성스러운 힘으로 튼튼하게 만들어졌다.','%1는 %2을 휘둘렀다!','%2은 %3에게 박치기를 했다!','%3에 성스러운 힘이 작용했다!','','오른쪽을 바라보는 여신상','','','빈약(15%)','어느 교회 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('194') || 0]=['24','打撃','直線２','','7','4500','항상 왼쪽을 바라보는 신기한 여신상. 박치기에 사용하면 강할 것 같다.','%1는 %2을 휘둘렀다!','%2은 %3에게 박치기를 했다!','%3에 성스러운 힘이 작용했다!','','왼쪽을 바라보는 여신상','','','마법을 봉인한다(50%)','어느 교회 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('195') || 0]=['40','斬撃 / 炎','前３','','7','18000','흉폭한 드래곤을 본떠 만든 박력 있는 조각상. 드래곤의 영혼이 봉인되어 있다.','%1는 %2을 휘둘렀다!','%2의 송곳니가 %3에 박혔다!','','맙소사 %2이 불을 뿜었다!','드래곤 조각상','','','화염 발생','영혼이 떠도는 동굴에 있습니다.','3 매스 앞으로'];
_tr_w["kor"][Number('196') || 0]=['18','打撃','','','6','3000','레어 아이템이 들어 있을 법한 보물상자. 무기로 사용하려면 던지는 수밖에 없어 보인다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','','','보물상자','','','','주변에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('197') || 0]=['19','打撃','直線２','','4','1800','눈이 부실 정도로 빛나는 황금 기둥. 맨눈으로 보면 눈에 나쁠 것 같다.','%1는 %2을 휘둘렀다!','%2은 빛을 내면서 %3에 정통으로 맞았다!','%3(은)는 찬란한 황금을 맨눈으로 쳐다보고 말았다!','','황금 기둥','','','명중률을 낮춘다(50%)','황금색으로 빛나는 동굴 안에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('198') || 0]=['15','斬撃','','','4','900','부러졌어도 빛을 내는 황금 기둥. 금이라서 강도면에서는 문제 없다.','%1는 %2을 휘둘렀다!','%2이 부러져 까칠한 단면이 %3에 꽂혔다!','%3(은)는 찬란한 황금을 맨눈으로 쳐다보고 말았다!','','부러진 황금 기둥','','','명중률을 낮춘다(30%)','황금색으로 빛나는 동굴 안에 있습니다.',''];
_tr_w["kor"][Number('199') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('200') || 0]=['20','打撃','','','7','500','빨갛고 사랑스러운 곰 인형. 곰 인형… 맞지?','%1는 %2을 휘둘렀다!','%2은 %3에게 박치기를 했다!','','','빨간 곰','Catherine','','','마을 어떤 집 안에 앉아 있습니다.',''];
_tr_w["kor"][Number('201') || 0]=['20','打撃','','','7','500','파랗고 쿨하게 생긴 곰 인형. 곰 인형… 맞지?','%1는 %2을 휘둘렀다!','%2은 %3에게 박치기를 했다!','','','파란 곰','James','','','남쪽 곶에 앉아 있습니다.',''];
_tr_w["kor"][Number('202') || 0]=['20','打撃','','','7','500','새카맣게 생긴 곰 인형. 곰 인형… 맞지?','%1는 %2을 휘둘렀다!','%2은 %3에게 박치기를 했다!','','','까만 곰','Tom','','','동쪽 어딘가에 앉아 있습니다.',''];
_tr_w["kor"][Number('203') || 0]=['20','打撃','','','7','500','귀엽게 생긴 갈색 곰 인형. 곰 인형… 맞지?','%1는 %2을 휘둘렀다!','%2은 %3에게 박치기를 했다!','','','갈색 곰','Brown','','','북서쪽 어딘가에 앉아 있습니다.',''];
_tr_w["kor"][Number('204') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('205') || 0]=['30','打撃','前９','','7','2500','오랫동안 살아 왔던 우리 집의 오른쪽 절반. 휭휭 휘두르면 무기로 쓸 수 있다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','자택 우측','','','','당신 집의 오른쪽 절반을 이루고 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('206') || 0]=['30','打撃','前９','','7','3200','우리 집의 왼쪽 절반인 무기 상점 공간. 휭휭 휘두르면 무기로 쓸 수 있다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','자택 좌측','','','','당신 집의 왼쪽 절반을 이루고 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('207') || 0]=['29','打撃','前９','','7','2200','전혀 모르는 사람이 사는 집. 무기로 사용하면 집주인한테 혼날 것 같다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','민가','','','','마을 몇 군데에 떨어져 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('208') || 0]=['27','打撃','前９','','7','1900','나무꾼이 사는 낡은 집. 낡고 지저분하지만 무기가 될 듯하다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','나무꾼의 집','','','','남쪽 어딘가에 떨어져 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('209') || 0]=['25','打撃','前９','','7','2800','마녀가 살고 있을 법한 수상한 집. 지붕 색깔이 화려하다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','마녀의 집','','','','서쪽 숲에 숨겨져 있다.','9 매스 앞으로'];
_tr_w["kor"][Number('210') || 0]=['32','打撃','前９','','7','3300','바닷가에 있는 신성한 교회. 신에게 감사하면서 휘두르자.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','%3에 성스러운 힘이 작용했다!','','교회','','','능력 강화를 초기화한다','북동쪽 해안가에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('211') || 0]=['31','打撃','前９','','7','3000','뭐든 살 수 있는 편리한 편의점. 편리하니까 무기로 쓰기에도 편리하다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','편의점','','','','북쪽 어딘가에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('212') || 0]=['25','打撃','前９','','7','1500','누군가 설치한 간이 텐트. 휴대가 가능한 편리한 무기다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','텐트','','','','동쪽 어딘가에 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('213') || 0]=['33','打撃','前９','','7','4500','강한 적이 우글거리는 동굴. 동굴 자체도 강하므로 강한 무기가 된다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','강철의 동굴','','','','남서쪽 어딘가에 자리잡고 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('214') || 0]=['24','打撃','前９','','7','4400','약하지만 영악한 적이 살고 있는 동굴. 촌스러운 색깔이 특징적이다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','약자의 동굴','','','','남동쪽 어딘가에 자리잡고 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('215') || 0]=['28','打撃','前９','','7','4800','마법을 좋아하는 적이 사는 동굴. 동굴 자체에도 마력이 응축되어 있다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','환술의 동굴','','','무작위로 【궁극】 마법 공격이 발동','남쪽 숲 안에 자리잡고 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('216') || 0]=['26','打撃','前９','','7','5200','폐허가 된 성에 영혼이 살면서 동굴이 되었다. 끔찍한 저주의 힘이 느껴진다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','영혼의 동굴','','','즉사(5%)','북동쪽 어딘가에 자리잡고 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('217') || 0]=['35','打撃','前９','','7','7500','전설의 무기가 잠들어 있는 황금 동굴. 사실은 동굴 본체도 강한 무기가 된다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','전설의 동굴','','','','마을 북쪽에서 어떤 기척이 느껴진다.','9 매스 앞으로'];
_tr_w["kor"][Number('218') || 0]=['38','打撃','前９','','7','8700','마왕이 살고 있다는 사악한 성. 무기로 사용하면 마왕처럼 강력할 것 같다.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','마왕성','','','','북쪽 끝 어딘가에 우뚝 서 있습니다.','9 매스 앞으로'];
_tr_w["kor"][Number('219') || 0]=['45','打撃','前９','','7','9800','언제나 평화로운 평온한 마을. 태어나서 줄곧 살아온 마을.','%1는 %2을 휙휙 휘둘렀다!','%2은 굉장한 위력으로 %3에 세차게 부딪혔다!','','','평온한 마을','','','','당신이 살고 있는 마을입니다.','9 매스 앞으로'];
_tr_w["kor"][Number('220') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('221') || 0]=['8','打撃','直線２','','5','600','보통 금보다 조금 큰 금. 큰 만큼 위력도 세졌다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3이(가) %2에 맞았다!','%3에 금이 갔다!','','약간 큰 금','','','방어력을 낮춘다(100%)','철로 만들어진 동굴에 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('222') || 0]=['6','打撃','','','5','300','벽 아래쪽에 뚫려 있는 구멍. 노후화가 원인인 것 같다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3이(가) %2에 맞았다!','%3에 금이 갔다!','','벽에 뚫린 구멍','','','방어력을 낮춘다(70%)','철로 만들어진 동굴에 있습니다.',''];
_tr_w["kor"][Number('223') || 0]=['4','打撃','','','5','500','표면이 무너져 속이 훤히 보이는 벽. 금방 무너질 것 같아서 무기로 쓰기에는 좀….','%1는 %2을 손에 들고 휘둘러 보았다!','%3이(가) %2에 맞았다!','','','반쯤 무너진 벽','','','','영혼이 떠도는 동굴에 있습니다.',''];
_tr_w["kor"][Number('224') || 0]=['2','打撃','','','3','100','바닥에 까맣게 물든 부분. 더러워서 만지고 싶지 않다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 약간 검게 물들었다!','검정 반점의 일부가 %3의 눈에 들어갔다!','','검정 반점','','','명중률을 낮춘다(100%)','북쪽 끝 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('225') || 0]=['2','打撃','','','3','100','녹색으로 물든 질퍽거리는 지면. 후텁지근한 지역에 많다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3(은)는 약간 녹색으로 물들었다!','질퍽거리는 녹색 물체가 %3의 발에 엉겨 붙었다!','','녹색 습지','','','회피율을 낮춘다(100%)','철로 만들어진 동굴에 있습니다.',''];
_tr_w["kor"][Number('226') || 0]=['2','打撃','','','3','200','어디에나 있는 길. 이것이 있으면 걷기 쉽다.','%1는 %2을 손에 들고 휘둘러 보았다!','%3이(가) %2에 맞았다!','','','길','','','','주변에 잔뜩 떨어져 있습니다.',''];
_tr_w["kor"][Number('227') || 0]=['4','斬撃','','','3','400','돌로 만든 흰색 타일. 4장이 1세트다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 모서리로 %3(을)를 베었다!','','','흰색 타일','','','','근처 편의점 안에 있습니다.',''];
_tr_w["kor"][Number('228') || 0]=['3','斬撃','','','3','200','오래 돼서 금이 간 낡은 흰색 타일.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 모서리로 %3(을)를 베었다!','','','금이 간 흰색 타일','','','','영혼이 떠도는 동굴에 있습니다.',''];
_tr_w["kor"][Number('229') || 0]=['5','斬撃','','','3','500','돌로 만든 파란색 타일. 흰색 타일보다 조금 강하다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 모서리로 %3(을)를 베었다!','','','파란색 타일','','','','북동쪽 어딘가에 있습니다.',''];
_tr_w["kor"][Number('230') || 0]=['3','斬撃','','','3','250','낡아서 금이 간 파란색 타일. 보기에도 낡아 보인다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 모서리로 %3(을)를 베었다!','','','금이 간 파란색 타일','','','','북동쪽 어딘가에 있습니다.',''];
_tr_w["kor"][Number('231') || 0]=['5','斬撃','','','3','400','깨끗한 마룻바닥. 청소가 잘 되어 있다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 모서리로 %3(을)를 베었다!','','','마룻바닥','','','','주변의 집 안에 있습니다.',''];
_tr_w["kor"][Number('232') || 0]=['8','斬撃','','','3','500','화려한 체크 무늬 타일. 취향이 조금… 아니, 아무것도 아니다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 모서리로 %3(을)를 베었다!','','','체크 무늬 타일','','','','신기하게 생긴 집 안에 있습니다.',''];
_tr_w["kor"][Number('233') || 0]=['6','斬撃','','','3','300','나무로 된 교회 바닥. 자연스러운 분위기를 자아낸다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 모서리로 %3(을)를 베었다!','','','교회 바닥','','','','어느 교회 안에 있습니다.',''];
_tr_w["kor"][Number('234') || 0]=['12','打撃','','','6','1500','무기를 손질할 수 있는 모루. 큰 그루터기 위에 올려져 있다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','','','모루','','','','주변에 놓여져 있습니다.',''];
_tr_w["kor"][Number('235') || 0]=['7','なし','','','2','600','약자의 동굴 안에 있는 거대한 독의 늪을 작게 잘라낸 것.','%1는 %2을 손에 들고 휘둘러 보았다!','','','','독의 늪(소)','','','독(100%)','독의 늪으로 덮인 동굴에 있습니다.',''];
_tr_w["kor"][Number('236') || 0]=['15','打撃','','','4','2000','마계에서 만들어진 사악한 등롱. 자줏빛 화염이 지옥을 연상시킨다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','%2에서 어둠의 화염이 이글거린다!','마계의 등롱','','','화염 발생','마족의 왕이 사는 성 안에 있습니다.',''];
_tr_w["kor"][Number('237') || 0]=['12','斬撃','直線２','4','20','15000','곰 형제에게 받은 신기한 검. 검을 손에 들면 곰 형제와의 추억이 떠오른다.','%1는 %2을 휘둘렀다!','','','','곰의 검','','','4회 공격','4마리의 곰한테서 받을 수 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('238') || 0]=['11','打撃','直線２','','15','9000','하늘을 날 수 있는 굉장한 빗자루. 마녀가 하늘을 나는 모습이 떠오른다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','','','하늘을 나는 빗자루','','','하늘을 난다','당신의 여동생이 몰래 가지고 있습니다.','2 매스 직선'];
_tr_w["kor"][Number('239') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('240') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('241') || 0]=['13','打撃','','','3','499','누군가의 시체인 것 같다. 말을 걸어도 대답이 없다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3(은)는 저주에 걸렸다!','','시체','','','즉사(5%)','마을 남쪽으로 나가면 떨어져 있습니다.',''];
_tr_w["kor"][Number('242') || 0]=['13','打撃','','','3','399','동물 시체인 것 같다. \'손!\'을 해도 반응이 없다.','%1는 %2을 휘둘렀다!','%2은 %3에 정통으로 맞았다!','%3(은)는 저주에 걸렸다!','','동물 시체','','','즉사(5%)','남서쪽 어딘가에 떨어져 있습니다.',''];
_tr_w["kor"][Number('243') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('244') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('245') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('246') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('247') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('248') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('249') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('250') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('251') || 0]=['en','打撃','','','6','500','밖을 방황하는 끈적거리는 액체. 사람을 발견하면 이유도 없이 덤벼든다.','가라! %2!','%2은 온몸으로 %3에 철퍼덕 부딪혔다!','','','슬라임','','','','마을 근처에 자주 출몰합니다.',''];
_tr_w["kor"][Number('252') || 0]=['en','打撃','','','6','700','빨갛게 변색한 변종 슬라임. 더 난폭해졌다.','가라! %2!','%2은 온몸으로 %3에 철퍼덕 부딪혔다!','','','레드 슬라임','','','','북쪽 어딘가에 출몰합니다.',''];
_tr_w["kor"][Number('253') || 0]=['en','打撃','','','6','2000','황금색으로 빛나는 희귀종 슬라임. 비싸게 팔릴 것 같다.','가라! %2!','%2은 온몸으로 %3에 철퍼덕 부딪혔다!','','','골드 슬라임','','','','황금색으로 빛나는 동굴 안에 출몰합니다.',''];
_tr_w["kor"][Number('254') || 0]=['en','打撃','','','6','800','항상 불타고 있는 말. 게다가 더위를 많이 탄다고 한다.','가라! %2!','%2은 %3(을)를 발로 차버렸다!','','','불타는 말','','','화염을 뿜는다','마을 밖 서쪽 어딘가에 출몰합니다.',''];
_tr_w["kor"][Number('255') || 0]=['en','打撃','','','6','1600','불길한 색의 화염을 휘감은 말. 지옥에서 왔다고 한다.','가라! %2!','%2은 %3(을)를 발로 차버렸다!','','','옥염마','','','화염을 뿜는다','마족의 왕이 사는 성 안에 출몰합니다.',''];
_tr_w["kor"][Number('256') || 0]=['en','打撃','','','6','600','장난꾸러기 몬스터. 아무것이나 막 던진다.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','고블린','','','무기를 던진다','마을 밖 남쪽 어딘가에 출몰합니다.',''];
_tr_w["kor"][Number('257') || 0]=['en','打撃','','','6','900','흉포한 소의 화신. 품 안에 감춰둔 수많은 무기를 던진다.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','미노타우르스','','','무기를 던진다','북쪽 어딘가에 출몰합니다.',''];
_tr_w["kor"][Number('258') || 0]=['en','打撃','','','7','500','내장까지 바위로 된 거인. 칼로 베어도 꿈쩍도 안 한다.','가라! %2!','%2은 %3에게 바위 팔을 내리쳤다!','','','바위남','','','','철로 만들어진 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('259') || 0]=['en','打撃','','','7','700','모든 세포가 바위로 이루어진 거인. 팔의 파괴력은 상상을 초월한다.','가라! %2!','%2은 %3에게 바위 팔을 내리쳤다!','','','바위 아저씨','','','','황금색으로 빛나는 동굴 안에 출몰합니다.',''];
_tr_w["kor"][Number('260') || 0]=['en','打撃','','','6','600','굉장히 딱딱한 껍질을 가진 해충. 바위를 깨부술 정도로 강하다.','가라! %2!','%2는 바위 깨기를 구사했다!','','','강철 벌레','','','방어력을 낮춘다(50%)','철로 만들어진 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('261') || 0]=['en','打撃','','','6','3000','세상에서 가장 딱딱한 생물. 물리 공격은 전혀 먹히질 않는다.','가라! %2!','%2는 갑옷 깨기를 구사했다!','','','다이아몬드 벌레','','','방어력을 낮춘다(50%)','마족의 왕이 사는 성 안에 출몰합니다.',''];
_tr_w["kor"][Number('262') || 0]=['en','打撃','','','5','1000','불에서 태어난 화염의 정령. 다가가면 순식간에 잿덩어리가 될 것 같다.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','화염 정령','','','【화염 마법】을 외운다','마력으로 가득 찬 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('263') || 0]=['en','打撃','','','5','1000','얼음에서 태어난 얼음의 정령. 주변 기온은 항상 영하다.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','얼음 정령','','','【얼음 마법】을 외운다','마력으로 가득 찬 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('264') || 0]=['en','打撃','','','5','1000','천둥에서 태어난 천둥의 정령. 화가 나면 하늘에서 벼락이 친다.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','천둥 정령','','','【천둥 마법】을 외운다.','마력으로 가득 찬 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('265') || 0]=['en','打撃','','','5','1800','모든 정령을 다스리는 정령의 왕. 그 모습은 아름답고 신비롭다.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','정령왕','','','무작위로 【궁극】 공격 마법을 외운다','황금색으로 빛나는 동굴 안에 출몰합니다.',''];
_tr_w["kor"][Number('266') || 0]=['en','打撃','','','5','800','눈이 많이 달려 기분 나쁘게 생긴 몬스터. 공격력을 낮추는 짜증나는 녀석.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','레드 게이즈','','','【공격 약화 마법】을 외운다','남쪽 어딘가에 출몰합니다.',''];
_tr_w["kor"][Number('267') || 0]=['en','打撃','','','5','700','새파란 몸체가 기분 나쁘게 생긴 몬스터. 수많은 눈으로 노려보면 방어가 내려간다.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','블루 게이즈','','','【방어 약화 마법】을 외운다','남쪽 어딘가에 출몰합니다.',''];
_tr_w["kor"][Number('268') || 0]=['en','打撃','','','5','1500','피를 너무 많이 뒤집어 써서 색이 변한 게이즈. 이 녀석의 약화 마법에는 조심할 것.','가라! %2!','%2는 온몸으로 %3에 부딪혔다!','','','블러드 게이즈','','','【전체 약화 마법】을 외운다','마족의 왕이 사는 성 안에 출몰합니다.',''];
_tr_w["kor"][Number('269') || 0]=['en','打撃','','','6','900','신기한 베일로 둘러싸인 해파리. 마법이 전혀 통하지 않는다.','가라! %2!','%2는 독 촉수로 %3(을)를 공격했다!','','','마법 해파리','','','감전(25%)','마력으로 가득 찬 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('270') || 0]=['en','打撃','','','6','1100','사람들을 죽음으로 이끄는 끔찍한 몬스터. 사신이 보냈다고 한다.','가라! %2!','%2은 웃으면서 %3(을)를 공격했다!','','','저승사자','','','【어둠 마법】을 외운다','영혼이 떠도는 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('271') || 0]=['en','打撃','','','6','1700','죽음을 지배하는 사악한 몬스터. 눈에 띄기만 해도 지옥으로 끌고 간다.','가라! %2!','%2은 웃으면서 %3(을)를 공격했다!','','','사신','','','【어둠 마법∼궁극】을 외운다','마족의 왕이 사는 성 안에 출몰합니다.',''];
_tr_w["kor"][Number('272') || 0]=['en','打撃','','','6','699','시체에 영혼이 깃들어 움직이기 시작했다. 웃으면서 뼈를 던진다.','가라! %2!','%2은 웃으면서 %3(을)를 공격했다!','','','스켈턴','','','','영혼이 떠도는 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('273') || 0]=['en','打撃','','','6','700','포자를 뿌리면서 공격해 오는 굉장히 위험한 버섯','가라! %2!','%2은 온몸으로 %3에 부딪혔다!','','','대머리 버섯','','','포자를 뿌린다','서쪽 숲에 출몰합니다.',''];
_tr_w["kor"][Number('274') || 0]=['en','打撃','','','4','1000','그냥 보기에도 독으로 가득 찬 버섯. 절대 먹지 않도록 조심할 것.','가라! %2!','%2은 온몸으로 %3에 부딪혔다!','','','도깨비 버섯','','','포자를 뿌린다','북쪽 어딘가에 출몰합니다.',''];
_tr_w["kor"][Number('275') || 0]=['en','打撃','','','6','500','항상 우글우글 몰려다니는 벌레. 생김새도 징그럽지만 정말 성가시다.','가라! %2!','%2가 우글우글한다.','','','우글 벌레','','','','독의 늪으로 덮인 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('276') || 0]=['en','打撃','','','3','800','상상을 초월하는 음량을 자랑하는 새. 아침에 울면 안 일어나고는 못 배긴다.','가라! %2!','%2는 150dB의 울음소리를 냈다!','','','데시벨 버드','','','마법 공격을 봉인한다(50%)','마을 밖 서쪽 어딘가에 출몰합니다.',''];
_tr_w["kor"][Number('277') || 0]=['en','打撃','','','3','1200','나약하고 눈물이 많지만 영악하다. 마법으로 자신을 강화하는 성가신 적.','가라! %2!','%2은 울면서 %3(을)를 할퀴었다!','','','울보 데빌','','','무작위로 강화 마법을 외운다','독의 늪으로 덮인 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('278') || 0]=['en','打撃','','','3','900','심신이 모두 듬직한 데빌. 다만 영악함만은 그대로다.','가라! %2!','%2은 용감하게 %3(을)를 할퀴었다!','','','용감 데빌','','','무작위로 강화 마법을 외운다','황금색으로 빛나는 동굴 안에 출몰합니다.',''];
_tr_w["kor"][Number('279') || 0]=['en','打撃','','2','5','600','퍼덕거리며 날아와 물어뜯는다. 움직임이 민첩하므로 주의가 필요.','가라! %2!','%2는 %3(을)를 재빠르게 2번 물었다!','','','박쥐','','','2회 공격','마을 근처에 출몰합니다.',''];
_tr_w["kor"][Number('280') || 0]=['en','打撃','','2','5','900','어둠을 틈타 공격해 오는 성가신 녀석. 물리면 꽤 아프다.','가라! %2!','%2는 %3(을)를 재빠르게 2번 물었다!','','','어둠 박쥐','','','2회 공격','영혼이 떠도는 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('281') || 0]=['30','なし','','','1','1100','아무튼 굉장히 위험한 바위. 빈사 상태가 되면 자폭한다.','가라! %2!','%2는 대폭발을 일으켰다!','','','폭탄 바위','','','폭발한다','철로 만들어진 동굴에 출몰합니다.',''];
_tr_w["kor"][Number('282') || 0]=['en','打撃','','','6','2200','우글 벌레의 왕. 무기를 풍화시키는 우글우글 구슬을 던져 공격한다.','가라! %2!','%2가 우글우글한다.','','','우글우글 벌레','','','','마족의 왕이 사는 성 안에 출몰합니다.',''];
_tr_w["kor"][Number('283') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('284') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('285') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('286') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('287') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('288') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('289') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('290') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('291') || 0]=['en','斬撃','','3','8','8500','튼튼한 갑옷이 온 몸을 뒤덮은 거대한 용. 용의 발톱은 그 무엇이든 찢어 발긴다.','가라! %2!','%2는 강철 발톱으로 몇 번이고 할퀴었다!','','','철갑용 엘림살루에','','','3회 공격','철로 만들어진 동굴의 가장 깊은 곳에 출몰합니다.',''];
_tr_w["kor"][Number('292') || 0]=['en','打撃','直線３','','7','7700','약자의 동굴에 나타난 불길한 괴조. 싸울 때 자신을 강화하는 마법을 사용한다.','가라! %2!','%2는 독물을 뿜어댔다!','','','괴조 비자야','','','독(50%),【전체 강화 마법】을 외운다','독의 늪으로 덮인 동굴의 가장 깊은 곳에 출몰합니다.','3 매스 직선'];
_tr_w["kor"][Number('293') || 0]=['en','打撃','前６','','9','9900','의지를 가진 수수께끼의 덩어리. 촉수를 사용해 공격하지만, 본체에는 아무것도 없다.','가라! %2!','%2는 %3(을)를 덮쳤다!','','','마괴 티르소스','','','','마력으로 가득 찬 동굴의 가장 깊은 곳에 출몰합니다.','6 매스 앞으로'];
_tr_w["kor"][Number('294') || 0]=['en','打撃','直線３','','4','2400','불길한 색을 띈 촉수. 거대한 마력을 품고 있는 듯하다.','가라! %2!','%2는 채찍처럼 휘더니 %3에 정통으로 맞았다!','','','마촉수','','','무작위로 공격 마법을 외운다','마력으로 가득 찬 동굴의 가장 깊은 곳에 출몰합니다.','3 매스 직선'];
_tr_w["kor"][Number('295') || 0]=['en','斬撃','','','7','8800','영혼의 동굴에 나타나는 기사의 영혼. 강한 원념과 한을 품고 있다.','가라! %2!','%2는 %3(을)를 꿰뚫어 매달았다!','','','영혼기사 다인슬라이프','','','【어둠 마법∼궁극】을 외운다','영혼이 떠도는 동굴의 가장 깊은 곳에 출몰합니다.',''];
_tr_w["kor"][Number('296') || 0]=['en','打撃','直線２','','15','19999','세상 모든 무기를 지배하는 무기의 신. 모든 것을 초월한 존재.','가라! %2!','%2은 「대신참」을 구사했다!','%3(은)는 신의 위엄에 겁먹었다….','','무기의 신','아메노오하바리','','1턴 행동 불능(30%)','황금색으로 빛나는 동굴의 가장 깊은 곳에 출몰합니다.','2 매스 직선'];
_tr_w["kor"][Number('297') || 0]=['en','打撃','','2','12','19999','세계를 멸망시키려는 마왕. 실은 세계 평화를 바라고 있다.','가라! %2!','%2는 「마계권골」을 구사했다!','','','마왕 도르하바','','','2회 공격,혼란(20%)','마왕성의 최상층에 출현합니다.',''];
_tr_w["kor"][Number('298') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('299') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('300') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('301') || 0]=['14','斬撃','','','5','800','마을 근처를 배회하는 주인 없는 개. 사람을 잘 따르고 귀엽다.','가라! %2!','%2는 %3(을)를 물었다!','%2는 멍멍 짖으며 %3(을)를 위협했다!','','개','','','1턴 행동 불능(20%)','마을 어딘가를 산책하고 있습니다.',''];
_tr_w["kor"][Number('302') || 0]=['13','打撃','','','5','800','마을 근처를 배회하는 길고양이. 야옹~야옹~ 한다.','가라! %2!','%2는 %3에게 날카로운 고양이 펀치를 날렸다!','%2는 야옹~야옹~ 하며 %3에게 응석을 부리기 시작했다!','','고양이','','','공격력을 낮춘다(50%)','마을 어딘가를 산책하고 있습니다.',''];
_tr_w["kor"][Number('303') || 0]=['16','打撃','','','6','1500','누군가가 기르는 말. 털 손질이 잘 되어 아름답다.','가라! %2!','%2은 %3(을)를 발로 차버렸다!','','','말','','','날려 버린다','북서쪽에서 어슬렁대고 있다.',''];
_tr_w["kor"][Number('304') || 0]=['19','打撃','','','6','1600','힌트를 알려주는 굉장한 양. 푸근푸근한 털이 귀엽다.','가라! %2!','%2은 온몸으로 %3에 부딪혔다!','%2의 복슬복슬한 털이 %3에 엉켜 붙었다!','','양','','','행동 횟수를 낮춘다(30%)','저예요.',''];
_tr_w["kor"][Number('305') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('306') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('307') || 0]=['18','打撃','','','6','720','평온한 마을에서 혼자서 생활하는 여성. 미인에다 성격도 좋다.','가라! %2!','%2은 미소를 지으며 박치기를 했다!','','','마을 여성','Piranka','','','마을 어떤 집에 살고 있습니다.',''];
_tr_w["kor"][Number('308') || 0]=['8','打撃','','','6','1050','마을에 사는 한 가정의 대들보. 항상 밖에서 아이들과 놀고 있다.','가라! %2!','%2은 대들보 펀치를 날렸다!','','','한 집안의 아빠','Tel-ho','','','마을 어딘가에서 아이를 지켜봅니다.',''];
_tr_w["kor"][Number('309') || 0]=['25','打撃','','','3','960','평온한 마을에 사는 가족의 어머니. 평소에는 상냥하지만 화나면 무섭다.','가라! %2!','%2는 꿀밤을 날렸다!','','%2는 화염을 뿜어댔다!','한 집안의 엄마','Hokkoma','','화염을 뿜는다','마을 어떤 집에 살고 있습니다.',''];
_tr_w["kor"][Number('310') || 0]=['12','打撃','','','5','210','마을에서 뛰어 노는 활기찬 형제 중 형. 항상 「와~아」 소리를 지른다.','가라! %2!','%2는 「와~아!」 소리를 지르며 펀치를 날렸다!','','','활기찬 형제','Kubota','','','마을 어딘가에서 신나게 놀고 있습니다.',''];
_tr_w["kor"][Number('311') || 0]=['6','打撃','','','5','180','형과 함께 놀고 있는 귀여운 동생. 장래의 꿈은 Yout◯uber일 것이다.','가라! %2!','%2는 필살 울트라 라이더 킥을 날렸다!','','','활기찬 형제','Nagina','','','마을 어딘가에서 신나게 놀고 있습니다.',''];
_tr_w["kor"][Number('312') || 0]=['17','打撃','','','6','1530','평온한 식당을 운영하는 오너 쉐프. 싸울 때는 발 기술을 잘 쓴다.','가라! %2!','%2는 강렬한 발차기를 했다!','','','식당의 요리사','A flavor','','','마을 어떤 식당에서 요리사를 하고 있습니다.',''];
_tr_w["kor"][Number('313') || 0]=['11','打撃','','','6','840','평온한 식당에 매일 오는 단골 손님. 최근 혈당치가 많이 높다.','가라! %2!','%2은 고혈당 펀치를 반복해서 날렸다!','%3도 당뇨병에 걸렸다!','','식당 단골','Dongfa','','행동 횟수를 낮춘다(20%)','마을 어떤 식당에서 식사를 하고 있습니다.',''];
_tr_w["kor"][Number('314') || 0]=['13','打撃','','','6','1500','세계에서 활동하는 자칭 요리 평론가. 평온한 식당에 대한 칭찬이 자자하다.','가라! %2!','%2은 튀어나온 배로 %3(을)를 짓뭉갰다!','','','요리 평론가','Katsubaruga','','','마을 어떤 식당에서 식사를 하고 있습니다.',''];
_tr_w["kor"][Number('315') || 0]=['4','打撃','','','4','2520','마을에 사는 상냥한 눈빛을 가진 노인. 매일 아내의 무덤을 찾아간다.','가라! %2!','%2은 약하게 온몸으로 부딪혔다!','맙소사 %2의 죽은 아내가 %3에게 저주를 걸었다!','','성묘하는 노인','Braig','','즉사(5%)','마을 어딘가에서 성묘를 하고 있습니다.',''];
_tr_w["kor"][Number('316') || 0]=['25','打撃','','','6','1740','우물가 수다에 빠지지 않는 아주머니. 한번 말하기 시작하면 2시간은 떠든다','가라! %2!','%2는 히스테리로 가득 찬 따귀를 날렸다!','%3(은)는 히스테릭한 아주머니를 보고 충격을 받았다!','','우물가의 아주머니','Fukiya','','혼란(20%)','마을 어딘가의 우물가에서 수다를 떨고 있습니다.',''];
_tr_w["kor"][Number('317') || 0]=['15','打撃','','2','6','1680','소문을 좋아하는 아주머니. 항상 우물가에서 수다에 열중이다.','가라! %2!','%2는 왕복 따귀를 날렸다!','','','우물가의 아주머니','Illisa','','2회 공격','마을 어딘가의 우물가에서 수다를 떨고 있습니다.',''];
_tr_w["kor"][Number('318') || 0]=['7','打撃','','','4','2160','마을에서 자주 산책하는 노인. 묻지도 않은 얘기를 계속 떠든다.','가라! %2!','%2은 지팡이로 %3(을)를 때렸다!','','','산책 중인 노인','Rod','','','마을 어딘가를 산책하고 있습니다.',''];
_tr_w["kor"][Number('319') || 0]=['10','打撃','','','6','1050','마을을 산책하던 아저씨. 고향에서 먼 곳까지 돈을 벌러 왔다고 한다.','가라! %2!','%2는 산책으로 단련한 다리로 %3(을)를 걷어찼다!','','','산책 중인 아저씨','Ranyapa','','날려 버린다','마을 어딘가를 산책하고 있습니다.',''];
_tr_w["kor"][Number('320') || 0]=['','','','','','0','','','','','','','','','','',''];
_tr_w["kor"][Number('321') || 0]=['','','','','','0','','','','','','','','','','',''];
_tr_w["kor"][Number('322') || 0]=['5','打撃','','','6','630','전설의 무기를 찾고 있는 모험가. 세끼 식사보다 엄마를 더 좋아한다.','가라! %2!','%2는 엄마한테 배운 펀치를 반복해서 날렸다!','','','모험가','Kikangji','','','남쪽 어딘가를 돌아다니고 있습니다.',''];
_tr_w["kor"][Number('323') || 0]=['13','打撃','','','9','1470','매일 연못에서 목욕하는 할아버지. 누군가 신고 좀 해줬으면 좋겠다.','가라! %2!','%2는 웃으면서 온몸으로 %3에 부딪혔다!','%3(은)는 변태를 보고 충격을 받았다!','','목욕 할아버지','Tetsken','','혼란(40%)','마을 밖 남쪽 어딘가에 서 있습니다.',''];
_tr_w["kor"][Number('324') || 0]=['18','打撃','','','6','810','해변에서 야영을 하는 청년. 굳이 말하자면 인도어파이다.','가라! %2!','%2은 야영 도구를 휘둘러 공격했다!','','','야영객','Bastar','','','남쪽 어딘가에서 야영을 하고 있습니다.',''];
_tr_w["kor"][Number('325') || 0]=['15','打撃','','','6','540','나홀로 야영을 즐기는 여성. 할아버지의 영향을 받아 야영을 시작했다.','가라! %2!','%2은 야영 도구를 휘둘러 공격했다!','','','야영객','Ikpa','','','남쪽 어딘가에서 야영을 하고 있습니다.',''];
_tr_w["kor"][Number('326') || 0]=['17','打撃','','','6','960','혼자서 흥분하고 있는 야영객. 모닥불을 보며 혼자 들떠 있다.','가라! %2!','%2은 야영 도구를 휘둘러 공격했다!','','%2은 캠프파이어를 시작했다!','야영객','Charak','','화염 발생','남쪽 어딘가에서 야영을 하고 있습니다.',''];
_tr_w["kor"][Number('327') || 0]=['14','斬撃','','','6','750','자아를 찾고자 여행 중인 여성. 동굴의 위험성을 알려 주었다.','가라! %2!','%2은 단검으로 %3(을)를 베었다!','','','여행 중인 여성','Cathrus','','','남쪽 어딘가를 여행하고 있습니다.',''];
_tr_w["kor"][Number('328') || 0]=['22','斬撃','','','6','600','독의 늪의 위험성을 알려준 모험가. 약초까지 나눠준 친절한 사람이다.','가라! %2!','%2는 애용하는 검으로 %3(을)를 베었다!','','','모험가','Fureil','','','남동쪽 어딘가에 서 있습니다.',''];
_tr_w["kor"][Number('329') || 0]=['28','斬撃','','','8','1950','낡은 오두막에서 혼자 사는 나무꾼. 도끼를 잘 다룬다.','가라! %2!','%2은 나무를 베듯이 도끼를 휘둘렀다!','','','남자 나무꾼','Wonmar','','방어력을 낮춘다(30%)','어느 오두막에 살고 있습니다.',''];
_tr_w["kor"][Number('330') || 0]=['5','打撃','','','4','2310','느긋하게 여행을 다니는 노인. 인생의 정답을 찾아다니고 있다.','가라! %2!','%2은 힘없이 박치기를 했다!','','','여행 중인 노인','Brood','','','마을 밖 서쪽 어딘가를 여행하고 있습니다.',''];
_tr_w["kor"][Number('331') || 0]=['11','斬撃','','','6','990','수련을 위해 여행을 다니는 모험가. 실력자인 척 하지만 사실 그렇지도 않다.','가라! %2!','%2는 검으로 %3(을)를 공격했다!','','','모험가','Esto','','','마을 밖 서쪽 어딘가를 돌아다니고 있습니다.',''];
_tr_w["kor"][Number('332') || 0]=['9','打撃','','','6','5400','숲에 사는 작은 마녀. 이래 봬도 그녀의 마력은 엄청나다.','가라! %2!','%2는 지팡이를 휘둘러 공격했다!','','%2의 지팡이 끝에서 마법이 튀어나왔다!','마녀','Cremon','','무작위로 【궁극】 공격 마법을 외운다','신기하게 생긴 집에 살고 있습니다.',''];
_tr_w["kor"][Number('333') || 0]=['17','打撃','','','6','1260','매일 시장을 향해 걸어가는 여성. 시장에 도착하는 날을 꿈꾸고 있다.','가라! %2!','%2는 걸으며 단련한 다리로 %3(을)를 차버렸다!','','','시장에 가는 여성','Takana','','날려 버린다','마을 밖 동쪽 어딘가에 서 있습니다.',''];
_tr_w["kor"][Number('334') || 0]=['24','打撃','','','6','630','시장에 가게를 낸 여성. 공부를 대단히 열심히 한다.','가라! %2!','%2은 모닝 스타를 휘둘렀다!','','','장사하는 여성','Yawarra','','혼란(20%)','동쪽에서 장사를 하고 있습니다.',''];
_tr_w["kor"][Number('335') || 0]=['20','斬撃','','2','6','1020','시장에서 채소를 파는 남성. 실은 그랜드마더 보이.','가라! %2!','%2은 재빠른 검술로 %3(을)를 베었다!','','','장사하는 남성','Rapier','','2회 공격','동쪽에서 장사를 하고 있습니다.',''];
_tr_w["kor"][Number('336') || 0]=['14','打撃','','','7','1650','딱 봐도 사기 치는 장사꾼. 이런 사람과는 엮이지 않는 것이 좋다.','가라! %2!','%2은 튀어나온 배로 %3(을)를 짓뭉갰다!','','','장사하는 남성','Pusa','','','동쪽에서 장사를 하고 있습니다.',''];
_tr_w["kor"][Number('337') || 0]=['19','打撃','','','6','720','시장을 구경하러 온 빨강머리 남자. 가진 건 단돈 28엔.','가라! %2!','%2은 오른손 스트레이트 펀치를 날렸다!','','','시장 손님','Shrink','','','동쪽의 시장에서 물건을 구경하고 있습니다.',''];
_tr_w["kor"][Number('338') || 0]=['8','斬撃','','','6','660','시장에 온 여성. 반값 세일을 기다리고 있다.','가라! %2!','%2은 시장에서 산 검으로 %3(을)를 공격했다!','','','시장 손님','Scruma','','','동쪽의 시장에서 물건을 구경하고 있습니다.',''];
_tr_w["kor"][Number('339') || 0]=['12','打撃','','','4','1350','한가해서 시장에 놀러 온 남성. 수상한 항아리를 산 모양이다.','가라! %2!','%2은 시장에서 산 검으로 %3(을)를 공격했다!','','','시장 손님','Bundy','','반드시 회심의 일격','동쪽의 시장에서 물건을 구경하고 있습니다.',''];
_tr_w["kor"][Number('340') || 0]=['18','打撃','','2','6','1440','돈이 많은 시장 손님. 돈다발로 부채질을 하며 걷고 있다.','가라! %2!','%2은 돈다발을 꺼내어 따귀를 때렸다!','%3(은)는 큰 돈을 보고 흥분했다!','','시장 손님','Murgiza','','2회 공격,상대의 행동 횟수를 낮춘다','동쪽의 시장에서 물건을 구경하고 있습니다.',''];
_tr_w["kor"][Number('341') || 0]=['17','打撃','','','5','1230','바닷가에 있는 교회의 목사. 투척교를 신봉한다.','가라! %2!','%2는 아멘 펀치를 날렸다!','%3에 성스러운 힘이 작용했다!','','교회 목사','Falsio','','능력 강화를 초기화한다','어느 교회 안에 서 있습니다.',''];
_tr_w["kor"][Number('342') || 0]=['22','打撃','','','6','1140','상냥하게 생긴 여신도. 좋아하는 면 요리는 바지락 칼국수.','가라! %2!','%2는 온몸으로 힘껏 들이받았다!','','','교회 여신도','Lancia','','','어느 교회 안에 서 있습니다.',''];
_tr_w["kor"][Number('343') || 0]=['19','打撃','','','5','2460','교회에서 기도하는 노파. 투척교의 열렬한 신자다.','가라! %2!','%2는 기도하면서 태클했다!','','','기도하는 노파','Versa','','','어느 교회 안에서 기도하고 있습니다.',''];
_tr_w["kor"][Number('344') || 0]=['13','打撃','','','6','450','교회에서 기도하는 청년. 정말 착실하다.','가라! %2!','%2은 기도하면서 박치기를 했다!','','','기도하는 청년','Irarufu','','','어느 교회 안에서 기도하고 있습니다.',''];
_tr_w["kor"][Number('345') || 0]=['14','打撃','','','6','750','안경이 잘 어울리는 녹색 머리카락의 모험가. 폐허 순례가 취미.','가라! %2!','%2는 지팡이를 휘둘러 공격했다!','%2의 지팡이 끝에서 마법이 튀어나왔다!','','모험가','Clique','','【얼음 마법】 발동','북동쪽 어딘가에 서 있습니다.',''];
_tr_w["kor"][Number('346') || 0]=['10','打撃','','','5','2040','낚시와 바다를 사랑하는 노인. 하지만 낚시 실력은 그저 그렇다.','가라! %2!','%2은 낚싯대를 휘둘러 공격했다!','맙소사 낚싯바늘에는 썩은 지렁이가 달려 있었다!','','낚시꾼','Phyzigro','','독(60%)','북서쪽 곶에서 낚시를 하고 있습니다.',''];
_tr_w["kor"][Number('347') || 0]=['12','斬撃','','3','6','510','기가 드셀 것 같은 여성 모험가. 취업 활동 중이라고 한다.','가라! %2!','%2는 단검으로 %3(을)를 순식간에 베었다!','','','모험가','Dagger','','3회 공격','북쪽 어딘가를 돌아다니고 있습니다.',''];
_tr_w["kor"][Number('348') || 0]=['27','打撃','','','6','660','새로 들어온 편의점 점원. 손님한테 인사조차 안 한다.','가라! %2!','%2은 바코드 리더로 %3(을)를 때렸다!','','','편의점 점원','Miaota','','','어느 편의점 안에 서 있습니다.',''];
_tr_w["kor"][Number('349') || 0]=['15','斬撃','','','6','900','편의점에 물건을 사러 온 남성. 어떤 주스를 살지 망설이고 있다.','가라! %2!','%2은 편의점의 카트로 %3(을)를 베었다!','','','편의점 손님','Arsato','','','어느 편의점 안에 서 있습니다.',''];
_tr_w["kor"][Number('350') || 0]=['33','打撃','','','7','990','편의점 앞에 모여 있는 험상궂은 양키. 쳐다보기만 해도 시비를 걸 것 같다.','가라! %2!','%2는 %3(을)를 두들겨 팼다!','%3(은)는 %2에게 겁을 먹었다!','','양아치','Reserville','','1턴 행동 불능(20%)','어느 편의점 앞에 모여 있습니다.',''];
_tr_w["kor"][Number('351') || 0]=['29','斬撃','','','5','2160','한쪽 눈의 상처가 특징인 숙련된 여행자. 딱 보기에도 강해 보인다.','가라! %2!','%2는 대검을 호쾌하게 휘둘렀다!','','','여행 중인 아저씨','Gladys','','','북쪽 어딘가를 여행하고 있습니다.',''];
_tr_w["kor"][Number('352') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('353') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('354') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('355') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('356') || 0]=['2','打撃','','','8','180','\\n[1]의 여동생. 어리지만 야무진 성격. 집안일은 물론 가게 일도 혼자 도맡아 한다.','가라! %2!','%2는 용기를 내어 펀치를 날려 보았다!','%2는 %3의 지갑에서 돈을 훔쳤다!','','폰미','','','돈을 획득한다','당신의 여동생입니다.',''];
_tr_w["kor"][Number('357') || 0]=['30','斬撃','','','9','1140','의욕이 없어 보이는 용자. 마왕이 그냥 혼자 쓰러져 주길 바라고 있다.','가라! %2!','%2는 귀찮은 듯이 검을 휘둘렀다!','%3(은)는 %2에게 의욕을 빼앗겼다!','','용사 슈발디','','','공격력을 낮춘다(40%)','당신 가게가 커지면 찾아올 것입니다.',''];
_tr_w["kor"][Number('358') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('359') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('360') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('361') || 0]=['21','斬撃','直線２','','20','18564','마왕이 준 어둠의 대검. 일격 일격이 무서우리만치 묵직하다.','%1는 %2을 휘둘렀다!','','','','마왕의 검','','','반드시 회심의 일격','마족의 왕이 가지고 있을 것이다.','2 매스 직선'];
_tr_w["kor"][Number('362') || 0]=['17','打撃','','','8','1000','획득한 트로피를 넣어 두는 진열장. 손에 들면 꽤 무겁다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3에 정통으로 맞았다!','','','트로피 진열장','','','','당신의 집 안에 있습니다.',''];
_tr_w["kor"][Number('363') || 0]=['0','なし','','','20','14500','다양한 벌레를 잡을 수 있는 그물망. 흡인력은 블랙홀 수준이다.','%1는 %2을 휘둘렀다!','','','','곤충 채집망','','','눈 앞의 물건을 무기로서 획득','당신의 여동생이 몰래 가지고 있습니다.',''];
_tr_w["kor"][Number('364') || 0]=['25','打撃','','','9','2','소프트크림처럼 예쁘게 싼 똥. 색과 윤기까지 완벽하다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3의 얼굴에 정통으로 맞았다!','%2은 정말 냄새가 심하다!','','똥','','','공격력을 낮춘다(20%),방어력을 낮춘다(20%),행동 횟수를 낮춘다(20%)','북동쪽 어딘가에 뜨끈뜨끈한 것이 떨어져 있습니다.',''];
_tr_w["kor"][Number('365') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('366') || 0]=['','','','','','','','%1는 용기를 내어 펀치를 날려 보았다!','','','','(맨손)','','','3회 공격','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','%1는 \i[261]약초를 사용했다!','','','','(약초)','','','화염 발생','',''];
_tr_w["kor"][Number('3') || 0]=['','','','','','','많이 갖고 다니고 싶은 모험의 필수품. 체력도 상태 이상도 회복 가능.','%1는 %2을 손에 들고 휘둘러 보았다!','%2은 %3의 입안으로 빨려 들어가 버렸다!','','','약초','','','','적의 HP를 회복',''];
_tr_w["kor"][Number('4') || 0]=['','','','','','','무기의 숨겨진 힘을 끌어내는 신비한 돌. 획득하기 어려운 귀중한 소재다.','%1는 %2을 손에 들고 휘둘러 보았다!','%2의 모서리가 %3에 박혔다!','','','알마의 돌','','','','가끔 적이 떨어 뜨립니다.',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kor"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];

})();
    (function(){


_tr_w["han"][Number('4') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('5') || 0]=['10','斬撃','前３','3','5','200','武器之神‧舞契所創，\n獨一無二，用來除錯的武器。','%1揮動%2了！','%2直接命中%3的頭了！','%2散發的氣息刺激到%3了！','從%2當中噴出火焰了！','除錯壺','','','連擊3次','不存在。','3個方格正面'];
_tr_w["han"][Number('6') || 0]=['10','斬撃','','','30','1000','可以大量生產的銅劍。\n任何人都能輕鬆駕馭，威力卻無法期待。','%1揮動%2了！','','','','旅人的劍','','','','在妳家的寶箱裡沉睡著。',''];
_tr_w["han"][Number('7') || 0]=['20','斬撃 / 炎','','','30','2100','封印火焰魔力的魔法劍。\n只要揮動一下，附近就會陷入火海。','%1揮動%2了！','','','火焰居然從劍尖竄出了！','業火之劍','','','產生火焰','在南方地區的寶箱裡沉睡著。',''];
_tr_w["han"][Number('8') || 0]=['25','斬撃 / 雷','','','30','7500','舉世知名的名劍。\n人稱雷鎚，可以斬斷一切。','%1揮動%2了！','','','雷電居然從劍尖竄出了！','王者之劍','','','產生雷電','在寶箱裡沉睡著。',''];
_tr_w["han"][Number('9') || 0]=['60','斬撃','','','999','19999','武器之神創造的黃金劍。\n是可以斬斷萬物的最強武器。','%1揮動%2了！','','','','暴強‧草叢生劍','','','','對不起，就連我也不清楚……',''];
_tr_w["han"][Number('10') || 0]=['15','斬撃','直線２','','20','1700','巨大又銳利的鐵製大劍。\n出乎意料地輕，因此能快速揮動。','%1揮動%2了！','','','','巨劍','','','','在北方地區的寶箱裡沉睡著。','2個方格直線'];
_tr_w["han"][Number('11') || 0]=['27','斬撃','直線２','','20','3600','古代巨人族愛用的大劍。\n只要揮動一下，甚至能劈開大陸。','%1揮動%2了！','','','','巨人的大劍','','','','某處的便利商店正在販售。','2個方格直線'];
_tr_w["han"][Number('12') || 0]=['12','斬撃','','','30','1200','為了暗殺而製作的短劍。\n想要刺進致命處需要高超的技術。','%1把%2刺了出去！','','','','刺客匕首','','','致命（10%）','在村中某處人家的寶箱裡沉睡著。',''];
_tr_w["han"][Number('13') || 0]=['14','斬撃 / 闇','','2','30','4666','遭到惡魔詛咒的短劍。\n利用敏捷的動作，瞬間將敵人碎屍萬段。','%1把%2刺了出去！','','','','惡魔的短刀','','','連擊2次,中毒（30%）','在東北方沿海地區的寶箱裡沉睡著。',''];
_tr_w["han"][Number('14') || 0]=['16','打撃','','','20','1500','單純是為了破壞東西而製造出來的鎚子。\n不能小看它的破壞力。','%1揮動%2了！','','','','衝擊鎚','','','混亂（20%）','在村子東北方的寶箱裡沉睡著。',''];
_tr_w["han"][Number('15') || 0]=['28','打撃','','','20','8000','鎚頭像星星一樣的鎚矛。\n不要拿來按摩喔。','%1揮動%2了！','','','','晨星鎚','','','混亂（20%）','東方地區的路邊攤正在販售。',''];
_tr_w["han"][Number('16') || 0]=['13','斬撃 / 氷','直線３','','30','1800','以海神武器聞名的長槍。\n有著三叉的矛可以貫穿萬物。','%1使用%2刺穿了！','','','','三叉戟','','','','在充滿毒沼的洞窟的寶箱裡沉睡著。','3個方格直線'];
_tr_w["han"][Number('17') || 0]=['26','斬撃 / 炎','直線３','','30','7600','來自地獄的長槍。\n會釋放地獄火焰，貫穿並燒盡一切。','%1使用%2刺穿了！','','','','地獄魔槍','','','','東方地區的路邊攤正在販售。','3個方格直線'];
_tr_w["han"][Number('18') || 0]=['15','斬撃','','','20','2000','沉重而銳利的鋼鐵斧頭。\n砍在鎧甲上也能造成威力強大的傷害。','%1揮動%2了！','','','','鋼鐵斧頭','','','降低防禦力（30%）','在東北地區的寶箱裡沉睡著。',''];
_tr_w["han"][Number('19') || 0]=['28','斬撃','','','20','9500','天下無雙的斧頭，沒有砍不斷的東西。\n傳說是鬼神之王曾經使用的斧頭。','%1揮動%2了！','','','','巨人斧頭','','','降低防禦力（30%）','在居住於某個小屋裡的男人手上。',''];
_tr_w["han"][Number('20') || 0]=['8','打撃 / 炎','','','20','2100','魔導士自古以來愛用的魔杖。\n會噴出火球，燒盡一切。','%1揮動%2了！','','','魔杖頂端噴出火焰了！','魔導士之杖','','','產生【火焰魔法】','在鐵所形成的洞窟的寶箱裡沉睡著。',''];
_tr_w["han"][Number('21') || 0]=['8','打撃 / 雷','','','20','2500','隱藏著雷神之力的魔杖。\n只要一揮動，雷電之鎚便會從天而降。','%1揮動%2了！','','','魔杖頂端釋放出雷電了！','雷鎚之杖','','','產生【雷電魔法】','實現村民的願望便能獲得。',''];
_tr_w["han"][Number('22') || 0]=['8','打撃 / 氷','','','20','2600','據說是傳說中的冰魔女擁有的魔杖。\n會呼喚暴風雪，凍結全世界。','%1揮動%2了！','','','魔杖頂端出現冰塊了！','冰魔女之杖','','','產生【寒冰魔法】','在充滿魔力的洞窟的寶箱裡沉睡著。',''];
_tr_w["han"][Number('23') || 0]=['18','打撃 / 闇','','','20','6666','住在地獄的大惡魔所使用的魔杖。\n會喚醒黑暗的力量。','%1揮動%2了！','','','魔杖頂端釋放出黑暗的力量了！','大惡魔之杖','','','產生【黑暗魔法‧極】','在魔族之王居住的城堡的寶箱裡沉睡著。',''];
_tr_w["han"][Number('24') || 0]=['18','打撃 / 光','','','20','9999','過去大賢者為了封印黑暗而使用的魔杖。\n裡面封入了神聖的力量。','%1揮動%2了！','','','魔杖頂端釋放出神聖的力量了！','大賢者之杖','','','隨機產生【極】攻擊魔法','在居住於某地的魔女手上。',''];
_tr_w["han"][Number('25') || 0]=['20','斬撃','','','20','1500','用來挖掘牆壁的道具。\n非常足以當作武器使用。','%1揮動%2了！','','','','冰鎬','','','降低防禦力（50%）','掉落在西南地區。',''];
_tr_w["han"][Number('26') || 0]=['17','斬撃','','','20','1300','用來挖洞的道具，\n也能當作武器使用。','%1揮動%2了！','%2直接命中%3了！','順便在%3的腳邊挖了個洞！','','鐵鍬','','','1回合不能行動（30%）','掉落在南方地區。',''];
_tr_w["han"][Number('27') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('28') || 0]=['7','打撃','前３','','3','300','在各地生長的樹。\n因為濫伐而破壞森林是一個大問題。','%1揮動%2了！','%2直接命中%3的頭了！','','','樹','','','','到處都長了很多。','3個方格正面'];
_tr_w["han"][Number('29') || 0]=['6','打撃','前６','4','3','1000','一次撿起四棵樹而得到的武器。\n有種比平常划算四倍的感覺。','%1同時揮動四棵樹了！','樹直接命中%3的頭了！','','','樹×4','','','連擊4次','到處都長了很多。','6個方格正面'];
_tr_w["han"][Number('30') || 0]=['3','打撃','','','3','50','原本生長在各處的小型枯樹。\n生物注定總有一天會枯朽。','%1試著揮動%2了！','%2刷的一聲，擊中%3了！','','','小枯樹','','','HP一定會剩下1','在東南地區生長了很多。',''];
_tr_w["han"][Number('31') || 0]=['5','打撃','直線２','','3','200','乾枯凋零，徒留空虛的樹。\n令人有種悲傷的感覺。','%1試著揮動%2了！','%2刷的一聲，擊中%3了！','','','枯樹','','','HP一定會剩下1','在東南地區生長了很多。','2個方格直線'];
_tr_w["han"][Number('32') || 0]=['9','打撃','','','5','200','拿來當椅子正好的樹墩。\n光是看著就讓人想坐下去。','%1試著拿起%2揮動了！','%2直接命中%3的頭了！','%3坐在%2上休息了！','','樹墩','','','1回合不能行動（75%）','生長在村裡的某處。',''];
_tr_w["han"][Number('33') || 0]=['8','打撃','','','2','100','倒掉枯萎的樹。\n現在已是森林的裝飾品。','%1揮動%2了！','%2直接命中%3的頭了！','%3沒有站穩，倒在地上了！','','倒下的樹','','','1回合不能行動（90%）','掉落在村裡的某處。',''];
_tr_w["han"][Number('34') || 0]=['10','打撃','直線２','','6','700','生長在南國的樹。\n看著它就會感受到南國風情。','%1爽朗地揮動%2了！','%2直接命中%3的頭了！','','','椰子樹','','','','生長在南方沿海地區。','2個方格直線'];
_tr_w["han"][Number('35') || 0]=['4','斬撃','','','3','100','顏色和樣子都很傳統的草。\n另外還存在著B和C。','%1試著拿起%2揮動了！','%2割傷%3的左手食指了！','','','草A','','','','到處都長了很多。',''];
_tr_w["han"][Number('36') || 0]=['5','斬撃','','','3','100','比草A大了一圈的草。\n因為比較大，有一點划算的感覺。','%1試著拿起%2揮動了！','%2割傷%3的左手無名指了！','','','草B','','','','到處都長了很多。',''];
_tr_w["han"][Number('37') || 0]=['4','斬撃','','3','3','150','比草A、B大上許多的草。\n在草裡面是長男般的存在。','%1試著拿起%2揮動了！','%2割傷%3的右手大姆指、食指和無名指了！','','','草C','','','連擊3次','到處都長了很多。',''];
_tr_w["han"][Number('38') || 0]=['2','斬撃','','','2','50','枯萎而結束一生的草。\n拿來當武器有些靠不住。','%1試著拿起%2揮動了！','%2沙的一聲，擊中%3了！','','','枯草','','','HP一定會剩下1','在東南地區生長了很多。',''];
_tr_w["han"][Number('39') || 0]=['8','斬撃','','','3','200','純白而可憐的鮮花。\n看著它，心靈彷彿被洗滌了。','%1試著拿起%2揮動了！','花瓣割傷%3的左手小指了！','%3受%2撫慰，失去戰意了！','','白花','','','降低攻擊力（50%）','到處都長了很多。',''];
_tr_w["han"][Number('40') || 0]=['9','斬撃','','','3','250','火紅又熱情的花朵。\n可愛到讓人喪失戰意。','%1試著拿起%2揮動了！','花瓣割傷%3的右手中指了！','%3受%2撫慰，失去戰意了！','','紅花','','','降低攻擊力（40%）','到處都長了很多。',''];
_tr_w["han"][Number('41') || 0]=['6','斬撃','','','3','400','品種不明，但很漂亮的花。\n光是看著它，血壓就會下降喔。','%1試著拿起%2揮動了！','花瓣割傷%3的左手中指了！','%3受%2撫慰，失去戰意了！','','漂亮的花','','','降低攻擊力（75%）','生長在村子和森林裡。',''];
_tr_w["han"][Number('42') || 0]=['20','打撃','','','6','900','散發出可怕惡臭的花朵。\n如果可以，並不想撿起來。','%1捏著鼻子揮動%2了！','%2直接命中%3的鼻孔旁邊了！','%3似乎因為太臭而覺得不舒服！','','臭花','','','中毒（100%）','生長在西方的森林。',''];
_tr_w["han"][Number('43') || 0]=['6','打撃','前６','3','4','900','怎麼看都像是會跳出那個的草地。\n我就不要說得更深入了。','%1揮動%2了！','無數野生寶◯夢從%2裡跳出並襲擊%3！','','','草地','','','連擊3次','到處都長了很多。','6個方格正面'];
_tr_w["han"][Number('44') || 0]=['10','斬撃','前９','3','4','1800','仔細一看，形狀很像一把劍的草地。\n感覺好像也滿鋒利的。','%1揮動%2了！','用形狀像劍的草地斬殺%3了！','','','好像很強的草地','','','連擊3次','離開村子後往西走，它就生長在那裡。','9個方格正面'];
_tr_w["han"][Number('45') || 0]=['7','打撃','前６','3','4','1500','好像會有稀有生物跳出來的草叢。\n比如異色的……哎呀，好像有人來了。','%1揮動%2了！','無數異色寶◯夢從%2裡跳出並襲擊%3！','','','黃金草地','','','連擊3次','離開村子後往東走，它就生長在那裡。','6個方格正面'];
_tr_w["han"][Number('46') || 0]=['7','打撃','','','5','300','修剪得很漂亮的草叢。\n即使拿來打人也不太痛。','%1試著拿起%2揮動了！','%2嘩啦一聲，擊中%3了！','','','草叢','','','','裝飾在民宅附近。',''];
_tr_w["han"][Number('47') || 0]=['7','打撃','','','5','600','開滿漂亮花朵的時髦花叢。\n光是擺在那裡就能美化街道。','%1試著拿起%2揮動了！','%2嘩啦一聲，擊中%3了！','','','花叢','','','','裝飾在妳家附近。',''];
_tr_w["han"][Number('48') || 0]=['6','斬撃','','3','4','200','展現秋天樣貌的落葉地毯。\n總覺得隨風飄零的樣子很浪漫。','%1試著拿起%2揮動了！','許許多多的%2劃傷%3了！','','','落葉','','','連擊3次','離開村子後往東走，那裡地上很多。',''];
_tr_w["han"][Number('49') || 0]=['5','打撃','','','5','500','閃著金黃色光芒的芒草。\n隨風搖擺的模樣很美。','%1不斷搖晃%2！','%2在%3的腰側搔癢！','%3大笑過頭，彷彿要失去意識！','','芒草','','','混亂（100%）','離開村子後往東走，它就生長在那裡。',''];
_tr_w["han"][Number('50') || 0]=['6','打撃','','','6','400','別看它這樣，它可是毒蘑菇。\n蘑菇不可貌相，海水不可斗量。','%1試著拿起%2揮動了！','%2啪一聲地擊中%3的臉了！','%2碎片掉進%3的嘴裡了！','','蘑菇','','','中毒（75%）','生長在西方的森林。',''];
_tr_w["han"][Number('51') || 0]=['8','打撃','','','6','300','看上去有些那個的蘑菇。\n內含豐富的劇毒，很好吃。','%1試著拿起%2揮動了！','%2啪一聲地擊中%3的臉了！','%2碎片掉進%3的嘴裡了！','','看似不妙的蘑菇','','','劇毒（50%）','生長在西方的森林。',''];
_tr_w["han"][Number('52') || 0]=['14','打撃','直線３','','6','700','攀附牆壁生長的植物。\n好像可以當成鞭子一樣使用。','%1揮動%2了！','%2像鞭子一樣彎曲後直接命中%3了！','','','爬牆虎','','','','生長在南方地區的懸崖。','3個方格直線'];
_tr_w["han"][Number('53') || 0]=['12','斬撃','直線３','','6','800','纏繞其他東西而成長的植物。\n因為很長，可以攻擊到很遠的地方。','%1揮動%2了！','%2迅猛地竄出，刺中%3了！','','','藤蔓','','','','生長在南方地區的懸崖。','3個方格直線'];
_tr_w["han"][Number('54') || 0]=['16','斬撃','','','5','1200','生長在沙漠地帶的帶刺植物。\n被刺給刺到還滿痛的。','%1忍住痛楚，揮動%2了！','%2的刺深深刺進%3的全身了！','','','仙人掌','','','','生長在西南地區。',''];
_tr_w["han"][Number('55') || 0]=['15','打撃','','','4','600','品種不明的怪草，\n成分也是一團謎。','%1試著拿起%2揮動了！','%2割傷%3的左手小指了！','%2內含的神祕成分對%3奏效了！','','怪草','','','降低防禦力（100%）','生長在西方的森林。',''];
_tr_w["han"][Number('56') || 0]=['13','斬撃','','','4','300','經常長在牆上的草。\n其實含有催眠成分。','%1試著拿起%2揮動了！','%2割傷%3的右手背了！','%2居然含有催眠成分！','','長在牆上的草','','','睡眠（25%）','在鐵形成的洞窟裡。',''];
_tr_w["han"][Number('57') || 0]=['10','打撃','','','5','200','怎麼看都很詭異的植物幼苗。\n根部似乎有神奇的力量。','%1試著拿起%2揮動了！','%2反覆刺中%3了！','%2居然吸收%3的能力了！','','詭異的幼苗','','','複製能力變化','生長在充滿毒沼的洞窟。',''];
_tr_w["han"][Number('58') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('59') || 0]=['3','打撃','','','3','50','掉在地上的土石。\n跑進眼睛裡會很痛。','%1試著握住並揮動%2了！','%2掉在%3的頭上了！','土跑進%3的眼睛裡了！','','土','','','降低命中率（100%）','掉落在各處。',''];
_tr_w["han"][Number('60') || 0]=['6','打撃','','','4','200','土壤堆積在一起形成了一座小山。\n從頭上掉下來會在精神上覺得很難受。','%1揮動%2了！','%2掉在%3的頭上了！','土跑進%3的眼睛裡了！','','土山','','','降低命中率（50%）','在西南地區。',''];
_tr_w["han"][Number('61') || 0]=['14','打撃','前６','','5','400','大量土壤堆積而成的山。\n積少成多，積土成山。','%1揮動%2了！','%2掉在%3的頭上了！','土跑進%3的眼睛裡了！','','大土山','','','降低命中率（50%）','在西南地區。','6個方格正面'];
_tr_w["han"][Number('62') || 0]=['11','打撃','','','3','300','特徵是茶色的石頭A。\n其他還有石頭B和石頭C。','%1揮動%2了！','%2直接命中%3了！','','','石頭A','','','','在西南地區地上很多。',''];
_tr_w["han"][Number('63') || 0]=['12','打撃','','','3','400','特徵是黑色且外形好看的石頭B。\n拿著突出的部分會很好揮動。','%1揮動%2了！','%2直接命中%3了！','','','石頭B','','','','在西南地區地上很多。',''];
_tr_w["han"][Number('64') || 0]=['15','打撃','','','3','500','特徵是白色且堅硬的石頭C。\n號稱是A～C之中最硬的石頭。','%1揮動%2了！','%2直接命中%3了！','','','石頭C','','','','在西南地區地上很多。',''];
_tr_w["han"][Number('65') || 0]=['12','打撃','直線２','','3','700','屹立在地，威風凜凜的巨石。\n硬度意外地比看上去的還要軟。','%1揮動%2了！','%2直接命中%3了！','','','巨石','','','','在西南地區地上很多。','2個方格直線'];
_tr_w["han"][Number('66') || 0]=['14','打撃','','','3','200','很鮮明的綠色石頭。\n除了顏色外，和一般的石頭沒兩樣。','%1揮動%2了！','%2直接命中%3了！','','','綠石','','','','在充滿毒沼的洞窟裡地上很多。',''];
_tr_w["han"][Number('67') || 0]=['14','打撃','直線２','','3','600','綠石的巨大版本。\n以揮動的方式做使用。','%1揮動%2了！','%2直接命中%3了！','','','綠巨石','','','','在充滿毒沼的洞窟裡地上很多。','2個方格直線'];
_tr_w["han"][Number('68') || 0]=['11','打撃','','','1','100','腐壞並覆滿苔蘚的石頭。\n碰到它就覺得自己好像也要腐壞了。','%1揮動%2了！','%2直接命中%3了！','%3的防禦也腐壞了！','','腐壞的石頭','','','降低防禦力（50%）','在東北地區地上很多。',''];
_tr_w["han"][Number('69') || 0]=['7','打撃','','','3','50','石頭碎裂後掉在地上的東西。\n很脆弱，難以將它拿來當作武器使用。','%1試著拿起%2揮動了！','%2直接命中%3了！','','','石頭碎片','','','','在西南地區地上很多。',''];
_tr_w["han"][Number('70') || 0]=['13','斬撃','','','3','700','綻放美麗光輝的神祕水晶。\n用突出的部分刺人會很痛。','%1揮動%2了！','%2突出的部分刺到%3了！','','%2居然釋放出魔法了！','水晶','','','隨機產生攻擊魔法','在充滿魔力的洞窟裡地上很多。',''];
_tr_w["han"][Number('71') || 0]=['15','斬撃','直線２','','3','1200','綻放出耀眼光芒的巨大水晶。\n觸碰後可以感覺到魔力。','%1揮動%2了！','%2突出的部分刺到%3了！','','%2居然釋放出魔法了！','巨大水晶','','','隨機產生攻擊魔法','在充滿魔力的洞窟裡地上很多。','2個方格直線'];
_tr_w["han"][Number('72') || 0]=['6','斬撃','','','3','300','自水晶上切下的東西。\n裡面似乎沒有殘留魔力。','%1試著拿起%2揮動了！','%2突出的部分刺到%3了！','','','水晶碎片','','','','在充滿魔力的洞窟裡地上很多。',''];
_tr_w["han"][Number('73') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('74') || 0]=['13','打撃','前９','','3','800','高聳在晴空下的懸崖。\n揮舞這個打人還滿痛的。','%1揮動%2了！','%2直接狠狠地擊中%3了！','','','懸崖','','','','在村子外面地上很多。','9個方格正面'];
_tr_w["han"][Number('75') || 0]=['9','なし','','','7','200','經常在房間角落形成的蜘蛛網。\n黏性很強，一旦黏在身上就很麻煩。','%1嫌棄地拿起%2並試著揮動了。','%2黏在%3的臉上了！','%2還黏在%3的腳上了！','','蜘蛛網','','','降低行動次數（60%）','在靈魂徬徨的洞窟裡。',''];
_tr_w["han"][Number('76') || 0]=['5','なし','','','4','400','地面上的裂縫。\n被它打到的人好像也會裂開。','%1試著拿起%2揮動了！','%2擊中%3了！','%3裂開了！','','裂縫','','','降低防禦力（100%）','生長在西南地區。',''];
_tr_w["han"][Number('77') || 0]=['23','打撃','','','5','600','某個人挖的陷阱，非常深。\n用來困住敵人很方便。','%1試著拿起%2揮動了！','%2擊中%3了！','%3徹底掉進%2了！','','陷阱','','','2回合不能行動（25%）','在村裡的某處。',''];
_tr_w["han"][Number('78') || 0]=['28','打撃','前９','','5','9500','深不見底的巨大坑洞。\n一旦掉下去就回不來了。','%1試著拿起%2揮動了！','%2擊中%3了！','%2把%3吸進去了！','','巨大坑洞','','','3回合不能行動（20%）','在鐵形成的洞窟裡。','9個方格正面'];
_tr_w["han"][Number('79') || 0]=['14','なし','前６','','5','2000','住著稻田魚和青蛙的小水塘。\n揮動後會造成氾濫。','%1試著拿起%2揮動了！','','','池水氾濫了！','小池塘','','','產生水,降低雷耐性（100%）','到處都有。','6個方格正面'];
_tr_w["han"][Number('80') || 0]=['20','なし','前６','','5','2500','漂浮著荷葉，頗為風雅的池塘。\n因為多了荷花，威力比普通池塘高。','%1試著拿起%2揮動了！','','','池水氾濫了！','荷塘','','','產生水,降低雷耐性（100%）','離開村子後往南走，它就在那裡。','6個方格正面'];
_tr_w["han"][Number('81') || 0]=['16','なし','前６','','5','1400','危險的毒沼，積蓄了含有劇毒的液體。\n揮動後周圍會發生慘案。','%1試著拿起%2揮動了！','','','毒水氾濫了！','毒沼','','','產生毒水,中毒（80%）','在東南地區有很多。','6個方格正面'];
_tr_w["han"][Number('82') || 0]=['45','なし','前９','','6','19999','占地球表面積70%的生命之源。\n拿起來揮動會引發海嘯喔。','%1試著拿起%2揮動了！','','','居然發生海嘯了！','海','','','產生海嘯,降低雷耐性（100%）','在西北方岬角可以一覽無遺。','9個方格正面'];
_tr_w["han"][Number('83') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('84') || 0]=['16','打撃','','','6','800','傳播資訊給觀看者的看板。\n用看板的角打人挺痛的。','%1揮動%2了！','%2的一角擊中%3了！','','','看板','','','','在村裡的某處。',''];
_tr_w["han"][Number('85') || 0]=['24','斬撃','','','6','1000','特化為指路用途的看板。\n非常難看懂，備受批評。','%1揮動%2了！','%2突出的部分劈開%3了！','%3因為很難看懂的路標而困惑不已！','','路標看板','','','混亂（50%）','在村裡的某處。',''];
_tr_w["han"][Number('86') || 0]=['18','打撃','','','7','500','特化為生火用途的木棒。\n用粗木柴攻擊人還滿痛的。','%1揮動%2了！','比較粗的%2擊中%3了！','%3變成像木柴一樣易燃的體質了！','','木柴','','','降低火耐性（100%）','掉落在村裡的某處。',''];
_tr_w["han"][Number('87') || 0]=['28','打撃','','','5','1000','經常變成歐巴桑的聚會地點。\n一旦掉到裡面就很難出來。','%1揮動%2了！','%2的牆面擊中%3了！','%3掉到水井裡面了！','','水井','','','1回合不能行動（25%）','在村裡的某處。',''];
_tr_w["han"][Number('88') || 0]=['17','打撃 / 炎','直線３','','5','2000','不斷用溫和的燈光照亮街道的街燈。\n攻擊距離很長，當作武器使用也很優秀。','%1揮動%2了！','%2發著亮光，直接擊中%3了！','','','街燈','','','','在南方地區。','3個方格直線'];
_tr_w["han"][Number('89') || 0]=['16','打撃','','','5','1200','保護田地不受烏鴉侵擾的田地英雄。\n總是沒有表情，人類看了也會害怕。','%1揮動%2了！','%2面無表情地直接命中%3了！','%3在害怕%2！','','稻草人','','','1回合不能行動（40%）','在東方地區。',''];
_tr_w["han"][Number('90') || 0]=['15','打撃','','','5','400','在家門前放置的時尚玩意。\n因為是石製的，揍起人來會痛喔。','%1試著拿起%2揮動了！','%2比較尖銳的部分擊中%3了！','','','門前小徑','','','','掉落在村裡的某處。',''];
_tr_w["han"][Number('91') || 0]=['14','打撃','','','4','200','沒有放任何東西的木桶。\n敲擊後會發出好聽的聲音。','%1揮動%2了！','%2咚的一聲，撞上%3了！','','','空桶','','','','掉落在村裡的某處。',''];
_tr_w["han"][Number('92') || 0]=['14','打撃','','','5','500','裝著水的桶子。\n可以用來潑水喔。','%1揮動%2了！','%2咚的一聲，撞上%3了！','','水從%2裡溢出來了！','水桶','','','產生水,降低雷耐性（100%）','掉落在村裡的某處。',''];
_tr_w["han"][Number('93') || 0]=['12','打撃','','','5','800','衣服似乎才洗到一半。\n只要揮動它，就能噴出洗濯劑來攻擊。','%1揮動%2了！','%2直接命中%3了！','','洗濯劑從%2裡噴出來了！','洗衣套組','','','產生洗濯劑,降低冰耐性（100%）','離開村子後往南走，它就掉在那裡。',''];
_tr_w["han"][Number('94') || 0]=['7','打撃','','','4','100','就只是塊木板。\n只想得到拿去打人的使用方法。','%1揮動%2了！','%2直接命中%3了！','','','木板','','','','掉落在西南地區。',''];
_tr_w["han"][Number('95') || 0]=['13','打撃','直線２','','3','700','一直支撐著建築物的幕後功臣。\n形狀正好適合拿來揮舞。','%1揮動%2了！','%2直接命中%3了！','','','柱子','','','','生長在東北地區。','2個方格直線'];
_tr_w["han"][Number('96') || 0]=['9','斬撃','','','3','600','斷掉的柱子。\n斷面猶如鋸齒。','%1揮動%2了！','%2因斷裂而呈鋸齒狀的一面刺中%3了！','','','斷柱','','','','生長在東北地區。',''];
_tr_w["han"][Number('97') || 0]=['13','打撃','直線２','','3','400','腐爛並長出苔蘚的柱子。看得出在製作\n完成後，經歷了很長的歲月。','%1揮動%2了！','%2直接命中%3了！','%3的防禦也腐壞了！','','腐爛的柱子','','','降低防禦力（50%）','生長在東北地區。','2個方格直線'];
_tr_w["han"][Number('98') || 0]=['9','斬撃','','','3','200','不但腐爛還斷掉的柱子，\n徹底完成了柱子的使命。','%1揮動%2了！','%2因斷裂而呈鋸齒狀的一面刺中%3了！','%3的防禦也腐壞了！','','腐爛的斷柱','','','降低防禦力（50%）','生長在東北地區。',''];
_tr_w["han"][Number('99') || 0]=['12','打撃','','','6','2000','好像寫著很重要的事情的石碑。\n但是要解讀文字非常困難。','%1揮動%2了！','%2直接命中%3了！','%3為了解讀石碑文字而拼了老命！','','石碑','','','混亂（70%）','在有洞窟的地方。',''];
_tr_w["han"][Number('100') || 0]=['13','斬撃','','','5','900','為祭祀死者而造的簡易墳墓。\n粗暴對待它會遭天譴喔。','%1揮動%2了！','%2的尖端刺中%3了！','粗暴對待墳墓的天譴降在%3身上了……','','木墓','','','致命（5%）','生長在村裡的某處。',''];
_tr_w["han"][Number('101') || 0]=['17','打撃','','','5','2400','為祭祀死者而造的石造墳墓。\n粗暴對待它會遭天譴喔。','%1揮動%2了！','%2直接猛烈命中%3了！','粗暴對待墳墓的天譴降在%3身上了……','','石墓','','','致命（10%）','在靈魂徬徨的洞窟裡。',''];
_tr_w["han"][Number('102') || 0]=['12','打撃','直線２','3','6','3200','可以買到各種飲料的便利機器。\n不知道為什麼只有「熱飲」。','%1揮動%2了！','熱呼呼的飲料淋在%3的頭上了！','','','自動販賣機','','','連擊3次','──','2個方格直線'];
_tr_w["han"][Number('103') || 0]=['13','打撃','','','6','800','用來分類垃圾的箱子。\n不可以揮動它，害垃圾掉得到處都是。','%1揮動%2了！','%2掉出一堆垃圾，同時直接命中%3了！','%3直接吸進了垃圾散發的臭味。','','垃圾桶','','','中毒（100%）','在某處的便利商店附近。',''];
_tr_w["han"][Number('104') || 0]=['14','打撃','','','7','1100','手工製作，非常堅固的石灶。\n添加了火種，可以輕易起火。','%1揮動%2了！','%2直接命中%3了！','火種黏在%3身上了！','','石灶','','','降低火耐性（100%）','掉落在南方地區。',''];
_tr_w["han"][Number('105') || 0]=['25','炎','','','6','2200','熊熊燃燒的火焰。\n撿起來時嚴重燒傷了。','%1忍住燒傷，揮動%2了！','%3像營火一樣熊熊燃燒起來了！','','','火','','','','掉落在南方地區。',''];
_tr_w["han"][Number('106') || 0]=['30','打撃','直線４','','7','8500','飄浮在空中，偶而會旋轉的神奇鐵板。\n好像可以利用它的旋轉當成武器。','%1揮動%2了！','%2在旋轉的同時直接命中%3了！','','','神奇的鐵板','','','','在鐵形成的洞窟裡。','4個方格直線'];
_tr_w["han"][Number('107') || 0]=['25','打撃','','','7','4000','每次操作好像都有東西在旋轉，\n是個神奇的按鈕。意外地硬。','%1揮動%2了！','%2在切換的同時直接命中%3了！','','','神奇的按鈕','','','','在鐵形成的洞窟裡。',''];
_tr_w["han"][Number('108') || 0]=['30','打撃','直線２','','7','3800','每次走過去都會改變方向的神奇單行橋。\n也可以拿來當武器使用。','%1揮動%2了！','%2在改變方向的同時直接命中%3了！','','','神奇的橋','','','','在充滿毒沼的洞窟裡。','2個方格直線'];
_tr_w["han"][Number('109') || 0]=['28','打撃','前９','','7','6000','給予刺激就會發光的神奇面板。\n利用這個設計的謎題相當難解開。','%1揮動%2了！','%2直接命中%3了！','%3不知道正確的路線！','','神奇的面板','','','混亂（25%）','在充滿魔力的洞窟裡。','9個方格正面'];
_tr_w["han"][Number('110') || 0]=['32','打撃 / 炎','','','7','5500','感覺有種神祕氛圍的燈籠。\n只要點一次火，就會永遠亮著。','%1揮動%2了！','%2直接命中%3了！','','%2不知為何自燃了！','神奇的燈籠','','','產生火焰','在靈魂徬徨的洞窟裡。',''];
_tr_w["han"][Number('111') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('112') || 0]=['12','打撃','','','1','600','常見的陶壺，在每個家庭裡都會有。\n拿去打人會出現暴擊並碎掉。','%1揮動%2了！','%2狠狠砸中%3了！','','','壺','','','一定會暴擊','在每間房子裡都有。',''];
_tr_w["han"][Number('113') || 0]=['12','打撃','','','1','1200','用蓋子遮住內容物的可疑陶壺。\n裡面究竟有什麼……？','%1揮動%2了！','%2狠狠砸中%3了！','從壺裡噴出來的「馬糞」掉進%3的嘴裡了！','','有蓋子的壺','','','一定會暴擊,中毒（100%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('114') || 0]=['15','打撃','','','3','700','常見且到處都有的木桶。\n當武器使用時要拿去砸敵人。','%1揮動%2了！','%2狠狠砸中%3了！','','','木桶','','','','掉落在村裡的某處。',''];
_tr_w["han"][Number('115') || 0]=['11','斬撃','','','6','600','方形的小桌子。\n用來放些小東西很方便。','%1揮動%2了！','%2的一角刺中%3了！','','','小桌子','','','','在某處的小屋裡。',''];
_tr_w["han"][Number('116') || 0]=['8','打撃','','','6','600','圓圓又可愛的小桌子。\n沒有桌角，所以威力有點靠不住。','%1揮動%2了！','%2直接命中%3了！','','','圓桌','','','','在某處的小屋裡。',''];
_tr_w["han"][Number('117') || 0]=['13','斬撃','直線２','','6','900','大小剛好的方桌。\n使用時要用桌角狠狠地刺人。','%1揮動%2了！','%2的一角刺中%3了！','','','中型桌子','','','','在村中某處的房子裡。','2個方格直線'];
_tr_w["han"][Number('118') || 0]=['14','斬撃','前６','','6','2000','有好幾個人要坐在一起時很方便的桌子。\n能把這個當成武器使用才叫獨當一面。','%1揮動%2了！','%2的一角刺中%3了！','','','大型桌子','','','','在每間房子裡都有。','6個方格正面'];
_tr_w["han"][Number('119') || 0]=['13','斬撃','直線３','','6','1400','微妙地有點長的微妙的桌子。\n用桌角刺人會微妙地痛。','%1揮動%2了！','%2的一角刺中%3了！','','','有點長的桌子','','','','在村中某處的房子裡。','3個方格直線'];
_tr_w["han"][Number('120') || 0]=['15','斬撃','直線４','','6','2500','長得不得了的桌子。\n它的攻擊距離就武器看來也很優秀。','%1揮動%2了！','%2的一角刺中%3了！','','','長桌','','','','在村中某處的房子裡。','4個方格直線'];
_tr_w["han"][Number('121') || 0]=['12','斬撃','','','6','1100','與花瓶成套的桌子。\n看著就讓人覺得平靜。','%1揮動%2了！','%2的一角刺中%3了！','%3受花瓶撫慰，失去戰意了！','','花瓶桌','','','降低攻擊力（60%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('122') || 0]=['15','斬撃','','','6','1200','與鏡子成套的桌子。\n這面鏡子只會映照出真實。','%1揮動%2了！','%2的一角刺中%3了！','鏡子反射的光芒擊中%3了！','','鏡子桌','','','重置能力強化','在村中某處的房子裡。',''];
_tr_w["han"][Number('123') || 0]=['17','斬撃','直線２','','6','3600','怎麼看都像教會裡會有的桌子。\n聖經裡面似乎有神奇的力量。','%1揮動%2了！','%2的一角刺中%3了！','某種神聖的力量對%3起作用了！','','教會桌','','','降低火耐性（50%）,降低冰耐性（50%）,降低雷耐性（50%）','在某處的教會裡。','2個方格直線'];
_tr_w["han"][Number('124') || 0]=['17','斬撃','','','6','4000','與詭異水晶成套的桌子。\n水晶雖然詭異，卻感覺得到魔力。','%1揮動%2了！','%2的一角刺中%3了！','','水晶居然釋放出魔法了！','水晶桌','','','隨機產生攻擊魔法','在某處奇怪的房子裡。',''];
_tr_w["han"][Number('125') || 0]=['8','打撃','','','6','300','看起來很便宜，卻很好坐的圓椅。\n作為武器似乎不能有所期待。','%1揮動%2了！','%2直接命中%3了！','%3坐在%2上休息了！','','圓椅','','','1回合不能行動（50%）','在村中某處人家裡有很多。',''];
_tr_w["han"][Number('126') || 0]=['10','斬撃','','','6','300','四角的小椅子。\n有角，所以比圓椅強。','%1揮動%2了！','%2的一角刺中%3了！','%3坐在%2上休息了！','','四角椅','','','1回合不能行動（50%）','在村中某處人家裡有很多。',''];
_tr_w["han"][Number('127') || 0]=['12','斬撃','直線２','','6','1500','看起來很高級，感覺很不錯的椅子。\n坐起來也非常舒適。','%1揮動%2了！','%2的靠背一角刺中%3了！','%3坐在%2上放鬆地休息了！','','感覺不錯的椅子','','','1回合不能行動（80%）','在某處的小屋裡。','2個方格直線'];
_tr_w["han"][Number('128') || 0]=['8','打撃','直線２','3','6','1000','整理了各種書的書架。\n使用時要讓書掉下來砸敵人。','%1揮動%2了！','無數的書砸在%3的頭上了！','%3讀完各種書後變聰明了！','','書架','','','連擊3次,虛弱（30%）','在每間房子裡都有。','2個方格直線'];
_tr_w["han"][Number('129') || 0]=['30','斬撃','直線２','','6','1500','收納衣服的櫃子。\n左腳的小姆指撞到它會痛得要命。','%1揮動%2了！','%2的一角砸中%3左腳的小姆指了！','%3按住左腳小姆指，好像很痛苦！','','五斗櫃','','','降低行動次數（30%）','在村中某處的房子裡。','2個方格直線'];
_tr_w["han"][Number('130') || 0]=['9','打撃','直線２','3','6','1200','掛著衣服的櫃子。\n裡面其實非常不得了。','%1揮動%2了！','無數的衣服砸在%3的頭上了！','%3因為被太多衣服壓住，很難行動！','','衣櫃','','','連擊3次,降低回避率（100%）','在村中某處的房子裡。','2個方格直線'];
_tr_w["han"][Number('131') || 0]=['7','打撃','直線２','3','6','1400','不知為何放有麵包的櫃子。\n法國麵包很硬，所以能當作武器。','%1揮動%2了！','無數的法國麵包砸在%3的頭上了！','%3開始瘋狂地吃麵包！','','放了麵包的櫃子','','','連擊3次,1回合不能行動（50%）','在某處的小屋裡。','2個方格直線'];
_tr_w["han"][Number('132') || 0]=['8','打撃','直線２','3','6','900','放了莫名其妙的藥等物品的櫃子。\n總之揮動使用就萬事大吉。','%1揮動%2了！','無數莫名其妙的東西砸在%3的頭上了！','%3在害怕莫名其妙的東西！','','莫名其妙的櫃子','','','連擊3次,混亂（50%）','在某處的小屋裡。','2個方格直線'];
_tr_w["han"][Number('133') || 0]=['12','打撃','','','6','600','安裝在牆上放書的置物架。\n老實說在這個位置上沒有人拿得到書。','%1試著拿起%2揮動了！','%2直接命中%3了！','%3讀完書後變聰明了！','','放書的架子','','','虛弱（30%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('134') || 0]=['13','打撃','','','6','800','放了各種調味料，很方便的置物架。\n要小心調味料過期。','%1試著拿起%2揮動了！','%2直接命中%3了！','調味料的賞味期限居然是兩年前！','','調味料架','','','中毒（30%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('135') || 0]=['12','打撃','','','6','700','可愛花朵與花瓶的套組。\n美麗的花配上美麗的花瓶很重要。','%1試著拿起%2揮動了！','%2直接命中%3了！','%3受花撫慰，失去戰意了！','','花瓶','','','降低攻擊力（50%）','在某處的教會裡。',''];
_tr_w["han"][Number('136') || 0]=['14','打撃','','','5','400','隨便放進了一些東西的木箱。\n使用時正常地砸下去即可。','%1試著拿起%2揮動了！','%2直接命中%3了！','','','木箱','','','','在村中某處的房子裡。',''];
_tr_w["han"][Number('137') || 0]=['6','打撃','前６','5','3','1800','許多木箱層層疊起的高塔。\n使用時要讓它順利地在敵人頭上崩塌。','%1揮動%2了！','無數的木箱砸在%3的頭上了！','','','木箱塔','','','連擊5次','在某處的小屋裡。','6個方格正面'];
_tr_w["han"][Number('138') || 0]=['8','打撃','','','3','300','變得破破爛爛的木箱。\n其實很脆弱，不太能使用。','%1試著拿起%2揮動了！','%2直接命中%3了！','','','破爛木箱','','','','在某處的小屋裡。',''];
_tr_w["han"][Number('139') || 0]=['18','打撃','直線２','','5','700','跟人差不多大的大木箱。\n如果能揮動，就會展現出不錯的威力。','%1揮動%2了！','%2直接命中%3了！','','','大木箱','','','','在村裡的某處。','2個方格直線'];
_tr_w["han"][Number('140') || 0]=['12','打撃','','','5','500','塞得滿滿的米袋。\n揮動時不要忘了感謝農家。','%1試著拿起%2揮動了！','塞得滿滿的%2直接命中%3了！','','','米袋','','','','在村中某處的房子裡。',''];
_tr_w["han"][Number('141') || 0]=['9','打撃','直線２','3','5','1400','滿滿的米袋堆疊起來的高塔。\n使用時要在敵人的頭上引發崩塌。','%1揮動%2了！','無數的米袋砸在%3的頭上了！','','','米袋塔','','','連擊3次','在村中某處的房子裡。','2個方格直線'];
_tr_w["han"][Number('142') || 0]=['5','打撃','','5','6','1100','有許多看起來很好吃的蘋果，\n但究竟是……？','%1試著拿起%2揮動了！','%2砸在%3的頭上了！','居然全都是毒蘋果！','','許多蘋果','','','連擊5次,劇毒（60%）','在某處奇怪的房子裡。',''];
_tr_w["han"][Number('143') || 0]=['16','打撃','直線２','','5','1400','清理得很乾淨的流理臺。\n試著揮動後發現攻擊距離滿長的。','%1揮動%2了！','%2直接命中%3了！','','','流理臺','','','','在村中某處的房子裡。','2個方格直線'];
_tr_w["han"][Number('144') || 0]=['16','打撃','','','5','900','乾淨，沒有油污的瓦斯爐。\n會發出火焰，所以當作武器還滿強的。','%1揮動%2了！','%2直接命中%3了！','','用大火加熱到焦糖色吧！','瓦斯爐','','','產生火焰','在村中某處的房子裡。',''];
_tr_w["han"][Number('145') || 0]=['18','斬撃','','','6','900','有點髒的換氣扇。\n旋轉的扇葉好像能應用在武器上。','%1試著拿起%2揮動了！','%2旋轉時把%3捲進去了！','','','換氣扇','','','','在村中某處的房子裡。',''];
_tr_w["han"][Number('146') || 0]=['23','斬撃','','','5','800','東西切到一半的砧板與菜刀。\n菜刀還滿鋒利的。','%1揮動%2了！','把%3放在砧板上剁碎了！','','','砧板套組','','','降低斬擊耐性（100%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('147') || 0]=['11','打撃','','3','6','700','有平底鍋和杓子等各種廚具。\n當然也能當作武器使用。','%1揮動%2了！','用各種廚具揍%3了！','','','廚具套組','','','連擊3次','在村中某處的房子裡。',''];
_tr_w["han"][Number('148') || 0]=['20','打撃 / 氷','直線２','','6','1900','有一堆剩餘材料的冰箱。\n用冷氣凍結敵人的戰術有可能實現。','%1揮動%2了！','%2直接命中%3了！','','冰箱門打開後冷氣外洩了！','冰箱','','','產生冷氣','在村中某處的房子裡。','2個方格直線'];
_tr_w["han"][Number('149') || 0]=['12','打撃','','','6','700','將魚從頭到尾都烤熟的一道菜。\n有許多魚刺。','%1試著揮動%2了！※請愛惜食物！','%2猛烈地撞上%3了！','%3被魚刺噎到了！','','烤魚','','','封印魔法（50%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('150') || 0]=['14','打撃','','','6','600','簡單美味的炒菜。\n熱騰騰的，好像會燙傷舌頭。','%1試著揮動%2了！※請愛惜食物！','熱騰騰的青菜襲擊%3了！','','','炒青菜套餐','','','','在村中某處的房子裡。',''];
_tr_w["han"][Number('151') || 0]=['12','打撃','','','6','750','可以自由添飯的划算套餐。\n但總覺得豬肉的顏色有點紅。','%1試著揮動%2了！※請愛惜食物！','%2猛烈地撞上%3了！','%2居然沒有熟。%3拉肚子了！','','薑汁燒豬肉套餐','','','中毒（30%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('152') || 0]=['7','打撃','','','6','320','喝了它，下午就能繼續努力的飲料。\n咖啡因會趕跑瞌睡蟲。','%1用%2乾杯了！','%3：「乾杯～！！」','%3徹底清醒過來了！','','飯後咖啡','','','提升睡眠耐性（100%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('153') || 0]=['12','打撃','','','6','3000','特徵是味道濃郁的厚重紅酒。\n光是聞到香氣就會心情愉快。','%1用%2乾杯了！','%3：「乾杯～！！」','%3因為紅酒的香氣而醉了！','','高級紅酒','','','封印普通攻擊外的攻擊（50%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('154') || 0]=['21','打撃','','','6','50','像是一道料理又好像不是的某種東西。\n為什麼會端出這個給客人呢……','%1試著拿起%2揮動了！','%3把%2吸進嘴裡了！','%3的樣子明顯怪怪的！','','像是料理的東西','','','中毒（75%）,封印魔法（50%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('155') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('156') || 0]=['14','打撃','直線２','','6','2400','睡起來超級舒服的柔軟床鋪。\n會誘使敵人沉入睡眠。','%1揮動%2了！','%2波的一聲，擊中%3了！','%2把%3吸引過去了！','','床','','','睡眠（55%）','在村中某處的房子裡。','2個方格直線'];
_tr_w["han"][Number('157') || 0]=['18','打撃','前６','','6','7000','可以兩個人一起睡覺的雙人床。\n睡起來舒服極了。','%1揮動%2了！','%2波的一聲，擊中%3了！','%2把%3吸引過去了！','','雙人床','','','睡眠（90%）','在村中某處的房子裡。','6個方格正面'];
_tr_w["han"][Number('158') || 0]=['7','打撃','直線２','','4','300','很樸素，感覺很不好睡的棉被。\n隔天早上關節會痠痛。','%1揮動%2了！','%2啪的一聲，擊中%3了！','%3決定只有今晚忍耐著蓋%2睡覺！','','破舊棉被','','','睡眠（25%）','在某處的小屋裡。','2個方格直線'];
_tr_w["han"][Number('159') || 0]=['18','打撃','直線２','','6','1400','散發奢華氣氛的暖爐。\n要握住上面較細小的部分來揮動。','%1揮動%2了！','%2直接命中%3了！','','%2不知為何自燃了！','暖爐','','','產生火焰','在村中某處的房子裡。','2個方格直線'];
_tr_w["han"][Number('160') || 0]=['11','斬撃','','','5','400','某個人寫的日記。\n閱讀時會回想起自己的黑歷史。','%1試著拿起%2揮動了！','%2割傷%3的左手大姆指了！','%3回想起黑歷史了……','','日記','','','混亂（50%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('161') || 0]=['5','斬撃','','','4','200','寫了一些東西的紙。\n被邊緣割到手會有點痛。','%1試著拿起%2揮動了！','%2的邊緣割斷%3的左手背了！','','','某張紙','','','','在某處的小屋裡。',''];
_tr_w["han"][Number('162') || 0]=['13','打撃','','','4','500','不知道裝了什麼東西的袋子。\n除了拿來打人外沒有其他用法。','%1試著拿起%2揮動了！','%2直接命中%3了！還是不知道裡面裝了什麼東西。','','','某個袋子','','','','在村中某處的房子裡。',''];
_tr_w["han"][Number('163') || 0]=['27','打撃','前６','','6','3700','年代久遠，但擦得很乾淨的鋼琴。\n彈奏的同時拿來毆打敵人吧。','%1揮動%2了！','%2的鍵盤撞到%3，彈奏出不協和音了！','%3聽到不協和音後變得不舒服了！','','鋼琴','','','重置能力強化','在村中某處的房子裡。','6個方格正面'];
_tr_w["han"][Number('164') || 0]=['15','打撃','','','6','1000','包裝得很可愛的盒子。\n好像能從裡面感受到魔力。','%1揮動%2了！','%2直接命中%3了！','','%2裡面居然釋放出魔法了！','禮物盒','','','隨機產生攻擊魔法','在某處奇怪的房子裡。',''];
_tr_w["han"][Number('165') || 0]=['12','打撃','','','5','1500','魔女之家裡的衣服。\n撿起來時稍微有點罪惡感。','%1試著揮動%2了！散發出一股香味！','%2擊中%3的臉了！','','','魔女的衣服','','','','在某處奇怪的房子裡。',''];
_tr_w["han"][Number('166') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('167') || 0]=['11','打撃','前６','3','6','1400','便利商店裡擺放日用品等東西的商品架。\n我是撿起來當武器，所以不算偷竊。','%1揮動%2了！','各種商品砸在%3的頭上了！','商品居然全都是黑心貨！','','便利商店商品架','','','連擊3次,混亂（50%）','在某處的便利商店裡。','6個方格正面'];
_tr_w["han"][Number('168') || 0]=['10','打撃 / 氷','前６','3','6','2200','集合各種冷凍食品的專櫃。\n這股冷氣好像能當作武器使用。','%1揮動%2了！','冷凍食品砸在%3的頭上了！','','冷氣外洩了！','冷凍食品櫃','','','連擊3次,產生冷氣','在某處的便利商店裡。','6個方格正面'];
_tr_w["han"][Number('169') || 0]=['12','打撃','前６','3','6','1800','擺著各式各樣便當的專區。\n全部都貼著「半價」標籤。','%1揮動%2了！','各種便當砸在%3的頭上了！','便當居然全都過期了！','','便當賣場','','','連擊3次,中毒（50%）','在某處的便利商店裡。','6個方格正面'];
_tr_w["han"][Number('170') || 0]=['13','打撃','','','7','3000','我家裡很重要的電話機。\n要小心偽裝成親友的詐騙電話。','%1揮動%2了！','%2直接命中%3了！','%3接到偽裝成親友的詐騙電話，上當了！','','電話','','','混亂（100%）','在妳家裡。',''];
_tr_w["han"][Number('171') || 0]=['9','打撃','前９','','6','400','相當破舊的地毯，\n還散發出一點怪味。','%1揮動%2了！','%3被%2絆倒，臉用力地撞到地板上了！','%3被絆倒，行動變遲緩了！','','破舊地毯','','','降低回避率（75%）','在某處的小屋裡。','9個方格正面'];
_tr_w["han"][Number('172') || 0]=['14','打撃','前９','','6','800','可愛的藍色地毯。\n觸感很柔軟。','%1揮動%2了！','%3被%2絆倒，臉用力地撞到地板上了！','%3被絆倒，行動變遲緩了！','','藍色地毯','','','降低回避率（50%）','在村中某處的房子裡。','9個方格正面'];
_tr_w["han"][Number('173') || 0]=['18','打撃','前９','','6','2500','豪華的大紅色地毯。\n使用最頂級的布料製作而成。','%1揮動%2了！','%3被%2絆倒，臉用力地撞到地板上了！','%3被絆倒，行動變遲緩了！','','豪華的地毯','','','降低回避率（25%）','在某處的教會裡。','9個方格正面'];
_tr_w["han"][Number('174') || 0]=['13','打撃','前９','','6','1100','散發出神奇氛圍的地毯。\n但布料意外廉價。','%1揮動%2了！','%3被%2絆倒，臉用力地撞到地板上了！','%3被絆倒，行動變遲緩了！','','詭異的地毯','','','降低回避率（100%）','在某處奇怪的房子裡。','9個方格正面'];
_tr_w["han"][Number('175') || 0]=['16','打撃','','','5','800','不斷行進、顯示正確時間的時鐘。\n試著用這個揍人好像也很有趣。','%1試著拿起%2揮動了！','%2在時間一分一秒流逝的同時直接命中%3了！','%3周圍的時空居然產生變化了！','','時鐘','','','降低行動次數（50%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('176') || 0]=['21','斬撃','','','7','1200','可以掛在牆上的寶劍複製品。\n但是好像還滿鋒利的。','%1揮動%2了！','%2正常地劈開%3了！','','','掛在牆上的劍','','','','在村中某處的房子裡。',''];
_tr_w["han"][Number('177') || 0]=['17','打撃','','','7','900','可以掛在牆上的盾牌複製品。\n雖然是複製品，卻還滿堅硬的。','%1揮動%2了！','%2直接狠狠地擊中%3了！','','','掛在牆上的盾','','','降低防禦力（50%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('178') || 0]=['24','斬撃','','','7','1500','可以掛在牆上的斧頭複製品。\n非常重，就像是真正的斧頭一樣。','%1揮動%2了！','%2正常地劈開%3了！','','','掛在牆上的斧頭','','','','在某處的小屋裡。',''];
_tr_w["han"][Number('179') || 0]=['16','斬撃','前３','','6','600','手繪的地圖。\n是冒險的必需品。','%1試著拿起%2揮動了！','%2擊中%3了！','%3不擅長看地圖！','','地圖','','','混亂（75%）','在某處的小屋裡。','3個方格正面'];
_tr_w["han"][Number('180') || 0]=['14','斬撃','','','5','700','美麗花朵的畫。\n畫框很硬，可以當武器。','%1試著拿起%2揮動了！','畫框直接命中%3了！','%3受美麗花朵撫慰，失去戰意了！','','花的畫','','','降低攻擊力（60%）','在某處奇怪的房子裡。',''];
_tr_w["han"][Number('181') || 0]=['15','斬撃','','','5','800','陌生人的臉。你是誰呀？\n藝術真是難懂。','%1試著拿起%2揮動了！','畫框直接命中%3了！','%3開始思考何謂藝術了！','','肖像畫','','','混亂（40%）','在某處奇怪的房子裡。',''];
_tr_w["han"][Number('182') || 0]=['13','斬撃','直線２','','5','1000','像國旗一樣鮮紅的旗子。\n雖然不能期待威力，卻很帥氣。','%1揮動%2了！','%2在飄揚的同時擊中%3了！','','','紅旗','','','','在靈魂徬徨的洞窟裡。','2個方格直線'];
_tr_w["han"][Number('183') || 0]=['7','斬撃','直線２','','3','300','變得破破爛爛的旗子。\n它幾乎就只是一個垃圾。','%1揮動%2了！','%2在飄揚的同時擊中%3了！','','','破爛旗子','','','','在靈魂徬徨的洞窟裡。','2個方格直線'];
_tr_w["han"][Number('184') || 0]=['16','斬撃','前６','','5','900','幾乎叫人懷疑品味的鮮紅色窗簾。\n遮光性好像很高。','%1試著拿起%2揮動了！','%2輕飄飄地擊中%3了！','%2擋住%3的視線了！','','鮮紅色窗簾','','','降低命中率（90%）','在某處奇怪的房子裡。','6個方格正面'];
_tr_w["han"][Number('185') || 0]=['12','打撃','','','5','700','擦得很乾淨的窗戶。\n因為是雙重結構，出乎意料地堅固。','%1試著拿起%2揮動了！','%2直接命中%3了！','','','窗戶','','','','在村中某處的房子裡。',''];
_tr_w["han"][Number('186') || 0]=['12','打撃','','','5','900','附綠色窗簾的窗戶。\n可以用窗簾遮蔽敵人的視線。','%1試著拿起%2揮動了！','%2直接命中%3了！','窗簾擋住%3的視線了！','','綠窗簾窗戶','','','降低命中率（100%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('187') || 0]=['13','打撃','','','5','1000','附紅色窗簾，稍微大一點的窗戶。\n非常堅固，即使敲打它也不會破掉。','%1試著拿起%2揮動了！','%2直接命中%3了！','窗簾擋住%3的視線了！','','紅窗簾窗戶','','','降低命中率（100%）','在村中某處的房子裡。',''];
_tr_w["han"][Number('188') || 0]=['15','打撃','','','5','1600','畫著漂亮圖案的窗戶。\n說到教會就會想到這個。','%1試著拿起%2揮動了！','%2直接命中%3了！','','','彩繪玻璃窗','','','','在某處的教會裡。',''];
_tr_w["han"][Number('189') || 0]=['20','打撃','','','4','800','在安全性部份令人有點不安的門。\n當作武器則令人覺得可靠。','%1用力揮動%2了！','%2直接命中%3了！','','','門','','','','就在每間房子的入口處。',''];
_tr_w["han"][Number('190') || 0]=['16','打撃','前６','','6','1200','結實的鐵製牢房。\n揮動起來會是十分強大的武器。','%1揮動%2了！','%2直接命中%3了！','%3有種被關進牢房裡的感覺！','','牢房','','','1回合不能行動（30%）','在鐵形成的洞窟裡。','6個方格正面'];
_tr_w["han"][Number('191') || 0]=['17','打撃','直線２','','6','900','堅固的門，可以用牢房鑰匙打開。\n強度有保證。','%1揮動%2了！','%2直接命中%3了！','%3有種被關進牢房裡的感覺！','','牢房的門','','','1回合不能行動（20%）','在鐵形成的洞窟裡。','2個方格直線'];
_tr_w["han"][Number('192') || 0]=['28','打撃','直線２','','7','6000','神情勇猛的戰士雕像。\n手上的劍好像可以當成武器使用。','%1揮動%2了！','%2的劍劈開%3了！','','','戰士雕像','','','','掉落在東方地區。','2個方格直線'];
_tr_w["han"][Number('193') || 0]=['24','打撃','直線２','','7','4500','永遠會面向右邊的神奇女神像。\n利用神聖之力做得相當堅固。','%1揮動%2了！','%2給了%3一記頭槌！','某種神聖的力量對%3起作用了！','','朝右女神像','','','虛弱（15%）','在某處的教會裡。','2個方格直線'];
_tr_w["han"][Number('194') || 0]=['24','打撃','直線２','','7','4500','永遠會面向左邊的神奇女神像。\n用它施展頭槌好像會很厲害。','%1揮動%2了！','%2給了%3一記頭槌！','某種神聖的力量對%3起作用了！','','朝左女神像','','','封印魔法（50%）','在某處的教會裡。','2個方格直線'];
_tr_w["han"][Number('195') || 0]=['40','斬撃 / 炎','前３','','7','18000','仿照凶暴的龍打造出的雕像，魄力十足。\n裡面封印了龍的靈魂。','%1揮動%2了！','%2的牙齒刺中%3了！','','%2居然吐出火焰了！','龍雕像','','','產生火焰','在靈魂徬徨的洞窟裡。','3個方格正面'];
_tr_w["han"][Number('196') || 0]=['18','打撃','','','6','3000','裡面好像會有稀有道具的寶箱。\n作為武器似乎除了砸人外沒有其他用法。','%1試著拿起%2揮動了！','%2直接命中%3了！','','','寶箱','','','','掉落在各處。',''];
_tr_w["han"][Number('197') || 0]=['19','打撃','直線２','','4','1800','用黃金做成的柱子，閃亮到刺眼。\n直視它好像對眼睛不好。','%1揮動%2了！','%2在發光的同時直接命中%3了！','%3不小心直視黃金的光輝了！','','黃金柱子','','','降低命中率（50%）','在閃著金黃光芒的洞窟裡。','2個方格直線'];
_tr_w["han"][Number('198') || 0]=['15','斬撃','','','4','900','即使斷掉了也在發光的黃金柱子。\n雖然是黃金，強度卻無可挑剔。','%1揮動%2了！','%2因斷裂而呈鋸齒狀的一面刺中%3了！','%3不小心直視黃金的光輝了！','','黃金斷柱','','','降低命中率（30%）','在閃著金黃光芒的洞窟裡。',''];
_tr_w["han"][Number('199') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('200') || 0]=['20','打撃','','','7','500','可愛的紅色熊娃娃。\n是娃娃……對吧？','%1揮動%2了！','%2給了%3一記頭槌！','','','紅熊','Catherine','','','坐在村中某處人家裡。',''];
_tr_w["han"][Number('201') || 0]=['20','打撃','','','7','500','酷酷的藍色熊娃娃。\n是娃娃……對吧？','%1揮動%2了！','%2給了%3一記頭槌！','','','藍熊','James','','','坐在南方岬角。',''];
_tr_w["han"][Number('202') || 0]=['20','打撃','','','7','500','陰鬱的黑色熊娃娃。\n是娃娃……對吧？','%1揮動%2了！','%2給了%3一記頭槌！','','','黑熊','Tom','','','坐在東方地區。',''];
_tr_w["han"][Number('203') || 0]=['20','打撃','','','7','500','可愛的褐色熊娃娃。\n是娃娃……對吧？','%1揮動%2了！','%2給了%3一記頭槌！','','','褐熊','Brown','','','坐在西北地區。',''];
_tr_w["han"][Number('204') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('205') || 0]=['30','打撃','前９','','7','2500','長年生活在這的我家房子右半邊。\n快速揮動就能當成武器使用。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','自宅右','','','','由妳家的右半邊形成。','9個方格正面'];
_tr_w["han"][Number('206') || 0]=['30','打撃','前９','','7','3200','我家房子的左半邊，開武器店的地方。\n快速揮動就能當成武器使用。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','自宅左','','','','由妳家的左半邊形成。','9個方格正面'];
_tr_w["han"][Number('207') || 0]=['29','打撃','前９','','7','2200','不相關的外人住的房子。\n當武器用不知道會不會挨罵，真是不安。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','民宅','','','','在村裡掉落了數個。','9個方格正面'];
_tr_w["han"][Number('208') || 0]=['27','打撃','前９','','7','1900','樵夫住的破舊房子。\n雖然又破又髒，但可以當作武器。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','樵夫家','','','','掉落在西方地區。','9個方格正面'];
_tr_w["han"][Number('209') || 0]=['25','打撃','前９','','7','2800','好像有魔女居住的奇怪房子。\n屋頂的顏色很鮮豔。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','魔女家','','','','藏在西方的森林裡。','9個方格正面'];
_tr_w["han"][Number('210') || 0]=['32','打撃','前９','','7','3300','建設在沿海地區的神聖教會。\n一邊感謝神，一邊揮動它吧。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','某種神聖的力量對%3起作用了！','','教會','','','重置能力強化','建設在東北方沿海地區。','9個方格正面'];
_tr_w["han"][Number('211') || 0]=['31','打撃','前９','','7','3000','什麼都買得到，很方便的便利商店。\n因為很便利，也可以當作武器喔。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','便利商店','','','','建設在北方地區。','9個方格正面'];
_tr_w["han"][Number('212') || 0]=['25','打撃','前９','','7','1500','某個人設置的簡易帳篷。\n可以直接拿著走，是很方便的武器。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','帳篷','','','','設置在東方地區。','9個方格正面'];
_tr_w["han"][Number('213') || 0]=['33','打撃','前９','','7','4500','居民全是堅硬敵人的洞窟。\n洞窟本身也很硬，所以當作武器會很強。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','鋼鐵洞窟','','','','佇立在西南地區。','9個方格正面'];
_tr_w["han"][Number('214') || 0]=['24','打撃','前９','','7','4400','住著弱小卻聰明的敵人的洞窟。\n特徵是看起來有毒的顏色。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','弱者洞窟','','','','佇立在東南地區。','9個方格正面'];
_tr_w["han"][Number('215') || 0]=['28','打撃','前９','','7','4800','棲息著喜歡魔法的敵人的洞窟。\n洞窟本身也積蓄了魔力。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','幻術洞窟','','','隨機產生【極】攻擊魔法','佇立在西方的森林裡。','9個方格正面'];
_tr_w["han"][Number('216') || 0]=['26','打撃','前９','','7','5200','靈魂棲息在廢棄的城堡而形成了洞窟。\n可以感受到可怕的詛咒之力。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','靈魂洞窟','','','致命（5%）','佇立在東北地區。','9個方格正面'];
_tr_w["han"][Number('217') || 0]=['35','打撃','前９','','7','7500','黃金洞窟，傳說中的武器沉睡在這裡。\n其實洞窟本身也是滿厲害的武器。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','傳說洞窟','','','','可以從村子的北邊感覺到氣息。','9個方格正面'];
_tr_w["han"][Number('218') || 0]=['38','打撃','前９','','7','8700','魔王居住的邪惡城堡。\n當作武器使用就跟魔王一樣強大。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','魔王城','','','','聳立在極北地區。','9個方格正面'];
_tr_w["han"][Number('219') || 0]=['45','打撃','前９','','7','9800','總是很和平的和平村。自出生起，\n我就住在這個我最愛的村子裡。','%1快速揮動%2了！','%2非常猛烈地砸中%3了！','','','和平村','','','','妳住的村子。','9個方格正面'];
_tr_w["han"][Number('220') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('221') || 0]=['8','打撃','直線２','','5','600','比普通裂縫還要大的裂縫。\n比較大所以威力也比較高。','%1試著拿起%2揮動了！','%2擊中%3了！','%3裂開了！','','大型裂縫','','','降低防禦力（100%）','在鐵形成的洞窟裡。','2個方格直線'];
_tr_w["han"][Number('222') || 0]=['6','打撃','','','5','300','牆壁下方的空洞。\n是不是因為老舊而出現的呢？','%1試著拿起%2揮動了！','%2擊中%3了！','%3裂開了！','','牆壁凹洞','','','降低防禦力（70%）','在鐵形成的洞窟裡。',''];
_tr_w["han"][Number('223') || 0]=['4','打撃','','','5','500','外層剝落而露出內層的牆壁。\n很脆弱，當作武器有點微妙。','%1試著拿起%2揮動了！','%2擊中%3了！','','','裸露的牆壁','','','','在靈魂徬徨的洞窟裡。',''];
_tr_w["han"][Number('224') || 0]=['2','打撃','','','3','100','地面黑塊的一部分。\n好像很髒，所以我不太想碰。','%1試著拿起%2揮動了！','%3變得有點黑了！','黑塊的一部分掉進%3的眼睛裡了！','','黑塊','','','降低命中率（100%）','掉落在極北地區。',''];
_tr_w["han"][Number('225') || 0]=['2','打撃','','','3','100','溼滑的綠色地面。\n在陰溼的地方有很多。','%1試著拿起%2揮動了！','%3變得有點綠了！','溼滑的綠色物體纏繞在%3的腳上了！','','綠色地面','','','降低回避率（100%）','在鐵形成的洞窟裡。',''];
_tr_w["han"][Number('226') || 0]=['2','打撃','','','3','200','到處都有的道路。\n有了這個就很好走路。','%1試著拿起%2揮動了！','%2擊中%3了！','','','路','','','','到處都掉落了很多。',''];
_tr_w["han"][Number('227') || 0]=['4','斬撃','','','3','400','石製的白磚。\n四塊白磚是一組。','%1試著拿起%2揮動了！','%2的一角劈開%3了！','','','白磚','','','','在某處的便利商店裡。',''];
_tr_w["han"][Number('228') || 0]=['3','斬撃','','','3','200','破破爛爛的白磚，\n因為長年老化而裂開了。','%1試著拿起%2揮動了！','%2的一角劈開%3了！','','','裂開的白磚','','','','在靈魂徬徨的洞窟裡。',''];
_tr_w["han"][Number('229') || 0]=['5','斬撃','','','3','500','石製的藍磚。\n比白磚硬了一點。','%1試著拿起%2揮動了！','%2的一角劈開%3了！','','','藍磚','','','','在東北地區。',''];
_tr_w["han"][Number('230') || 0]=['3','斬撃','','','3','250','變得老舊而裂開的藍磚。\n一看就覺得破破爛爛。','%1試著拿起%2揮動了！','%2的一角劈開%3了！','','','裂開的藍磚','','','','在東北地區。',''];
_tr_w["han"][Number('231') || 0]=['5','斬撃','','','3','400','很乾淨的地板。\n似乎有在好好掃地。','%1試著拿起%2揮動了！','%2的一角劈開%3了！','','','地板','','','','在每間房子裡都有。',''];
_tr_w["han"][Number('232') || 0]=['8','斬撃','','','3','500','華麗的格紋地磚。\n品味有一點……沒事。','%1試著拿起%2揮動了！','%2的一角劈開%3了！','','','格紋地磚','','','','在某處奇怪的房子裡。',''];
_tr_w["han"][Number('233') || 0]=['6','斬撃','','','3','300','教會的木頭地板。\n有種大自然的氣息。','%1試著拿起%2揮動了！','%2的一角劈開%3了！','','','教會地板','','','','在某處的教會裡。',''];
_tr_w["han"][Number('234') || 0]=['12','打撃','','','6','1500','可以保養武器的鐵砧。\n擺放在大樹墩上面。','%1試著拿起%2揮動了！','%2直接命中%3了！','','','鐵砧','','','','擺放在各處。',''];
_tr_w["han"][Number('235') || 0]=['7','なし','','','2','600','把弱者洞窟裡的巨大毒沼\n切下一小部分後得到的東西。','%1試著拿起%2揮動了！','','','','毒沼（小）','','','中毒（100%）','在充滿毒沼的洞窟裡。',''];
_tr_w["han"][Number('236') || 0]=['15','打撃','','','4','2000','在魔界製造的邪惡燈籠。\n紫色火焰相當有魔界的感覺。','%1揮動%2了！','%2直接命中%3了！','','黑暗的火焰從%2裡溢出了！','魔界燈籠','','','產生火焰','在魔族之王居住的城堡裡。',''];
_tr_w["han"][Number('237') || 0]=['12','斬撃','直線２','4','20','15000','熊給的神奇寶劍。\n握住劍就會回想起與那些熊之間的回憶。','%1揮動%2了！','','','','熊之劍','','','連擊4次','可以從四隻熊手中獲得授予。','2個方格直線'];
_tr_w["han"][Number('238') || 0]=['11','打撃','直線２','','15','9000','可以騎著它在空中飛翔的厲害掃帚。\n看上去就像是魔女會騎的樣子。','%1揮動%2了！','%2直接命中%3了！','','','飛天掃帚','','','能在空中飛翔','妳的妹妹偷偷收藏著。','2個方格直線'];
_tr_w["han"][Number('239') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('240') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('241') || 0]=['13','打撃','','','3','499','好像是某個人的屍骨。\n即使跟他說話，他也沒有反應。','%1揮動%2了！','%2直接命中%3了！','%3遭到詛咒了！','','屍骨','','','致命（5%）','離開村子後往南走，它就掉在那裡。',''];
_tr_w["han"][Number('242') || 0]=['13','打撃','','','3','399','好像是動物的屍骨。\n即使叫牠握手，牠也沒有回應。','%1揮動%2了！','%2直接命中%3了！','%3遭到詛咒了！','','動物的屍骨','','','致命（5%）','掉落在西南地區。',''];
_tr_w["han"][Number('243') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('244') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('245') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('246') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('247') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('248') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('249') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('250') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('251') || 0]=['en','打撃','','','6','500','在外面徘徊的神祕黏液。\n看到人就會毫無理由地攻擊。','上吧！%2！','%2彈跳著衝撞%3了！','','','史萊姆','','','','經常出現在村子附近。',''];
_tr_w["han"][Number('252') || 0]=['en','打撃','','','6','700','變成紅色的史萊姆亞種，\n而且更加凶暴。','上吧！%2！','%2彈跳著衝撞%3了！','','','紅色史萊姆','','','','會出現在北方地區。',''];
_tr_w["han"][Number('253') || 0]=['en','打撃','','','6','2000','發出金色光彩的史萊姆稀有種。\n如果拿去賣，價格好像會很不錯。','上吧！%2！','%2彈跳著衝撞%3了！','','','黃金史萊姆','','','','會出現在閃著金黃光芒的洞窟裡。',''];
_tr_w["han"][Number('254') || 0]=['en','打撃','','','6','800','身體總是處於發火狀態的馬。\n牠好像一直都覺得有點熱。','上吧！%2！','%2踢飛%3了！','','','火焰馬','','','會噴出火焰','從西邊離開村子就會出現。',''];
_tr_w["han"][Number('255') || 0]=['en','打撃','','','6','1600','聽說是來自地獄的馬，\n身上纏繞著不祥之色的火焰。','上吧！%2！','%2踢飛%3了！','','','獄焰馬','','','會噴出火焰','會出現在魔族之王居住的城堡裡。',''];
_tr_w["han"][Number('256') || 0]=['en','打撃','','','6','600','喜歡惡作劇的怪物，\n也喜歡投擲東西。','上吧！%2！','%2衝撞%3了！','','','哥布林','','','會投擲武器','從南邊離開村子就會出現。',''];
_tr_w["han"][Number('257') || 0]=['en','打撃','','','6','900','凶暴的牛怪物。\n會把收在懷裡的無數武器丟過來。','上吧！%2！','%2衝撞%3了！','','','牛頭怪','','','會投擲武器','會出現在北方地區。',''];
_tr_w["han"][Number('258') || 0]=['en','打撃','','','7','500','連內臟都是岩石形成的壯碩男人。\n即使用刀劍砍劈也安然無恙。','上吧！%2！','%2用岩石手臂毆打%3了！','','','岩石男','','','','會出現在鐵形成的洞窟裡。',''];
_tr_w["han"][Number('259') || 0]=['en','打撃','','','7','700','所有的細胞都是岩石形成的壯碩男人。\n其手臂的破壞力超群。','上吧！%2！','%2用岩石手臂毆打%3了！','','','岩石大叔','','','','會出現在閃著金黃光芒的洞窟裡。',''];
_tr_w["han"][Number('260') || 0]=['en','打撃','','','6','600','硬得不得了的害蟲。\n其硬度甚至可以擊碎岩石。','上吧！%2！','%2施展出碎石術了！','','','鋼鐵蟲','','','降低防禦力（50%）','會出現在鐵形成的洞窟裡。',''];
_tr_w["han"][Number('261') || 0]=['en','打撃','','','6','3000','世界第一硬的生物。\n物理攻擊根本對牠沒用。','上吧！%2！','%2施展出碎鎧術了！','','','鑽石蟲','','','降低防禦力（50%）','會出現在魔族之王居住的城堡裡。',''];
_tr_w["han"][Number('262') || 0]=['en','打撃','','','5','1000','自火焰中誕生的火焰精靈。\n一靠近他就會立刻著火。','上吧！%2！','%2衝撞%3了！','','','火焰精靈','','','會吟唱【火焰魔法】','會出現在充滿魔力的洞窟裡。',''];
_tr_w["han"][Number('263') || 0]=['en','打撃','','','5','1000','自寒冰中誕生的寒冰精靈。\n他周圍的氣溫一直都在冰點下。','上吧！%2！','%2衝撞%3了！','','','寒冰精靈','','','會吟唱【寒冰魔法】','會出現在充滿魔力的洞窟裡。',''];
_tr_w["han"][Number('264') || 0]=['en','打撃','','','5','1000','自雷電中誕生的雷電精靈。\n一生氣就會從空中降下雷電。','上吧！%2！','%2衝撞%3了！','','','雷電精靈','','','會吟唱【雷電魔法】','會出現在充滿魔力的洞窟裡。',''];
_tr_w["han"][Number('265') || 0]=['en','打撃','','','5','1800','管束所有精靈的精靈之王。\n他的模樣既美麗又神祕。','上吧！%2！','%2衝撞%3了！','','','精靈王','','','會隨機吟唱【極】攻擊魔法','會出現在閃著金黃光芒的洞窟裡。',''];
_tr_w["han"][Number('266') || 0]=['en','打撃','','','5','800','擁有無數眼睛的噁心怪物。\n會降低別人攻擊力的討厭傢伙。','上吧！%2！','%2衝撞%3了！','','','紅眼怪','','','會吟唱【攻擊虛弱魔法】','會出現在南方地區。',''];
_tr_w["han"][Number('267') || 0]=['en','打撃','','','5','700','深藍色的身體令人覺得噁心的怪物。\n無數的眼睛一瞪過來，防禦就會下降。','上吧！%2！','%2衝撞%3了！','','','藍眼怪','','','會吟唱【防禦虛弱魔法】','會出現在南方地區。',''];
_tr_w["han"][Number('268') || 0]=['en','打撃','','','5','1500','沾上太多飛濺的血而變色的多眼怪。\n要小心這傢伙的虛弱魔法。','上吧！%2！','%2衝撞%3了！','','','血眼怪','','','會吟唱【全虛弱魔法】','會出現在魔族之王居住的城堡裡。',''];
_tr_w["han"][Number('269') || 0]=['en','打撃','','','6','900','被神奇薄膜包覆住的水母。\n魔法好像對牠完全沒有用。','上吧！%2！','%2用電人的觸手攻擊%3了！','','','魔法水母','','','觸電（25%）','會出現在充滿魔力的洞窟裡。',''];
_tr_w["han"][Number('270') || 0]=['en','打撃','','','6','1100','引誘人們邁向死亡的恐怖怪物。\n據說是死神所派來的。','上吧！%2！','%2邊笑邊攻擊%3了！','','','死亡使者','','','會吟唱【黑暗魔法】','會出現在靈魂徬徨的洞窟裡。',''];
_tr_w["han"][Number('271') || 0]=['en','打撃','','','6','1700','掌管死亡的邪惡怪物。\n會把發現的人拖下地獄。','上吧！%2！','%2邊笑邊攻擊%3了！','','','死神','','','會吟唱【黑暗魔法‧極】','會出現在魔族之王居住的城堡裡。',''];
_tr_w["han"][Number('272') || 0]=['en','打撃','','','6','699','靈魂附在屍骨上行動。\n會邊笑邊用骨頭揍人。','上吧！%2！','%2邊笑邊攻擊%3了！','','','骷髏','','','','會出現在靈魂徬徨的洞窟裡。',''];
_tr_w["han"][Number('273') || 0]=['en','打撃','','','6','700','在撒孢子的同時會襲擊人，\n是非常危險的蘑菇。','上吧！%2！','%2衝撞%3了！','','','妖怪蘑菇','','','會撒孢子','會出現在西方的森林。',''];
_tr_w["han"][Number('274') || 0]=['en','打撃','','','4','1000','外觀一看就覺得有毒。\n絕對不要吃它。','上吧！%2！','%2衝撞%3了！','','','鬼蘑菇','','','會撒孢子','會出現在北方地區。',''];
_tr_w["han"][Number('275') || 0]=['en','打撃','','','6','500','總是在蠕動的蟲。\n不但噁心，還非常難纏。','上吧！%2！','%2蠕動了！','','','蠕動蟲','','','','會出現在充滿毒沼的洞窟裡。',''];
_tr_w["han"][Number('276') || 0]=['en','打撃','','','3','800','會用超乎想像的音量鳴叫的鳥。\n早上聽見牠的鳴叫聲，沒有人會起不來。','上吧！%2！','%2發出150分貝的鳴叫聲了！','','','分貝鳥','','','封印普通攻擊外的攻擊（50%）','離開村子後往西走就會出現。',''];
_tr_w["han"][Number('277') || 0]=['en','打撃','','','3','1200','弱小又愛哭，但是很狡猾。\n會用魔法強化自己的難纏敵人。','上吧！%2！','%2邊哭邊用爪子撓%3！','','','愛哭鬼惡魔','','','會隨機吟唱強化魔法','會出現在充滿毒沼的洞窟裡。',''];
_tr_w["han"][Number('278') || 0]=['en','打撃','','','3','900','身心都很強韌的惡魔。\n但是狡滑程度也沒有衰退。','上吧！%2！','%2勇敢地用爪子撓%3！','','','勇敢惡魔','','','會隨機吟唱強化魔法','會出現在閃著金黃光芒的洞窟裡。',''];
_tr_w["han"][Number('279') || 0]=['en','打撃','','2','5','600','會拍著翅膀飛過來咬人。\n動作很快，要小心。','上吧！%2！','%2迅速地咬了%3兩次！','','','蝙蝠','','','連擊2次','會出現在村子附近。',''];
_tr_w["han"][Number('280') || 0]=['en','打撃','','2','5','900','會混在黑暗裡襲擊人的麻煩傢伙。\n被牠咬到還滿痛的。','上吧！%2！','%2迅速地咬了%3兩次！','','','黑暗蝙蝠','','','連擊2次','會出現在靈魂徬徨的洞窟裡。',''];
_tr_w["han"][Number('281') || 0]=['30','なし','','','1','1100','總而言之是很危險的石頭。\n一旦瀕死就會自爆。','上吧！%2！','%2大爆炸了！','','','炸彈石','','','會爆炸','會出現在鐵形成的洞窟裡。',''];
_tr_w["han"][Number('282') || 0]=['en','打撃','','','6','2200','蠕動蟲之王。\n會噴出讓武器風化的蠕蠕彈來攻擊。','上吧！%2！','%2蠕動了！','','','蠕蠕蟲','','','','會出現在魔族之王居住的城堡裡。',''];
_tr_w["han"][Number('283') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('284') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('285') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('286') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('287') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('288') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('289') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('290') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('291') || 0]=['en','斬撃','','3','8','8500','全身包覆著堅硬鎧甲的巨龍。\n牠的爪子可以劈開任何東西。','上吧！%2！','%2用鋼鐵爪子劈了又劈！','','','鎧龍艾利姆薩爾耶','','','連擊3次','會出現在鐵形成的洞窟最深處。',''];
_tr_w["han"][Number('292') || 0]=['en','打撃','直線３','','7','7700','出現在弱者洞窟的詭異怪鳥。\n在戰鬥時會採取強化自己的戰略。','上吧！%2！','%2吐出毒液了！','','','怪鳥毘闍耶','','','中毒（50%）,會吟唱【全強化魔法】','會出現在充滿毒沼的洞窟最深處。','3個方格直線'];
_tr_w["han"][Number('293') || 0]=['en','打撃','前６','','9','9900','擁有意志的神祕塊狀物。\n會操控觸手戰鬥，本體卻什麼也不會做。','上吧！%2！','%2壓在%3身上了！','','','魔塊希瑟斯','','','','會出現在充滿魔力的洞窟最深處。','6個方格正面'];
_tr_w["han"][Number('294') || 0]=['en','打撃','直線３','','4','2400','顏色詭異的觸手。\n似乎積蓄了龐大的魔力。','上吧！%2！','%2像鞭子一樣彎曲後直接命中%3了！','','','魔觸手','','','會隨機吟唱攻擊魔法','會出現在充滿魔力的洞窟最深處。','3個方格直線'];
_tr_w["han"][Number('295') || 0]=['en','斬撃','','','7','8800','在靈魂洞窟出現的騎士靈魂。\n感覺得到強烈的憎恨與怨念。','上吧！%2！','%2刺穿%3了！','','','靈騎丹斯雷夫','','','會吟唱【黑暗魔法‧極】','會出現在靈魂徬徨的洞窟最深處。',''];
_tr_w["han"][Number('296') || 0]=['en','打撃','直線２','','15','19999','掌管武器的武器之神。\n超越一切的存在。','上吧！%2！','%2施展出「大神斬」了！','%3在害怕神的威嚴……','','武器之神','天之尾羽張','','1回合不能行動（30%）','會出現在閃著金黃光芒的洞窟最深處。','2個方格直線'];
_tr_w["han"][Number('297') || 0]=['en','打撃','','2','12','19999','企圖毀滅世界的魔王。\n其實在祈求世界和平。','上吧！%2！','%2施展出「魔界拳」了！','','','魔王多爾哈巴','','','連擊2次,混亂（20%）','會出現在魔王城的頂樓。',''];
_tr_w["han"][Number('298') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('299') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('300') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('301') || 0]=['14','斬撃','','','5','800','附近的野狗。\n很親人，很可愛。','上吧！%2！','%2咬住%3了！','%2汪汪叫著，讓%3害怕了！','','狗','','','1回合不能行動（20%）','在村中某處散步。',''];
_tr_w["han"][Number('302') || 0]=['13','打撃','','','5','800','附近的野貓。\n喉嚨在發出咕嚕咕嚕的聲音。','上吧！%2！','%2對%3施展出銳利的貓拳了！','%2邊從喉嚨發出聲音，邊開始向%3撒嬌。','','貓','','','降低攻擊力（50%）','在村中某處散步。',''];
_tr_w["han"][Number('303') || 0]=['16','打撃','','','6','1500','某個人飼養的馬。\n毛色非常漂亮美麗。','上吧！%2！','%2踢飛%3了！','','','馬','','','會撞飛敵人','在西北地區閒晃。',''];
_tr_w["han"][Number('304') || 0]=['19','打撃','','','6','1600','很厲害的羊，會告訴人提示。\n蓬鬆柔軟的毛很可愛。','上吧！%2！','%2衝撞%3了！','%2柔軟的毛纏在%3身上了！','','羊','','','降低行動次數（30%）','是我。',''];
_tr_w["han"][Number('305') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('306') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('307') || 0]=['18','打撃','','','6','720','在和平村獨居的女性。\n是一位美女，個性也很好。','上吧！%2！','%2溫柔地施展頭槌了！','','','女性村民','Piranka','','','住在村中某處的房子裡。',''];
_tr_w["han"][Number('308') || 0]=['8','打撃','','','6','1050','家裡住村中且關係融洽的一家之主。\n總是在屋外陪小孩玩。','上吧！%2！','%2施展出一家之主拳了！','','','一家之父','Tel-ho','','','在村中某處看著小孩。',''];
_tr_w["han"][Number('309') || 0]=['25','打撃','','','3','960','家裡住在和平村的母親。\n平常很溫柔，一生氣好像就很恐怖。','上吧！%2！','%2施展出拳擊了！','','%2吐出火焰了！','一家之母','Hokkoma','','會噴出火焰','住在村中某處的房子裡。',''];
_tr_w["han"][Number('310') || 0]=['12','打撃','','','5','210','朝氣十足地在村裡到處跑的兄弟檔。\n這位是哥哥，老是在大喊「耶～」。','上吧！%2！','%2一邊大喊「耶～！」一邊出拳了！','','','朝氣蓬勃的兄弟','Kubota','','','在村中某處朝氣十足地玩樂。',''];
_tr_w["han"][Number('311') || 0]=['6','打撃','','','5','180','和哥哥一起玩的可愛弟弟。\n將來的夢想好像是成為Yout◯uber。','上吧！%2！','%2施展出必殺超人力霸騎士踢了！','','','朝氣蓬勃的兄弟','Nagina','','','在村中某處朝氣十足地玩樂。',''];
_tr_w["han"][Number('312') || 0]=['17','打撃','','','6','1530','管理和平餐廳的主廚。\n戰鬥時擅長腳上功夫。','上吧！%2！','%2狠狠踹出了一腳！','','','餐廳主廚','A flavor','','','在村中某處的房子裡擔任主廚。',''];
_tr_w["han"][Number('313') || 0]=['11','打撃','','','6','840','每天都光臨和平餐廳的常客。\n最近血糖值不太妙。','上吧！%2！','%2連續施展出高血糖拳了！','%3也成了高血糖！','','餐廳常客','Dongfa','','降低行動次數（20%）','在村中某處的房子裡吃飯。',''];
_tr_w["han"][Number('314') || 0]=['13','打撃','','','6','1500','走遍世界各地，自稱是美食家。\n對和平餐廳讚不絕口。','上吧！%2！','%2用凸出的肚子壓扁%3了！','','','美食家','Katsubaruga','','','在村中某處的房子裡吃飯。',''];
_tr_w["han"][Number('315') || 0]=['4','打撃','','','4','2520','住在村裡，目光溫和的老人。\n每天都會在妻子的墓前參拜。','上吧！%2！','%2虛弱地衝撞上去了！','%2過世的妻子居然詛咒%3了！','','掃墓的老人','Braig','','致命（5%）','在村中某處掃墓。',''];
_tr_w["han"][Number('316') || 0]=['25','打撃','','','6','1740','經常在水井邊開會的歐巴桑。\n一開始講話就有兩個小時不會停。','上吧！%2！','%2施展出抓狂巴掌了！','%3看到抓狂的歐巴桑後嚇到了！','','井邊的歐巴桑','Fukiya','','混亂（20%）','在村中某處開水井邊會議。',''];
_tr_w["han"][Number('317') || 0]=['15','打撃','','2','6','1680','最愛八卦的歐巴桑。\n總是在水井邊聊天聊到忘我。','上吧！%2！','%2施展出連環巴掌了！','','','井邊的歐巴桑','Illisa','','連擊2次','在村中某處開水井邊會議。',''];
_tr_w["han"][Number('318') || 0]=['7','打撃','','','4','2160','經常在村裡散步的老人。\n會說些聽都沒聽過的事情。','上吧！%2！','%2用拐杖揍%3了！','','','正在散步的老人','Rod','','','在村中某處散步。',''];
_tr_w["han"][Number('319') || 0]=['10','打撃','','','6','1050','在村裡散步的大叔。\n好像是從遙遠的故鄉來這裡賺錢的。','上吧！%2！','%2用散步鍛鍊出來的腳踢飛%3了！','','','正在散步的大叔','Ranyapa','','會撞飛敵人','在村中某處散步。',''];
_tr_w["han"][Number('320') || 0]=['','','','','','0','','','','','','','','','','',''];
_tr_w["han"][Number('321') || 0]=['','','','','','0','','','','','','','','','','',''];
_tr_w["han"][Number('322') || 0]=['5','打撃','','','6','630','追尋傳說武器的冒險者。\n愛媽咪勝過愛吃飯。','上吧！%2！','%2連續施展出媽咪教的拳法了！','','','冒險者','Kikangji','','','行走在南方地區。',''];
_tr_w["han"][Number('323') || 0]=['13','打撃','','','9','1470','每天都在池塘淋浴的老頭。\n希望趕快有人去報警抓他。','上吧！%2！','%2邊笑邊衝撞%3了！','%3看到變態後嚇到了！','','淋浴老頭','Tetsken','','混亂（40%）','從南邊離開村子，他就站在那裡。',''];
_tr_w["han"][Number('324') || 0]=['18','打撃','','','6','810','在海邊露營的青年。\n真要說起來，他其實是室內派。','上吧！%2！','%2揮動露營道具發動攻擊了！','','','露營者','Bastar','','','在南方地區露營。',''];
_tr_w["han"][Number('325') || 0]=['15','打撃','','','6','540','喜歡獨自露營的女性。\n受祖父影響而開始露營。','上吧！%2！','%2揮動露營道具發動攻擊了！','','','露營者','Ikpa','','','在南方地區露營。',''];
_tr_w["han"][Number('326') || 0]=['17','打撃','','','6','960','獨自狂歡的露營者。\n看著火焰而興奮不已。','上吧！%2！','%2揮動露營道具發動攻擊了！','','%2生起營火了！','露營者','Charak','','產生火焰','在南方地區露營。',''];
_tr_w["han"][Number('327') || 0]=['14','斬撃','','','6','750','踏上旅途尋找自我的女性。\n她告訴了我洞窟的危險性。','上吧！%2！','%2用短劍劈開%3了！','','','旅行的女性','Cathrus','','','在南方地區旅行。',''];
_tr_w["han"][Number('328') || 0]=['22','斬撃','','','6','600','告訴我毒沼危險性的冒險者。\n還給了我藥草，是非常親切的人。','上吧！%2！','%2用愛劍劈開%3了！','','','冒險者','Fureil','','','站在東南地區。',''];
_tr_w["han"][Number('329') || 0]=['28','斬撃','','','8','1950','在老舊的小屋裡獨自生活的樵夫。\n很擅長使用斧頭。','上吧！%2！','%2像是要伐木似地揮動斧頭了！','','','樵夫','Wonmar','','降低防禦力（30%）','住在某處的小屋裡。',''];
_tr_w["han"][Number('330') || 0]=['5','打撃','','','4','2310','悠閒旅行的老人。\n持續尋找著何謂人生的答案。','上吧！%2！','%2虛弱地施展頭槌了！','','','旅行的老人','Brood','','','從西邊離開村子，他就在那裡旅行。',''];
_tr_w["han"][Number('331') || 0]=['11','斬撃','','','6','990','為追求強大實力而外出旅行的冒險者男性。\n雖然擺出老練的姿態，但並沒有那麼強。','上吧！%2！','%2用劍攻擊%3了！','','','冒險者','Esto','','','從西邊離開村子，他就在那裡行走。',''];
_tr_w["han"][Number('332') || 0]=['9','打撃','','','6','5400','住在森林的小魔女。\n別看她這樣，其實魔法能力超群。','上吧！%2！','%2揮動魔杖攻擊了！','','%2的魔杖頂端釋放出魔法了！','魔女','Cremon','','會隨機吟唱【極】攻擊魔法','住在某處奇怪的房子裡。',''];
_tr_w["han"][Number('333') || 0]=['17','打撃','','','6','1260','每天都會走向市集的女性。\n夢想著有一天會抵達市集。','上吧！%2！','%2用走路鍛鍊出來的腳踢飛%3了！','','','走向市集的女性','Takana','','會撞飛敵人','從東邊離開村子，她就在那裡行走。',''];
_tr_w["han"][Number('334') || 0]=['24','打撃','','','6','630','在市集擺攤的女性。\n相當熱心學習。','上吧！%2！','%2揮動晨星鎚了！','','','擺攤的女性','Yawarra','','混亂（20%）','在東方地區擺攤。',''];
_tr_w["han"][Number('335') || 0]=['20','斬撃','','2','6','1020','在市集賣菜的男性。\n其實相當黏奶奶。','上吧！%2！','%2用快劍斬殺%3了！','','','擺攤的男性','Rapier','','連擊2次','在東方地區擺攤。',''];
_tr_w["han"][Number('336') || 0]=['14','打撃','','','7','1650','攤位上賣著一看就像是假貨的人。\n最好不要和這種人扯上關係。','上吧！%2！','%2用凸出的肚子壓扁%3了！','','','擺攤的男性','Pusa','','','在東方地區擺攤。',''];
_tr_w["han"][Number('337') || 0]=['19','打撃','','','6','720','開心逛市集的紅髮男人。\n持有金錢是 28 圓。','上吧！%2！','%2施展出右直拳了！','','','市集的客人','Shrink','','','在東方地區的市集物色商品。',''];
_tr_w["han"][Number('338') || 0]=['8','斬撃','','','6','660','來到市集的女性。\n正在等半價特賣活動開始。','上吧！%2！','%2用在市集買的劍攻擊%3了！','','','市集的客人','Scruma','','','在東方地區的市集物色商品。',''];
_tr_w["han"][Number('339') || 0]=['12','打撃','','','4','1350','因為很閒而來市集的男性。\n好像買了奇怪的陶壺。','上吧！%2！','%2用在市集買的陶壺去砸%3了！','','','市集的客人','Bundy','','一定會暴擊','在東方地區的市集物色商品。',''];
_tr_w["han"][Number('340') || 0]=['18','打撃','','2','6','1440','有錢的市集客人。\n走路時還在用一疊鈔票搧風。','上吧！%2！','%2拿出鈔票甩了對方一巴掌！','%3看見大把鈔票後興奮不已！','','市集的客人','Murgiza','','連擊2次,提升敵人的行動次數','在東方地區的市集物色商品。',''];
_tr_w["han"][Number('341') || 0]=['17','打撃','','','5','1230','沿海教會的牧師，\n也是投擲教的教徒。','上吧！%2！','%2施展出阿門拳了！','某種神聖的力量對%3起作用了！','','教會的牧師','Falsio','','重置能力強化','站在某處的教會裡。',''];
_tr_w["han"][Number('342') || 0]=['22','打撃','','','6','1140','好像很溫柔的修女。\n喜歡的麵類是平打烏龍麵。','上吧！%2！','%2氣勢十足地衝撞上去了！','','','教會的修女','Lancia','','','站在某處的教會裡。',''];
_tr_w["han"][Number('343') || 0]=['19','打撃','','','5','2460','在教會禱告的婆婆。\n是投擲教的狂熱信徒。','上吧！%2！','%2邊禱告邊擒抱住敵人了！','','','禱告的婆婆','Versa','','','在某處的教會裡禱告。',''];
_tr_w["han"][Number('344') || 0]=['13','打撃','','','6','450','在教會禱告的青年。\n個性非常認真。','上吧！%2！','%2邊禱告邊施展頭槌了！','','','禱告的青年','Irarufu','','','在某處的教會裡禱告。',''];
_tr_w["han"][Number('345') || 0]=['14','打撃','','','6','750','很適合戴眼鏡的綠髮冒險者。\n興趣是廢墟巡禮。','上吧！%2！','%2揮動魔杖攻擊了！','%2的魔杖頂端釋放出魔法了！','','冒險者','Clique','','產生【寒冰魔法】','站在東北地區。',''];
_tr_w["han"][Number('346') || 0]=['10','打撃','','','5','2040','喜歡釣魚和大海的老人，\n但沒有什麼釣魚的技術。','上吧！%2！','%2揮動釣竿攻擊了！','釣竿的前端居然勾著腐爛的蚯蚓！','','釣客','Phyzigro','','中毒（60%）','在西北方岬角釣魚。',''];
_tr_w["han"][Number('347') || 0]=['12','斬撃','','3','6','510','好勝的冒險者女性。\n好像正在找工作。','上吧！%2！','%2用短劍迅速劈開%3了！','','','冒險者','Dagger','','連擊3次','行走在北方地區。',''];
_tr_w["han"][Number('348') || 0]=['27','打撃','','','6','660','便利商店的新人店員。\n只會按照教學手冊上教的話說。','上吧！%2！','%2用讀取條碼的那個揍%3了！','','','便利商店店員','Miaota','','','站在某處的便利商店裡。',''];
_tr_w["han"][Number('349') || 0]=['15','斬撃','','','6','900','在便利商店購物的男性。\n在煩惱要買哪一款果汁。','上吧！%2！','%2用便利商店卡劈開%3了！','','','便利商店的客人','Arsato','','','站在某處的便利商店裡。',''];
_tr_w["han"][Number('350') || 0]=['33','打撃','','','7','990','聚在便利商店前的冷酷不良少年。\n惹他生氣好像會很可怕。','上吧！%2！','%2把%3打得落花流水！','%3怕%2怕得屁滾尿流！','','不良少年','Reserville','','1回合不能行動（20%）','蹲在某處的便利商店前。',''];
_tr_w["han"][Number('351') || 0]=['29','斬撃','','','5','2160','特徵是一隻眼睛受傷的老練旅人。\n外表一看就覺得很厲害。','上吧！%2！','%2豪邁地揮動大劍了！','','','旅行的大叔','Gladys','','','在北方地區旅行。',''];
_tr_w["han"][Number('352') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('353') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('354') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('355') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('356') || 0]=['2','打撃','','','8','180','\\n[1]的妹妹。年紀小卻很能幹。\n家事和顧店都是她一個人做。','上吧！%2！','%2試著使出加油拳法！','%2從%3的錢包裡偷了錢！','','器美','','','會獲得錢','是妳的妹妹。',''];
_tr_w["han"][Number('357') || 0]=['30','斬撃','','','9','1140','看起來沒有幹勁的勇者。\n似乎想要輕鬆地打倒魔王。','上吧！%2！','%2不耐煩地揮劍了！','%2吸走%3的幹勁了！','','勇者修瓦爾迪','','','降低攻擊力（40%）','妳的店發展起來後，他就會到訪吧。',''];
_tr_w["han"][Number('358') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('359') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('360') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('361') || 0]=['21','斬撃','直線２','','20','18564','魔王給的黑暗大劍。\n每一擊都沉重得可怕。','%1揮動%2了！','','','','魔王的劍','','','一定會暴擊','應該在魔族之王手中吧。','2個方格直線'];
_tr_w["han"][Number('362') || 0]=['17','打撃','','','8','1000','收藏獲得獎杯用的箱子。\n拿起來非常重。','%1試著拿起%2揮動了！','%2直接命中%3了！','','','獎杯箱','','','','在妳家裡。',''];
_tr_w["han"][Number('363') || 0]=['0','なし','','','20','14500','好像能抓到各種昆蟲的網子。\n吸力可媲美黑洞。','%1揮動%2了！','','','','捕蟲網','','','將面前的物品當作武器獲得','妳的妹妹偷偷收藏著。',''];
_tr_w["han"][Number('364') || 0]=['25','打撃','','','9','2','完美的螺旋狀糞便。\n含水量和顏色都非常完美。','%1試著拿起%2揮動了！','%2直接命中%3的臉了！','%2超級臭！','','大便','','','降低攻擊力（20%）,降低防禦力（20%）,降低行動次數（20%）','溫熱地掉在東北地區。',''];
_tr_w["han"][Number('365') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('366') || 0]=['','','','','','','','%1試著使出加油拳法！','','','','（空手）','','','連擊3次','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','%1使用\i[261]藥草了！','','','','（藥草）','','','產生火焰','',''];
_tr_w["han"][Number('3') || 0]=['','','','','','','希望能事先大量準備好的冒險必需品。\n可以恢復體力和異常狀態喔。','%1試著拿起%2揮動了！','%3把%2吸進嘴裡了！','','','藥草','','','','恢復敵人的HP',''];
_tr_w["han"][Number('4') || 0]=['','','','','','','能引出武器隱藏之力的神奇石頭。\n是很難獲得的貴重材料。','%1試著拿起%2揮動了！','%2的一角刺中%3了！','','','魂石','','','','有時候敵人會掉下來。',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["han"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];

})();
    (function(){


_tr_w["kan"][Number('4') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('5') || 0]=['10','斬撃','前３','3','5','200','武器之神‧舞契所創，\n獨一無二，用來除錯的武器。','%1揮動%2了！','%2直接命中%3的頭了！','%2散發的氣息刺激到%3了！','從%2當中噴出火焰了！','除錯壺','','','連擊3次','不存在。','3个方格正面'];
_tr_w["kan"][Number('6') || 0]=['10','斬撃','','','30','1000','可以大量生产的铜剑。\n任何人都能轻松驾驭，威力却是未知数。','%1挥动了%2！','','','','旅人的剑','','','','在你家的宝箱里沉睡着。',''];
_tr_w["kan"][Number('7') || 0]=['20','斬撃 / 炎','','','30','2100','封印了火焰魔力的魔法剑。\n只要挥动一下，附近就会陷入火海。','%1挥动了%2！','','','剑尖居然窜出了火焰！','业火之剑','','','产生火焰','在南方地区的宝箱里沉睡着。',''];
_tr_w["kan"][Number('8') || 0]=['25','斬撃 / 雷','','','30','7500','举世知名的神剑。\n人称雷锤，可以斩断一切。','%1挥动了%2！','','','剑尖居然窜出了雷电！','王者之剑','','','产生雷电','在宝箱里沉睡着。',''];
_tr_w["kan"][Number('9') || 0]=['60','斬撃','','','999','19999','武器之神创造的黄金剑。\n是可以斩断万物的最强武器。','%1挥动了%2！','','','','无敌‧草丛生剑','','','','对不起，就连我也不清楚……',''];
_tr_w["kan"][Number('10') || 0]=['15','斬撃','直線２','','20','1700','巨大又锐利的铁制大剑。\n出乎意料的轻，能够以很快的速度挥动。','%1挥动了%2！','','','','巨剑','','','','在北方地区的宝箱里沉睡着。','2个方格直线'];
_tr_w["kan"][Number('11') || 0]=['27','斬撃','直線２','','20','3600','古代巨人族爱用的大剑。\n只要挥动一下，甚至能劈开大陆。','%1挥动了%2！','','','','巨人的大剑','','','','某处的便利商店正在贩售。','2个方格直线'];
_tr_w["kan"][Number('12') || 0]=['12','斬撃','','','30','1200','为暗杀而制的短剑。\n想要刺进致命位置需要高超的技术。','%1将%2刺了出去！','','','','刺客匕首','','','致命（10%）','在村中某处人家的宝箱里沉睡着。',''];
_tr_w["kan"][Number('13') || 0]=['14','斬撃 / 闇','','2','30','4666','遭到恶魔诅咒的短刀。\n利用敏捷的动作，瞬间将敌人碎尸万段。','%1将%2刺了出去！','','','','恶魔的短刀','','','连击2次,中毒（30%）','在东北方沿海地区的宝箱里沉睡着。',''];
_tr_w["kan"][Number('14') || 0]=['16','打撃','','','20','1500','单纯是为了破坏东西而制造出来的锤子。\n不能小看它的破坏力。','%1挥动了%2！','','','','冲击锤','','','混乱（20%）','在村子东北方的宝箱里沉睡着。',''];
_tr_w["kan"][Number('15') || 0]=['28','打撃','','','20','8000','锤头像星星一样。\n不要拿来按摩喔。','%1挥动了%2！','','','','晨星锤','','','混乱（20%）','东方地区的路边摊正在贩售。',''];
_tr_w["kan"][Number('16') || 0]=['13','斬撃 / 氷','直線３','','30','1800','以海神武器闻名的长枪。\n有着三叉的矛可以贯穿万物。','%1使用%2刺穿了目标！','','','','三叉戟','','','','在充满毒沼的洞窟宝箱里沉睡着。','3个方格直线'];
_tr_w["kan"][Number('17') || 0]=['26','斬撃 / 炎','直線３','','30','7600','来自地狱的长枪。\n会释放地狱火焰，贯穿并燃尽一切。','%1使用%2刺穿了目标！','','','','地狱魔枪','','','','东方地区的路边摊正在贩售。','3个方格直线'];
_tr_w["kan"][Number('18') || 0]=['15','斬撃','','','20','2000','沉重而锐利的钢铁斧头。\n砍在铠甲上也能造成巨大的伤害。','%1挥动了%2！','','','','钢铁斧头','','','降低防御力（30%）','在东北地区的宝箱里沉睡着。',''];
_tr_w["kan"][Number('19') || 0]=['28','斬撃','','','20','9500','天下无双的斧头，没有砍不断的东西。\n传说是鬼神之王曾经使用过的斧头。','%1挥动了%2！','','','','巨人斧头','','','降低防御力（30%）','在居住于某个小屋里的男人手中。',''];
_tr_w["kan"][Number('20') || 0]=['8','打撃 / 炎','','','20','2100','魔导士自古以来爱用的魔杖。\n会喷出火球，烧尽一切。','%1挥动了%2！','','','魔杖顶端喷出了火焰！','魔导士之杖','','','可施放【火焰魔法】','在铁洞窟的宝箱里沉睡着。',''];
_tr_w["kan"][Number('21') || 0]=['8','打撃 / 雷','','','20','2500','隐藏着雷神之力的魔杖。\n只要一挥动，雷神之锤便会从天而降。','%1挥动了%2！','','','魔杖顶端释放出了雷电！','雷神之杖','','','可施放【雷电魔法】','实现村民的愿望便能获得。',''];
_tr_w["kan"][Number('22') || 0]=['8','打撃 / 氷','','','20','2600','据说是传说中的冰魔女拥有的魔杖。\n会呼唤暴风雪，冻结全世界。','%1挥动了%2！','','','魔杖顶端出现了冰块！','冰魔女之杖','','','可施放【寒冰魔法】','在充满魔力的洞窟宝箱里沉睡着。',''];
_tr_w["kan"][Number('23') || 0]=['18','打撃 / 闇','','','20','6666','住在地狱的大恶魔所使用的魔杖。\n会唤醒黑暗的力量。','%1挥动了%2！','','','魔杖顶端释放了出黑暗的力量！','大恶魔之杖','','','可施放【黑暗魔法‧极】','在魔族之王居住的城堡的宝箱里沉睡着。',''];
_tr_w["kan"][Number('24') || 0]=['18','打撃 / 光','','','20','9999','过去大贤者为了封印黑暗而使用的魔杖。\n里面封入了神圣的力量。','%1挥动了%2！','','','魔杖顶端释放出了神圣的力量！','大贤者之杖','','','随机施放【极】攻击魔法','在居住于某地的魔女手中。',''];
_tr_w["kan"][Number('25') || 0]=['20','斬撃','','','20','1500','用来挖掘墙壁的道具。\n足以当作武器使用。','%1挥动了%2！','','','','冰镐','','','降低防御力（50%）','掉落在西南地区。',''];
_tr_w["kan"][Number('26') || 0]=['17','斬撃','','','20','1300','用来挖洞的工具，\n也能当作武器使用。','%1挥动了%2！','%2直接命中%3了！','顺便在%3的脚边挖了个洞！','','铁锹','','','1回合不能行动（30%）','掉落在南方地区。',''];
_tr_w["kan"][Number('27') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('28') || 0]=['7','打撃','前３','','3','300','在各地生长的树。 \n因为滥伐而破坏森林是一个大问题。','%1挥动了%2！','%2直接命中了%3的头部！','','','树','','','','随处可见。','3个方格正面'];
_tr_w["kan"][Number('29') || 0]=['6','打撃','前６','4','3','1000','一次捡起四棵树而得到的武器。\n有种比平常划算四倍的感觉。','%1同时挥动了四棵树！','树直接命中了%3的头部！','','','树×4','','','连击4次','随处可见。','4个方格正面'];
_tr_w["kan"][Number('30') || 0]=['3','打撃','','','3','50','原本生长在各处的小型枯树。\n生物注定有一天会枯朽。','%1试着挥动了%2！','%2刷的一声，击中了%3！','','','小枯树','','','HP一定会剩下1','在东南地区长了很多。',''];
_tr_w["kan"][Number('31') || 0]=['5','打撃','直線２','','3','200','干枯凋零，徒留空虚的树。\n令人有种悲伤的感觉。','%1试着挥动了%2！','%2刷的一声，击中了%3！','','','枯树','','','HP一定会剩下1','在东南地区长了很多。','2个方格直线'];
_tr_w["kan"][Number('32') || 0]=['9','打撃','','','5','200','很适合拿来当椅子的树墩。\n光是看着就让人想坐下去。','%1拿起%2试着挥动了一下！','%2直接命中了%3的头部！','%3坐在%2上开始休息！','','树墩','','','1回合不能行动（75%）','长在村里的某处。',''];
_tr_w["kan"][Number('33') || 0]=['8','打撃','','','2','100','因枯萎而倒下的树。\n现在已成为森林的装饰品。','%1挥动了%2！','%2直接命中了%3的头部！','%3没有站稳，倒在了地上！','','倒下的树','','','1回合不能行动（90%）','掉落在村里的某处。',''];
_tr_w["kan"][Number('34') || 0]=['10','打撃','直線２','','6','700','生长在南国的树。\n看着它就能感受到南国风情。','%1爽朗地挥动了%2！','%2直接命中了%3的头部！','','','椰子树','','','','生长在南方沿海地区。','2个方格直线'];
_tr_w["kan"][Number('35') || 0]=['4','斬撃','','','3','100','颜色和样子都很普通的草。\n另外还有B和C。','%1拿起%2试着挥动了一下！','%2割伤了%3的左手食指！','','','草A','','','','随处可见。',''];
_tr_w["kan"][Number('36') || 0]=['5','斬撃','','','3','100','比草A大了一圈的草。\n因为比较大，似乎有一点划算。','%1拿起%2试着挥动了一下！','%2割伤了%3的左手无名指！','','','草B','','','','随处可见。',''];
_tr_w["kan"][Number('37') || 0]=['4','斬撃','','3','3','150','比草A、B大上许多的草。\n在草里面是老大。','%1拿起%2试着挥动了一下！','%2割伤了%3的右手大姆指、食指和无名指！','','','草C','','','连击3次','随处可见。',''];
_tr_w["kan"][Number('38') || 0]=['2','斬撃','','','2','50','因为枯萎而结束一生的草。\n拿来当武器有些靠不住。','%1拿起%2试着挥动了一下！','%2沙的一声，击中了%3！','','','枯草','','','HP一定会剩下1','在东南地区长了很多。',''];
_tr_w["kan"][Number('39') || 0]=['8','斬撃','','','3','200','纯白而惹人怜爱的鲜花。\n看着它，心灵彷佛被洗涤了。','%1拿起%2试着挥动了一下！','花瓣割伤了%3的左手小指！','%3受到%2抚慰，失去战意了！','','白花','','','降低攻击力（50%）','随处可见。',''];
_tr_w["kan"][Number('40') || 0]=['9','斬撃','','','3','250','火红又热情的花朵。\n可爱到让人丧失战意。','%1拿起%2试着挥动了一下！','花瓣割伤了%3的右手中指！','%3受到%2抚慰，失去战意了！','','红花','','','降低攻击力（40%）','随处可见。',''];
_tr_w["kan"][Number('41') || 0]=['6','斬撃','','','3','400','品种不明，但很漂亮的花。\n光是看着它，血压就会下降喔。','%1拿起%2试着挥动了一下！','花瓣割伤了%3的左手中指！','%3受到%2抚慰，失去战意了！','','漂亮的花','','','降低攻击力（75%）','生长在村子和森林里。',''];
_tr_w["kan"][Number('42') || 0]=['20','打撃','','','6','900','散发出可怕恶臭的花朵。\n如果可以，没人想捡起来。','%1捏着鼻子挥动了%2！','%2直接命中了%3的鼻孔旁！','%3似乎因为太臭而觉得不舒服！','','臭花','','','中毒（100%）','生长在西方的森林里。',''];
_tr_w["kan"][Number('43') || 0]=['6','打撃','前６','3','4','900','怎么看都像是会跳出那个的草地。\n具体我就不多说了。','%1挥动了%2！','无数野生宝◯梦从%2里跳出并袭击了%3！','','','草地','','','连击3次','随处可见。','6个方格正面'];
_tr_w["kan"][Number('44') || 0]=['10','斬撃','前９','3','4','1800','仔细一看，形状很像一把剑的草地。\n好像也挺锋利的。','%1挥动了%2！','用形状像剑的草地斩杀了%3！','','','好像很强的草地','','','连击3次','离开村子后往西走，它就生长在那里。','9个方格正面'];
_tr_w["kan"][Number('45') || 0]=['7','打撃','前６','3','4','1500','好像会有稀有生物跳出来的草丛。\n比如异色的……哎呀，好像有人来了。','%1挥动了%2！','无数异色宝◯梦从%2里跳出并袭击了%3！','','','黄金草地','','','连击3次','离开村子后往东走，它就生长在那里。','6个方格正面'];
_tr_w["kan"][Number('46') || 0]=['7','打撃','','','5','300','修剪得很漂亮的草丛。\n即使拿来打人也不太痛。','%1拿起%2试着挥动了一下！','%2哗啦一声，击中了%3！','','','草丛','','','','通常装饰在民宅附近。',''];
_tr_w["kan"][Number('47') || 0]=['7','打撃','','','5','600','开满漂亮花朵的花丛。\n光是摆在那里就能美化街道。','%1拿起%2试着挥动了一下！','%2哗啦一声，击中了%3！','','','花丛','','','','装饰在你家附近。',''];
_tr_w["kan"][Number('48') || 0]=['6','斬撃','','3','4','200','展现秋天样貌的落叶地毯。\n总觉得它们随风飘零的样子很浪漫。','%1拿起%2试着挥动了一下！','许许多多的%2划伤了%3！','','','落叶','','','连击3次','离开村子后往东走，那里地上很多。',''];
_tr_w["kan"][Number('49') || 0]=['5','打撃','','','5','500','闪着金黄色光芒的芒草。\n随风摇摆的模样很美。','%1不断摇晃%2！','%2在%3的腰部挠痒痒！','%3大笑不止，彷佛要失去意识！','','芒草','','','混乱（100%）','离开村子后往东走，它就生长在那里。',''];
_tr_w["kan"][Number('50') || 0]=['6','打撃','','','6','400','别看它这样，它可是毒蘑菇。\n蘑菇不可貌相，海水不可斗量。','%1拿起%2试着挥动了一下！','%2啪的一声地击中了%3的脸！','%2碎片掉进了%3的嘴里！','','蘑菇','','','中毒（75%）','生长在西方的森林里。',''];
_tr_w["kan"][Number('51') || 0]=['8','打撃','','','6','300','看上去有些那个的蘑菇。\n内含剧毒，很好吃。','%1拿起%2试着挥动了一下！','%2啪的一声地击中了%3的脸！','%2碎片掉进了%3的嘴里！','','看似不妙的蘑菇','','','中毒（50%）','生长在西方的森林里。',''];
_tr_w["kan"][Number('52') || 0]=['14','打撃','直線３','','6','700','攀附墙壁生长的植物。\n好像可以当成鞭子使用。','%1挥动了%2！','%2像鞭子一样弯曲后直接命中了%3！','','','爬山虎','','','','生长在南方地区的断崖。','3个方格直线'];
_tr_w["kan"][Number('53') || 0]=['12','斬撃','直線３','','6','800','缠绕其他东西而生长的植物。\n因为很长，可以攻击到很远的地方。','%1挥动了%2！','%2迅猛地窜出，刺中了%3！','','','藤蔓','','','','生长在南方地区的断崖。','3个方格直线'];
_tr_w["kan"][Number('54') || 0]=['16','斬撃','','','5','1200','生长在沙漠地带的带刺植物。\n被刺到还蛮痛的。','%1忍住痛楚，挥动了%2！','%2的刺深深刺进了%3的身体！','','','仙人掌','','','','生长在西南地区。',''];
_tr_w["kan"][Number('55') || 0]=['15','打撃','','','4','600','品种不明的怪草，\n成分也是一团谜。','%1拿起%2试着挥动了一下！','%2割伤了%3的左手小指！','%2内含的神秘成分对%3奏效了！','','怪草','','','降低防御力（100%）','生长在西方的森林里。',''];
_tr_w["kan"][Number('56') || 0]=['13','斬撃','','','4','300','长在墙上的草。\n其实含有催眠成分。','%1拿起%2试着挥动了一下！','%2割伤了%3的右手背！','%2居然含有催眠成分！','','长在墙上的草','','','睡眠（25%）','存在于铁洞窟里。',''];
_tr_w["kan"][Number('57') || 0]=['10','打撃','','','5','200','怎么看都很诡异的植物幼苗。\n根部似乎有神奇的力量。','%1拿起%2试着挥动了一下！','%2反复刺中了%3！','%2居然吸收了%3的能力！','','诡异的幼苗','','','复制能力变化','生长在充满毒沼的洞窟里。',''];
_tr_w["kan"][Number('58') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('59') || 0]=['3','打撃','','','3','50','掉在地上的土石。\n弄到眼睛里会很痛。','%1试着握住并挥动了%2！','%2掉在了%3的头上！','土掉进%3的眼睛里了！','','土','','','降低命中率（100%）','掉落在各处。',''];
_tr_w["kan"][Number('60') || 0]=['6','打撃','','','4','200','土壤堆积在一起形成了一座小山。\n从上方掉下来会让人觉得很有压迫感。','%1挥动了%2！','%2掉在了%3的头上！','土掉进%3的眼睛里了！','','土山','','','降低命中率（50%）','存在于西南地区。',''];
_tr_w["kan"][Number('61') || 0]=['14','打撃','前６','','5','400','大量土壤堆积而成的山。\n积少成多，积土成山。','%1挥动了%2！','%2掉在了%3的头上！','土掉进%3的眼睛里了！','','大土山','','','降低命中率（50%）','存在于西南地区。','6个方格正面'];
_tr_w["kan"][Number('62') || 0]=['11','打撃','','','3','300','呈茶色的石头A。\n另外还有石头B和石头C。','%1挥动了%2！','%2直接命中了%3！','','','石头A','','','','在西南地区的地上有很多。',''];
_tr_w["kan"][Number('63') || 0]=['12','打撃','','','3','400','呈黑色且外形很好看的石头B。\n拿着突出的部分能很好地挥动。','%1挥动了%2！','%2直接命中了%3！','','','石头B','','','','在西南地区的地上有很多。',''];
_tr_w["kan"][Number('64') || 0]=['15','打撃','','','3','500','呈白色且坚硬的石头C。\n是A～C之中最硬的石头。','%1挥动了%2！','%2直接命中了%3！','','','石头C','','','','在西南地区的地上有很多。',''];
_tr_w["kan"][Number('65') || 0]=['12','打撃','直線２','','3','700','屹立在地上，威风凛凛的巨石。\n硬度意外地比外表看上去的软。','%1挥动了%2！','%2直接命中了%3！','','','巨石','','','','在西南地区的地上有很多。','2个方格直线'];
_tr_w["kan"][Number('66') || 0]=['14','打撃','','','3','200','颜色鲜明的绿石头。\n除了颜色外，和一般的石头没两样。','%1挥动了%2！','%2直接命中了%3！','','','绿石','','','','在充满毒沼的洞窟里地上有很多。',''];
_tr_w["kan"][Number('67') || 0]=['14','打撃','直線２','','3','600','巨大的绿石。\n以挥动的方式使用。','%1挥动了%2！','%2直接命中了%3！','','','绿巨石','','','','在充满毒沼的洞窟里地上有很多。','2个方格直线'];
_tr_w["kan"][Number('68') || 0]=['11','打撃','','','1','100','腐坏并覆满苔藓的石头。\n碰到它就觉得自己好像也要腐坏了。','%1挥动了%2！','%2直接命中了%3！','%3的防御也被破坏了！','','腐坏的石头','','','降低防御力（50%）','在东北地区地上有很多。',''];
_tr_w["kan"][Number('69') || 0]=['7','打撃','','','3','50','石头碎裂后掉在地上的东西。\n很脆弱，难以将它拿来当作武器使用。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','','','石头碎片','','','','在西南地区的地上有很多。',''];
_tr_w["kan"][Number('70') || 0]=['13','斬撃','','','3','700','绽放美丽光辉的神祕水晶。\n用突出的部分刺人会很痛。','%1挥动了%2！','%2突出的部分刺到了%3！','','%2居然释放出魔法了！','水晶','','','随机施放攻击魔法','在充满魔力的洞窟里地上有很多。',''];
_tr_w["kan"][Number('71') || 0]=['15','斬撃','直線２','','3','1200','绽放出耀眼光芒的巨大水晶。\n触碰后可以感觉到魔力。','%1挥动了%2！','%2突出的部分刺到了%3！','','%2居然释放出魔法了！','巨型水晶','','','随机施放攻击魔法','在充满魔力的洞窟里地上有很多。','2个方格直线'];
_tr_w["kan"][Number('72') || 0]=['6','斬撃','','','3','300','自水晶上切下的东西。\n里面似乎没有残留魔力。','%1拿起%2试着挥动了一下！','%2突出的部分刺到了%3！','','','水晶碎片','','','','在充满魔力的洞窟里地上有很多。',''];
_tr_w["kan"][Number('73') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('74') || 0]=['13','打撃','前９','','3','800','高耸在晴空下的悬崖。\n挥舞这个打人还蛮痛的。','%1挥动了%2！','%2直接狠狠地击中了%3！','','','悬崖','','','','在村子外面地上有很多。','9个方格正面'];
_tr_w["kan"][Number('75') || 0]=['9','なし','','','7','200','经常在房间角落形成的蜘蛛网。\n黏性很强，一旦黏在身上就很麻烦。','%1嫌弃地拿起%2并试着挥动了一下。','%2黏在了%3的脸上！','%2还黏在了%3的脚上！','','蜘蛛网','','','减少行动次数（60%）','存在于灵魂旁徨的洞窟里。',''];
_tr_w["kan"][Number('76') || 0]=['5','なし','','','4','400','地面上的裂缝。\n被它打到的人好像也会裂开。','%1拿起%2试着挥动了一下！','%2击中了%3！','%3裂开了！','','裂缝','','','降低防御力（100%）','生长在西南地区。',''];
_tr_w["kan"][Number('77') || 0]=['23','打撃','','','5','600','某个人挖的陷阱，非常深。\n用来困住敌人很方便。','%1拿起%2试着挥动了一下！','%2击中了%3！','%3彻底掉进了%2！','','陷阱','','','2回合不能行动（25%）','存在于村里的某处。',''];
_tr_w["kan"][Number('78') || 0]=['28','打撃','前９','','5','9500','深不见底的巨大坑洞。\n一旦掉下去就回不来了。','%1拿起%2试着挥动了一下！','%2击中了%3！','%2把%3吸进去了！','','巨大坑洞','','','3回合不能行动（20%）','存在于铁洞窟里。','9个方格正面'];
_tr_w["kan"][Number('79') || 0]=['14','なし','前６','','5','2000','住着小鱼和青蛙的小水塘。\n挥动后会造成池水泛滥。','%1拿起%2试着挥动了一下！','','','池水泛滥了！','小池塘','','','产生水,降低雷耐性（100%）','随处可见。','6个方格正面'];
_tr_w["kan"][Number('80') || 0]=['20','なし','前６','','5','2500','漂浮着荷叶，颇为风雅的池塘。\n因为多了荷花，威力比普通池塘高。','%1拿起%2试着挥动了一下！','','','池水泛滥了！','荷塘','','','产生水,降低雷耐性（100%）','离开村子后往南走就能找到。','6个方格正面'];
_tr_w["kan"][Number('81') || 0]=['16','なし','前６','','5','1400','危险的毒沼，积蓄了含有剧毒的液体。\n挥动后周围会发生惨案。','%1拿起%2试着挥动了一下！','','','毒液泛滥了！','毒沼','','','产生毒液,中毒（80%）','在东南地区有很多。','6个方格正面'];
_tr_w["kan"][Number('82') || 0]=['45','なし','前９','','6','19999','占地球表面积70%的生命之源。\n拿起来挥动会引发海啸喔。','%1拿起%2试着挥动了一下！','','','居然引发海啸了！','海','','','产生海啸,降低雷耐性（100%）','在西北方岬角可以一览无遗。','9个方格正面'];
_tr_w["kan"][Number('83') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('84') || 0]=['16','打撃','','','6','800','向围观者传播信息的看板。\n用看板的角打人挺痛的。','%1挥动了%2！','%2的一角击中了%3！','','','看板','','','','存在于村里的某处。',''];
_tr_w["kan"][Number('85') || 0]=['24','斬撃','','','6','1000','特别用于指路的看板。\n非常难看懂，备受批评。','%1挥动了%2！','%2突出的部分劈开了%3！','%3因为很难看懂路标而感到困惑不已！','','路标看板','','','混乱（50%）','存在于村里的某处。',''];
_tr_w["kan"][Number('86') || 0]=['18','打撃','','','7','500','特别用来生火的木棒。\n用粗木柴攻击人还蛮痛的。','%1挥动了%2！','比较粗的%2击中了%3！','%3变成像木柴一样易燃的体质了！','','木柴','','','降低火耐性（100%）','掉落在村里的某处。',''];
_tr_w["kan"][Number('87') || 0]=['28','打撃','','','5','1000','大妈们经常用来聚会的地点。\n一旦掉进去就很难出来。','%1挥动了%2！','%2的墙面击中了%3！','%3掉到水井里面了！','','水井','','','1回合不能行动（25%）','存在于村里的某处。',''];
_tr_w["kan"][Number('88') || 0]=['17','打撃 / 炎','直線３','','5','2000','不断用温和的灯光照亮街道的街灯。\n攻击距离很长，当作武器使用也很方便。','%1挥动了%2！','%2散发着亮光，直接击中了%3！','','','街灯','','','','存在于南方地区。','3个方格直线'];
_tr_w["kan"][Number('89') || 0]=['16','打撃','','','5','1200','保护田地不受乌鸦侵扰的田地英雄。\n总是没有表情，人类看了也会害怕。','%1挥动了%2！','%2面无表情地直接命中了%3！','%3对%2感到害怕！','','稻草人','','','1回合不能行动（40%）','存在于东方地区。',''];
_tr_w["kan"][Number('90') || 0]=['15','打撃','','','5','400','在家门前放置的时尚玩意。\n因为是石制的，用来揍人会痛喔。','%1拿起%2试着挥动了一下！','%2比较尖锐的部分击中了%3！','','','门前小径','','','','掉落在村里的某处。',''];
_tr_w["kan"][Number('91') || 0]=['14','打撃','','','4','200','没有放任何东西的木桶。\n敲击后会发出好听的声音。','%1挥动了%2！','%2咚的一声，撞上了%3！','','','空桶','','','','掉落在村里的某处。',''];
_tr_w["kan"][Number('92') || 0]=['14','打撃','','','5','500','装着水的桶子。\n可以用来泼水喔。','%1挥动了%2！','%2咚的一声，撞上了%3！','','水从%2里溢出来了！','水桶','','','产生水,降低雷耐性（100%）','掉落在村里的某处。',''];
_tr_w["kan"][Number('93') || 0]=['12','打撃','','','5','800','衣服似乎才洗到一半。\n只要挥动它，就能喷出洗涤剂来攻击。','%1挥动了%2！','%2直接命中%3了！','','洗涤剂从%2里喷出来了！','洗衣套组','','','产生洗涤剂,降低冰耐性（100%）','离开村子后往南走，它就掉在那里。',''];
_tr_w["kan"][Number('94') || 0]=['7','打撃','','','4','100','就只是块木板。\n只想得到能够拿去打人。','%1挥动了%2！','%2直接命中%3了！','','','木板','','','','掉落在西南地区。',''];
_tr_w["kan"][Number('95') || 0]=['13','打撃','直線２','','3','700','一直支撑着建筑物的幕后功臣。\n形状正好适合拿来挥舞。','%1挥动了%2！','%2直接命中了%3！','','','柱子','','','','存在于东北地区。','2个方格直线'];
_tr_w["kan"][Number('96') || 0]=['9','斬撃','','','3','600','断掉的柱子。\n断面犹如锯齿。','%1挥动了%2！','%2因断裂而呈锯齿状的一面刺中了%3！','','','断柱','','','','存在于东北地区。',''];
_tr_w["kan"][Number('97') || 0]=['13','打撃','直線２','','3','400','腐烂并长出苔藓的柱子。看得出在制作\n完成后，经历了很长的岁月。','%1挥动了%2！','%2直接命中了%3！','%3的防御也被破坏了！','','腐烂的柱子','','','降低防御力（50%）','存在于东北地区。','2个方格直线'];
_tr_w["kan"][Number('98') || 0]=['9','斬撃','','','3','200','不但腐烂并且断了的柱子，\n彻底完成了柱子的使命。','%1挥动了%2！','%2因断裂而呈锯齿状的一面刺中了%3！','%3的防御也被破坏了！','','腐烂的断柱','','','降低防御力（50%）','存在于东北地区。',''];
_tr_w["kan"][Number('99') || 0]=['12','打撃','','','6','2000','好像写着重要文字的石碑。\n但是要解读文字非常困难。','%1挥动了%2！','%2直接命中了%3！','%3为了解读石碑文字而拼了老命！','','石碑','','','混乱（70%）','在有洞窟的地方。',''];
_tr_w["kan"][Number('100') || 0]=['13','斬撃','','','5','900','为祭祀死者而造的简易坟墓。\n粗暴对待它会遭天谴喔。','%1挥动了%2！','%2的尖端刺中了%3！','粗暴对待坟墓的天谴降在了%3身上……','','木墓','','','致命（5%）','长在村里的某处。',''];
_tr_w["kan"][Number('101') || 0]=['17','打撃','','','5','2400','为祭祀死者而造的石造坟墓。\n粗暴对待它会遭天谴喔。','%1挥动了%2！','%2直接给了%3一记重击！','粗暴对待坟墓的天谴降在了%3身上……','','石墓','','','致命（10%）','存在于灵魂旁徨的洞窟里。',''];
_tr_w["kan"][Number('102') || 0]=['12','打撃','直線２','3','6','3200','可以买到各种饮料的便利机器。\n不知道为什么只有「热饮」。','%1挥动了%2！','热乎乎的饮料淋在了%3头上！','','','自动贩卖机','','','连击3次','──','2个方格直线'];
_tr_w["kan"][Number('103') || 0]=['13','打撃','','','6','800','用来分类垃圾的箱子。 \n不可以挥动它，害垃圾掉得到处都是。','%1挥动了%2！','%2掉出一堆垃圾，同时直接命中了%3！','%3直接吸入了垃圾散发的臭味。','','垃圾桶','','','中毒（100%）','存在于某处的便利商店附近。',''];
_tr_w["kan"][Number('104') || 0]=['14','打撃','','','7','1100','手工制作，非常坚固的石灶。\n添加了火种，可以轻易燃起火焰。','%1挥动了%2！','%2直接命中了%3！','火种黏在了%3身上！','','石灶','','','降低火耐性（100%）','掉落在南方地区。',''];
_tr_w["kan"][Number('105') || 0]=['25','炎','','','6','2200','熊熊燃烧的火焰。\n捡起来会被严重烧伤。','%1忍住烧伤，挥动了%2！','%3像营火一样熊熊燃烧起来了！','','','火','','','','掉落在南方地区。',''];
_tr_w["kan"][Number('106') || 0]=['30','打撃','直線４','','7','8500','飘浮在空中，偶而会旋转的神奇铁板。\n好像可以利用它的旋转当成武器。','%1挥动了%2！','%2在旋转的同时直接命中了%3！','','','神奇的铁板','','','','存在于铁洞窟里。','4个方格直线'];
_tr_w["kan"][Number('107') || 0]=['25','打撃','','','7','4000','每次操作好像都有东西在旋转，\n是个神奇的按钮。出人意料的硬。','%1挥动了%2！','%2在切换的同时直接命中了%3！','','','神奇的按钮','','','','存在于铁洞窟里。',''];
_tr_w["kan"][Number('108') || 0]=['30','打撃','直線２','','7','3800','每次走过去都会改变方向的神奇单行桥。\n也可以拿来当武器使用。','%1挥动了%2！','%2在改变方向的同时直接命中了%3！','','','神奇的桥','','','','存在于充满毒沼的洞窟里。','2个方格直线'];
_tr_w["kan"][Number('109') || 0]=['28','打撃','前９','','7','6000','触碰它就会发光的神奇面板。\n利用这个设计的谜题相当难解。','%1挥动了%2！','%2直接命中了%3！','%3不知道正确的路线！','','神奇的面板','','','混乱（25%）','存在于充满魔力的洞窟里。','9个方格正面'];
_tr_w["kan"][Number('110') || 0]=['32','打撃 / 炎','','','7','5500','给人一种神秘感的灯笼。\n只要点一次火，就会永远亮着。','%1挥动了%2！','%2直接命中了%3！','','%2不知为何自燃了！','神奇的灯笼','','','产生火焰','存在于灵魂旁徨的洞窟里。',''];
_tr_w["kan"][Number('111') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('112') || 0]=['12','打撃','','','1','600','常见的陶壶，在每个家庭里都会有。\n拿去打人会造成暴击并碎掉。','%1挥动了%2！','%2狠狠地砸中了%3！','','','壶','','','一定会暴击','每间房子里都有。',''];
_tr_w["kan"][Number('113') || 0]=['12','打撃','','','1','1200','用盖子遮住内容物的可疑陶壶。\n里面究竟有什么……？','%1挥动了%2！','%2狠狠地砸中了%3！','从壶里喷出来的「马粪」掉进了%3嘴里！','','有盖子的壶','','','一定会暴击,中毒（100%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('114') || 0]=['15','打撃','','','3','700','常见且到处都有的木桶。\n当武器使用时可以拿去砸人。','%1挥动了%2！','%2狠狠地砸中了%3！','','','木桶','','','','掉落在村里的某处。',''];
_tr_w["kan"][Number('115') || 0]=['11','斬撃','','','6','600','方形的小桌子。\n用来放些小东西很方便。','%1挥动了%2！','%2的一角撞到了%3！','','','小桌子','','','','存在于某处的小屋里。',''];
_tr_w["kan"][Number('116') || 0]=['8','打撃','','','6','600','圆圆又可爱的小桌子。\n没有桌角，所以威力有点堪忧。','%1挥动了%2！','%2直接命中了%3！','','','小圆桌','','','','存在于某处的小屋里。',''];
_tr_w["kan"][Number('117') || 0]=['13','斬撃','直線２','','6','900','大小刚好的方桌。\n使用时要用桌角狠狠地撞人。','%1挥动了%2！','%2的一角撞到了%3！','','','中型桌子','','','','存在于村中某处的房子里。','2个方格直线'];
_tr_w["kan"][Number('118') || 0]=['14','斬撃','前６','','6','2000','可以让好几个人坐在一起的桌子。\n能把这个当成武器使用才叫独当一面。','%1挥动了%2！','%2的一角撞到了%3！','','','大桌子','','','','每间房子里都有。','6个方格正面'];
_tr_w["kan"][Number('119') || 0]=['13','斬撃','直線３','','6','1400','稍微有点长的桌子。\n用桌角撞人会稍微有点痛。','%1挥动了%2！','%2的一角撞到了%3！','','','有点长的桌子','','','','存在于村中某处的房子里。','3个方格直线'];
_tr_w["kan"][Number('120') || 0]=['15','斬撃','直線４','','6','2500','长得不得了的桌子。\n当作武器的话，它的攻击距离也很出色。','%1挥动了%2！','%2的一角撞到了%3！','','','长桌','','','','存在于村中某处的房子里。','4个方格直线'];
_tr_w["kan"][Number('121') || 0]=['12','斬撃','','','6','1100','与花瓶成套的桌子。\n看着就让人觉得平静。','%1挥动了%2！','%2的一角撞到了%3！','%3受花瓶抚慰，失去战意了！','','花瓶桌','','','降低攻击力（60%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('122') || 0]=['15','斬撃','','','6','1200','与镜子成套的桌子。\n这面镜子只会映照出真实的景象。','%1挥动了%2！','%2的一角撞到了%3！','镜子反射的光芒射中%3了！','','镜子桌','','','重置能力强化','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('123') || 0]=['17','斬撃','直線２','','6','3600','怎么看都像教会里会有的桌子。\n圣经里面似乎有神奇的力量。','%1挥动了%2！','%2的一角撞到了%3！','某种神圣的力量对%3起作用了！','','教会桌','','','降低火耐性（50%）,降低冰耐性（50%）,降低雷耐性（50%）','存在于某处的教会里。','2个方格直线'];
_tr_w["kan"][Number('124') || 0]=['17','斬撃','','','6','4000','与诡异水晶成套的桌子。\n水晶虽然诡异，却感觉得到魔力。','%1挥动了%2！','%2的一角撞到了%3！','','水晶居然施放出魔法了！','水晶桌','','','随机施放攻击魔法','存在于某处奇怪的房子里。',''];
_tr_w["kan"][Number('125') || 0]=['8','打撃','','','6','300','看起来很便宜，坐上去却很舒服的圆椅。\n似乎不太适合作为武器。','%1挥动了%2！','%2直接命中了%3！','%3坐在%2上开始休息！','','圆椅','','','1回合不能行动（50%）','在村中某处人家里有很多。',''];
_tr_w["kan"][Number('126') || 0]=['10','斬撃','','','6','300','四角的小椅子。\n有角，所以比圆椅好用。','%1挥动了%2！','%2的一角撞到了%3！','%3坐在%2上开始休息！','','四角椅','','','1回合不能行动（50%）','在村中某处人家里有很多。',''];
_tr_w["kan"][Number('127') || 0]=['12','斬撃','直線２','','6','1500','看起来很高级，感觉很不错的椅子。\n坐起来也非常舒适。','%1挥动了%2！','%2的靠背一角撞到了%3！','%3坐在%2上放松地休息了！','','感觉不错的椅子','','','1回合不能行动（80%）','存在于某处的小屋里。','2个方格直线'];
_tr_w["kan"][Number('128') || 0]=['8','打撃','直線２','3','6','1000','整理了各种书的书架。\n使用时要让书掉下来砸到敌人。','%1挥动了%2！','无数的书砸在了%3的头上！','%3读完各种书后变聪明了！','','书架','','','连击3次,虚弱（30%）','每间房子里都有。','2个方格直线'];
_tr_w["kan"][Number('129') || 0]=['30','斬撃','直線２','','6','1500','收纳衣服的柜子。\n左脚的小姆指撞到它会痛得要命。','%1挥动了%2！','%2的一角砸中%3左脚的小姆指了！','%3按住左脚小姆指，好像很痛苦！','','五斗柜','','','减少行动次数（30%）','存在于村中某处的房子里。','2个方格直线'];
_tr_w["kan"][Number('130') || 0]=['9','打撃','直線２','3','6','1200','挂着衣服的柜子。\n里面其实大有乾坤。','%1挥动了%2！','无数的衣服砸在了%3的头上！','%3因为被太多衣服压住，很难行动！','','衣柜','','','连击3次,降低回避率（100%）','存在于村中某处的房子里。','2个方格直线'];
_tr_w["kan"][Number('131') || 0]=['7','打撃','直線２','3','6','1400','不知为何放有面包的柜子。\n法式面包很硬，所以能当作武器。','%1挥动了%2！','无数的法式面包砸在了%3的头上！','%3开始疯狂地吃面包！','','放了面包的柜子','','','连击3次,1回合不能行动（50%）','存在于某处的小屋里。','2个方格直线'];
_tr_w["kan"][Number('132') || 0]=['8','打撃','直線２','3','6','900','放了莫名其妙的药等物品的柜子。\n总之挥动使用就对了。','%1挥动了%2！','无数莫名其妙的东西砸在了%3的头上！','%3对莫名其妙的东西感到害怕！','','莫名其妙的柜子','','','连击3次,混乱（50%）','存在于某处的小屋里。','2个方格直线'];
_tr_w["kan"][Number('133') || 0]=['12','打撃','','','6','600','安装在墙上放书的置物架。\n老实说在这个位置上没有人拿得到书。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','%3读完书后变聪明了！','','放书的架子','','','虚弱（30%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('134') || 0]=['13','打撃','','','6','800','放了各种调味料，很方便的置物架。\n要小心调味料过期。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','调味料的保质期限居然是两年前！','','调味料架','','','中毒（30%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('135') || 0]=['12','打撃','','','6','700','可爱花朵与花瓶的套组。\n美丽的花配上美丽的花瓶很重要。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','%3受花抚慰，失去战意了！','','花瓶','','','降低攻击力（50%）','存在于某处的教会里。',''];
_tr_w["kan"][Number('136') || 0]=['14','打撃','','','5','400','随便放进了一些东西的木箱。\n使用时正常地砸下去即可。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','','','木箱','','','','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('137') || 0]=['6','打撃','前６','5','3','1800','许多木箱层层叠起的高塔。\n使用时要让它顺利地在敌人头上崩塌。','%1挥动了%2！','无数的木箱砸在了%3的头上！','','','木箱塔','','','连击5次','存在于某处的小屋里。','6个方格正面'];
_tr_w["kan"][Number('138') || 0]=['8','打撃','','','3','300','变得破破烂烂的木箱。\n其实很脆弱，不太好用。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','','','破烂木箱','','','','存在于某处的小屋里。',''];
_tr_w["kan"][Number('139') || 0]=['18','打撃','直線２','','5','700','跟人差不多大的大木箱。\n如果能挥动，就会展现出不错的威力。','%1挥动了%2！','%2直接命中了%3！','','','大木箱','','','','存在于村里的某处。','2个方格直线'];
_tr_w["kan"][Number('140') || 0]=['12','打撃','','','5','500','塞得满满的米袋。\n挥动时不要忘了感谢农民伯伯。','%1拿起%2试着挥动了一下！','塞得满满的%2直接命中了%3！','','','米袋','','','','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('141') || 0]=['9','打撃','直線２','3','5','1400','满满的米袋堆叠起来的高塔。\n使用时要在敌人的头上引发崩塌。','%1挥动了%2！','无数的米袋砸在了%3的头上！','','','米袋塔','','','连击3次','存在于村中某处的房子里。','2个方格直线'];
_tr_w["kan"][Number('142') || 0]=['5','打撃','','5','6','1100','有许多看起来很好吃的苹果，\n但究竟是……？','%1拿起%2试着挥动了一下！','%2砸在了%3的头上！','居然全都是毒苹果！','','许多苹果','','','连击5次,剧毒（60%）','存在于某处奇怪的房子里。',''];
_tr_w["kan"][Number('143') || 0]=['16','打撃','直線２','','5','1400','清理得很干净的流理台。\n试着挥动后发现攻击距离还挺远。','%1挥动了%2！','%2直接命中了%3！','','','流理台','','','','存在于村中某处的房子里。','2个方格直线'];
_tr_w["kan"][Number('144') || 0]=['16','打撃','','','5','900','干净且没有油污的燃气灶。\n会发出火焰，所以当作武器还挺强的。','%1挥动了%2！','%2直接命中了%3！','','用大火加热到焦糖色吧！','燃气灶','','','产生火焰','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('145') || 0]=['18','斬撃','','','6','900','有点脏的换气扇。\n旋转的扇叶好像能用作武器。','%1拿起%2试着挥动了一下！','%2旋转时把%3卷进去了！','','','换气扇','','','','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('146') || 0]=['23','斬撃','','','5','800','东西切到一半的砧板与菜刀。\n菜刀还满锋利的。','%1挥动了%2！','把%3放在砧板上剁碎了！','','','砧板套组','','','降低斩击耐性（100%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('147') || 0]=['11','打撃','','3','6','700','有平底锅和勺子等各种厨具。\n当然也能当作武器使用。','%1挥动了%2！','用各种厨具揍了%3一顿！','','','厨具套组','','','连击3次','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('148') || 0]=['20','打撃 / 氷','直線２','','6','1900','有一堆剩余材料的冰箱。\n或许能用冷气冻住敌人。','%1挥动了%2！','%2直接命中了%3！','','冰箱门打开后冷气外泄了！','冰箱','','','产生冷气','存在于村中某处的房子里。','2个方格直线'];
_tr_w["kan"][Number('149') || 0]=['12','打撃','','','6','700','将鱼从头到尾都烤熟的一道菜。\n有许多鱼刺。','%1试着挥动了%2！※请爱惜食物！','%2猛烈地撞上了%3！','%3被鱼刺噎到了！','','烤鱼','','','封印魔法（50%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('150') || 0]=['14','打撃','','','6','600','简单美味的炒菜。\n热腾腾的，好像会烫伤舌头。','%1试着挥动了%2！※请爱惜食物！','热腾腾的青菜袭击了%3！','','','炒青菜套餐','','','','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('151') || 0]=['12','打撃','','','6','750','可以自由添饭的划算套餐。\n但总觉得猪肉的颜色有点红。','%1试着挥动了%2！※请爱惜食物！','%2猛烈地撞上了%3！','%2居然没有熟。%3拉肚子了！','','姜汁烧猪肉套餐','','','中毒（30%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('152') || 0]=['7','打撃','','','6','320','喝了它，下午就能继续努力的饮料。\n咖啡因会赶跑瞌睡虫。','%1用%2干杯了！','%3：「干杯～！！」','%3彻底清醒过来了！','','饭后咖啡','','','提升睡眠耐性（100%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('153') || 0]=['12','打撃','','','6','3000','味道浓郁而厚重的红酒。\n光是闻到香气就会心情愉快。','%1用%2干杯了！','%3：「干杯～！！」','%3沉醉在红酒的香气中了！','','高级红酒','','','封印普通攻击外的攻击（50%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('154') || 0]=['21','打撃','','','6','50','像是一道料理又好像不是的某种东西。\n为什么会端出这个给客人呢……','%1拿起%2试着挥动了一下！','%3把%2吸进了嘴里！','%3的样子明显怪怪的！','','像是料理的东西','','','中毒（75%）,封印魔法（50%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('155') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('156') || 0]=['14','打撃','直線２','','6','2400','睡起来超级舒服的柔软床铺。\n会诱使敌人入眠。','%1挥动了%2！','%2砰的一声，击中了%3！','%2把%3吸引过去了！','','床','','','睡眠（55%）','存在于村中某处的房子里。','2个方格直线'];
_tr_w["kan"][Number('157') || 0]=['18','打撃','前６','','6','7000','可以两个人一起睡觉的双人床。\n睡起来舒服极了。','%1挥动了%2！','%2砰的一声，击中了%3！','%2把%3吸引过去了！','','双人床','','','睡眠（90%）','存在于村中某处的房子里。','6个方格正面'];
_tr_w["kan"][Number('158') || 0]=['7','打撃','直線２','','4','300','很朴素，感觉很不好睡的棉被。\n隔天早上关节会酸痛。','%1挥动了%2！','%2啪的一声，击中了%3！','%3决定只有今晚忍着盖%2睡觉！','','破旧棉被','','','睡眠（25%）','存在于某处的小屋里。','2个方格直线'];
_tr_w["kan"][Number('159') || 0]=['18','打撃','直線２','','6','1400','散发奢华气氛的暖炉。\n要握住上面较细小的部分来挥动。','%1挥动了%2！','%2直接命中了%3！','','%2不知为何自燃了！','暖炉','','','产生火焰','存在于村中某处的房子里。','2个方格直线'];
_tr_w["kan"][Number('160') || 0]=['11','斬撃','','','5','400','某个人写的日记。\n阅读时会回想起自己的黑历史。','%1拿起%2试着挥动了一下！','%2割伤了%3的左手大姆指！','%3回想起黑历史了……','','日记','','','混乱（50%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('161') || 0]=['5','斬撃','','','4','200','写了一些东西的纸。\n被边缘割到手会有点痛。','%1拿起%2试着挥动了一下！','%2的边缘割伤了%3的左手背！','','','某张纸','','','','存在于某处的小屋里。',''];
_tr_w["kan"][Number('162') || 0]=['13','打撃','','','4','500','不知道装了什么东西的袋子。\n除了拿来打人外没有其他用处。','%1拿起%2试着挥动了一下！','%2直接命中%3了！还是不知道里面装了什么东西。','','','某个袋子','','','','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('163') || 0]=['27','打撃','前６','','6','3700','年代久远，但擦得很干净的钢琴。\n弹奏的同时拿来殴打敌人吧。','%1挥动了%2！','%2的键盘撞到%3，弹奏了出不协和音！','%3听到不协和音后变得很不舒服！','','钢琴','','','重置能力强化','存在于村中某处的房子里。','6个方格正面'];
_tr_w["kan"][Number('164') || 0]=['15','打撃','','','6','1000','包装得很可爱的盒子。\n好像能从里面感受到魔力。','%1挥动了%2！','%2直接命中了%3！','','%2里面居然施放出了魔法！','礼物盒','','','随机施放攻击魔法','存在于某处奇怪的房子里。',''];
_tr_w["kan"][Number('165') || 0]=['12','打撃','','','5','1500','魔女家里的衣服。\n捡起来时稍微有点罪恶感。','%1试着挥动了%2！散发出一股香味！','%2击中了%3的脸！','','','魔女的衣服','','','','存在于某处奇怪的房子里。',''];
_tr_w["kan"][Number('166') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('167') || 0]=['11','打撃','前６','3','6','1400','便利店里摆放日用品等东西的商品架。\n我是捡起来当武器的，所以不算偷窃。','%1挥动了%2！','各种商品砸在了%3的头上！','商品居然全都是黑心货！','','便利店商品架','','','连击3次,混乱（50%）','存在于某处的便利店里。','6个方格正面'];
_tr_w["kan"][Number('168') || 0]=['10','打撃 / 氷','前６','3','6','2200','存放各种冷冻食品的专柜。\n这股冷气好像能当作武器使用。','%1挥动了%2！','冷冻食品砸在了%3的头上！','','冷气外泄了！','冷冻食品柜','','','连击3次,产生冷气','存在于某处的便利店里。','6个方格正面'];
_tr_w["kan"][Number('169') || 0]=['12','打撃','前６','3','6','1800','摆着各式各样便当的专区。\n全部都贴着「半价」标签。','%1挥动了%2！','各种便当砸在了%3的头上！','便当居然全都过期了！','','便当卖场','','','连击3次,中毒（50%）','存在于某处的便利店里。','6个方格正面'];
_tr_w["kan"][Number('170') || 0]=['13','打撃','','','7','3000','我家里很重要的电话机。\n要小心伪装成亲友的诈骗电话。','%1挥动了%2！','%2直接命中了%3！','%3接到伪装成亲友的诈骗电话，上当了！','','电话','','','混乱（100%）','在你家里。',''];
_tr_w["kan"][Number('171') || 0]=['9','打撃','前９','','6','400','相当破旧的地毯，\n还散发出一点怪味。','%1挥动了%2！','%3被%2绊倒，脸用力地撞到地板上了！','%3被绊倒，行动变迟缓了！','','破旧地毯','','','降低回避率（75%）','存在于某处的小屋里。','9个方格正面'];
_tr_w["kan"][Number('172') || 0]=['14','打撃','前９','','6','800','可爱的蓝色地毯。\n触感很柔软。','%1挥动了%2！','%3被%2绊倒，脸用力地撞到地板上了！','%3被绊倒，行动变迟缓了！','','蓝色地毯','','','降低回避率（50%）','存在于村中某处的房子里。','9个方格正面'];
_tr_w["kan"][Number('173') || 0]=['18','打撃','前９','','6','2500','豪华的大红色地毯。\n使用最顶级的布料制作而成。','%1挥动了%2！','%3被%2绊倒，脸用力地撞到地板上了！','%3被绊倒，行动变迟缓了！','','豪华的地毯','','','降低回避率（25%）','存在于某处的教会里。','9个方格正面'];
_tr_w["kan"][Number('174') || 0]=['13','打撃','前９','','6','1100','散发出神奇氛围的地毯。\n但布料意外廉价。','%1挥动了%2！','%3被%2绊倒，脸用力地撞到地板上了！','%3被绊倒，行动变迟缓了！','','诡异的地毯','','','降低回避率（100%）','存在于某处奇怪的房子里。','9个方格正面'];
_tr_w["kan"][Number('175') || 0]=['16','打撃','','','5','800','不断行进，显示正确时间的时钟。\n试着用这个揍人好像也很有趣。','%1拿起%2试着挥动了一下！','%2在时间一分一秒流逝的同时直接命中了%3！','%3周围的时空居然产生了变化！','','时钟','','','减少行动次数（50%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('176') || 0]=['21','斬撃','','','7','1200','可以挂在墙上的宝剑复制品。\n但是好像还挺锋利的。','%1挥动了%2！','%2正常地劈开了%3！','','','挂在墙上的剑','','','','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('177') || 0]=['17','打撃','','','7','900','可以挂在墙上的盾牌复制品。\n虽然是复制品，却还挺坚硬的。','%1挥动了%2！','%2直接狠狠地击中了%3！','','','挂在墙上的盾','','','降低防御力（50%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('178') || 0]=['24','斬撃','','','7','1500','可以挂在墙上的斧头复制品。\n非常重，就像是真正的斧头一样。','%1挥动了%2！','%2正常地劈开了%3！','','','挂在墙上的斧头','','','','存在于某处的小屋里。',''];
_tr_w["kan"][Number('179') || 0]=['16','斬撃','前３','','6','600','手绘的地图。\n是冒险的必需品。','%1拿起%2试着挥动了一下！','%2击中了%3！','%3不擅长看地图！','','地图','','','混乱（75%）','存在于某处的小屋里。','3个方格正面'];
_tr_w["kan"][Number('180') || 0]=['14','斬撃','','','5','700','画有美丽花朵的画。\n画框很硬，可以当武器。','%1拿起%2试着挥动了一下！','画框直接命中了%3！','%3受美丽花朵抚慰，失去战意了！','','花的画','','','降低攻击力（60%）','存在于某处奇怪的房子里。',''];
_tr_w["kan"][Number('181') || 0]=['15','斬撃','','','5','800','陌生人的脸。你是谁呀？\n艺术真是难懂。','%1拿起%2试着挥动了一下！','画框直接命中了%3！','%3开始思考何谓艺术了！','','肖像画','','','混乱（40%）','存在于某处奇怪的房子里。',''];
_tr_w["kan"][Number('182') || 0]=['13','斬撃','直線２','','5','1000','像国旗一样鲜红的旗子。\n虽然威力存疑，却很帅气。','%1挥动了%2！','%2在飘扬的同时击中了%3！','','','红旗','','','','存在于灵魂旁徨的洞窟里。','2个方格直线'];
_tr_w["kan"][Number('183') || 0]=['7','斬撃','直線２','','3','300','变得破破烂烂的旗子。\n几乎就只是垃圾。','%1挥动了%2！','%2在飘扬的同时击中了%3！','','','破烂旗子','','','','存在于灵魂旁徨的洞窟里。','2个方格直线'];
_tr_w["kan"][Number('184') || 0]=['16','斬撃','前６','','5','900','几乎叫人怀疑品味的鲜红色窗帘。\n遮光性好像很好。','%1拿起%2试着挥动了一下！','%2轻飘飘地击中了%3！','%2挡住了%3的视线！','','鲜红色窗帘','','','降低命中率（90%）','存在于某处奇怪的房子里。','6个方格正面'];
_tr_w["kan"][Number('185') || 0]=['12','打撃','','','5','700','擦得很干净的窗户。\n因为是双重结构，出乎意料地坚固。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','','','窗户','','','','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('186') || 0]=['12','打撃','','','5','900','附有绿色窗帘的窗户。\n可以用窗帘遮蔽敌人的视线。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','窗帘挡住了%3的视线！','','绿窗帘窗户','','','降低命中率（100%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('187') || 0]=['13','打撃','','','5','1000','附有红色窗帘，稍微大一点的窗户。\n非常坚固，即使敲打它也不会碎裂。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','窗帘挡住了%3的视线！','','红窗帘窗户','','','降低命中率（100%）','存在于村中某处的房子里。',''];
_tr_w["kan"][Number('188') || 0]=['15','打撃','','','5','1600','画着漂亮图案的窗户。\n说到教会就会想到这个。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','','','彩绘玻璃窗','','','','存在于某处的教会里。',''];
_tr_w["kan"][Number('189') || 0]=['20','打撃','','','4','800','安全性似乎不佳的门。\n当作武器则很可靠。','%1用力挥动了%2！','%2直接命中了%3！','','','门','','','','每间房子的入口处都有。',''];
_tr_w["kan"][Number('190') || 0]=['16','打撃','前６','','6','1200','结实的铁制牢房。\n挥动起来会是十分强大的武器。','%1挥动了%2！','%2直接命中了%3！','%3有种被关进牢房里的感觉！','','牢房','','','1回合不能行动（30%）','存在于铁洞窟里。','6个方格正面'];
_tr_w["kan"][Number('191') || 0]=['17','打撃','直線２','','6','900','坚固的门，可以用牢房钥匙打开。\n强度有保证。','%1挥动了%2！','%2直接命中了%3！','%3有种被关进牢房里的感觉！','','牢房的门','','','1回合不能行动（20%）','存在于铁洞窟里。','2个方格直线'];
_tr_w["kan"][Number('192') || 0]=['28','打撃','直線２','','7','6000','神情勇猛的战士雕像。\n手上的剑好像可以当成武器使用。','%1挥动了%2！','%2的剑劈开了%3！','','','战士雕像','','','','掉落在东方地区。','2个方格直线'];
_tr_w["kan"][Number('193') || 0]=['24','打撃','直線２','','7','4500','永远会面向右边的神奇女神像。\n借助神圣之力做得相当坚固。','%1挥动了%2！','%2给了%3一记头槌！','某种神圣的力量对%3起作用了！','','朝右女神像','','','虚弱（15%）','存在于某处的教会里。','2个方格直线'];
_tr_w["kan"][Number('194') || 0]=['24','打撃','直線２','','7','4500','永远会面向左边的神奇女神像。\n用它施展头槌好像会很厉害。','%1挥动了%2！','%2给了%3一记头槌！','某种神圣的力量对%3起作用了！','','朝左女神像','','','封印魔法（50%）','存在于某处的教会里。','2个方格直线'];
_tr_w["kan"][Number('195') || 0]=['40','斬撃 / 炎','前３','','7','18000','仿照凶暴的龙打造出的雕像，魄力十足。\n里面封印了龙的灵魂。','%1挥动了%2！','%2的牙齿刺中了%3！','','%2居然吐出了火焰！','龙雕像','','','产生火焰','存在于灵魂旁徨的洞窟里。','3个方格正面'];
_tr_w["kan"][Number('196') || 0]=['18','打撃','','','6','3000','里面好像会有稀有道具的宝箱。\n作为武器似乎除了砸人外没有其他用法。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','','','宝箱','','','','掉落在各处。',''];
_tr_w["kan"][Number('197') || 0]=['19','打撃','直線２','','4','1800','用黄金做成的柱子，闪亮到刺眼。\n直视它好像对眼睛不好。','%1挥动了%2！','%2在发光的同时直接命中了%3！','%3不小心直视了黄金的光辉！','','黄金柱子','','','降低命中率（50%）','存在于闪着金黄光芒的洞窟里。','直線２マス'];
_tr_w["kan"][Number('198') || 0]=['15','斬撃','','','4','900','即使断掉了也在发光的黄金柱子。\n虽然是黄金，强度却无可挑剔。','%1挥动了%2！','%2因断裂而呈锯齿状的一面刺中了%3！','%3不小心直视了黄金的光辉！','','黄金断柱','','','降低命中率（30%）','存在于闪着金黄光芒的洞窟里。',''];
_tr_w["kan"][Number('199') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('200') || 0]=['20','打撃','','','7','500','可爱的红色熊娃娃。\n是娃娃……对吧？','%1挥动了%2！','%2给了%3一记头槌！','','','红熊','Catherine','','','坐在村中某处人家里。',''];
_tr_w["kan"][Number('201') || 0]=['20','打撃','','','7','500','酷酷的蓝色熊娃娃。\n是娃娃……对吧？','%1挥动了%2！','%2给了%3一记头槌！','','','蓝熊','James','','','坐在南方岬角。',''];
_tr_w["kan"][Number('202') || 0]=['20','打撃','','','7','500','阴郁的黑色熊娃娃。\n是娃娃……对吧？','%1挥动了%2！','%2给了%3一记头槌！','','','黑熊','Tom','','','坐在东方地区。',''];
_tr_w["kan"][Number('203') || 0]=['20','打撃','','','7','500','可爱的褐色熊娃娃。\n是娃娃……对吧？','%1挥动了%2！','%2给了%3一记头槌！','','','褐熊','Brown','','','坐在西北地区。',''];
_tr_w["kan"][Number('204') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('205') || 0]=['30','打撃','前９','','7','2500','我家房子的右半边，我常年生活于此。\n快速挥动就能当成武器使用。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','右厢房','','','','你家的右半边房子。','9个方格正面'];
_tr_w["kan"][Number('206') || 0]=['30','打撃','前９','','7','3200','我家房子的左半边，开武器店的地方。\n快速挥动就能当成武器使用。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','左厢房','','','','你家的左半边房子。','9个方格正面'];
_tr_w["kan"][Number('207') || 0]=['29','打撃','前９','','7','2200','不相关的外人住的房子。\n当武器用不知道会不会挨骂，真是不安。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','民宅','','','','村里有数间这样的房子。','9个方格正面'];
_tr_w["kan"][Number('208') || 0]=['27','打撃','前９','','7','1900','樵夫住的破旧房子。\n虽然又破又脏，但可以当作武器。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','樵夫家','','','','掉落在西方地区。','9个方格正面'];
_tr_w["kan"][Number('209') || 0]=['25','打撃','前９','','7','2800','好像有魔女居住的奇怪房子。\n屋顶的颜色很鲜艳。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','魔女家','','','','藏在西方的森林里。','9个方格正面'];
_tr_w["kan"][Number('210') || 0]=['32','打撃','前９','','7','3300','建设在沿海地区的神圣教会。\n一边感谢神，一边挥动它吧。','%1挥动了%2！','%2非常猛烈地砸中了%3！','某种神圣的力量对%3起作用了！','','教会','','','重置能力强化','建在东北方沿海地区。','9个方格正面'];
_tr_w["kan"][Number('211') || 0]=['31','打撃','前９','','7','3000','什么都买得到，很方便的便利店。\n因为很便利，也可以当作武器喔。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','便利店','','','','建在北方地区。','9个方格正面'];
_tr_w["kan"][Number('212') || 0]=['25','打撃','前９','','7','1500','某个人设置的简易帐篷。\n可以直接拿走，是很方便的武器。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','帐篷','','','','设在东方地区。','9个方格正面'];
_tr_w["kan"][Number('213') || 0]=['33','打撃','前９','','7','4500','居民全是坚硬敌人的洞窟。\n洞窟本身也很硬，所以当作武器会很强。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','钢铁洞窟','','','','伫立在西南地区。','9个方格正面'];
_tr_w["kan"][Number('214') || 0]=['24','打撃','前９','','7','4400','住着弱小却聪明的敌人的洞窟。\n颜色看起来有毒。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','弱者洞窟','','','','伫立在东南地区。','9个方格正面'];
_tr_w["kan"][Number('215') || 0]=['28','打撃','前９','','7','4800','栖息着喜欢魔法的敌人的洞窟。\n洞窟本身也积蓄了魔力。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','幻术洞窟','','','随机施放【极】攻击魔法','伫立在西方的森林里。','9个方格正面'];
_tr_w["kan"][Number('216') || 0]=['26','打撃','前９','','7','5200','灵魂栖息在废弃的城堡而形成了洞窟。\n可以感受到可怕的诅咒之力。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','灵魂洞窟','','','致命（5%）','伫立在东北地区。','9个方格正面'];
_tr_w["kan"][Number('217') || 0]=['35','打撃','前９','','7','7500','黄金洞窟，传说中的武器沉睡在这里。\n其实洞窟本身也是很厉害的武器。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','传说洞窟','','','','可以从村子的北边感觉到气息。','9个方格正面'];
_tr_w["kan"][Number('218') || 0]=['38','打撃','前９','','7','8700','魔王居住的邪恶城堡。\n当作武器使用就跟魔王一样强大。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','魔王城','','','','耸立在极北地区。','9个方格正面'];
_tr_w["kan"][Number('219') || 0]=['45','打撃','前９','','7','9800','总是很和平的和平村。自出生起，\n我就住在这个我最爱的村子里。','%1挥动了%2！','%2非常猛烈地砸中了%3！','','','和平村','','','','你住的村子。','9个方格正面'];
_tr_w["kan"][Number('220') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('221') || 0]=['8','打撃','直線２','','5','600','比普通裂缝还要大的裂缝。\n比较大所以威力也比较高。','%1拿起%2试着挥动了一下！','%2击中了%3！','%3裂开了！','','大型裂缝','','','降低防御力（100%）','存在于铁洞窟里。','2个方格直线'];
_tr_w["kan"][Number('222') || 0]=['6','打撃','','','5','300','墙壁下方的空洞。\n是不是因为老旧而出现的呢？','%1拿起%2试着挥动了一下！','%2击中了%3！','%3裂开了！','','墙壁凹洞','','','降低防御力（70%）','存在于铁洞窟里。',''];
_tr_w["kan"][Number('223') || 0]=['4','打撃','','','5','500','外层剥落而露出内层的墙壁。\n很脆弱，当作武器有点微妙。','%1拿起%2试着挥动了一下！','%2击中了%3！','','','裸露的墙壁','','','','存在于灵魂旁徨的洞窟里。',''];
_tr_w["kan"][Number('224') || 0]=['2','打撃','','','3','100','地面黑块的一部分。\n好像很脏，所以我不太想碰。','%1拿起%2试着挥动了一下！','%3变得有点黑了！','黑块的一部分掉进了%3的眼睛里！','','黑块','','','降低命中率（100%）','掉落在极北地区。',''];
_tr_w["kan"][Number('225') || 0]=['2','打撃','','','3','100','湿滑的绿色地面。\n在阴湿的地方有很多。','%1拿起%2试着挥动了一下！','%3变得有点绿了！','湿滑的绿色物体缠绕在了%3的脚上！','','绿色地面','','','降低回避率（100%）','存在于铁洞窟里。',''];
_tr_w["kan"][Number('226') || 0]=['2','打撃','','','3','200','到处都有的道路。\n有了这个就很好走路。','%1拿起%2试着挥动了一下！','%2击中了%3！','','','路','','','','到处都掉落了很多。',''];
_tr_w["kan"][Number('227') || 0]=['4','斬撃','','','3','400','石制的白砖。\n四块白砖是一组。','%1拿起%2试着挥动了一下！','%2的一角劈开了%3！','','','白砖','','','','存在于某处的便利店里。',''];
_tr_w["kan"][Number('228') || 0]=['3','斬撃','','','3','200','破破烂烂的白砖，\n因为长年老化而裂开了。','%1拿起%2试着挥动了一下！','%2的一角劈开了%3！','','','裂开的白砖','','','','存在于灵魂旁徨的洞窟里。',''];
_tr_w["kan"][Number('229') || 0]=['5','斬撃','','','3','500','石制的蓝砖。\n比白砖硬了一点。','%1拿起%2试着挥动了一下！','%2的一角劈开了%3！','','','蓝砖','','','','存在于东北地区。',''];
_tr_w["kan"][Number('230') || 0]=['3','斬撃','','','3','250','变得老旧而裂开的蓝砖。\n一看就觉得破破烂烂。','%1拿起%2试着挥动了一下！','%2的一角劈开了%3！','','','裂开的蓝砖','','','','存在于东北地区。',''];
_tr_w["kan"][Number('231') || 0]=['5','斬撃','','','3','400','很干净的地板。\n似乎有在好好扫地。','%1拿起%2试着挥动了一下！','%2的一角劈开了%3！','','','地板','','','','每间房子里都有。',''];
_tr_w["kan"][Number('232') || 0]=['8','斬撃','','','3','500','华丽的格纹地砖。\n品味有一点……没事。','%1拿起%2试着挥动了一下！','%2的一角劈开了%3！','','','格纹地砖','','','','存在于某处奇怪的房子里。',''];
_tr_w["kan"][Number('233') || 0]=['6','斬撃','','','3','300','教会的木头地板。\n有种大自然的气息。','%1拿起%2试着挥动了一下！','%2的一角劈开了%3！','','','教会地板','','','','存在于某处的教会里。',''];
_tr_w["kan"][Number('234') || 0]=['12','打撃','','','6','1500','可以保养武器的铁砧。\n摆放在大树墩上面。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','','','铁砧','','','','摆放在各处。',''];
_tr_w["kan"][Number('235') || 0]=['7','なし','','','2','600','把弱者洞窟里的巨大毒沼\n切下一小部分后得到的东西。','%1拿起%2试着挥动了一下！','','','','毒沼（小）','','','中毒（100%）','存在于充满毒沼的洞窟里。',''];
_tr_w["kan"][Number('236') || 0]=['15','打撃','','','4','2000','在魔界制造的邪恶灯笼。\n紫色火焰相当有魔界的感觉。','%1挥动了%2！','%2直接命中了%3！','','黑暗的火焰从%2里溢出了！','魔界灯笼','','','产生火焰','存在于魔族之王居住的城堡里。',''];
_tr_w["kan"][Number('237') || 0]=['12','斬撃','直線２','4','20','15000','熊给的神奇宝剑。\n握住剑就会回想起与那些熊之间的回忆。','%1挥动了%2！','','','','熊之剑','','','连击4次','可以从四只熊手中获得。','2个方格直线'];
_tr_w["kan"][Number('238') || 0]=['11','打撃','直線２','','15','9000','可以骑着它在空中飞翔的厉害扫帚。\n看上去就像是魔女会骑的样子。','%1挥动了%2！','%2直接命中了%3！','','','飞天扫帚','','','能在空中飞翔','你的妹妹偷偷收藏着。','2个方格直线'];
_tr_w["kan"][Number('239') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('240') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('241') || 0]=['13','打撃','','','3','499','好像是某个人的尸骨。\n即使跟他说话，他也没有反应。','%1挥动了%2！','%2直接命中了%3！','%3遭到了诅咒！','','尸骨','','','致命（5%）','离开村子后往南走，它就掉在那里。',''];
_tr_w["kan"][Number('242') || 0]=['13','打撃','','','3','399','好像是动物的尸骨。\n即使叫它握手，它也没有回应。','%1挥动了%2！','%2直接命中了%3！','%3遭到了诅咒！','','动物的尸骨','','','致命（5%）','掉落在西南地区。',''];
_tr_w["kan"][Number('243') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('244') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('245') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('246') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('247') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('248') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('249') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('250') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('251') || 0]=['en','打撃','','','6','500','在外面徘徊的神祕黏液。\n看到人就会毫无理由地攻击。','上吧！%2！','%2弹跳着冲撞了%3！','','','史莱姆','','','','经常出现在村子附近。',''];
_tr_w["kan"][Number('252') || 0]=['en','打撃','','','6','700','变成红色的史莱姆亚种，\n而且更加凶暴。','上吧！%2！','%2弹跳着冲撞了%3！','','','红色史莱姆','','','','会出现在北方地区。',''];
_tr_w["kan"][Number('253') || 0]=['en','打撃','','','6','2000','发出金色光彩的史莱姆稀有种。\n如果拿去卖，价格好像会很不错。','上吧！%2！','%2弹跳着冲撞了%3！','','','黄金史莱姆','','','','会出现在闪着金黄光芒的洞窟里。',''];
_tr_w["kan"][Number('254') || 0]=['en','打撃','','','6','800','身体总是处于燃烧状态的马。\n它好像一直都觉得有点热。','上吧！%2！','%2踢飞了%3！','','','火焰马','','','会喷出火焰','从西边离开村子就能看见。',''];
_tr_w["kan"][Number('255') || 0]=['en','打撃','','','6','1600','听说是来自地狱的马，\n身上缠绕着不祥之色的火焰。','上吧！%2！','%2踢飞了%3！','','','狱焰马','','','会喷出火焰','会出现在魔族之王居住的城堡里。',''];
_tr_w["kan"][Number('256') || 0]=['en','打撃','','','6','600','喜欢恶作剧的怪物，\n也喜欢投掷东西。','上吧！%2！','%2冲撞了%3！','','','哥布林','','','会投掷武器','从南边离开村子就能看见。',''];
_tr_w["kan"][Number('257') || 0]=['en','打撃','','','6','900','凶暴的牛怪物。\n会把收在怀里的无数武器丢过来。','上吧！%2！','%2冲撞了%3！','','','牛头怪','','','会投掷武器','会出现在北方地区。',''];
_tr_w["kan"][Number('258') || 0]=['en','打撃','','','7','500','连内脏都是岩石形成的壮硕男人。\n即使用刀剑砍劈也安然无恙。','上吧！%2！','%2用岩石手臂殴打了%3！','','','岩石男','','','','会出现在铁洞窟里。',''];
_tr_w["kan"][Number('259') || 0]=['en','打撃','','','7','700','所有的细胞都是岩石形成的壮硕男人。\n其手臂的破坏力超群。','上吧！%2！','%2用岩石手臂殴打了%3！','','','岩石大叔','','','','会出现在闪着金黄光芒的洞窟里。',''];
_tr_w["kan"][Number('260') || 0]=['en','打撃','','','6','600','硬得不得了的害虫。\n其硬度甚至可以击碎岩石。','上吧！%2！','%2施展出碎石术了！','','','钢铁虫','','','降低防御力（50%）','会出现在铁洞窟里。',''];
_tr_w["kan"][Number('261') || 0]=['en','打撃','','','6','3000','世界第一硬的生物。\n物理攻击根本对它没用。','上吧！%2！','%2施展出碎铠术了！','','','钻石虫','','','降低防御力（50%）','会出现在魔族之王居住的城堡里。',''];
_tr_w["kan"][Number('262') || 0]=['en','打撃','','','5','1000','自火焰中诞生的火焰精灵。\n一靠近他就会立刻着火。','上吧！%2！','%2冲撞了%3！','','','火焰精灵','','','会吟唱【火焰魔法】','会出现在充满魔力的洞窟里。',''];
_tr_w["kan"][Number('263') || 0]=['en','打撃','','','5','1000','自寒冰中诞生的寒冰精灵。\n他周围的气温一直都在冰点下。','上吧！%2！','%2冲撞了%3！','','','寒冰精灵','','','会吟唱【寒冰魔法】','会出现在充满魔力的洞窟里。',''];
_tr_w["kan"][Number('264') || 0]=['en','打撃','','','5','1000','自雷电中诞生的雷电精灵。\n一生气就会从空中降下雷电。','上吧！%2！','%2冲撞了%3！','','','雷电精灵','','','会吟唱【雷电魔法】','会出现在充满魔力的洞窟里。',''];
_tr_w["kan"][Number('265') || 0]=['en','打撃','','','5','1800','管束所有精灵的精灵之王。\n他的模样既美丽又神秘。','上吧！%2！','%2冲撞了%3！','','','精灵王','','','会随机吟唱【极】攻击魔法','会出现在闪着金黄光芒的洞窟里。',''];
_tr_w["kan"][Number('266') || 0]=['en','打撃','','','5','800','拥有无数眼睛的恶心怪物。\n会降低别人攻击力的家伙，非常讨厌。','上吧！%2！','%2冲撞了%3！','','','红眼怪','','','会吟唱【攻击虚弱魔法】','会出现在南方地区。',''];
_tr_w["kan"][Number('267') || 0]=['en','打撃','','','5','700','深蓝色的身体令人觉得恶心的怪物。\n无数的眼睛一瞪过来，防御就会下降。','上吧！%2！','%2冲撞了%3！','','','蓝眼怪','','','会吟唱【防御虚弱魔法】','会出现在南方地区。',''];
_tr_w["kan"][Number('268') || 0]=['en','打撃','','','5','1500','沾上太多飞溅的血而变色的多眼怪。\n要小心这家伙的虚弱魔法。','上吧！%2！','%2冲撞了%3！','','','血眼怪','','','会吟唱【全虚弱魔法】','会出现在魔族之王居住的城堡里。',''];
_tr_w["kan"][Number('269') || 0]=['en','打撃','','','6','900','被神奇薄膜包覆住的水母。\n魔法好像对它完全没有用。','上吧！%2！','%2用电人的触手攻击了%3！','','','魔法水母','','','触电（25%）','会出现在充满魔力的洞窟里。',''];
_tr_w["kan"][Number('270') || 0]=['en','打撃','','','6','1100','引诱人们迈向死亡的恐怖怪物。\n据说是死神所派来的。','上吧！%2！','%2边笑边攻击了%3！','','','死亡使者','','','会吟唱【黑暗魔法】','会出现在灵魂旁徨的洞窟里。',''];
_tr_w["kan"][Number('271') || 0]=['en','打撃','','','6','1700','掌管死亡的邪恶怪物。\n会把发现的人拖下地狱。','上吧！%2！','%2边笑边攻击了%3！','','','死神','','','会吟唱【黑暗魔法‧极】','会出现在魔族之王居住的城堡里。',''];
_tr_w["kan"][Number('272') || 0]=['en','打撃','','','6','699','灵魂附在尸骨上行动。\n会边笑边用骨头揍人。','上吧！%2！','%2边笑边攻击了%3！','','','骷髅','','','','会出现在灵魂旁徨的洞窟里。',''];
_tr_w["kan"][Number('273') || 0]=['en','打撃','','','6','700','在撒孢子的同时会袭击人，\n是非常危险的蘑菇。','上吧！%2！','%2冲撞了%3！','','','妖怪蘑菇','','','会撒孢子','会出现在西方的森林。',''];
_tr_w["kan"][Number('274') || 0]=['en','打撃','','','4','1000','外观一看就觉得有毒。\n绝对不要吃它。','上吧！%2！','%2冲撞了%3！','','','鬼蘑菇','','','会撒孢子','会出现在北方地区。',''];
_tr_w["kan"][Number('275') || 0]=['en','打撃','','','6','500','总是在蠕动的虫。\n不但恶心，还非常难缠。','上吧！%2！','%2蠕动了！','','','蠕动虫','','','','会出现在充满毒沼的洞窟里。',''];
_tr_w["kan"][Number('276') || 0]=['en','打撃','','','3','800','会用超乎想像的音量鸣叫的鸟。\n早上听见它的鸣叫声，没有人会起不来。','上吧！%2！','%2发出150分贝的鸣叫声了！','','','分贝鸟','','','封印普通攻击外的攻击（50%）','离开村子后往西走就能看见。',''];
_tr_w["kan"][Number('277') || 0]=['en','打撃','','','3','1200','弱小又爱哭，但是很狡猾。\n会用魔法强化自己的难缠敌人。','上吧！%2！','%2边哭边用爪子挠%3！','','','爱哭鬼恶魔','','','会随机吟唱强化魔法','会出现在充满毒沼的洞窟里。',''];
_tr_w["kan"][Number('278') || 0]=['en','打撃','','','3','900','身心都很强韧的恶魔。\n但是狡滑程度也不遑多让。','上吧！%2！','%2勇敢地用爪子挠%3！','','','勇敢恶魔','','','会随机吟唱强化魔法','会出现在闪着金黄光芒的洞窟里。',''];
_tr_w["kan"][Number('279') || 0]=['en','打撃','','2','5','600','会拍着翅膀飞过来咬人。\n动作很快，要小心。','上吧！%2！','%2迅速地咬了%3两次！','','','蝙蝠','','','连击2次','会出现在村子附近。',''];
_tr_w["kan"][Number('280') || 0]=['en','打撃','','2','5','900','会混在黑暗里袭击人的麻烦家伙。\n被它咬到还蛮痛的。','上吧！%2！','%2迅速地咬了%3两次！','','','黑暗蝙蝠','','','连击2次','会出现在灵魂旁徨的洞窟里。',''];
_tr_w["kan"][Number('281') || 0]=['30','なし','','','1','1100','总而言之是很危险的石头。\n一旦濒死就会自爆。','上吧！%2！','%2产生了大爆炸！','','','炸弹石','','','会爆炸','会出现在铁洞窟里。',''];
_tr_w["kan"][Number('282') || 0]=['en','打撃','','','6','2200','蠕动虫之王。\n会喷出让武器风化的蠕蠕弹来攻击。','上吧！%2！','%2蠕动了！','','','蠕蠕虫','','','','会出现在魔族之王居住的城堡里。',''];
_tr_w["kan"][Number('283') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('284') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('285') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('286') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('287') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('288') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('289') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('290') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('291') || 0]=['en','斬撃','','3','8','8500','全身包覆着坚硬铠甲的巨龙。\n它的爪子可以劈开任何东西。','上吧！%2！','%2用钢铁爪子劈了又劈！','','','铠龙艾利姆萨尔耶','','','连击3次','会出现在铁洞窟最深处。',''];
_tr_w["kan"][Number('292') || 0]=['en','打撃','直線３','','7','7700','出现在弱者洞窟的诡异怪鸟。\n在战斗时会采取强化自己的战略。','上吧！%2！','%2吐出了毒液！','','','怪鸟毗闍耶','','','中毒（50%）,会吟唱【全强化魔法】','会出现在充满毒沼的洞窟最深处。','3个方格直线'];
_tr_w["kan"][Number('293') || 0]=['en','打撃','前６','','9','9900','拥有意志的神秘块状物。\n会操控触手战斗，本体却什么也不会做。','上吧！%2！','%2压在了%3身上！','','','魔块希瑟斯','','','','会出现在充满魔力的洞窟最深处。','6个方格正面'];
_tr_w["kan"][Number('294') || 0]=['en','打撃','直線３','','4','2400','颜色诡异的触手。\n似乎积蓄了庞大的魔力。','上吧！%2！','%2像鞭子一样弯曲后直接命中了%3！','','','魔触手','','','会随机吟唱攻击魔法','会出现在充满魔力的洞窟最深处。','3个方格直线'];
_tr_w["kan"][Number('295') || 0]=['en','斬撃','','','7','8800','在灵魂洞窟出现的骑士灵魂。\n感觉得到强烈的憎恨与怨念。','上吧！%2！','%2刺穿了%3！','','','灵骑丹斯雷夫','','','会吟唱【黑暗魔法‧极】','会出现在灵魂旁徨的洞窟最深处。',''];
_tr_w["kan"][Number('296') || 0]=['en','打撃','直線２','','15','19999','掌管武器的武器之神。\n超越一切的存在。','上吧！%2！','%2施展出了「大神斩」！','%3在害怕神的威严……','','武器之神','天之尾羽张','','1回合不能行动（30%）','会出现在闪着金黄光芒的洞窟最深处。','2个方格直线'];
_tr_w["kan"][Number('297') || 0]=['en','打撃','','2','12','19999','企图毁灭世界的魔王。\n其实在祈求世界和平。','上吧！%2！','%2施展出了「魔界拳」！','','','魔王多尔哈巴','','','连击2次,混乱（20%）','会出现在魔王城的顶楼。',''];
_tr_w["kan"][Number('298') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('299') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('300') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('301') || 0]=['14','斬撃','','','5','800','附近的野狗。\n很黏人，很可爱。','上吧！%2！','%2咬住了%3！','%2汪汪叫着，让%3害怕了！','','狗','','','1回合不能行动（20%）','在村中某处散步。',''];
_tr_w["kan"][Number('302') || 0]=['13','打撃','','','5','800','附近的野猫。\n喉咙在发出咕噜咕噜的声音。','上吧！%2！','%2对%3施展出了锐利的猫拳！','%2边从喉咙发出声音，边开始向%3撒娇。','','猫','','','降低攻击力（50%）','在村中某处散步。',''];
_tr_w["kan"][Number('303') || 0]=['16','打撃','','','6','1500','某个人饲养的马。\n毛色非常漂亮美丽。','上吧！%2！','%2踢飞了%3！','','','马','','','会撞飞敌人','在西北地区闲晃。',''];
_tr_w["kan"][Number('304') || 0]=['19','打撃','','','6','1600','很厉害的羊，会告诉人提示。\n蓬松柔软的毛很可爱。','上吧！%2！','%2冲撞了%3！','%2柔软的毛缠在了%3身上！','','羊','','','减少行动次数（30%）','是我。',''];
_tr_w["kan"][Number('305') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('306') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('307') || 0]=['18','打撃','','','6','720','在和平村独居的女性。\n是一位美女，个性也很好。','上吧！%2！','%2温柔地施展了头槌！','','','女性村民','Piranka','','','住在村中某处的房子里。',''];
_tr_w["kan"][Number('308') || 0]=['8','打撃','','','6','1050','家里住村中且关系融洽的一家之主。\n总是在屋外陪小孩玩。','上吧！%2！','%2施展出了一家之主拳！','','','一家之父','Tel-ho','','','在村中某处看着小孩。',''];
_tr_w["kan"][Number('309') || 0]=['25','打撃','','','3','960','家里住在和平村的母亲。\n平常很温柔，一生气好像就很恐怖。','上吧！%2！','%2施展了出拳击！','','%2吐出了火焰！','一家之母','Hokkoma','','会喷出火焰','住在村中某处的房子里。',''];
_tr_w["kan"][Number('310') || 0]=['12','打撃','','','5','210','朝气十足地在村里到处跑的兄弟档。\n这位是哥哥，老是在大喊「耶～」。','上吧！%2！','%2一边大喊「耶～！」一边出拳了！','','','朝气蓬勃的兄弟','Kubota','','','在村中某处朝气十足地玩乐。',''];
_tr_w["kan"][Number('311') || 0]=['6','打撃','','','5','180','和哥哥一起玩的可爱弟弟。\n将来的梦想好像是成为Yout◯uber。','上吧！%2！','%2施展出必杀超人力霸骑士踢了！','','','朝气蓬勃的兄弟','Nagina','','','在村中某处朝气十足地玩乐。',''];
_tr_w["kan"][Number('312') || 0]=['17','打撃','','','6','1530','管理和平餐厅的主厨。\n战斗时擅长脚上功夫。','上吧！%2！','%2狠狠踹出了一脚！','','','餐厅主厨','A flavor','','','在村中某处的房子里担任主厨。',''];
_tr_w["kan"][Number('313') || 0]=['11','打撃','','','6','840','每天都光临和平餐厅的常客。\n最近血糖值不太妙。','上吧！%2！','%2连续施展出了高血糖拳！','%3也成了高血糖！','','餐厅常客','Dongfa','','减少行动次数（20%）','在村中某处的房子里吃饭。',''];
_tr_w["kan"][Number('314') || 0]=['13','打撃','','','6','1500','走遍世界各地，自称是美食家。\n对和平餐厅赞不绝口。','上吧！%2！','%2用凸出的肚子压扁了%3！','','','美食家','Katsubaruga','','','在村中某处的房子里吃饭。',''];
_tr_w["kan"][Number('315') || 0]=['4','打撃','','','4','2520','住在村里，目光温和的老人。\n每天都会在妻子的墓前参拜。','上吧！%2！','%2虚弱地撞了上去！','%2过世的妻子居然诅咒了%3！','','扫墓的老人','Braig','','致命（5%）','在村中某处扫墓。',''];
_tr_w["kan"][Number('316') || 0]=['25','打撃','','','6','1740','经常在水井边开会的大妈。\n一开始讲话就有两个小时不会停。','上吧！%2！','%2施展出了抓狂巴掌！','%3看到抓狂的大妈后吓到了！','','井边的大妈','Fukiya','','混乱（20%）','在村中某处开水井边会议。',''];
_tr_w["kan"][Number('317') || 0]=['15','打撃','','2','6','1680','最爱八卦的大妈。\n总是在水井边聊天聊到忘我。','上吧！%2！','%2施展出了连环巴掌！','','','井边的大妈','Illisa','','连击2次','在村中某处开水井边会议。',''];
_tr_w["kan"][Number('318') || 0]=['7','打撃','','','4','2160','经常在村里散步的老人。\n会说些听都没听过的事情。','上吧！%2！','%2用拐杖揍了%3！','','','正在散步的老人','Rod','','','在村中某处散步。',''];
_tr_w["kan"][Number('319') || 0]=['10','打撃','','','6','1050','在村里散步的大叔。\n好像是从遥远的故乡来这里赚钱的。','上吧！%2！','%2用散步锻炼出来的脚踢飞了%3！','','','正在散步的大叔','Ranyapa','','会撞飞敌人','在村中某处散步。',''];
_tr_w["kan"][Number('320') || 0]=['','','','','','0','','','','','','','','','','',''];
_tr_w["kan"][Number('321') || 0]=['','','','','','0','','','','','','','','','','',''];
_tr_w["kan"][Number('322') || 0]=['5','打撃','','','6','630','追寻传说武器的冒险者。\n爱妈妈胜过爱吃饭。','上吧！%2！','%2连续施展出了妈妈教的拳法！','','','冒险者','Kikangji','','','行走在南方地区。',''];
_tr_w["kan"][Number('323') || 0]=['13','打撃','','','9','1470','每天都在池塘淋浴的老头。\n希望赶快有人去报警抓他。','上吧！%2！','%2边笑边冲撞了%3！','%3看到变态后吓到了！','','淋浴老头','Tetsken','','混乱（40%）','从南边离开村子，他就站在那里。',''];
_tr_w["kan"][Number('324') || 0]=['18','打撃','','','6','810','在海边露营的青年。\n真要说起来，他其实是室内派。','上吧！%2！','%2挥动露营道具发动了攻击！','','','露营者','Bastar','','','在南方地区露营。',''];
_tr_w["kan"][Number('325') || 0]=['15','打撃','','','6','540','喜欢独自露营的女性。\n受祖父影响而开始露营。','上吧！%2！','%2挥动露营道具发动了攻击！','','','露营者','Ikpa','','','在南方地区露营。',''];
_tr_w["kan"][Number('326') || 0]=['17','打撃','','','6','960','独自狂欢的露营者。\n看着火焰而兴奋不已。','上吧！%2！','%2挥动露营道具发动了攻击！','','%2生起营火了！','露营者','Charak','','产生火焰','在南方地区露营。',''];
_tr_w["kan"][Number('327') || 0]=['14','斬撃','','','6','750','踏上旅途寻找自我的女性。\n她告诉了我洞窟的危险性。','上吧！%2！','%2用短剑劈开了%3！','','','旅行的女性','Cathrus','','','在南方地区旅行。',''];
_tr_w["kan"][Number('328') || 0]=['22','斬撃','','','6','600','告诉我毒沼危险性的冒险者。\n还给了我药草，是非常亲切的人。','上吧！%2！','%2用心爱的剑劈开了%3！','','','冒险者','Fureil','','','站在东南地区。',''];
_tr_w["kan"][Number('329') || 0]=['28','斬撃','','','8','1950','在老旧的小屋里独自生活的樵夫。\n很擅长使用斧头。','上吧！%2！','%2像是要伐木似地挥动了斧头！','','','樵夫','Wonmar','','降低防御力（30%）','住在某处的小屋里。',''];
_tr_w["kan"][Number('330') || 0]=['5','打撃','','','4','2310','悠闲旅行的老人。\n持续寻找着人生的答案。','上吧！%2！','%2虚弱地施展了头槌！','','','旅行的老人','Brood','','','从西边离开村子，他就在那里旅行。',''];
_tr_w["kan"][Number('331') || 0]=['11','斬撃','','','6','990','为追求强大实力而外出旅行的冒险者男性。\n虽然摆出老练的姿态，但并没有那么强。','上吧！%2！','%2用剑攻击了%3！','','','冒险者','Esto','','','从西边离开村子，他就在那里走动。',''];
_tr_w["kan"][Number('332') || 0]=['9','打撃','','','6','5400','住在森林的小魔女。\n别看她这样，其实魔法能力超群。','上吧！%2！','%2挥动魔杖攻击了！','','%2的魔杖顶端释放出魔法了！','魔女','Cremon','','会随机吟唱【极】攻击魔法','住在某处奇怪的房子里。',''];
_tr_w["kan"][Number('333') || 0]=['17','打撃','','','6','1260','每天都会走向市集的女性。\n梦想着有一天会抵达市集。','上吧！%2！','%2用走路锻炼出来的脚踢飞了%3！','','','走向市集的女性','Takana','','会撞飞敌人','从东边离开村子，她就在那里走动。',''];
_tr_w["kan"][Number('334') || 0]=['24','打撃','','','6','630','在市集摆摊的女性。\n相当好学。','上吧！%2！','%2挥动了晨星锤！','','','摆摊的女性','Yawarra','','混乱（20%）','在东方地区摆摊。',''];
_tr_w["kan"][Number('335') || 0]=['20','斬撃','','2','6','1020','在市集卖菜的男性。\n其实相当黏奶奶。','上吧！%2！','%2用快剑斩杀了%3！','','','摆摊的男性','Rapier','','连击2次','在东方地区摆摊。',''];
_tr_w["kan"][Number('336') || 0]=['14','打撃','','','7','1650','摊位上卖着一看就像是假货的人。\n最好不要和这种人扯上关系。','上吧！%2！','%2用凸出的肚子压扁了%3！','','','摆摊的男性','Pusa','','','在东方地区摆摊。',''];
_tr_w["kan"][Number('337') || 0]=['19','打撃','','','6','720','开心逛市集的红发男人。\n持有28元金钱。','上吧！%2！','%2施展出了右直拳！','','','市集的客人','Shrink','','','在东方地区的市集物色商品。',''];
_tr_w["kan"][Number('338') || 0]=['8','斬撃','','','6','660','来到市集的女性。\n正在等半价特卖活动开始。','上吧！%2！','%2用在市集买的剑攻击了%3！','','','市集的客人','Scruma','','','在东方地区的市集物色商品。',''];
_tr_w["kan"][Number('339') || 0]=['12','打撃','','','4','1350','因为很闲而来市集的男性。\n好像买了奇怪的陶壶。','上吧！%2！','%2用在市集买的陶壶砸向了%3！','','','市集的客人','Bundy','','一定会暴击','在东方地区的市集物色商品。',''];
_tr_w["kan"][Number('340') || 0]=['18','打撃','','2','6','1440','有钱的市集客人。\n走路时还在用一叠钞票扇风。','上吧！%2！','%2拿出钞票甩了对方一巴掌！','%3看见大把钞票后兴奋不已！','','市集的客人','Murgiza','','连击2次,提升敌人的行动次数','在东方地区的市集物色商品。',''];
_tr_w["kan"][Number('341') || 0]=['17','打撃','','','5','1230','沿海教会的牧师，\n也是投掷教的教徒。','上吧！%2！','%2施展出了阿门拳！','某种神圣的力量对%3起作用了！','','教会的牧师','Falsio','','重置能力强化','站在某处的教会里。',''];
_tr_w["kan"][Number('342') || 0]=['22','打撃','','','6','1140','好像很温柔的修女。\n喜欢的面类是平打乌龙面。','上吧！%2！','%2气势十足地撞了上去！','','','教会的修女','Lancia','','','站在某处的教会里。',''];
_tr_w["kan"][Number('343') || 0]=['19','打撃','','','5','2460','在教会祷告的婆婆。\n是投掷教的狂热信徒。','上吧！%2！','%2边祷告边抱住了敌人！','','','祷告的婆婆','Versa','','','在某处的教会里祷告。',''];
_tr_w["kan"][Number('344') || 0]=['13','打撃','','','6','450','在教会祷告的青年。\n个性非常认真。','上吧！%2！','%2边祷告边施展了头槌！','','','祷告的青年','Irarufu','','','在某处的教会里祷告。',''];
_tr_w["kan"][Number('345') || 0]=['14','打撃','','','6','750','很适合戴眼镜的绿发冒险者。\n兴趣是参观废墟。','上吧！%2！','%2挥动魔杖攻击了！','%2的魔杖顶端释放出魔法了！','','冒险者','Clique','','可施放【寒冰魔法】','站在东北地区。',''];
_tr_w["kan"][Number('346') || 0]=['10','打撃','','','5','2040','喜欢钓鱼和大海的老人，\n但没有什么钓鱼的技术。','上吧！%2！','%2挥动钓竿攻击了！','钓竿的前端居然勾着腐烂的蚯蚓！','','钓客','Phyzigro','','中毒（60%）','在西北方岬角钓鱼。',''];
_tr_w["kan"][Number('347') || 0]=['12','斬撃','','3','6','510','好胜的冒险者女性。\n好像正在找工作。','上吧！%2！','%2用短剑迅速劈开了%3！','','','冒险者','Dagger','','连击3次','行走在北方地区。',''];
_tr_w["kan"][Number('348') || 0]=['27','打撃','','','6','660','便利店的新人店员。\n只会按照教学手册上教的话说。','上吧！%2！','%2用读取条码的那个揍了%3！','','','便利店店员','Miaota','','','站在某处的便利店里。',''];
_tr_w["kan"][Number('349') || 0]=['15','斬撃','','','6','900','在便利店购物的男性。\n在烦恼要买哪一款果汁。','上吧！%2！','%2用便利店购物卡劈开了%3！','','','便利店的客人','Arsato','','','站在某处的便利店里。',''];
_tr_w["kan"][Number('350') || 0]=['33','打撃','','','7','990','聚在便利店前的冷酷不良少年。\n惹他生气好像会很可怕。','上吧！%2！','%2把%3打得落花流水！','%3怕%2怕得屁滚尿流！','','不良少年','Reserville','','1回合不能行动（20%）','蹲在某处的便利店前。',''];
_tr_w["kan"][Number('351') || 0]=['29','斬撃','','','5','2160','一只眼睛受伤的老练旅人。\n外表一看就感觉很厉害。','上吧！%2！','%2豪迈地挥动了大剑！','','','旅行的大叔','Gladys','','','在北方地区旅行。',''];
_tr_w["kan"][Number('352') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('353') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('354') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('355') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('356') || 0]=['2','打撃','','','8','180','\\n[1]的妹妹。年纪小却很能干。\n家事和顾店都是她一个人做。','上吧！%2！','%2试着使出加油拳法！','%2从%3的钱包里偷了钱！','','器美','','','会获得钱','是你的妹妹。',''];
_tr_w["kan"][Number('357') || 0]=['30','斬撃','','','9','1140','看起来没有干劲的勇者。\n似乎想要轻松地打倒魔王。','上吧！%2！','%2不耐烦地挥剑了！','%2吸走了%3的干劲！','','勇者修瓦尔迪','','','降低攻击力（40%）','你的店发展起来后，他就会到访吧。',''];
_tr_w["kan"][Number('358') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('359') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('360') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('361') || 0]=['21','斬撃','直線２','','20','18564','魔王给的黑暗大剑。\n每一击都沉重得可怕。','%1挥动了%2！','','','','魔王的剑','','','一定会暴击','应该在魔族之王手中吧。','2个方格直线'];
_tr_w["kan"][Number('362') || 0]=['17','打撃','','','8','1000','用来收藏获得奖杯的箱子。\n拿起来非常重。','%1拿起%2试着挥动了一下！','%2直接命中了%3！','','','奖杯箱','','','','在你家里。',''];
_tr_w["kan"][Number('363') || 0]=['0','なし','','','20','14500','好像能抓到各种昆虫的网子。\n吸力可媲美黑洞。','%1挥动了%2！','','','','捕虫网','','','将面前的物品当作武器获得','你的妹妹偷偷收藏着。',''];
_tr_w["kan"][Number('364') || 0]=['25','打撃','','','9','2','完美的螺旋状粪便。\n含水量和颜色都非常完美。','%1拿起%2试着挥动了一下！','%2直接命中了%3的脸！','%2臭死了！','','大便','','','降低攻击力（20%）,降低防御力（20%）,降低行动次数（20%）','温热地掉在东北地区。',''];
_tr_w["kan"][Number('365') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('366') || 0]=['','','','','','','','%1試著使出加油拳法！','','','','（空手）','','','連擊3次','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','%1使用了\i[261]药草！','','','','（药草）','','','产生火焰','',''];
_tr_w["kan"][Number('3') || 0]=['','','','','','','希望能事先大量准备好的冒险必需品。\n可以恢复体力和异常状态喔。','%1拿起%2试着挥动了一下！','%3把%2吸進嘴裡了！','','','药草','','','','恢复敌人的HP',''];
_tr_w["kan"][Number('4') || 0]=['','','','','','','能引出武器隐藏之力的神奇石头。\n是很难得的贵重材料。','%1拿起%2试着挥动了一下！','%2的一角刺中%3了！','','','魂石','','','','有时候敌人会掉下来。',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];
_tr_w["kan"][Number('') || 0]=['','','','','','','','','','','','','','','','',''];

})();
})();
